/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    michael.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"
UInt32 getUInt32( Byte * p )
// Convert from Byte[] to UInt32 in a portable way
{
    UInt32 res = 0;
    int i;
    for( i=0; i<4; i++ )
    {
    res |= (*p++) << (8*i);
    }
    return res;
}


void putUInt32( Byte * p, UInt32 val )
// Convert from UInt32 to Byte[] in a portable way
{
    int i;
    for( i=0; i<4; i++ )
    {
    *p++ = (Byte) (val & 0xff);
    val >>= 8;
    }
}


/***********************************************************************/
/*                                                                     */
/*   FUNCTION DESCRIPTION                   MICclear                   */
/*      Initial variable require by MIC computation                    */
/*                                                                     */
/*   AUTHOR                                                            */
/*     Liam,Hwu            ZyDAS Technology   Corporation              */
/*                                                                     */
/***********************************************************************/

void MICclear(MICvar *MIC)
{
// Reset the state to the empty message.
    MIC->L = MIC->K0;
    MIC->R = MIC->K1;
    MIC->nBytesInM = 0;
    MIC->M = 0;
}


/***********************************************************************/
/*                                                                     */
/*   FUNCTION DESCRIPTION                   MICsetKey                  */
/*      Set MIC key (Tx or Rx)                                         */
/*                                                                     */
/*   AUTHOR                                                            */
/*     Liam,Hwu            ZyDAS Technology   Corporation              */
/*                                                                     */
/***********************************************************************/

void MICsetKey( Byte * key,MICvar *MIC )
{
    // Set the key
    MIC->K0 = getUInt32( key );
    MIC->K1 = getUInt32( key + 4 );
    // and reset the message
    MICclear(MIC);
}

/***********************************************************************/
/*                                                                     */
/*   FUNCTION DESCRIPTION                   MICappendByte              */
/*      Compute MIC for adding a single byte                           */
/*                                                                     */
/*   AUTHOR                                                            */
/*     Liam,Hwu            ZyDAS Technology   Corporation              */
/*                                                                     */
/***********************************************************************/

void MICappendByte( Byte b,MICvar *MIC )
{
    // Append the byte to our word-sized buffer
    MIC->M |= b << (8* MIC->nBytesInM);
    MIC->nBytesInM++;
    // Process the word if it is full.
    if( MIC->nBytesInM >= 4 )
    {
    MIC->L ^= MIC->M;
    MIC->R ^= ROL32(MIC->L, 17 );
    MIC->L += MIC->R;
    MIC->R ^= ((MIC->L & 0xff00ff00) >> 8) | ((MIC->L & 0x00ff00ff) << 8);
    MIC->L += MIC->R;
    MIC->R ^= ROL32( MIC->L, 3 );
    MIC->L += MIC->R;
    MIC->R ^= ROR32( MIC->L, 2 );
    MIC->L += MIC->R;
    // Clear the buffer
    MIC->M = 0;
    MIC->nBytesInM = 0;
    }
}

/***********************************************************************/
/*                                                                     */
/*   FUNCTION DESCRIPTION                   MICappendByte              */
/*      Compute MIC for adding a single byte                           */
/*                                                                     */
/*   AUTHOR                                                            */
/*     Liam,Hwu            ZyDAS Technology   Corporation              */
/*                                                                     */
/***********************************************************************/

void MICgetMIC( Byte * dst,MICvar *MIC )
{
    // Append the minimum padding
    MICappendByte( 0x5a,MIC );
    MICappendByte( 0,MIC );
    MICappendByte( 0,MIC );
    MICappendByte( 0,MIC );
    MICappendByte( 0,MIC );
    // and then zeroes until the length is a multiple of 4
    while( MIC->nBytesInM != 0 )
    {
    MICappendByte( 0,MIC );
    }
    // The appendByte function has already computed the result.
    putUInt32( dst, MIC->L );
    putUInt32( dst+4,MIC->R );
    // Reset to the empty message.
    MICclear(MIC);
}
