/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    oid_req.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

#define NUM_REG_MASK ( sizeof (MacRegMaskTab) / sizeof (MAC_REGISTER_MASK_TBL) )

typedef struct _MAC_REGISTER_MASK_TBL {

    ULONG		Address;
	ULONG		ReadWriteMask;

} MAC_REGISTER_MASK_TBL;

MAC_REGISTER_MASK_TBL MacRegMaskTab[ ] = {
	//Address				Mask
	// GPIO
	RF_IF_CLK,				0x1,		//0x0400
	RF_IF_DATA,				0x1,		//0x0404
	PE1_PE2,				0x83,		//0x0408
	PE2_DLY,				0xff,		//0x040C
	LE1,					0x1,		//0x0410
	LE2,					0x1,		//0x0414
	GPI_EN,					0x0,		//0x0418
	RADIO_PD,				0x1,		//0x042C
	RF2948_PD,				0x1,		//0x042C
	LED1,					0x1,		//0x0430
	LED2,					0x1,		//0x0434

	EnablePSManualAGC,		0x0,		//0x043C	// 1: enable
	CONFIGPhilips,			0x3,		//0x0440

	SA2400_SER_AP,			0x0,		//0x0444
	I2C_WRITE,				0x0,		//0x0444	// Same as SA2400_SER_AP (for compatible with ZD1201)

	SA2400_SER_RP,			0x0,		//0x0448


	AfterPNP,				0x1,		//0x0454
	RADIO_PE,				0x0,		//0x0458
	RstBusMaster,			0x0,		//0x045C

	RFCFG,					0x7,		//0x0464
	HSTSCHG,				0x0,		//0x046C

	PHY_ON,					0x0,		//0x0474
	RX_DELAY,				0x0,		//0x0478
	RX_PE_DELAY,			0xff,		//0x047C


	GPIO_1,					0x0,		//0x0490
	GPIO_2,					0x0,		//0x0494


	EncryBufMux,			0x0,		//0x04A8

	// Power Management Registers
	PS_Ctrl,				0x0,		//0x0500

	ADDA_MBIAS_WarmTime,	0x0,		//0x0508

	InterruptCtrl,			0x0,		//0x0510
	TSF_LowPart,			0x0,		//0x0514
	TSF_HighPart,			0x0,		//0x0518
	ATIMWndPeriod,			0xffff,		//0x051C
	BCNInterval,			0xffffff,	//0x0520
	Pre_TBTT,				0xffff,		//0x0524	//In unit of TU(1024us)

	// General Registers
	PCI_TxAddr_p1,			0x00000000,
	PCI_TxAddr_p2,			0xffffffff,
	PCI_RxAddr_p1,			0x00000000,
	PCI_RxAddr_p2,			0xffffffff,
	//MACAddr_P1,				0x00000000,
	//MACAddr_P2,				0x00000000,
	BSSID_P1,				0xffffffff,
	BSSID_P2,				0x0000ffff,
	BCNPLCPCfg,				0xffffffff,
	GroupHash_P1,			0xffffffff,
	GroupHash_P2,			0xffffffff,
	PHYDelay,				0x000000ff,
	BCNFIFO,				0x000000ff,
	SnifferOn,				0x00000001,
	EncryType,				0x0000000f,
	RetryMAX,				0x0000001f,
	CtlReg1,				0x000003fc,
	DeviceState,			0x00000000,
	UnderrunCnt,			0x00000000,
	SendToHost,				0xffffffff,
	Ack_Timeout_Ext,		0x0000001f,
	BCN_FIFO_Semaphore,		0x00000001,
	IFS_Value,				0x0fffffff,
	TotalRxFrm,				0x00000000,
	CRC32Cnt,				0x00000000,
	CRC16Cnt,				0x00000000,
	DecrypErr_UNI,			0x00000000,
	RxFIFOOverrun,			0x00000000,
	DecrypErr_Mul,			0x00000000,
	NAV_CNT,				0x00000000,
	NAV_CCA,				0x00000000,
	RetryCnt,				0x00000000,
	ReadTcbAddress,			0x00000000,
	ReadRfdAddress,			0x00000000,
	CWmin_CWmax,			0x03ff7fff,
	TotalTxFrm,				0x00000000,
};



NDIS_STATUS
ZD1211Old_SetInformation(
                   IN NDIS_HANDLE MiniportAdapterContext,
                   IN NDIS_OID Oid,
                   IN PVOID InformationBuffer,
                   IN ULONG InformationBufferLength,
                   OUT PULONG BytesRead,
                   OUT PULONG BytesNeeded
                   )
{
    PZD1205_ADAPTER     Adapter;
    NDIS_STATUS         Status;
	ULONG				DebugPrint;
	ULONG				IoAddress;
	ULONG				IoValue;
#ifdef HOST_IF_USB
	ZD1211_IRQL			OldIrql;
#endif
	USHORT		i;
	PULONG		pulong;
	ULONG		tmpvalue;
	ULONG		tmpkey;
	ULONG		reg;
	BOOLEAN		Write_Enable = FALSE;
	PULONG      zdval;
	PZD_IMAGE_BLOCK_STRUCT	pImageBlockStruct;
    PRID_STRUCT			rid_p;
	PZD_CMD_STRUCT		cmd_p;
	PUCHAR		pKey;
	USHORT		loop;

    DEBUGFUNC(_T("ZD1211Old_SetInformation"));
    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_OID;
 
	//INITSTR((_T(" 0x%x\n"), Oid));

	switch (Oid) {

		case OID_ZD_IO32:
			IoAddress = (ULONG)(*((PULONG)InformationBuffer) & 0xffffffff);
            IoValue   = (ULONG)(*((PULONG)InformationBuffer+ 1) );
            //INITSTR((_T("IO Write: IO Addr=%x IO Data=%x\n"),IoAddress,IoValue));
                
            if (IoAddress & 0x8000){    // Write command
                IoAddress &= 0x7fff;
                if (IoAddress == 0x888){
                    Adapter->adapterReady = FALSE;
                    Adapter->TestFlag |= LOOPBACKTEST;
                    Adapter->LBFrameLen = (USHORT)IoValue;
                //JWEI 2004/03/05
                    Adapter->LBTxCnt = 0;
                    Adapter->LBRxCnt = 0;
                    if (Adapter->LBFrameLen >= (4095 - 28 - 8))
                //    if (Adapter->LBFrameLen > 1500)
                        Adapter->LBFrameLen = 1500;
                    Adapter->LBRxFrameLen = Adapter->LBFrameLen;
                    AcquireCtrOfPhyReg(Adapter);
                #if fZD1211_LOOPBACK
                    if ((!Adapter->RateLock) || (Adapter->CardSetting.TxRate < 4))
                    {
                    // Set rate (same as 0x903)
                        ZD1205_WRITE_REGISTER(Adapter,CR30, 0x4b);
                        Adapter->CardSetting.TxRate = 0xB;  // 54M
                        Adapter->RateLock = 1;
                    }
                #endif
#if !(defined(GCCK) && defined(OFDM))
					ZD1205_WRITE_REGISTER(Adapter,CR33, 0x2c);
#else
                // delay Tx_PE
                    if (Adapter->CardSetting.TxRate == 0xB) // 54M
                        ZD1205_WRITE_REGISTER(Adapter,CR131, 0xA8);
                    else if (Adapter->CardSetting.TxRate >= 0x8)// 48M - 24M
                        ZD1205_WRITE_REGISTER(Adapter,CR131, 0xC0);
                    else                    // 18M - 6M
                        ZD1205_WRITE_REGISTER(Adapter,CR131, 0xF0);

                    ZD1205_WRITE_REGISTER(Adapter,CR96, 0x02);
#endif
					//For digital loopback test, CR47 should be set to 0x18. 
					//This is to control the TX digital level to fit RX required 
					//level
					ZD1205_WRITE_REGISTER(Adapter,CR47, 0x18);
					ReleaseCtrOfPhyReg(Adapter);

				}
#if (defined(GCCK) && defined(OFDM))
				else if (IoAddress == 0x885){
					Adapter->PSThreshhold = (ULONG)IoValue;
				}
				else if (IoAddress == 0x886){
					Adapter->CountrySelect = (UINT)IoValue;
				}
				else if (IoAddress == 0x887){
					Adapter->adapterReady = FALSE;
					Adapter->TestFlag |= LOOPBACKTEST;
					Adapter->LBFrameLen = (USHORT)IoValue;
                //JWEI 2004/03/05
                    Adapter->LBTxCnt = 0;
                    Adapter->LBRxCnt = 0;
                    if (Adapter->LBFrameLen >= (4095 - 28 - 8))
                //    if (Adapter->LBFrameLen > 1500)
                        Adapter->LBFrameLen = 1500;
                    //    Adapter->LBFrameLen = 0;
                    Adapter->LBRxFrameLen = Adapter->LBFrameLen;
                    AcquireCtrOfPhyReg(Adapter);
                //JWEI 2004/01/30
                #if fZD1211_LOOPBACK
                    if ((!Adapter->RateLock) || (Adapter->CardSetting.TxRate > 3))
                    {
                    // Set rate (same as 0x903)
                        ZD1205_WRITE_REGISTER(Adapter,CR30, 0x4b);
                        Adapter->CardSetting.TxRate = 0x3;  // 11M
                        Adapter->RateLock = 1;
                    }
                // Disable OFDM Loopback
                    ZD1205_WRITE_REGISTER(Adapter,CR96, 0x00);
                // Enable CCK Loopback
                #endif
                    ZD1205_WRITE_REGISTER(Adapter,CR33, 0x0C);
                    //For digital loopback test, CR47 should be set to 0x18. 
                    //This is to control the TX digital level to fit RX required 
                    //level
					ZD1205_WRITE_REGISTER(Adapter,CR47, 0x18);
					ReleaseCtrOfPhyReg(Adapter);

                // move following code to 0x8004
                //    Mgt_SendNotification(Adapter, RQ_LBTEST);
				}
#endif
#ifdef HOST_IF_USB
				else if (IoAddress == 0x002){
					Adapter->DisableTx = FALSE;
					ZD1205_WRITE_REGISTER(Adapter,0x0F02, 0);
					
					//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
					ZDDbg((_T("Get SpinLock\n")));
					ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
						USB_StartIntEP(Adapter);
					//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
					ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
					ZD1205EnableInterrupt(Adapter);
				}
#endif
#ifdef OFDM
				else if (IoAddress == 0x003){
					Adapter->DisableTx = TRUE;
					ZD1205_WRITE_REGISTER(Adapter,0x0F03, 0);
				}
                else if (IoAddress == 0x004)
                {
                #if !fZD1211_LOOPBACK
					Adapter->adapterReady = FALSE;
					Adapter->TestFlag |= LOOPBACKTEST;
					Adapter->LBFrameLen = (USHORT)IoValue;
#if !(defined(GCCK) && defined(OFDM))
					AcquireCtrOfPhyReg(Adapter);
					ZD1205_WRITE_REGISTER(Adapter,CR33, 0x2c);
					//For digital loopback test, CR47 should be set to 0x18. 
					//This is to control the TX digital level to fit RX required 
					//level
					ZD1205_WRITE_REGISTER(Adapter,CR47, 0x18);
					ReleaseCtrOfPhyReg(Adapter);
#endif
                #else
                    Adapter->LBTxTestCnt = cLBTEST_COUNT;
                    if (IoValue)
                        Adapter->LBTxTestCnt = IoValue;
                    Mgt_SendNotification(Adapter, RQ_LBTEST);
#endif
				}
				else if (IoAddress == 0x005){
#ifdef HOST_IF_USB
					Adapter->USBDbgFlag = IoValue;
#endif
				}
				else if (IoAddress == 0x007){
					if (IoValue){
					
						AcquireCtrOfPhyReg(Adapter);
						for (i=CR0; i<PHY_END; i+=4){

							ZD1205_READ_REGISTER(Adapter,i, &tmpvalue);
							ZDDbgPrint((_T("CR%d=0x%x\n"), i/4, tmpvalue&0xFFFF));

						}
						ReleaseCtrOfPhyReg(Adapter);
					}
				}
				else if (IoAddress == 0x008){
					if (IoValue){
						for (i=RF_IF_CLK; i<=FIFO_Length; i+=4){
							ZD1205_READ_REGISTER(Adapter,i, &tmpvalue);
							ZDDbgPrint((_T("MAC 0x%x=0x%x\n"), i, tmpvalue));
						}
                    //JWEI 2003/12/31
                        for (i = E2P_SUBID; i < E2P_END; i += 4)
                        {
							ZD1205_READ_REGISTER(Adapter,i, &tmpvalue);
							ZDDbgPrint((_T("MAC 0x%x=0x%x\n"), i, tmpvalue));
						}
					}
				}
#endif
				else if (IoAddress == 0x009){
					if (IoValue){
						TurnOn_RF(Adapter);
					}
					else{
						TurnOff_RF(Adapter);
					}
				}
#ifdef HOST_IF_USB
				else if (IoAddress == 0x00A){
					Adapter->TxRxEPRestartWaitTime = IoValue;
				}
#endif
				else if (IoAddress == 0x00C){
					// Terminate Mgt thread 
					Mgt_SendNotification(Adapter, RQ_TERMINATION);
					while(Adapter->h_ZD1205MgtThread){
						ZD1205_Sleep(10000);
					}
				}
#ifdef HOST_IF_USB
				else if (IoAddress == 0x00D){
			
					//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
					ZDDbg((_T("Get SpinLock\n")));
					ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
						USB_ResetTxQueue(Adapter);
					//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
					ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
				}
#endif
				else if (IoAddress == 0x00E){
			
					INITSTR((_T("CAM_TKIP Mode\n")));
					Change_EncryMode(Adapter, TKIP, TKIP);
					SetKeyContext(Adapter, DEFAULT_KEY_SET, TKIP, 
							(UCHAR)0, Adapter->TxSeed.TK);
/*				
					// The key is being set by a Supplicant.
					MICsetKey(&pNdis80211Key->KeyMaterial[16],
						&Adapter->RxMicKey[keyIndex]);	
					MICsetKey(&pNdis80211Key->KeyMaterial[24],
						&Adapter->TxMicKey);
						
					INITSTRWPA((_T("RxMicKey=")));
					DumpWPA((char *)Adapter->RxMicKey,8,FALSE,1);	
					INITSTRWPA((_T("TxMicKey=")));
					DumpWPA((char *)&Adapter->TxMicKey,8,FALSE,1);
*/
				}
				else if (IoAddress == 0x00F){
			
					SetKeyContext(Adapter, DEFAULT_KEY_SET, TKIP, 
						(UCHAR)Adapter->groupKeyIndex, Adapter->RxSeed[Adapter->groupKeyIndex].TK);
/*					
					 // The key is being set by a Supplicant.
					MICsetKey(&pNdis80211Key->KeyMaterial[16],
						&Adapter->RxMicKey[keyIndex]);	
*/				
				}
				else if (IoAddress == 0x011){
					Adapter->ZDConfigRunning = IoValue;
				}
				else if (IoAddress == 0x012){
			#if fMERGE_RX_FRAME
////					if (!ChangeToFlashAble(Adapter))
////						break;
			#endif			
				}
				else if (IoAddress == 0x019){
					Adapter->NotCheckConnect = (USHORT)IoValue;
				}
				else if (IoAddress == 0x01E){
					Adapter->PkInstalled = 0;
					Mgt_SendNotification(Adapter, RQ_DISCONNECT);
				}
				else if (IoAddress == 0x700){
					Adapter->SetProtectionModeTimer = IoValue;
				}
				else if (IoAddress == 0x710){
					Adapter->PHYTestIndex = (UCHAR) IoValue;
				}
				else if (IoAddress == 0x711){
					Adapter->PHYTestRssiBound = (UCHAR) IoValue;
				}
				else if (IoAddress == 0x712){
					Adapter->PHYTestTimer = (UCHAR) IoValue;
				}
				else if (IoAddress == 0x713){
					Adapter->SetProtectionModeTimer = IoValue;
				}
				else if (IoAddress == 0x720){
					Adapter->PHYFreCountThr = IoValue;
				}
				else if (IoAddress == 0x721){
					Adapter->PHYFreTimer = IoValue;
				}
				else if (IoAddress == 0x722){
					Adapter->PHYVarThr = IoValue;
				}
				else if (IoAddress == 0x730){
                    Adapter->PHYLowPower = (UCHAR) IoValue;
					if ( !(Adapter->PHYLowPower & BIT_1) )
					{
						Adapter->CR31Flag = 0;
						Adapter->TraceSetPoint = 1;
                        UpdateIntegrationValue(Adapter, Adapter->CardSetting.Channel);
					}
				}
				else if (IoAddress == 0x740){
				    Adapter->CardSetting.DTIMPeriod = IoValue;
				}
				else if (IoAddress == 0x750){
				    Adapter->IPCFlag = (UCHAR) IoValue;
				}
				else if (IoAddress == 0x760){
				    switch(IoValue)
					{
					    case 36:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x30cf3);
							Set_IF_Synthesizer(Adapter, 0x0ccc4);
							break;
					    case 40:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x00d03);
							Set_IF_Synthesizer(Adapter, 0x00004);
							break;
					    case 44:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x10d03);
							Set_IF_Synthesizer(Adapter, 0x33334);
							break;
					    case 48:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x20d13);
							Set_IF_Synthesizer(Adapter, 0x26664);
							break;
					    case 52:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x20d23);
							Set_IF_Synthesizer(Adapter, 0x19994);
							break;
					    case 56:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x30d33);
							Set_IF_Synthesizer(Adapter, 0x0ccc4);
							break;
					    case 60:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x00d43);
							Set_IF_Synthesizer(Adapter, 0x00004);
							break;
					    case 64:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x10d43);
							Set_IF_Synthesizer(Adapter, 0x33334);
							break;
					    case 68:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x20d53);
							Set_IF_Synthesizer(Adapter, 0x26664);
							break;
					    case 72:
					        Set_IF_Synthesizer(Adapter, 0x18655);
                            Set_IF_Synthesizer(Adapter, 0x20d63);
							Set_IF_Synthesizer(Adapter, 0x19994);
							break;
					}
				}
				else if (IoAddress == 0x889){
					if (IoValue){
						Adapter->TestFlag |= LENGTH_SCAN;
					}
					else{
						Adapter->TestFlag &= ~LENGTH_SCAN;
					}
				}
				else if (IoAddress == 0x893){
					UINT i;
					ULONG tmpvalue, tmpvalue1;
					
					ZDDbgPrint((_T(" Dump debug fifo\n")));
					ZD1205_WRITE_REGISTER(Adapter,0x804, 0x200);
					ZD1205_READ_REGISTER(Adapter,0x804, &tmpvalue);
					tmpvalue >>= 16;
					ZDDbgPrint((_T("length = %x\n"),tmpvalue));
					for (i=0;i<tmpvalue;i++){
						ZD1205_READ_REGISTER(Adapter,0x800, &tmpvalue1);
						ZDDbgPrint((_T("%8x  "), tmpvalue1));
					}
				}
				else if (IoAddress == 0x88E){
#ifdef HOST_IF_USB
					//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
					ZDDbg((_T("Get SpinLock\n")));
					ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
						USB_StartRxEP(Adapter);
					//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
					ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#endif
				}
				else if (IoAddress == 0x900){
					Adapter->DbgMsgFlag = IoValue;
				}
				else if (IoAddress == 0x901){
					Adapter->CardSetting.FragThreshold = IoValue;
				}
				else if (IoAddress == 0x903){	// Select TxRate
					if (IoValue & 0x80){
						AcquireCtrOfPhyReg(Adapter);
						ZD1205_WRITE_REGISTER(Adapter,CR30, 0x49);
						ReleaseCtrOfPhyReg(Adapter);
					}
					else{
						AcquireCtrOfPhyReg(Adapter);
						ZD1205_WRITE_REGISTER(Adapter,CR30, 0x4b);
						ReleaseCtrOfPhyReg(Adapter);
					}
					IoValue &= ~0x80;
					Adapter->CardSetting.TxRate = (UCHAR)IoValue;
					Adapter->RateLock = 1;
				}
				else if (IoAddress == 0x904){
					Adapter->RF_Mode = (UCHAR)IoValue;
					Set_RF_Channel(Adapter, 1, (UCHAR)Adapter->RF_Mode, 0);	
				}
				else if (IoAddress == 0x905){
					Adapter->TestPS = (BOOLEAN)IoValue;
				}
				else if (IoAddress == 0x906){  // not check sequence number
					Adapter->notCheckSN = (UCHAR) IoValue;
				}
				else if (IoAddress == 0x908){
					Adapter->CardSetting.RTSThreshold = IoValue;
				}
				else if (IoAddress == 0x909){
					//Adapter->ShowSeq = (BOOLEAN)IoValue;
				}
				else if (IoAddress == 0x90b){
				}
				else if (IoAddress == 0x90c){
					Adapter->adapterReady = FALSE;
				}
				else if (IoAddress == 0x90e){
					SleepResetDevice(Adapter);
				}
				else if (IoAddress == 0x90f){
					// Read EEPROM
					while(1){
						ZD1205_WRITE_REGISTER(Adapter,0x714, 0x424);
						ZD1205_Sleep(1000*IoValue);
						ZD1205_READ_REGISTER(Adapter,0x908, &tmpvalue);
						if (tmpvalue != 0x78563412){
							ZDDbgPrint((_T("EEPROM Read failure\n")));
							ZDDbgPrint((_T("Read 0x908 = %x\n"), tmpvalue));
							ASSERT(0);
							break;
						}
					}
				}
				else if (IoAddress == 0x911){
					Adapter->SelfCTSMode = (USHORT)IoValue;
				}
				else if (IoAddress == 0x920){ //Fake MIC
					Adapter->FakeMic = (BOOLEAN)IoValue;
				}
				else if (IoAddress == 0x921){ //simulate MIC error, pairwise key
					ReportAuthEvent(Adapter,  NDIS_802_11_AUTH_REQUEST_PAIRWISE_ERROR);
				}
				else if (IoAddress == 0x922){ //simulate MIC error, group key
					ReportAuthEvent(Adapter,  NDIS_802_11_AUTH_REQUEST_GROUP_ERROR);
				}	
				else if (IoAddress == 0xa00){
					Set_IF_Synthesizer(Adapter, IoValue);
                    AcquireCtrOfPhyReg(Adapter);
                    ZD1205_WRITE_REGISTER(Adapter, CR203, 0x06);
                    ReleaseCtrOfPhyReg(Adapter);
	                Adapter->CR203Flag = 2;
				}
				else if (IoAddress == 0xa01){		// GCT-CR2
					tmpvalue = 0;
					for (i=0;i<16;i++){
						tmpvalue |= (((IoValue >> i) & BIT_0) << (15-i));
					}
					tmpvalue |= 0x080000;
					Set_IF_Synthesizer(Adapter, tmpvalue);
				}
				else if (IoAddress == 0xa02){		// GCT-CR5
					tmpvalue = 0;
					for (i=0;i<16;i++){
						tmpvalue |= (((IoValue >> i) & BIT_0) << (15-i));
					}
					tmpvalue |= 0x140000;
					Set_IF_Synthesizer(Adapter, tmpvalue);
				}
				else if (IoAddress == 0xa03){		// GCT-CRE
					tmpvalue = 0;
					for (i=0;i<16;i++){
						tmpvalue |= (((IoValue >> i) & BIT_0) << (15-i));
					}
					tmpvalue |= 0x0e0000;
					Set_IF_Synthesizer(Adapter, tmpvalue);
				}
				else if (IoAddress == 0xa04){		// I Channel, Q Channel
					Adapter->DbgMsgFlag = 0;
					AcquireCtrOfPhyReg(Adapter);
					ZD1205_WRITE_REGISTER(Adapter,CR120, 0x1);
					for (i=0;i<IoValue;i++){
						ZD1205_WRITE_REGISTER(Adapter,CR50, 0x1);

						ZD1205_READ_REGISTER(Adapter,CR51, &tmpvalue);
						ZD1205_READ_REGISTER(Adapter,CR52, &tmpvalue);
						ZD1205_READ_REGISTER(Adapter,CR53, &tmpvalue);
						ZD1205_READ_REGISTER(Adapter,CR54, &tmpvalue);
					}
					ZD1205_WRITE_REGISTER(Adapter,CR120, 0x0);
					ReleaseCtrOfPhyReg(Adapter);
				}
				else if (IoAddress == 0xa05){		// I Channel, Q Channel
					Adapter->DbgMsgFlag = 0;
					AcquireCtrOfPhyReg(Adapter);
					ZD1205_WRITE_REGISTER(Adapter,CR190, 0x20);
					for (i=0;i<IoValue;i++){
						ZD1205_WRITE_REGISTER(Adapter,CR189, 0x1);

						ZD1205_READ_REGISTER(Adapter,CR191, &tmpvalue);
						ZD1205_READ_REGISTER(Adapter,CR192, &tmpvalue);
					}
					ZD1205_WRITE_REGISTER(Adapter,CR190, 0x0);
					ReleaseCtrOfPhyReg(Adapter);
				}
				else if (IoAddress == 0xa06){
					Adapter->PSMSupported = (BOOLEAN)IoValue;
				}
				else if (IoAddress == 0xa07){
				}
				else if (IoAddress == 0xa08){
					// Dump CAM memory
					ZD1205_READ_REGISTER(Adapter,CAM_MODE, &IoValue);				
					ZD1205_WRITE_REGISTER(Adapter,CAM_MODE, (IoValue | HOST_RW));
					for (i=0;i<424;i++){
						ZD1205_WRITE_REGISTER(Adapter,CAM_ADDRESS, i);
						// Read twice to ensure valid data
						ZD1205_READ_REGISTER(Adapter,CAM_DATA, &tmpvalue);
						ZD1205_READ_REGISTER(Adapter,CAM_DATA, &tmpvalue);
					}
					ZD1205_WRITE_REGISTER(Adapter,CAM_MODE, IoValue);
				}
				else if (IoAddress == 0xa09){
					ULONGLONG	Timer1;
					// PHY CR Registers Read/Write Test
#ifdef CE_ALL
					Timer1 = GetTickCount();
#else
					Timer1 = KeQueryInterruptTime();
#endif
					srand((UINT)Timer1);
					AcquireCtrOfPhyReg(Adapter);
					for (i=0;i<0x0200;i+=4){
						if ( (i==0x00) || ((i>=0xc8) && (i<=0xfc)) ||
							((i>=0x1cc) && (i<=0x1d8)) || ((i>=0x1e0) && (i<=0x1ec))){
							// Skip Read Only Register
							continue;
						}
						tmpkey = (UCHAR)rand();
						ZD1205_WRITE_REGISTER(Adapter,i, tmpkey);
						ZD1205_READ_REGISTER(Adapter,i, &tmpvalue);
						if (tmpvalue != tmpkey){
							ZDDbgPrint((_T("CR %x Failed (Wr: %x, Rd: %x)\n"), 
								i, tmpkey, tmpvalue));
						}
						else{
							if (IoValue == 0x1){
								ZDDbgPrint((_T("CR %x Success (Wr: %x, Rd: %x)\n"),
											i, tmpkey, tmpvalue));
							}
						}
					}
					ReleaseCtrOfPhyReg(Adapter);
					ZDDbgPrint((_T("PHY CR Registers Read/Write Test End\n")));
				}
				else if (IoAddress == 0xa0a){
					ULONGLONG	Timer1;
					// MAC Registers Read/Write Test
					ZDDbgPrint((_T("MAC Registers Read/Write Test Starting....\n")));
#ifdef CE_ALL
					Timer1 = GetTickCount();
#else
					Timer1 = KeQueryInterruptTime();
#endif
					srand((UINT)Timer1);
					for (i=0;i<NUM_REG_MASK;i++){
						tmpkey = (ULONG)rand();
						tmpkey |= (tmpkey << 16);
						tmpkey &= MacRegMaskTab[i].ReadWriteMask;
						ZD1205_WRITE_REGISTER(Adapter,MacRegMaskTab[i].Address, tmpkey);
						ZD1205_READ_REGISTER(Adapter,MacRegMaskTab[i].Address, &tmpvalue);
						tmpvalue &= MacRegMaskTab[i].ReadWriteMask;
						if (tmpvalue != tmpkey){
							ZDDbgPrint((_T("MAC %x Failed (Wr: %x, Rd: %x)\n"), 
								MacRegMaskTab[i].Address, tmpkey, tmpvalue));
						}
						else{
							if (IoValue == 0x1){
								ZDDbgPrint((_T("MAC %x Success (Wr: %x, Rd: %x)\n"),
											MacRegMaskTab[i].Address, tmpkey, tmpvalue));
							}
						}
					}
					ZDDbgPrint((_T("MAC Registers Read/Write Test End\n")));
				}
				else if (IoAddress == 0xa0b){
					ProcessBusAbortInterrupt(Adapter);
				}
				else if (IoAddress == 0xa0c){
					if (IoValue != 0){
						// Turn on Sniffer mode
						ZD1205_WRITE_REGISTER(Adapter,SnifferOn, 0x1);
						Adapter->TestFlag |= DUMP_RX_FRAME;
					}
					else{
						ZD1205_WRITE_REGISTER(Adapter,SnifferOn, 0x0);
						Adapter->TestFlag &= ~DUMP_RX_FRAME;
					}
				}
				else if (IoAddress == 0xa0d){
					ULONGLONG	Timer1;
					// EEPROM Read/Write Test
					ZDDbgPrint((_T("EEPROM Read/Write Testing...........\n")));
#ifdef CE_ALL
					Timer1 = GetTickCount();
#else
					Timer1 = KeQueryInterruptTime();
#endif
					srand((UINT)Timer1);
					for (tmpvalue=0;tmpvalue<IoValue;tmpvalue++){
						tmpkey = (ULONG)rand();
						tmpkey |= (tmpkey << 16);
						for (i=0;i<256;i++){
							ZD1205_WRITE_REGISTER(Adapter,E2P_SUBID+(i*4), tmpkey);
						}
						// Write to EEPROM
						ZD1205_WRITE_REGISTER(Adapter,EEPROM_PROTECT0, 0x55aa44bb);
						ZD1205_WRITE_REGISTER(Adapter,EEPROM_PROTECT1, 0x33cc22dd);
						ZD1205_WRITE_REGISTER(Adapter,ROMDIR, 0x422);

						// Sleep 
						for (i=0;i<1000;i++)
							NdisStallExecution(5000);

						// Reset Registers
						for (i=0;i<256;i++){
							ZD1205_WRITE_REGISTER(Adapter,E2P_SUBID+(i*4), 0);
						}

						// Reload EEPROM
						ZD1205_WRITE_REGISTER(Adapter,ROMDIR, 0x424);

						// Sleep
						for (i=0;i<1000;i++)
							NdisStallExecution(5000);

						// Check if right
						for (i=0;i<256;i++){
							ZD1205_READ_REGISTER(Adapter,E2P_SUBID+(i*4), &reg);
							if (reg != tmpkey){
								ZDDbgPrint((_T("EEPROM Addr (%x) error (Wr: %x, Rd: %x)\n"),
									E2P_SUBID+(i*4), tmpkey, reg));
							}
						}
					}
				}
				else if (IoAddress == 0xa0e){
					if (IoValue)
						Adapter->TestFlag |= LOCKED_CLOCK;
					else
						Adapter->TestFlag &= ~LOCKED_CLOCK;
				}
				else if (IoAddress == 0xa10){
					InitializeAdapter(Adapter, FALSE);
				}
				else if (IoAddress == 0xa11){
					PHY_Reg_Preset(Adapter);
				}
				else if (IoAddress == 0xa12){
					pulong   = (PULONG)(InformationBuffer);
					AcquireCtrOfPhyReg(Adapter);
					for (i=0;i<0x0330;i+=4){
						// Note the length should not exceed 256
						*(pulong+2) +=1; // in unit of ULONG
						ZD1205_READ_REGISTER(Adapter,i, &tmpvalue);
						*(pulong+3+(i/4)) = tmpvalue;
						ZDDbgPrint((_T("Addr %x = %x\n"), i, tmpvalue));
					}
					ReleaseCtrOfPhyReg(Adapter);			
				}
				else if (IoAddress == 0xa14){
					AcquireCtrOfPhyReg(Adapter);
					ZD1205_WRITE_REGISTER(Adapter, CR158, 0x01);
					ZD1205_READ_REGISTER(Adapter, CR153, &tmpvalue);
					ZDDbgPrint((_T("%d \n"), (tmpvalue)));
					ZD1205_READ_REGISTER(Adapter, CR154, &tmpvalue);
					ZDDbgPrint((_T("%d \n"), (tmpvalue)));
					for (i=0;i<16;i++){
						ZD1205_READ_REGISTER(Adapter, CR156, &tmpvalue);
						ZD1205_WRITE_REGISTER(Adapter, CR172, 0x0);
						ZDDbgPrint((_T("%2d : %d \n"), i, (signed char)tmpvalue));
					}
					ZD1205_WRITE_REGISTER(Adapter, CR158, 0x0);
					ReleaseCtrOfPhyReg(Adapter);			
				}
            //``JWEI 2003/12/23
                else if (IoAddress == 0xa15)
                {
                    Adapter->Ant_MonitorDur2 = (UCHAR) IoValue;
                }
                else if (IoAddress == 0xa16)
                {
                    Adapter->NiceSQThr = (UCHAR) IoValue;
					Adapter->NiceSQThr_OFDM = (UCHAR) IoValue;
                }
                else if (IoAddress == 0xa17)
                {
                    Adapter->EnableSwAntennaDiv = (BOOLEAN) IoValue;
                }
                else if (IoAddress == 0xa18)
                {
                    Adapter->Ant_MonitorDur1 = (UCHAR) IoValue;
                }
                else if (IoAddress == 0xa19) 
                {                           //JWEI 2004/02/13
                    Adapter->PhyTest = (UCHAR) IoValue;
                    if (Adapter->PhyTest & BIT_7)
                    {
                        AcquireCtrOfPhyReg(Adapter);
                        ZD1205_WRITE_REGISTER(Adapter, CR10, 0x89);
                        ReleaseCtrOfPhyReg(Adapter);
                    }
                }
            //JWEI 2004/02/25
                else if (IoAddress == 0xa1a)
                {
                    Adapter->EnableTxPwrCtrl = (BOOLEAN) IoValue;
                }
            //JWEI 2004/03/17
        #ifdef HOST_IF_USB
            #if fPROG_FLASH
                else if (IoAddress == 0xa1b)
                {

                    ////if (IoValue == 1)
                    ////    EraseFlash(Adapter);
                    ////else
                    ////    FlashSecErase(Adapter, (USHORT) IoValue);
                }
                else if (IoAddress == 0xa1c)
                {
                #if fMERGE_RX_FRAME
////                    if (!ChangeToFlashAble(Adapter))
////                        break;
                #endif
////                    ProgFlashFile(Adapter, L"WS11U.img", IoValue);
                }
                else if (IoAddress == 0xa1d)
                {
#define SEQ_BUF_SIZE    0x40
                CHAR    DownloadBuffer[SEQ_BUF_SIZE];
                USHORT  ii, addr0;
                USHORT  WriteIndex = 0;

                #if 0
                    AcquireCtrOfPhyReg(Adapter);
                    JWeiDbgPrint5(("Word Program.\n"));
                //    FlashCmdWrite(Adapter, 0x40);
                    ZD1211_READ_REGISTER(Adapter, CR203, &tmpvalue, TRUE);
                    #if 0
                    mFLASH_SET_EVEN_ADDR(tmpvalue);
                    mFILL_WRITE_REGISTER(((0) >> 1), 0x40);
                //    ZD1205_Sleep(1000);     // Sleep 1ms
                    mFILL_WRITE_REGISTER(((4) >> 1), 0x88);
                    #endif
                    #if 1
                    mFLASH_SET_ODD_ADDR(tmpvalue);
                    mFILL_WRITE_REGISTER(((0) >> 1), 0x40);
                    mFILL_WRITE_REGISTER(((4) >> 1), 0x99);
            #endif
                    ZD1211_WriteMultiRegister(Adapter, WriteAddr, WriteData, WriteIndex, FALSE);
                    ReleaseCtrOfPhyReg(Adapter);
                #endif

                #if 1
                    JWeiDbgPrint5(("Block Program.\n"));
                    addr0 = (USHORT) IoValue * 2;
                    #if 0
                    AcquireCtrOfPhyReg(Adapter);
                    ZD1211_READ_REGISTER(Adapter, CR203, &tmpvalue, TRUE);
                    mFLASH_SET_EVEN_ADDR(tmpvalue);
                    mFILL_WRITE_REGISTER(addr0, 0xE8);
                    mFILL_WRITE_REGISTER(addr0, 0x07);
                    for (ii = 0; ii < 4; ii ++)
                        mFILL_WRITE_REGISTER(addr0 + ii, 0x60 + ii * 2);

                    mFLASH_SET_ODD_ADDR(tmpvalue);
                    for (ii = 0; ii < 4; ii ++)
                        mFILL_WRITE_REGISTER(addr0 + ii, 0x60 + ii * 2 + 1);
                    mFILL_WRITE_REGISTER(0, 0xD0);
                    ZD1211_WriteMultiRegister(Adapter, WriteAddr, WriteData, WriteIndex, FALSE);

                //    for (ii = 0; ii < WriteIndex; ii ++)
                //        JWeiDbgPrint5(("[%04X=%04X], ", WriteAddr[ii], WriteData[ii]));
                //    JWeiDbgPrint5(("--> %d\n", WriteIndex));
                    ReleaseCtrOfPhyReg(Adapter);

            #else
                        #if fMERGE_RX_FRAME && 0
////                    if (!ChangeToFlashAble(Adapter))
////                        break;
                        #endif
                //    for (ii = 0; ii < SEQ_BUF_SIZE / 4; ii ++)
                //        ((PULONG) DownloadBuffer)[ii] = 0x13572400L + ii;
                    for (ii = 0; ii < SEQ_BUF_SIZE; ii ++)
                        DownloadBuffer[ii] = 0x60 + ii;
////                    ProgFlash(Adapter, IoValue * 2, SEQ_BUF_SIZE, DownloadBuffer);
                    #endif
                //    DumpReadMultipleReg(Adapter, 0);
            #endif
                }
            #endif
            #ifdef fREAD_MUL_REG
                else if (IoAddress == 0xa1e)
				{
                    ////DumpReadMultipleReg(Adapter, (USHORT) IoValue);
				}
            #endif
            #if fPROG_FLASH
                else if (IoAddress == 0xa1f)
                {
                USHORT  ii;

                    AcquireCtrOfPhyReg(Adapter);
                    Adapter->AllowAccessRegister = FALSE;
                    ////InitHighAddr(Adapter);
                #if fREAD_MUL_REG
                /* Dump Flash data */
                    for (ii = 0; ii < 0x40; ii += 0x10)
                    {
                        /////low_addr = SetAbsAddr(Adapter, IoValue + ii, NULL);
                        ////DumpReadMultipleReg(Adapter, low_addr);
                    }
                #endif
                    Adapter->AllowAccessRegister = TRUE;
                    ReleaseCtrOfPhyReg(Adapter);
                }
                else if (IoAddress == 0xa20)
                {
                    AcquireCtrOfPhyReg(Adapter);

                    ////FlashCmdWrite(Adapter, (UCHAR) IoValue);
                    ReleaseCtrOfPhyReg(Adapter);
                }
            #endif
        #endif
				else if (IoAddress == 0xa21){
					Adapter->RateLine1 = (USHORT)IoValue;
				}
				else if (IoAddress == 0xa22){
					Adapter->RateLine2 = (USHORT)IoValue;
				}
				else if (IoAddress == 0xa23){
					Adapter->RateLine3 = (USHORT)IoValue;
				}
				else if (IoAddress == 0xa24){
					Adapter->RiseConditionThreshold = (USHORT)IoValue;
				}
				else if (IoAddress == 0xa25){
					Adapter->DownConditionThreshold = (USHORT)IoValue;
				}
            #ifdef fREAD_MUL_REG
                else if (IoAddress == 0xa26)
                {                           //JWEI 2004/05/14
                UCHAR  addr1, addr2, data1, data2;

                    data1 = (UCHAR) (IoValue & 0xFF);
                    addr1 = (UCHAR) ((IoValue >> 8) & 0xFF);
                    data2 = (UCHAR) ((IoValue >> 16) & 0xFF);
                    addr2 = (UCHAR) ((IoValue >> 24) & 0xFF);
                    JWeiDbgPrint5(("%02X=%02X; %02X=%02X\t", addr1, data1, addr2, data2));
                    ZD1211_READ_REGISTER(Adapter, addr2, &tmpvalue, FALSE);
                    JWeiDbgPrint5(("%08X\n", tmpvalue));
                }
                else if (IoAddress == 0xa27)
                {
                USHORT  addr1, data1;
                USHORT  WriteAddr[cMAX_MULTI_WRITE_REG_NUM];
                USHORT  WriteData[cMAX_MULTI_WRITE_REG_NUM];
                USHORT  WriteIndex = 0;

                    data1 = (USHORT) (IoValue & 0xFFFF);
                    addr1 = (USHORT) ((IoValue >> 16) & 0xFFFF);
                    JWeiDbgPrint5(("%04X=%04X\n", addr1, data1));
                    mFILL_WRITE_REGISTER(addr1, data1);
                    ZD1211_WriteMultiRegister(Adapter, WriteAddr, WriteData, WriteIndex, FALSE);
                }
            #endif
                else if (IoAddress == 0xa28)
                {
                /* IoValue is in unit of dB */
                    Adapter->SetPointLevel = (USHORT) IoValue * 8;
                    Adapter->IntValueLevel = (USHORT) IoValue * 4;
                    UpdateIntegrationValue(Adapter, Adapter->CardSetting.Channel);
                }
				else if (IoAddress == 0xa29)
				{
					Adapter->bUseSignalQuality4AntDiv=(BOOLEAN)IoValue;
				}
				else if (IoAddress == 0xa2a)
				{
					AcquireCtrOfPhyReg(Adapter);
					
					//ZD1205_READ_REGISTER(Adapter, CR10, &tmpvalue);
					if (IoValue==0) 
					{ // Fix at Antenna 0
						if (Adapter->CurrentAntenna==1)
							SwitchAntenna(Adapter);
						Adapter->EnableSwAntennaDiv=FALSE;
#if DBG_ANTDIV
						DbgPrint("   Fix at Ant %d\n", Adapter->CurrentAntenna);
#endif
					}
					else if (IoValue==1)
					{ // Fix at Antenna 1
						if (Adapter->CurrentAntenna==0)
							SwitchAntenna(Adapter);
						Adapter->EnableSwAntennaDiv=FALSE;
#if DBG_ANTDIV
						DbgPrint("   Fix at Ant %d\n", Adapter->CurrentAntenna);
#endif

					}
					else if (IoValue ==2)
					{ // Enable Software Antenna diversity.
#if DBG_ANTDIV
						DbgPrint(" Enable SW Antenna diversity\n");
#endif
						Adapter->EnableSwAntennaDiv=TRUE;
					}
				}
				else if (IoAddress == 0xa2b)
				{
					Adapter->MinSamplesInMT2=IoValue;
				}
				else if (IoAddress == 0xa2c)
				{
					Adapter->bTxFoloRx=(BOOLEAN)IoValue;
				}
				else if ((IoAddress & 0x7f00)== 0xf00){
					IoAddress &= 0xff;					
					ZD1205_READ_REGISTER(Adapter,CAM_MODE, &IoValue);
					
					ZD1205_WRITE_REGISTER(Adapter,CAM_MODE, (IoValue | HOST_RW));
					ZD1205_WRITE_REGISTER(Adapter,CAM_ADDRESS, IoAddress);
					// Read twice to ensure valid data
					ZD1205_READ_REGISTER(Adapter,CAM_DATA, &tmpvalue);
					ZD1205_READ_REGISTER(Adapter,CAM_DATA, &tmpvalue);
					
					ZD1205_WRITE_REGISTER(Adapter,CAM_MODE, IoValue);
					ZDDbgPrint((_T("CAMAddr %x = %x\n"), IoAddress, tmpvalue));
				}
				else if ((IoAddress & 0x7000) == 0x1000){
					// Set Phy Registers
					IoAddress &= 0xfff;
					AcquireCtrOfPhyReg(Adapter);
	
					INITSTR((_T("Write Phy %x = %x\n"), IoAddress, IoValue));
					ZD1205_WRITE_REGISTER(Adapter,IoAddress, IoValue);
					
					ReleaseCtrOfPhyReg(Adapter);
				}
				else{
					Write_Enable = TRUE;
				}
/*
				if (Write_Enable){
					*(PULONG)((UCHAR*)Adapter->CSRAddress+IoAddress) = (ULONG)IoValue;
				}
*/			}
			else{						// Read Command
				//for (i=0; i<IoValue; i++){
				//	reg = *((PULONG)((UCHAR*)Adapter->CSRAddress+IoAddress));
				//	ZDDbgPrint((_T("%8x  "), reg));
				//}
			}
			*BytesRead = InformationBufferLength;
			Status = NDIS_STATUS_SUCCESS;

            break;
		case OID_ZD_IO_WRITE:
			IoAddress = (ULONG)(*((PULONG)InformationBuffer) & 0xffffffff);
            IoValue   = (ULONG)(*((PULONG)InformationBuffer+ 1) );
        //JWEI 2004/02/10
        #ifdef HOST_IF_USB
            if ((IoAddress & 0xF000) != 0)
            {
                IoValue = IoAddress >> 16;
                IoAddress = IoAddress & 0xFFFF;
                ZD1211_WRITE_REGISTER(Adapter, IoAddress, IoValue, FALSE);
            }
            else
        #endif
            {
                //if (!NeedWriteProtection(Adapter, IoAddress, IoValue))
                //    ZD1205_WRITE_REGISTER(Adapter,IoAddress, IoValue);
			}
			ZDDbgPrint((_T("Write IoAddress %x with IoValue %x\n"), IoAddress, IoValue));
			Status = NDIS_STATUS_SUCCESS;
			break;
		case OID_ZD_SET_TALLIES:
			Status = NDIS_STATUS_SUCCESS;
			break;
        case OID_ZD_TEST:
            INITSTRTX(DebugPrint, (_T("OID_ZD_TEST\n")));
            zdval = InformationBuffer;
			if ( *zdval == 0 )                 
			    Adapter->ZDConfigRunning = 0;
            else
			    Adapter->ZDConfigRunning = 1;
                
			INITSTRTX(DebugPrint, (_T("ZDConfig = %x\n"), Adapter->ZDConfigRunning));
			Status = NDIS_STATUS_SUCCESS;

			break;
		case OID_ZD_DOWNLOAD_ROM:
			pImageBlockStruct = (PZD_IMAGE_BLOCK_STRUCT)InformationBuffer;
#ifdef HOST_IF_USB

			////Status = USB_Write_EEPROM(	Adapter,
			////	(PUSHORT)pImageBlockStruct->buffer, 
			////	InformationBufferLength+1-sizeof(ZD_IMAGE_BLOCK_STRUCT));

			break;
#endif
			for (i=0;i<((InformationBufferLength+1-sizeof(ZD_IMAGE_BLOCK_STRUCT)));i+=8){
				tmpvalue = pImageBlockStruct->buffer[i];
				tmpvalue |= ((ULONG)pImageBlockStruct->buffer[i+2] << 8);
				tmpvalue |= ((ULONG)pImageBlockStruct->buffer[i+4] << 16);
				tmpvalue |= ((ULONG)pImageBlockStruct->buffer[i+6] << 24);
				ZDDbgPrint((_T(" Write EEPROM %x with %x\n"), E2P_SUBID+(i/2), tmpvalue));
				ZD1205_WRITE_REGISTER(Adapter,E2P_SUBID+(i/2), tmpvalue);
			}
			ZD1205_WRITE_REGISTER(Adapter,EEPROM_PROTECT0, 0x55aa44bb);
			ZD1205_WRITE_REGISTER(Adapter,EEPROM_PROTECT1, 0x33cc22dd);
			ZD1205_WRITE_REGISTER(Adapter,ROMDIR, 0x422);
			Status = NDIS_STATUS_SUCCESS;
			break;
		case OID_ZD_CMD:
			INITSTRTX(DebugPrint, (_T("OID_ZD_CMD\n")));
			cmd_p = (PZD_CMD_STRUCT)InformationBuffer;
			Status = NDIS_STATUS_SUCCESS;
			switch(cmd_p->u.send.cmd){
				case 0:
					break;
				case 1:
					INITSTR((_T("Enable NIC card\n")));
					
					//++ Turn on RF
					TurnOn_RF(Adapter);
					//--
					// Connect to network
					Mgt_SendNotification(Adapter, RQ_CONNECTNETWORK);

					break;
				case 2:
					INITSTR((_T("Disable NIC card\n")));
					//++ Turn off RF
					TurnOff_RF(Adapter);
					//--
					Adapter->adapterReady = FALSE;
					ZDDbgPrint((_T("Adapter DisConnect! - 2\n")));
					Mgt_SendNotification(Adapter, RQ_DISCONNECT);
					break;
				case 0x31:
					// Write PHY registers
					AcquireCtrOfPhyReg(Adapter);

					switch(cmd_p->u.send.param0){
						case 8:		//CR4
							IoAddress = 0x20;
							break;
						case 10:	//CR5
							IoAddress = 0x10;
							break;
						case 12:	//CR6
							IoAddress = 0x14;
							break;
						case 14:	//CR7
							IoAddress = 0x18;
							break;
						case 16:	//CR8
							IoAddress = 0x1C;
							break;
						case 264:	//CR132
							IoAddress = 0x0210;

						default:
							IoAddress = (cmd_p->u.send.param0 << 1);
							break;
					}
					//INITSTR((_T("Write Phy Reg (%x) = %x\n"), IoAddress, 
					//									cmd_p->u.send.param1));
					//if (!NeedWriteProtection(Adapter, IoAddress, cmd_p->u.send.param1)){
					//ZD1205_WRITE_REGISTER(Adapter,IoAddress, cmd_p->u.send.param1);
					//}

					ReleaseCtrOfPhyReg(Adapter);
					break;
				case 0x33:
					IoAddress = cmd_p->u.send.param0;
					IoValue = (cmd_p->u.send.param1 + ((ULONG)cmd_p->u.send.param2 << 16));
					//if (!NeedWriteProtection(Adapter, IoAddress, IoValue)){
					//ZD1205_WRITE_REGISTER(Adapter,IoAddress, IoValue);
					//}
					break;
				case CMD_ContinuousTx:
					ZDDbgPrint((_T("CMD_ContinuousTx\n")));
					if(cmd_p->u.send.param0){	// Start
						switch(cmd_p->u.send.param1){
							case 0x0:	// Normal continous transmit
								ZDDbgPrint((_T("Normal continous\n")));
								if (Adapter->CardSetting.TxRate >= 4){
									AcquireCtrOfPhyReg(Adapter);
									ZD1205_WRITE_REGISTER(Adapter,CR159, 4);
									ReleaseCtrOfPhyReg(Adapter);

									ZD1205_WRITE_REGISTER(Adapter,0x644, 7);
								}
								else{
									AcquireCtrOfPhyReg(Adapter);
									ZD1205_READ_REGISTER(Adapter,CR22, &tmpvalue);
									tmpvalue |= BIT_7;
									ZD1205_WRITE_REGISTER(Adapter,CR22, tmpvalue);
									ReleaseCtrOfPhyReg(Adapter);

									ZD1205_WRITE_REGISTER(Adapter,0x644, 3);
								}
								
								break;
							case 0x1:	// CW transmit
								ZDDbgPrint((_T("Start CW transmit\n")));
								AcquireCtrOfPhyReg(Adapter);
								ZD1205_READ_REGISTER(Adapter,CR32, &tmpvalue);
								tmpvalue &= ~0x4;
								ZD1205_WRITE_REGISTER(Adapter,CR32, tmpvalue);

								ZD1205_READ_REGISTER(Adapter,CR9, &tmpvalue);
								tmpvalue |= 0x18;
								ZD1205_WRITE_REGISTER(Adapter,CR9, tmpvalue);
								ReleaseCtrOfPhyReg(Adapter);
								
								ZD1205_WRITE_REGISTER(Adapter,RFCFG, 0x4);
								while(1){
									ZD1205_READ_REGISTER(Adapter, NAV_CCA, &tmpvalue);
									if (tmpvalue & BIT_25){
										NdisStallExecution(1);
									}
									else{
										break;
									}
								}
								ZD1205_WRITE_REGISTER(Adapter,RFCFG, 0x5);
								break;
							case 0x2:	// Carrier suppression
								ZDDbgPrint((_T("Start Carrier suppression transmit\n")));
								AcquireCtrOfPhyReg(Adapter);
								ZD1205_READ_REGISTER(Adapter,CR9, &tmpvalue);
								tmpvalue &= ~0x10;
								tmpvalue |= 0x8;
								ZD1205_WRITE_REGISTER(Adapter,CR9, tmpvalue);

								ZD1205_READ_REGISTER(Adapter,CR32, &tmpvalue);
								tmpvalue |= 0x4;
								ZD1205_WRITE_REGISTER(Adapter,CR32, tmpvalue);
								ReleaseCtrOfPhyReg(Adapter);
								
								ZD1205_WRITE_REGISTER(Adapter,RFCFG, 0x4);
								while(1){
									ZD1205_READ_REGISTER(Adapter, NAV_CCA, &tmpvalue);
									if (tmpvalue & BIT_25){
										NdisStallExecution(1);
									}
									else{
										break;
									}
								}
								ZD1205_WRITE_REGISTER(Adapter,RFCFG, 0x5);
								break;
							default:
								break;
						}
					}
					else{						// Stop
						switch(cmd_p->u.send.param1){
							case 0x0:	// Normal continous transmit
								ZDDbgPrint((_T("Stop Normal Continuous Transmit\n")));
								
								if (Adapter->CardSetting.TxRate >= 4){
									AcquireCtrOfPhyReg(Adapter);
									ZD1205_WRITE_REGISTER(Adapter,CR159, 0);
									ReleaseCtrOfPhyReg(Adapter);

									ZD1205_WRITE_REGISTER(Adapter,0x644, 0);
								}
								else{
									AcquireCtrOfPhyReg(Adapter);
									ZD1205_WRITE_REGISTER(Adapter,CR22, 0x23);
									ReleaseCtrOfPhyReg(Adapter);

									ZD1205_WRITE_REGISTER(Adapter,0x644, 0);
								}
								
								break;
							case 0x1:	// CW transmit
								ZDDbgPrint((_T("Stop CW transmit\n")));
								AcquireCtrOfPhyReg(Adapter);
								ZD1205_READ_REGISTER(Adapter,CR9, &tmpvalue);
								tmpvalue &= ~0x18;
								ZD1205_WRITE_REGISTER(Adapter,CR9, tmpvalue);
								ReleaseCtrOfPhyReg(Adapter);
								
								ZD1205_WRITE_REGISTER(Adapter,RFCFG, 0x0);
								break;
							case 0x2:	// Carrier suppression
								ZDDbgPrint((_T("Stop Carrier suppression transmit\n")));
								AcquireCtrOfPhyReg(Adapter);
								ZD1205_READ_REGISTER(Adapter,CR9, &tmpvalue);
								tmpvalue &= ~0x18;
								ZD1205_WRITE_REGISTER(Adapter,CR9, tmpvalue);

								ZD1205_READ_REGISTER(Adapter,CR32, &tmpvalue);
								tmpvalue &= ~0x4;
								ZD1205_WRITE_REGISTER(Adapter,CR32, tmpvalue);
								ReleaseCtrOfPhyReg(Adapter);
								
								ZD1205_WRITE_REGISTER(Adapter,RFCFG, 0x0);
								break;
							default:
								break;
						}
					}
					break;
				case CMD_AdjustPSTxPower:
					AcquireCtrOfPhyReg(Adapter);
                    ZD1205_READ_REGISTER(Adapter,CR47, &tmpvalue);
					ReleaseCtrOfPhyReg(Adapter);
					Adapter->TxGainSetting = (UCHAR)tmpvalue;
					ZDDbgPrint((_T("Current Tx Gain Setting = %x\n"), Adapter->TxGainSetting));
					ZDDbgPrint((_T("Delta = %x\n"), cmd_p->u.send.param0));
                    if (cmd_p->u.send.param0 == 0x1)
                    {                       //``JWEI 2003/12/22
                        DecreaseTxPower(Adapter, cTX_CCK);
                    }
                    else if (cmd_p->u.send.param0 == 0xffff)
                    {
                        IncreaseTxPower(Adapter, cTX_CCK);
					}
					cmd_p->u.rsp.resp0 = (USHORT)Adapter->TxGainSetting;
					tmpvalue = Adapter->TxGainSetting;
					//GCT
					AcquireCtrOfPhyReg(Adapter);
                    ZD1205_WRITE_REGISTER(Adapter,CR47, tmpvalue);
					ReleaseCtrOfPhyReg(Adapter);

					break;
				case CMD_RateControl:
					
					Adapter->CardSetting.TxRate = (UCHAR)cmd_p->u.send.param0;
					
					AcquireCtrOfPhyReg(Adapter);

					if ( Adapter->preambleMode == 1 )
						Adapter->CardSetting.PreambleType = 0x00;
					else if ( Adapter->preambleMode == 2 )
						Adapter->CardSetting.PreambleType = 0x20;

					if (Adapter->CardSetting.PreambleType == 0x20){
						// short premable
						cmd_p->u.send.param0 |= BIT_5;
					}
					else{
						// long premable
						cmd_p->u.send.param0 &= ~BIT_5;
					}
					if (Adapter->CountrySelect == 1) cmd_p->u.send.param0 &= ~BIT_6;//USA
					if (Adapter->CountrySelect == 2) cmd_p->u.send.param0 |= BIT_6;	//japan

					ZD1205_WRITE_REGISTER(Adapter,CR5, cmd_p->u.send.param0);
					ReleaseCtrOfPhyReg(Adapter);
					Adapter->RateLock = 1;
					break;
				case CMD_DOZE_FOREVER:
					ZD1205_WRITE_REGISTER(Adapter, PS_Ctrl, 0x3);
					break;
				default:
					Status = NDIS_STATUS_NOT_SUPPORTED;
					break;
			}
			*BytesRead = InformationBufferLength;
			break;		
        case OID_ZD_SETRID:
			INITSTRTX(DebugPrint, (_T("OID_ZD_SETRID\n")));
            rid_p = (PRID_STRUCT)InformationBuffer;			
			Status = NDIS_STATUS_SUCCESS;

			switch(rid_p->rid){
			case RID_CFG_PREAMBLE: // long = 1, short = 2, auto = 3
				Adapter->preambleMode = (UCHAR) rid_p->u.data[0];
				break;
			case RID_CFG_PORT_TYPE:
				switch (rid_p->u.data[0]){
					case INDEPENDENT_BSS:	// IBSS mode
						INITSTR((_T("IBSS mode\n")));
						Adapter->CardSetting.BSSType = INDEPENDENT_BSS;
						break;
					case INFRASTRUCTURE_BSS:	// Infrastructure mode
						INITSTR((_T("Infrastructure mode\n")));
						Adapter->CardSetting.BSSType = INFRASTRUCTURE_BSS;
						break;
					case PSEUDO_IBSS:	// Pseudo IBSS mode
						INITSTR((_T("Pseudo IBSS mode\n")));
						Adapter->CardSetting.BSSType = PSEUDO_IBSS;
						break;
					default:
						ASSERT(0);
						break;
				}
				break;
			case RID_TX_RATE:
				Adapter->RateLock = 1;
				switch (rid_p->u.data[0]){
#ifndef OFDM
					case 0x2000:
						INITSTR((_T("TxRate = 8.25Mbps\n")));
						Adapter->CardSetting.TxRate = 0xd;
						break;
					case 0x1000:	// 60.5Mbps
						INITSTR((_T("TxRate = 60.5Mbps\n")));
						Adapter->CardSetting.TxRate = 0xc;
						break;
					case 0x800:	// 55Mbps
						INITSTR((_T("TxRate = 55Mbps\n")));
						Adapter->CardSetting.TxRate = 0xb;
						break;
					case 0x400:	// 49.5Mbps
						INITSTR((_T("TxRate = 49.5Mbps\n")));
						Adapter->CardSetting.TxRate = 0xa;
						break;
					case 0x200:	// 44Mbps
						INITSTR((_T("TxRate = 44Mbps\n")));
						Adapter->CardSetting.TxRate = 9;
						break;
					case 0x100:	// 38.5Mbps
						INITSTR((_T("TxRate = 38.5Mbps\n")));
						Adapter->CardSetting.TxRate = 8;
						break;
					case 0x80:	// 33Mbps
						INITSTR((_T("TxRate = 33Mbps\n")));
						Adapter->CardSetting.TxRate = 7;
						break;
					case 0x40:	// 27.5Mbps
						INITSTR((_T("TxRate = 27.5Mbps\n")));
						Adapter->CardSetting.TxRate = 6;
						break;
					case 0x20:	// 22Mbps
						INITSTR((_T("TxRate = 22Mbps\n")));
						Adapter->CardSetting.TxRate = 5;
						break;
					case 0x10:	// 16.5M
						INITSTR((_T("TxRate = 16.5Mbps\n")));
						Adapter->CardSetting.TxRate = 4;
						break;
#else
					case 0x800:	// 54 Mbps
						INITSTR((_T("TxRate = 54Mbps\n")));
						Adapter->CardSetting.TxRate = 0xb;
						break;
					case 0x400:	// 48 Mbps
						INITSTR((_T("TxRate = 48Mbps\n")));
						Adapter->CardSetting.TxRate = 0xa;
						break;
					case 0x200:	// 36 Mbps
						INITSTR((_T("TxRate = 36Mbps\n")));
						Adapter->CardSetting.TxRate = 9;
						break;
					case 0x100:	// 24 Mbps
						INITSTR((_T("TxRate = 24Mbps\n")));
						Adapter->CardSetting.TxRate = 8;
						break;
					case 0x80:	// 18 Mbps
						INITSTR((_T("TxRate = 18Mbps\n")));
						Adapter->CardSetting.TxRate = 7;
						break;
					case 0x40:	// 12 Mbps
						INITSTR((_T("TxRate = 12Mbps\n")));
						Adapter->CardSetting.TxRate = 6;
						break;
					case 0x20:	// 9 Mbps
						INITSTR((_T("TxRate = 9Mbps\n")));
						Adapter->CardSetting.TxRate = 5;
						break;
					case 0x10:	// 6 M
						INITSTR((_T("TxRate = 6Mbps\n")));
						Adapter->CardSetting.TxRate = 4;
						break;
#endif
					case 15: // Full automatic
						INITSTR((_T("TxRate = Full automatic\n")));
#ifdef OFDM
						Adapter->CardSetting.TxRate = Adapter->CardSetting.HighestTxRate;
#else
						Adapter->CardSetting.TxRate = 3;
#endif
						Adapter->RateLock = 0;
						break;
					case 8:	 // 11 M
						INITSTR((_T("TxRate = 11 Mbps\n")));
						Adapter->CardSetting.TxRate = 3;
						break;
					case 4:	 // 5.5 M
						INITSTR((_T("TxRate = 5.5 Mbps\n")));
						Adapter->CardSetting.TxRate = 2;
						break;
					case 3:  // Auto 1, 2 M
						INITSTR((_T("TxRate = Auto 1, 2 Mbps\n")));
						Adapter->CardSetting.TxRate = 1;
						break;
					case 2:	 // 2 M
						INITSTR((_T("TxRate = 2 Mbps\n")));
						Adapter->CardSetting.TxRate = 1;
						break;
					case 1:  // 1 M
						INITSTR((_T("TxRate = 1 Mbps\n")));
						Adapter->CardSetting.TxRate = 0;
						break;
				}
				break;
			case RID_CFG_CHANNEL:
				INITSTR((_T("Channel = %x\n"), rid_p->u.data[0]));
				Set_RF_Channel(Adapter, (UCHAR)rid_p->u.data[0], (UCHAR)Adapter->RF_Mode, 1);  
				break;
			case RID_CFG_WEP_FLAGS:  // not used
				// rid_p->u.data[0] : bit1,0  (1,1) Enable WEP
				//							  (0,0) Disable WEP
				//					  bit6,5  (0,0) Increment IV
				//							  (0,1) Random IV
				//					  bit8	  (0)   64-bit WEP
				//							  (1)   128-bit WEP
				//Adapter->WepOption = rid_p->u.data[0];
				if ( (rid_p->u.data[0] & 0x103) == 0x3 ){
					INITSTR((_T("WEP64\n")));
					Change_EncryMode(Adapter, WEP64, WEP64);
				}
				else if ((rid_p->u.data[0] & 0x103) == 0x103){
					INITSTR((_T("WEP128\n")));
					Change_EncryMode(Adapter, WEP128, WEP128);
				}
				else{
					INITSTR((_T("NO_WEP\n")));
					Change_EncryMode(Adapter, NO_WEP, NO_WEP);
				}

				if ( (rid_p->u.data[0] & 0x60) == 0x00 ){
					INITSTR((_T("WepIvType = INCREMENT\n")));
					Adapter->CardSetting.WepIvType = INCREMENT;
				}
				else{
					INITSTR((_T("WepIvType = RANDOM\n")));
					Adapter->CardSetting.WepIvType = RANDOM;
				}
				break;
			case RID_CFG_DEFAULT_KEY_1:
				INITSTR((_T("Set Key 1\n")));
				pKey = (PUCHAR)(&rid_p->u.data[0]);
				if ( Adapter->CardSetting.EncryMode == WEP64 ){
					memcpy(Adapter->AllKeyBuf, pKey, 5);
					loop = 20;
					SetKeyContext(Adapter, DEFAULT_KEY_SET, WEP64, 0, pKey);
				}
				else if (Adapter->CardSetting.EncryMode == WEP128){
					memcpy(Adapter->AllKeyBuf, pKey, 13);
					loop = 52;
					SetKeyContext(Adapter, DEFAULT_KEY_SET, WEP128, 0, pKey);
				}
				else{
					loop = 0;
				}
#ifndef AMAC
				for (i=0;i<loop;i+=4){
					ZD1205_READ_REGISTER(Adapter,WEPKey0+i, &tmpkey);
					tmpkey &= ~0x000000ff;
					tmpkey |= ((ULONG)*(pKey) << 0);
					pKey++;
					ZD1205_WRITE_REGISTER(Adapter,WEPKey0+i, tmpkey);
				}
#endif
				break;
			case RID_CFG_DEFAULT_KEY_2:
				INITSTR((_T("Set Key 2\n")));
				pKey = (PUCHAR)(&rid_p->u.data[0]);
				if ( Adapter->CardSetting.EncryMode == WEP64 ){
					memcpy(Adapter->AllKeyBuf+5, pKey, 5);
					loop = 20;
					SetKeyContext(Adapter, DEFAULT_KEY_SET, WEP64, 1, pKey);
				}
				else if (Adapter->CardSetting.EncryMode == WEP128){
					memcpy(Adapter->AllKeyBuf+13, pKey, 13);
					loop = 52;
					SetKeyContext(Adapter, DEFAULT_KEY_SET, WEP128, 1, pKey);
				}
				else{
					loop = 0;
				}
#ifndef AMAC
				for (i=0;i<loop;i+=4){
					ZD1205_READ_REGISTER(Adapter,WEPKey0+i, &tmpkey);
					tmpkey &= ~0x0000ff00;
					tmpkey |= ((ULONG)*(pKey) << 8);
					pKey++;
					ZD1205_WRITE_REGISTER(Adapter,WEPKey0+i, tmpkey);
				}
#endif
				break;
			case RID_CFG_DEFAULT_KEY_3:
				INITSTR((_T("Set Key 3\n")));
				pKey = (PUCHAR)(&rid_p->u.data[0]);
				if ( Adapter->CardSetting.EncryMode == WEP64 ){
					memcpy(Adapter->AllKeyBuf+10, pKey, 5);
					loop = 20;
					SetKeyContext(Adapter, DEFAULT_KEY_SET, WEP64, 2, pKey);
				}
				else if (Adapter->CardSetting.EncryMode == WEP128){
					memcpy(Adapter->AllKeyBuf+26, pKey, 13);
					loop = 52;
					SetKeyContext(Adapter, DEFAULT_KEY_SET, WEP128, 2, pKey);
				}
				else{
					loop = 0;
				}
#ifndef AMAC
				for (i=0;i<loop;i+=4){
					ZD1205_READ_REGISTER(Adapter,WEPKey0+i, &tmpkey);
					tmpkey &= ~0x00ff0000;
					tmpkey |= ((ULONG)*(pKey) << 16);
					pKey++;
					ZD1205_WRITE_REGISTER(Adapter,WEPKey0+i, tmpkey);
				}
#endif
				break;
			case RID_CFG_DEFAULT_KEY_4:
				INITSTR((_T("Set Key 4\n")));
				pKey = (PUCHAR)(&rid_p->u.data[0]);
				if ( Adapter->CardSetting.EncryMode == WEP64 ){
					memcpy(Adapter->AllKeyBuf+15, pKey, 5);
					loop = 20;
					SetKeyContext(Adapter, DEFAULT_KEY_SET, WEP64, 3, pKey);
				}
				else if (Adapter->CardSetting.EncryMode == WEP128){
					memcpy(Adapter->AllKeyBuf+39, pKey, 13);
					loop = 52;
					SetKeyContext(Adapter, DEFAULT_KEY_SET, WEP128, 3, pKey);
				}
				else{
					loop = 0;
				}
#ifndef AMAC
				for (i=0;i<loop;i+=4){
					ZD1205_READ_REGISTER(Adapter,WEPKey0+i, &tmpkey);
					tmpkey &= ~0xff000000;
					tmpkey |= ((ULONG)*(pKey) << 24);
					pKey++;
					ZD1205_WRITE_REGISTER(Adapter,WEPKey0+i, tmpkey);
				}
#endif
				break;
			case RID_CFG_DEFAULT_KEY_ID:
				INITSTR((_T("Set EncryKeyId = %x\n"), rid_p->u.data[0]));
				Adapter->CardSetting.EncryKeyId = (UCHAR)rid_p->u.data[0];
				break;
			case RID_CFG_SSID:
				INITSTR((_T("Set SSID = %s\n"), (char*)&rid_p->u.data[1]));
				strcpy(&Adapter->CardSetting.Info_SSID[2], (char*)&rid_p->u.data[1]);
				Adapter->CardSetting.Info_SSID[1] = (UCHAR)strlen((char*)&rid_p->u.data[1]);
				break;
			case RID_FRAG_THRESHOLD:
				INITSTR((_T("Set Fragmentation threshold %x\n"), rid_p->u.data[0]));
				Adapter->CardSetting.FragThreshold = (ULONG) rid_p->u.data[0] - ( rid_p->u.data[0] % 2 );
				break;
			case RID_RTS_THRESHOLD:
				INITSTR((_T("Set RTS threshold %x\n"), rid_p->u.data[0]));
				Adapter->CardSetting.RTSThreshold = (ULONG) rid_p->u.data[0];
				break;
			case RID_CFG_PM_EPS:
				INITSTR((_T("Set RID_CFG_PM_EPS %x\n"), rid_p->u.data[0]));
				Adapter->PSThreshhold = ((ULONG) rid_p->u.data[0]) * 5000;
				if (rid_p->u.data[0] == 4) Adapter->TestPS = 1;
				else  Adapter->TestPS = 0;
				INITSTR((_T("Current Adapter->PSThreshhold %d\n"), Adapter->PSThreshhold));
				break;
			case RID_CFG_PM_ENABLE:
				Adapter->PSMSupported = (BOOLEAN)rid_p->u.data[0];
				if ((BOOLEAN)rid_p->u.data[0]){
					Adapter->CardSetting.ATIMWindow = 0x5;
				}
				else{
					Adapter->CardSetting.ATIMWindow = 0x0;
				}
				tmpvalue = Adapter->CardSetting.BeaconInterval;
				UpdatePreTBTT(Adapter, tmpvalue-5);
				break;
			case RID_EEPROM_REFRESH:
				ZD1205_WRITE_REGISTER(Adapter,ROMDIR, 0x424);
				break;
			case RID_EEPROM_PWR_CAL_VAL:
				// set point
				ZDDbgPrint((_T("RID_EEPROM_WRITE_PWR_CAL_VAL\n")));
				for( i=0; i<4; i++ ){
				    tmpvalue = (UCHAR)rid_p->u.data[i*4];
				    tmpvalue |= ((ULONG)(rid_p->u.data[i*4+1] & 0xff) << 8);
					if ( i!=3 ){
				        tmpvalue |= ((ULONG)(rid_p->u.data[i*4+2] & 0xff) << 16);
				        tmpvalue |= ((ULONG)(rid_p->u.data[i*4+3] & 0xff) << 24);
					}
				    ZD1205_WRITE_REGISTER(Adapter,E2P_PWR_CAL_VALUE+i*4, tmpvalue);
                    ZDDbgPrint((_T("tmpvalue = %x\n"), tmpvalue));
                }
				// integration value
				for( i=0; i<4; i++ ){
				    tmpvalue = (UCHAR)rid_p->u.data[i*4+16];
				    tmpvalue |= ((ULONG)(rid_p->u.data[i*4+17] & 0xff) << 8);
					if ( i!=3 ){
				        tmpvalue |= ((ULONG)(rid_p->u.data[i*4+18] & 0xff) << 16);
				        tmpvalue |= ((ULONG)(rid_p->u.data[i*4+19] & 0xff) << 24);
					}
					ZD1205_WRITE_REGISTER(Adapter,E2P_PWR_CAL_VALUE+i*4+16, tmpvalue);
                    ZDDbgPrint((_T("tmpvalue = %x\n"), tmpvalue));
                }
				break;
			case RID_EEPROM_MACADDR:
				tmpvalue = (UCHAR)rid_p->u.data[0];
				tmpvalue |= ((ULONG)(rid_p->u.data[1] & 0xff) << 8);
				tmpvalue |= ((ULONG)(rid_p->u.data[2] & 0xff) << 16);
				tmpvalue |= ((ULONG)(rid_p->u.data[3] & 0xff) << 24);
				ZD1205_WRITE_REGISTER(Adapter,E2P_MACADDR_P1, tmpvalue);
				tmpvalue = (UCHAR)rid_p->u.data[4];
				tmpvalue |= ((ULONG)(rid_p->u.data[5] & 0xff) << 8);
				ZD1205_WRITE_REGISTER(Adapter,E2P_MACADDR_P2, tmpvalue);
				break;
			case RID_EEPROM_HW_VER:
				IoValue = (UCHAR)rid_p->u.data[0];
				IoValue |= ((ULONG)(rid_p->u.data[1] & 0xff) << 8);
				ZD1205_READ_REGISTER(Adapter,E2P_POD, &tmpvalue);
				tmpvalue &= 0x0000ffff;
				tmpvalue |= (IoValue << 16);
				ZD1205_WRITE_REGISTER(Adapter,E2P_POD, tmpvalue);
				break;
			case RID_EEPROM_ALLOW_CH:
				IoValue = (UCHAR)rid_p->u.data[0];
				IoValue |= ((ULONG)(rid_p->u.data[1] & 0xff) << 8);
				ZD1205_READ_REGISTER(Adapter,E2P_ALLOWED_CHANNEL, &tmpvalue);
				tmpvalue &= 0xffff0000;
				tmpvalue |= (IoValue);
				ZD1205_WRITE_REGISTER(Adapter,E2P_ALLOWED_CHANNEL, tmpvalue);
				break;
			case RID_EEPROM_DEFAULT_CH:
				IoValue = (UCHAR)rid_p->u.data[0];
				IoValue |= ((ULONG)(rid_p->u.data[1] & 0xff) << 8);
				ZD1205_READ_REGISTER(Adapter,E2P_ALLOWED_CHANNEL, &tmpvalue);
				tmpvalue &= 0x0000ffff;
				tmpvalue |= (IoValue << 16);
				ZD1205_WRITE_REGISTER(Adapter,E2P_ALLOWED_CHANNEL, tmpvalue);
				break;
			case RID_EEPROM_REGION_CODE:
        //JWEI 2003/12/31
        #ifndef HOST_IF_USB
                IoValue = (UCHAR)rid_p->u.data[0];
                ZD1205_READ_REGISTER(Adapter,E2P_REGION_CODE, &tmpvalue);
                tmpvalue &= 0xffff0000;
                tmpvalue |= (IoValue);
                ZD1205_WRITE_REGISTER(Adapter,E2P_REGION_CODE, tmpvalue);
        #endif
				break;
			case RID_EEPROM_SUBSYS_ID:
				tmpvalue = (UCHAR)rid_p->u.data[0];
				tmpvalue |= ((ULONG)(rid_p->u.data[1] & 0xff) << 8);
				tmpvalue |= ((ULONG)(rid_p->u.data[2] & 0xff) << 16);
				tmpvalue |= ((ULONG)(rid_p->u.data[3] & 0xff) << 24);
				ZD1205_WRITE_REGISTER(Adapter,E2P_SUBID, tmpvalue);
				break;
			case RID_EEPROM_WRITE:
				// Write to EEPROM
				ZD1205_WRITE_REGISTER(Adapter,EEPROM_PROTECT0, 0x55aa44bb);
				ZD1205_WRITE_REGISTER(Adapter,EEPROM_PROTECT1, 0x33cc22dd);
				ZD1205_WRITE_REGISTER(Adapter,ROMDIR, 0x422);
				break;
			case RID_ANTENNA_SELECT:
				break;
			case RID_CFG_BASIC_RATES:
				break;
			case RID_TARCE_SET_POINT:
                ZDDbgPrint((_T("set RID_TARCE_SET_POINT\n")));
				if ( rid_p->u.data[0] & BIT_0 ){
				    Adapter->TraceSetPoint = 1;
				}else{
				    Adapter->TraceSetPoint = 0;
				}
				break;
			case RID_TX_POWER_SET_POINT:
				ZDDbgPrint((_T("set RID_TX_POWER_SET_POINT\n")));
				for( i=0; i<7; i++ ){
					Adapter->SetPoint[i*2] = rid_p->u.data[i] % 256;
                    Adapter->SetPoint[i*2+1] = rid_p->u.data[i] >> 8;
					ZDDbgPrint((_T("%x "), rid_p->u.data[i]));
				}
					ZDDbgPrint((_T("\n")));
				break;
			case RID_CHANNEL_LIST:
				Adapter->AllowedChannel &= ~0xffff;
				Adapter->AllowedChannel |= (USHORT)rid_p->u.data[0];
				ZDDbgPrint((_T("Set RID_CHANNEL_LIST, Adapter->AllowedChannel = %x\n"),
								Adapter->AllowedChannel));
				break;
			case RID_TxPowerGain:
				Write_TxGain(Adapter, (USHORT)rid_p->u.data[0]);
				break;
			case 0xFC7F:
				if ( rid_p->u.data[0] )
                    Adapter->PERPort = ((rid_p->u.data[0] << 7) & 0x7f00)|
                                       ((rid_p->u.data[0] >> 8) & 0xff);
				else
					Adapter->PERPort = 0;
				ZDDbgPrint((_T("Adapter->PERPort = %d\n"), Adapter->PERPort));
				break;
			case 0xFCB5:
				Adapter->MultiDomainCapability = rid_p->u.data[0];
				break;
			case 0xFCB6:
				Adapter->OperationRegionCode = rid_p->u.data[0];
				break;
			default:
				Status = NDIS_STATUS_NOT_SUPPORTED;
				break;
			}

			*BytesRead = InformationBufferLength;
            break;

	    case OID_ZD_RESET_TALLIES:
			Reset_Tally(Adapter);
			break;

		default:
			Status = NDIS_STATUS_NOT_SUPPORTED;
			break;
			
	}

	return Status;
}



NDIS_STATUS
ZD1211Old_QueryInformation(
                     IN NDIS_HANDLE MiniportAdapterContext,
                     IN NDIS_OID Oid,
                     IN PVOID InformationBuffer,
                     IN ULONG InformationBufferLength,
                     OUT PULONG BytesWritten,
                     OUT PULONG BytesNeeded
                     )

{
    PZD1205_ADAPTER		Adapter;
    NDIS_STATUS			Status = NDIS_STATUS_SUCCESS;
	ULONG				DebugPrint;
	PULONGLONG			pTally;
	PLOCAL_TALLY_STRUCT pLocalTally;
    PRID_STRUCT			rid_p;
	ULONG				IoAddress;
	PZD_CMD_STRUCT		cmd_p;
	ULONG				tmpvalue;
	UCHAR				tmpChar;
	PUCHAR				pch;
	USHORT				i,j;
	ZD1211_IRQL  		OldIrql;
    PNDIS_802_11_BSSID_LIST        pZDBSSIDList;
	ULONG				tmpUlong;


    DEBUGFUNC(_T("ZD1211Old_QueryInformation"));

    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_OID;
 		

    switch (Oid)
    {

	case OID_ZD_GET_TALLIES_1202:
		pTally = (PULONGLONG)InformationBuffer;
		*(pTally) = Adapter->Hw_TotalRxFrm;
		*(pTally+1) = Adapter->Hw_CRC32Cnt;
		*(pTally+2) = Adapter->Hw_CRC16Cnt;
		*(pTally+3) = Adapter->Hw_DecrypErr_UNI;
		*(pTally+4) = Adapter->Hw_DecrypErr_Mul;
		*(pTally+5) = Adapter->Hw_RxFIFOOverrun;
		*(pTally+6) = Adapter->DriverRxFrmCnt;
		*(pTally+7) = Adapter->DriverDiscardedFrm;
		*(pTally+8) = Adapter->LessThanDataMinLen;
		*(pTally+9) = Adapter->GreaterThanMaxLen;
		*(pTally+10) = Adapter->PLCP_MacLen_Mismatch;
		*(pTally+11) = Adapter->NotifyNDISRxFrmCnt;
		*(pTally+12) = Adapter->Hw_TotalTxFrm;
		*(pTally+13) = Adapter->Hw_UnderrunCnt;
		*(pTally+14) = Adapter->Hw_RetryCnt;
		*(pTally+15) = Adapter->RetryFailCnt;
		*(pTally+16) = Adapter->LossAP;
//		MoveBytes = 0;
		break;
	case OID_ZD_GET_TALLIES:
		pLocalTally = (PLOCAL_TALLY_STRUCT)InformationBuffer;
#ifdef HOST_IF_USB
		pLocalTally->reserved1 = Adapter->REG_6e4_Add;
#else
		pLocalTally->reserved1 = 0;
#endif
#if	(defined(GCCK) && defined(OFDM))
		pLocalTally->reserved2 = Adapter->rx11bDataFrame;
		pLocalTally->reserved3 = Adapter->rxOFDMDataFrame;
#else
		pLocalTally->reserved2 = 0;
		pLocalTally->reserved3 = 0;
#endif

#ifdef HOST_IF_USB
		pLocalTally->reserved4 = Adapter->Hw_TotalTxFrm;
#else
		pLocalTally->reserved4 = 0;
#endif
		pLocalTally->reserved7 = Adapter->rxBroadcastFrm;
		pLocalTally->reserved8 = Adapter->rxBroadcastOctets;
		pLocalTally->rxCRC32ErrCnt = Adapter->Hw_CRC32Cnt;
		pLocalTally->rxDecrypFailCnt = (Adapter->Hw_DecrypErr_UNI+
											Adapter->Hw_DecrypErr_Mul); 
		pLocalTally->rxDiscardedCnt = Adapter->DriverDiscardedFrm;
		pLocalTally->rxMulticastFrames = Adapter->rxMulticastFrm;
		pLocalTally->rxMultiOctets = Adapter->rxMulticastOctets;
		pLocalTally->rxPLCPCRCErrCnt = Adapter->Hw_CRC16Cnt;
		pLocalTally->rxTotalCnt = (Adapter->rxMulticastFrm+Adapter->rxUnicastFrm);
		pLocalTally->rxUnicastFrames = Adapter->rxUnicastFrm;
		pLocalTally->rxUniOctets = Adapter->rxUnicastOctets;
		pLocalTally->txMulticastFrames = Adapter->txMulticastFrm;
		pLocalTally->txMultiOctets = Adapter->txMulticastOctets;
		pLocalTally->txMultipleRetriesFrames = Adapter->Hw_RetryCnt;
		pLocalTally->txRetryLimitExceeded = Adapter->RetryFailCnt;
		pLocalTally->txUnicastFrames = Adapter->txUnicastFrm;
		pLocalTally->txUniOctets = Adapter->txUnicastOctets;
//		MoveBytes = 0;
		break;
	case OID_ZD_IO_READ:
		IoAddress = (ULONG)(*((PULONG)InformationBuffer) & 0xffffffff);
    //JWEI 2004/02/10
    #ifdef HOST_IF_USB
        if ((IoAddress & 0xF000) != 0)
            ZD1211_READ_REGISTER(Adapter, IoAddress, &tmpvalue, FALSE);
        else
    #endif
        {
            //if (!NeedReadProtection(Adapter,IoAddress, &tmpvalue))
            //    ZD1205_READ_REGISTER(Adapter,IoAddress, &tmpvalue);
		}
		
        if (sizeof(ULONG) > InformationBufferLength)
        {
            // Not enough room in InformationBuffer. Punt
            *BytesNeeded = sizeof(ULONG);

            Status = NDIS_STATUS_BUFFER_TOO_SHORT;
        }
        else
        {
            // Copy result into InformationBuffer
            *BytesWritten = sizeof(ULONG);
            NdisMoveMemory(InformationBuffer, &tmpvalue, sizeof(ULONG));
        }

		break;
	case OID_ZD_CMD:
		INITSTRTX(DebugPrint, (_T("OID_ZD_CMD\n")));
		cmd_p = (PZD_CMD_STRUCT)InformationBuffer;
		switch(cmd_p->u.send.cmd){
		case 0x30:
			// Read Phy registers
			AcquireCtrOfPhyReg(Adapter);
			
			switch(cmd_p->u.send.param0){
			case 8:		//CR4
				IoAddress = 0x20;
				break;
			case 10:	//CR5
				IoAddress = 0x10;
				break;
			case 12:	//CR6
				IoAddress = 0x14;
				break;
			case 14:	//CR7
				IoAddress = 0x18;
				break;
			case 16:	//CR8
				IoAddress = 0x1C;
				break;
			default:
				IoAddress = (cmd_p->u.send.param0 << 1);
				break;
			}
			//INITSTR((_T("Read Phy Reg (%x)\n"), IoAddress));
			//if (!NeedReadProtection(Adapter,IoAddress, &tmpvalue)){
			//	ZD1205_READ_REGISTER(Adapter,IoAddress, &tmpvalue);
			//}
			//cmd_p->u.rsp.resp0 = (USHORT)tmpvalue;
			
			ReleaseCtrOfPhyReg(Adapter);
			break;
		case 0x32:		// Read MAC Registers
			IoAddress = (cmd_p->u.send.param0 + (cmd_p->u.send.param1 << 16));
			//if (!NeedReadProtection(Adapter,IoAddress, &tmpvalue)){
			//	ZD1205_READ_REGISTER(Adapter,IoAddress, &tmpvalue);
			//}
			//cmd_p->u.rsp.resp0 = (USHORT)(tmpvalue & 0xffff);
			//cmd_p->u.rsp.resp1 = (USHORT)(tmpvalue >> 16);
			break;
		case CMD_QueryTxPwr:
			AcquireCtrOfPhyReg(Adapter);
			ZD1205_READ_REGISTER(Adapter,CR58, &tmpvalue);
			ReleaseCtrOfPhyReg(Adapter);	
			ZDDbgPrint((_T("Query Tx Power = %x\n"), tmpvalue));
			cmd_p->u.rsp.resp0 = (USHORT)tmpvalue;
			break;
		case CMD_QueryIntValue:
            ZDDbgPrint((_T("Query Integration Value = %x\n"), Adapter->TxGainSetting));
			cmd_p->u.rsp.resp0 = (USHORT)Adapter->TxGainSetting;
			break;
		default:
			Status = NDIS_STATUS_NOT_SUPPORTED;
			break;		
		}
//		MoveBytes = 0;
		break;


	case OID_ZD_GETRID:
 		INITSTRTX(DebugPrint, (_T("OID_ZD_GETRID\n")));
		rid_p = (PRID_STRUCT)InformationBuffer;
//		MoveBytes = 0;
		switch(rid_p->rid){
			case RID_CFG_PREAMBLE: // long = 1, short = 2, auto = 3
 				INITSTRTX(DebugPrint, (_T("RID_CFG_PREAMBLE")));
				rid_p->u.data[0] = (USHORT) Adapter->preambleMode;
				Status = NDIS_STATUS_SUCCESS;
//				MoveBytes = 0;
				break;
			case RID_CFG_OWN_SSID:
			case RID_CFG_SSID:
            // jhlee 3313
			// the case used by ZDConfig
			case RID_CURRENT_SSID:   
 				INITSTRTX(DebugPrint, (_T("RID_CFG_SSID\n")));
				rid_p->u.data[0] = Adapter->CardSetting.Info_SSID[1];
				pch = (PCHAR)&rid_p->u.data[1];
				for (i=0;i<Adapter->CardSetting.Info_SSID[1];i++){
					*(pch+i) = Adapter->CardSetting.Info_SSID[2+i];
				}
				Status = NDIS_STATUS_SUCCESS;
//				MoveBytes = 0;
				break;
			case RID_CFG_PORT_TYPE:
 				INITSTRTX(DebugPrint, (_T("RID_CFG_PORT_TYPE\n")));
				rid_p->u.data[0] = Adapter->CardSetting.BSSType;
				Status = NDIS_STATUS_SUCCESS;
//				MoveBytes = 0;
				break;
			case RID_CFG_CHANNEL:
			case RID_CURR_CHANNEL:
 				INITSTRTX(DebugPrint, (_T("0xFDC1\n")));
				rid_p->u.data[0] = Adapter->CardSetting.Channel;
				Status = NDIS_STATUS_SUCCESS;
//				MoveBytes = 0;
				break;
			case RID_TX_RATE:
			case RID_CURR_TX_RATE:
 				INITSTRTX(DebugPrint, (_T("RID_TX_RATE\n")));
				if (!Adapter->RateLock){
					rid_p->u.data[0] = 0xf;	//Auto Rate
				}
				else{
					rid_p->u.data[0] = (0x1 << Adapter->CardSetting.TxRate);
				}
				Status = NDIS_STATUS_SUCCESS;
//				MoveBytes = 0;
				break;
			case RID_CFG_WEP_FLAGS:
 				INITSTRTX(DebugPrint, (_T("RID_CFG_WEP_FLAGS\n")));
				// rid_p->u.data[0] : bit1,0  (1,1) Enable WEP
				//							  (0,0) Disable WEP
				//					  bit6,5  (0,0) Increment IV
				//							  (0,1) Random IV
				//					  bit8	  (0)   64-bit WEP
				//							  (1)   128-bit WEP
				if ( Adapter->CardSetting.EncryMode == WEP128 )
				    rid_p->u.data[0] = 0x0100;
				else
                    rid_p->u.data[0] = 0;
				Status = NDIS_STATUS_SUCCESS;
//				MoveBytes = 0;
				break;
			case RID_CFG_DEFAULT_KEY_ID:
 				INITSTRTX(DebugPrint, (_T("RID_CFG_DEFAULT_KEY_ID\n")));
				rid_p->u.data[0] = Adapter->CardSetting.EncryKeyId;
				Status = NDIS_STATUS_SUCCESS;
//				MoveBytes = 0;
				break;
			case RID_PORT_STATUS:
 				INITSTRTX(DebugPrint, (_T("RID_PORT_STATUS\n")));
				if (Adapter->adapterReady){
					if (Adapter->CardSetting.BSSType == PSEUDO_IBSS){
						rid_p->u.data[0] = CONNECT_TO_PIBSS;
					}
					else{
						rid_p->u.data[0] = CONNECT_TO_NETWORK;
					}
				}
				else{
					rid_p->u.data[0] = SEARCHING;
				}
				Status = NDIS_STATUS_SUCCESS;
//				MoveBytes = 0;
				break;
			case RID_COMMS_QUALITY:
 				//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
				ZDDbg((_T("Get SpinLock\n")));
				ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
 				INITSTRTX(DebugPrint, (_T("RID_COMMS_QUALITY\n")));

				rid_p->u.data[1] = Adapter->RxSignalStrength;
				rid_p->u.data[0] = (USHORT)(Adapter->RxSignalQuality);
                INITSTRTX(DebugPrint, (_T("RxSignalQuility = %d\n"), rid_p->u.data[0]));                
                INITSTRTX(DebugPrint, (_T("RxSignalStrength = %d\n"), Adapter->RxSignalStrength));                 
				rid_p->u.data[2] = 1;
				//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
				ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
				break;
			case RID_MONITOR:
 				INITSTRTX(DebugPrint, (_T("RID_MONITOR\n")));
				rid_p->u.data[0] = (USHORT)Adapter->Hw_CRC32Cnt;
				rid_p->u.data[1] = (USHORT)Adapter->Hw_CRC16Cnt;
				rid_p->u.data[2] = (USHORT)Adapter->Hw_RetryCnt;
				rid_p->u.data[3] = (USHORT)Adapter->Hw_TotalRxFrm;
				rid_p->u.data[4] = (USHORT)Adapter->Hw_TotalTxFrm;
				rid_p->u.data[5] = Adapter->RxSignalStrength;
				rid_p->u.data[6] = Adapter->RxSignalQuality1;
				rid_p->u.data[7] = Adapter->RxSignalQuality2;
                rid_p->u.data[8] = 0;
                rid_p->u.data[9] = 0;
				// For Loopback-Throughput-Test
				if (Adapter->TestFlag & LOOPBACKTEST){
#ifdef CE_ALL
					Adapter->Timer1 = GetTickCount();
					Adapter->Timer1 *= 10000;
#else
					Adapter->Timer1 = KeQueryInterruptTime();
#endif
					
					INITSTR((_T("LBTx throughput = %d Kbps\n"), 
						Adapter->LBTxCnt/((ULONG)(Adapter->Timer1-Adapter->Timer2)/10000)));
					rid_p->u.data[8] = (USHORT)(Adapter->LBTxCnt/((ULONG)(Adapter->Timer1-Adapter->Timer2)/10000));
					INITSTR((_T("LBRx throughput = %d Kbps\n"), 
						Adapter->LBRxCnt/((ULONG)(Adapter->Timer1-Adapter->Timer2)/10000)));
					rid_p->u.data[9] = (USHORT)(Adapter->LBRxCnt/((ULONG)(Adapter->Timer1-Adapter->Timer2)/10000));
					INITSTR((_T("LBErrFrame = %x\n"), Adapter->LBErrFrame));
					Adapter->LBTxCnt = 0;
					Adapter->LBRxCnt = 0;
					Adapter->LBErrFrame = 0;
					
#ifdef CE_ALL
					Adapter->Timer2 = GetTickCount();
					Adapter->Timer2 *= 10000;
#else
					Adapter->Timer2 = KeQueryInterruptTime();
#endif
                    
                //JWEI 2004/01/29
                #if !fZD1211_LOOPBACK
                    if (!(Adapter->Notification & RQ_LBTEST))
                        Mgt_SendNotification(Adapter, RQ_LBTEST);
                #endif
				}
				Reset_Tally(Adapter);
				Status = NDIS_STATUS_SUCCESS;
//				MoveBytes = 0;
				break;
			case RID_FRAG_THRESHOLD:
				INITSTR((_T("Query Fragmentation threshold %x\n"), Adapter->CardSetting.FragThreshold));
				rid_p->u.data[0] = (USHORT) Adapter->CardSetting.FragThreshold;
				Status = NDIS_STATUS_SUCCESS;
				break;
			case RID_RTS_THRESHOLD:
				INITSTR((_T("Query RTS threshold %x\n"), Adapter->CardSetting.RTSThreshold));
				rid_p->u.data[0] = (USHORT) Adapter->CardSetting.RTSThreshold;
				Status = NDIS_STATUS_SUCCESS;
				break;
			case RID_CFG_PM_EPS:
				INITSTR((_T("Query RID_CFG_PM_EPS %x\n"), Adapter->PSThreshhold / 5000));
				rid_p->u.data[0] = (USHORT) (Adapter->PSThreshhold / 5000);
				Status = NDIS_STATUS_SUCCESS;
				break;
			case RID_CFG_PM_ENABLE:
				rid_p->u.data[0] = (Adapter->PSMSupported);
				Status = NDIS_STATUS_SUCCESS;
				break;
			case RID_EEPROM_CONTENT:
				for (i=0;i<256;i+=4){
					ZD1205_READ_REGISTER(Adapter,E2P_SUBID+i, &tmpvalue);
					rid_p->u.data[i] = (UCHAR)tmpvalue;
					rid_p->u.data[i+1] = (UCHAR)(tmpvalue >> 8);
					rid_p->u.data[i+2] = (UCHAR)(tmpvalue >> 16);
					rid_p->u.data[i+3] = (UCHAR)(tmpvalue >> 24);
				}
				rid_p->length = 256;
				Status = NDIS_STATUS_SUCCESS;
				break;
			case RID_ANTENNA_SELECT:
				Status = NDIS_STATUS_SUCCESS;
				break;
			case RID_TARCE_SET_POINT:
                ZDDbgPrint((_T("query RID_TARCE_SET_POINT\n")));               
				rid_p->u.data[0] = (((USHORT)Adapter->TraceSetPoint) << 8);
				Status = NDIS_STATUS_SUCCESS;
				break;
			case RID_TX_POWER_SET_POINT:
				ZDDbgPrint((_T("query RID_TX_POWER_SET_POINT\n")));
				for( i=0; i<7; i++ )
					rid_p->u.data[i] = Adapter->SetPoint[i*2] + 
					                   (Adapter->SetPoint[i*2+1] << 8);				    
				Status = NDIS_STATUS_SUCCESS;	
				break;
			case 0xFC7F:  // for ZDConfig PER test
				if ( Adapter->PERPort ){
                    rid_p->u.data[0] = rid_p->u.data[0] & 0x01;
					rid_p->u.data[0] = rid_p->u.data[0] | (USHORT)((Adapter->PERPort & 0xff) << 8) | (USHORT)((Adapter->PERPort & 0xff00) >> 7);
				}else{
				    rid_p->u.data[0] = 0;
				}
				Status = NDIS_STATUS_SUCCESS;
				break;
			case 0xFCB5:
				rid_p->u.data[0] = Adapter->MultiDomainCapability;
				Status = NDIS_STATUS_SUCCESS;
				break;
			case 0xFCB6:
				rid_p->u.data[0] = Adapter->OperationRegionCode;
				Status = NDIS_STATUS_SUCCESS;
				break;
			case 0xFCB7:
				rid_p->u.data[0] = (USHORT)Adapter->AllowedChannel;
				Status = NDIS_STATUS_SUCCESS;
				break;
			case 0xFC43:
				rid_p->u.data[0] = (USHORT)Adapter->EnableHiddenSSID;
				Status = NDIS_STATUS_SUCCESS;
				break;
			default:
 				INITSTRTX(DebugPrint, (_T("default\n")));
				Status = NDIS_STATUS_NOT_SUPPORTED;
//				MoveBytes = 0;
				break;
		}
		break;
	
	case OID_LINK_QUALITY:
 		INITSTRTX(DebugPrint, (_T("OID_LINK_QUALITY\n")));
		if (Adapter->ChosenBSS != 0xff){
			tmpChar = Adapter->BSSInfo[Adapter->ChosenBSS].SignalQuality;
		}
		else{
			tmpChar = 0;
		}

		if (sizeof(UCHAR) > InformationBufferLength)
        {
            // Not enough room in InformationBuffer. Punt
            *BytesNeeded = sizeof(UCHAR);

            Status = NDIS_STATUS_BUFFER_TOO_SHORT;
        }
        else
        {
            // Copy result into InformationBuffer
            *BytesWritten = sizeof(UCHAR);
            NdisMoveMemory(InformationBuffer, &tmpChar, sizeof(UCHAR));
        }
		break;

	case OID_LINK_SIGNAL_STRENGTH:
 		INITSTRTX(DebugPrint, (_T("OID_LINK_SIGNAL_STRENGTH\n")));
		if (Adapter->ChosenBSS != 0xff){
			tmpChar = Adapter->BSSInfo[Adapter->ChosenBSS].SignalStrength;
		}
		else{
			tmpChar = 0;
		}

		if (sizeof(UCHAR) > InformationBufferLength)
        {
            // Not enough room in InformationBuffer. Punt
            *BytesNeeded = sizeof(UCHAR);

            Status = NDIS_STATUS_BUFFER_TOO_SHORT;
        }
        else
        {
            // Copy result into InformationBuffer
            *BytesWritten = sizeof(UCHAR);
            NdisMoveMemory(InformationBuffer, &tmpChar, sizeof(UCHAR));
        }
		break;

	case OID_ZD_802_11_BSSID_LIST:
		INITSTRTX(DebugPrint, (_T("OID_ZD_802_11_BSSID_LIST\n")));
		if (InformationBuffer == 0 || 
			(InformationBufferLength < 
			 (sizeof(ULONG) +  Adapter->BSSCnt * sizeof(NDIS_WLAN_BSSID)) )
			)
		{
			Status = NDIS_STATUS_INVALID_LENGTH;
			// The value of BytesWritten will be assigned to BytesNeeded
			//  at exit of this routine.
			//ASSERT(0);
			*BytesWritten = 0;
			*BytesNeeded = (sizeof(ULONG) + Adapter->BSSCnt * sizeof(NDIS_WLAN_BSSID));
			
			break;
		}
		pZDBSSIDList = (PNDIS_802_11_BSSID_LIST) InformationBuffer;					
		pZDBSSIDList->NumberOfItems = 0;
		for(i=0; i<Adapter->BSSCnt; i++)
		{
			if (Adapter->BSSInfo[i].OnUpdating){ 
				continue;
			}
			for (j=0;j<i;j++){
				if (memcmp(Adapter->BSSInfo[j].BSSID, Adapter->BSSInfo[i].BSSID, 6)
					== 0){
					break;
				}
			}
			if (j!=i){ // It's a duplication
				continue;
			}
			// the size of structure
			pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].Length=sizeof(NDIS_WLAN_BSSID);
			// BSSID (Mac address of AP)
			NdisMoveMemory(
				&pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].MacAddress[0],
				&Adapter->BSSInfo[i].BSSID[0],
				6
				);
			//SSID
			pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].Ssid.SsidLength =
				Adapter->BSSInfo[i].SSID[1];
			NdisMoveMemory(
				&pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].Ssid.Ssid[0],
				&Adapter->BSSInfo[i].SSID[2],
				Adapter->BSSInfo[i].SSID[1]);
				// RSSI (Received signal strength Indication
			pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].Rssi=
				Adapter->BSSInfo[i].SignalStrength;

			pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].Privacy=
				(Adapter->BSSInfo[i].Capability[0] & BIT_4);
			pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].NetworkTypeInUse =
				Ndis802_11DS;
			// The size of configuration structure
			pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].Configuration.Length=
				sizeof(NDIS_802_11_CONFIGURATION);
			// Beacon Interval
			pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].Configuration.BeaconPeriod=
				(((USHORT)Adapter->BSSInfo[i].BeaconInterval[1]) >> 8) +
				Adapter->BSSInfo[i].BeaconInterval[0];				
			// ATIM window size
			pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].Configuration.ATIMWindow=
				(((USHORT)Adapter->BSSInfo[i].ATIMWindow[1]) >> 8) +
				Adapter->BSSInfo[i].ATIMWindow[0];							
			// Channel Freq in KHz
			pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].Configuration.DSConfig=
				(2412+(Adapter->BSSInfo[i].Channel-1)*5)*1000;				
			//    if (adapter_p->portType==PORT_TYPE_PSIBSS)
			if (Adapter->BSSInfo[i].Capability[0] & 2) // IBSS, If bit1 of Capability Information field equal to 1
				pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].InfrastructureMode=
					Ndis802_11IBSS;
				//     else if(adapter_p->portType==PORT_TYPE_BSS)
			if (Adapter->BSSInfo[i].Capability[0] & 1) // ESS, If bit0 of Capability Information field equal to 1
				pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].InfrastructureMode=
					Ndis802_11Infrastructure;
				
			NdisMoveMemory(
					&pZDBSSIDList->Bssid[pZDBSSIDList->NumberOfItems].SupportedRates[0],
					&Adapter->BSSInfo[i].SupportedRates[2],
					8);
			pZDBSSIDList->NumberOfItems ++;
		}
        *BytesWritten = pZDBSSIDList->NumberOfItems*sizeof(NDIS_WLAN_BSSID) + sizeof(ULONG);
        *BytesNeeded = 0;
	
		break;

	case OID_ZD_AP_MODE:
		if (Adapter->CardSetting.BSSType == AP_BSS){
			tmpUlong = TRUE;
		}
		else{
			tmpUlong = FALSE;
		}
		// Copy result into InformationBuffer
        *BytesWritten = sizeof(ULONG);
        NdisMoveMemory(InformationBuffer, &tmpUlong, sizeof(ULONG));

		break;
	case OID_ZD_FILTER_MODE:
		tmpUlong = Adapter->AP_MacAddrFilterMode;
		// Copy result into InformationBuffer
        *BytesWritten = sizeof(ULONG);
        NdisMoveMemory(InformationBuffer, &tmpUlong, sizeof(ULONG));
		break;
	case OID_ZD_FILTER_LIST:
#ifdef HOST_IF_USB
		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#endif
		memcpy(InformationBuffer, Adapter->MACFilterListData, InformationBufferLength);
#ifdef HOST_IF_USB
		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#endif
		break;
	case OID_ZD_GET_STA_LIST:
#ifdef HOST_IF_USB
		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#endif
		{
			PMAC_LIST_DATA		pMacListData;

			pMacListData = (PMAC_LIST_DATA)InformationBuffer;
			NdisZeroMemory(pMacListData, InformationBufferLength);
			if (AP_ReportSTAsMacList(Adapter, pMacListData, InformationBufferLength)){
			}
			else{
				Status = NDIS_STATUS_INVALID_LENGTH;
			}
		}
#ifdef HOST_IF_USB
		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#endif
		break;
	}

	return Status;
}
