/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    zd_req.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

#define EEPROM_SIZE	4096

typedef struct   _ZD_RD_STRUCT {  
	ULONG      ZDRdLength;
	ULONG      ZDRdFuncId;
	ULONG      Buffer[1];
} ZD_RD_STRUCT, *PZD_RD_STRUCT;

typedef struct   _ZD_CUSTOM_STRUCT {  
	ULONG      ZDCustomLength;
	ULONG      ZDFuncId;
	ULONG      DataBuffer[1];
} ZD_CUSTOM_STRUCT, *PZD_CUSTOM_STRUCT;

UCHAR	USB_DES_CODE1[] = {0x12, 0x01, 0x00, 0x02, 0xFF, 0xFF, 0xFF, 0x40};
UCHAR	USB_DES_CODE2[] = {0x10, 0x20, 0x00, 0x01, 0x04, 0x03, 0x09, 0x04};

void
InitializeAccessPoint(PZD1205_ADAPTER Adapter)
{
	DEBUGFUNC(_T("InitializeAccessPoint"));
    INITSTR((_T("\n")));
        Adapter->CardSetting.Capability = 0x421;
    Adapter->CheckBeacon = 0;

	Adapter->CardSetting.Info_SupportedRates[0] = 0x01;
	Adapter->CardSetting.Info_SupportedRates[1] = 0x04;
	Adapter->CardSetting.Info_SupportedRates[2] = 0x82;
	Adapter->CardSetting.Info_SupportedRates[3] = 0x84;
	Adapter->CardSetting.Info_SupportedRates[4] = 0x8b;
	Adapter->CardSetting.Info_SupportedRates[5] = 0x96;
	
    Adapter->CardSetting.Info_ExtSupportedRates[0] = 0x32;
    Adapter->CardSetting.Info_ExtSupportedRates[1] = 0x08;
    Adapter->CardSetting.Info_ExtSupportedRates[2] = 0x0C;
    Adapter->CardSetting.Info_ExtSupportedRates[3] = 0x12;
    Adapter->CardSetting.Info_ExtSupportedRates[4] = 0x18;
    Adapter->CardSetting.Info_ExtSupportedRates[5] = 0x24;
    Adapter->CardSetting.Info_ExtSupportedRates[6] = 0x30;
    Adapter->CardSetting.Info_ExtSupportedRates[7] = 0x48;
    Adapter->CardSetting.Info_ExtSupportedRates[8] = 0x60;
    Adapter->CardSetting.Info_ExtSupportedRates[9] = 0x6C;

	Adapter->PernmentSupportedRates[0] = 0x01;
	Adapter->PernmentSupportedRates[1] = 0x04;
	Adapter->PernmentSupportedRates[2] = 0x82;
	Adapter->PernmentSupportedRates[3] = 0x84;
	Adapter->PernmentSupportedRates[4] = 0x0b;
	Adapter->PernmentSupportedRates[5] = 0x16;
	
	Adapter->PernmentExtSupportedRates[0] = 0x32;
	Adapter->PernmentExtSupportedRates[1] = 0x08;
	Adapter->PernmentExtSupportedRates[2] = 0x0C;
	Adapter->PernmentExtSupportedRates[3] = 0x12;
	Adapter->PernmentExtSupportedRates[4] = 0x18;
	Adapter->PernmentExtSupportedRates[5] = 0x24;
	Adapter->PernmentExtSupportedRates[6] = 0x30;
	Adapter->PernmentExtSupportedRates[7] = 0x48;
	Adapter->PernmentExtSupportedRates[8] = 0x60;
	Adapter->PernmentExtSupportedRates[9] = 0x6c;

	Adapter->BasicRates[0] = 4;
	Adapter->BasicRates[1] = 0x02;
	Adapter->BasicRates[2] = 0x04;
	Adapter->BasicRates[3] = 0x0b;
	Adapter->BasicRates[4] = 0x16;
	
	Adapter->EnableHiddenSSID = 0;
	APMgt_SendNotification(Adapter, BIT_1|BIT_3);
}

void
setBasicRate(PZD1205_ADAPTER Adapter, UCHAR *pRate, ULONG RateCount)
{
    UCHAR i, j, len1, len2;

	ZDDbgPrint((_T("set Basic Rate = ")));
	for(i=0; i<RateCount; i++)
		ZDDbgPrint((_T("%x "), pRate[i]));
	ZDDbgPrint((_T("\n")));

	Adapter->BasicRates[0] = (UCHAR) RateCount;
    memcpy(&Adapter->BasicRates[1], pRate, RateCount);

	len1 = Adapter->CardSetting.Info_SupportedRates[1] + 2;
	len2 = Adapter->CardSetting.Info_ExtSupportedRates[1] + 2;

	for(j=2; j<len1; j++)
		    Adapter->CardSetting.Info_SupportedRates[j] &= 0x7f;
    
	for(j=2; j<len2; j++)
		    Adapter->CardSetting.Info_ExtSupportedRates[j] &= 0x7f;

	Adapter->GOnlyMode = 0;
	for(i=0; i<RateCount; i++){

        if ( (pRate[i] & 0x7f) > 0x16 )
            Adapter->GOnlyMode = 1;

	    for(j=2; j<len1; j++)
		    if (Adapter->CardSetting.Info_SupportedRates[j] == (pRate[i] & 0x7f))
			{
		        Adapter->CardSetting.Info_SupportedRates[j] |= 0x80;
			    break;
			}

	    for(j=2; j<len2; j++)
		    if (Adapter->CardSetting.Info_ExtSupportedRates[j] == (pRate[i] & 0x7f))
			{
		        Adapter->CardSetting.Info_ExtSupportedRates[j] |= 0x80;
			    break;
			}
	}
}

NDIS_STATUS
ZD1211_Custom_SetInformation(
                   IN NDIS_HANDLE MiniportAdapterContext,
                   IN NDIS_OID Oid,
                   IN PVOID InformationBuffer,
                   IN ULONG InformationBufferLength,
                   OUT PULONG BytesRead,
                   OUT PULONG BytesNeeded
                   )
{
    PZD1205_ADAPTER		Adapter;
    NDIS_STATUS			Status = NDIS_STATUS_SUCCESS;
	PZD_CUSTOM_STRUCT	pZDCustom;
	PZD_RD_STRUCT		pZDRD;
	ULONG				DebugPrint;
	ULONG				IoAddress;
	UINT				i;
	ULONG				tmpvalue;
	ZD1211_IRQL         OldIrql;
	PUCHAR              delMac;
	PUSHORT             pshort;
    
	DEBUGFUNC(_T("ZD1211_Custom_SetInformation"));

    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_OID;

    switch (Oid)
    {
	case OID_ZD_CUSTOM:
        
		if ((sizeof(ZD_CUSTOM_STRUCT) - sizeof(ULONG)) > InformationBufferLength)
        {
            // Not enough room in InformationBuffer. 
            *BytesNeeded = sizeof(ZD_CUSTOM_STRUCT) - sizeof(ULONG);// - sizeof(ULONG) is for Allowed channel setting
			*BytesRead = 0;
            Status = NDIS_STATUS_BUFFER_TOO_SHORT;

			break;
        }

		pZDCustom = (PZD_CUSTOM_STRUCT)InformationBuffer;
		INITSTRTX(DebugPrint, (_T("pZDCustom->ZDFuncId = %x\n"), pZDCustom->ZDFuncId));

		switch (pZDCustom->ZDFuncId){

        case ZDAdapterOperationMode:
			DebugPrint = 1;
			INITSTRTX(DebugPrint, (_T("ZDAdapterOperationMode\n")));
			if (InformationBufferLength == 0){
				Status =  (NDIS_STATUS_INVALID_LENGTH);
				break;
			}
            INITSTRTX(DebugPrint, (_T("value = %d\n"), pZDCustom->DataBuffer[0]));
            if ( pZDCustom->DataBuffer[0] == ZD_ModeAccessPoint ){
				// Turn off 5-LED when adapter is operating at Software AP
				if (Adapter->LED_Ctrl_Type == 2)
				{
					ZD1211_WRITE_REGISTER(Adapter, SOCGPIO, 0 , FALSE);
				}
				SetBSSType(Adapter, AP_BSS);
				//InitializeAccessPoint(Adapter);
			}
			break;

		case ZDPreambleMode:
			
			// long = 1, short = 2, auto = 3
 			INITSTRTX(DebugPrint, (_T("ZDPreambleMode")));
				
			switch (pZDCustom->DataBuffer[0]){
				case ZD_PreambleLong:
					Adapter->preambleMode = 1;
					break;

				case ZD_PreambleShort:
					Adapter->preambleMode = 2;
					break;

				case ZD_PreambleAuto:
					Adapter->preambleMode = 3;
					break;
			}
			break;

		case ZDAdapterRegion:

			Adapter->OperationRegionCode = (USHORT)pZDCustom->DataBuffer[0];
			
			if (Adapter->OperationRegionCode == 0){
				
				ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &tmpvalue);

				Adapter->OperationRegionCode = (USHORT)(tmpvalue >> 16);
				ZDDbgPrint((_T("Restore OperationRegionCode=0x%x\n"), Adapter->OperationRegionCode));
			}
			
			Set_RF_Channel(Adapter, Adapter->CardSetting.Channel, (UCHAR)Adapter->RF_Mode, 1);  

			break;

		case ZDAdapterSupportChannel:

			{	
				char	*pChannelNumber = (char *)(pZDCustom->DataBuffer);
				UINT	TotalChannel = pZDCustom->ZDCustomLength - sizeof(ZD_RD_STRUCT) + sizeof(ULONG);

				if (TotalChannel > 14){

					*BytesRead = 0;
					Status = NDIS_STATUS_INVALID_LENGTH;

					break;
				}

				if (TotalChannel == 0){

					ZD1205_READ_REGISTER(Adapter, E2P_ALLOWED_CHANNEL, &Adapter->AllowedChannel);
				
					ZDDbgPrint((_T("Restore RID_CHANNEL_LIST, Adapter->AllowedChannel = %x\n"),
								Adapter->AllowedChannel));
					
					break;
				}

				Adapter->AllowedChannel &= ~0xffff;
				
				for (i=0; i < TotalChannel; i++){

					Adapter->AllowedChannel |= (1 << (*(pChannelNumber)-1));
				
					ZDDbgPrint((_T("AllowedChannel = %x\n"),
								*(pChannelNumber)));

					pChannelNumber++;
				}

				ZDDbgPrint((_T("Set RID_CHANNEL_LIST, Adapter->AllowedChannel = %x\n"),
								Adapter->AllowedChannel));
			}
			break;
		case ZDAccessPointTxPower:
            INITSTRTX(DebugPrint, (_T("ZDAccessPointTxPower\n")));
			Adapter->APTxPowerLevel = (UCHAR) (pZDCustom->DataBuffer[0]);
			break;
		case ZDAccessPointBasicRate:
            INITSTRTX(DebugPrint, (_T("ZDAccessPointBasicRate\n")));
            {
			    UCHAR *pBasicRate = (char *)(pZDCustom->DataBuffer);
				ULONG RateCount = pZDCustom->ZDCustomLength - sizeof(ZD_RD_STRUCT) + 4;
				
				setBasicRate(Adapter, pBasicRate, RateCount);
			}
			break;


		case ZDFirmwareVersion:

			break;

		case ZDUseZDXOid:
	
			if (pZDCustom->DataBuffer[0])
			    Adapter->ZDConfigRunning = 1;
			else
			    Adapter->ZDConfigRunning = 0;

			break;

		case ZDRadioState:
	
			if (pZDCustom->DataBuffer[0]){
				Adapter->ZDConfigTurnOffRF = 0;
			    TurnOn_RF(Adapter);
				if (Adapter->bImmDiscAfterRFOff){
					Mgt_SendNotification(Adapter, RQ_CONNECTNETWORK);
				}
			}
			else{
				Adapter->ZDConfigTurnOffRF = 1;
			    TurnOff_RF(Adapter);
				if (Adapter->bImmDiscAfterRFOff){
					Mgt_SendNotification(Adapter, RQ_DISCONNECT);
				}

			}

			break;

		case ZDGetAccessPointStationList:

			break;

		case ZDAccessPointFilterMode:
			// 0 = Disable  1 = Accept    2 = Reject
			//INITSTRTX(DebugPrint, ("ZDAccessPointFilterMode\n"));
            //INITSTRTX(DebugPrint, ("value = %d\n", pZDCustom->DataBuffer[0]));
			INITSTR((_T("ZDAccessPointFilterMode\n")));
            INITSTR((_T("value = %d\n"), pZDCustom->DataBuffer[0]));
		    Adapter->AP_MacAddrFilterMode = (UCHAR) pZDCustom->DataBuffer[0];
			break;

	    case ZDAccessPointFilterList:
			Mgt_SendNotification(Adapter, RQ_AP_DISASSOCIATE_ALL);
			INITSTRTX(DebugPrint, (_T("ZDAccessPointFilterList\n")));
#ifdef HOST_IF_USB
		    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
			ZDDbg((_T("Get SpinLock\n")));
			ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#endif
			{
                PMAC_LIST_DATA pMACFilterList;
                ULONG  length;

				pMACFilterList = (PMAC_LIST_DATA) pZDCustom->DataBuffer;
                length = pZDCustom->ZDCustomLength - sizeof(ZD_RD_STRUCT) + 4;
		        memcpy(Adapter->MACFilterListData, pMACFilterList, length);
				Adapter->MACFilterListDataLength = length;
			}
#ifdef HOST_IF_USB
		    //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#endif
            for(i=0; i<16;i++){
				if ( Adapter->MACFilterListData[i].Valid ){
                    INITSTRTX(DebugPrint,(_T("MACFilter = %x  %x  %x  %x  %x  %x\n"), 
					                        Adapter->MACFilterListData[i].MAC_Addr[0],
					                        Adapter->MACFilterListData[i].MAC_Addr[1],
					                        Adapter->MACFilterListData[i].MAC_Addr[2],
					                        Adapter->MACFilterListData[i].MAC_Addr[3],
					                        Adapter->MACFilterListData[i].MAC_Addr[4],
					                        Adapter->MACFilterListData[i].MAC_Addr[5]
											));
                    ZDDbgPrint((_T("MACFilter = %x  %x  %x  %x  %x  %x\n"), 
					                        Adapter->MACFilterListData[i].MAC_Addr[0],
					                        Adapter->MACFilterListData[i].MAC_Addr[1],
					                        Adapter->MACFilterListData[i].MAC_Addr[2],
					                        Adapter->MACFilterListData[i].MAC_Addr[3],
					                        Adapter->MACFilterListData[i].MAC_Addr[4],
					                        Adapter->MACFilterListData[i].MAC_Addr[5]
											));
				}
			}
		    break;
        case ZDAccessPointHideSSID:
			INITSTR((_T("ZDAccessPointHideSSID\n")));
            INITSTR((_T("value = %d\n"), pZDCustom->DataBuffer[0]));
			Adapter->EnableHiddenSSID = (UCHAR) pZDCustom->DataBuffer[0];
		    break;

        case ZDMultiDomainCapability:
			INITSTR((_T("ZDMultiDomainCapability\n")));
            INITSTR((_T("value = %d\n"), pZDCustom->DataBuffer[0]));
			Adapter->b802_11D = (UCHAR) pZDCustom->DataBuffer[0];
			break;

		case ZDDelOneStationFromAPStationList:
			INITSTR((_T("ZDMultiDomainCapability\n")));
            delMac = (PUCHAR) pZDCustom->DataBuffer;
			memcpy(Adapter->DelMac, delMac, 6);
			Mgt_SendNotification(Adapter, RQ_AP_FREE_A_STA);
			break;
        
		case ZDTxPowerLevel:
            INITSTR((_T("ZDTxPowerLevel\n")));
            pshort = (PUSHORT) pZDCustom->DataBuffer;
            Adapter->SetPointLevel = (*pshort) * 8;
            Adapter->IntValueLevel = (*pshort) * 4;
            UpdateIntegrationValue(Adapter, Adapter->CardSetting.Channel);
			break;
        
		case ZDAdhocMode:
            INITSTR((_T("ZDAdhocMode\n")));
			if ( pZDCustom->DataBuffer[0] == 0 )
			{
			    Adapter->bIBSS_G_MODE = 0;
				Adapter->bIBSS_PureG_MODE = 0;
			}
			else if ( pZDCustom->DataBuffer[0] == 1 )
			{
			    Adapter->bIBSS_G_MODE = 1;
				Adapter->bIBSS_PureG_MODE = 0;  // send self-CTS
			}
			else
			{
				Adapter->bIBSS_G_MODE = 2;
                Adapter->bIBSS_PureG_MODE = 1;  // not send self-CTS 
			}

			break;

		case ZDUseGinUsb1_1:
			INITSTRTX(DebugPrint, (_T("ZDusbswitch\n")));
            Adapter->busbswitch = (BOOLEAN)pZDCustom->DataBuffer[0] ;
			break;

		case ZDIssueDeAuthentication:
			Mgt_SendNotification(Adapter, RQ_DEAUTHENTICATION); 
			break;

		default:
				
			Status = NDIS_STATUS_NOT_SUPPORTED;

			break;
		}

		break;

	case OID_ZD_RD:

		if (sizeof(ZD_RD_STRUCT) > InformationBufferLength)
        {
            // Not enough room in InformationBuffer. Punt
            *BytesNeeded = sizeof(ZD_RD_STRUCT);
			*BytesRead = 0;
            Status = NDIS_STATUS_BUFFER_TOO_SHORT;

			break;
        }

		pZDRD = (PZD_RD_STRUCT)InformationBuffer;

		switch(pZDRD->ZDRdFuncId){

		case ZDAccessPHYRegister1B:

			//pZDRD->Buffer[0] = register ID, CR(XX)
			//pZDRD->Buffer[1] = value;

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}

			// Write PHY registers
			AcquireCtrOfPhyReg(Adapter);

			switch(pZDRD->Buffer[0]){
				case 4:		//CR4
					IoAddress = 0x20;
					break;
				case 5:	//CR5
					IoAddress = 0x10;
					break;
				case 6:	//CR6
					IoAddress = 0x14;
					break;
				case 7:	//CR7
					IoAddress = 0x18;
					break;
				case 8:	//CR8
					IoAddress = 0x1C;
					break;
				case 132:	//CR132
					IoAddress = 0x0210;

				default:
					IoAddress = (pZDRD->Buffer[0] << 2);
					break;
			}
			//if (!NeedWriteProtection(Adapter, IoAddress, pZDRD->Buffer[1])){
				//ZD1205_WRITE_REGISTER(Adapter,IoAddress, pZDRD->Buffer[1]);
#ifdef HOST_IF_USB
                ZD1211_WRITE_REGISTER(Adapter, IoAddress, (pZDRD->Buffer[1]), TRUE);
#else
				ZD1205_WRITE_REGISTER(Adapter, IoAddress, (pZDRD->Buffer[1]));
#endif
			//}
			INITSTR((_T("Write Phy Reg (%x) = %x\n"), IoAddress, 
												pZDRD->Buffer[1]));

			ReleaseCtrOfPhyReg(Adapter);
				
			*BytesRead = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));

			break;

		case ZDAccessMACRegister4B:

			//pZDRD->Buffer[0] = register address
			//pZDRD->Buffer[1] = value;

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}
			//if (pZDRD->Buffer[0] >= 0x1000){
#ifdef HOST_IF_USB
                ZD1211_WRITE_REGISTER(Adapter, pZDRD->Buffer[0], (pZDRD->Buffer[1]), FALSE);
#else
				ZD1205_WRITE_REGISTER(Adapter, pZDRD->Buffer[0], (pZDRD->Buffer[1]));
#endif
			//}
			//else if (!NeedWriteProtection(Adapter, pZDRD->Buffer[0], pZDRD->Buffer[1])){
#ifdef HOST_IF_USB
                ZD1211_WRITE_REGISTER(Adapter, pZDRD->Buffer[0], (pZDRD->Buffer[1]), TRUE);
#else
				ZD1205_WRITE_REGISTER(Adapter, pZDRD->Buffer[0], (pZDRD->Buffer[1]));
#endif
			//}
			
			*BytesRead = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));

			break;

		case ZDAccessROMData:

			// pZDRD->Buffer[0] = offset
			// pZDRD->Buffer[1] = length
			// pZDRD->Buffer[2] = data

			ZDDbgPrint((_T("ZDAccessROM Write\n")));

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + 8))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + 8);
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + 8);
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}
			{
				int		WriteLength = pZDRD->Buffer[1];
				int		WriteOffset = pZDRD->Buffer[0];	
				PUCHAR	pDataPtr = (PUCHAR)&(pZDRD->Buffer[2]);
				ULONG	tmpULONG;
				PUCHAR	pTmpBuffer = 0, pOldTmpBuffer;
			#if fLOAD_BOOTCODE
				ULONG   EEPVer, OrgVer;

			//JWEI 2004/03/03
				ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &EEPVer);
				EEPVer &= 0x0000FFFF;
			//    if (Adapter->pDeviceDesc->bcdDevice == 0x0101)
				if (EEPVer < 0x4301)
				{
					ZD1205_READ_REGISTER(Adapter, E2P_DEVICE_VER, &OrgVer);
					tmpvalue = OrgVer & 0xFFFF0000;
					tmpvalue |= 0x0101;
					ZD1205_WRITE_REGISTER(Adapter, E2P_DEVICE_VER, tmpvalue);
					JWeiDbgPrint4((" --> Set Ver:0x%08X\n", tmpvalue));
				}
			#endif

				NdisAllocateMemory(	(PVOID *)(&pTmpBuffer),
									E2P_END - E2P_SUBID,
									0,
									HighestAcceptableMax
									);
				if (!pTmpBuffer){
					Status = NDIS_STATUS_RESOURCES;
					break;
				}

				pOldTmpBuffer = pTmpBuffer;

				for (i=E2P_SUBID; i<E2P_END; i+=4){
				
					ZD1205_READ_REGISTER(Adapter, i, &tmpULONG);

					*pTmpBuffer = (UCHAR)(tmpULONG & 0xFF);
					pTmpBuffer++;
					*pTmpBuffer = (UCHAR)((tmpULONG >> 8) & 0xFF); 
					pTmpBuffer++;
					*pTmpBuffer = (UCHAR)((tmpULONG >> 16) & 0xFF); 
					pTmpBuffer++;
					*pTmpBuffer = (UCHAR)((tmpULONG >> 24) & 0xFF); 
					pTmpBuffer++;
				}

				pTmpBuffer = pOldTmpBuffer;

				// check read data
				if (memcmp(USB_DES_CODE1, pTmpBuffer + 0x34, sizeof(USB_DES_CODE1)) != 0){
					
					Status = NDIS_STATUS_RESOURCES;
					break;
				}
				if (memcmp(USB_DES_CODE2, pTmpBuffer + 0x42, sizeof(USB_DES_CODE2)) != 0){
					
					Status = NDIS_STATUS_RESOURCES;
					break;
				}

				if ((WriteOffset+WriteLength) <= (E2P_END - E2P_SUBID))
					memcpy(pTmpBuffer+WriteOffset, pDataPtr, WriteLength);
				else
					Status = NDIS_STATUS_INVALID_LENGTH;

				for (i=E2P_SUBID; i<E2P_END; i+=4){
				
					tmpULONG = *pTmpBuffer;
					pTmpBuffer++;
					tmpULONG |= ((ULONG)(*pTmpBuffer)) << 8;
					pTmpBuffer++;
					tmpULONG |= ((ULONG)(*pTmpBuffer)) << 16;
					pTmpBuffer++;
					tmpULONG |= ((ULONG)(*pTmpBuffer)) << 24;
					pTmpBuffer++;

					ZD1205_WRITE_REGISTER(Adapter, i, tmpULONG);
					ZDDbgPrint((_T("Write 0x%x with 0x%x\n"), i, tmpULONG));
				}

				{
				   //patrick : to fix boot code ver 2 update error  
					ULONG	tmpBootVer;

 					ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &tmpBootVer);
					tmpBootVer &= 0xFFFF0000;
					tmpBootVer |= Adapter->pDeviceDesc->bcdDevice;
					ZD1205_WRITE_REGISTER(Adapter, E2P_SUBID, tmpBootVer);
 					ZD1205_READ_REGISTER(Adapter, E2P_DEVICE_VER, &tmpBootVer);
					tmpBootVer &= 0xFFFF0000;
					tmpBootVer |= Adapter->pDeviceDesc->bcdDevice;
					ZD1205_WRITE_REGISTER(Adapter, E2P_DEVICE_VER, tmpBootVer);
				}

				// force firmware to write to eeprom
            //JWEI 2004/02/18
#ifdef HOST_IF_USB
        #ifndef fWRITE_WORD_REG
                ZD1211_READ_REGISTER(Adapter, ZD1211_CLOCK_CTRL, &tmpULONG, FALSE);
                tmpULONG |= bmZD1211_CLOCK_EEPROM;
                ZD1211_WRITE_REGISTER(Adapter, ZD1211_CLOCK_CTRL, tmpULONG, FALSE);

                ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &tmpULONG);
                ZD1205_WRITE_REGISTER(Adapter, E2P_SUBID, tmpULONG);
                ZDDbgPrint((_T("Write 0x%x with 0x%x\n"), E2P_SUBID, tmpULONG));
        #else
            /* update all EEPROM DATA field */
                WriteEEPROM(Adapter, cEPDATA_OFFSET, cFIRMWARE_EEPROM_OFFSET,
                            ((E2P_END - E2P_SUBID) / 2));
        #endif
#endif
                NdisFreeMemory( pOldTmpBuffer, (E2P_END - E2P_SUBID), 0);

        #if fLOAD_BOOTCODE
            // Restore Original value
                if (EEPVer < 0x4301)
                    ZD1205_WRITE_REGISTER(Adapter, E2P_DEVICE_VER, OrgVer);
        #endif
			}
			break;

		case ZDROMUpdate:

	#if fMERGE_RX_FRAME
////			if (!ChangeToFlashAble(Adapter))
////				break;
	#endif			
			{
				PUCHAR	pImage = (PUCHAR)pZDRD->Buffer;
				USHORT	ImageLen = (USHORT)(pZDRD->ZDRdLength)-sizeof(ZD_RD_STRUCT)+sizeof(ULONG);
				PUCHAR	pBuffer, pOrgBuffer;
				int		BufferLen = EEPROM_SIZE*2;

				if (ImageLen > EEPROM_SIZE){

					Status = NDIS_STATUS_INVALID_LENGTH;
					break;
				}

	#ifdef HOST_IF_USB
				NdisAllocateMemory(	&pBuffer,
									BufferLen,
									0,
									HighestAcceptableMax
									);

				if (!pBuffer){
					Status = NDIS_STATUS_INVALID_LENGTH;
					break;
				}
				pOrgBuffer = pBuffer;
				NdisZeroMemory(pBuffer, BufferLen);

				ZDDbgPrint((_T("ZDROMUpdate Write\n")));

				for(i=0; i<ImageLen; i++) {		// Transfer byte to word.
					*pBuffer = *pImage;
					pBuffer++;
					pImage++;
					*pBuffer = 0;
					pBuffer++;
				}
				pBuffer = pOrgBuffer;

				////Status = USB_Write_EEPROM(	Adapter,
				////							(PUSHORT)pBuffer, 
				////							BufferLen
				////							);
				
				NdisFreeMemory( pBuffer, BufferLen, 0);

				break;
	#endif
			}				
			break;

		case ZDContinuousTx:

			//pZDRD->Buffer[0] = mode
			//pZDRD->Buffer[1] = start or stop;

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}

			Adapter->ZDContinuousTxMode = pZDRD->Buffer[0];
			Adapter->ZDContinuousTxStart = pZDRD->Buffer[1];
			Adapter->LastZDContinuousTxRate = Adapter->CardSetting.LastSentTxRate;
						
			ZDDbgPrint((_T("ZDContinuousTx TxMode=%d TxStart=%d TxRate=0x%x\n"),
							Adapter->ZDContinuousTxMode,
							Adapter->ZDContinuousTxStart,
							Adapter->LastZDContinuousTxRate));

			if(pZDRD->Buffer[1] == ContTx_Start){	// Start
				switch(pZDRD->Buffer[0]){

					UCHAR	tmpChr = 0;
					UINT	RateTmp= 0;

					case ContTx_Normal:	// Normal continous transmit
						
						ZDDbgPrint((_T("Start ContTx_Normal\n")));

						Adapter->ContinuousTxEnable = 1;
						AcquireCtrOfPhyReg(Adapter);
						//Adapter->PHYTestTimer = 0;
						ZD1205_WRITE_REGISTER(Adapter,CR122, 0xFF);
						ReleaseCtrOfPhyReg(Adapter);

						AcquireCtrOfPhyReg(Adapter);

						switch (Adapter->LastZDContinuousTxRate){
						
							case 4:	//6M
								RateTmp = 0XB;
								break;
							case 5:	//9M
								RateTmp = 0XF;
								break;
							case 6:	//12M
								RateTmp = 0XA;
								break;
							case 7:	//18M
								RateTmp = 0XE;
								break;
							case 8:	//24M
								RateTmp = 0X9;
								break;
							case 9:	//36M
								RateTmp = 0XD;
								break;
							case 0xA:	//48M
								RateTmp = 0X8;
								break;
							case 0XB:	//54M
								RateTmp = 0XC;
								break;

							default:
								RateTmp = 0;
								break;
						}

						ZDDbgPrint((_T("RateTmp=0x%x\n"), RateTmp));
						if (RateTmp){
							ZD1205_WRITE_REGISTER(Adapter, CR132, RateTmp);

							//AcquireCtrOfPhyReg(Adapter);
							ZD1205_WRITE_REGISTER(Adapter,CR159, 4);
							ReleaseCtrOfPhyReg(Adapter);

							ZD1205_WRITE_REGISTER(Adapter,0x644, 7);

							break;
						}

						tmpChr = Adapter->LastZDContinuousTxRate;
						ZDDbgPrint((_T("tmpChr=0x%x\n"), tmpChr));

						if ( Adapter->preambleMode == 1 )
							Adapter->CardSetting.PreambleType = 0x00;
						else if ( Adapter->preambleMode == 2 )
							Adapter->CardSetting.PreambleType = 0x20;

						if (Adapter->CardSetting.PreambleType == 0x20){
							// short premable
							tmpChr |= BIT_5;
						}
						else{
							// long premable
							tmpChr &= ~BIT_5;
						}

						if (Adapter->OperationRegionCode == ZD_Region_USA) tmpChr &= ~BIT_6;//USA
						if (Adapter->OperationRegionCode == ZD_Region_Japan) tmpChr |= BIT_6;	//japan

						ZD1205_WRITE_REGISTER(Adapter, CR5, tmpChr);
						ReleaseCtrOfPhyReg(Adapter);

						ZD1205_WRITE_REGISTER(Adapter,0x644, 3);
						
						break;
					case ContTx_CW:	// CW transmit
						ZDDbgPrint((_T("Start CW transmit\n")));
						Adapter->ContinuousTxEnable = 1;
						AcquireCtrOfPhyReg(Adapter);

						ZD1205_READ_REGISTER(Adapter,CR9, &tmpvalue);
						tmpvalue |= 0x18;
						ZD1205_WRITE_REGISTER(Adapter,CR9, tmpvalue);
						ReleaseCtrOfPhyReg(Adapter);
						
						ZD1205_WRITE_REGISTER(Adapter,0x644, 0x3);

						break;
					case ContTx_CarrierSuppression:	// Carrier suppression
						ZDDbgPrint((_T("Start Carrier suppression transmit\n")));
						Adapter->ContinuousTxEnable = 1;
						AcquireCtrOfPhyReg(Adapter);
						ZD1205_READ_REGISTER(Adapter,CR9, &tmpvalue);
						tmpvalue &= ~0x10;
						tmpvalue |= 0x8;
						ZD1205_WRITE_REGISTER(Adapter,CR9, tmpvalue);

						ZD1205_READ_REGISTER(Adapter,CR32, &tmpvalue);
						tmpvalue |= 0x4;
						ZD1205_WRITE_REGISTER(Adapter,CR32, tmpvalue);
						ReleaseCtrOfPhyReg(Adapter);
						
						ZD1205_WRITE_REGISTER(Adapter,0x644, 0x3);

						break;
					default:
						break;
				}
				if (Adapter->ContinuousTxEnable)
				{
		            AcquireCtrOfPhyReg(Adapter);
		            ZD1205_WRITE_REGISTER(Adapter,CR122, 0xFF);
		            ReleaseCtrOfPhyReg(Adapter);				
				}
			}
			else{						// Stop
				switch(pZDRD->Buffer[0]){
					case ContTx_Normal:	// Normal continous transmit
						ZDDbgPrint((_T("Stop Normal Continuous Transmit\n")));
						
						Adapter->ContinuousTxEnable = 0;
						AcquireCtrOfPhyReg(Adapter);
						Adapter->PHYTestTimer = 30;
						ZD1205_WRITE_REGISTER(Adapter,CR122, 0x0);
						ReleaseCtrOfPhyReg(Adapter);

						if (Adapter->LastZDContinuousTxRate >= 4){
							AcquireCtrOfPhyReg(Adapter);
							ZD1205_WRITE_REGISTER(Adapter,CR159, 0);
							ReleaseCtrOfPhyReg(Adapter);

							ZD1205_WRITE_REGISTER(Adapter,0x644, 0);
						}
						else{

							ZD1205_WRITE_REGISTER(Adapter,0x644, 0);
						}
						
						break;
					case ContTx_CW:	// CW transmit
						ZDDbgPrint((_T("Stop CW transmit\n")));
						Adapter->ContinuousTxEnable = 0;
						AcquireCtrOfPhyReg(Adapter);
						ZD1205_READ_REGISTER(Adapter,CR9, &tmpvalue);
						tmpvalue &= ~0x18;
						ZD1205_WRITE_REGISTER(Adapter,CR9, tmpvalue);
						ReleaseCtrOfPhyReg(Adapter);
						
						ZD1205_WRITE_REGISTER(Adapter,0x644, 0x0);
						break;
					case ContTx_CarrierSuppression:	// Carrier suppression
						ZDDbgPrint((_T("Stop Carrier suppression transmit\n")));
						Adapter->ContinuousTxEnable = 0;
						AcquireCtrOfPhyReg(Adapter);
						ZD1205_READ_REGISTER(Adapter,CR9, &tmpvalue);
						tmpvalue &= ~0x18;
						ZD1205_WRITE_REGISTER(Adapter,CR9, tmpvalue);

						ZD1205_READ_REGISTER(Adapter,CR32, &tmpvalue);
						tmpvalue &= ~0x4;
						ZD1205_WRITE_REGISTER(Adapter,CR32, tmpvalue);
						ReleaseCtrOfPhyReg(Adapter);
						
						ZD1205_WRITE_REGISTER(Adapter,0x644, 0x0);
						break;
					default:
						break;
				}

				ZD1205_WRITE_REGISTER(Adapter, PS_Ctrl, 0x1);

			}

			break;
		
		case ZDGetNICAdapterTally:

			Reset_Tally(Adapter);

			break;

		case ZDSetMACAddress:

			if (pZDRD->ZDRdLength != (sizeof(ZD_RD_STRUCT) + 2))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + 2);
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + 2);
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}

			ZDDbgPrint((_T("ZDSetMACAddress %x\n"), pZDRD->Buffer[0]));

			{
				PUCHAR	ptmpUChar = (PUCHAR)pZDRD->Buffer;
			
				tmpvalue = 0;
				tmpvalue = *ptmpUChar;
				tmpvalue |= ((ULONG)(*(ptmpUChar+1) & 0xff) << 8);
				tmpvalue |= ((ULONG)(*(ptmpUChar+2) & 0xff) << 16);
				tmpvalue |= ((ULONG)(*(ptmpUChar+3) & 0xff) << 24);
				ZD1205_WRITE_REGISTER(Adapter,E2P_MACADDR_P1, tmpvalue);
				ZD1205_READ_REGISTER(Adapter,E2P_MACADDR_P2, &tmpvalue);
				tmpvalue &= 0xFFFF0000;
				tmpvalue |= *(ptmpUChar+4);
				tmpvalue |= ((ULONG)(*(ptmpUChar+5) & 0xff) << 8);
				ZD1205_WRITE_REGISTER(Adapter,E2P_MACADDR_P2, tmpvalue);
			}
			{
				// force firmware to write to EEPROM
#ifdef HOST_IF_USB
        #if fLOAD_BOOTCODE
            ULONG   EEPVer, OrgVer;

            //JWEI 2004/03/03
                ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &EEPVer);
                EEPVer &= 0x0000FFFF;
            //    if (Adapter->pDeviceDesc->bcdDevice == 0x0101)
                if (EEPVer < 0x4301)
                {
                    ZD1205_READ_REGISTER(Adapter, E2P_DEVICE_VER, &OrgVer);
                    tmpvalue = OrgVer & 0xFFFF0000;
                    tmpvalue |= 0x0101;
                    ZD1205_WRITE_REGISTER(Adapter, E2P_DEVICE_VER, tmpvalue);
                    JWeiDbgPrint4((" --> Set Ver:0x%08X\n", tmpvalue));
                }
        #endif
				{
				   //patrick : to fix boot code ver 2 update error  
					ULONG	tmpBootVer;

 					ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &tmpBootVer);
					tmpBootVer &= 0xFFFF0000;
					tmpBootVer |= Adapter->pDeviceDesc->bcdDevice;
					ZD1205_WRITE_REGISTER(Adapter, E2P_SUBID, tmpBootVer);
 					ZD1205_READ_REGISTER(Adapter, E2P_DEVICE_VER, &tmpBootVer);
					tmpBootVer &= 0xFFFF0000;
					tmpBootVer |= Adapter->pDeviceDesc->bcdDevice;
					ZD1205_WRITE_REGISTER(Adapter, E2P_DEVICE_VER, tmpBootVer);
				}
				

        #ifndef fWRITE_WORD_REG
            //JWEI 2004/02/18
                ZD1211_READ_REGISTER(Adapter, ZD1211_CLOCK_CTRL, &tmpvalue, FALSE);
                tmpvalue |= bmZD1211_CLOCK_EEPROM;
                ZD1211_WRITE_REGISTER(Adapter, ZD1211_CLOCK_CTRL, tmpvalue, FALSE);

                ZD1205_READ_REGISTER(Adapter,E2P_SUBID, &tmpvalue);
                ZD1205_WRITE_REGISTER(Adapter,E2P_SUBID, tmpvalue);
        #else
            /* update all EEPROM DATA field */
                WriteEEPROM(Adapter, cEPDATA_OFFSET, cFIRMWARE_EEPROM_OFFSET,
                            ((E2P_END - E2P_SUBID) / 2));
        #endif
#endif

        #if fLOAD_BOOTCODE
            // Restore Original value
                if (EEPVer < 0x4301)
                    ZD1205_WRITE_REGISTER(Adapter, E2P_DEVICE_VER, OrgVer);
        #endif
			}
			
			*BytesRead = (sizeof(ZD_RD_STRUCT) + 2);

			break;
#ifdef HOST_IF_USB
		case ZDBootCodeUpdate:

			ZDDbgPrint((_T("ZDBootCodeUpdate\n")));

			if (pZDRD->ZDRdLength < sizeof(ZD_RD_STRUCT))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT));
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}

            if (1)
			{
				ULONG   EEPVer, OrgVer;
		#ifndef fWRITE_WORD_REG
				ULONG           tmpvalue;
				USB_EEPROM_DATA Data;
		#endif

				// request firmware to reprogram EEPROM
                ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &OrgVer);
                OrgVer &= 0xFFFF0000;
                ZD1205_READ_REGISTER(Adapter, E2P_DEVICE_VER, &EEPVer);
                EEPVer &= 0x0000FFFF;
                EEPVer |= OrgVer;
                ZD1205_WRITE_REGISTER(Adapter, E2P_SUBID, EEPVer);
        #ifdef fWRITE_WORD_REG
                WriteEEPROM(Adapter, 0, cBOOTCODE_START_ADDR, cEEPROM_SIZE - cLOAD_VECT_LEN);
        #else
                ZD1211_READ_REGISTER(Adapter, ZD1211_CLOCK_CTRL, &tmpvalue, FALSE);
                tmpvalue |= bmZD1211_CLOCK_EEPROM;
                ZD1211_WRITE_REGISTER(Adapter, ZD1211_CLOCK_CTRL, tmpvalue, FALSE);

                Data.RequestID  = EEPROM_END;
                Data.Data[0]    = 0;

                ////if (!USB_WRITE_EEPROM_DATA( Adapter, &Data,
                ////                            sizeof(USHORT) * 2
                ////                          ))
                ////    JWeiDbgPrint5(("Fail 4\n"));
        #endif
            }

			break;

		case ZDFlashErase:

			//pZDRD->Buffer[0] = section to erase
 			ZDDbgPrint((_T("ZDFlashErase\n")));

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT));
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}
#if fPROG_FLASH
	#if fMERGE_RX_FRAME
////			if (!ChangeToFlashAble(Adapter))
////				break;
	#endif			
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue &= ~0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif           
			{
                extern void EraseFlash(PZD1205_ADAPTER Adapter);
                extern void FlashSecErase(PZD1205_ADAPTER Adapter, USHORT Sec0);

                ////if (pZDRD->Buffer[0] == 1)
                ////    EraseFlash(Adapter);
                ////else
                ////    FlashSecErase(Adapter, (USHORT) pZDRD->Buffer[0]);
            }
#endif
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue |= 0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif
			break;

		case ZDFlashProgram:

			//pZDRD->Buffer[0] = offset
			//pZDRD->Buffer[1] = Buffer len in Bytes
			//pZDRD->Buffer[2] = Buffer
 			
			ZDDbgPrint((_T("ZDFlashProgram\n")));

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + 8))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + 8);
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + 8);
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}			
         
#if fPROG_FLASH
	#if fMERGE_RX_FRAME
////			if (!ChangeToFlashAble(Adapter))
////				break;
	#endif			
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue &= ~0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif  			
			{
                //extern void ProgFlash(PZD1205_ADAPTER Adapter, PWCHAR FileName, ULONG StartAddr);
                //ProgFlash(Adapter, L"WS11U.img", pZDRD->Buffer[0]);
				

////                ProgRet = ProgFlash(Adapter, pZDRD->Buffer[0], pZDRD->Buffer[1], (PUCHAR)&(pZDRD->Buffer[2]));

				////if (!ProgRet) pZDRD->Buffer[1] = 0;
			}
#endif
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue |= 0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif
			break;
#endif // HOST_IF_USB
		case ZDEEPROMDataWrite:

			ZDDbgPrint((_T("ZDEEPROMDataWrite\n")));

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT));
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}				
			{
				// force firmware to write to EEPROM
#ifdef HOST_IF_USB
        #if fLOAD_BOOTCODE
            ULONG   EEPVer, OrgVer;

            //JWEI 2004/03/03
                ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &EEPVer);
                EEPVer &= 0x0000FFFF;
            //    if (Adapter->pDeviceDesc->bcdDevice == 0x0101)
                if (EEPVer < 0x4301)
                {
                    ZD1205_READ_REGISTER(Adapter, E2P_DEVICE_VER, &OrgVer);
                    tmpvalue = OrgVer & 0xFFFF0000;
                    tmpvalue |= 0x0101;
                    ZD1205_WRITE_REGISTER(Adapter, E2P_DEVICE_VER, tmpvalue);
                    JWeiDbgPrint4((" --> Set Ver:0x%08X\n", tmpvalue));
                }
        #endif
				{
				   //patrick : to fix boot code ver 2 update error  
					ULONG	tmpBootVer;

 					ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &tmpBootVer);
					tmpBootVer &= 0xFFFF0000;
					tmpBootVer |= Adapter->pDeviceDesc->bcdDevice;
					ZD1205_WRITE_REGISTER(Adapter, E2P_SUBID, tmpBootVer);
 					ZD1205_READ_REGISTER(Adapter, E2P_DEVICE_VER, &tmpBootVer);
					tmpBootVer &= 0xFFFF0000;
					tmpBootVer |= Adapter->pDeviceDesc->bcdDevice;
					ZD1205_WRITE_REGISTER(Adapter, E2P_DEVICE_VER, tmpBootVer);
				}

        #ifndef fWRITE_WORD_REG
            //JWEI 2004/02/18
                ZD1211_READ_REGISTER(Adapter, ZD1211_CLOCK_CTRL, &tmpvalue, FALSE);
                tmpvalue |= bmZD1211_CLOCK_EEPROM;
                ZD1211_WRITE_REGISTER(Adapter, ZD1211_CLOCK_CTRL, tmpvalue, FALSE);

                ZD1205_READ_REGISTER(Adapter,E2P_SUBID, &tmpvalue);
                ZD1205_WRITE_REGISTER(Adapter,E2P_SUBID, tmpvalue);
        #else
            /* update all EEPROM DATA field */
                WriteEEPROM(Adapter, cEPDATA_OFFSET, cFIRMWARE_EEPROM_OFFSET,
                            ((E2P_END - E2P_SUBID) / 2));
        #endif
#endif

        #if fLOAD_BOOTCODE
            // Restore Original value
                if (EEPVer < 0x4301)
                    ZD1205_WRITE_REGISTER(Adapter, E2P_DEVICE_VER, OrgVer);
        #endif
			}

			break;
				
		case ZDTxPowerGainControl:

			ZDDbgPrint((_T("ZDTxPowerGainControl\n")));

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT));
				*BytesRead = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}
			
			if (pZDRD->Buffer[0] == 0){
				DecreaseTxPower(Adapter, cTX_CCK);
#if fTX_GAIN_OFDM	// because OFDM power not set at CR31 when fTX_GAIN_OFDM=1
				DecreaseTxPower(Adapter, cTX_OFDM);
#endif
			}
			if (pZDRD->Buffer[0] == 1){
				IncreaseTxPower(Adapter, cTX_CCK);
#if fTX_GAIN_OFDM	// because OFDM power not set at CR31 when fTX_GAIN_OFDM=1
				IncreaseTxPower(Adapter, cTX_OFDM);
#endif
			}

		default:
			
			Status = NDIS_STATUS_NOT_SUPPORTED;

			break;
		}

		break;

	default:

		break;
	}

	return Status;
}


NDIS_STATUS
ZD1211_Custom_QueryInformation(
                     IN NDIS_HANDLE MiniportAdapterContext,
                     IN NDIS_OID Oid,
                     IN PVOID InformationBuffer,
                     IN ULONG InformationBufferLength,
                     OUT PULONG BytesWritten,
                     OUT PULONG BytesNeeded
                     )
{
    PZD1205_ADAPTER		Adapter;
    NDIS_STATUS			Status = NDIS_STATUS_SUCCESS;
	PZD_CUSTOM_STRUCT	pZDCustom;
	PZD_RD_STRUCT		pZDRD;
 	ULONG				DebugPrint;
	ULONG				IoAddress;
	ULONG				tmpvalue;
	UINT				i;
	ZD1211_IRQL         OldIrql;
	PUSHORT             pshort;
   
	DEBUGFUNC(_T("ZD1211_Custom_QueryInformation"));

    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_OID;
			
	INITSTRTX(DebugPrint, (_T("ZD1211_Custom_QueryInformation\n")));
    INITSTRTX(DebugPrint, (_T("Oid = %x :\n"), Oid));
    switch (Oid)
    {
	case OID_ZD_CUSTOM:

		if (sizeof(ZD_CUSTOM_STRUCT) > InformationBufferLength)
        {
            // Not enough room in InformationBuffer. Punt
            *BytesNeeded = sizeof(ZD_CUSTOM_STRUCT);
			*BytesWritten = 0;
            Status = NDIS_STATUS_BUFFER_TOO_SHORT;

			break;
        }

		pZDCustom = (PZD_CUSTOM_STRUCT)InformationBuffer;
        INITSTRTX(DebugPrint, (_T("value = %x :\n"), pZDCustom->ZDFuncId));

		switch (pZDCustom->ZDFuncId){

		case ZDAdapterOperationMode:

			if (Adapter->CardSetting.BSSType == AP_BSS){
				pZDCustom->DataBuffer[0] = ZD_ModeAccessPoint;
			}
			else{
				pZDCustom->DataBuffer[0] = ZD_ModeStation;
			}

			break;

		case ZDLinkStatus:

			INITSTRTX(DebugPrint, (_T("ZDLinkStatus\n")));
			
			if (Adapter->adapterReady){
				if (Adapter->CardSetting.BSSType == PSEUDO_IBSS){
					pZDCustom->DataBuffer[0] = ZD_LinkStatus_ConnectPIBSS;
				}
				else if (Adapter->CardSetting.BSSType == INDEPENDENT_BSS){
					pZDCustom->DataBuffer[0] = ZD_LinkStatus_ConnectIBSS;
				}
				else if (Adapter->CardSetting.BSSType == AP_BSS){
                    pZDCustom->DataBuffer[0] = ZD_LinkStatus_StartHostAP;
				}
				else{
					pZDCustom->DataBuffer[0] = ZD_LinkStatus_ConnectESS;
				}
			}
			else{
				pZDCustom->DataBuffer[0] = ZD_LinkStatus_Searching;
			}
			
			pZDCustom->ZDCustomLength = sizeof(ULONG);

			break;

		case ZDCommQuality:

			DebugPrint = 0;
			INITSTRTX(DebugPrint, (_T("ZDCommQuality %ddB\n"), Adapter->RxSignalQualityIndB));
			INITSTRTX(DebugPrint, (_T("ZDCommQuality %d\n"), Adapter->RxSignalQuality));

			pZDCustom->DataBuffer[0] = Adapter->RxSignalQuality;
			
			if (pZDCustom->ZDCustomLength == (sizeof(ZD_CUSTOM_STRUCT)+1))
                pZDCustom->DataBuffer[1] = Adapter->RxSignalQualityIndB;
			
			pZDCustom->ZDCustomLength = sizeof(ULONG);

			break;

		case ZDPreambleMode:

			// long = 1, short = 2, auto = 3
 			INITSTRTX(DebugPrint, (_T("ZDPreambleMode")));
				
			switch (Adapter->preambleMode){
				case 1:
					pZDCustom->DataBuffer[0] = ZD_PreambleLong;
					break;

				case 2:
					pZDCustom->DataBuffer[0] = ZD_PreambleShort;
					break;

				case 3:
					pZDCustom->DataBuffer[0] = ZD_PreambleAuto;
					break;
			}

			break;
		
		case ZDAccessPointTxPower:
            INITSTRTX(DebugPrint, (_T("ZDAccessPointTxPower\n")));
			pZDCustom->DataBuffer[0] = Adapter->APTxPowerLevel;
			break;
		case ZDAdapterRegion:

			ZDDbgPrint((_T("ZDAdapterRegion 0x%x\n"), Adapter->OperationRegionCode));
			pZDCustom->DataBuffer[0] = Adapter->OperationRegionCode;

			break;

		case ZDAdapterSupportChannel:
			{
				int		AllowChannelCount = 0;
				ULONG	tmpAllowChannel = Adapter->AllowedChannel;
				char	*pChannelNumber = (char *)(pZDCustom->DataBuffer);

				for (i=0; i<14; i++){
					if (tmpAllowChannel & (1 << i))
						AllowChannelCount++;
				}

				if (pZDCustom->ZDCustomLength < (AllowChannelCount + sizeof(ZD_RD_STRUCT) - sizeof(ULONG)))
				{
					pZDCustom->ZDCustomLength = (AllowChannelCount + sizeof(ZD_RD_STRUCT) - sizeof(ULONG));

					*BytesNeeded = pZDCustom->ZDCustomLength;
					*BytesWritten = 0;
					Status = NDIS_STATUS_BUFFER_TOO_SHORT;
					
					break;
				}

				for (i=0; i<14; i++){
					if (tmpAllowChannel & (1 << i))
						*(pChannelNumber++) = i+1;
				}

				pZDCustom->ZDCustomLength = (AllowChannelCount + sizeof(ZD_RD_STRUCT) - sizeof(ULONG));
				*BytesWritten = pZDCustom->ZDCustomLength;

			}
			break;

		case ZDFirmwareVersion:

			break;

		case ZDUseZDXOid:
	
			if (Adapter->ZDConfigRunning)
			    pZDCustom->DataBuffer[0] = 1;
			else
			    pZDCustom->DataBuffer[0] = 0;

			break;

		case ZDGetAccessPointStationList:
            INITSTRTX(DebugPrint, (_T("ZDGetAccessPointStationList\n")));
#ifdef HOST_IF_USB
		    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
			ZDDbg((_T("Get SpinLock\n")));
			ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#endif
			{
			    PMAC_LIST_DATA		pMacListData;
				ULONG               length;

			    pMacListData = (PMAC_LIST_DATA)pZDCustom->DataBuffer;
				length = pZDCustom->ZDCustomLength - sizeof(ZD_CUSTOM_STRUCT) + 4;
			    NdisZeroMemory(pMacListData, length);
			    if (!AP_ReportSTAsMacList(Adapter, pMacListData, length)){
				    Status = NDIS_STATUS_INVALID_LENGTH;
				}

				*BytesWritten = pZDCustom->ZDCustomLength;
			}
#ifdef HOST_IF_USB
		    //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#endif

			break;

		case ZDAccessPointFilterMode:
			INITSTRTX(DebugPrint, (_T("ZDAccessPointFilterMode\n")));
		    tmpvalue = Adapter->AP_MacAddrFilterMode;
		    // Copy result into InformationBuffer
            *BytesWritten = sizeof(ULONG);
            NdisMoveMemory(InformationBuffer, &tmpvalue, sizeof(ULONG));
			break;

		case ZDAccessPointFilterList:
			INITSTRTX(DebugPrint, (_T("ZDAccessPointFilterList\n")));
#ifdef HOST_IF_USB
		    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
			ZDDbg((_T("Get SpinLock\n")));
			ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#endif
		    memcpy(InformationBuffer, Adapter->MACFilterListData, Adapter->MACFilterListDataLength);

#ifdef HOST_IF_USB
		    //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#endif
			*BytesWritten = Adapter->MACFilterListDataLength;
			break;
		
		case ZDDesiredSSID:
			
			if (pZDCustom->ZDCustomLength < (sizeof(NDIS_802_11_SSID) + sizeof(ZD_CUSTOM_STRUCT) - sizeof(ULONG)))
			{
				pZDCustom->ZDCustomLength = (sizeof(NDIS_802_11_SSID) + sizeof(ZD_CUSTOM_STRUCT) - sizeof(ULONG));
				*BytesNeeded = pZDCustom->ZDCustomLength;
				*BytesWritten = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}

			{
				NDIS_802_11_SSID *pSSID = (NDIS_802_11_SSID *)pZDCustom->DataBuffer;
				
				INITSTRTX(DebugPrint, (_T("ZDDesiredSSID\n")));
				
				pSSID->SsidLength = Adapter->CardSetting.Info_SSID[1];
				for (i=0;i<Adapter->CardSetting.Info_SSID[1];i++){
					pSSID->Ssid[i] = Adapter->CardSetting.Info_SSID[2+i];
				}
			}
			break;

		case ZDWepInfo:

			  //LONG ZDWEPDefaultKeyId; 
              //LONG ZDWEPKeyLength;

			if (pZDCustom->ZDCustomLength < (sizeof(ZD_CUSTOM_STRUCT) + sizeof(ULONG)))
			{
				pZDCustom->ZDCustomLength = (sizeof(ZD_CUSTOM_STRUCT) + sizeof(ULONG));
				*BytesNeeded = pZDCustom->ZDCustomLength;
				*BytesWritten = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}
			INITSTRTX(DebugPrint, (_T("ZDWepInfo\n")));

			pZDCustom->DataBuffer[0] = (ULONG)Adapter->CardSetting.EncryKeyId;
			pZDCustom->DataBuffer[1] = (ULONG)Adapter->wepKeyLen;
			break;

        case ZDMultiDomainCapability:
			INITSTRTX(DebugPrint, (_T("ZDMultiDomainCapability\n")));
			pZDCustom->DataBuffer[0] = (ULONG) Adapter->b802_11D;
			break;

        case ZDUSBType:
			INITSTRTX(DebugPrint, (_T("ZDUSBType\n")));
			if (Adapter->IsUSB2_0) 
			    pZDCustom->DataBuffer[0] = ZD_USB20;
			else
			    pZDCustom->DataBuffer[0] = ZD_USB11;
			break;

		case ZDRadioState:
	        INITSTRTX(DebugPrint, (_T("ZDRadioState\n")));
			if ( Adapter->ZDConfigTurnOffRF )
			    pZDCustom->DataBuffer[0] = 0;
			else 
                pZDCustom->DataBuffer[0] = 1;
			break;
             
		case ZDTxPowerLevel:
            INITSTR((_T("ZDTxPowerLevel\n")));
            pshort = (PUSHORT) pZDCustom->DataBuffer;
            *pshort = Adapter->IntValueLevel / 4;
			break;

		case ZDAdhocMode:
            INITSTR((_T("ZDAdhocMode\n")));
            pZDCustom->DataBuffer[0] = Adapter->bIBSS_G_MODE;
            break;
		case ZDUseGinUsb1_1:
			INITSTRTX(DebugPrint, (_T("ZDusbswitch\n")));
			pZDCustom->DataBuffer[0] = Adapter->busbswitch;
			break;

		default:
	
			Status = NDIS_STATUS_NOT_SUPPORTED;

			break;
		}

		break;

	case OID_ZD_RD:

		if (sizeof(ZD_RD_STRUCT) > InformationBufferLength)
        {
            // Not enough room in InformationBuffer. Punt
            *BytesNeeded = sizeof(ZD_RD_STRUCT);
			*BytesWritten = 0;

            Status = NDIS_STATUS_BUFFER_TOO_SHORT;

			break;
        }

		pZDRD = (PZD_RD_STRUCT)InformationBuffer;

		switch(pZDRD->ZDRdFuncId){

		case ZDAccessPHYRegister1B:

			//pZDRD->Buffer[0] = register ID, CR(XX)
			//pZDRD->Buffer[1] = value;

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesWritten = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}

			// Write PHY registers
			AcquireCtrOfPhyReg(Adapter);

			switch(pZDRD->Buffer[0]){
				case 4:		//CR4
					IoAddress = 0x20;
					break;
				case 5:	//CR5
					IoAddress = 0x10;
					break;
				case 6:	//CR6
					IoAddress = 0x14;
					break;
				case 7:	//CR7
					IoAddress = 0x18;
					break;
				case 8:	//CR8
					IoAddress = 0x1C;
					break;
				case 132:	//CR132
					IoAddress = 0x0210;

				default:
					IoAddress = (pZDRD->Buffer[0] << 2);
					break;
			}
			//if (!NeedReadProtection(Adapter, IoAddress, &(pZDRD->Buffer[1]))){
				//ZD1205_READ_REGISTER(Adapter,IoAddress, &(pZDRD->Buffer[1]));
#ifdef HOST_IF_USB
                ZD1211_READ_REGISTER(Adapter, IoAddress, &(pZDRD->Buffer[1]), TRUE);
#else
				ZD1205_READ_REGISTER(Adapter, IoAddress, &(pZDRD->Buffer[1]));
#endif
			//}
			pZDRD->Buffer[1] &= 0xFFFF;
			INITSTR((_T("Read Phy Reg (0x%x) = %x\n"), IoAddress, 
												pZDRD->Buffer[1]));

			ReleaseCtrOfPhyReg(Adapter);
			
			pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
			*BytesWritten = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));

			break;

		case ZDAccessMACRegister4B:
		
			//pZDRD->Buffer[0] = register address
			//pZDRD->Buffer[1] = value;

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesWritten = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}

			//if (pZDRD->Buffer[0] >= 0x1000){
#ifdef HOST_IF_USB
                ZD1211_READ_REGISTER(Adapter, pZDRD->Buffer[0], &(pZDRD->Buffer[1]), FALSE);
#else
				ZD1205_READ_REGISTER(Adapter, pZDRD->Buffer[0], &(pZDRD->Buffer[1]));
#endif
			//}
			//else if (!NeedReadProtection(Adapter, pZDRD->Buffer[0], &(pZDRD->Buffer[1]))){
#ifdef HOST_IF_USB
                ZD1211_READ_REGISTER(Adapter, pZDRD->Buffer[0], &(pZDRD->Buffer[1]), TRUE);
#else
				ZD1205_READ_REGISTER(Adapter, pZDRD->Buffer[0], &(pZDRD->Buffer[1]));
#endif
			//}
			
			pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
			*BytesWritten = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));

			break;

		case ZDAccessROMData:

			// pZDRD->Buffer[0] = offset
			// pZDRD->Buffer[1] = length
			// pZDRD->Buffer[2] = data

			ZDDbgPrint((_T("ZDAccessROM read\n")));

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + 8))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + 8);
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + 8);
				*BytesWritten = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}
			{
				int		ReadLength = pZDRD->Buffer[1];
				int		ReadOffset = pZDRD->Buffer[0];	
				PUCHAR	pDataPtr = (PUCHAR)&(pZDRD->Buffer[2]);
				ULONG	tmpULONG;
				PUCHAR	pTmpBuffer = 0, pOldTmpBuffer;

				NdisAllocateMemory(	(PVOID *)(&pTmpBuffer),
									E2P_END - E2P_SUBID,
									0,
									HighestAcceptableMax
									);
				if (!pTmpBuffer){
					Status = NDIS_STATUS_RESOURCES;
					break;
				}

				pOldTmpBuffer = pTmpBuffer;

				for (i=E2P_SUBID; i<E2P_END; i+=4){
				
					ZD1205_READ_REGISTER(Adapter, i, &tmpULONG);

					*pTmpBuffer = (UCHAR)(tmpULONG & 0xFF);
					pTmpBuffer++;
					*pTmpBuffer = (UCHAR)((tmpULONG >> 8) & 0xFF); 
					pTmpBuffer++;
					*pTmpBuffer = (UCHAR)((tmpULONG >> 16) & 0xFF); 
					pTmpBuffer++;
					*pTmpBuffer = (UCHAR)((tmpULONG >> 24) & 0xFF); 
					pTmpBuffer++;
				}
				
				pTmpBuffer = pOldTmpBuffer;

				// check read data
				if (memcmp(USB_DES_CODE1, pTmpBuffer + 0x34, sizeof(USB_DES_CODE1)) != 0){
					
					Status = NDIS_STATUS_RESOURCES;
					break;
				}

				// This descriptor may change
				/*
				if (memcmp(USB_DES_CODE2, pTmpBuffer + 0x42, sizeof(USB_DES_CODE2)) != 0){
					
					Status = NDIS_STATUS_RESOURCES;
					break;
				}*/

			//Patrick : Fix read boot code ver 2 error
				*((PUSHORT)(pTmpBuffer+0x40)) = Adapter->pDeviceDesc->bcdDevice;

				if ((ReadOffset+ReadLength) <= (E2P_END - E2P_SUBID))
					memcpy(pDataPtr, pOldTmpBuffer+ReadOffset, ReadLength);
				else
					Status = NDIS_STATUS_RESOURCES;

				NdisFreeMemory( pOldTmpBuffer, (E2P_END - E2P_SUBID), 0);

			}

			break;

		case ZDContinuousTx:

			//pZDRD->Buffer[0] = mode
			//pZDRD->Buffer[1] = start or stop;

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));
				*BytesWritten = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}

			pZDRD->Buffer[0] = Adapter->ZDContinuousTxMode;
			pZDRD->Buffer[1] = Adapter->ZDContinuousTxStart;

			pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG));

			break;
		
		case ZDGetNICAdapterTally:

			//pZDRD->Buffer[0] = comm_tally
			ZDDbgPrint((_T("ZDGetNICAdapterTally\n")));

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)*37))	//ZD_RD_STRUCT contain one ULONG
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)*37);
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)*37);
				*BytesWritten = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				ZDDbgPrint((_T("ZDGetNICAdapterTally BUFFER_TOO_SHORT\n")));
				break;
			}
/*
			pZDRD->Buffer[0] = Adapter->txUnicastFrm;		//txUnicastFrames
			pZDRD->Buffer[1] = Adapter->txMulticastFrm;		//txMulticastFrames
#ifdef HOST_IF_USB
			pZDRD->Buffer[2] = Adapter->REG_6e4_Add;		//Mac Address 0x6e4
#endif
			pZDRD->Buffer[3] = Adapter->txUnicastOctets;	//txUniOctets  byte size
			pZDRD->Buffer[4] = Adapter->txMulticastOctets;	//txMultiOctets  byte size
			pZDRD->Buffer[5] = Adapter->rx11bDataFrame;		//Receive 11b data frame count
			pZDRD->Buffer[6] = Adapter->rxOFDMDataFrame;	//Receive 11g data frame count
			pZDRD->Buffer[7] = Adapter->Hw_RetryCnt;		//txMultipleRetriesFrames
			pZDRD->Buffer[8] = Adapter->RetryFailCnt;		//txRetryLimitExceeded
			pZDRD->Buffer[9] = Adapter->Hw_TotalTxFrm;		//Hardware total Tx Frame
			pZDRD->Buffer[10] = Adapter->rxUnicastFrm;		//rxUnicastFrames
			pZDRD->Buffer[11] = Adapter->rxMulticastFrm;	//rxMulticastFrames
			pZDRD->Buffer[12] = Adapter->Hw_CRC16Cnt;		//rxPLCPCRCErrCnt
			pZDRD->Buffer[13] = Adapter->rxUnicastOctets;	//rxUniOctets  byte size
			pZDRD->Buffer[14] = Adapter->rxMulticastOctets;	//rxMultiOctets  byte size
			pZDRD->Buffer[15] = Adapter->Hw_CRC32Cnt;		//rxCRC32ErrCnt
			pZDRD->Buffer[16] = Adapter->DriverDiscardedFrm;//rxDiscardedCnt
			pZDRD->Buffer[17] = (Adapter->rxMulticastFrm+Adapter->rxUnicastFrm);//rxTotalCnt
			pZDRD->Buffer[18] = (Adapter->Hw_DecrypErr_UNI+Adapter->Hw_DecrypErr_Mul);//rxDecrypFailCnt
			pZDRD->Buffer[19] = Adapter->rxBroadcastFrm;	//Receive broadcast frame count
			pZDRD->Buffer[20] = Adapter->rxBroadcastOctets;	//Receive broadcast frame byte size
			pZDRD->Buffer[21] = Adapter->Hw_TotalRxFrm;//
			pZDRD->Buffer[22] = Adapter->Hw_CRC32Cnt;//
			pZDRD->Buffer[23] = Adapter->Hw_CRC16Cnt;//
			pZDRD->Buffer[24] = Adapter->Hw_DecrypErr_UNI;//
			pZDRD->Buffer[25] = Adapter->Hw_DecrypErr_Mul;//
			pZDRD->Buffer[26] = Adapter->Hw_RxFIFOOverrun;//
			pZDRD->Buffer[27] = Adapter->DriverRxFrmCnt;//
			pZDRD->Buffer[28] = Adapter->DriverDiscardedFrm;//
			pZDRD->Buffer[29] = Adapter->LessThanDataMinLen;//
			pZDRD->Buffer[30] = Adapter->GreaterThanMaxLen;//
			pZDRD->Buffer[31] = Adapter->PLCP_MacLen_Mismatch;//
			pZDRD->Buffer[32] = Adapter->NotifyNDISRxFrmCnt;//
			pZDRD->Buffer[33] = Adapter->Hw_TotalTxFrm;//
			pZDRD->Buffer[34] = Adapter->Hw_UnderrunCnt;//
			pZDRD->Buffer[35] = Adapter->Hw_RetryCnt;//
			pZDRD->Buffer[36] = Adapter->RetryFailCnt;//
			pZDRD->Buffer[37] = Adapter->LossAP;//
*/

			pZDRD->Buffer[0] = Adapter->txUnicastFrm;		//txUnicastFrames
			pZDRD->Buffer[1] = Adapter->txMulticastFrm;		//txMulticastFrames
			pZDRD->Buffer[2] = (ULONG) Adapter->txUnicastOctets;	//txUniOctets  byte size
			pZDRD->Buffer[3] = Adapter->txMulticastOctets;	//txMultiOctets  byte size
			pZDRD->Buffer[4] = Adapter->txFrmUpperNDIS;
			pZDRD->Buffer[5] = Adapter->txFrmDrvMgt;
			pZDRD->Buffer[6] = Adapter->RetryFailCnt;		
			pZDRD->Buffer[7] = Adapter->Hw_TotalTxFrm;		//Hardware total Tx Frame
			pZDRD->Buffer[8] = Adapter->Hw_RetryCnt;		//txMultipleRetriesFrames
			pZDRD->Buffer[9] = Adapter->Hw_UnderrunCnt;//
			
			
			pZDRD->Buffer[10] = Adapter->DriverRxFrmCnt;//
			pZDRD->Buffer[11] = Adapter->rxUnicastFrm;		//rxUnicastFrames
			pZDRD->Buffer[12] = Adapter->rxMulticastFrm;	//rxMulticastFrames
			pZDRD->Buffer[13] = (Adapter->rxMulticastFrm+Adapter->rxUnicastFrm);//rxTotalCnt
			pZDRD->Buffer[14] = Adapter->NotifyNDISRxFrmCnt;//
			pZDRD->Buffer[15] = (ULONG)Adapter->rxUnicastOctets;	//rxUniOctets  byte size
			pZDRD->Buffer[16] = Adapter->rxMulticastOctets;	//rxMultiOctets  byte size
			pZDRD->Buffer[17] = Adapter->DriverDiscardedFrm;// Discard by ValidateFrame
			pZDRD->Buffer[18] = Adapter->LessThanDataMinLen;//
			pZDRD->Buffer[19] = Adapter->GreaterThanMaxLen;//
			pZDRD->Buffer[20] = Adapter->DriverDiscardedFrmCauseByMulticastList;
			pZDRD->Buffer[21] = Adapter->DriverDiscardedFrmCauseByFrmCtrl;
			pZDRD->Buffer[22] = Adapter->rxNeedFrgFrm;		// need more frg frm
			pZDRD->Buffer[23] = Adapter->DriverRxMgtFrmCnt;
			pZDRD->Buffer[24] = Adapter->rxBroadcastFrm;	//Receive broadcast frame count
			pZDRD->Buffer[25] = Adapter->rxBroadcastOctets;	//Receive broadcast frame byte size
			pZDRD->Buffer[26] = Adapter->rx11bDataFrame;		//Measured quality 11b data frame count
			pZDRD->Buffer[27] = Adapter->rxOFDMDataFrame;	//Measured quality 11g data frame count
			
			
			pZDRD->Buffer[28] = Adapter->Hw_TotalRxFrm;//
			pZDRD->Buffer[29] = Adapter->Hw_CRC16Cnt;		//rxPLCPCRCErrCnt
			pZDRD->Buffer[30] = Adapter->Hw_CRC32Cnt;		//rxCRC32ErrCnt
			pZDRD->Buffer[31] = Adapter->Hw_DecrypErr_UNI;//
			pZDRD->Buffer[32] = Adapter->Hw_DecrypErr_Mul;//
			pZDRD->Buffer[33] = (Adapter->Hw_DecrypErr_UNI+Adapter->Hw_DecrypErr_Mul);//rxDecrypFailCnt
			pZDRD->Buffer[34] = Adapter->Hw_RxFIFOOverrun;//
			pZDRD->Buffer[35] = Adapter->LossAP;//			
                    #ifdef HOST_IF_USB
                        pZDRD->Buffer[36] = Adapter->Continue2Rx;               //Reserve 1
                        pZDRD->Buffer[37] = Adapter->WaitLenInfoCnt;            //Reserve 2
                    #endif
			pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) + sizeof(ULONG)*37);

			break;

		case ZDSetMACAddress:

			break;

		case ZDFlashRead:

			// pZDRD->Buffer[0] = offset in bytes
			// pZDRD->Buffer[1] = length in bytes
			// pZDRD->Buffer[2] = data

			ZDDbgPrint((_T("ZDFlashRead %d\n"), pZDRD->Buffer[0]));

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT) - 4 + pZDRD->Buffer[1]))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT) - 4 + pZDRD->Buffer[1]);
				*BytesNeeded = (sizeof(ZD_RD_STRUCT) - 4 + pZDRD->Buffer[1]);
				*BytesWritten = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}
      
           
#if fPROG_FLASH
//#define cMAX_READ_NUM           cMAX_MULTI_READ_REG_NUM
#define cMAX_READ_NUM           8

////            if (!ChangeToFlashAble(Adapter))
////                break;
 #ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue &= ~0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif     	
			{

            USHORT  ii, low_addr;
            USHORT  ReadAddr[cMAX_MULTI_READ_REG_NUM];
            USHORT  ReadData[cMAX_MULTI_READ_REG_NUM];
            USHORT  ReadIndex;
            USHORT  BufferLenInWords = (USHORT)pZDRD->Buffer[1]/2;

                AcquireCtrOfPhyReg(Adapter);
                Adapter->AllowAccessRegister = FALSE;
                ////InitHighAddr(Adapter);
            #if fREAD_MUL_REG
            /* Dump Flash data */
                for (ii = 0; ii < BufferLenInWords; ii += cMAX_READ_NUM)
                {
                    PUSHORT tmpReadData = (PUSHORT)(&(pZDRD->Buffer[2])) + ii;

                    ////low_addr = SetAbsAddr(Adapter, pZDRD->Buffer[0]/2 + ii, NULL);
                    

                    for (ReadIndex = 0; ReadIndex < cMAX_READ_NUM;)
                        mFILL_READ_REGISTER(low_addr ++);

                    USB_PACKAGE_READ_REGISTER(Adapter, ReadAddr, ReadData, ReadIndex, FALSE);
                    
                    //DumpReadMultipleReg(Adapter, low_addr);

                    if ((ii + cMAX_READ_NUM) < BufferLenInWords){
                        NdisMoveMemory((PUCHAR)tmpReadData, ReadData, cMAX_READ_NUM * 2);
                    }
                    else{
                        NdisMoveMemory((PUCHAR)tmpReadData, ReadData, (BufferLenInWords - ii) * 2);
                    }
               }
                
                if (pZDRD->Buffer[1]%2)
                {
                PUCHAR  tmpReadByte = (PUCHAR)(&(pZDRD->Buffer[2])) + pZDRD->Buffer[1] - 1;

                    ////low_addr = SetAbsAddr(Adapter, pZDRD->Buffer[0] + ii, NULL);
                    //DumpReadMultipleReg(Adapter, low_addr);
                    
                    ZDDbgPrint((_T("ZDFlashRead One byte: %X\n "), low_addr));

                    for (ReadIndex = 0; ReadIndex < cMAX_READ_NUM;)
                        mFILL_READ_REGISTER(low_addr ++);

                    USB_PACKAGE_READ_REGISTER(Adapter, ReadAddr, ReadData, ReadIndex, FALSE);
                    
                    *tmpReadByte = (UCHAR)ReadData[0];
                }
            #endif
                Adapter->AllowAccessRegister = TRUE;
                ReleaseCtrOfPhyReg(Adapter);

#endif
			}
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue |= 0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif
			break;

		case ZDFlashIDQuery:

			// pZDRD->Buffer[0] = The ID

			ZDDbgPrint((_T("ZDFlashIDQuery\n")));

			if (pZDRD->ZDRdLength < (sizeof(ZD_RD_STRUCT)))
			{
				pZDRD->ZDRdLength = (sizeof(ZD_RD_STRUCT));
				*BytesNeeded = (sizeof(ZD_RD_STRUCT));
				*BytesWritten = 0;
				Status = NDIS_STATUS_BUFFER_TOO_SHORT;
				break;
			}
            
#if fPROG_FLASH
	#if fMERGE_RX_FRAME
////			if (!ChangeToFlashAble(Adapter))
////				break;
	#endif			
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue &= ~0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif           
			{
				extern void FlashCmdWrite(PZD1205_ADAPTER Adapter, UCHAR Cmd);
				// Change to Read Flash ID mode
				////FlashCmdWrite(Adapter, 0x90);
			}

			// Read Address 0 and 1
			{
                USHORT  low_addr;
				
                AcquireCtrOfPhyReg(Adapter);
	       		Adapter->AllowAccessRegister = FALSE;

            #if fREAD_MUL_REG
				{
					USHORT  ReadAddr[cMAX_MULTI_READ_REG_NUM];
					USHORT	ReadData[cMAX_MULTI_READ_REG_NUM];
					PUSHORT	tmpReadData = (PUSHORT)(&(pZDRD->Buffer[0]));
					USHORT  ReadIndex = 0;

					////low_addr = SetAbsAddr(Adapter, 0, NULL);
					

					for (ReadIndex = 0; ReadIndex < 2;)
						mFILL_READ_REGISTER(low_addr ++);

					USB_PACKAGE_READ_REGISTER(Adapter, ReadAddr, ReadData, ReadIndex, FALSE);
					
					//DumpReadMultipleReg(Adapter, low_addr);

					*tmpReadData = ReadData[0] & 0xFF;
					*(tmpReadData + 1) = ReadData[1] & 0xFF;
               }
            #endif
				Adapter->AllowAccessRegister = TRUE;
                ReleaseCtrOfPhyReg(Adapter);
            }

#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue |= 0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif
#endif

			break;
		
		case ZDFlashGetChkSum:
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue &= ~0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif           

#if fPROG_FLASH
////			if (!ChangeToFlashAble(Adapter))
////				break;
			// pZDRD->Buffer[0] = The CheckSum
////            pZDRD->Buffer[0] = GetCheckSum(Adapter, 0, 0x200000L);
#endif
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue |= 0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif
			break;

		case ZDFlashGetSubChkSum:
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue &= ~0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif           

#if fPROG_FLASH
////			if (!ChangeToFlashAble(Adapter))
////				break;
			// pZDRD->Buffer[0] = The Start address in Word
			// pZDRD->Buffer[1] = The lenth in Word
			// pZDRD->Buffer[2] = The return value

////            pZDRD->Buffer[2] = GetCheckSum(Adapter, pZDRD->Buffer[0], pZDRD->Buffer[1]);
#endif
#ifdef fQuickPhySet
			{
				ULONG		tmpvalue;

				ZD1211_READ_REGISTER(Adapter,CtlReg1, &tmpvalue, TRUE);
				tmpvalue |= 0x80;
				ZD1211_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue, TRUE);
			}
#endif
			break;

		default:
			
			Status = NDIS_STATUS_NOT_SUPPORTED;

			break;
		}

	default:

		break;
	}

	return Status;
}
