/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    equates.h

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#ifndef _EQUATES_H
#define _EQUATES_H

//-------------------------------------------------------------------------
// OEM Message Tags
//-------------------------------------------------------------------------
#define stringTag   0xFEFA      // Length Byte After String
#define lStringTag  0xFEFB      // Length Byte Before String
#define zStringTag  0xFEFC      // Zero-Terminated String Tag
#define nStringTag  0xFEFD      // No Length Byte Or 0-Term

//-------------------------------------------------------------------------
// Adapter Types Supported
//-------------------------------------------------------------------------
#define FLASH32_EISA    (0 * 4)
#define FLASH32_PCI     (1 * 4)
#define D29C_EISA       (2 * 4)
#define D29C_PCI        (3 * 4)
#define D100_PCI        (4 * 4)

//-------------------------------------------------------------------------
// Phy related constants
//-------------------------------------------------------------------------
#define PHY_503                 0
#define PHY_100_A               0x000003E0
#define PHY_100_C               0x035002A8
#define PHY_TX_ID               0x015002A8
#define PHY_NSC_TX              0x5c002000
#define PHY_OTHER               0xFFFF

#define PHY_MODEL_REV_ID_MASK   0xFFF0FFFF
#define PARALLEL_DETECT         0
#define N_WAY                   1

#define RENEGOTIATE_TIME        35 // (3.5 Seconds)

#define CONNECTOR_AUTO          0
#define CONNECTOR_TPE           1
#define CONNECTOR_MII           2

//-------------------------------------------------------------------------
// Ethernet Frame Sizes
//-------------------------------------------------------------------------
#define ETHERNET_ADDRESS_LENGTH         6
#define ETHERNET_HEADER_SIZE            14
#define MINIMUM_ETHERNET_PACKET_SIZE    60
#define MAXIMUM_ETHERNET_PACKET_SIZE    1514
//#define	MAX_WLAN_SIZE					2400
#define	MAX_WLAN_SIZE					4100

#define MAX_MULTICAST_ADDRESSES         32
#define TCB_BUFFER_SIZE                 0XE0 // 224
#define COALESCE_BUFFER_SIZE            MAX_WLAN_SIZE
#define ETH_MAX_COPY_LENGTH             0x80 // 128

// Make receive area 1536 for 16 bit alignment.
//#define RCB_BUFFER_SIZE       MAXIMUM_ETHERNET_PACKET_SIZE
#define RCB_BUFFER_SIZE                 1520 // 0x5F0

//- Area reserved for all Non Transmit command blocks
#define MAX_NON_TX_CB_AREA              512

//-------------------------------------------------------------------------
// Ndis/Adapter driver constants
//-------------------------------------------------------------------------
#define MAX_PHYS_DESC               16
#define MAX_RECEIVE_DESCRIPTORS     1024 // 0x400
#define NUM_RMD                     10

//--------------------------------------------------------------------------
// System wide Equates
//--------------------------------------------------------------------------
#define MAX_NUMBER_OF_EISA_SLOTS    15
#define MAX_NUMBER_OF_PCI_SLOTS     15

//--------------------------------------------------------------------------
//    Equates Added for NDIS 4
//--------------------------------------------------------------------------
#define  NUM_BYTES_PROTOCOL_RESERVED_SECTION    16
#define  MAX_NUM_ALLOCATED_RFDS                 64
#define  MIN_NUM_RFD                            4
#define  MAX_ARRAY_SEND_PACKETS                 8
// limit our receive routine to indicating this many at a time
#define  MAX_ARRAY_RECEIVE_PACKETS              16
#define  MAC_RESERVED_SWRFDPTR                  0
#define  MAX_PACKETS_TO_ADD                     32

//-------------------------------------------------------------------------
//- Miscellaneous Equates
//-------------------------------------------------------------------------
#define CR      0x0D        // Carriage Return
#define LF      0x0A        // Line Feed

#ifndef FALSE
#define FALSE       0
#define TRUE        1
#endif

#define DRIVER_NULL ((ULONG)0xffffffff)
#define DRIVER_ZERO 0

//-------------------------------------------------------------------------
// Bit Mask definitions
//-------------------------------------------------------------------------
#define BIT_0       0x1
#define BIT_1       0x2
#define BIT_2       0x4
#define BIT_3       0x8
#define BIT_4       0x10
#define BIT_5       0x20
#define BIT_6       0x40
#define BIT_7       0x80
#define BIT_8       0x100
#define BIT_9       0x200
#define BIT_10      0x400
#define BIT_11      0x800
#define BIT_12      0x1000
#define BIT_13      0x2000
#define BIT_14      0x4000
#define BIT_15      0x8000
#define BIT_16		0x10000
#define BIT_17		0x20000
#define BIT_18		0x40000
#define BIT_19		0x80000
#define BIT_20		0x100000
#define BIT_21		0x200000
#define BIT_22		0x400000
#define BIT_23		0x800000
#define BIT_24      0x1000000
#define BIT_25		0x2000000
#define BIT_26		0x4000000
#define BIT_27		0x8000000
#define BIT_28      0x10000000
#define BIT_29      0x20000000	//WPA support
#define BIT_30      0x40000000
#define BIT_31      0x80000000

#define BIT_0_2     0x0007
#define BIT_0_3     0x000F
#define BIT_0_4     0x001F
#define BIT_0_5     0x003F
#define BIT_0_6     0x007F
#define BIT_0_7     0x00FF
#define BIT_0_8     0x01FF
#define BIT_0_13    0x3FFF
#define BIT_0_15    0xFFFF
#define BIT_1_2     0x0006
#define BIT_1_3     0x000E
#define BIT_2_5     0x003C
#define BIT_3_4     0x0018
#define BIT_4_5     0x0030
#define BIT_4_6     0x0070
#define BIT_4_7     0x00F0
#define BIT_5_7     0x00E0
#define BIT_5_9     0x03E0
#define BIT_5_12    0x1FE0
#define BIT_5_15    0xFFE0
#define BIT_6_7     0x00c0
#define BIT_7_11    0x0F80
#define BIT_8_10    0x0700
#define BIT_8_12	(BIT_12+BIT_11+BIT_10+BIT_9+BIT_8)
#define BIT_9_13    0x3E00
#define BIT_12_15   0xF000

#define BIT_16_20   0x001F0000
#define BIT_21_25   0x03E00000
#define BIT_26_27   0x0C000000

#define	SR_1M		0x2
#define SR_2M		0x4
#define SR_5_5M		0xb
#define SR_11M		0x16
#define	SR_16_5M	0x21
#define	SR_27_5M	0x37

#define	SR_6M		0xc
#define	SR_9M		0x12
#define	SR_12M		0x18
#define	SR_18M		0x24
#define	SR_24M		0x30
#define	SR_36M		0x48
#define	SR_48M		0x60
#define	SR_54M		0x6c


//JWEI 2004/03/21
#define mBIT(b)                 (1 << (b))
#define mMASK(w)                (mBIT(w) - 1)
#define mSET_MASK(a, b)         ((a) | (b))
#define mCLR_MASK(a, b)         ((a) & (~(b)))
#define mSET_BIT(a, b)          mSET_MASK(a, mBIT(b))
#define mCLR_BIT(a, b)          mCLR_MASK(a, mBIT(b))
#define mCHK_BIT1(a, b)         ((a) & mBIT(b))
#define mTEST_BIT(a, b)         mCHK_BIT1(a, b)

// in order to make our custom oids hopefully somewhat unique
// we will use 0xFF (indicating implementation specific OID)
//               A0 (first byte of non zero intel unique identifier)
//               C9 (second byte of non zero intel unique identifier)
//               XX (the custom OID number - providing 255 possible custom oids)
#define OID_CUSTOM_DRIVER_SET       0xFFA0C901
#define OID_CUSTOM_DRIVER_QUERY     0xFFA0C902
#define OID_CUSTOM_ARRAY            0xFFA0C903
#define OID_CUSTOM_STRING           0xFFA0C904


#define NO_WEP						0x0
#define AES							0x1
#define WEP64						0x2
#define WEP128						0x3
#define TKIP                        0x4
#define WEP256						0x5

#define RANDOM						0x0
#define INCREMENT					0x1


#define RQ_AUTHENTICATION			0x01
#define RQ_ASSOCIATION				0x02
#define RQ_CONNECTNETWORK			0x04
#define RQ_BSSID_LIST_SCAN			0x08
#define RQ_TERMINATION				0x10
#define RQ_DISCONNECT				0x20
#define RQ_PSPOLL					0x40
#define RQ_INFORM_PS_CHANGE			0x80
#define RQ_LBTEST					0x100
#define RQ_ZD_BSSID_LIST_SCAN		0x200
#define	RQ_ATIM_WND_END				0x400
#define	RQ_DEAUTHENTICATION			0x800
#define	RQ_ROAMING					0x1000
#define RQ_TKIPDBG					0x2000
#ifdef HOST_IF_USB
#define RQ_RESET_TXEP				0x4000
#define RQ_RESTART_TXRXEP			0x8000
#endif
#define	RQ_AP_UPDATEBEACON			0x10000
#define	RQ_AP_DISASSOCIATE_ALL		0x20000
#define RQ_SCANTEST					0x40000
#define RQ_AP_FREE_A_STA            0x80000
#define RQ_AP_INDICATE_BRIDGE       0x100000
#define RQ_SET_OID                  0x200000  
#define RQ_TESTIPC                  0x400000
#define RQ_UPDATE_CW				0x800000 // Added by jxiao

#define	MAX_TX_PWR					0x20
#define	MIN_TX_PWR					0x0

#define MAXIM2_MAX_TX_PWR_SET       0x7F
#define MAXIM2_MIN_TX_PWR_SET       0x0
#define RFMD_MAX_TX_PWR_SET         0xF0
#define RFMD_MIN_TX_PWR_SET         0x60
#define GCT_MAX_TX_PWR_SET          0x3F
#define GCT_MIN_TX_PWR_SET          0x0
#define AL2210_MAX_TX_PWR_SET       0xFF
#define AL2210_MIN_TX_PWR_SET       0x60
#define AL2230_MAX_TX_PWR_SET       (0x7F - 1)
#define AL2230_MIN_TX_PWR_SET       (0x00 + 1)

#define MAX_TX_PWR_READING          0xF0
#define MIN_TX_PWR_READING          0x30

#define TRACKING_NUM                10
#define cTX_CCK                     1       //JWEI 2003/12/22
#define cTX_OFDM                    2       // 6M - 36M
#define cTX_48M                     3
#define cTX_54M                     4
//``JWEI 2004/05/19
#define cPWR_CTRL_GUARD             4       // CR57: 4 -> 0.5 dB
#define cPWR_INT_VALUE_GUARD        8       // CR31: 4 -> 1 dB; 8 -> 2 dB
#define cPWR_STRONG_SIG_DROP        (0x18 - cPWR_INT_VALUE_GUARD)

#define cLBTEST_COUNT               1000
#define cLBTEST_PATN                0x55

    #ifndef fWRITE_WORD_REG
#define cMAX_MULTI_WRITE_REG_NUM    9
#define cMIN_MULTI_WRITE_REG_NUM    1
    #else
#define cMAX_MULTI_WRITE_REG_NUM    14
#define cMIN_MULTI_WRITE_REG_NUM    0
#define cMAX_MULTI_RF_REG_NUM       28
    #endif
#define cMAX_MULTI_READ_REG_NUM     15

/* Firmware Interface */
#define cTX_QUEUE_LEN               4
// make sure already Tx by HMAC (for UMAC System)
// 1.Host->UMAC, 2.In UMAC Queue, 3.HMAC Sent
#define cTX_SENT_LEN                (cTX_QUEUE_LEN + 2)
//JWEI 2004/02/19
#ifdef fNEW_CODE_MAP
#define cFIRMWARE_OLD_ADDR          0xEC00
#define cFIRMWARE_START_ADDR        0xEE00
#else
//#define cFIRMWARE_START_ADDR        0xD000
#define cFIRMWARE_START_ADDR        0xEC00
#endif
#define cFIRMWARE_EXT_CODE          0x1000
#define cADDR_ENTRY_TABLE           (cFIRMWARE_START_ADDR + 0x1D)
#define cBOOTCODE_START_ADDR        0xF800
#define cINT_VECT_ADDR              0xFFF5
#define cEEPROM_SIZE                0x800   // 2k word (4k byte)
// in word (16 bit width)
#define cLOAD_CODE_LEN              0xE
#define cLOAD_VECT_LEN              (0x10000 - 0xFFF7)
#define cEPDATA_OFFSET              (cLOAD_CODE_LEN + cLOAD_VECT_LEN)
#define USB_BASE_ADDR_EEPROM        0x9900
#define USB_BASE_ADDR_HOST          0x9B00
#define BASE_ADDR_MASK_HOST         (~0x00FF)
#define cFIRMWARE_EEPROM_OFFSET     (cBOOTCODE_START_ADDR + cEPDATA_OFFSET)

#define CAM							0x0
#define	PSM							0x1
#define	ACC_1						0x0
#define	ACC_2						0x1

#define INDEPENDENT_BSS				0x0
#define INFRASTRUCTURE_BSS			0x1
#define PSEUDO_IBSS					0x3
#define	AP_BSS						0x4

#define OPEN_SYSTEM					0x0
#define SHARED_KEY					0x1
#define	SEQ_1						0x1
#define	SEQ_2						0x2
#define	SEQ_3						0x3
#define	SEQ_4						0x4

#define	AUTH_SUCCESS				0
#define	AUTH_CHAL_FAIL				15

#define	CHALLENGE					0x1

#define	ASSO_SUCCESS				0
#define ASSO_FAIL                   1
#define ASSO_FAIL_RATE              18

#define	REASON_NO_ACTIVITY			0x4
#define	REASON_AP_ABORT				0x5
#define	REASON_NONAUTHEN			0x6
#define	REASON_NONASSOC				0x7

#define NO_CIPHER					0x0

// Device Bus-Master (Tx) state
#define TX_IDLE						0x00
#define TX_READ_TBD					0x01
#define TX_READ_DATA0				0x02
#define TX_READ_DATA1				0x03
#define TX_CHK_TBD_CNT				0x04
#define TX_READ_TCB					0x05
#define TX_CHK_TCB					0x06
#define TX_WAIT_DATA				0x07
#define TX_RETRYFAILURE				0x08
#define TX_REDOWNLOAD				0x09
#define TX_IDLE_ON_RETRYFAILURE		0x0a

// Device Bus-Master (Rx) state
#define RX_IDLE						0x00
#define RX_WAIT_DATA				0x10
#define RX_DATA0					0x20
#define RX_DATA1					0x30
#define RX_READ_RCB					0x50
#define RX_CHK_RCB					0x60
#define RX_WAIT_STS					0x70
#define RX_FRM_ERR					0x80
#define RX_CHK_DATA					0x90


#define	CONNECT_TO_PIBSS			0x9
#define CONNECT_TO_NETWORK			0x4
#define SEARCHING					0x2

#define PCI_DBG						BIT_0
#define PS_DBG						BIT_1
#define LOOPBACKTEST				BIT_2
#define LENGTH_SCAN					BIT_3
#define	DUMP_RX_FRAME				BIT_4
#define LOCKED_CLOCK				BIT_5

#define DBGMSG_TX					0x00000001
#define DBGMSG_OID					0x00000002

#define DBGMSG_INIT					0x00000004
#define DBGMSG_USBIO				0x00000008
#define DBGMSG_RX					0x00000010
#define DBGMSG_INT					0x00000020
#define DBGMSG_EP0					0x00000040
#define DBGMSG_SCAN                 0x00000080


#define DEFAULT_KEY_SET				0x80

#define CAM_AP                      0x1
#define	CAM_STA						0x2
#define CAM_HOST					0x4

#define NIC_LINK_DETECTION_DELAY	100

// For CAM
#define	WRITE_EN					0x80000000
#define	HOST_RW						BIT_8
#define	HOST_PENDING				BIT_31
#define DEFAULT_KEY_START_DWADDR	386
#define DEFAULT_ENC_TYPE_DWADDR		65
#define CAM_WEP_64					0x1
#define CAM_TKIP					0x2
#define CAM_AES_CCM					0x4
#define CAM_WEP_128					0x5
#define CAM_WEP_256					0x6

// For Rate Adaption
#define FALL_RATE					0x0
#define	RISE_RATE					0x1

#define	NOTIFY_NDIS					TRUE
// MAC_PA_STATE
#define	MAC_INI						0x0
#define	MAC_OPERATION				0x1

#define REG_MAX_WAIT				500

// Rate definition
#define CCK_1M						0x0
#define CCK_2M						0x1
#define CCK_5_5M					0x2
#define CCK_11M						0x3

#define	MAX_SUPPORT_DEVICE_NUM		5
// RF TYPE
#define UW2451_RF					0x2
#define uChip_RF					0x3
#define	AL2230_RF					0x4
#define	GRF5103						0x5
#define	THETA_RF					0x6
#define	AL2210_RF					0x7
#define	MAXIM_NEW_RF				0x8
#define	GCT_RF						0x9
#define	PV2000_RF					0xA
#define	RALINK_RF					0xB
#define	INTERSIL_RF					0xC
#define	RFMD_RF						0xD
#define	MAXIM_NEW_RF2				0xE
#define	PHILIPS_RF					0xF

// Feature Bit Map
#define	FBM_ANTTENA_DIVERSITY		0x00000001
#define	FBM_802_11D					0x00000002
#define	FBM_27_5_MBPS				0x00000004

// For Software AP
#define MAX_AID 			31
#define TX_MGT_NUM			8
#define	MGT_CONTENT_SIZE	256

#define	UPPER_NDIS					0x01
#define	STA_MGT						0x02
#define	AP_MGT						0x04
#define	AP_FORWARD_MUL				0x08
#define	AP_FORWARD_UNI				0x10

#define	DISABLE						0x0
#define	ACCEPT						0x1
#define	REJECT						0x2

#define EPINT_IORDRsp			    0x90
#define EPINT_RetryFial_Event	    0xa0

#define DBG_SHOW_MAX_BUFFER_LENGTH  256

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if	!(defined(AMAC))

#define PLCP_HEADER_SIZE			4
#define EXT_INFO_SIZE				4

#else

#define PLCP_HEADER_SIZE			5

#ifdef HOST_IF_USB
#define EXT_INFO_SIZE				5
#else
#define EXT_INFO_SIZE				8
#endif

#endif

#define	MGT_FRM_HEADER_SIZE			24
#define DATA_FRM_HEADER_SIZE		24
#define IV_SIZE						4
#define	ICV_SIZE					4
#define CRC32_SIZE					4
#define EXT_IV_SIZE					4	//WPA support
#define MIC_SIZE					8
#define TIMER_PERIOD_FOR_ONE_SEC    1000

#define BCNINTERVAL_OFFSET			PLCP_HEADER_SIZE+32

#define PLCP_EXTINFO_SIZE			(PLCP_HEADER_SIZE+EXT_INFO_SIZE)
// PLCP_HEADER_SIZE(4) +
// EXT_INFO_SIZE(4)

#define	ENCAP_LEN_CIPHER_ON			(PLCP_HEADER_SIZE +  \
									 DATA_FRM_HEADER_SIZE+ \
									 IV_SIZE +  \
									 ICV_SIZE + \
									 CRC32_SIZE + \
									 EXT_INFO_SIZE)
// PLCP_HEADER_SIZE(4) +
// DATA_FRM_HEADER_SIZE(24) +
// IV(4) +
// ICV(4) +
// CRC32(4) +
// EXT_INFO_SIZE(4)

#define ENCAP_LEN_CIPHER_OFF		(PLCP_HEADER_SIZE + \
									 DATA_FRM_HEADER_SIZE + \
									 CRC32_SIZE + \
									 EXT_INFO_SIZE)
// PLCP_HEADER_SIZE(4) +
// DATA_FRM_HEADER_SIZE(24) +
// CRC32(4) +
// EXT_INFO_SIZE(4)

#define OFFSET_SIZE_CIPHER_ON		(PLCP_HEADER_SIZE + \
									 DATA_FRM_HEADER_SIZE + \
									 IV_SIZE - \
									 ETHERNET_HEADER_SIZE)

// PLCP_HEADER_SIZE(4) + 
// DATA_FRM_HEADER_SIZE(24)+
// IV(4) -
// ETHERNET_HEADER_SIZE(14)

#define OFFSET_SIZE_CIPHER_OFF		(PLCP_HEADER_SIZE + \
									 DATA_FRM_HEADER_SIZE - \
									 ETHERNET_HEADER_SIZE)
// PLCP_HEADER_SIZE(4) + 
// DATA_FRM_HEADER_SIZE(24)-
// ETHERNET_HEADER_SIZE(14)

#define OFFSET1_SIZE_CIPHER_ON		(PLCP_HEADER_SIZE + \
									 DATA_FRM_HEADER_SIZE + \
									 IV_SIZE)
// PLCP_HEADER_SIZE(4) + 
// DATA_FRM_HEADER_SIZE(24)+
// IV(4)

#define OFFSET1_SIZE_CIPHER_OFF		(PLCP_HEADER_SIZE + \
									 DATA_FRM_HEADER_SIZE)
// PLCP_HEADER_SIZE(4) + 
// DATA_FRM_HEADER_SIZE(24)+

#define REMOVE_SIZE_CIPHER_ON		(PLCP_HEADER_SIZE + \
									 DATA_FRM_HEADER_SIZE + \
									 IV_SIZE + \
									 ICV_SIZE + \
									 CRC32_SIZE + \
									 EXT_INFO_SIZE - \
									 ETHERNET_HEADER_SIZE)
// PLCP_HEADER_SIZE(4) +
// DATA_FRM_HEADER_SIZE(24) +
// IV(4) +
// ICV(4) +
// CRC32(4) +
// EXT_INFO_SIZE(4) -
// ETHERNET_HEADER_SIZE(14)

#define REMOVE_SIZE_CIPHER_OFF		(PLCP_HEADER_SIZE + \
									 DATA_FRM_HEADER_SIZE + \
									 CRC32_SIZE + \
									 EXT_INFO_SIZE - \
									 ETHERNET_HEADER_SIZE)
// PLCP_HEADER_SIZE(4) +
// DATA_FRM_HEADER_SIZE(24) +
// CRC32(4) +
// EXT_INFO_SIZE(4) -
// ETHERNET_HEADER_SIZE(14)

#define REMOVE1_SIZE_CIPHER_ON		(PLCP_HEADER_SIZE + \
									 DATA_FRM_HEADER_SIZE + \
									 IV_SIZE + \
									 ICV_SIZE + \
									 CRC32_SIZE + \
									 EXT_INFO_SIZE)
// PLCP_HEADER_SIZE(4) +
// DATA_FRM_HEADER_SIZE(24) +
// IV(4) +
// ICV(4) +
// CRC32(4) +
// EXT_INFO_SIZE(4)

#define REMOVE1_SIZE_CIPHER_OFF		(PLCP_HEADER_SIZE + \
									 DATA_FRM_HEADER_SIZE + \
									 CRC32_SIZE + \
									 EXT_INFO_SIZE)
// PLCP_HEADER_SIZE(4) +
// DATA_FRM_HEADER_SIZE(24) +
// CRC32(4) +
// EXT_INFO_SIZE(4)

//-----------------------------------------------------------

#endif      // _EQUATES_H

