/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    init.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

NDIS_STATUS UpdateBootCode( IN PZD1205_ADAPTER Adapter,
                            PUSHORT pCheckSum, PUSHORT pEEPROMData,
                            ULONG EEPROMLen);

//-----------------------------------------------------------------------------
// Procedure:   SetupSharedAdapterMemory
//
// Description: This routine is responsible for the allocation of the
//              data structures inside the Adapter structure.
//
// Arguments:
//    Adapter - the adapter structure to allocate for.
//
// Returns:
//    NDIS_STATUS_SUCCESS - If the shared memory structures were setup
//    not NDIS_STATUS_SUCCESS- If not enough memory or map registers could be
//                             allocated
//-----------------------------------------------------------------------------

NDIS_STATUS
SetupSharedAdapterMemory(
                         IN PZD1205_ADAPTER Adapter
                         )

{
    NDIS_STATUS             Status;

    DEBUGFUNC(_T("SetupSharedAdapterMemory"));

    INITSTR((_T("\n")));


	Status = NDIS_STATUS_SUCCESS;
	Adapter->USBCSRAddress = 0x9000;

    // set this up here to make sure we always have enough MAP REGISTERS
    // for all the TBDs we have allocated

    // we don't have to worry so much about using map registers now
    // that NT 3.51 came out with less restrictions on them.

    //Adapter->NumTcb = Adapter->RegNumTcb;

    // Total number of TBDs equals number of TCBs * number of TBDs per TCB
    Adapter->NumTbd = (Adapter->NumTcb * Adapter->NumTbdPerTcb);
	INITSTR((_T("Adapter->NumTcb = %d,  Adapter->NumtbdPerTcb = %d\n"), Adapter->NumTcb,Adapter->NumTbdPerTcb));

    // Allocate cached memory for the SW receive frame descriptors. (RFD)
    Adapter->RecvCachedSize = (Adapter->NumRfd * sizeof(ZD1205SwRfd) +
							   Adapter->NumRfd * sizeof(ZD1205_DEFRAG_ENTITY));
    Status = ZD1205_ALLOC_MEM(&Adapter->RecvCached, Adapter->RecvCachedSize);

    if (Status != NDIS_STATUS_SUCCESS)
    {
        //ZD1205LogError(Adapter, EVENT_12, NDIS_ERROR_CODE_OUT_OF_RESOURCES, 0);

        Adapter->RecvCached = (PUCHAR) 0;
        INITSTR((_T("Could not allocate %d bytes for RecvCached mem\n"), Adapter->RecvCachedSize));
        return Status;
    }

    INITSTR((_T("Allocated %08x %8d bytes for RecvCached mem\n"), Adapter->RecvCached, Adapter->RecvCachedSize));

    NdisZeroMemory((PVOID) Adapter->RecvCached, Adapter->RecvCachedSize);

    // Allocate memory for the shared receive resources with enough
    // extra to paragraph align everything.
    Adapter->RecvUnCachedSize = (Adapter->NumRfd * sizeof(RFD_STRUC));

    // Allocate the shared memory for the receive data structures.

    NdisAllocateMemory(
        (PVOID) &Adapter->RecvUnCached,
        Adapter->RecvUnCachedSize,
		0,
		HighestAcceptableMax
        );

    if (!Adapter->RecvUnCached)
    {
        //ZD1205LogError(Adapter, EVENT_13, NDIS_ERROR_CODE_OUT_OF_RESOURCES, 0);

        INITSTR((_T("Could not allocate %d bytes for RecvUnCached mem\n"), Adapter->RecvUnCachedSize));
        return NDIS_STATUS_FAILURE;
    }

    INITSTR((_T("Allocated %08x %8d bytes for RecvUnCached mem\n"), Adapter->RecvUnCached, Adapter->RecvUnCachedSize));
    NdisZeroMemory((PVOID) Adapter->RecvUnCached, Adapter->RecvUnCachedSize);

    // Allocate cached memory for the SW transmit structures.
	// The Memory allocated by ZD1205_ALLOC_MEM will be 
	// 4-byte alignment.
	// When you apply the sizeof operator to a structure or union type name, 
	// or to an identifier of structure or union type, the result is the number 
	// of bytes in the structure or union, including internal and 
	// trailing padding (so that the first member of the next structure will on 
	// its needed boundary).
    Adapter->XmitCachedSize = (Adapter->NumTcb * sizeof(ZD1205SwTcb)) +
        (Adapter->NumCoalesce * sizeof(COALESCE));

    Status = ZD1205_ALLOC_MEM( &Adapter->XmitCached, Adapter->XmitCachedSize);

    if (Status != NDIS_STATUS_SUCCESS)
    {
        //ZD1205LogError(Adapter, EVENT_14, NDIS_ERROR_CODE_OUT_OF_RESOURCES, 0);

        Adapter->XmitCached = (PUCHAR) 0;
        INITSTR((_T("Could not allocate %d bytes for XmitCached mem\n"), Adapter->XmitCachedSize));
        return Status;
    }

    INITSTR((_T("Allocated %08x %8d bytes for XmitCached mem\n"), Adapter->XmitCached, Adapter->XmitCachedSize));

    // initialize this recently allocated area to zeros
    NdisZeroMemory((PVOID) Adapter->XmitCached, Adapter->XmitCachedSize);

    // Allocate memory for the shared transmit resources with enough extra mem
    // to paragraph align (4-byte alignment) everything
    Adapter->XmitUnCachedSize =
        (Adapter->NumTcb * sizeof(HWTCB_STRUC)) +
        (Adapter->NumTbd * sizeof(TBD_STRUC))+
		(Adapter->NumTcb * sizeof(CTRL_STRUC))+
		(Adapter->NumTcb * sizeof(HEADER_STRUC))+
        ((Adapter->NumCoalesce) * sizeof(HWCOALESCE))+
        (Adapter->NumTcb * sizeof(MIC_STRUC)); //WPA support

    // Allocate the shared memory for the command block data structures.
    NdisAllocateMemory(
        (PVOID) &Adapter->XmitUnCached,
        Adapter->XmitUnCachedSize,
        0,
        HighestAcceptableMax);

    if (!Adapter->XmitUnCached)
    {
        //ZD1205LogError(Adapter, EVENT_15, NDIS_ERROR_CODE_OUT_OF_RESOURCES, 0);

        INITSTR((_T("Could not allocate %d bytes for XmitUnCached mem\n"), Adapter->XmitUnCachedSize));
        return NDIS_STATUS_FAILURE;
    }

    INITSTR((_T("Allocated %08x   %8d bytes for XmitUnCached mem\n"), 
				Adapter->XmitUnCached, Adapter->XmitUnCachedSize));

    //INITSTR((_T("Adapter->XmitUnCachedSize = %d\n"), Adapter->XmitUnCachedSize));

    // initialize this recently allocated area to zeros
    NdisZeroMemory((PVOID) Adapter->XmitUnCached, Adapter->XmitUnCachedSize);


    return NDIS_STATUS_SUCCESS;
}

//-----------------------------------------------------------------------------
// Procedure:   SetupTransmitQueues
//
// Description: Setup TCBs, TBDs and coalesce buffers at INIT time. This
//              routine may also be called at RESET time.
//
// Arguments:
//    Adapter - ptr to Adapter object instance
//    DebugPrint - A boolean value that will be TRUE if this routine is to
//                 write all of transmit queue debug info to the debug terminal.
//
// Returns:    (none)
//-----------------------------------------------------------------------------

VOID
SetupTransmitQueues(
                    IN PZD1205_ADAPTER Adapter,
                    IN BOOLEAN DebugPrint
                    )

{

    // TCB local variables
    PZD1205SwTcb  SwTcbPtr;         // cached TCB list logical pointers
    PHWTCB_STRUC HwTcbPtr;           // uncached TCB list logical pointers
    ULONG       HwTcbPhys;          // uncached TCB list physical pointer
    UINT        TcbCount;

    // TBD local variables
    PTBD_STRUC  HwTbdPtr;           // uncached TBD list pointers
    ULONG       HwTbdPhys;          // uncached TBD list physical pointer

	PCTRL_STRUC	HwCtrlPtr;
	ULONG		HwCtrlPhys;

	PHEADER_STRUC	HwHeaderPtr;
	ULONG			HwHeaderPhys;

    UINT        CoalesceCount;
    PCOALESCE   Coalesce;
    PUCHAR      CoalesceBufVirtPtr;
    ULONG       CoalesceBufPhysPtr;

	//WPA support
    PMIC_STRUC	HwMicPtr;
	ULONG		HwMicPhys;

    DEBUGFUNC(_T("SetupTransmitQueues"));
    INITSTR((_T("\n")));

    // init the lists
    QueueInitList(&Adapter->TxCBList);
    QueueInitList(&Adapter->ActiveChainList);
    //QueueInitList(&Adapter->CompletedChainList);
    QueueInitList(&Adapter->CoalesceBufferList);

    // print some basic sizing debug info
    INITSTRTX(DebugPrint, (_T("sizeof(SwTcb)=%x\n"), sizeof(ZD1205SwTcb)));
    INITSTRTX(DebugPrint, (_T("sizeof(HwTcb)=%x\n"), sizeof(HWTCB_STRUC)));
    INITSTRTX(DebugPrint, (_T("sizeof(HwTbd)=%x\n"), sizeof(TBD_STRUC)));
    INITSTRTX(DebugPrint, (_T("sizeof(CTRL_STRUC)=%x\n"), sizeof(CTRL_STRUC)));
    INITSTRTX(DebugPrint, (_T("sizeof(HEADER_STRUC)=%x\n"), sizeof(HEADER_STRUC)));
    INITSTRTX(DebugPrint, (_T("sizeof(HWCOALESCE)=%x\n"), sizeof(HWCOALESCE)));
    INITSTRTX(DebugPrint, (_T("Adapter->NumTcb=%d\n"), Adapter->NumTcb));
    INITSTRTX(DebugPrint, (_T("Adapter->NumTbdPerTcb=%d\n"), Adapter->NumTbdPerTcb));
    INITSTRTX(DebugPrint, (_T("Adapter->NumTbd=%d\n"), Adapter->NumTbd));

    // Setup the initial pointers to the HW and SW TCB data space
    SwTcbPtr = (PZD1205SwTcb) Adapter->XmitCached;
    HwTcbPtr = (PHWTCB_STRUC) Adapter->XmitUnCached;
#ifndef HOST_IF_USB
    HwTcbPhys = NdisGetPhysicalAddressLow(Adapter->XmitUnCachedPhys);
#else
    HwTcbPhys = (ULONG)Adapter->XmitUnCached;
#endif
    // Setup the initial pointers to the TBD data space.
    // TBDs are located immediately following the TCBs
    HwTbdPtr = (PTBD_STRUC) (Adapter->XmitUnCached +
        (sizeof(HWTCB_STRUC) * Adapter->NumTcb));
    HwTbdPhys = HwTcbPhys + (sizeof(HWTCB_STRUC) * Adapter->NumTcb);

	// Setup yhe initial pointers to the Control Setting space
	// CTRLs are located immediately following the TBDs
	HwCtrlPtr = (PCTRL_STRUC) ((ULONG)HwTbdPtr + 
		(sizeof(TBD_STRUC) * Adapter->NumTbd));
	HwCtrlPhys = HwTbdPhys + 
		(sizeof(TBD_STRUC) * Adapter->NumTbd);

	// Setup yhe initial pointers to the Mac Header space
	// MACHEADERs are located immediately following the CTRLs
	HwHeaderPtr = (PHEADER_STRUC) ( (ULONG)HwCtrlPtr +
		(sizeof(CTRL_STRUC) * Adapter->NumTcb) );
	HwHeaderPhys = HwCtrlPhys + 
		(sizeof(CTRL_STRUC) * Adapter->NumTcb);

	//WPA support
	HwMicPtr = (PMIC_STRUC) ((ULONG)HwHeaderPtr + (sizeof(HEADER_STRUC) * Adapter->NumTcb) +
		(sizeof(HWCOALESCE) * Adapter->NumCoalesce) );
	HwMicPhys = HwHeaderPhys + (sizeof(HEADER_STRUC) * Adapter->NumTcb) +
		(sizeof(HWCOALESCE) * Adapter->NumCoalesce);

    // Go through and set up each TCB
    for (TcbCount = 0; TcbCount < Adapter->NumTcb;
    TcbCount++, SwTcbPtr++, HwTcbPtr++, HwTcbPhys += sizeof(HWTCB_STRUC),
        HwTbdPtr = (PTBD_STRUC) (((ULONG_PTR) HwTbdPtr) +
        ((ULONG) (sizeof(TBD_STRUC) * Adapter->NumTbdPerTcb))),
        HwTbdPhys += (sizeof(TBD_STRUC) * Adapter->NumTbdPerTcb),
		HwCtrlPtr++, HwCtrlPhys += sizeof(CTRL_STRUC),
		HwHeaderPtr++, HwHeaderPhys += sizeof(HEADER_STRUC),
		HwMicPtr++, HwMicPhys += sizeof(MIC_STRUC)) //WPA support
    {

        // point the cached TCB to the logical address of the uncached one
		SwTcbPtr->TcbCount = TcbCount;
        SwTcbPtr->Tcb = HwTcbPtr;
        SwTcbPtr->TcbPhys = (ULONG)HwTcbPhys;
        SwTcbPtr->FirstTbd = HwTbdPtr;
        SwTcbPtr->FirstTbdPhys = HwTbdPhys;

		SwTcbPtr->HwCtrlPtr = HwCtrlPtr;
		SwTcbPtr->HwCtrlPhys = HwCtrlPhys;

		SwTcbPtr->HwHeaderPtr = HwHeaderPtr;
		SwTcbPtr->HwHeaderPhys = HwHeaderPhys;

		//WPA support
		SwTcbPtr->HwMicPtr = HwMicPtr;
		SwTcbPtr->HwMicPhys = HwMicPhys;

        // initialize the uncached TCB contents -- status is zeroed
        HwTcbPtr->CbStatus = 0xffffffff;
        HwTcbPtr->CbCommand = CB_S_BIT;
        HwTcbPtr->TxCbFirstTbdAddrLowPart = HwTbdPhys;
		HwTcbPtr->TxCbFirstTbdAddrHighPart = 0;
		HwTcbPtr->TxCbTbdNumber = 0;
        if (TcbCount == (Adapter->NumTcb -1)){
			// Turn around
#ifndef HOST_IF_USB
            HwTcbPtr->NextCbPhyAddrLowPart =  NdisGetPhysicalAddressLow(Adapter->XmitUnCachedPhys);
#else
            HwTcbPtr->NextCbPhyAddrLowPart =  (ULONG)Adapter->XmitUnCached;
#endif			
			HwTcbPtr->NextCbPhyAddrHighPart = 0;
		}
        else{
            HwTcbPtr->NextCbPhyAddrLowPart = HwTcbPhys + sizeof(HWTCB_STRUC);
			HwTcbPtr->NextCbPhyAddrHighPart = 0;
		}

        // add this TCB to the free list
        QueuePutTail(&Adapter->TxCBList, &SwTcbPtr->Link);
    }

    // Setup pointers to the first coalesce buffer.  The SW coalesce structures
    // will be located immediately after the last SwTcb, and the HW coalesce
    // structures will located after the last TBD on a 2k boundary.
    Coalesce = (PCOALESCE) SwTcbPtr;
    CoalesceBufVirtPtr = (PUCHAR) HwHeaderPtr;
    CoalesceBufPhysPtr = (ULONG) HwHeaderPhys;

    // Go through each coalesce buffer
    for (CoalesceCount = 0; CoalesceCount < Adapter->NumCoalesce; CoalesceCount++, Coalesce++)
    {
//            CoalesceCount, Coalesce, CoalesceBufVirtPtr, CoalesceBufPhysPtr));

        QueuePutTail(&Adapter->CoalesceBufferList, &Coalesce->Link);

        // Set the phys and virtual pointers for each coalesce buffer
        Coalesce->HwCoalesce = (PHWCOALESCE)CoalesceBufVirtPtr;
        Coalesce->HwCoalescePhys = CoalesceBufPhysPtr;

        CoalesceBufPhysPtr += sizeof(HWCOALESCE);
        CoalesceBufVirtPtr += sizeof(HWCOALESCE);
    }


    // Initialize the Transmit queueing pointers to NULL
    Adapter->FirstTxQueue = (PNDIS_PACKET) NULL;
    Adapter->LastTxQueue = (PNDIS_PACKET) NULL;
    Adapter->NumPacketsQueued = 0;
}
//CountryToRegion
// This routine convert Country code (Read from registry storage)
//   to Region code.
// Mapping table 
//
#define NUM_COUNTRY_TBL ( sizeof (countryToRegionTbl) / sizeof (COUNTRY_TABLE) )

typedef struct _COUNTRY_TABLE {
	UCHAR		CountryCode1;
	UCHAR		CountryCode2;
	UCHAR		Zero;
	UCHAR		regionCode;
} COUNTRY_TABLE, *PCOUNTRY_TABLE;

COUNTRY_TABLE countryToRegionTbl[] = {
    {'A','F',0,  49},   // %Afghanistan%
    {'A','L',0,  49},   // %Albania%
    {'D','Z',0,  49},   // %Algeria%
    {'A','S',0,  16},   // %AmericanSamoa%
    {'A','D',0,  49},   // %Andorra%
    {'A','O',0,  49},   // %Angola%
    {'A','I',0,  49},   // %Anguilla%
    {'A','Q',0,  16},   // %Antarctica%
    {'A','G',0,  16},   // %AntiguaandBarbuda%
    {'A','R',0,  16},   // %Argentina%
    {'A','M',0,  49},   // %Armenia%
    {'A','W',0,  16},   // %Aruba%
    {'A','U',0,  16},   // %Australia%
    {'A','T',0,  49},   // %Austria%
    {'A','Z',0,  49},   // %Azerbaijan%
    {'B','S',0,  16},   // %BahamasThe%
    {'B','H',0,  49},   // %Bahrain%
    {'B','D',0,  49},   // %Bangladesh%
    {'B','B',0,  16},   // %Barbados%
    {'B','Y',0,  49},   // %Belarus%
    {'B','E',0,  49},   // %Belgium%
    {'B','Z',0,  49},   // %Belize%
    {'B','J',0,  49},   // %Benin%
    {'B','M',0,  16},   // %Bermuda%
    {'B','T',0,  49},   // %Bhutan%
    {'B','O',0,  16},   // %Bolivia%
    {'B','A',0,  49},   // %BosniaandHerzegovina%
    {'B','W',0,  49},   // %Botswana%
    {'B','V',0,  16},   // %BouvetIsland%
    {'B','R',0,  16},   // %Brazil%
    {'I','O',0,  49},   // %BritishIndianOceanTerritory%
    {'B','N',0,  49},   // %Brunei%
    {'B','G',0,  49},   // %Bulgaria%
    {'B','F',0,  49},   // %BurkinaFaso%
    {'B','I',0,  49},   // %Burundi%
    {'K','H',0,  49},   // %Cambodia%
    {'C','M',0,  16},   // %Cameroon%
    {'C','A',0,  16},   // %Canada%
    {'C','V',0,  49},   // %CapeVerde%
    {'K','Y',0,  49},   // %CaymanIslands%
    {'C','F',0,  16},   // %CentralAfricanRepublic%
    {'T','D',0,  16},   // %Chad%
    {'C','L',0,  16},   // %Chile%
    {'C','N',0,  16},   // %China%
    {'C','X',0,  16},   // %ChristmasIsland%
    {'C','C',0,  16},   // %CocosKeelingIslands%
    {'C','O',0,  16},   // %Colombia%
    {'K','M',0,  49},   // %Comoros%
	{'C','D',0,  49},   // %CongoDRC%
	{'C','G',0,  49},   // %Congo%
    {'C','K',0,  16},   // %CookIslands%
    {'C','R',0,  16},   // %CostaRica%
    {'C','I',0,  49},   // %CotedIvoire%
    {'H','R',0,  49},   // %Croatia%
    {'C','U',0,  00},   // %Cuba%
    {'C','Y',0,  49},   // %Cyprus%
    {'C','Z',0,  49},   // %CzechRepublic%
    {'D','K',0,  49},   // %Denmark%
    {'D','J',0,  49},   // %Djibouti%
    {'D','M',0,  49},   // %Dominica%
    {'D','O',0,  49},   // %DominicanRepublic%
    {'T','L',0,  00},   // %EastTimor%
    {'E','C',0,  16},   // %Ecuador%
    {'E','G',0,  49},   // %Egypt%
    {'S','V',0,  00},   // %ElSalvador%
    {'G','Q',0,  00},   // %EquatorialGuinea%
    {'E','R',0,  00},   // %Eritrea%
    {'E','E',0,  49},   // %Estonia%
    {'E','T',0,  00},   // %Ethiopia%
    {'F','K',0,  00},   // %FalklandIslandsIslasMalvinas%
    {'F','O',0,  00},   // %FaroeIslands%
    {'F','J',0,  00},   // %FijiIslands%
    {'F','I',0,  49},   // %Finland%
    {'F','R',0,  50},   // %France%
    {'G','F',0,  00},   // %FrenchGuiana%
    {'P','F',0,  00},   // %FrenchPolynesia%
    {'T','F',0,  00},   // %FrenchSouthernandAntarcticLands%
    {'G','A',0,  00},   // %Gabon%
    {'G','M',0,  00},   // %Gambia%
    {'G','E',0,  00},   // %Georgia%
    {'D','E',0,  49},   // %Germany%
    {'G','H',0,  00},   // %Ghana%
    {'G','I',0,  00},   // %Gibraltar%
    {'G','R',0,  49},   // %Greece%
    {'G','L',0,  00},   // %Greenland%
    {'G','D',0,  00},   // %Grenada%
    {'G','P',0,  00},   // %Guadeloupe%
    {'G','U',0,  00},   // %Guam%
    {'G','T',0,  00},   // %Guatemala%
    {'G','W',0,  00},   // %GuineaBissau%
    {'G','N',0,  00},   // %Guinea%
    {'G','Y',0,  00},   // %Guyana%
    {'H','T',0,  00},   // %Haiti%
    {'H','M',0,  00},   // %HeardIslandandMcDonaldIslands%
    {'H','N',0,  00},   // %Honduras%
    {'H','K',0,  16},   // %HongKongSAR%
    {'H','U',0,  49},   // %Hungary%
    {'I','S',0,  49},   // %Iceland%
    {'I','N',0,  49},   // %India%
    {'I','D',0,  49},   // %Indonesia%
    {'I','R',0,  00},   // %Iran%
    {'I','Q',0,  00},   // %Iraq%
    {'I','E',0,  49},   // %Ireland%
    {'I','L',0,  80},   // %Israel%
    {'I','T',0,  49},   // %Italy%
    {'J','M',0,  00},   // %Jamaica%
    {'J','P',0,  64},   // %Japan%
    {'J','O',0,  49},   // %Jordan%
    {'K','Z',0,  00},   // %Kazakhstan%
    {'K','E',0,  00},   // %Kenya%
    {'K','I',0,  00},   // %Kiribati%
    {'K','R',0,  16},   // %Korea%
    {'K','W',0,  49},   // %Kuwait%
    {'K','G',0,  00},   // %Kyrgyzstan%
    {'L','A',0,  00},   // %Laos%
    {'L','V',0,  49},   // %Latvia%
    {'L','B',0,  00},   // %Lebanon%
    {'L','S',0,  00},   // %Lesotho%
    {'L','R',0,  00},   // %Liberia%
    {'L','Y',0,  00},   // %Libya%
    {'L','I',0,  49},   // %Liechtenstein%
    {'L','T',0,  49},   // %Lithuania%
    {'L','U',0,  49},   // %Luxembourg%
    {'M','O',0,  00},   // %MacauSAR%
    {'M','K',0,  00},   // %MacedoniaFormerYugoslavRepublicof%
    {'M','G',0,  00},   // %Madagascar%
    {'M','W',0,  00},   // %Malawi%
    {'M','Y',0,  16},   // %Malaysia%
    {'M','V',0,  00},   // %Maldives%
    {'M','L',0,  00},   // %Mali%
    {'M','T',0,  49},   // %Malta%
    {'M','H',0,  00},   // %MarshallIslands%
    {'M','Q',0,  00},   // %Martinique%
    {'M','R',0,  00},   // %Mauritania%
    {'M','U',0,  00},   // %Mauritius%
    {'Y','T',0,  00},   // %Mayotte%
    {'M','X',0,  81},   // %Mexico%
    {'F','M',0,  00},   // %Micronesia%
    {'M','D',0,  00},   // %Moldova%
    {'M','C',0,  49},   // %Monaco%
    {'M','N',0,  00},   // %Mongolia%
    {'M','S',0,  00},   // %Montserrat%
    {'M','A',0,  00},   // %Morocco%
    {'M','Z',0,  00},   // %Mozambique%
    {'M','M',0,  00},   // %Myanmar%
    {'N','A',0,  00},   // %Namibia%
    {'N','R',0,  00},   // %Nauru%
    {'N','P',0,  00},   // %Nepal%
    {'A','N',0,  00},   // %NetherlandsAntilles%
    {'N','L',0,  49},   // %NetherlandsThe%
    {'N','C',0,  00},   // %NewCaledonia%
    {'N','Z',0,  16},   // %NewZealand%
    {'N','I',0,  16},   // %Nicaragua%
    {'N','E',0,  00},   // %Niger%
    {'N','G',0,  00},   // %Nigeria%
    {'N','U',0,  00},   // %Niue%
    {'N','F',0,  00},   // %NorfolkIsland%
    {'K','P',0,  00},   // %NorthKorea%
    {'M','P',0,  00},   // %NorthernMarianaIslands%
    {'N','O',0,  49},   // %Norway%
    {'O','M',0,  00},   // %Oman%
    {'P','K',0,  00},   // %Pakistan%
    {'P','W',0,  00},   // %Palau%
    {'P','S',0,  00},   // %PalestinianAuthority%
    {'P','A',0,  16},   // %Panama%
    {'P','G',0,  00},   // %PapuaNewGuinea%
    {'P','Y',0,  00},   // %Paraguay%
    {'P','E',0,  16},   // %Peru%
    {'P','H',0,  16},   // %Philippines%
    {'P','N',0,  00},   // %Pitcairn%
    {'P','L',0,  49},   // %Poland%
    {'P','T',0,  49},   // %Portugal%
    {'P','R',0,  00},   // %PuertoRico%
    {'Q','A',0,  00},   // %Qatar%
    {'R','E',0,  00},   // %Reunion%
    {'R','O',0,  00},   // %Romania%
    {'R','U',0,  16},   // %Russia%
    {'R','W',0,  00},   // %Rwanda%
    {'W','S',0,  00},   // %Samoa%
    {'S','M',0,  00},   // %SanMarino%
    {'S','T',0,  00},   // %SaoTomeandPrincipe%
    {'S','A',0,  16},   // %SaudiArabia%
    {'S','N',0,  00},   // %Senegal%
    {'S','C',0,  00},   // %Seychelles%
    {'S','L',0,  00},   // %SierraLeone%
    {'S','G',0,  50},   // %Singapore%
    {'S','K',0,  49},   // %Slovakia%
    {'S','I',0,  49},   // %Slovenia%
    {'S','B',0,  16},   // %SolomonIslands%
    {'S','O',0,  49},   // %Somalia%
    {'Z','A',0,  49},   // %SouthAfrica%
    {'G','S',0,  00},   // %SouthGeorgiaandtheSouthSandwichIslands%
    {'E','S',0,  49},   // %Spain%
    {'L','K',0,  00},   // %SriLanka%
    {'S','H',0,  00},   // %StHelena%
    {'K','N',0,  00},   // %StKittsandNevis%
    {'L','C',0,  00},   // %StLucia%
    {'P','M',0,  00},   // %StPierreandMiquelon%
    {'V','C',0,  00},   // %StVincentandtheGrenadines%
    {'S','D',0,  00},   // %Sudan%
    {'S','R',0,  00},   // %Suriname%
    {'S','J',0,  00},   // %SvalbardandJanMayen%
    {'S','Z',0,  00},   // %Swaziland%
    {'S','E',0,  49},   // %Sweden%
    {'C','H',0,  49},   // %Switzerland%
    {'S','Y',0,  00},   // %Syria%
    {'T','W',0,  49},   // %Taiwan%
    {'T','J',0,  00},   // %Tajikistan%
    {'T','Z',0,  00},   // %Tanzania%
    {'T','H',0,  49},   // %Thailand%
    {'T','G',0,  00},   // %Togo%
    {'T','K',0,  00},   // %Tokelau%
    {'T','O',0,  00},   // %Tonga%
    {'T','T',0,  00},   // %TrinidadandTobago%
    {'T','N',0,  00},   // %Tunisia%
    {'T','R',0,  49},   // %Turkey%
    {'T','M',0,  00},   // %Turkmenistan%
    {'T','C',0,  00},   // %TurksandCaicosIslands%
    {'T','V',0,  00},   // %Tuvalu%
    {'U','G',0,  00},   // %Uganda%
    {'U','A',0,  00},   // %Ukraine%
    {'A','E',0,  49},   // %UnitedArabEmirates%
    {'G','B',0,  49},   // %UnitedKingdom%
    {'U','M',0,  16},   // %UnitedStatesMinorOutlyingIslands%
    {'U','S',0,  16},   // %UnitedStates%
    {'U','Y',0,  16},   // %Uruguay%
    {'U','Z',0,  00},   // %Uzbekistan%
    {'V','U',0,  00},   // %Vanuatu%
    {'V','A',0,  00},   // %VaticanCity%
    {'V','E',0,  16},   // %Venezuela%
    {'V','N',0,  00},   // %VietNam%
    {'V','G',0,  00},   // %VirginIslandsBritish%
    {'V','I',0,  00},   // %VirginIslands%
    {'W','F',0,  00},   // %WallisandFutuna%
    {'E','H',0,  00},   // %WesternSahara%
    {'Y','E',0,  00},   // %Yemen%
    {'Y','U',0,  00},   // %Yugoslavia%
    {'Z','M',0,  00},   // %Zambia%
    {'Z','W',0,  00},   // %Zimbabwe%
    {'0','0',0,  00}    // Default
};
// return ZD_Region_USA if no entry matched.
USHORT CountryToRegion(PUCHAR pCountryCode)
{
	UINT	i;
	PCOUNTRY_TABLE	pCountryTbl=&countryToRegionTbl[0];
	
	for (i=0; i<NUM_COUNTRY_TBL; i++)
	{
		if (memcmp(&pCountryTbl->CountryCode1, pCountryCode, 2)==0)
		{
			if (pCountryTbl->regionCode==0)
				return ZD_Region_USA; // Use USA as default
			else
				return pCountryTbl->regionCode;
		}
		pCountryTbl++;
	}
	return ZD_Region_USA;
}

//-----------------------------------------------------------------------------
// Procedure:   InitializeAdapter
//
// Description: This routine performs a reset on the adapter, and configures
//              the adapter.  This includes configuring the ZD1205 LAN
//              controller, validating and setting the node address, detecting
//              and configuring the Phy chip on the adapter, and initializing
//              all of the on chip counters.
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//
// Returns:
//      TRUE - If the adapter was initialized
//      FALSE - If the adapter failed initialization
//-----------------------------------------------------------------------------

BOOLEAN
InitializeAdapter(
                  IN PZD1205_ADAPTER Adapter,
				  BOOLEAN	AbnormalPOR
                  )

{
	ULONG		Address;
	ULONG		tmpvalue;
	UCHAR	BeaconInterval[2];
    DEBUGFUNC(_T("InitializeAdapter"));

    #if fPROG_FLASH
    Adapter->AllowAccessRegister = TRUE;
    #endif
	//++ Must locate in the front of InitializeAdapter
	//	 to help to detect abnormal POR
	// Write after PNP port
	ZD1205_WRITE_REGISTER(Adapter, AfterPNP, 1);

//JWEI 2004/03/18
#if fWRITE_WORD_REG || fREAD_MUL_REG
// Must get this information before any register write
    ZD1211_READ_REGISTER(Adapter, cADDR_ENTRY_TABLE, &tmpvalue, FALSE);
    Adapter->AddrEntryTable = (USHORT) tmpvalue;

    //INITSTR((_T("Adapter->AddrEntryTable = %x\n"), Adapter->AddrEntryTable)); 

#endif
	// Store the permanent node address
	ZD1205_READ_REGISTER(Adapter,E2P_MACADDR_P1, &tmpvalue);
	ZDDbgPrint((_T("E2P_MACADDR_P1 = %x\n"), tmpvalue));
	Adapter->AiPermanentNodeAddress[0] = (UCHAR)tmpvalue;
	Adapter->AiPermanentNodeAddress[1] = (UCHAR)(tmpvalue >> 8);
	Adapter->AiPermanentNodeAddress[2] = (UCHAR)(tmpvalue >> 16);
	Adapter->AiPermanentNodeAddress[3] = (UCHAR)(tmpvalue >> 24);
	ZD1205_READ_REGISTER(Adapter,E2P_MACADDR_P2, &tmpvalue);
	ZDDbgPrint((_T("E2P_MACADDR_P2 = %x\n"), tmpvalue));
	Adapter->AiPermanentNodeAddress[4] = (UCHAR)tmpvalue;
	Adapter->AiPermanentNodeAddress[5] = (UCHAR)(tmpvalue >> 8);

    // Check if there was a node address over-ride.  If there isn't then we'll
    // use our adapter's permanent node address as our "current" node address.
    // If a node address over-ride is present in the registry, then we'll use
    // that override address as our node address instead of the permanent
    // address in the adapter's EEPROM
    if ((Adapter->AiNodeAddress[0]==0) &&
        (Adapter->AiNodeAddress[1]==0) &&
        (Adapter->AiNodeAddress[2]==0) &&
        (Adapter->AiNodeAddress[3]==0) &&
        (Adapter->AiNodeAddress[4]==0) &&
        (Adapter->AiNodeAddress[5]==0))
    {
        // No node address override so use the permanent address
        INITSTR((_T("InitializeAdapter: No node address over-ride, using permanent address\n")));
        NdisMoveMemory(Adapter->AiNodeAddress,
            Adapter->AiPermanentNodeAddress,
            ETH_LENGTH_OF_ADDRESS);
    }

    INITSTR((_T("InitializeAdapter: Node Address is  %.2x %.2x %.2x %.2x %.2x %.2x\n"),
        Adapter->AiNodeAddress[0],
        Adapter->AiNodeAddress[1],
        Adapter->AiNodeAddress[2],
        Adapter->AiNodeAddress[3],
        Adapter->AiNodeAddress[4],
        Adapter->AiNodeAddress[5]
        ));

    // Validate our current node address (make sure its not a mulicast)
    if ((UCHAR) Adapter->AiNodeAddress[0] & 1)
    {
        INITSTR((_T("InitializeAdapter: Node address invalid -- its a MC address\n")));


	}

	// Set GPI_EN be zero. ie. Disable GPI (Requested by Ahu)
	ZD1205_WRITE_REGISTER(Adapter,GPI_EN, 0x0);

	// Set Phy registers
	PHY_Reg_Preset(Adapter);

	// Region code: high word of long(0x900) in e2prom.
	ZD1205_READ_REGISTER(Adapter, E2P_SUBID, &tmpvalue);
	if (Adapter->CountryCode[0] == 0xFF) 
	{// use default value sotred at E2PROM

		Adapter->OperationRegionCode = (USHORT)(tmpvalue >> 16);
	}
	else
	{

		Adapter->OperationRegionCode = CountryToRegion(Adapter->CountryCode);
	}

	// Get Allowed Channel and Default Channel
	ZD1205_READ_REGISTER(Adapter, E2P_ALLOWED_CHANNEL, &Adapter->AllowedChannel);
	ASSERT(Adapter->AllowedChannel);
	//Adapter->AllowedChannel = 0x000107fF;	// mean default 1, allow BIT_0 ~ BIT_11

	ZDDbgPrint((_T("AllowedChannel=0x%x\n"), Adapter->AllowedChannel));
	DbgPrint("AllowedChannel=0x%x\r\n", Adapter->AllowedChannel);
	if (Adapter->CountryCode[0] != 0xFF)
	{// Country code is specified in s/w registry, the channel usage
	 //  dependent on it.
		
		switch (Adapter->OperationRegionCode)
		{
		case ZD_Region_Default:			// default channel 1-11
		case ZD_Region_USA:				// USA and ANZ use ch1-11
		case ZD_Region_Canada:			// 
			Adapter->AllowedChannel=ZD_CH_1_11;
			break;
		case ZD_Region_EuropeAustralia:
		case ZD_Region_Spain:			// ETSI use ch1-13
			Adapter->AllowedChannel=ZD_CH_1_13;
			break;
		case ZD_Region_France:			//France,Singapore ch10-13
			Adapter->AllowedChannel=ZD_CH_10_13+(10<<16);// default channel(use high word):10
			break;
		case ZD_Region_Japan:			// Japan use ch1-14
			Adapter->AllowedChannel=ZD_CH_1_14;
			break;
		case ZD_Region_Israel:			// Israel use ch3-9
			Adapter->AllowedChannel=ZD_CH_3_9+(3<<16); // default channel(use high word):3
			break;
		case ZD_Region_Mexico:			// Mexico use ch10,11.
			Adapter->AllowedChannel=ZD_CH_10_11+(10<<16);//default channel(use high word):10
			break;
		default:
			Adapter->AllowedChannel=ZD_CH_1_11;
		}
	}

	if ((Adapter->AllowedChannel & 0xFFFF0000)==0)
	{ // No default channel specified, default to use channel 1
		Adapter->AllowedChannel |= 0x10000;
	}
	// Update PHY registers according to region-code
	//ZD1205_READ_REGISTER(Adapter, E2P_REGION_CODE, &Adapter->OperationRegionCode);
	// using default filterband  Roger Huang 2004/01/15
	Set_FilterBand(Adapter, 0);

	//++ Set RF module
	//Adapter->SA2400DelayTime = 0x1000;
	ZD1205_READ_REGISTER(Adapter, E2P_POD, &Adapter->RF_Mode);
	ZDDbgPrint((_T("Adapter->RF_Mode = %x\n"), Adapter->RF_Mode));
//JWEI 2004/02/10
#ifdef HOST_IF_USB
    // Read PA type
	Adapter->PA_Type = ((Adapter->RF_Mode) >> 16) & BIT_0_3; // Extract from Bit16-19

// LED2, Link LED in two LED case.
// LED1, Data LED in two LED case,or Data+Link LED in single LED case.
    Adapter->LinkLEDn = LED1; // Default: Link use the same LED as Data.
    if (Adapter->RF_Mode & BIT_4) // Two LED?
        Adapter->LinkLEDn = LED2; // Yes,Use LED2 as Link LED.

    if (Adapter->RF_Mode & BIT_8)
		Adapter->bOverWritePhyRegFromE2P = 1;
    
	if (Adapter->RF_Mode & BIT_9)
		Adapter->bIsNormalSize = 1;

    Adapter->LinkLED_OnDur = 2;
    Adapter->LinkLED_OffDur = 1;
    Adapter->DataLED = 0;
    if (Adapter->RF_Mode & BIT_24)
    {
        Adapter->LinkLED_OnDur = ((Adapter->RF_Mode) >> 25) & 0x3;
        Adapter->LinkLED_OffDur = ((Adapter->RF_Mode) >> 27) & 0x3;
        if (Adapter->RF_Mode & BIT_29)
            Adapter->DataLED = 1;
    }
    JWeiDbgPrint7(("On=%d, Off=%d, Data=%d\n", Adapter->LinkLED_OnDur, Adapter->LinkLED_OffDur, Adapter->DataLED));

    if ( !(Adapter->RF_Mode & BIT_10) ) // The IPC protection: the default is disablesd 
		Adapter->IPCFlag = 4;

    if ( Adapter->RF_Mode & BIT_11 ) // The channel 6 issue for CyberTian  
		Adapter->CTFlag = 1;
	if (Adapter->RF_Mode & BIT_14) // If set, Decrease initial Tx power by 2dB
	{
		Adapter->nPwrIntValueGuard = cPWR_INT_VALUE_GUARD;
	}
	else
	{
		Adapter->nPwrIntValueGuard = 0; 
	}
//---chi---2004/08/10
	Adapter->LED_Ctrl_Type = ((Adapter->RF_Mode) >> 20) & 0x3;
	if (Adapter->LED_Ctrl_Type==2) // for 5LED signal strength indicator, turn off 5 led firstly.
	{
		ZD1211_WRITE_REGISTER(Adapter, SOCGPIO, 0x1F00 , FALSE);
	}

#endif

	Adapter->RF_Mode = (Adapter->RF_Mode & 0xf);

	Set_RF_Channel(Adapter, (Adapter->AllowedChannel >> 16), (UCHAR)Adapter->RF_Mode, 0);

	if (!AbnormalPOR){
		Adapter->ChosenBSS = 0xff;
	}
	/********************************************************************/
	/* Set MAC related registers										*/			
	/********************************************************************/
	// Release the CAM_MODE control
	ZD1205_WRITE_REGISTER(Adapter,CAM_MODE, CAM_STA);
	Adapter->CurrentCAMMode = CAM_STA;

	// Use Ack_Timeout_Ext to tolerance some peers that response slowly.
	// The influence is that the retry frame will be less competitive. It's acceptable.
	//ZD1205_WRITE_REGISTER(Adapter,Ack_Timeout_Ext, 0x3f);
	ZD1205_WRITE_REGISTER(Adapter,Ack_Timeout_Ext, 0x80);

#ifdef OFDM
	ZD1205_WRITE_REGISTER(Adapter,AckTime80211, 0x100);
#endif

	// Set RX_PE_DELAY 0x10 to enlarge the time for decharging tx power.
	ZD1205_WRITE_REGISTER(Adapter,RX_PE_DELAY, 0x70);

	// Keep 44MHz oscillator always on.
	ZD1205_WRITE_REGISTER(Adapter,PS_Ctrl, 0x10000000);
	// ADDA_MBIAS_WarmTime BIT_24~BIT_31 must be set 0x30 to prevent TX_PEX high after
	// Wakeup.
	ZD1205_WRITE_REGISTER(Adapter,ADDA_MBIAS_WarmTime, 0x30000808);

	if (!AbnormalPOR){
		// Set Pseudo IBSS mode
		Adapter->CardSetting.BSSType = INFRASTRUCTURE_BSS;

		// Disable Fragmentation, RTS function
		Adapter->CardSetting.FragThreshold = 0xffff;
		Adapter->CardSetting.RTSThreshold = 0xffff;

		// Set Authentication Type = Auto
		Adapter->CardSetting.AuthMode = Ndis802_11AuthModeAutoSwitch;
	}
	Adapter->CardSetting.FragThreshold=Adapter->fragThreshold;
	Adapter->CardSetting.RTSThreshold=Adapter->rTSThreshold;
	{// Find the maximum/minimum allowed channel number
		int		i;
		UINT	maxAllowedChannelNumber;
		UINT	minAllowedChannelNumber;
		UINT	channelMapping=Adapter->AllowedChannel & ZD_CH_1_14;
		for (i=0; i<14; i++)
		{
			if (((1<<i) & channelMapping))
			{
				minAllowedChannelNumber=i+1;
				break;
			}
		}
		for (i=13; i>=0; i--)
		{
			if (((1<<i) & channelMapping))
			{
				maxAllowedChannelNumber=i+1;
				break;
			}
		}

		if (Adapter->iBSSChannel > maxAllowedChannelNumber)
			Adapter->iBSSChannel = maxAllowedChannelNumber;
		if (Adapter->iBSSChannel < minAllowedChannelNumber)
			Adapter->iBSSChannel = minAllowedChannelNumber;
	}

	Adapter->CardSetting.Channel=(UCHAR)Adapter->iBSSChannel;
	// Set CWmin, CWmax, Slot time
	// CWmin = 0x7f (Bit0 ~ Bit9)
	// CWmax = 0x7f (Bit16 ~ Bit25)
	// Slot Timer = 0x2 ((Bit10 ~ Bit14) + 1)
	Adapter->CWmin_CWmax_ShortSlot = 0x7f043f;
	Adapter->CWmin_CWmax_NormalSlot = 0x7f043f;
    Adapter->CWmin_CWmax_LongSlot = 0x7f047f;

	// If we change CWmin_CWmax, take care of Wep_Protect
#if defined(OFDM) 
	ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_ShortSlot);
	Adapter->ShortSlotEnable = TRUE;
#endif

	// Set RetryMax 8
	ZD1205_WRITE_REGISTER(Adapter,RetryMAX, 0x2);

	// Set SIFS = 20us -- Solve fragmentation problem with Dlink AP.
	//ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x1447c032);

	if (!AbnormalPOR){
		// Set No Encryption
		Adapter->CardSetting.EncryMode = 0;
	}
	ZD1205_WRITE_REGISTER(Adapter,EncryType, Adapter->CardSetting.EncryMode);	// No Encryption

	// Turn off sniffer mode	
	ZD1205_WRITE_REGISTER(Adapter,SnifferOn, 0);

	// Set Hashing table
	if (!AbnormalPOR){
		Adapter->ulGroupHash_P1 = 0;
		Adapter->ulGroupHash_P2 = 0x80000000;
	}
	ZD1205_WRITE_REGISTER(Adapter,GroupHash_P1, Adapter->ulGroupHash_P1);
	ZD1205_WRITE_REGISTER(Adapter,GroupHash_P2, Adapter->ulGroupHash_P2);

	ZD1205_READ_REGISTER(Adapter, FW_USB_SPEED, &tmpvalue);
	Adapter->InitUSB2_0 = (UCHAR) tmpvalue;
	Adapter->IsUSB2_0 = (UCHAR) tmpvalue;


	// Set default supported rates (54M .... 11M, 5.5M, 2M, 1M)
	Adapter->PernmentSupportedRates[0] = 0x01;
	Adapter->PernmentSupportedRates[1] = 0x04;
	Adapter->PernmentSupportedRates[2] = 0x82;
	Adapter->PernmentSupportedRates[3] = 0x84;
	Adapter->PernmentSupportedRates[4] = 0x0b;
	Adapter->PernmentSupportedRates[5] = 0x16;
	
	Adapter->PernmentExtSupportedRates[0] = 0x32;
	Adapter->PernmentExtSupportedRates[1] = 0x08;
	Adapter->PernmentExtSupportedRates[2] = 0x0C;
	Adapter->PernmentExtSupportedRates[3] = 0x12;
	Adapter->PernmentExtSupportedRates[4] = 0x18;
	Adapter->PernmentExtSupportedRates[5] = 0x24;
	Adapter->PernmentExtSupportedRates[6] = 0x30;
	Adapter->PernmentExtSupportedRates[7] = 0x48;
	Adapter->PernmentExtSupportedRates[8] = 0x60;
	Adapter->PernmentExtSupportedRates[9] = 0x6c;

	Adapter->CardSetting.Info_SupportedRates[0] = 0x01;
	Adapter->CardSetting.Info_SupportedRates[1] = 0x04;
	Adapter->CardSetting.Info_SupportedRates[2] = 0x82;
	Adapter->CardSetting.Info_SupportedRates[3] = 0x84;
	Adapter->CardSetting.Info_SupportedRates[4] = 0x0b;
	Adapter->CardSetting.Info_SupportedRates[5] = 0x16;
	
	Adapter->CardSetting.Info_ExtSupportedRates[0] = 0x32;
	Adapter->CardSetting.Info_ExtSupportedRates[1] = 0x08;
	Adapter->CardSetting.Info_ExtSupportedRates[2] = 0x0C;
	Adapter->CardSetting.Info_ExtSupportedRates[3] = 0x12;
	Adapter->CardSetting.Info_ExtSupportedRates[4] = 0x18;
	Adapter->CardSetting.Info_ExtSupportedRates[5] = 0x24;
	Adapter->CardSetting.Info_ExtSupportedRates[6] = 0x30;
	Adapter->CardSetting.Info_ExtSupportedRates[7] = 0x48;
	Adapter->CardSetting.Info_ExtSupportedRates[8] = 0x60;
	Adapter->CardSetting.Info_ExtSupportedRates[9] = 0x6c;

	ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
	tmpvalue |= BIT_0;
	ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
	
	ZD1205_WRITE_REGISTER(Adapter,MandatoryRateTbl, 0x150f); //1,2,5.5,11,6,12,24


	if (Adapter->defaultTxRate==0)
	{ // Auto
		UCHAR k;

		for (k=0; k<=0xb; k++)
			Adapter->AllowRateArray[k] = k;

		Adapter->AllowRateArrayCount = 12;
	}
	else
	{ // Fixed at a particular rate.
		Adapter->AllowRateArray[0]=Rate_Convert((UCHAR)Adapter->defaultTxRate,0);
		Adapter->AllowRateArrayCount = 1;
	}
	


	ZD1205_WRITE_REGISTER(Adapter,CtlReg1, 0xa4);	//Bit0:		
											//Bit1:		PwrMgt mode
											//Bit2-4 :	Highest basic rate
											//Bit5:		Lock bit
											//Bit6:		PLCP weight select
											//Bit7:		PLCP switch

	if (!AbnormalPOR){



	// Set default SSID "Any"
		Adapter->CardSetting.Info_SSID[0] = 0x0;
		Adapter->CardSetting.Info_SSID[1] = 0x0;
    }

	Adapter->BeaconUpdateLockEvent = CreateEvent(NULL, FALSE, TRUE, NULL);
	Adapter->AP_MgtEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	// Set default Beacon Interval
	BeaconInterval[0] = 0x64;
	BeaconInterval[1] = 0x0;
    ZD1205_WRITE_REGISTER(Adapter,BCNInterval, 0x00000064);
	UpdateBeaconInterval(Adapter, &BeaconInterval[0]);	
	
	// Set default capability, support short preamble.
	Adapter->CardSetting.Capability = 0x0420;

	// Set default BSSID = 00:00:00:00:00:00
	Adapter->BSSID[0] = 0x00;
	Adapter->BSSID[1] = 0x00;
	Adapter->BSSID[2] = 0x00;
	Adapter->BSSID[3] = 0x00;
	Adapter->BSSID[4] = 0x00;
	Adapter->BSSID[5] = 0x00;
	ZD1205_WRITE_REGISTER(Adapter,BSSID_P1, 0x00000000);
	ZD1205_WRITE_REGISTER(Adapter,BSSID_P2, 0x00000000);

	// Set MAC Address
	Address = 0;
	Address = ( (Adapter->AiNodeAddress[0]) |
		((ULONG)Adapter->AiNodeAddress[1] << 8) |
		((ULONG)Adapter->AiNodeAddress[2] << 16) |
		((ULONG)Adapter->AiNodeAddress[3] << 24) 
		);
	ZD1205_WRITE_REGISTER(Adapter,MACAddr_P1, Address);
	Address = 0;
	Address = ( (Adapter->AiNodeAddress[4]) |
		((ULONG)Adapter->AiNodeAddress[5] << 8) 
		);
	ZD1205_WRITE_REGISTER(Adapter,MACAddr_P2, Address);

	// Set ADDA_PwrDwn_Ctrl 
	ZD1205_WRITE_REGISTER(Adapter,ADDA_PwrDwn_Ctrl, 0x0);

	// Initialize BCNATIM needed registers	
	//++ Make sure BcnInterval > PreTBTT > ATIMWnd >= 0
	ZD1205_WRITE_REGISTER(Adapter,BCNInterval, 100);
	ZD1205_WRITE_REGISTER(Adapter,Pre_TBTT, 95);
	ZD1205_WRITE_REGISTER(Adapter,ATIMWndPeriod, 0);
	//--
	ZD1205_WRITE_REGISTER(Adapter,BCNPLCPCfg, 0x00f00401);
	ZD1205_WRITE_REGISTER(Adapter,PHYDelay, 0);	
	
	Adapter->BakerPreamble = FALSE;

	ZD1205_WRITE_REGISTER(Adapter,Wep_Protect, 0x114);

	// Set default debug port
	ZD1205_WRITE_REGISTER(Adapter,0x804, 0x408);

	//ZD1205_WRITE_REGISTER(Adapter,AfterPNP, 1);

	//Adapter->RateLine1 = 100;
	//Adapter->RateLine2 = 10;
	//Adapter->RateLine3 = 3;

	//Adapter->RiseConditionThreshold = 5;
	//Adapter->DownConditionThreshold = 3;
	// jxiaoDbg for rate adaptation
	//DbgPrint("RateLine1 = %u\n", Adapter->RateLine1);
	//DbgPrint("RateLine2 = %u\n", Adapter->RateLine2);
	//DbgPrint("RateLine3 = %u\n", Adapter->RateLine3);
	//DbgPrint("AutoRateMonitorInterval=%u\n",Adapter->autoRateMonitorInterval); // Used by Adapter->AutoRateMonitorTimer
	//DbgPrint("RiseConditionThreshold=%u\n",Adapter->RiseConditionThreshold);
	//DbgPrint("DownConditionThreshold=%u\n",Adapter->DownConditionThreshold);
	

	//Adapter->IntMask = 0x4F;
    Adapter->IntMask = 0x4B;  // skip retry_fail

	//Adapter->ShowSeq = FALSE;
	//Adapter->DbgMsgFlag |=  (DBGMSG_TX);
	//Adapter->DbgMsgFlag |=  (DBGMSG_OID);
	//Adapter->DbgMsgFlag = 0;
    //Adapter->stop = 0;

#ifdef HOST_IF_USB
	Adapter->USBDbgFlag = 0;
//	Adapter->USBDbgFlag |= DBGMSG_TX;
//	Adapter->USBDbgFlag |= DBGMSG_OID;
//	Adapter->USBDbgFlag |= DBGMSG_RX;
//	Adapter->USBDbgFlag |= DBGMSG_TX;
//	Adapter->USBDbgFlag |= DBGMSG_TX;
//	Adapter->USBDbgFlag = 0XFFFF;
#endif

	//Adapter->LastScanFrmLen = -1;
	Adapter->func8021x = TRUE;

	//WPA support
	Adapter->PkInstalled = 0;
	Adapter->GkInstalled = 0;
	Adapter->MICFailureCount = 0;
	Adapter->MICFailureTimer = 0;
	Adapter->DisallowAssociateTimer = 0;
#if fANT_DIVERSITY
	// For Antenna Diversity Parameters
	//Adapter->Ant_MonitorDur1 = 100;
	//Adapter->Ant_MonitorDur2 = 1; // 1: 20ms, 2:30ms, 3:40ms .....
	Adapter->bTxFoloRx=TRUE;
	Adapter->MinSamplesInMT2 = 2;
    //Adapter->NiceSQThr = 48;            // set by Advanced Page
    Adapter->NiceSQThr_OFDM = Adapter->NiceSQThr;// set by Advanced Page
    //Adapter->EnableSwAntennaDiv = TRUE; // set by Advanced Page
#endif
	Adapter->WorseSQThr = 0x48;
    Adapter->PhyTest = 4;                   //JWEI 2004/02/13
    Adapter->JWeiDbgCnt3 = 0;
#if fWRITE_WORD_REG
    //Adapter->FlashType = 0xFF;
    #endif
        
	//InitializeProtectedRegs(Adapter);
	//Adapter->ProtectRegs = FALSE;

	
	Adapter->CardSetting.LastSentTxRate = 0xb; // the default is 54M
	if ( (Adapter->IsUSB2_0) || (Adapter->busbswitch == 1))
	    Adapter->CardSetting.LastSentRxRate = 0xb; // the default is 54M
	else
	    Adapter->CardSetting.LastSentRxRate = 0x3; // the default is 11M

    ZD1205_WRITE_REGISTER(Adapter,RTS_CTS_Rate, 0x30000);
    Adapter->CardSetting.ERP[0] = 0x2A;
	Adapter->CardSetting.ERP[1] = 1;
	Adapter->CardSetting.ERP[2] = 0;
    Adapter->SetProtectionModeTimer = 3;
    Adapter->PHYTestIndex = 5;
	Adapter->PHYTestRssiBound = 0x3a;
	Adapter->MinFrameCountDuring3Sec=2; // Added at 4811A
	Adapter->PHYTestTimer = 30;
	Adapter->TrafficBound = 200;
	Adapter->TrafficTimer = 10;
	Adapter->PHYFreCountThr = 0; // disable
	Adapter->PHYFreTimer = 5;  // 50 ms
    Adapter->PHYVarThr = 5;
	Adapter->PHYLowPower = 3;  // Tx/Rx enable
	Adapter->CR122Flag = 2; // initial value
    Adapter->CR31Flag = 2; // initial value
	Adapter->CR203Flag = 2; // initial value
	Adapter->AdapterMaxRate = 0x0B;  // initail max rate = 54M
    Adapter->Infra_Process_Associate=FALSE;
	NdisZeroMemory(Adapter->WPAMICErrorLog,14);
    Adapter->CardSetting.RTSThreshold = 0xffff;

//chi--2004/08/23	
	Adapter->IBSS_BSSID_update=FALSE;

    // originally for INFRASTRUCTURE mode
	if (Adapter->RF_Mode == AL2230_RF ||
			Adapter->RF_Mode == AL2210_RF)
	{
		ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0xA47c00a);
	}
	else
	{
 		ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x547c00a);
	}
	ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_NormalSlot);
	Adapter->LongSlot = 0;


//chi--2004/08/06

	if (Adapter->LED_Ctrl_Type == 1 || Adapter->ZDConfigTurnOffRF==1)
	{
		 Adapter->ZDConfigTurnOffRF = 1;
		 TurnOff_RF(Adapter);
	}
	if (Adapter->PSMode==1)
	{
		USHORT	usBcnIntval;
		Adapter->PSMSupported = 1;
		Adapter->CardSetting.ATIMWindow = 0x5; // for IBSS

		//if (tmpvalue == Ndis802_11PowerModeMAX_PSP)
		//	Adapter->TestPS = 1;

		//if (tmpvalue == Ndis802_11PowerModeFast_PSP)
			Adapter->TestPS = 0;
		
		usBcnIntval = Adapter->CardSetting.BeaconInterval;
		UpdatePreTBTT(Adapter, usBcnIntval-5);
	}
	return TRUE;
}

//-----------------------------------------------------------------------------
// Procedure:    SetupReceiveQueues
//
// Description: Setup the Receive Frame Area (RFA) at INIT time.  This
//              includes setting up each RFD in the RFA, and linking all of the
//              RFDs together.
//              Also set up our buffers for NDIS 5 and multiple receive indications
//              through a packet array.
//
// Arguments:
//    Adapter - ptr to Adapter object instance
//
// Returns:    (none)
//-----------------------------------------------------------------------------

VOID
SetupReceiveQueues(
                   IN PZD1205_ADAPTER Adapter

                   )
{
    // RFD local variables
    ZD1205SwRfd *SwRfdPtr, *SwRfdNext;  // cached RFD list logical pointers
    RFD_STRUC *HwRfdPtr, *HwRfdNext;  // uncached RFD list logical pointers
	ZD1205_DEFRAG_ENTITY *DefragEntityPtr, *DefragEntityNext;
    ULONG HwRfdPhys;  // uncached RFD list physical pointer
    UINT    RfdCount;
    NDIS_STATUS AllocationStatus;

    DEBUGFUNC(_T("SetupReceiveQueues"));

    INITSTR((_T("\n")));

    /*************************************************/
    /* initialize the logical and physical RFD lists */
    /*************************************************/

    // init the lists
    QueueInitList(&Adapter->RfdList);
	QueueInitList(&Adapter->FreeDefragEntityList);
	QueueInitList(&Adapter->DefragList);

    SwRfdNext = (ZD1205SwRfd *)Adapter->RecvCached;     // cached RFDs logical
    HwRfdNext = (RFD_STRUC *)Adapter->RecvUnCached;   // uncached RFDs logical
	DefragEntityNext = (ZD1205_DEFRAG_ENTITY *)(Adapter->RecvCached +
							(Adapter->NumRfd * sizeof(ZD1205SwRfd)));

    // uncached RFD physical address
#ifndef HOST_IF_USB
    HwRfdPhys = NdisGetPhysicalAddressLow(Adapter->RecvUnCachedPhys);
#else
    HwRfdPhys = (ULONG)Adapter->RecvUnCached;
#endif

    INITSTR((_T("sizeof(SwRfd)=%x\n"), sizeof(ZD1205SwRfd)));
    INITSTR((_T("sizeof(HwRfd)=%x\n"), sizeof(RFD_STRUC)));
    INITSTR((_T("Adapter->NumRfd=%d\n"), Adapter->NumRfd));

    // Set up a pool of data for us to build our packet array out of
    // for indicating groups of packets to NDIS
    // this could be quite the memory hog, but makes management
    // of the pointers associated with Asynchronous memory allocation
    // easier
	// Mark2o09++	Allocate enough Packets & Buffers in advance, so that we can avoid 
	//				handling the problem about allocation failure.
    NdisAllocatePacketPool(&AllocationStatus,
        &Adapter->ReceivePacketPool,
        Adapter->NumRfd+(Adapter->NumTcb+1),
        NUM_BYTES_PROTOCOL_RESERVED_SECTION);

    ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);

    // Set up our pool of buffer descriptors...
    // we will at most have 1 per packet, so just allocate as
    // many buffers as we have packets.
    NdisAllocateBufferPool(&AllocationStatus,
        &Adapter->ReceiveBufferPool,
        Adapter->NumRfd+(Adapter->NumTcb+1));

    // AllocateBufferPool suppossedly always returns success...
    ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	// Mark2o09--

    // Setup each RFD
    for (RfdCount = 0; RfdCount < Adapter->NumRfd; RfdCount++)
    {
        // point to the next RFD (after the first time through)
        SwRfdPtr = SwRfdNext;
        HwRfdPtr = HwRfdNext;
		DefragEntityPtr = DefragEntityNext;

        // store the physical/logical address in the Software RFD Structure
        SwRfdPtr->Rfd = HwRfdPtr;
        SwRfdPtr->RfdPhys = HwRfdPhys;


        // point to the next RFD (add sizeof RFD struc)
        // The compiler takes care of translating +1 into a +sizeof(RFD_STRUC)
        SwRfdNext = SwRfdPtr + 1;
        HwRfdNext = HwRfdPtr + 1;
		DefragEntityNext = DefragEntityPtr + 1;
        HwRfdPhys += sizeof(RFD_STRUC);

        // initialize the physical RFD contents
        HwRfdPtr->CbStatus = 0xffffffff;
        HwRfdPtr->ActualCount= 0;
		HwRfdPtr->MaxSize = MAX_WLAN_SIZE;
        if (RfdCount < (Adapter->NumRfd - 1))
        {
            // if not the last RFD...
            HwRfdPtr->CbCommand =0;
            HwRfdPtr->NextCbPhyAddrLowPart = HwRfdPhys;
			HwRfdPtr->NextCbPhyAddrHighPart = 0;
        }
        else
        {
            // if this is the last RFD...
            HwRfdPtr->CbCommand = RFD_EL_BIT;
            HwRfdPtr->NextCbPhyAddrLowPart = DRIVER_NULL;
			HwRfdPtr->NextCbPhyAddrHighPart = 0;
        }


        // Add this RFD to the free list
        QueuePutTail(&Adapter->RfdList, &SwRfdPtr->Link);
		QueuePutTail(&Adapter->FreeDefragEntityList, &DefragEntityPtr->Link);
		QueueInitList(&DefragEntityPtr->ElementList);
    }
}

//------------------------------------------------------------------------------
// Table definition
//
//------------------------------------------------------------------------------
ULONG CH3683Tbl[ ] = {		//channel value for Intersil chip
	0x0FE5E,	//Null
	0x0FE5E,	//Ch1
	0x0FEAE,	//Ch2
	0x0FEFE,	//Ch3
	0x1004E,    //Ch 4 
	0x1009E,    //Ch 5 
	0x100EE,    //Ch 6 
	0x1023E,    //Ch 7 
	0x1028E,    //Ch 8 
	0x102DE,    //Ch 9 
	0x1042E,    //Ch 10
	0x1047E,    //Ch 11
	0x104CE,    //Ch 12
	0x1061E,    //Ch 13
	0x106DE	    //Ch 14
};

ULONG CH1021Tbl[ ] = {		//channel value for Philips chip
	0x080C4,    //Null  
	0x080C4,    //Ch 1  
	0x08114,    //Ch 2  
	0x08164,    //Ch 3  
	0x081B4,    //Ch 4  
	0x08204,    //Ch 5  
	0x08254,    //Ch 6  
	0x082A4,    //Ch 7  
	0x082F4,    //Ch 8  
	0x08344,    //Ch 9  
	0x08394,    //Ch 10 
	0x083E4,    //Ch 11 
	0x08434,    //Ch 12 
	0x08484,    //Ch 13 
	0x08544     //Ch 14 
};

ULONG SA2400Tbl[ ] = {		//channel value for Philips zero-IF chip
	0x00096C80,   //Null 
	0x00096C80,   //Ch 1 
	0x08097080,   //Ch 2 
	0x10097480,   //Ch 3 
	0x18097880,   //Ch 4 
	0x00098080,   //Ch 5 
	0x08098480,   //Ch 6 
	0x10098880,   //Ch 7 
	0x18098C80,   //Ch 8 
	0x00099480,   //Ch 9 
	0x08099880,   //Ch 10
	0x10099C80,   //Ch 11
	0x1809A080,   //Ch 12
	0x0009A880,   //Ch 13
	0x0009B480    //Ch 14
};

ULONG Si4126Tbl[ ] = {
	0x007F60,   //Null  
	0x007F60,   //Ch 1  
	0x007FB0,   //Ch 2  
	0x008000,   //Ch 3  
	0x008050,   //Ch 4  
	0x0080A0,   //Ch 5  
	0x0080F0,   //Ch 6  
	0x008140,   //Ch 7  
	0x008190,   //Ch 8  
	0x0081E0,   //Ch 9  
	0x008230,   //Ch 10 
	0x008280,   //Ch 11 
	0x0082D0,   //Ch 12 
	0x008320,   //Ch 13 
	0x0083E0    //Ch 14 
};

ULONG RFMD2958t[] = {
    0x1422BD,   //Null 
    0x185D17,   //Null 
    0x181979,   //Ch 1
    0x1e6666,   //Ch 1
    0x181989,   //Ch 2
    0x1e6666,   //Ch 2
    0x181999,   //Ch 3
    0x1e6666,   //Ch 3
    0x1819a9,   //Ch 4
    0x1e6666,   //Ch 4
    0x1819b9,   //Ch 5
    0x1e6666,   //Ch 5
    0x1819c9,   //Ch 6
    0x1e6666,   //Ch 6
    0x1819d9,   //Ch 7
    0x1e6666,   //Ch 7
    0x1819e9,   //Ch 8
    0x1e6666,   //Ch 8
    0x1819f9,   //Ch 9
    0x1e6666,   //Ch 9
    0x181a09,   //Ch 10
    0x1e6666,   //Ch 10
    0x181a19,   //Ch 11
    0x1e6666,   //Ch 11
    0x181a29,   //Ch 12
    0x1e6666,   //Ch 12
    0x181a39,   //Ch 13
    0x1e6666,   //Ch 13
    0x181a60,   //Ch 14
    0x1c0000    //Ch 14
};


ULONG PVITbl[] = {
    0x0096C3,   //Null 
    0x0096C3,   //Ch 1
    0x009713,   //Ch 2
    0x009763,   //Ch 3
    0x0097B3,   //Ch 4
    0x009803,   //Ch 5
    0x009853,   //Ch 6
    0x0098A3,   //Ch 7
    0x0098F3,   //Ch 8
    0x009943,   //Ch 9
    0x009993,   //Ch 10
    0x0099E3,   //Ch 11
    0x009A33,   //Ch 12
    0x009AB3,   //Ch 13
    0x009B43    //Ch 14
};

ULONG max2820Tbl[ ] = {
    0x300C,		//Null 
    0x300C,		//Ch 1
    0x3011,		//Ch 2
    0x3016,		//Ch 3
    0x301B,		//Ch 4
    0x3020,		//Ch 5
    0x3025,		//Ch 6
    0x302A,		//Ch 7
    0x302F,		//Ch 8
    0x3034,		//Ch 9
    0x3039,		//Ch 10
    0x303E,		//Ch 11
    0x3043,		//Ch 12
    0x3048,		//Ch 13
    0x3054		//Ch 14
};

ULONG	M2827BF[] = {
    0x0ccd4,    //;Null 
    0x0ccd4,    //;Ch 1
    0x22224,    //;Ch 2
    0x37774,    //;Ch 3
    0x0ccd4,    //;Ch 4
    0x22224,    //;Ch 5
    0x37774,    //;Ch 6
    0x0ccd4,    //;Ch 7
    0x22224,    //;Ch 8
    0x37774,    //;Ch 9
    0x0ccd4,    //;Ch 10
    0x22224,    //;Ch 11
    0x37774,    //;Ch 12
    0x0ccd4,    //;Ch 13
    0x199a4    //;Ch 14
};

ULONG	M2827BN[] = {
    0x30a03,    //;Null 
    0x30a03,    //;Ch 1
    0x00a13,    //;Ch 2
    0x10a13,    //;Ch 3
    0x30a13,    //;Ch 4
    0x00a23,    //;Ch 5
    0x10a23,    //;Ch 6
    0x30a23,    //;Ch 7
    0x00a33,    //;Ch 8
    0x10a33,    //;Ch 9
    0x30a33,    //;Ch 10
    0x00a43,    //;Ch 11
    0x10a43,    //;Ch 12
    0x30a43,    //;Ch 13
    0x20a53    //;Ch 14
};

ULONG	M2827BF2[] = {
    0x33334,    //;Null 
    0x33334,    //;Ch 1
    0x08884,    //;Ch 2
    0x1ddd4,    //;Ch 3
    0x33334,    //;Ch 4
    0x08884,    //;Ch 5
    0x1ddd4,    //;Ch 6
    0x33334,    //;Ch 7
    0x08884,    //;Ch 8
    0x1ddd4,    //;Ch 9
    0x33334,    //;Ch 10
    0x08884,    //;Ch 11
    0x1ddd4,    //;Ch 12
    0x33334,    //;Ch 13
    0x26664    //;Ch 14
};

ULONG	M2827BN2[] = {
    0x10a03,    //;Null 
    0x10a03,    //;Ch 1
    0x20a13,    //;Ch 2
    0x30a13,    //;Ch 3
    0x10a13,    //;Ch 4
    0x20a23,    //;Ch 5
    0x30a23,    //;Ch 6
    0x10a23,    //;Ch 7
    0x20a33,    //;Ch 8
    0x30a33,    //;Ch 9
    0x10a33,    //;Ch 10
    0x20a43,    //;Ch 11
    0x30a43,    //;Ch 12
    0x10a43,    //;Ch 13
    0x20a53    //;Ch 14
};

ULONG RT2420Tbl[ ] = {
    0xC3FB2,    //Null 
    0xC3FB2,	//Ch 1
    0xC3FDA,	//Ch 2
    0xC4002,	//Ch 3
    0xC402A,	//Ch 4
    0xC4052,	//Ch 5
    0xC407A,	//Ch 6
    0xC40A2,	//Ch 7
    0xC40CA,	//Ch 8
    0xC40F2,	//Ch 9
    0xC411A,	//Ch 10
    0xC4142,	//Ch 11
    0xC416A,	//Ch 12
    0xC4192,	//Ch 13
    0xC41F2		//Ch 14
};

ULONG GRF5101T[] = {
    0x1A0000,   //Null 
    0x1A0000,   //Ch 1
    0x1A8000,   //Ch 2
    0x1A4000,   //Ch 3
    0x1AC000,   //Ch 4
    0x1A2000,   //Ch 5
    0x1AA000,   //Ch 6
    0x1A6000,   //Ch 7
    0x1AE000,   //Ch 8
    0x1A1000,   //Ch 9
    0x1A9000,   //Ch 10
    0x1A5000,   //Ch 11
    0x1AD000,   //Ch 12
    0x1A3000,   //Ch 13
    0x1AB000    //Ch 14
};

ULONG GRF5103T[] = {
    0x1C0008,   //Null 
    0x1C0008,   //Ch 1
    0x1C8008,   //Ch 2
    0x1C4008,   //Ch 3
    0x1CC008,   //Ch 4
    0x1C2008,   //Ch 5
    0x1CA008,   //Ch 6
    0x1C6008,   //Ch 7
    0x1CE008,   //Ch 8
    0x1C1008,   //Ch 9
    0x1C9008,   //Ch 10
    0x1C5008,   //Ch 11
    0x1CD008,   //Ch 12
    0x1C3008,   //Ch 13
    0x1CB008    //Ch 14
};

ULONG AL2210TB[] = {
    0x2396c0,   //;Null 
    0x0196c0,   //;Ch 1
    0x019710,   //;Ch 2
    0x019760,   //;Ch 3
    0x0197b0,   //;Ch 4
    0x019800,   //;Ch 5
    0x019850,   //;Ch 6
    0x0198a0,   //;Ch 7
    0x0198f0,   //;Ch 8
    0x019940,   //;Ch 9
    0x019990,   //;Ch 10
    0x0199e0,   //;Ch 11
    0x019a30,   //;Ch 12
    0x019a80,   //;Ch 13
    0x019b40    //;Ch 14
};

ULONG AL2230TB[] = {
    0x03f790, 0x033331, 0x00000d,   //;Null 
    0x03f790, 0x033331, 0x00000d,   //;Ch 1
    0x03f790, 0x0b3331, 0x00000d,  //;Ch 2
    0x03e790, 0x033331, 0x00000d,  //;Ch 3
    0x03e790, 0x0b3331, 0x00000d,  //;Ch 4
    0x03f7a0, 0x033331, 0x00000d,  //;Ch 5
    0x03f7a0, 0x0b3331, 0x00000d,  //;Ch 6
    0x03e7a0, 0x033331, 0x00000d,  //;Ch 7
    0x03e7a0, 0x0b3331, 0x00000d,  //;Ch 8
    0x03f7b0, 0x033331, 0x00000d,  //;Ch 9
    0x03f7b0, 0x0b3331, 0x00000d,  //;Ch 10
    0x03E7b0, 0x033331, 0x00000d,  //;Ch 11
    0x03e7b0, 0x0b3331, 0x00000d,  //;Ch 12
    0x03f7c0, 0x033331, 0x00000d,  //;Ch 13
    0x03e7c0, 0x066661, 0x00000d   //;Ch 14
};

ULONG T5024N[] = {
    0x025b1,    //;Null 
    0x025b1,    //;Ch 1
    0x025c5,    //;Ch 2
    0x025d9,    //;Ch 3
    0x025ed,    //;Ch 4
    0x02601,    //;Ch 5
    0x02615,    //;Ch 6
    0x02629,    //;Ch 7
    0x0263d,    //;Ch 8
    0x02651,    //;Ch 9
    0x02665,    //;Ch 10
    0x02679,    //;Ch 11
    0x0268d,    //;Ch 12
    0x026a1,    //;Ch 13
    0x026d1     //;Ch 14
};
//------------------------------------------------------------------------------
// Procedure:	 PHY_Reg_Preset
//
// Description:  Preset Phy registers initially
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//
// Returns:		(none)
//
// Note:
//		1. CR113 not present in ZD1205 (Dr.Wang confirmed)
//		2. Should provide RF_Mode register
//		4. Do not write PLCP field in driver
//		5. Update according to the latest registers definition
//-------------------------------------------------------------------------------
#ifndef PHY_LOOKBACK_TEST	
VOID
PHY_Reg_Preset(
			   IN PZD1205_ADAPTER Adapter
			   )
{
    ULONG tmpvalue;

	ZDDbgPrint((_T(" ************ PHY setting :    PHY_Reg_Preset subroutine\n")));
	// Get Phy-Config permission
	AcquireCtrOfPhyReg(Adapter);

	ZD1205_WRITE_REGISTER(Adapter,CR0, 0x0A);
	ZD1205_WRITE_REGISTER(Adapter,CR1, 0x06);
	ZD1205_WRITE_REGISTER(Adapter,CR2, 0x26);
	ZD1205_WRITE_REGISTER(Adapter,CR3, 0x38);
	ZD1205_WRITE_REGISTER(Adapter,CR4, 0x80);

	ZD1205_WRITE_REGISTER(Adapter,CR9, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR10, 0x81);
    //``JWEI 2003/12/26
    #if fTX_PWR_CTRL && fTX_GAIN_OFDM
    //    ZD1205_READ_REGISTER(Adapter, CR11, &tmpvalue);
    //    tmpvalue |= BIT_6;
    //    ZD1205_WRITE_REGISTER(Adapter, CR11, tmpvalue);
        ZD1205_WRITE_REGISTER(Adapter, CR11, BIT_6);
    #else
        ZD1205_WRITE_REGISTER(Adapter,CR11, 0x00);
    #endif
	ZD1205_WRITE_REGISTER(Adapter,CR12, 0xF0);
	ZD1205_WRITE_REGISTER(Adapter,CR13, 0x8c);
	ZD1205_WRITE_REGISTER(Adapter,CR14, 0x80);
	ZD1205_WRITE_REGISTER(Adapter,CR15, 0x3D);
	ZD1205_WRITE_REGISTER(Adapter,CR16, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR17, 0x1E);

	ZD1205_WRITE_REGISTER(Adapter,CR18, 0x0A);

	ZD1205_WRITE_REGISTER(Adapter,CR19, 0x48);
	ZD1205_WRITE_REGISTER(Adapter,CR20, 0x0e);
	ZD1205_WRITE_REGISTER(Adapter,CR21, 0x0C);
	ZD1205_WRITE_REGISTER(Adapter,CR22, 0x23);
	ZD1205_WRITE_REGISTER(Adapter,CR23, 0x90);
	ZD1205_WRITE_REGISTER(Adapter,CR24, 0x14);
	ZD1205_WRITE_REGISTER(Adapter,CR25, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR27, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR28, 0x7F);
	ZD1205_WRITE_REGISTER(Adapter,CR29, 0x80);
#ifndef ASIC
	// For FWT
	ZD1205_WRITE_REGISTER(Adapter,CR30, 0x4b);
#else
	// For Jointly decoder
	ZD1205_WRITE_REGISTER(Adapter,CR30, 0x49);
#endif
	ZD1205_WRITE_REGISTER(Adapter,CR31, 0x60);
	ZD1205_WRITE_REGISTER(Adapter,CR32, 0x43);
	ZD1205_WRITE_REGISTER(Adapter,CR33, 0x08);
	ZD1205_WRITE_REGISTER(Adapter,CR34, 0x06);
	ZD1205_WRITE_REGISTER(Adapter,CR35, 0x0a);
	ZD1205_WRITE_REGISTER(Adapter,CR36, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR37, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR38, 0x38);
	ZD1205_WRITE_REGISTER(Adapter,CR39, 0x0C);
	ZD1205_WRITE_REGISTER(Adapter,CR40, 0x84);
	ZD1205_WRITE_REGISTER(Adapter,CR41, 0x2A);
	ZD1205_WRITE_REGISTER(Adapter,CR42, 0x80);
	ZD1205_WRITE_REGISTER(Adapter,CR43, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x12);

	ZD1205_WRITE_REGISTER(Adapter,CR46, 0xff);
	//ZD1205_WRITE_REGISTER(Adapter,CR47, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR47, 0x08);
	ZD1205_WRITE_REGISTER(Adapter,CR48, 0x26);
	ZD1205_WRITE_REGISTER(Adapter,CR49, 0x5B);

	ZD1205_WRITE_REGISTER(Adapter,CR64, 0xD0);
	ZD1205_WRITE_REGISTER(Adapter,CR65, 0x04);
	ZD1205_WRITE_REGISTER(Adapter,CR66, 0x58);
	ZD1205_WRITE_REGISTER(Adapter,CR67, 0xc9);
	ZD1205_WRITE_REGISTER(Adapter,CR68, 0x88);
	ZD1205_WRITE_REGISTER(Adapter,CR69, 0x41);
	ZD1205_WRITE_REGISTER(Adapter,CR70, 0x23);
	ZD1205_WRITE_REGISTER(Adapter,CR71, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR72, 0xFF);
	ZD1205_WRITE_REGISTER(Adapter,CR73, 0x32);
	ZD1205_WRITE_REGISTER(Adapter,CR74, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR75, 0x65);
	ZD1205_WRITE_REGISTER(Adapter,CR76, 0x41);
	ZD1205_WRITE_REGISTER(Adapter,CR77, 0x1b);
	ZD1205_WRITE_REGISTER(Adapter,CR78, 0x30);
	//++ Modify for CRC32 error, Bad receival on Ack
	// For details, refer to Programming Guid.
	//ZD1205_WRITE_REGISTER(Adapter,CR79, 0x40);
	//ZD1205_WRITE_REGISTER(Adapter,CR80, 0x3c);
	//ZD1205_WRITE_REGISTER(Adapter,CR81, 0x3c);
	ZD1205_WRITE_REGISTER(Adapter,CR79, 0xf0);
	ZD1205_WRITE_REGISTER(Adapter,CR80, 0x64);
	ZD1205_WRITE_REGISTER(Adapter,CR81, 0x64);
	//--
	ZD1205_WRITE_REGISTER(Adapter,CR82, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR83, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR84, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR85, 0x02);
	ZD1205_WRITE_REGISTER(Adapter,CR86, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR87, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR88, 0xFF);
	ZD1205_WRITE_REGISTER(Adapter,CR89, 0xFC);
	ZD1205_WRITE_REGISTER(Adapter,CR90, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR91, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR92, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR93, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR94, 0x01);
#ifdef HOST_IF_USB	
	ZD1205_WRITE_REGISTER(Adapter,CR95, 0x00);
#else
	ZD1205_WRITE_REGISTER(Adapter,CR95, 0x20);
#endif
	ZD1205_WRITE_REGISTER(Adapter,CR96, 0x50);
	ZD1205_WRITE_REGISTER(Adapter,CR97, 0x37);
#ifdef HOST_IF_USB	
	ZD1205_WRITE_REGISTER(Adapter,CR98, 0x35);
#else
	ZD1205_WRITE_REGISTER(Adapter,CR98, 0x0D);
#endif
	ZD1205_WRITE_REGISTER(Adapter,CR99, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR100, 0x01);
	ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR102, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR103, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR104, 0x18);
	ZD1205_WRITE_REGISTER(Adapter,CR105, 0x12);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x04);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR108, 0x0a);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0xaa);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0xaa);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x25);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x25);
	ZD1205_WRITE_REGISTER(Adapter,CR113, 0x00);

	ZD1205_WRITE_REGISTER(Adapter,CR119, 0x1e);
#ifndef HOST_IF_USB	
	ZD1205_WRITE_REGISTER(Adapter,CR123, 0x27);
#endif
	ZD1205_WRITE_REGISTER(Adapter,CR125, 0x90);
	ZD1205_WRITE_REGISTER(Adapter,CR126, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR127, 0x00);
#ifndef HOST_IF_USB	
	ZD1205_WRITE_REGISTER(Adapter,CR159, 0x70);
#endif

#if defined(ECCK_60_5)
	ZD1205_WRITE_REGISTER(Adapter,CR11, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR14, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x33);
	ZD1205_WRITE_REGISTER(Adapter,CR64, 0x6e);
	ZD1205_WRITE_REGISTER(Adapter,CR75, 0x33);
	ZD1205_WRITE_REGISTER(Adapter,CR76, 0x57);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR128, 0x94);
	ZD1205_WRITE_REGISTER(Adapter,CR129, 0x44);
	ZD1205_WRITE_REGISTER(Adapter,CR130, 0x39);
	ZD1205_WRITE_REGISTER(Adapter,CR131, 0x37);
	ZD1205_WRITE_REGISTER(Adapter,CR132, 0x3c);
	ZD1205_WRITE_REGISTER(Adapter,CR133, 0x48);
	ZD1205_WRITE_REGISTER(Adapter,CR134, 0x6e);
	ZD1205_WRITE_REGISTER(Adapter,CR135, 0x88);
	ZD1205_WRITE_REGISTER(Adapter,CR136, 0x6e);
	ZD1205_WRITE_REGISTER(Adapter,CR137, 0xdc);
	ZD1205_WRITE_REGISTER(Adapter,CR138, 0x03);
	ZD1205_WRITE_REGISTER(Adapter,CR139, 0x57);
	ZD1205_WRITE_REGISTER(Adapter,CR140, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR141, 0x9b);
	ZD1205_WRITE_REGISTER(Adapter,CR142, 0x45);
	ZD1205_WRITE_REGISTER(Adapter,CR143, 0x45);
	ZD1205_WRITE_REGISTER(Adapter,CR144, 0x32);
	ZD1205_WRITE_REGISTER(Adapter,CR145, 0x3c);
	ZD1205_WRITE_REGISTER(Adapter,CR146, 0xbe);
	ZD1205_WRITE_REGISTER(Adapter,CR147, 0xe1);
	ZD1205_WRITE_REGISTER(Adapter,CR148, 0x56);
	ZD1205_WRITE_REGISTER(Adapter,CR149, 0x9b);
	ZD1205_WRITE_REGISTER(Adapter,CR150, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR151, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR152, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR153, 0x28);
	ZD1205_WRITE_REGISTER(Adapter,CR154, 0x22);
	ZD1205_WRITE_REGISTER(Adapter,CR155, 0x1c);
	ZD1205_WRITE_REGISTER(Adapter,CR156, 0x55);
	ZD1205_WRITE_REGISTER(Adapter,CR157, 0x65);
	ZD1205_WRITE_REGISTER(Adapter,CR158, 0x78);
	ZD1205_WRITE_REGISTER(Adapter,CR159, 0x93);
	ZD1205_WRITE_REGISTER(Adapter,CR163, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR164, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR165, 0x01);
	ZD1205_WRITE_REGISTER(Adapter,CR166, 0x9b);
	ZD1205_WRITE_REGISTER(Adapter,CR167, 0x9b);
	ZD1205_WRITE_REGISTER(Adapter,CR168, 0x9b);
	ZD1205_WRITE_REGISTER(Adapter,CR169, 0x9b);
	ZD1205_WRITE_REGISTER(Adapter,CR170, 0x45);
	ZD1205_WRITE_REGISTER(Adapter,CR171, 0x45);
	ZD1205_WRITE_REGISTER(Adapter,CR172, 0x45);
	ZD1205_WRITE_REGISTER(Adapter,CR173, 0x77);
	ZD1205_WRITE_REGISTER(Adapter,CR174, 0x70);
	ZD1205_WRITE_REGISTER(Adapter,CR175, 0x6e);
	ZD1205_WRITE_REGISTER(Adapter,CR176, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR177, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR178, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR179, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR180, 0x2b);
	// Note: CR204 must lead the CR203
	ZD1205_WRITE_REGISTER(Adapter,CR204, 0xfd);
	ZD1205_WRITE_REGISTER(Adapter,CR203, 0x30);
#endif

#if (defined(GCCK) && defined(OFDM))
	ZD1205_WRITE_REGISTER(Adapter,CR21, 0x0e);
#ifdef HOST_IF_USB
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x33);
#else
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x32);
#endif
	ZD1205_WRITE_REGISTER(Adapter,CR47, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR83, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR84, 0x04);
	ZD1205_WRITE_REGISTER(Adapter,CR85, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR86, 0x0C);
	ZD1205_WRITE_REGISTER(Adapter,CR87, 0x12);
	ZD1205_WRITE_REGISTER(Adapter,CR88, 0x0C);
	ZD1205_WRITE_REGISTER(Adapter,CR89, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR90, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR91, 0x04);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR115, 0x26);
	ZD1205_WRITE_REGISTER(Adapter,CR116, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR117, 0xfc);
	ZD1205_WRITE_REGISTER(Adapter,CR118, 0xfa);
	ZD1205_WRITE_REGISTER(Adapter,CR128, 0x14);
	ZD1205_WRITE_REGISTER(Adapter,CR129, 0x12);
	ZD1205_WRITE_REGISTER(Adapter,CR130, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR131, 0x0C);
	ZD1205_WRITE_REGISTER(Adapter,CR136, 0xdf);
	ZD1205_WRITE_REGISTER(Adapter,CR137, 0xA0);
	ZD1205_WRITE_REGISTER(Adapter,CR138, 0xa8);
	ZD1205_WRITE_REGISTER(Adapter,CR139, 0xb4);
#ifdef HOST_IF_USB
	ZD1205_WRITE_REGISTER(Adapter,CR140, 0x99);
#else
	ZD1205_WRITE_REGISTER(Adapter,CR140, 0x98);
#endif
	ZD1205_WRITE_REGISTER(Adapter,CR141, 0x82);
#ifdef HOST_IF_USB
	ZD1205_WRITE_REGISTER(Adapter,CR142, 0x54);
#else
	ZD1205_WRITE_REGISTER(Adapter,CR142, 0x53);
#endif
	ZD1205_WRITE_REGISTER(Adapter,CR143, 0x1c);
	ZD1205_WRITE_REGISTER(Adapter,CR144, 0x6c);
	ZD1205_WRITE_REGISTER(Adapter,CR147, 0x07);
	ZD1205_WRITE_REGISTER(Adapter,CR148, 0x4c);
	ZD1205_WRITE_REGISTER(Adapter,CR149, 0x50);
	ZD1205_WRITE_REGISTER(Adapter,CR150, 0x0e);
	ZD1205_WRITE_REGISTER(Adapter,CR151, 0x18);
	ZD1205_WRITE_REGISTER(Adapter,CR160, 0xfe);
	ZD1205_WRITE_REGISTER(Adapter,CR161, 0xee);
	ZD1205_WRITE_REGISTER(Adapter,CR162, 0xaa);
	ZD1205_WRITE_REGISTER(Adapter,CR163, 0xfa);
	ZD1205_WRITE_REGISTER(Adapter,CR164, 0xfa);
	ZD1205_WRITE_REGISTER(Adapter,CR165, 0xea);
	ZD1205_WRITE_REGISTER(Adapter,CR166, 0xbe);
	ZD1205_WRITE_REGISTER(Adapter,CR167, 0xbe);
	ZD1205_WRITE_REGISTER(Adapter,CR168, 0x6a);
	ZD1205_WRITE_REGISTER(Adapter,CR169, 0xba);
	ZD1205_WRITE_REGISTER(Adapter,CR170, 0xba);
	ZD1205_WRITE_REGISTER(Adapter,CR171, 0xba);
	// Note: CR204 must lead the CR203
	ZD1205_WRITE_REGISTER(Adapter,CR204, 0x7d);
	ZD1205_WRITE_REGISTER(Adapter,CR203, 0x30);

#ifndef HOST_IF_USB
	ZD1205_WRITE_REGISTER(Adapter, CR240, 0x80);
#endif
#endif

	// Release Phy-Config permission
	ReleaseCtrOfPhyReg(Adapter);

	return;
}
#endif

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_Intersil_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//		1. TR_SW, TR_SW_BAR not present in ZD1205 (Dr.Wang Confirmed)
//-------------------------------------------------------------------------------
	
VOID
HW_Set_Intersil_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{	
	ULONG		tmpvalue;

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 2);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue &= ~BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif

#if	defined(ECCK_60_5)
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_WRITE_REGISTER(Adapter,CR46, 0x00);
	ReleaseCtrOfPhyReg(Adapter);
#endif
	//Set_RF_Synthesizer(Adapter, CH3683Tbl[ChannelNo]);
	if (! InitChOnly){
		//Set_RF_Synthesizer(Adapter, 0x75);
		//Set_RF_Synthesizer(Adapter, 0x2C0);

		Set_IF_Synthesizer(Adapter, 0x35);
		Set_IF_Synthesizer(Adapter, 0x67);
		Set_IF_Synthesizer(Adapter, 0x160);
		Set_IF_Synthesizer(Adapter, 0x5C66);
#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
		ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 3);
#else
		AcquireCtrOfPhyReg(Adapter);
		ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
		tmpvalue |= BIT_4;
		ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
		ReleaseCtrOfPhyReg(Adapter);
#endif
		ZD1205_WRITE_REGISTER(Adapter,RADIO_PE, 0);
		//ZD1205_WRITE_REGISTER(Adapter,TR_SW, 0);
		//ZD1205_WRITE_REGISTER(Adapter,TR_SW_BAR, 1);
	}

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 3);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue |= BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif

	return;
}

//------------------------------------------------------------------------------
// Procedure:	 Delay_For_ZeroIF
//
// Description:  
//
// Arguments:
//				(none)
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
VOID
Delay_For_ZeroIF(
					IN PZD1205_ADAPTER Adapter
				)
{

	//for (i=0;i<Adapter->SA2400DelayTime;i++){
	//}
    
	NdisStallExecution(1);

	return;
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_ZeroIF_Chip
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//		1. RSSI_MGC not present in ZD1205 (To be defined)
//		2. I2C_WRITE(600044->244) not present in ZD1205
//-------------------------------------------------------------------------------
	
VOID
HW_Set_ZeroIF_Chip(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	// Get Phy-Config permission
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_WRITE_REGISTER(Adapter,CR2, 0x26);
	ZD1205_WRITE_REGISTER(Adapter,CR10, 0x99);
	ZD1205_WRITE_REGISTER(Adapter,CR15, 0x3e);
	ZD1205_WRITE_REGISTER(Adapter,CR17, 0x2d);
	ZD1205_WRITE_REGISTER(Adapter,CR20, 0x8c);
	ZD1205_WRITE_REGISTER(Adapter,CR23, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR24, 0x12);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x11);
	ZD1205_WRITE_REGISTER(Adapter,CR27, 0x04);
	ZD1205_WRITE_REGISTER(Adapter,CR28, 0x3f);
	ZD1205_WRITE_REGISTER(Adapter,CR29, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR34, 0x2e);
	ZD1205_WRITE_REGISTER(Adapter,CR35, 0x4c);
	ZD1205_WRITE_REGISTER(Adapter,CR41, 0x28);
	ZD1205_WRITE_REGISTER(Adapter,CR64, 0xc0);
	ZD1205_WRITE_REGISTER(Adapter,CR113, 0xf0);
	ZD1205_WRITE_REGISTER(Adapter,CR119, 0x1e);
	ZD1205_WRITE_REGISTER(Adapter,CR125, 0x40);
	// Release Phy-Config permission
	ReleaseCtrOfPhyReg(Adapter);

	//ZD1205_WRITE_REGISTER(Adapter,RSSI_MGC, 0);
	ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, SA2400Tbl[ChannelNo]);
		Delay_For_ZeroIF(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0xbb5081);
		Delay_For_ZeroIF(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0x8082);
		Delay_For_ZeroIF(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0x83);
		Delay_For_ZeroIF(Adapter);
	if ( ! InitChOnly ){
		ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0x3c9dfb85);
		Delay_For_ZeroIF(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0x13ff86);
		Delay_For_ZeroIF(Adapter);
		//ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0xf87);
		//ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0xc087);
		ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0xd587);
		Delay_For_ZeroIF(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0x1932084);
		Delay_For_ZeroIF(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0x1932684);
		//++ Requested by Dr.Wang
		//Delay_For_ZeroIF(Adapter);
		//ZD1205_WRITE_REGISTER(Adapter,I2C_WRITE, 0xd087);
		//--
	}
	Delay_For_ZeroIF(Adapter);
	//ZD1205_WRITE_REGISTER(Adapter,RSSI_MGC, 1);

	return;
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_RFMD_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
//3-step LNA for RF2959
/*
VOID
HW_Set_RFMD_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	ULONG		tmpvalue;
	// Get Phy-Config permission
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_WRITE_REGISTER(Adapter,CR2, 0x1E);
	ZD1205_WRITE_REGISTER(Adapter,CR9, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR10, 0xB1);
	ZD1205_WRITE_REGISTER(Adapter,CR11, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR15, 0xD0);
	ZD1205_WRITE_REGISTER(Adapter,CR17, 0x60);
	ZD1205_WRITE_REGISTER(Adapter,CR20, 0x0c);
	ZD1205_WRITE_REGISTER(Adapter,CR21, 0x0E);
	ZD1205_WRITE_REGISTER(Adapter,CR23, 0x50);
	ZD1205_WRITE_REGISTER(Adapter,CR24, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR27, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR29, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR31, 0xe0);
	ZD1205_WRITE_REGISTER(Adapter,CR32, 0x43);
	ZD1205_WRITE_REGISTER(Adapter,CR33, 0x28);
	ZD1205_WRITE_REGISTER(Adapter,CR38, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR34, 0x0f);
	ZD1205_WRITE_REGISTER(Adapter,CR35, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR41, 0x23);
	ZD1205_WRITE_REGISTER(Adapter,CR46, 0xFF);
	ZD1205_WRITE_REGISTER(Adapter,CR47, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR82, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR83, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR84, 0x04);
	ZD1205_WRITE_REGISTER(Adapter,CR85, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR86, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR87, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR88, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR89, 0x0c);
	ZD1205_WRITE_REGISTER(Adapter,CR90, 0x18);
	ZD1205_WRITE_REGISTER(Adapter,CR91, 0x18);
	ZD1205_WRITE_REGISTER(Adapter,CR92, 0x0a);
	ZD1205_WRITE_REGISTER(Adapter,CR93, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR94, 0x01);
	ZD1205_WRITE_REGISTER(Adapter,CR95, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR96, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR97, 0x37);
	ZD1205_WRITE_REGISTER(Adapter,CR98, 0x35);
	ZD1205_WRITE_REGISTER(Adapter,CR99, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR100, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR102, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR103, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR104, 0x18);
	ZD1205_WRITE_REGISTER(Adapter,CR105, 0x12);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x26);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR108, 0x0a);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x1F);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR115, 0x26);
	ZD1205_WRITE_REGISTER(Adapter,CR116, 0x28);
	ZD1205_WRITE_REGISTER(Adapter,CR117, 0xFC);
	ZD1205_WRITE_REGISTER(Adapter,CR118, 0xF0);
	ZD1205_WRITE_REGISTER(Adapter,CR119, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR122, 0xfe);
	ZD1205_WRITE_REGISTER(Adapter,CR127, 0x03);
	ZD1205_WRITE_REGISTER(Adapter,CR138, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR148, 0x44);
	ZD1205_WRITE_REGISTER(Adapter,CR169, 0xBB);
	ZD1205_WRITE_REGISTER(Adapter,CR170, 0xBB);
	//ZD1205_WRITE_REGISTER(Adapter,CR38, 0x30);
	// Release Phy-Config permission
	ReleaseCtrOfPhyReg(Adapter);

	Set_IF_Synthesizer(Adapter, 0x000007);  //REG0(CFG1)
	Set_IF_Synthesizer(Adapter, 0x07dd43);  //REG1(IFPLL1)
	Set_IF_Synthesizer(Adapter, 0x080959);  //REG2(IFPLL2)
	Set_IF_Synthesizer(Adapter, 0x0e6666);
	Set_IF_Synthesizer(Adapter, 0x116a57);  //REG4
	Set_IF_Synthesizer(Adapter, 0x17dd43);  //REG5
	Set_IF_Synthesizer(Adapter, 0x1819f9);  //REG6
	Set_IF_Synthesizer(Adapter, 0x1e6666);
	Set_IF_Synthesizer(Adapter, 0x214554);
	Set_IF_Synthesizer(Adapter, 0x25e7fa);
	Set_IF_Synthesizer(Adapter, 0x27fffa);
	//Set_IF_Synthesizer(Adapter, 0x294128);  //Register control TX power
	Set_IF_Synthesizer(Adapter, 0x28252d);    //External control TX power (CR31_CCK, CR51_6-36M, CR52_48M, CR53_54M
	Set_IF_Synthesizer(Adapter, 0x2c0000);
	Set_IF_Synthesizer(Adapter, 0x300000);
	
	Set_IF_Synthesizer(Adapter, 0x340000);  //REG13(0xD)
	Set_IF_Synthesizer(Adapter, 0x381e0f);  //REG14(0xE)
	Set_IF_Synthesizer(Adapter, 0x6c180f);  //REG27(0x11)
	
	Set_IF_Synthesizer(Adapter, RFMD2958t[ChannelNo*2]);
	Set_IF_Synthesizer(Adapter, RFMD2958t[ChannelNo*2+1]);

	return;
}    */ 


//2-step LNA for RF2959
VOID
HW_Set_RFMD_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	DEBUGFUNC(_T("HW_Set_RFMD_Chips"));

	//INITSTR((_T("-------------------->>> start\n")));

	// Get Phy-Config permission
	if ( ! InitChOnly ){
		AcquireCtrOfPhyReg(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,CR2, 0x1E);
		ZD1205_WRITE_REGISTER(Adapter,CR9, 0x20);
		//ZD1205_WRITE_REGISTER(Adapter,CR10, 0xB1);
		ZD1205_WRITE_REGISTER(Adapter,CR10, 0x89);
		ZD1205_WRITE_REGISTER(Adapter,CR11, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR15, 0xD0);
		ZD1205_WRITE_REGISTER(Adapter,CR17, 0x68);
		ZD1205_WRITE_REGISTER(Adapter,CR19, 0x4a);
		ZD1205_WRITE_REGISTER(Adapter,CR20, 0x0c);
		ZD1205_WRITE_REGISTER(Adapter,CR21, 0x0E);
		ZD1205_WRITE_REGISTER(Adapter,CR23, 0x48);

		if (Adapter->bIsNormalSize)
			ZD1205_WRITE_REGISTER(Adapter,CR24, 0x14);//cca threshold
		else
			ZD1205_WRITE_REGISTER(Adapter,CR24, 0x20);//cca threshold

		ZD1205_WRITE_REGISTER(Adapter,CR26, 0x90);
		ZD1205_WRITE_REGISTER(Adapter,CR27, 0x30);
		ZD1205_WRITE_REGISTER(Adapter,CR29, 0x20);
		ZD1205_WRITE_REGISTER(Adapter,CR31, 0xb2);
		//ZD1205_WRITE_REGISTER(Adapter,CR31, 0xaa);
		ZD1205_WRITE_REGISTER(Adapter,CR32, 0x43);
		ZD1205_WRITE_REGISTER(Adapter,CR33, 0x28);
		ZD1205_WRITE_REGISTER(Adapter,CR38, 0x30);
		ZD1205_WRITE_REGISTER(Adapter,CR34, 0x0f);
		ZD1205_WRITE_REGISTER(Adapter,CR35, 0xF0);
		ZD1205_WRITE_REGISTER(Adapter,CR41, 0x2a);
		ZD1205_WRITE_REGISTER(Adapter,CR46, 0x7F);
		ZD1205_WRITE_REGISTER(Adapter,CR47, 0x1c);
		ZD1205_WRITE_REGISTER(Adapter,CR51, 0xc5);
		ZD1205_WRITE_REGISTER(Adapter,CR52, 0xc5);
		ZD1205_WRITE_REGISTER(Adapter,CR53, 0xc5);
		ZD1205_WRITE_REGISTER(Adapter,CR79, 0x58);
		ZD1205_WRITE_REGISTER(Adapter,CR80, 0x30);
		ZD1205_WRITE_REGISTER(Adapter,CR81, 0x30);
		ZD1205_WRITE_REGISTER(Adapter,CR82, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR83, 0x24);
		ZD1205_WRITE_REGISTER(Adapter,CR84, 0x04);
		ZD1205_WRITE_REGISTER(Adapter,CR85, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR86, 0x10);
		if (Adapter->PA_Type == 0)  //Normal PA
			ZD1205_WRITE_REGISTER(Adapter,CR87, 0x2A);
		else if (Adapter->PA_Type == 1)// Generic No Power Ctrl PA
			ZD1205_WRITE_REGISTER(Adapter,CR87, 0);
		ZD1205_WRITE_REGISTER(Adapter,CR88, 0x10);
		ZD1205_WRITE_REGISTER(Adapter,CR89, 0x24);
		ZD1205_WRITE_REGISTER(Adapter,CR90, 0x18);
		//ZD1205_WRITE_REGISTER(Adapter,CR91, 0x18);
		ZD1205_WRITE_REGISTER(Adapter,CR91, 0x00); // to solve continuous CTS frames problem
		ZD1205_WRITE_REGISTER(Adapter,CR92, 0x0a);
		ZD1205_WRITE_REGISTER(Adapter,CR93, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR94, 0x01);
		ZD1205_WRITE_REGISTER(Adapter,CR95, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR96, 0x40);
		ZD1205_WRITE_REGISTER(Adapter,CR97, 0x37);
#ifdef HOST_IF_USB	
		ZD1205_WRITE_REGISTER(Adapter,CR98, 0x05);
#else
		ZD1205_WRITE_REGISTER(Adapter,CR98, 0x0D);
		ZD1205_WRITE_REGISTER(Adapter,CR121, 0x06);
		ZD1205_WRITE_REGISTER(Adapter,CR125, 0xAA);
#endif
		ZD1205_WRITE_REGISTER(Adapter,CR99, 0x18);
		//ZD1205_WRITE_REGISTER(Adapter,CR100, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13);
		ZD1205_WRITE_REGISTER(Adapter,CR102, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR103, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR104, 0x18);
		ZD1205_WRITE_REGISTER(Adapter,CR105, 0x12);
		
		if (Adapter->bIsNormalSize)
			ZD1205_WRITE_REGISTER(Adapter,CR106, 0x1a);
		else
			ZD1205_WRITE_REGISTER(Adapter,CR106, 0x22);

		ZD1205_WRITE_REGISTER(Adapter,CR107, 0x24);
		ZD1205_WRITE_REGISTER(Adapter,CR108, 0x0a);
		ZD1205_WRITE_REGISTER(Adapter,CR109, 0x13);
		ZD1205_WRITE_REGISTER(Adapter,CR110, 0x2F);
		ZD1205_WRITE_REGISTER(Adapter,CR111, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR112, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR115, 0x40);
		ZD1205_WRITE_REGISTER(Adapter,CR116, 0x40);
		ZD1205_WRITE_REGISTER(Adapter,CR117, 0xF0);
		ZD1205_WRITE_REGISTER(Adapter,CR118, 0xF0);
		ZD1205_WRITE_REGISTER(Adapter,CR119, 0x16);
		//ZD1205_WRITE_REGISTER(Adapter,CR122, 0xfe);
		if ( Adapter->ContinuousTxEnable )
		    ZD1205_WRITE_REGISTER(Adapter,CR122, 0xFF);
		else
            ZD1205_WRITE_REGISTER(Adapter,CR122, 0x00);
		Adapter->CR122Flag = 2;
		Adapter->CR31Flag = 2;
		ZD1205_WRITE_REGISTER(Adapter,CR127, 0x03);
		ZD1205_WRITE_REGISTER(Adapter,CR131, 0x08);
		if ( Adapter->CTFlag )
	        ZD1205_WRITE_REGISTER(Adapter,CR137, 0x90); 
		ZD1205_WRITE_REGISTER(Adapter,CR138, 0x28);
		ZD1205_WRITE_REGISTER(Adapter,CR148, 0x44);
#ifdef HOST_IF_USB
		ZD1205_WRITE_REGISTER(Adapter,CR150, 0x10);
#else
		ZD1205_WRITE_REGISTER(Adapter,CR150, 0x0e);
#endif
		ZD1205_WRITE_REGISTER(Adapter,CR169, 0xBB);
		ZD1205_WRITE_REGISTER(Adapter,CR170, 0xBB);
		//ZD1205_WRITE_REGISTER(Adapter,CR38, 0x30);
		// Release Phy-Config permission
		ReleaseCtrOfPhyReg(Adapter);

		//Set_IF_Synthesizer(Adapter, 0x000007);  //REG0(CFG1)
		//Set_IF_Synthesizer(Adapter, 0x06fd43);  //REG1(IFPLL1)
		//Set_IF_Synthesizer(Adapter, 0x080959);  //REG2(IFPLL2)
		//Set_IF_Synthesizer(Adapter, 0x0e6666);
		//Set_IF_Synthesizer(Adapter, 0x116a7f);
		//Set_IF_Synthesizer(Adapter, 0x16dd43);
		//Set_IF_Synthesizer(Adapter, 0x181979);
		//Set_IF_Synthesizer(Adapter, 0x1e6666);
		//Set_IF_Synthesizer(Adapter, 0x21457f);
		//Set_IF_Synthesizer(Adapter, 0x25e7fa);
		//Set_IF_Synthesizer(Adapter, 0x284128);    //External control TX power (CR31_CCK, CR51_6-36M, CR52_48M, CR53_54M
		//Set_IF_Synthesizer(Adapter, 0x2c0000);
		//Set_IF_Synthesizer(Adapter, 0x300000);
		
		//Set_IF_Synthesizer(Adapter, 0x340000);  //REG13(0xD)
		//Set_IF_Synthesizer(Adapter, 0x381e0f);  //REG14(0xE)
		//Set_IF_Synthesizer(Adapter, 0x6c180f);  //REG27(0x11)
                                                              

		Set_IF_Synthesizer(Adapter, 0x000007);  //REG0(CFG1)
		Set_IF_Synthesizer(Adapter, 0x07dd43);  //REG1(IFPLL1)
		Set_IF_Synthesizer(Adapter, 0x080959);  //REG2(IFPLL2)
		Set_IF_Synthesizer(Adapter, 0x0e6666);
		Set_IF_Synthesizer(Adapter, 0x116a57);  //REG4
		Set_IF_Synthesizer(Adapter, 0x17dd43);  //REG5
		Set_IF_Synthesizer(Adapter, 0x1819f9);  //REG6
		Set_IF_Synthesizer(Adapter, 0x1e6666);
		Set_IF_Synthesizer(Adapter, 0x214554);
		Set_IF_Synthesizer(Adapter, 0x25e7fa);
		Set_IF_Synthesizer(Adapter, 0x27fffa);
		//Set_IF_Synthesizer(Adapter, 0x294128);  //Register control TX power
		// set in Set_RF_Channel( )
		//Set_IF_Synthesizer(Adapter, 0x28252c);    //External control TX power (CR31_CCK, CR51_6-36M, CR52_48M, CR53_54M
		Set_IF_Synthesizer(Adapter, 0x2c0000);
		Set_IF_Synthesizer(Adapter, 0x300000);
		
		Set_IF_Synthesizer(Adapter, 0x340000);  //REG13(0xD)
		Set_IF_Synthesizer(Adapter, 0x381e0f);  //REG14(0xE)
		Set_IF_Synthesizer(Adapter, 0x6c180f);  //REG27(0x11)
	}
	else{
		AcquireCtrOfPhyReg(Adapter);
		if ( Adapter->ContinuousTxEnable )
		    ZD1205_WRITE_REGISTER(Adapter,CR122, 0xFF);
		else
            ZD1205_WRITE_REGISTER(Adapter,CR122, 0x00);
		ReleaseCtrOfPhyReg(Adapter);
		Adapter->CR122Flag = 2;
        Adapter->CR31Flag = 2;  

		Set_IF_Synthesizer(Adapter, RFMD2958t[ChannelNo*2]);
		Set_IF_Synthesizer(Adapter, RFMD2958t[ChannelNo*2+1]);
	}
	//INITSTR((_T("-------------------->>> end\n")));

	return;
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_Maxim_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
HW_Set_Maxim_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	// Get Phy-Config permission
	AcquireCtrOfPhyReg(Adapter);

	ZD1205_WRITE_REGISTER(Adapter,CR33, 0x08);

	// Release Phy-Config permission
	ReleaseCtrOfPhyReg(Adapter);

	Set_IF_Synthesizer(Adapter, max2820Tbl[ChannelNo]);
	if ( ! InitChOnly ){
		Set_IF_Synthesizer(Adapter, 0x0007);
		Set_IF_Synthesizer(Adapter, 0x101e);
		Set_IF_Synthesizer(Adapter, 0x2001);
		Set_IF_Synthesizer(Adapter, 0x4310);
		Set_IF_Synthesizer(Adapter, 0x5000);
	}

}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_Maxim_New_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
HW_Set_Maxim_New_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	ULONG		tmpvalue;

	// Get Phy-Config permission
	AcquireCtrOfPhyReg(Adapter);

#ifdef HOST_IF_USB
	ZD1205_WRITE_REGISTER(Adapter,CR23, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR15, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR28, 0x3e);
	ZD1205_WRITE_REGISTER(Adapter,CR29, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x11);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x33);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x2a);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x1a);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR10, 0x89);
	ZD1205_WRITE_REGISTER(Adapter,CR17, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x93);
	ZD1205_WRITE_REGISTER(Adapter,CR34, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR35, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR41, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x32);
	ZD1205_WRITE_REGISTER(Adapter,CR46, 0x90);
	ZD1205_WRITE_REGISTER(Adapter,CR89, 0x18);
	ZD1205_WRITE_REGISTER(Adapter,CR92, 0x0a);
	ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR102, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x09);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR115, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR116, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR117, 0xf4);
	ZD1205_WRITE_REGISTER(Adapter,CR118, 0xfa);
	ZD1205_WRITE_REGISTER(Adapter,CR120, 0x4f);
	
	ZD1205_WRITE_REGISTER(Adapter,CR121, 0x77);

	ZD1205_WRITE_REGISTER(Adapter,CR122, 0xfe);

#else

	ZD1205_WRITE_REGISTER(Adapter,CR23, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR15, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR28, 0x3e);
	ZD1205_WRITE_REGISTER(Adapter,CR29, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x11);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x33);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x2a);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x1a);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR10, 0x89);
	ZD1205_WRITE_REGISTER(Adapter,CR17, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x93);
	ZD1205_WRITE_REGISTER(Adapter,CR34, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR35, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR41, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x32);
	ZD1205_WRITE_REGISTER(Adapter,CR46, 0x90);
    ZD1205_WRITE_REGISTER(Adapter,CR79, 0x58);
    ZD1205_WRITE_REGISTER(Adapter,CR80, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR81, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR89, 0x18);
	ZD1205_WRITE_REGISTER(Adapter,CR92, 0x0a);
	ZD1205_WRITE_REGISTER(Adapter,CR98, 0x8D);

	ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR102, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR115, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR116, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR117, 0xf4);
	ZD1205_WRITE_REGISTER(Adapter,CR118, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR120, 0x4f);
	
	ZD1205_WRITE_REGISTER(Adapter,CR121, 0x06);
	
	ZD1205_WRITE_REGISTER(Adapter,CR122, 0xfe);

	ZD1205_WRITE_REGISTER(Adapter,CR125, 0xAA);
	ZD1205_WRITE_REGISTER(Adapter,CR127, 0x03);

	ZD1205_WRITE_REGISTER(Adapter,CR150, 0x0D);
#endif


	Adapter->CR122Flag = 2;
	Adapter->CR31Flag = 2;

	// Release Phy-Config permission
	ReleaseCtrOfPhyReg(Adapter);

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 2);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue &= ~BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif

	Set_IF_Synthesizer(Adapter, M2827BF[ChannelNo]);
	Set_IF_Synthesizer(Adapter, M2827BN[ChannelNo]);
	Set_IF_Synthesizer(Adapter, 0x00400);
	Set_IF_Synthesizer(Adapter, 0x00ca1);
	Set_IF_Synthesizer(Adapter, 0x10072);
	Set_IF_Synthesizer(Adapter, 0x18645);
	Set_IF_Synthesizer(Adapter, 0x04006);
	Set_IF_Synthesizer(Adapter, 0x000a7);
	Set_IF_Synthesizer(Adapter, 0x08258);
	Set_IF_Synthesizer(Adapter, 0x03fc9);
	Set_IF_Synthesizer(Adapter, 0x0040a);
	Set_IF_Synthesizer(Adapter, 0x0000b);
	Set_IF_Synthesizer(Adapter, 0x0026c);
#if	defined(ECCK_60_5)
	Set_IF_Synthesizer(Adapter, 0x04258);
#endif

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 3);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue |= BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_Maxim_New_Chips2
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
HW_Set_Maxim_New_Chips2(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	ULONG		tmpvalue;

	// Get Phy-Config permission
	AcquireCtrOfPhyReg(Adapter);

#ifdef HOST_IF_USB	
	ZD1205_WRITE_REGISTER(Adapter,CR23, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR15, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR28, 0x3e);
	ZD1205_WRITE_REGISTER(Adapter,CR29, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x11);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x33);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x2a);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x1a);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR10, 0x89);
	ZD1205_WRITE_REGISTER(Adapter,CR17, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x93);
	ZD1205_WRITE_REGISTER(Adapter,CR34, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR35, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR41, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x32);
	ZD1205_WRITE_REGISTER(Adapter,CR46, 0x90);
	ZD1205_WRITE_REGISTER(Adapter,CR89, 0x18);
	ZD1205_WRITE_REGISTER(Adapter,CR92, 0x0a);
	ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR102, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x09);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR115, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR116, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR117, 0xf4);
	ZD1205_WRITE_REGISTER(Adapter,CR118, 0xfa);
	ZD1205_WRITE_REGISTER(Adapter,CR120, 0x4f);
	
	ZD1205_WRITE_REGISTER(Adapter,CR121, 0x77);
	
	ZD1205_WRITE_REGISTER(Adapter,CR122, 0xfe);
#else
	ZD1205_WRITE_REGISTER(Adapter,CR23, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR15, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR28, 0x3e);
	ZD1205_WRITE_REGISTER(Adapter,CR29, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x11);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x33);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x2a);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x1a);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x2b);
	ZD1205_WRITE_REGISTER(Adapter,CR10, 0x89);
	ZD1205_WRITE_REGISTER(Adapter,CR17, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x93);
	ZD1205_WRITE_REGISTER(Adapter,CR34, 0x30);
	ZD1205_WRITE_REGISTER(Adapter,CR35, 0x40);
	ZD1205_WRITE_REGISTER(Adapter,CR41, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR44, 0x32);
	ZD1205_WRITE_REGISTER(Adapter,CR46, 0x90);
	ZD1205_WRITE_REGISTER(Adapter,CR89, 0x18);
	ZD1205_WRITE_REGISTER(Adapter,CR92, 0x0a);
	ZD1205_WRITE_REGISTER(Adapter,CR98, 0x8D);
	ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR102, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR106, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR107, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR109, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR110, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR111, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR112, 0x13);
	ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27);
	ZD1205_WRITE_REGISTER(Adapter,CR115, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR116, 0x24);
	ZD1205_WRITE_REGISTER(Adapter,CR117, 0xf4);
	ZD1205_WRITE_REGISTER(Adapter,CR118, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR120, 0x4f);

	ZD1205_WRITE_REGISTER(Adapter,CR121, 0x06);
	
	ZD1205_WRITE_REGISTER(Adapter,CR122, 0xfe);

	ZD1205_WRITE_REGISTER(Adapter,CR125, 0xAA);
	ZD1205_WRITE_REGISTER(Adapter,CR127, 0x03);

	ZD1205_WRITE_REGISTER(Adapter,CR150, 0x0D);
#endif

	Adapter->CR122Flag = 2;
	Adapter->CR31Flag = 2;

	// Release Phy-Config permission
	ReleaseCtrOfPhyReg(Adapter);

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 2);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue &= ~BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif

	Set_IF_Synthesizer(Adapter, M2827BF2[ChannelNo]);
	Set_IF_Synthesizer(Adapter, M2827BN2[ChannelNo]);
	Set_IF_Synthesizer(Adapter, 0x00400);
	Set_IF_Synthesizer(Adapter, 0x00ca1);
	Set_IF_Synthesizer(Adapter, 0x10072);
	Set_IF_Synthesizer(Adapter, 0x18645);
	Set_IF_Synthesizer(Adapter, 0x04006);
	Set_IF_Synthesizer(Adapter, 0x000a7);
	Set_IF_Synthesizer(Adapter, 0x08258);
	Set_IF_Synthesizer(Adapter, 0x03fc9);
	Set_IF_Synthesizer(Adapter, 0x0040a);
	Set_IF_Synthesizer(Adapter, 0x0000b);
	Set_IF_Synthesizer(Adapter, 0x0026c);

#if	defined(ECCK_60_5)
	Set_IF_Synthesizer(Adapter, 0x04258);
#endif

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 3);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue |= BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_Ralink_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
HW_Set_Ralink_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	ULONG		tmpvalue;
	// Get Phy-Config permission
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_WRITE_REGISTER(Adapter,CR2, 0x26);
	ZD1205_WRITE_REGISTER(Adapter,CR10, 0x91);
	ZD1205_WRITE_REGISTER(Adapter,CR15, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR20, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR23, 0x90);
	ZD1205_WRITE_REGISTER(Adapter,CR27, 0x50);
	ZD1205_WRITE_REGISTER(Adapter,CR33, 0x48);
	ZD1205_WRITE_REGISTER(Adapter,CR40, 0x86);
	ZD1205_WRITE_REGISTER(Adapter,CR41, 0x28);
	ZD1205_WRITE_REGISTER(Adapter,CR47, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR64, 0xa0);
	ZD1205_WRITE_REGISTER(Adapter,CR80, 0x90);
	ZD1205_WRITE_REGISTER(Adapter,CR113, 0xe0);
	ZD1205_WRITE_REGISTER(Adapter,CR119, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,CR125, 0x10);
	ZD1205_WRITE_REGISTER(Adapter,CR126, 0x20);
	// Release Phy-Config permission
	ReleaseCtrOfPhyReg(Adapter);

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 2);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue &= ~BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif
#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,RADIO_PD, 1);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue |= BIT_6;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif

	Set_IF_Synthesizer(Adapter, RT2420Tbl[ChannelNo]);
	if ( ! InitChOnly ){
		//Set_IF_Synthesizer(Adapter, 0x020b0);
		Set_IF_Synthesizer(Adapter, 0x220b0);
		//Set_IF_Synthesizer(Adapter, 0x00011);
		Set_IF_Synthesizer(Adapter, 0x00111);
	}

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 3);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue |= BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_AL2210_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
HW_Set_AL2210_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	ULONG		tmpvalue;

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 2);
#else
	AcquireCtrOfPhyReg(Adapter);
	//ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue &= ~BIT_4;
	//ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif

	if (!InitChOnly){
		AcquireCtrOfPhyReg(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,CR2, 0x24); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR10, 0x91);
		ZD1205_WRITE_REGISTER(Adapter,CR12, 0x90);
		ZD1205_WRITE_REGISTER(Adapter,CR15, 0xd0);
		ZD1205_WRITE_REGISTER(Adapter,CR16, 0x40);
		ZD1205_WRITE_REGISTER(Adapter,CR17, 0x5e);
		ZD1205_WRITE_REGISTER(Adapter,CR18, 0x04);
		ZD1205_WRITE_REGISTER(Adapter,CR21, 0x11); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR23, 0x66);
		ZD1205_WRITE_REGISTER(Adapter,CR24, 0x20);
		ZD1205_WRITE_REGISTER(Adapter,CR26, 0x90);
		ZD1205_WRITE_REGISTER(Adapter,CR27, 0x50); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR31, 0x80);
		ZD1205_WRITE_REGISTER(Adapter,CR34, 0x06);
		ZD1205_WRITE_REGISTER(Adapter,CR35, 0x43);
		ZD1205_WRITE_REGISTER(Adapter,CR38, 0x38);
		ZD1205_WRITE_REGISTER(Adapter,CR46, 0x10); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR47, 0x10);
		ZD1205_WRITE_REGISTER(Adapter,CR64, 0x64);
		ZD1205_WRITE_REGISTER(Adapter,CR79, 0xb5);
		ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR106, 0x24); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR109, 0x9);  //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR110, 0x27); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR111, 0x27); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR115, 0x30); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR117, 0xc0); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR119, 0x20); //3N07
		ZD1205_WRITE_REGISTER(Adapter,CR127, 0x3);
		ZD1205_WRITE_REGISTER(Adapter,CR140, 0xc9); //3N07
		ReleaseCtrOfPhyReg(Adapter);
		
		Set_IF_Synthesizer(Adapter, AL2210TB[ChannelNo]);
		Set_IF_Synthesizer(Adapter, 0x007cb1);
		Set_IF_Synthesizer(Adapter, 0x358372);
		Set_IF_Synthesizer(Adapter, 0x010a93);
		Set_IF_Synthesizer(Adapter, 0x472804);
		Set_IF_Synthesizer(Adapter, 0x456415);
		Set_IF_Synthesizer(Adapter, 0xEA5556);
		Set_IF_Synthesizer(Adapter, 0x800007);
		Set_IF_Synthesizer(Adapter, 0x7850f8);
		Set_IF_Synthesizer(Adapter, 0xf900c9);
		Set_IF_Synthesizer(Adapter, 0x00000A);
		Set_IF_Synthesizer(Adapter, 0x00000B);

		AcquireCtrOfPhyReg(Adapter);
		ZD1205_WRITE_REGISTER(Adapter, CR47, 0x0);
		ZD1205_READ_REGISTER(Adapter, RADIO_PD, &tmpvalue);
		tmpvalue &= ~BIT_0;
		ZD1205_WRITE_REGISTER(Adapter, RADIO_PD, tmpvalue);
		tmpvalue |= BIT_0;
		ZD1205_WRITE_REGISTER(Adapter, RADIO_PD, tmpvalue);
		ZD1205_WRITE_REGISTER(Adapter, RFCFG, 0x5);
		NdisStallExecution(100);
		ZD1205_WRITE_REGISTER(Adapter, RFCFG, 0x0);
		ZD1205_WRITE_REGISTER(Adapter, CR47, 0x1c);
		ReleaseCtrOfPhyReg(Adapter);
	}
	else{
		Set_IF_Synthesizer(Adapter, AL2210TB[ChannelNo]);
	}


#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 3);
#else
	AcquireCtrOfPhyReg(Adapter);
	//ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue |= BIT_4;
	//ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_Theta_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
HW_Set_Theta_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	ULONG	tmpvalue;

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 2);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue &= ~BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif
	Set_IF_Synthesizer(Adapter, T5024N[ChannelNo]);
	Set_IF_Synthesizer(Adapter, 0x04aa8);
	Set_IF_Synthesizer(Adapter, 0x00b0e);
	Set_IF_Synthesizer(Adapter, 0x082bf);
	Set_IF_Synthesizer(Adapter, 0x3c000);
	//Set_IF_Synthesizer(Adapter, 0x21341);
	Set_IF_Synthesizer(Adapter, 0x025b1);
	Set_IF_Synthesizer(Adapter, 0x20ffe);
	Set_IF_Synthesizer(Adapter, 0x20223);
#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 3);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue |= BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_AL2210_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
HW_Set_AL2230_RF_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	ULONG		tmpvalue;

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 2);
#else
	AcquireCtrOfPhyReg(Adapter);
	//ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue &= ~BIT_4;
	//ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif

	if (!InitChOnly){
		AcquireCtrOfPhyReg(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,CR15, 0x20);
		ZD1205_WRITE_REGISTER(Adapter,CR23, 0x40);
		ZD1205_WRITE_REGISTER(Adapter,CR24, 0x20);
		ZD1205_WRITE_REGISTER(Adapter,CR26, 0x11);
		ZD1205_WRITE_REGISTER(Adapter,CR28, 0x3e);
		ZD1205_WRITE_REGISTER(Adapter,CR29, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR44, 0x33);
		ZD1205_WRITE_REGISTER(Adapter,CR106, 0x2a);
		ZD1205_WRITE_REGISTER(Adapter,CR107, 0x1a);
		ZD1205_WRITE_REGISTER(Adapter,CR109, 0x9);
		ZD1205_WRITE_REGISTER(Adapter,CR110, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR111, 0x2b);
		ZD1205_WRITE_REGISTER(Adapter,CR112, 0x2b);
		ZD1205_WRITE_REGISTER(Adapter,CR119, 0xa);
#if (defined(GCCK) && defined(OFDM))
		ZD1205_WRITE_REGISTER(Adapter,CR10, 0x89);
		ZD1205_WRITE_REGISTER(Adapter,CR17, 0x28);  //for newest(3rd cut) AL2230
		ZD1205_WRITE_REGISTER(Adapter,CR26, 0x93);
		ZD1205_WRITE_REGISTER(Adapter,CR34, 0x30);
		ZD1205_WRITE_REGISTER(Adapter,CR35, 0x3E);  //for newest(3rd cut) AL2230
		ZD1205_WRITE_REGISTER(Adapter,CR41, 0x24);
#ifdef HOST_IF_USB
		ZD1205_WRITE_REGISTER(Adapter,CR44, 0x32);
#else
		ZD1205_WRITE_REGISTER(Adapter,CR44, 0x32);
#endif
		ZD1205_WRITE_REGISTER(Adapter,CR46, 0x96);  //for newest(3rd cut) AL2230  
		ZD1205_WRITE_REGISTER(Adapter,CR47, 0x1e);
		ZD1205_WRITE_REGISTER(Adapter,CR79, 0x58);
		ZD1205_WRITE_REGISTER(Adapter,CR80, 0x30);
		ZD1205_WRITE_REGISTER(Adapter,CR81, 0x30);
		ZD1205_WRITE_REGISTER(Adapter,CR87, 0x0A);
		ZD1205_WRITE_REGISTER(Adapter,CR89, 0x04);
		ZD1205_WRITE_REGISTER(Adapter,CR92, 0x0a);
		ZD1205_WRITE_REGISTER(Adapter,CR99, 0x18);
		//ZD1205_WRITE_REGISTER(Adapter,CR100, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13);
		ZD1205_WRITE_REGISTER(Adapter,CR102, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR106, 0x24);  //for newest(3rd cut) AL2230   
		ZD1205_WRITE_REGISTER(Adapter,CR107, 0x2A);
		ZD1205_WRITE_REGISTER(Adapter,CR109, 0x09);
		ZD1205_WRITE_REGISTER(Adapter,CR110, 0x13);  //for newest(3rd cut) AL2230  
		ZD1205_WRITE_REGISTER(Adapter,CR111, 0x1f);
		ZD1205_WRITE_REGISTER(Adapter,CR112, 0x1f);
		ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR115, 0x24);  //for newest(3rd cut) AL2230   
		ZD1205_WRITE_REGISTER(Adapter,CR116, 0x24);
		ZD1205_WRITE_REGISTER(Adapter,CR117, 0xf4);
		ZD1205_WRITE_REGISTER(Adapter,CR118, 0xfc);
		ZD1205_WRITE_REGISTER(Adapter,CR119, 0x10);
		ZD1205_WRITE_REGISTER(Adapter,CR120, 0x4f);
		ZD1205_WRITE_REGISTER(Adapter,CR121, 0x77);
		ZD1205_WRITE_REGISTER(Adapter,CR122, 0xf0);
	    ZD1205_WRITE_REGISTER(Adapter,CR137, 0x88);
#ifndef HOST_IF_USB
		ZD1205_WRITE_REGISTER(Adapter,CR150, 0x0D);
#endif
#elif (defined(ECCK_60_5))
		ZD1205_WRITE_REGISTER(Adapter,CR47, 0x18);
		ZD1205_WRITE_REGISTER(Adapter,CR106, 0x04);
		ZD1205_WRITE_REGISTER(Adapter,CR107, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR14, 0x80);
		ZD1205_WRITE_REGISTER(Adapter,CR10, 0x89);
		ZD1205_WRITE_REGISTER(Adapter,CR11, 0x00);
		ZD1205_WRITE_REGISTER(Adapter,CR161, 0x28);
		ZD1205_WRITE_REGISTER(Adapter,CR162, 0x26);

		ZD1205_WRITE_REGISTER(Adapter,CR24, 0x0e);
		ZD1205_WRITE_REGISTER(Adapter,CR41, 0x24);
		ZD1205_WRITE_REGISTER(Adapter,CR159, 0x93);
		ZD1205_WRITE_REGISTER(Adapter,CR160, 0xfc);
		ZD1205_WRITE_REGISTER(Adapter,CR161, 0x20);
		ZD1205_WRITE_REGISTER(Adapter,CR162, 0x26);
#endif

		ZD1205_WRITE_REGISTER(Adapter,CR252, 0xff);
		ZD1205_WRITE_REGISTER(Adapter,CR253, 0xff);
                
		ReleaseCtrOfPhyReg(Adapter);
		
		Set_IF_Synthesizer(Adapter, AL2230TB[ChannelNo*3]);
		Set_IF_Synthesizer(Adapter, AL2230TB[ChannelNo*3+1]);
		Set_IF_Synthesizer(Adapter, AL2230TB[ChannelNo*3+2]);
		Set_IF_Synthesizer(Adapter, 0x0b3331);
		Set_IF_Synthesizer(Adapter, 0x03b812);
		Set_IF_Synthesizer(Adapter, 0x00fff3);
		//Set_IF_Synthesizer(Adapter, 0x000da4);  //Reg4 update for MP version
		Set_IF_Synthesizer(Adapter, 0x0005a4);  //To improve AL2230 yield, improve phase noise, 4713
		//Set_IF_Synthesizer(Adapter, 0x04edc5);  //Reg5 update for MP version
		Set_IF_Synthesizer(Adapter, 0x044dc5);  //To improve AL2230 yield, improve phase noise, 4713
		Set_IF_Synthesizer(Adapter, 0x0805b6);  //Reg6 update for MP versio 
		//Set_IF_Synthesizer(Adapter, 0x011687);
		Set_IF_Synthesizer(Adapter, 0x0146C7); // Modified by jxiao for Bor-Chin on 2004/08/02
		Set_IF_Synthesizer(Adapter, 0x000688);
		//Set_IF_Synthesizer(Adapter, 0x0007b9); //register control TX power
		Set_IF_Synthesizer(Adapter, 0x0403b9);   //External control TX power (CR31)
		Set_IF_Synthesizer(Adapter, 0x00dbba);  //RegA update for MP version
		Set_IF_Synthesizer(Adapter, 0x00099b);  //RegB update for MP version 
		Set_IF_Synthesizer(Adapter, 0x0bdffc);  //RegC update for MP version
		Set_IF_Synthesizer(Adapter, 0x00000d);  //RegD update for MP version
		//Set_IF_Synthesizer(Adapter, 0x00500f);
		Set_IF_Synthesizer(Adapter, 0x00580f); //improve phase noise and remove phase calibration,4713
                /*
		AcquireCtrOfPhyReg(Adapter);
		ZD1205_WRITE_REGISTER(Adapter, CR47, 0x0);
		ZD1205_WRITE_REGISTER(Adapter, CR251, 0x2f); // shdnb(PLL_ON)=0
		ZD1205_WRITE_REGISTER(Adapter, CR251, 0x3f); // shdnb(PLL_ON)=1
		ZD1205_WRITE_REGISTER(Adapter, RFCFG1, 0x3); // Continuous TX
		NdisStallExecution(10);
		ReleaseCtrOfPhyReg(Adapter);
		Set_IF_Synthesizer(Adapter, 0x000880f);
		NdisStallExecution(100);
		Set_IF_Synthesizer(Adapter, 0x00080f);
		NdisStallExecution(100);
		AcquireCtrOfPhyReg(Adapter);
		ZD1205_WRITE_REGISTER(Adapter, RFCFG1, 0x0); // stop continuous TX
		ZD1205_WRITE_REGISTER(Adapter, CR47, 0x18);
		ReleaseCtrOfPhyReg(Adapter);
		                             */
		AcquireCtrOfPhyReg(Adapter);
		ZD1205_WRITE_REGISTER(Adapter, CR251, 0x2f); // shdnb(PLL_ON)=0
		ZD1205_WRITE_REGISTER(Adapter, CR251, 0x3f); // shdnb(PLL_ON)=1
		NdisStallExecution(10);
		ReleaseCtrOfPhyReg(Adapter);
		//Set_IF_Synthesizer(Adapter, 0x000d00f);
		Set_IF_Synthesizer(Adapter, 0x000d80f); //To improve AL2230 yield, 4713
		NdisStallExecution(100);
		//Set_IF_Synthesizer(Adapter, 0x0004c0f); //remove phase calibration, 4713
		//NdisStallExecution(100);
		//Set_IF_Synthesizer(Adapter, 0x00540f);  //remove phase calibration, 4713
		//NdisStallExecution(100);
		//Set_IF_Synthesizer(Adapter, 0x00700f);
		Set_IF_Synthesizer(Adapter, 0x00780f);  //To improve AL2230 yield, 4713
		NdisStallExecution(100);
                //Set_IF_Synthesizer(Adapter, 0x00500f);
		Set_IF_Synthesizer(Adapter, 0x00580f); //To improve AL2230 yield,4713
	}
	else{
		Set_IF_Synthesizer(Adapter, AL2230TB[ChannelNo*3]);
		Set_IF_Synthesizer(Adapter, AL2230TB[ChannelNo*3+1]);
		Set_IF_Synthesizer(Adapter, AL2230TB[ChannelNo*3+2]);
	}
    AcquireCtrOfPhyReg(Adapter);
	ZD1205_WRITE_REGISTER(Adapter,CR138, 0x28);
    ZD1205_WRITE_REGISTER(Adapter, CR203, 0x06);
    ReleaseCtrOfPhyReg(Adapter);
	Adapter->CR203Flag = 2;
	Adapter->CR31Flag = 2;


#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 3);
#endif
}


//------------------------------------------------------------------------------
// Procedure:	 HW_Set_UCHIP_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	

ULONG UCHIPTB[] = {
    0x0fe32,   //;Null
    0x0fe32,   //;Ch 1
    0x0fe5a,   //;Ch 2
    0x10002,   //;Ch 3
    0x1002a,   //;Ch 4
    0x10052,   //;Ch 5
    0x1007a,   //;Ch 6
    0x10222,   //;Ch 7
    0x1024a,   //;Ch 8
    0x10272,   //;Ch 9
    0x1041a,   //;Ch 10
    0x10442,   //;Ch 11
    0x1046a,   //;Ch 12
    0x10612,   //;Ch 13
    0x10672    //;Ch 14
};

VOID
HW_Set_UCHIP_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{

	ULONG		tmpvalue;

#if !( (defined(OFDM) && defined(GCCK)) || defined(ECCK_60_5) )
	ZD1205_WRITE_REGISTER(Adapter,PE1_PE2, 2);
#else
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_READ_REGISTER(Adapter,CR203, &tmpvalue);
	tmpvalue &= ~BIT_4;
	ZD1205_WRITE_REGISTER(Adapter,CR203, tmpvalue);
	ReleaseCtrOfPhyReg(Adapter);
#endif

	if (!InitChOnly){
		Set_IF_Synthesizer(Adapter, 0x00000);
		Set_IF_Synthesizer(Adapter, UCHIPTB[ChannelNo]);
		Set_IF_Synthesizer(Adapter, 0xf4c41);
		Set_IF_Synthesizer(Adapter, 0x7210d);
		Set_IF_Synthesizer(Adapter, 0xccc1b);
		Set_IF_Synthesizer(Adapter, 0x2900f);
		Set_IF_Synthesizer(Adapter, 0x00dc0);
		Set_IF_Synthesizer(Adapter, 0x04064);
		Set_IF_Synthesizer(Adapter, 0x01008);
		Set_IF_Synthesizer(Adapter, 0x6018c);
		Set_IF_Synthesizer(Adapter, 0x01010);
		Set_IF_Synthesizer(Adapter, 0x60194);
		Set_IF_Synthesizer(Adapter, 0x04018);
		Set_IF_Synthesizer(Adapter, 0x3183c);
	}
	else{	
		Set_IF_Synthesizer(Adapter, UCHIPTB[ChannelNo]);
	}

}


ULONG UW2451RF[] = {
    0x1100c7, 0x299999,   //;Null 
    0x1100c7, 0x299999,   //;Ch 1
    0x1100c7, 0x29999b,   //;Ch 2
    0x1100e7, 0x299998,   //;Ch 3
    0x1100e7, 0x29999a,   //;Ch 4
    0x1100e7, 0x299999,   //;Ch 5
    0x1100e7, 0x29999b,   //;Ch 6
    0x1100d7, 0x299998,   //;Ch 7
    0x1100d7, 0x29999a,   //;Ch 8
    0x1100d7, 0x299999,   //;Ch 9
    0x1100d7, 0x29999b,   //;Ch 10
    0x1100f7, 0x299998,   //;Ch 11
    0x1100f7, 0x29999a,   //;Ch 12
    0x1100f7, 0x299999,   //;Ch 13
    0x1000cf, 0x2ccccc    //;Ch 14
};

VOID
HW_Set_UW2451_RF_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{

	if (!InitChOnly){
		Set_IF_Synthesizer(Adapter, UW2451RF[ChannelNo*2]);
		Set_IF_Synthesizer(Adapter, UW2451RF[ChannelNo*2+1]);
		
		Set_IF_Synthesizer(Adapter, 0xc);
		Set_IF_Synthesizer(Adapter, 0xa);
		Set_IF_Synthesizer(Adapter, 0x4);

		//CR10->0x83
		ZD1205_WRITE_REGISTER(Adapter, CR10, 0x83);

	}
	else{
		Set_IF_Synthesizer(Adapter, UW2451RF[ChannelNo*2]);
		Set_IF_Synthesizer(Adapter, UW2451RF[ChannelNo*2+1]);
		
		Set_IF_Synthesizer(Adapter, 0xc);
		Set_IF_Synthesizer(Adapter, 0xa);
		Set_IF_Synthesizer(Adapter, 0x4);
		
		ZD1205_WRITE_REGISTER(Adapter, CR10, 0x83);
	}

}

//------------------------------------------------------------------------------
// Procedure:	 Init_RF_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//		RF_Mode
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
Init_RF_Chips(
					IN PZD1205_ADAPTER Adapter, 
					IN ULONG ChannelNo,
					IN UCHAR InitChOnly,
					IN UCHAR RF_Mode
			  )
{
    DEBUGFUNC(_T("Init_RF_Chips"));

	ZD1205_WRITE_REGISTER(Adapter,CONFIGPhilips, 0x0);

	switch(RF_Mode){
	case AL2230_RF:
		INITSTR((_T("AL2230_RF Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 24;
		HW_Set_AL2230_RF_Chips(Adapter, ChannelNo, InitChOnly);
		UpdateIntegrationValue(Adapter, ChannelNo);
		break;
	case THETA_RF:
		INITSTR((_T("Theta Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 18;
		break;
	case AL2210_RF:
		INITSTR((_T("AL2210 Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 24;
		HW_Set_AL2210_Chips(Adapter, ChannelNo, InitChOnly);
		break;
	case MAXIM_NEW_RF:
		Adapter->S_bit_cnt = 18;
		HW_Set_Maxim_New_Chips(Adapter, ChannelNo, InitChOnly);
            //``JWEI 2003/12/23
                UpdateIntegrationValue(Adapter, ChannelNo);
		break;
	case GCT_RF:
		Adapter->S_bit_cnt = 21;
		// We must make sure that the GCT setup-sequence is triggered by a uniformly clock,
		// otherwise it fails. Therefore, we must gurantee atomic, and not in sleep mode.
		AcquireDoNotSleep(Adapter);
		if (!Adapter->DeviceInSleep){
			//KeRaiseIrql(DISPATCH_LEVEL, &OldIrql);
			HW_Set_GCT_Chips(Adapter, ChannelNo, InitChOnly);
			//KeLowerIrql(OldIrql);
		}
		ReleaseDoNotSleep(Adapter);
		UpdateIntegrationValue(Adapter, ChannelNo);
		break;
	case GRF5103:
		Adapter->S_bit_cnt = 21;
		// We must make sure that the GCT setup-sequence is triggered by a uniformly clock,
		// otherwise it fails. Therefore, we must gurantee atomic, and not in sleep mode.
		AcquireDoNotSleep(Adapter);
		if (!Adapter->DeviceInSleep){
			//KeRaiseIrql(DISPATCH_LEVEL, &OldIrql);
			HW_Set_GRF5103_Chips(Adapter, ChannelNo, InitChOnly);
			//KeLowerIrql(OldIrql);
		}
		ReleaseDoNotSleep(Adapter);
		break;
	case PV2000_RF:
		INITSTR((_T("PV2000 Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 22;
		HW_Set_PV2000_Chips(Adapter, ChannelNo, InitChOnly);
		break;
	case RALINK_RF:
		INITSTR((_T("Ralink Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 20;
		HW_Set_Ralink_Chips(Adapter, ChannelNo, InitChOnly);
		break;
	case INTERSIL_RF:
		INITSTR((_T("Intersil Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 20;
		HW_Set_Intersil_Chips(Adapter, ChannelNo, InitChOnly);
		break;
	case RFMD_RF:
		//INITSTR((_T("RFMD Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 24;
		HW_Set_RFMD_Chips(Adapter, ChannelNo,InitChOnly);
		UpdateIntegrationValue(Adapter, ChannelNo);
		break;
	case MAXIM_NEW_RF2:
		INITSTR((_T("Maxim NEW2 Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 18;
		HW_Set_Maxim_New_Chips2(Adapter, ChannelNo, InitChOnly);
		break;
	case PHILIPS_RF:
		INITSTR((_T("Philips SA2400 Channel %x\n"), ChannelNo));
		ZD1205_WRITE_REGISTER(Adapter,RFCFG, 0x2);
		ZD1205_WRITE_REGISTER(Adapter,CONFIGPhilips, 0x3);
		HW_Set_ZeroIF_Chip(Adapter, ChannelNo, InitChOnly);
		ZD1205_WRITE_REGISTER(Adapter,EnablePSManualAGC, 1);
		ZD1205_WRITE_REGISTER(Adapter,RFCFG, 0x0);
		break;
	case uChip_RF:
		INITSTR((_T("UCHIP Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 20;
		HW_Set_UCHIP_Chips(Adapter, ChannelNo, InitChOnly);
		break;
	case UW2451_RF:
		INITSTR((_T("UW2451_RF Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 24;
		HW_Set_UW2451_RF_Chips(Adapter, ChannelNo, InitChOnly);
		break;
	default:
		INITSTR((_T("Invalid RF module parameter %x\n"), RF_Mode));
		INITSTR((_T("Intersil Channel %x\n"), ChannelNo));
		Adapter->S_bit_cnt = 20;
		HW_Set_Intersil_Chips(Adapter, ChannelNo, InitChOnly);
		//ASSERT(0);
		break;
	}

	OverWritePhyRegFromE2P(Adapter);

	return;
}

//------------------------------------------------------------------------------
// Procedure:	 Set_RF_Channel
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		RF_Mode
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
Set_RF_Channel(
					IN PZD1205_ADAPTER Adapter, 
					IN ULONG ChannelNo,
					IN UCHAR RF_Mode,
					IN UCHAR InitChOnly
			  )
{

	ULONG DebugPrint;

    DEBUGFUNC(_T("Set_RF_Channel"));

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_SCAN;

	
	// Check if this ChannelNo allowed?
	if (!((1 << (ChannelNo-1)) & Adapter->AllowedChannel)){
		// Not an allowed channel, we use default channel.
		INITSTRTX(DebugPrint, (_T("Channel = %d Not an allowed channel\n"), ChannelNo));
		INITSTRTX(DebugPrint, (_T("Set default channel = %d\n"), (Adapter->AllowedChannel >> 16)));
		ChannelNo = (Adapter->AllowedChannel >> 16);
#if DBG // jxiaoDbg
		//DbgPrint(" Default channel No =%d\n", ChannelNo); // jxiaoDbg
#endif
	}

	if ((ChannelNo > 14 ) || (ChannelNo < 1)){ // for the wrong content of the EEPROM
		//DbgPrint(" wrong channel\n"); // jxiaoDbg
		return;
	}
#if DBG // jxiaoDbg
	//DbgPrint(" Set channel No =%d\n", ChannelNo); // jxiaoDbg
#endif
	//Init_RF_Chips(Adapter, ChannelNo, 1);
	Adapter->CardSetting.Channel = (UCHAR)ChannelNo;
	//Init_RF_Chips(Adapter, ChannelNo, 0, RF_Mode);
	Init_RF_Chips(Adapter, ChannelNo, InitChOnly, RF_Mode);
	
    // When channnel == 14 , enable Japan spectrum mask
	if ( Adapter->OperationRegionCode == 0x40) { //Japan
		if (ChannelNo == 14){
		//if (1){
			Set_FilterBand(Adapter,Adapter->OperationRegionCode);  // for Japan, RegionCode = 0x40
			
			if (RF_Mode == RFMD_RF){
				AcquireCtrOfPhyReg(Adapter);
				ZD1205_WRITE_REGISTER(Adapter, CR47, 0x18);
				ReleaseCtrOfPhyReg(Adapter);
				Set_IF_Synthesizer(Adapter, 0x28252d);    //External control TX power (CR31_CCK, CR51_6-36M, CR52_48M, CR53_54M
			}
			else if (RF_Mode == AL2230_RF){ // Added by jxiao for BCWang
				AcquireCtrOfPhyReg(Adapter);
				ZD1205_WRITE_REGISTER(Adapter, CR47, 0x18);
				ReleaseCtrOfPhyReg(Adapter);
			    //Set_IF_Synthesizer(Adapter, 0x28252d);    //External control TX power (CR31_CCK, CR51_6-36M, CR52_48M, CR53_54M
			}
		}
		else{
			// For other channels, use default filter.
			Set_FilterBand(Adapter,0); 
			
			if (RF_Mode == RFMD_RF){
				// CR47 has been restored in Init_RF_Chips( ), its value is from EEPROM
				Set_IF_Synthesizer(Adapter, 0x28252d);    //External control TX power (CR31_CCK, CR51_6-36M, CR52_48M, CR53_54M
			}
		}
	} 
	

	ZD1205_Sleep(1000);
	return;
}

void
AcquireCtrOfPhyReg(
					IN PZD1205_ADAPTER Adapter
				  )
{
#ifndef fQuickPhySet
	ULONG		tmpvalue;

	ZD1205_READ_REGISTER(Adapter,CtlReg1, &tmpvalue);
	tmpvalue &= ~0x80;
	ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
#endif
}

void
ReleaseCtrOfPhyReg(
					IN PZD1205_ADAPTER Adapter
				  )
{
#ifndef fQuickPhySet
	ULONG		tmpvalue;

	ZD1205_READ_REGISTER(Adapter,CtlReg1, &tmpvalue);
	tmpvalue |= 0x80;
	ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
#endif
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_GCT_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
HW_Set_GCT_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	if (!InitChOnly){ 
		AcquireCtrOfPhyReg(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,CR47, 0x1c);
		ZD1205_WRITE_REGISTER(Adapter,CR15, 0xDc);
		ZD1205_WRITE_REGISTER(Adapter,CR113, 0xc0);
		ZD1205_WRITE_REGISTER(Adapter,CR20, 0x0c);
		ZD1205_WRITE_REGISTER(Adapter,CR17, 0x65);
		ZD1205_WRITE_REGISTER(Adapter,CR34, 0x04);
		ZD1205_WRITE_REGISTER(Adapter,CR35, 0x35);
		ZD1205_WRITE_REGISTER(Adapter,CR24, 0x20);
		ZD1205_WRITE_REGISTER(Adapter,CR9, 0xe0);
		ZD1205_WRITE_REGISTER(Adapter,CR127, 0x02);
		ZD1205_WRITE_REGISTER(Adapter,CR10, 0x91);
		ZD1205_WRITE_REGISTER(Adapter,CR23, 0x7F);
		ZD1205_WRITE_REGISTER(Adapter,CR27, 0x10);
		ZD1205_WRITE_REGISTER(Adapter,CR28, 0x7A);
		ZD1205_WRITE_REGISTER(Adapter,CR79, 0xb5);
		ZD1205_WRITE_REGISTER(Adapter,CR64, 0x80);
		//++ Enable D.C cancellation (CR33 Bit_5) to avoid
		//	 CCA always high.
		ZD1205_WRITE_REGISTER(Adapter,CR33, 0x08);
		//--
		ZD1205_WRITE_REGISTER(Adapter,CR38, 0x38);
		ReleaseCtrOfPhyReg(Adapter);

		Set_IF_Synthesizer(Adapter, 0x1F0000);
		Set_IF_Synthesizer(Adapter, 0x1F0000);
		Set_IF_Synthesizer(Adapter, 0x1F0200);
		Set_IF_Synthesizer(Adapter, 0x1F0600);
		Set_IF_Synthesizer(Adapter, 0x1F8600);
		Set_IF_Synthesizer(Adapter, 0x1F8600);
		Set_IF_Synthesizer(Adapter, 0x002050);
		Set_IF_Synthesizer(Adapter, 0x1F8000);
		Set_IF_Synthesizer(Adapter, 0x1F8200);
		Set_IF_Synthesizer(Adapter, 0x1F8600);
		Set_IF_Synthesizer(Adapter, 0x1c0000);
		Set_IF_Synthesizer(Adapter, 0x10c458);
		Set_IF_Synthesizer(Adapter, 0x088e92);
		Set_IF_Synthesizer(Adapter, 0x187b82);
		Set_IF_Synthesizer(Adapter, 0x0401cc);
		Set_IF_Synthesizer(Adapter, 0x140816);
		Set_IF_Synthesizer(Adapter, 0x0C7000);
		Set_IF_Synthesizer(Adapter, 0x1c0000);
		Set_IF_Synthesizer(Adapter, 0x02ccae);
		Set_IF_Synthesizer(Adapter, 0x128023);
		Set_IF_Synthesizer(Adapter, 0x0a0000);
		Set_IF_Synthesizer(Adapter, GRF5101T[ChannelNo]);
		Set_IF_Synthesizer(Adapter, 0x06e380);
		Set_IF_Synthesizer(Adapter, 0x16cb94);
		Set_IF_Synthesizer(Adapter, 0x0e1740);
		Set_IF_Synthesizer(Adapter, 0x014980);
		Set_IF_Synthesizer(Adapter, 0x116240);
		Set_IF_Synthesizer(Adapter, 0x090000);
		Set_IF_Synthesizer(Adapter, 0x192304);
		Set_IF_Synthesizer(Adapter, 0x05112f);
		Set_IF_Synthesizer(Adapter, 0x0d98a8);
		Set_IF_Synthesizer(Adapter, 0x0f8000);
		Set_IF_Synthesizer(Adapter, 0x1c0008);

		Set_IF_Synthesizer(Adapter, 0x1c0000);
		Set_IF_Synthesizer(Adapter, GRF5101T[ChannelNo]);
		Set_IF_Synthesizer(Adapter, 0x1c0008);
		
		Set_IF_Synthesizer(Adapter, 0x150000);
		Set_IF_Synthesizer(Adapter, 0x0C7000);
		Set_IF_Synthesizer(Adapter, 0x150800);
		Set_IF_Synthesizer(Adapter, 0x150000);
	}
	else{
		Set_IF_Synthesizer(Adapter, 0x1c0000);
		Set_IF_Synthesizer(Adapter, GRF5101T[ChannelNo]);
		Set_IF_Synthesizer(Adapter, 0x1c0008);
	}
}

//------------------------------------------------------------------------------
// Procedure:	 HW_Set_GRF5103_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//-------------------------------------------------------------------------------
	
VOID
HW_Set_GRF5103_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{
	if (!InitChOnly){ 
		AcquireCtrOfPhyReg(Adapter);
		ZD1205_WRITE_REGISTER(Adapter,CR47, 0x1c);
		ZD1205_WRITE_REGISTER(Adapter,CR15, 0xD0);
		ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR20, 0x0c);
		ZD1205_WRITE_REGISTER(Adapter,CR17, 0x70);
		ZD1205_WRITE_REGISTER(Adapter,CR34, 0x06);
		ZD1205_WRITE_REGISTER(Adapter,CR35, 0x40);
		ZD1205_WRITE_REGISTER(Adapter,CR24, 0x20);
		ZD1205_WRITE_REGISTER(Adapter,CR41, 0x2B);
		ZD1205_WRITE_REGISTER(Adapter,CR9, 0xe0);
		ZD1205_WRITE_REGISTER(Adapter,CR127, 0x02);
		ZD1205_WRITE_REGISTER(Adapter,CR10, 0xB3);
		ZD1205_WRITE_REGISTER(Adapter,CR23, 0x50);
		ZD1205_WRITE_REGISTER(Adapter,CR27, 0x30);
		ZD1205_WRITE_REGISTER(Adapter,CR28, 0x7A);
		ZD1205_WRITE_REGISTER(Adapter,CR46, 0xFF);
		ZD1205_WRITE_REGISTER(Adapter,CR79, 0xb5);
		ZD1205_WRITE_REGISTER(Adapter,CR64, 0x80);
		//++ Enable D.C cancellation (CR33 Bit_5) to avoid
		//	 CCA always high.
		ZD1205_WRITE_REGISTER(Adapter,CR33, 0x08);
		//--
		ZD1205_WRITE_REGISTER(Adapter,CR38, 0x38);
		ZD1205_WRITE_REGISTER(Adapter,CR101, 0x13);
		ZD1205_WRITE_REGISTER(Adapter,CR106, 0x26);
		ZD1205_WRITE_REGISTER(Adapter,CR107, 0x28);
		ZD1205_WRITE_REGISTER(Adapter,CR109, 0x13);
		ZD1205_WRITE_REGISTER(Adapter,CR110, 0x13);
		ZD1205_WRITE_REGISTER(Adapter,CR111, 0x20);
		ZD1205_WRITE_REGISTER(Adapter,CR112, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR113, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR114, 0x27);
		ZD1205_WRITE_REGISTER(Adapter,CR115, 0x26);
		ZD1205_WRITE_REGISTER(Adapter,CR116, 0x28);
		ZD1205_WRITE_REGISTER(Adapter,CR117, 0xFC);
		ZD1205_WRITE_REGISTER(Adapter,CR118, 0xF0);
		ZD1205_WRITE_REGISTER(Adapter,CR119, 0x10);
		ZD1205_WRITE_REGISTER(Adapter,CR38, 0x38);
		ReleaseCtrOfPhyReg(Adapter);

		Set_IF_Synthesizer(Adapter, 0x1F2000);
		Set_IF_Synthesizer(Adapter, 0x1F2000);
		Set_IF_Synthesizer(Adapter, 0x1F2200);
		Set_IF_Synthesizer(Adapter, 0x1F2600);
		Set_IF_Synthesizer(Adapter, 0x1FA600);
		Set_IF_Synthesizer(Adapter, 0x1FA600);
		Set_IF_Synthesizer(Adapter, 0x000020);
		//Set_IF_Synthesizer(Adapter, 0x1F8000);
		Set_IF_Synthesizer(Adapter, 0x1FA000);
		Set_IF_Synthesizer(Adapter, 0x1FA200);
		Set_IF_Synthesizer(Adapter, 0x1FA610);
		
		Set_IF_Synthesizer(Adapter, 0x00E07C);
		Set_IF_Synthesizer(Adapter, 0x107D08);
		Set_IF_Synthesizer(Adapter, 0x0801FC);
		Set_IF_Synthesizer(Adapter, 0x183F68);
		Set_IF_Synthesizer(Adapter, 0x04FFFE);
		Set_IF_Synthesizer(Adapter, 0x140060);
		Set_IF_Synthesizer(Adapter, 0x0CED9C);
		Set_IF_Synthesizer(Adapter, 0x1c0000);
		Set_IF_Synthesizer(Adapter, 0x02C826);
		Set_IF_Synthesizer(Adapter, 0x120014);
		Set_IF_Synthesizer(Adapter, 0x0a0000);
		Set_IF_Synthesizer(Adapter, 0x1a0000);
		Set_IF_Synthesizer(Adapter, 0x068100);
		Set_IF_Synthesizer(Adapter, 0x160000);
		Set_IF_Synthesizer(Adapter, 0x0e8000);
		Set_IF_Synthesizer(Adapter, 0x010380);
		Set_IF_Synthesizer(Adapter, 0x19FC10);
		Set_IF_Synthesizer(Adapter, 0x050000);
		Set_IF_Synthesizer(Adapter, 0x156B60);
		Set_IF_Synthesizer(Adapter, 0x0D0000);
		Set_IF_Synthesizer(Adapter, 0x1D0000);
		Set_IF_Synthesizer(Adapter, 0x030000);

		Set_IF_Synthesizer(Adapter, 0x0F84FF); //STANDBY

		Set_IF_Synthesizer(Adapter, 0x1C6008); //TURN ON PLL
		
		Set_IF_Synthesizer(Adapter, 0x09FA0C);
		Set_IF_Synthesizer(Adapter, 0x09FA4C);
		Set_IF_Synthesizer(Adapter, 0x09FB0C);
		Set_IF_Synthesizer(Adapter, 0x09FB4C);
		Set_IF_Synthesizer(Adapter, 0x090A84);
		Set_IF_Synthesizer(Adapter, 0x090AC4);
		Set_IF_Synthesizer(Adapter, 0x090B84);
		Set_IF_Synthesizer(Adapter, 0x090BC4);
		//Set_IF_Synthesizer(Adapter, 0x092A88);
		//Set_IF_Synthesizer(Adapter, 0x092B88);
		//Set_IF_Synthesizer(Adapter, 0x092BC8);
		//Set_IF_Synthesizer(Adapter, 0x092B88);
		Set_IF_Synthesizer(Adapter, 0x0F94FF); //PUT GRF5103 INTO RX MODE
		

	//	Set_IF_Synthesizer(Adapter, 0x1c0000);
		//Set_IF_Synthesizer(Adapter, GRF5103T[ChannelNo]);
		Set_IF_Synthesizer(Adapter, 0x1C600A);
		Set_IF_Synthesizer(Adapter, 0x1C6008);
		Set_IF_Synthesizer(Adapter, 0x1C6008);
		Set_IF_Synthesizer(Adapter, 0x1C6008);
		Set_IF_Synthesizer(Adapter, 0x1C6008);
		
		
		
	}
	else{
		Set_IF_Synthesizer(Adapter, 0x1c0000);
		Set_IF_Synthesizer(Adapter, GRF5103T[ChannelNo]);
		
	}
}


//------------------------------------------------------------------------------
// Procedure:	 HW_Set_PV2000_Chips
//
// Description:  
//
// Arguments:
//		Adapter - ptr to Adapter object instance
//		ChannelNo
//		Initial Channel only
//
// Returns:		(none)
//
// Note:
//		1. TR_SW, TR_SW_BAR not present in ZD1205 (Dr.Wang Confirmed)
//-------------------------------------------------------------------------------
	
VOID
HW_Set_PV2000_Chips(
						IN PZD1205_ADAPTER Adapter, 
						IN ULONG ChannelNo,
						IN UCHAR InitChOnly
					  )
{	

	// Get Phy-Config permission
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_WRITE_REGISTER(Adapter,CR23, 0x60);
	ZD1205_WRITE_REGISTER(Adapter,CR16, 0x30);
#ifndef ASIC
	ZD1205_WRITE_REGISTER(Adapter,CR30, 0x3a);
#else
	ZD1205_WRITE_REGISTER(Adapter,CR30, 0x38);
#endif
	ZD1205_WRITE_REGISTER(Adapter,CR22, 0x12);
	ZD1205_WRITE_REGISTER(Adapter,CR64, 0x90);
	ZD1205_WRITE_REGISTER(Adapter,CR2, 0x1f);
	ZD1205_WRITE_REGISTER(Adapter,CR1, 0x09);
	ZD1205_WRITE_REGISTER(Adapter,CR28, 0x20);
	ZD1205_WRITE_REGISTER(Adapter,CR29, 0x80);
	ZD1205_WRITE_REGISTER(Adapter,CR21, 0x0a);
	ZD1205_WRITE_REGISTER(Adapter,CR26, 0x10);
	// Release Phy-Config permission
	ReleaseCtrOfPhyReg(Adapter);

	Set_IF_Synthesizer(Adapter, 0x030440);
	Set_IF_Synthesizer(Adapter, 0x000001);
	Set_IF_Synthesizer(Adapter, 0x000012);
	Set_IF_Synthesizer(Adapter, PVITbl[ChannelNo]);
	Set_IF_Synthesizer(Adapter, 0x000004);
	Set_IF_Synthesizer(Adapter, 0x000005);
	Set_IF_Synthesizer(Adapter, 0x000216);
	Set_IF_Synthesizer(Adapter, 0x000007);
	Set_IF_Synthesizer(Adapter, 0x000008);
	Set_IF_Synthesizer(Adapter, PVITbl[ChannelNo]);

	return;
}


VOID
OverWritePhyRegFromE2P(
						IN PZD1205_ADAPTER Adapter
						)
{
	ULONG	tmpvalue;

	if (!Adapter->bOverWritePhyRegFromE2P) return;

#if defined(HMAC_1202)
	ZD1205_READ_REGISTER(Adapter,E2P_POD, &tmpvalue);
	if (tmpvalue & BIT_7){
		ZD1205_READ_REGISTER(Adapter,E2P_PHY_REG, &NumOfReg);
		NumOfReg = NumOfReg & 0xff;
		
		for (i=0;i<NumOfReg;i+=2){
			ZD1205_READ_REGISTER(Adapter,(E2P_PHY_REG+4+i*2), &tmpvalue);
			PhyIdx = (UCHAR)(tmpvalue & 0xff);
			PhyValue = (UCHAR)((tmpvalue >> 8) & 0xff);
			
			WritePhyReg(Adapter, PhyIdx, PhyValue);
			
			if ((i+1) < NumOfReg){
				PhyIdx = (UCHAR)((tmpvalue >> 16) & 0xff);
				PhyValue = (UCHAR)((tmpvalue >> 24) & 0xff);
				WritePhyReg(Adapter, PhyIdx, PhyValue);
			}
		}
	}
	return;
#endif

#ifdef HOST_IF_USB
	AcquireCtrOfPhyReg(Adapter);

	ZD1205_READ_REGISTER(Adapter, E2P_PHY_REG, &tmpvalue);
	ZD1205_WRITE_REGISTER(Adapter, CR47, (tmpvalue & 0xFF));
	
	ReleaseCtrOfPhyReg(Adapter);
#endif
}

VOID
WritePhyReg(
			IN PZD1205_ADAPTER Adapter,
			UCHAR PhyIdx,
			UCHAR PhyValue
			)
{
	ULONG	IoAddress;

	switch(PhyIdx){
		case 4:
			IoAddress = 0x20;
			break;
		case 5:
			IoAddress = 0x10;
			break;
		case 6:
			IoAddress = 0x14;
			break;
		case 7:
			IoAddress = 0x18;
			break;
		case 8:
			IoAddress = 0x1C;
			break;
		default:
			IoAddress = (((ULONG)PhyIdx) << 2);
			break;
	}
	AcquireCtrOfPhyReg(Adapter);
	ZD1205_WRITE_REGISTER(Adapter,IoAddress, PhyValue);
	ReleaseCtrOfPhyReg(Adapter);
}

VOID 
UpdateIntegrationValue(
			IN PZD1205_ADAPTER Adapter,
			ULONG	ChannelNo
			)
{
	ULONG	tmpvalue;

#if defined(HMAC_1202)
    ZD1205_READ_REGISTER(Adapter, E2P_POD, &tmpvalue);
    if (tmpvalue & BIT_7)
    {
        ZD1205_READ_REGISTER(Adapter,E2P_PWR_INT_VALUE+((ChannelNo-1) & 0xc), &tmpvalue);
        tmpvalue = (UCHAR)(tmpvalue >> (((ChannelNo-1) % 4) * 8));
        Write_TxGain(Adapter, tmpvalue);
    }
#endif
//``JWEI 2003/12/31
#ifdef HOST_IF_USB
//    ZD1205_READ_REGISTER(Adapter, E2P_PWR_INT_VALUE + ((ChannelNo - 1) & 0xc), &tmpvalue);
    tmpvalue = Adapter->IntValue[ChannelNo - 1] - Adapter->IntValueLevel;
//    JWeiDbgPrint3(1, (" <IntValue%d = 0x%02X,", ChannelNo, tmpvalue));
    JWeiDbgPrint3(0, (" <IntValue%d = 0x%02X>\n", ChannelNo, tmpvalue));
//    tmpvalue = (UCHAR) (tmpvalue >> (((ChannelNo - 1) % 4) * 8));
    Write_TxGain1(Adapter, (UCHAR) tmpvalue, cTX_CCK);
/*
    ZD1205_READ_REGISTER(Adapter, E2P_36M_INT_VALUE + ((ChannelNo - 1) & 0xc), &tmpvalue);
    JWeiDbgPrint3(0, (" OFDM = 0x%02X>", tmpvalue));
    tmpvalue = (UCHAR) (tmpvalue >> (((ChannelNo - 1) % 4) * 8));
    Write_TxGain1(Adapter, (UCHAR) tmpvalue, cTX_OFDM);
*/
#endif
}



void
AcquireDoNotSleep(
			IN PZD1205_ADAPTER Adapter
			)
{
#ifndef CE_ALL
	ZD1211_IRQL	OldIrql, irql;

	irql = KeGetCurrentIrql();
	if ( irql < DISPATCH_LEVEL )
	    KeRaiseIrql(DISPATCH_LEVEL, &OldIrql);
	Adapter->DoNotSleep ++;
	if ( irql < DISPATCH_LEVEL )
	    KeLowerIrql(OldIrql);
#endif
}

void
ReleaseDoNotSleep(
			IN PZD1205_ADAPTER Adapter
			)
{
#ifndef CE_ALL
	ZD1211_IRQL	OldIrql, irql;

	irql = KeGetCurrentIrql();
	if ( irql < DISPATCH_LEVEL )
	    KeRaiseIrql(DISPATCH_LEVEL, &OldIrql);
	Adapter->DoNotSleep --;
	if ( irql < DISPATCH_LEVEL )
	    KeLowerIrql(OldIrql);
#endif
}

void
Set_FilterBand(
			IN PZD1205_ADAPTER Adapter,
			ULONG	region_code
			)
{
	switch(region_code){
		case 0x40:	// Japan
			// Get Phy-Config permission
			AcquireCtrOfPhyReg(Adapter);
			//if ( Adapter->RF_Mode == MAXIM_NEW_RF ) 
			{

				ULONG	tmpLong;
				
				ZD1205_READ_REGISTER(Adapter,CR5, &tmpLong);

				tmpLong |= BIT_6;	//japan

				ZD1205_WRITE_REGISTER(Adapter,CR5, tmpLong);

			}
			// Release Phy-Config permission
			ReleaseCtrOfPhyReg(Adapter);
			break;
		default:	// non-Japan
			// Get Phy-Config permission
			AcquireCtrOfPhyReg(Adapter);
			{
				ULONG	tmpLong;
				
				ZD1205_READ_REGISTER(Adapter,CR5, &tmpLong);

				tmpLong &= ~BIT_6;//USA

				ZD1205_WRITE_REGISTER(Adapter,CR5, tmpLong);
			}
			// Release Phy-Config permission
			ReleaseCtrOfPhyReg(Adapter);

			break;
	}
}

