/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    inline.h

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

extern UCHAR BroadcastBSSID[6];

//-----------------------------------------------------------------------------
// Procedure:   WaitScb
//
// Description: This routine checks to see if the ZD1205 has accepted a command.
//              It does so by checking the command field in the SCB, which will
//              be zeroed by the ZD1205 upon accepting a command.  The loop waits
//              for up to 600 milliseconds for command acceptance.
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//
// Returns:
//      TRUE if the SCB cleared within 600 milliseconds.
//      FALSE if it didn't clear within 600 milliseconds
//-----------------------------------------------------------------------------
__inline
BOOLEAN
WaitScb(
        IN PZD1205_ADAPTER Adapter
)

{
	return TRUE;
}

//-----------------------------------------------------------------------------
// Procedure:   ZD1205StallExecution
//
// Description: This routine cause a delay for the number of milliseconds that
//              it is passed.
//
// Arguments:
//      MsecDelay - How many milliseconds to delay for
//
// Returns:
//      NOTHING
//-----------------------------------------------------------------------------
__inline
VOID
ZD1205StallExecution(
                   IN UINT MsecDelay
)

{
    // Delay in 100 usec increments
    MsecDelay *= 10;
    while (MsecDelay)
    {
        NdisStallExecution(100);
                MsecDelay--;
    }

}


//-----------------------------------------------------------------------------
// Procedure:   ZD1205IssueScbCommand
//
// Description: This general routine will issue a command to the ZD1205.
//
// Arguments:
//      Adapter - ptr to Adapter object instance.
//      ScbCommand - The command that is to be issued
//      WaitForSCB - A boolean value indicating whether or not a wait for SCB
//                   must be done before the command is issued to the chip
//
// Returns:
//      TRUE if the command was issued to the chip successfully
//      FALSE if the command was not issued to the chip
//-----------------------------------------------------------------------------
__inline
BOOLEAN
ZD1205IssueScbCommand(
                    IN PZD1205_ADAPTER Adapter,
                    IN UCHAR ScbCommandLow,
                    IN BOOLEAN WaitForScb
)

{
    return (TRUE);
}

__inline
void
Cal_Us_Service(
				UCHAR TxRate,
				USHORT Len,
				PUSHORT pLenInUs,
				PCHAR pService
				)
{
	ULONG		Remainder;
	LONG		Delta;
	DEBUGFUNC(_T("Cal_Us_Service"));

	*(pService) = 0x00;

	switch(TxRate){
		case 0:		// 1M bps
			*(pLenInUs) = Len << 3;
			break;
		case 1:		// 2M bps
			*(pLenInUs) = Len << 2;
			break;
		case 2:		// 5.5M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 4)/11);
			Remainder = (((ULONG)Len << 4) % 11);
			if ( Remainder ){
				*(pLenInUs) += 1;
			}
			break;
		case 3:		// 11M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/11);
			Remainder = (((ULONG)Len << 3) % 11);
			Delta = 11 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta >= 8 ){
					*(pService) |= BIT_7;
				}
			}
			break;
#if !(defined(GCCK) && defined(OFDM))
		case 4:		// 16.5M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 4)/33);
			Remainder = (((ULONG)Len << 4) % 33);
			Delta = 33 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 16 ){
				}
				else if ( (Delta >= 16) && (Delta < 32) ){
					*(pService) |= BIT_7;
				}
				else if ( Delta >= 32 ){
					*(pService) |= BIT_6;
				}
			}
			break;
		case 5:		// 22M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 2)/11);
			Remainder = (((ULONG)Len << 2) % 11);
			Delta = 11 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 4 ){
				}
				else if ( (Delta >= 4) && (Delta < 8) ){
					*(pService) |= BIT_7;
				}
				else if ( Delta >= 8 ){
					*(pService) |= BIT_6;
				}
			}
			break;
		case 6:		// 27.5M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 4)/55);
			Remainder = (((ULONG)Len << 4) % 55);
			Delta = 55 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 16 ){
				}
				else if ( (Delta >= 16) && (Delta < 32) ){
					*(pService) |= BIT_7;
				}
				else if ( (Delta >= 32) && (Delta < 48) ){
					*(pService) |= BIT_6;
				}
				else if ( Delta >= 48 ){
					*(pService) |= (BIT_6 | BIT_7);
				}
			}
			break;
		case 7:		// 33M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/33);
			Remainder = (((ULONG)Len << 3) % 33);
			Delta = 33 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 8 ){
				}
				else if ( (Delta >= 8) && (Delta < 16) ){
					*(pService) |= BIT_7;
				}
				else if ( (Delta >= 16) && (Delta < 24) ){
					*(pService) |= BIT_6;
				}
				else if ( (Delta >= 24) && (Delta < 32) ){
					*(pService) |= (BIT_6 | BIT_7);
				}
				else if ( Delta >= 32 ){
					*(pService) |= BIT_5;
				}
			}
			break;
		case 8:		// 38.5M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 4)/77);
			Remainder = (((ULONG)Len << 4) % 77);
			Delta = 77 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 16 ){
				}
				else if ( (Delta >= 16) && (Delta < 32) ){
					*(pService) |= BIT_7;
				}
				else if ( (Delta >= 32) && (Delta < 48) ){
					*(pService) |= BIT_6;
				}
				else if ( (Delta >= 48) && (Delta < 64) ){
					*(pService) |= (BIT_6 | BIT_7);
				}
				else if ( Delta >= 64){
					*(pService) |= BIT_5;
				}
			}
			break;
		case 9:		// 44M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 1)/11);
			Remainder = (((ULONG)Len << 1) % 11);
			Delta = 11 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 2 ){
				}
				else if ( (Delta >= 2) && (Delta < 4) ){
					*(pService) |= BIT_7;
				}
				else if ( (Delta >= 4) && (Delta < 6) ){
					*(pService) |= BIT_6;
				}
				else if ( (Delta >= 6) && (Delta < 8) ){
					*(pService) |= (BIT_6 | BIT_7);
				}
				else if ( (Delta >= 8) && (Delta < 10) ){
					*(pService) |= BIT_5;
				}
				else if ( Delta >= 10 ){
					*(pService) |= (BIT_5 | BIT_7);
				}
			}
			break;
		case 10:		// 49.5M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 4)/99);
			Remainder = (((ULONG)Len << 4) % 99);
			Delta = 99 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 16 ){
				}
				else if ( (Delta >= 16) && (Delta < 32) ){
					*(pService) |= BIT_7;
				}
				else if ( (Delta >= 32) && (Delta < 48) ){
					*(pService) |= BIT_6;
				}
				else if ( (Delta >= 48) && (Delta < 64) ){
					*(pService) |= (BIT_6 | BIT_7);
				}
				else if ( (Delta >= 64) && (Delta < 80) ){
					*(pService) |= BIT_5;
				}
				else if ( (Delta >= 80) && (Delta < 96) ){
					*(pService) |= (BIT_5 | BIT_7);
				}
				else if ( Delta >= 96 ){
					*(pService) |= (BIT_5 | BIT_6); 
				}
			}
			break;
		case 11:		// 55M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/55);
			Remainder = (((ULONG)Len << 3) % 55);
			Delta = 55 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 8 ){
				}
				else if ( (Delta >= 8) && (Delta < 16) ){
					*(pService) |= BIT_7;
				}
				else if ( (Delta >= 16) && (Delta < 24) ){
					*(pService) |= BIT_6;
				}
				else if ( (Delta >= 24) && (Delta < 32) ){
					*(pService) |= (BIT_6 | BIT_7);
				}
				else if ( (Delta >= 32) && (Delta < 40) ){
					*(pService) |= BIT_5;
				}
				else if ( (Delta >= 40) && (Delta < 48) ){
					*(pService) |= (BIT_5 | BIT_7);
				}
				else if ( Delta >= 48 ){
					*(pService) |= (BIT_5 | BIT_6);
				}
			}
			break;
		case 12:		// 60.5M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len << 4)/121);
			Remainder = (((ULONG)Len << 4) % 121);
			Delta = 121 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 16 ){
				}
				else if ( (Delta >= 16) && (Delta < 32) ){
					*(pService) |= BIT_7;
				}
				else if ( (Delta >= 32) && (Delta < 48) ){
					*(pService) |= BIT_6;
				}
				else if ( (Delta >= 48) && (Delta < 64) ){
					*(pService) |= (BIT_6 | BIT_7);
				}
				else if ( (Delta >= 64) && (Delta < 80) ){
					*(pService) |= BIT_5;
				}
				else if ( (Delta >= 80) && (Delta < 96) ){
					*(pService) |= (BIT_5 | BIT_7);
				}
				else if ( (Delta >= 96) && (Delta < 112) ){
					*(pService) |= (BIT_5 | BIT_6);
				}
				else if ( Delta >= 112 ){
					*(pService) |= (BIT_5 | BIT_6 | BIT_7);
				}
			}
			break;
		case 13:		// 8.25M bps
			*(pLenInUs) = (USHORT)(((ULONG)Len * 32)/33);
			Remainder = (((ULONG)Len * 32) % 33);
			Delta = 33 - Remainder;
			if ( Remainder ){
				*(pLenInUs) += 1;
				if ( Delta < 32 ){
				}
				else {
					*(pService) |= BIT_7;
				}
			}
			break;
#else
		case 0x04:	// 6M
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/6);
			break;
		case 0x05:	// 9M
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/9);
			break;
		case 0x06:	// 12M
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/12);
			break;
		case 0x07:	// 18M
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/18);
			break;
		case 0x08:	// 24M
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/24);
			break;
		case 0x09:	// 36M
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/36);
			break;
		case 0x0a:	// 48M
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/48);
			break;
		case 0x0b:	// 54M
			*(pLenInUs) = (USHORT)(((ULONG)Len << 3)/54);
			break;
#endif
		default:
			INITSTR((_T("Invalid TxRate\n")));
			ASSERT(0);
			break;
	}
}


#define CTRL_SIZE	25
__inline
ULONG
Cfg_CtrlSetting(
				IN PZD1205_ADAPTER Adapter,
				IN PZD1205SwTcb SwTcb,
				PETH_HEADER_STRUC pEnetHdr,
				UCHAR	FrameType,
				IN UINT  FragNum,
				UINT	CurrFragLen,
				UINT	NextFragLen,
				BOOLEAN		EnCipher,
				BOOLEAN		b_EAPOL
				)
{
	PCTRL_STRUC		pCtrlSet;
	UCHAR			tmp;
	USHORT			Len;
	USHORT			NextLen;
	USHORT			LenInUs;
	USHORT			NextLenInUs;
	UCHAR			Service;
	ULONG			DebugPrint;
	UCHAR			OldTxRate;
	UCHAR           mac_addr[6];
	UCHAR			OppositeRate, OppositePreamble;
	BOOLEAN			bBusrt = 0;
	DEBUGFUNC(_T("Cfg_CtrlSetting"));

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_TX;

	if (Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS)
		memcpy(mac_addr, Adapter->BSSID, 6);
	else
		memcpy(mac_addr, pEnetHdr->Destination, 6);

	QueryOppositeRatePreamble( Adapter, mac_addr, 
		                       &OppositeRate, 
							   &OppositePreamble,
							   SwTcb
							   );

	if ( Adapter->preambleMode == 1 ){ // Force long-preamble
        Adapter->CardSetting.PreambleType = 0x00;
	}
	else if ( Adapter->preambleMode == 2 ){ // Force short-preamble
		Adapter->CardSetting.PreambleType = 0x20;
	}
	else if ( Adapter->CardSetting.BSSType == INDEPENDENT_BSS ){
        Adapter->CardSetting.PreambleType = 0x00;
	}
	else{	// Auto		
		Adapter->CardSetting.PreambleType = OppositePreamble;

	}

	//ASSERT(!EnCipher);
	pCtrlSet = SwTcb->HwCtrlPtr;

	OldTxRate = Adapter->CardSetting.TxRate;

    // rate setting by jhlee
	if ( !Adapter->RateLock ){		
		Adapter->CardSetting.TxRate = OppositeRate;
		if ( Adapter->CardSetting.TxRate > Adapter->AdapterMaxRate )
            Adapter->CardSetting.TxRate = Adapter->AdapterMaxRate;
		if ( (!(mac_addr[0] & BIT_0))&&((FrameType & 0x0c) == DATA) )
		    Adapter->CardSetting.LastSentTxRate = Adapter->CardSetting.TxRate;
	}
	else{
		if ( (!(mac_addr[0] & BIT_0))&&((FrameType & 0x0c) == DATA) )
		    Adapter->CardSetting.LastSentTxRate = OldTxRate;
	}

	//if (FrameType == PROBERESPONSE){
		// Prevent probe-response from retrying too much when no ack received.
	//	Adapter->CardSetting.TxRate = 0x0;
	//}
	//else if (FrameType == PS_POLL){
		// For compatibility with D-Link AP
	//	Adapter->CardSetting.TxRate = 0x0;
	//}
	//else if (FrameType == PROBEREQUEST){
		// For compatibility with 11b BSSID
	//	Adapter->CardSetting.TxRate = 0x0;
	//}

//JWEI 2004/03/22
    if (((FrameType & 0x0c) == MANAGEMENT) && (FrameType != MGT_LBTEST))
    {
        //tmp |= BIT_3;
        //Adapter->SelfCTSOn = FALSE;
        Adapter->CardSetting.TxRate = 0x01;
	}

	// Rate protection for Short preamble.
	if ((Adapter->CardSetting.PreambleType == 0x20) && (Adapter->CardSetting.TxRate ==0x0)){
		Adapter->CardSetting.TxRate = 0x1;
	}

	// Set the control-setting
#if !(defined(GCCK) && defined(OFDM))
	pCtrlSet->CtrlSetting[0] = (Adapter->CardSetting.TxRate | Adapter->CardSetting.PreambleType);
#else
    if (Adapter->CardSetting.TxRate <= 3)
    {
        pCtrlSet->CtrlSetting[0] = (Adapter->CardSetting.TxRate | Adapter->CardSetting.PreambleType);
    //``JWEI 2003/12/22
    #if fTX_PWR_CTRL
        if ((Adapter->TxOFDMCnt > 0) && (Adapter->TxOFDMCnt < cTX_SENT_LEN))
            Adapter->TxOFDMCnt --;
        Adapter->TxPwrCCK ++;
    #endif
	}
	//else if ( (FrameType & 0x0c) == MANAGEMENT ){
	//	Adapter->CardSetting.TxRate = 0x04;
	//	pCtrlSet->CtrlSetting[0] = 0x1b;
	//    INITSTR((_T("TxRate1 = %d\n"), Adapter->CardSetting.TxRate));
    //}
    else
    {
    //``JWEI 2003/12/22
    #if fTX_PWR_CTRL
        Adapter->TxPwrOFDM ++;
        Adapter->TxOFDMCnt ++;
        if (Adapter->CardSetting.TxRate == 0x0A)
        {                                   // 48M
            if (Adapter->TxOFDMType != cTX_48M)
                Adapter->TxOFDMCnt = 0;
            Adapter->TxOFDMType = cTX_48M;
        }
        else if (Adapter->CardSetting.TxRate == 0x0B)
        {                                   // 54M
            if (Adapter->TxOFDMType != cTX_54M)
                Adapter->TxOFDMCnt = 0;
            Adapter->TxOFDMType = cTX_54M;
        }
        else
        {
            if (Adapter->TxOFDMType != cTX_OFDM)
                Adapter->TxOFDMCnt = 0;
            Adapter->TxOFDMType = cTX_OFDM;
        }
    #endif

		switch (Adapter->CardSetting.TxRate){
			case 0x04:	// 6M
				pCtrlSet->CtrlSetting[0] = 0x1b;
				break;
			case 0x05:	// 9M
				pCtrlSet->CtrlSetting[0] = 0x1f;
				break;
			case 0x06:	// 12M
				pCtrlSet->CtrlSetting[0] = 0x1a;
				break;
			case 0x07:	// 18M
				pCtrlSet->CtrlSetting[0] = 0x1e;
				break;
			case 0x08:	// 24M
				pCtrlSet->CtrlSetting[0] = 0x19;
				break;
			case 0x09:	// 36M
				pCtrlSet->CtrlSetting[0] = 0x1d;
				break;
			case 0x0a:	// 48M
				pCtrlSet->CtrlSetting[0] = 0x18;
				break;
			case 0x0b:	// 54M
				pCtrlSet->CtrlSetting[0] = 0x1c;
				break;
			default:
				ASSERT(0);
				break;
		}
	}
#endif
	//INITSTRTX(DebugPrint,(_T("Signal field = %x\n"), pCtrlSet->CtrlSetting[0]));

	// Length in byte
	if (EnCipher && (!b_EAPOL)){
		switch(Adapter->CardSetting.EncryMode){
			case 0:	// No encryption
				Len = CurrFragLen + 28; // Header(24) + CRC32(4)
				NextLen = NextFragLen + 28;
				break;
			case 1: // AES
				Len = CurrFragLen + 44; // Header(24) + CRC32(4) + IV(4) + ExtendedIV(4) + MIC(8)
				NextLen = NextFragLen + 44;
				break;
			case 2: // WEP 64
			case 3:
			case 5: //wep256 support 
				Len = CurrFragLen + 36; // Header(24) + CRC32(4) + IV(4) + ICV(4)
				NextLen = NextFragLen + 36;
				break;
			case 4: //WPA support (TKIP)
				INITSTRWPA((_T("TKIP EncryMode\n")));
				Len = CurrFragLen + 40; // Header(24) + CRC32(4) + IV(4) + ExtendedIV(4) + ICV(4)
				NextLen = NextFragLen + 40;
				break;	
			default:
				INITSTR((_T("Invalid EncryMode\n")));
				ASSERT(0);
				break;
		}
	}
	else{
		Len = CurrFragLen + 28; // Header(24) + CRC32(4)
		NextLen = NextFragLen + 28;
	}

	//++ Corret some exceptions
	if (FrameType == PS_POLL){
		Len = CurrFragLen + 20; // Header(16) + CRC32(4)
	}
	if (NextFragLen == 0){
		NextLen = 0;
	}
	//--
    //chi_2004/08/11-----------------------------------------------
	//Japan and channl==14 always CtrlSetting (Byte_0 BIT_6) = 1

	if (( Adapter->OperationRegionCode == 0x40) && (Adapter->CardSetting.Channel==14))
		//(Adapter->SaveChannel = 14)//Adapter->CardSetting.Channel==14)
	{
		pCtrlSet->CtrlSetting[0] |= BIT_6; 
	}

	//INITSTRTX(DebugPrint,(_T("TotalLen = %x\n"), Len));
	pCtrlSet->CtrlSetting[1] = (UCHAR)Len; // low byte
	pCtrlSet->CtrlSetting[2] = (UCHAR)(Len >> 8);   // high byte

	// TCB physical address
	pCtrlSet->CtrlSetting[3] = (UCHAR)(SwTcb->TcbPhys);
	pCtrlSet->CtrlSetting[4] = (UCHAR)(SwTcb->TcbPhys >> 8);
	pCtrlSet->CtrlSetting[5] = (UCHAR)(SwTcb->TcbPhys >> 16);
	pCtrlSet->CtrlSetting[6] = (UCHAR)(SwTcb->TcbPhys >> 24);
	pCtrlSet->CtrlSetting[7] = 0x00;
	pCtrlSet->CtrlSetting[8] = 0x00;
	pCtrlSet->CtrlSetting[9] = 0x00;
	pCtrlSet->CtrlSetting[10] = 0x00;

	// Misc
	tmp = 0;
	if (FragNum == 0)
		tmp |= BIT_0;	//need back off

	if (Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS){
		if (Adapter->BSSID[0] & BIT_0)
			tmp |= BIT_1;
	}else{
		if (pEnetHdr->Destination[0] & BIT_0)
			tmp |= BIT_1;
	}

	if (FrameType == PROBEREQUEST){
			tmp |= BIT_1;
	}

	//if ((FrameType & 0x0c) == MANAGEMENT){
		//tmp |= BIT_3;
	//}

	if (FrameType == PS_POLL){
		tmp |= BIT_2;
	}
#ifndef HOST_IF_USB
	if (DestPowerSave(Adapter, &(pEnetHdr->Destination[0]))){
		if ((FrameType & 0x0c) != MANAGEMENT){
			tmp |= BIT_4;
		}
	}
#endif
	if (Len > Adapter->CardSetting.RTSThreshold){
	    if ( ((FrameType & 0x0c) != MANAGEMENT) && 
			(!(pEnetHdr->Destination[0] & 0x01)) )
				tmp |= BIT_5;
	}

#if (defined(GCCK) && defined(OFDM))
	if ((Adapter->CardSetting.TxRate > 3) && (tmp & BIT_5)){
		// need to send RTS or CTS, in OFDM only send CTS, in CCK send RTS
		tmp &= ~BIT_5;
		tmp |= BIT_7;
	}

	if ((Adapter->CardSetting.TxRate > 3) && (Adapter->SelfCTSOn)){
		// id SelfCTS on, force send CTS when OFDM
		tmp |= BIT_7;
	}

	if ( (Adapter->CardSetting.TxRate > 3) &&
		 (Adapter->CardSetting.FragThreshold < 600) &&
		 ((Adapter->CardSetting.BSSType == AP_BSS) ||
		  (Adapter->CardSetting.BSSType == INDEPENDENT_BSS)) )
	{
	    tmp |= BIT_7; 
	}

	// for debug, Force on/off CTS, if not set mean auto by SelfCTSOn
	if ((Adapter->CardSetting.TxRate > 3) && (Adapter->SelfCTSMode == 1)){
		// id SelfCTSMode on, force send CTS
		tmp |= BIT_7;
	}
	else if (Adapter->SelfCTSMode == 2){
		// id SelfCTSMode off, force not send CTS
		tmp &= ~BIT_7;
	}
	
#endif

	if (EnCipher && (!b_EAPOL))
		tmp |= BIT_6;

	pCtrlSet->CtrlSetting[11] = tmp;


	// Address1
	if (Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS){
		memcpy(&pCtrlSet->CtrlSetting[12], &Adapter->BSSID[0], 6);
	}else{
		memcpy(&pCtrlSet->CtrlSetting[12], &pEnetHdr->Destination[0], 6);
	}

	if (FrameType == DATA){
		Adapter->TotalTxDataFrmBytes += Len;
		if (pCtrlSet->CtrlSetting[12] & BIT_0){
			Adapter->txMulticastFrm ++;
			Adapter->txMulticastOctets += CurrFragLen;
		}
		else{
			Adapter->txUnicastFrm ++;
			Adapter->txUnicastOctets += CurrFragLen;
			Adapter->txDataPerSec += CurrFragLen;
		}
	}

	// NextLen
	pCtrlSet->CtrlSetting[18] = (UCHAR)NextLen;
	pCtrlSet->CtrlSetting[19] = (UCHAR)(NextLen >> 8);

	// LenInUs
	Cal_Us_Service(Adapter->CardSetting.TxRate, Len, &LenInUs, &Service);
	if (Adapter->TestFlag & LOCKED_CLOCK){
		Service |= BIT_2;
	}


	pCtrlSet->CtrlSetting[20] = (UCHAR)LenInUs;
	pCtrlSet->CtrlSetting[21] = (UCHAR)(LenInUs >> 8);
//	INITSTRTX(DebugPrint,(_T("Len Low/High = %x, %x\n"), pCtrlSet->CtrlSetting[20],pCtrlSet->CtrlSetting[21]));

	// Service
#if !(defined(GCCK) && defined(OFDM))
	pCtrlSet->CtrlSetting[22] = Service;
#else
	if (Adapter->CardSetting.TxRate <= 3){
		pCtrlSet->CtrlSetting[22] = Service;
	}
	else{
		pCtrlSet->CtrlSetting[22] = 0;
	}
#endif
//	INITSTRTX(DebugPrint,(_T("Service = %x\n"), pCtrlSet->CtrlSetting[22]));


	// NextFrameLenInUs
	//if (Adapter->ShowSeq){
	//	INITSTRTX(DebugPrint,(_T("Serial No: %x\n"), (USHORT)Adapter->TxTotalCnt));
	//	pCtrlSet->CtrlSetting[23] = (UCHAR)Adapter->TxTotalCnt;
	//	pCtrlSet->CtrlSetting[24] = (UCHAR)(Adapter->TxTotalCnt >> 8);
	//	Adapter->TxTotalCnt++;
	//}
	//else{
		Cal_Us_Service(Adapter->CardSetting.TxRate, NextLen, &NextLenInUs, &Service);
//		INITSTRTX(DebugPrint,(_T("NextLenInUs = %x\n"), NextLenInUs));
		pCtrlSet->CtrlSetting[23] = (UCHAR)NextLenInUs;
		pCtrlSet->CtrlSetting[24] = (UCHAR)(NextLenInUs >> 8);
	//}

	Adapter->CardSetting.TxRate = OldTxRate;

#ifdef OFDM
	// NextLen
	// backup NextLen, because OFDM use these 2 bytes as total length,
	// so we backup here for Retry fail use.
	pCtrlSet->CtrlSetting[CTRL_SIZE+1] = (UCHAR)NextLen;
	pCtrlSet->CtrlSetting[CTRL_SIZE+2] = (UCHAR)(NextLen >> 8);
#endif

	return(CTRL_SIZE);
}


__inline
ULONG
Cfg_MacHeader(
				IN PZD1205_ADAPTER Adapter,
				IN PZD1205SwTcb SwTcb,
				PETH_HEADER_STRUC pEnetHdr,
				UCHAR	FrameType,
				IN UCHAR  LastFrag,
				BOOLEAN		EnCipher,
				UINT		FragIndex,
				BOOLEAN		b_EAPOL
				)
{
	PHEADER_STRUC	pHdr;
	UCHAR			HdrLen;
	PSTA_CONTEXT	STAContextPtr;
	DEBUGFUNC(_T("Cfg_MacHeader"));

	pHdr = SwTcb->HwHeaderPtr;

	HdrLen = 24;
	// Frame control
	pHdr->MacHeader[0] = FrameType;
	pHdr->MacHeader[1] = 0;
	if ((Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS) &&
		(FrameType == DATA)){
		pHdr->MacHeader[1] |= 0x1;
	}
	else if ((Adapter->CardSetting.BSSType == AP_BSS) &&
		(FrameType == DATA)){
		// Set FromDs = 1
		pHdr->MacHeader[1] |= 0x2;

		STAContextPtr = FindSTAContext(Adapter, pEnetHdr->Destination);
		if (STAContextPtr){
			if (STAContextPtr->FirstPendingQueue){
				pHdr->MacHeader[1] |= BIT_5;	// More data
			}
		}
	}
	if (!LastFrag){
		pHdr->MacHeader[1] |= 0x4;
	}
	if (Adapter->PwrState){
		pHdr->MacHeader[1] |= 0x10;
		ZDDbgPrint((_T("=====Set PwrMgt bit\n")));
	}
	if (EnCipher && (!b_EAPOL)){
		pHdr->MacHeader[1] |= 0x40;
	}

	// Duration
	if (FrameType == PS_POLL){
		pHdr->MacHeader[2] = (UCHAR)Adapter->AID;
		pHdr->MacHeader[3] = ((UCHAR)(Adapter->AID >> 8) | 0xc0);
	}
	else{
		pHdr->MacHeader[2] = 0x00;
		pHdr->MacHeader[3] = 0x00;
	}
	
	if ((Adapter->CardSetting.BSSType == INDEPENDENT_BSS) ||
		(Adapter->CardSetting.BSSType == PSEUDO_IBSS) ){
		// Address 1
		memcpy(&pHdr->MacHeader[4], &pEnetHdr->Destination[0], 6);
		// Address 2
		memcpy(&pHdr->MacHeader[10], &pEnetHdr->Source[0], 6);
		// Address 3
		if (FrameType == PROBEREQUEST){
			memcpy(&pHdr->MacHeader[16], BroadcastBSSID, 6);
		}else{
			memcpy(&pHdr->MacHeader[16], &Adapter->BSSID[0], 6);
		}
	}
	else if (Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS){
		// Address 1
		if (FrameType == PROBEREQUEST){
			memcpy(&pHdr->MacHeader[4], BroadcastBSSID, 6);
		}else{
			memcpy(&pHdr->MacHeader[4], &Adapter->BSSID[0], 6);
		}
		// Address 2
		// Address 2
		memcpy(&pHdr->MacHeader[10], &pEnetHdr->Source[0], 6);
		// Address 3
		memcpy(&pHdr->MacHeader[16], &pEnetHdr->Destination[0], 6);
	}
	else if (Adapter->CardSetting.BSSType == AP_BSS){
		// Address 1
		memcpy(&pHdr->MacHeader[4], &pEnetHdr->Destination[0], 6);
		// Address 2
		memcpy(&pHdr->MacHeader[10], &Adapter->BSSID[0], 6);
		// Address 3
		memcpy(&pHdr->MacHeader[16], &pEnetHdr->Source[0], 6);
	}
	else{
		ASSERT(0);
	}



	// Sequence
	pHdr->MacHeader[22] = (FragIndex & 0xf);
	pHdr->MacHeader[23] = 0x00;

	if (EnCipher && (!b_EAPOL)){
		// Set IV only for Encryption mode
		switch(Adapter->CardSetting.EncryMode){
		case 0:	// No encryption
			break;
		case 1:	// AES		
			INITSTRWPA((_T("AES EncryMode\n")));
			Adapter->iv16++;  
			if (Adapter->iv16 == 0) {
        		Adapter->iv32++;
				INITSTR((_T("iv16 = %x\n"), Adapter->iv16));
				INITSTR((_T("iv32 = %x\n"), Adapter->iv32));
			}	
			pHdr->MacHeader[24] = Lo8(Adapter->iv16); 
	//		pHdr->MacHeader[25] = 0;//(Hi8(Adapter->iv16) | 0x20) & 0x7F;
	//		pHdr->MacHeader[26] = Hi8(Adapter->iv16);
			pHdr->MacHeader[25] = Hi8(Adapter->iv16);
			pHdr->MacHeader[26] = 0;//(Hi8(Adapter->iv16) | 0x20) & 0x7F;
			pHdr->MacHeader[27] = Adapter->CardSetting.EncryKeyId | 0x20;



			pHdr->MacHeader[28] = (UCHAR)(Adapter->iv32);
			pHdr->MacHeader[29] = (UCHAR)(Adapter->iv32 >> 8);
			pHdr->MacHeader[30] = (UCHAR)(Adapter->iv32 >> 16);
			pHdr->MacHeader[31] = (UCHAR)(Adapter->iv32 >> 24);
		
			INITSTRWPA((_T("iv16 = %x\n"), Adapter->iv16));
			INITSTRWPA((_T("iv32 = %x\n"), Adapter->iv32));
		
			HdrLen = 32;
			break;	
		case 2:	// WEP 64, WEP128, wep256
		case 3:
		case 5:
			if (Adapter->CardSetting.WepIvType == RANDOM){
				srand( (UINT)Adapter->WepIv );
				Adapter->WepIv = (ULONG)rand();
			}else{
				Adapter->WepIv++;
			}
			pHdr->MacHeader[24] = (UCHAR)Adapter->WepIv;
			pHdr->MacHeader[25] = (UCHAR)(Adapter->WepIv >> 8);
			pHdr->MacHeader[26] = (UCHAR)(Adapter->WepIv >> 16);
			pHdr->MacHeader[27] = (Adapter->CardSetting.EncryKeyId << 6);
			HdrLen = 28;
			break;
		case 4: //WPA support (TKIP)
			INITSTRWPA((_T("TKIP EncryMode\n")));
			Adapter->iv16++;  
			if (Adapter->iv16 == 0) 
        		Adapter->iv32++;
			pHdr->MacHeader[24] = Hi8(Adapter->iv16); 
			pHdr->MacHeader[25] = (Hi8(Adapter->iv16) | 0x20) & 0x7F;
			pHdr->MacHeader[26] = Lo8(Adapter->iv16);
			pHdr->MacHeader[27] = Adapter->CardSetting.EncryKeyId | 0x20;

			pHdr->MacHeader[28] = (UCHAR)(Adapter->iv32);
			pHdr->MacHeader[29] = (UCHAR)(Adapter->iv32 >> 8);
			pHdr->MacHeader[30] = (UCHAR)(Adapter->iv32 >> 16);
			pHdr->MacHeader[31] = (UCHAR)(Adapter->iv32 >> 24);
		
			INITSTRWPA((_T("iv16 = %x\n"), Adapter->iv16));
			INITSTRWPA((_T("iv32 = %x\n"), Adapter->iv32));
		
			HdrLen = 32;
			break;	
		default:
			break;
		}
	}
	if (FrameType == PS_POLL){
		HdrLen = 16;
	}

	return(HdrLen);
}

__inline
BOOLEAN
IsMoreFrag(PZD1205_ADAPTER Adapter, 
		   PZD1205SwRfd SwRfd)
{
	PWLAN_DATA_FRM_HEADER_STRC	pWlanHdr;

	pWlanHdr = (PWLAN_DATA_FRM_HEADER_STRC)&SwRfd->Rfd->RxBuffer;
	if (pWlanHdr->FrameCtrl[1] & 0x4){
		return(TRUE);
	}
	else{
		return(FALSE);
	}
}

__inline
BOOLEAN
IsDataFrame(PZD1205_ADAPTER Adapter, 
			PZD1205SwRfd SwRfd)
{
	PWLAN_DATA_FRM_HEADER_STRC	pWlanHdr;

	pWlanHdr = (PWLAN_DATA_FRM_HEADER_STRC)&SwRfd->Rfd->RxBuffer;
	if ((pWlanHdr->FrameCtrl[0] & 0x0c) == 0x08){
		return(TRUE);
	}
	else{
		return(FALSE);
	}
}

__inline
BOOLEAN
IsMgtFrame(PZD1205_ADAPTER Adapter, 
			PZD1205SwRfd SwRfd)
{
	PWLAN_DATA_FRM_HEADER_STRC	pWlanHdr;

	pWlanHdr = (PWLAN_DATA_FRM_HEADER_STRC)&SwRfd->Rfd->RxBuffer;
	if ((pWlanHdr->FrameCtrl[0] & 0x0c) == 0x00){
		return(TRUE);
	}
	else{
		return(FALSE);
	}
}


__inline
WRITE_AES_KEY(PZD1205_ADAPTER Adapter)
{
	// Set Tx Key
	ZD1205_WRITE_REGISTER(Adapter,0x0E00, 0xFFFFFFFF);
	ZD1205_WRITE_REGISTER(Adapter,0x0E04, 0x0000FFFF);
	ZD1205_WRITE_REGISTER(Adapter,0x0E08, 0x00000000);
	ZD1205_WRITE_REGISTER(Adapter,0x0E0C, 0x01000000);
	ZD1205_WRITE_REGISTER(Adapter,0x0E10, 0x9C839C9D);
	ZD1205_WRITE_REGISTER(Adapter,0x0E14, 0x9C836362);
	ZD1205_WRITE_REGISTER(Adapter,0x0E18, 0x9C836362);
	ZD1205_WRITE_REGISTER(Adapter,0x0E1C, 0x9D836362);
	ZD1205_WRITE_REGISTER(Adapter,0x0E20, 0x36DD7064);
	ZD1205_WRITE_REGISTER(Adapter,0x0E24, 0xAA5E1306);
	ZD1205_WRITE_REGISTER(Adapter,0x0E28, 0x36DD7064);
	ZD1205_WRITE_REGISTER(Adapter,0x0E2C, 0xAB5E1306);
	ZD1205_WRITE_REGISTER(Adapter,0x0E30, 0x59BF281D);
	ZD1205_WRITE_REGISTER(Adapter,0x0E34, 0xF3E13B1B);
	ZD1205_WRITE_REGISTER(Adapter,0x0E38, 0xC53C4B7F);
	ZD1205_WRITE_REGISTER(Adapter,0x0E3C, 0x6E625879);
	ZD1205_WRITE_REGISTER(Adapter,0x0E40, 0xEF20827F);
	ZD1205_WRITE_REGISTER(Adapter,0x0E44, 0x1CC1B964);
	ZD1205_WRITE_REGISTER(Adapter,0x0E48, 0xD9FDF21B);
	ZD1205_WRITE_REGISTER(Adapter,0x0E4C, 0xB79FAA62);
	ZD1205_WRITE_REGISTER(Adapter,0x0E50, 0x458959C3);
	ZD1205_WRITE_REGISTER(Adapter,0x0E54, 0x5948E0A7);
	ZD1205_WRITE_REGISTER(Adapter,0x0E58, 0x80B512BC);
	ZD1205_WRITE_REGISTER(Adapter,0x0E5C, 0x372AB8DE);
	ZD1205_WRITE_REGISTER(Adapter,0x0E60, 0x5813BC8F);
	ZD1205_WRITE_REGISTER(Adapter,0x0E64, 0x015B5C28);
	ZD1205_WRITE_REGISTER(Adapter,0x0E68, 0x81EE4E94);
	ZD1205_WRITE_REGISTER(Adapter,0x0E6C, 0xB6C4F64A);
	ZD1205_WRITE_REGISTER(Adapter,0x0E70, 0x8E5DA08D);
	ZD1205_WRITE_REGISTER(Adapter,0x0E74, 0x8F06FCA5);
	ZD1205_WRITE_REGISTER(Adapter,0x0E78, 0x0EE8B231);
	ZD1205_WRITE_REGISTER(Adapter,0x0E7C, 0xB82C447B);
	ZD1205_WRITE_REGISTER(Adapter,0x0E80, 0xAF31D116);
	ZD1205_WRITE_REGISTER(Adapter,0x0E84, 0x20372DB3);
	ZD1205_WRITE_REGISTER(Adapter,0x0E88, 0x2EDF9F82);
	ZD1205_WRITE_REGISTER(Adapter,0x0E8C, 0x96F3DBF9);
	ZD1205_WRITE_REGISTER(Adapter,0x0E90, 0x36A1DCB4);
	ZD1205_WRITE_REGISTER(Adapter,0x0E94, 0x1696F107);
	ZD1205_WRITE_REGISTER(Adapter,0x0E98, 0x38496E85);
	ZD1205_WRITE_REGISTER(Adapter,0x0E9C, 0xAEBAB57C);
	ZD1205_WRITE_REGISTER(Adapter,0x0EA0, 0x26452857);
	ZD1205_WRITE_REGISTER(Adapter,0x0EA4, 0x30D3D950);
	ZD1205_WRITE_REGISTER(Adapter,0x0EA8, 0x089AB7D5);
	ZD1205_WRITE_REGISTER(Adapter,0x0EAC, 0xA62002A9);
	ZD1205_WRITE_REGISTER(Adapter,0x0EB0, 0x573F5C70);
	ZD1205_WRITE_REGISTER(Adapter,0x0EB4, 0xA9B4416A);
	ZD1205_WRITE_REGISTER(Adapter,0x0EB8, 0xBEA3F679);
	ZD1205_WRITE_REGISTER(Adapter,0x0EBC, 0x3365A7D1);
	ZD1205_WRITE_REGISTER(Adapter,0x0EC0, 0xAE7EB8E0);
	ZD1205_WRITE_REGISTER(Adapter,0x0EC4, 0x526983D4);
	ZD1205_WRITE_REGISTER(Adapter,0x0EC8, 0x7D47EDF3);
	ZD1205_WRITE_REGISTER(Adapter,0x0ECC, 0x66CA4EA3);
	ZD1205_WRITE_REGISTER(Adapter,0x0ED0, 0x5DFD70C1);
	ZD1205_WRITE_REGISTER(Adapter,0x0ED4, 0xA5D206A9);
	ZD1205_WRITE_REGISTER(Adapter,0x0ED8, 0xFB8EDAE7);
	ZD1205_WRITE_REGISTER(Adapter,0x0EDC, 0x4B949D46);
	ZD1205_WRITE_REGISTER(Adapter,0x0EE0, 0xBBFAE182);
	ZD1205_WRITE_REGISTER(Adapter,0x0EE4, 0x4BA50D52);
	ZD1205_WRITE_REGISTER(Adapter,0x0EE8, 0xF61DB5CF);
	ZD1205_WRITE_REGISTER(Adapter,0x0EEC, 0x11283B8D);
	ZD1205_WRITE_REGISTER(Adapter,0x0EF0, 0x76F5C305);
	ZD1205_WRITE_REGISTER(Adapter,0x0EF4, 0x974A1BA4);
	ZD1205_WRITE_REGISTER(Adapter,0x0EF8, 0xED3B6A9F);
	ZD1205_WRITE_REGISTER(Adapter,0x0EFC, 0xA550761A);
	ZD1205_WRITE_REGISTER(Adapter,0x0F00, 0xEDEA870B);
	ZD1205_WRITE_REGISTER(Adapter,0x0F04, 0x2F953648);
	ZD1205_WRITE_REGISTER(Adapter,0x0F08, 0xDA77D43E);
	ZD1205_WRITE_REGISTER(Adapter,0x0F0C, 0x4AA1EC34);
	ZD1205_WRITE_REGISTER(Adapter,0x0F10, 0xDAD50F17);
	ZD1205_WRITE_REGISTER(Adapter,0x0F14, 0x5E2A6D90);
	ZD1205_WRITE_REGISTER(Adapter,0x0F18, 0xB4EFA87D);
	ZD1205_WRITE_REGISTER(Adapter,0x0F1C, 0x9442D969);
	ZD1205_WRITE_REGISTER(Adapter,0x0F20, 0xB5AB1F2E);
	ZD1205_WRITE_REGISTER(Adapter,0x0F24, 0xBC54DA20);
	ZD1205_WRITE_REGISTER(Adapter,0x0F28, 0x68DF51FB);
	ZD1205_WRITE_REGISTER(Adapter,0x0F2C, 0x2885B2D3);
	ZD1205_WRITE_REGISTER(Adapter,0x0F30, 0xAB1F2EB8);
	ZD1205_WRITE_REGISTER(Adapter,0x0F34, 0x54DA20B5);
	ZD1205_WRITE_REGISTER(Adapter,0x0F38, 0xDF51FBBC);
	ZD1205_WRITE_REGISTER(Adapter,0x0F3C, 0xDAB2D368);
	ZD1205_WRITE_REGISTER(Adapter,0x0F40, 0x00000000);
	ZD1205_WRITE_REGISTER(Adapter,0x0F44, 0x00000000);
	ZD1205_WRITE_REGISTER(Adapter,0x0F48, 0x00000000);
	ZD1205_WRITE_REGISTER(Adapter,0x0F4C, 0x00000000);

	// Set Rx UniCast Key
	//1
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	//2
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	//3
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	//4
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	//5
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	//6
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	//7
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	//8
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00);

	// Set Rx multicast key
	//1
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xFFFFFFFF);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x0000FFFF);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00000000);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x01000000);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x9C839C9D);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x9C836362);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x9C836362);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x9D836362);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x36DD7064);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xAA5E1306);
	//2
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x36DD7064);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xAB5E1306);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x59BF281D);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xF3E13B1B);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xC53C4B7F);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x6E625879);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xEF20827F);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x1CC1B964);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xD9FDF21B);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xB79FAA62);
	//3
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x458959C3);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x5948E0A7);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x80B512BC);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x372AB8DE);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x5813BC8F);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x015B5C28);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x81EE4E94);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xB6C4F64A);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x8E5DA08D);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x8F06FCA5);
	//4
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x0EE8B231);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xB82C447B);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xAF31D116);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x20372DB3);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x2EDF9F82);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x96F3DBF9);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x36A1DCB4);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x1696F107);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x38496E85);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xAEBAB57C);
	//5
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x26452857);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x30D3D950);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x089AB7D5);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xA62002A9);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x573F5C70);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xA9B4416A);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xBEA3F679);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x3365A7D1);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xAE7EB8E0);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x526983D4);
	//6
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x7D47EDF3);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x66CA4EA3);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x5DFD70C1);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xA5D206A9);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xFB8EDAE7);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x4B949D46);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xBBFAE182);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x4BA50D52);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xF61DB5CF);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x11283B8D);
	//7
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x76F5C305);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x974A1BA4);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xED3B6A9F);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xA550761A);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xEDEA870B);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x2F953648);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xDA77D43E);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x4AA1EC34);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xDAD50F17);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x5E2A6D90);
	//8
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xB4EFA87D);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x9442D969);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xB5AB1F2E);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xBC54DA20);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x68DF51FB);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x2885B2D3);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xAB1F2EB8);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x54DA20B5);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xDF51FBBC);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0xDAB2D368);
	//
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00000000);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00000000);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00000000);
	ZD1205_WRITE_REGISTER(Adapter,0x6C0, 0x00000000);
}
#ifdef HOST_IF_USB
//-----------------------------------------------------------------------------
//JWEI 2004/02/09
#define fDISABLE_LED            0
__inline VOID iLED_ON(IN PZD1205_ADAPTER Adapter, ULONG LEDn)
{
    #if !fDISABLE_LED
ULONG   tmpvalue;
    if (Adapter->LED_Ctrl_Type ==1)
		return;//chi 2004/08/06

    ZD1205_READ_REGISTER(Adapter, rLED_CTRL, &tmpvalue);
    tmpvalue |= LEDn;
    ZD1205_WRITE_REGISTER(Adapter, rLED_CTRL, tmpvalue);
#ifdef ROBIN_KAO
    ZD1205_READ_REGISTER(Adapter, FW_LINK_STATUS, &tmpvalue);
    tmpvalue |= 0x1;
    ZD1205_WRITE_REGISTER(Adapter, FW_LINK_STATUS, tmpvalue);
#endif
    #endif
}

__inline VOID iLED_OFF(IN PZD1205_ADAPTER Adapter, ULONG LEDn)
{
    #if !fDISABLE_LED
ULONG   tmpvalue;
    if (Adapter->LED_Ctrl_Type ==1)
		return;//chi 2004/08/06

    ZD1205_READ_REGISTER(Adapter, rLED_CTRL, &tmpvalue);
    tmpvalue &= ~LEDn;
    ZD1205_WRITE_REGISTER(Adapter, rLED_CTRL, tmpvalue);
#ifdef ROBIN_KAO
	ZD1205_WRITE_REGISTER(Adapter, FW_LINK_STATUS, 0x0);
#endif
    #endif
}

//-------------------------------------------------------------
//RF_LED_ON   chi 2004/08/06-----------------------------------
//-------------------------------------------------------------
__inline VOID RF_LED_ON(IN PZD1205_ADAPTER Adapter)
{
	ULONG   tmpvalue;
    ZD1205_READ_REGISTER(Adapter, rLED_CTRL, &tmpvalue);
    tmpvalue |= LED2;
    ZD1205_WRITE_REGISTER(Adapter, rLED_CTRL, tmpvalue);
}

__inline VOID RF_LED_OFF(IN PZD1205_ADAPTER Adapter)
{
	ULONG   tmpvalue;
    ZD1205_READ_REGISTER(Adapter, rLED_CTRL, &tmpvalue);
    tmpvalue &= ~LED2;
    ZD1205_WRITE_REGISTER(Adapter, rLED_CTRL, tmpvalue);
}
//-------------------------------------------------------------

__inline VOID iLED_SWITCH(IN PZD1205_ADAPTER Adapter, ULONG LEDn)
{
    #if !fDISABLE_LED
ULONG   tmpvalue;

    ZD1205_READ_REGISTER(Adapter, rLED_CTRL, &tmpvalue);
    tmpvalue ^= LEDn;
    ZD1205_WRITE_REGISTER(Adapter, rLED_CTRL, tmpvalue);
    #endif
}

#else

__inline VOID iLED_ON(IN PZD1205_ADAPTER Adapter, ULONG LEDn)
{
    ZD1205_WRITE_REGISTER(Adapter, LEDn, 0x1);
}

__inline VOID iLED_OFF(IN PZD1205_ADAPTER Adapter, ULONG LEDn)
{
    ZD1205_WRITE_REGISTER(Adapter, LEDn, 0x0);
}

__inline VOID iLED_SWITCH(IN PZD1205_ADAPTER Adapter, ULONG LEDn)
{
ULONG   tmpvalue;

    ZD1205_READ_REGISTER(Adapter, LEDn, &tmpvalue);
    tmpvalue ^= 0x1;
    ZD1205_WRITE_REGISTER(Adapter, LEDn, tmpvalue);
}

#endif

