/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    manage.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

BOOLEAN
Authenticate_OpenSystem(IN PZD1205_ADAPTER Adapter)
{
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	USHORT						TypeLength;
	NDIS_STATUS					Status;
	PZD1205_DEFRAG_ENTITY		DefragEntityPtr = NULL;
	PZD1205SwRfd				SwRfd;
	PWLAN_AUTHENTICATE_FRM_STRUC	pAthFrm;
	ULONGLONG					Timer1;
	ZD1211_IRQL				OldIrql;
	DEBUGFUNC(_T("Authenticate_OpenSystem"));
	INITSTR((_T("\n")));

	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

	// Destination address
	GetEnetFrameLock(Adapter);
	Adapter->EnetFrame[0] = Adapter->BSSID[0];
	Adapter->EnetFrame[1] = Adapter->BSSID[1];
	Adapter->EnetFrame[2] = Adapter->BSSID[2];
	Adapter->EnetFrame[3] = Adapter->BSSID[3];
	Adapter->EnetFrame[4] = Adapter->BSSID[4];
	Adapter->EnetFrame[5] = Adapter->BSSID[5];

	// Source address
	Adapter->EnetFrame[6] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[7] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[8] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[9] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[10] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[11] = Adapter->AiNodeAddress[5];

	// TypeLength
	TypeLength = 0x06;
	Adapter->EnetFrame[12] = (UCHAR)(TypeLength >> 8); // For open-system
	Adapter->EnetFrame[13] = (UCHAR)(TypeLength);

	// Authentication algorithm number
	Adapter->EnetFrame[14] = 0x00;
	Adapter->EnetFrame[15] = 0x00;

	// Authentication transaction sequence number
	Adapter->EnetFrame[16] = 0x01;
	Adapter->EnetFrame[17] = 0x00;

	// Status code
	Adapter->EnetFrame[18] = 0x00;
	Adapter->EnetFrame[19] = 0x00;

	// Form a packet for authentication frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	while ( (Status = SetupNextSend(Adapter, Packet, AUTHENTICATE, NO_CIPHER, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql); // when exit
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return TRUE;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

#ifdef CE_ALL
    Timer1 = GetTickCount();
	while(1){
		// Check if time expired
		if (abs((LONG)(GetTickCount()-Timer1)) > 3000){ // 3s
#else
    Timer1 = KeQueryInterruptTime();
	while(1){
		// Check if time expired
		if (abs((LONG)(KeQueryInterruptTime()-Timer1)) > 30000000){ // 3s
#endif
			INITSTR((_T("Authentication fail timeout\n")));
			break;
		}
		if(! Mgt_RetrieveQueue(Adapter, &DefragEntityPtr)){
			// Sleep 1ms
			ZD1205_Sleep(1000);
			continue;
		}
		// We have retrieved a mgt frame
		if (QueueGetCount(&DefragEntityPtr->ElementList) > 1){
			// Make sure no fragmentation
			GetReturnedMgtFrame(Adapter, DefragEntityPtr);
			continue;
		}
		ASSERT(!QueueEmpty(&DefragEntityPtr->ElementList));
		SwRfd = (PZD1205SwRfd)QueueGetHead(
			&DefragEntityPtr->ElementList);
		pAthFrm = (PWLAN_AUTHENTICATE_FRM_STRUC)&SwRfd->Rfd->RxBuffer;
		if ( (pAthFrm->FrameCtrl[0] & 0xfc) == 0xb0 ){ // Authentication frame
			if ((pAthFrm->Trans_SeqNo[0] == 2) && 
				(pAthFrm->Trans_SeqNo[1] == 0) &&
				(pAthFrm->Status[0] == 0) &&
				(pAthFrm->Status[1] == 0) ){
				INITSTR((_T("Authenticate successfully\n")));
				GetReturnedMgtFrame(Adapter, DefragEntityPtr);
				return TRUE;
			}
		}
		GetReturnedMgtFrame(Adapter, DefragEntityPtr);
	}

	INITSTR((_T("Authentication fail\n")));
	return FALSE;
}

BOOLEAN
Authenticate_SharedKey(IN PZD1205_ADAPTER Adapter)
{
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	USHORT						TypeLength;
	NDIS_STATUS					Status;
	PZD1205_DEFRAG_ENTITY		DefragEntityPtr = NULL;
	PZD1205SwRfd				SwRfd;
	PWLAN_AUTHENTICATE_FRM_STRUC	pAthFrm;
	ULONGLONG					Timer1;
	BOOLEAN						AuthenStatus = FALSE;
	USHORT						i;
	ZD1211_IRQL				OldIrql;
	DEBUGFUNC(_T("Authenticate_SharedKey"));

	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

	// Destination address
	GetEnetFrameLock(Adapter);
	Adapter->EnetFrame[0] = Adapter->BSSID[0];
	Adapter->EnetFrame[1] = Adapter->BSSID[1];
	Adapter->EnetFrame[2] = Adapter->BSSID[2];
	Adapter->EnetFrame[3] = Adapter->BSSID[3];
	Adapter->EnetFrame[4] = Adapter->BSSID[4];
	Adapter->EnetFrame[5] = Adapter->BSSID[5];

	// Source address
	Adapter->EnetFrame[6] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[7] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[8] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[9] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[10] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[11] = Adapter->AiNodeAddress[5];

	// TypeLength
	TypeLength = 0x06;
	Adapter->EnetFrame[12] = (UCHAR)(TypeLength >> 8); // For open-system
	Adapter->EnetFrame[13] = (UCHAR)(TypeLength);

	// Authentication algorithm number
	Adapter->EnetFrame[14] = 0x01;
	Adapter->EnetFrame[15] = 0x00;

	// Authentication transaction sequence number
	Adapter->EnetFrame[16] = 0x01;
	Adapter->EnetFrame[17] = 0x00;

	// Status code
	Adapter->EnetFrame[18] = 0x00;
	Adapter->EnetFrame[19] = 0x00;

	// Form a packet for authentication frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	while ( (Status = SetupNextSend(Adapter, Packet, AUTHENTICATE, NO_CIPHER, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return TRUE;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);


#ifdef CE_ALL
    Timer1 = GetTickCount();
	while(1){
		// Check if time expired
		if (abs((LONG)(GetTickCount()-Timer1)) > 3000){ // 3s
#else
    Timer1 = KeQueryInterruptTime();
	while(1){
		// Check if time expired
		if (abs((LONG)(KeQueryInterruptTime()-Timer1)) > 30000000){ // 3s
#endif
			break;
		}

		if(! Mgt_RetrieveQueue(Adapter, &DefragEntityPtr)){
			// Sleep 1ms
			ZD1205_Sleep(1000);
			continue;
		}
		// We have retrieved a mgt frame
		if (QueueGetCount(&DefragEntityPtr->ElementList) > 1){
			// Make sure no fragmentation
			GetReturnedMgtFrame(Adapter, DefragEntityPtr);
			continue;
		}
		ASSERT(!QueueEmpty(&DefragEntityPtr->ElementList));
		SwRfd = (PZD1205SwRfd)QueueGetHead(
			&DefragEntityPtr->ElementList);
		pAthFrm = (PWLAN_AUTHENTICATE_FRM_STRUC)&SwRfd->Rfd->RxBuffer;
		if ( (pAthFrm->FrameCtrl[0] & 0xfc) == 0xb0 ){ // Authentication frame
			if ((pAthFrm->Trans_SeqNo[0] == 2) && 
				(pAthFrm->Trans_SeqNo[1] == 0) &&
				(pAthFrm->Status[0] == 0) &&
				(pAthFrm->Status[1] == 0) &&
				(pAthFrm->Al_Num[0] == 1) &&
				(pAthFrm->Al_Num[1] == 0)){
				INITSTR((_T("Authenticate_SharedKey 2nd frame successfully\n")));
				// Copy th challenge text

				// TypeLength
				TypeLength = 136; 
				Adapter->EnetFrame[12] = (UCHAR)(TypeLength >> 8); 
				Adapter->EnetFrame[13] = (UCHAR)(TypeLength);
				
				// Authentication algorithm number
				Adapter->EnetFrame[14] = 0x01;
				Adapter->EnetFrame[15] = 0x00;
				
				// Authentication transaction sequence number
				Adapter->EnetFrame[16] = 0x03;
				Adapter->EnetFrame[17] = 0x00;
				
				// Status code
				Adapter->EnetFrame[18] = 0x00;
				Adapter->EnetFrame[19] = 0x00;

				for (i=0;i<130;i++){	
					Adapter->EnetFrame[20+i] = pAthFrm->Challenge[i];
				}
				GetReturnedMgtFrame(Adapter, DefragEntityPtr);
				AuthenStatus = TRUE;
				break;
			}
		}
		GetReturnedMgtFrame(Adapter, DefragEntityPtr);
	}

	if (!AuthenStatus){
		INITSTR((_T("Authentication_SharedKey fail\n")));
		return FALSE;
	}

	AuthenStatus = FALSE;
	// Form a packet for authentication frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	while ( (Status = SetupNextSend(Adapter, Packet, AUTHENTICATE, 
						Adapter->CardSetting.EncryMode, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return TRUE;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

#ifdef CE_ALL
    Timer1 = GetTickCount();
	while(1){
		// Check if time expired
		if (abs((LONG)(GetTickCount()-Timer1)) > 3000){ // 3s
#else
    Timer1 = KeQueryInterruptTime();
	while(1){
		// Check if time expired
		if (abs((LONG)(KeQueryInterruptTime()-Timer1)) > 30000000){ // 3s
#endif
			INITSTR((_T("Authentication_SharedKey fail timeout\n")));
			break;
		}

		if(! Mgt_RetrieveQueue(Adapter, &DefragEntityPtr)){
			// Sleep 1ms
			ZD1205_Sleep(1000);
			continue;
		}
		// We have retrieved a mgt frame
		if (QueueGetCount(&DefragEntityPtr->ElementList) > 1){
			// Make sure no fragmentation
			GetReturnedMgtFrame(Adapter, DefragEntityPtr);
			continue;
		}
		ASSERT(!QueueEmpty(&DefragEntityPtr->ElementList));
		SwRfd = (PZD1205SwRfd)QueueGetHead(
			&DefragEntityPtr->ElementList);
		pAthFrm = (PWLAN_AUTHENTICATE_FRM_STRUC)&SwRfd->Rfd->RxBuffer;
		if ( (pAthFrm->FrameCtrl[0] & 0xfc) == 0xb0 ){ // Authentication frame
			if ((pAthFrm->Trans_SeqNo[0] == 4) && 
				(pAthFrm->Trans_SeqNo[1] == 0) &&
				(pAthFrm->Status[0] == 0) &&
				(pAthFrm->Status[1] == 0) &&
				(pAthFrm->Al_Num[0] == 1) &&
				(pAthFrm->Al_Num[1] == 0)){
				INITSTR((_T("Authenticate_SharedKey 4th frame successfully\n")));
				GetReturnedMgtFrame(Adapter, DefragEntityPtr);
				AuthenStatus = TRUE;
				break;
			}
		}
		GetReturnedMgtFrame(Adapter, DefragEntityPtr);
	}
	if (!AuthenStatus){
		INITSTR((_T("Authentication_SharedKey fail\n")));
		return FALSE;
	}
	return TRUE;
}

BOOLEAN
Associate(IN PZD1205_ADAPTER Adapter)
{
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	USHORT						TypeLength;
	NDIS_STATUS					Status;
	PZD1205_DEFRAG_ENTITY		DefragEntityPtr = NULL;
	PZD1205SwRfd				SwRfd;
	USHORT						i;
	USHORT						index = 0;
	PWLAN_ASSOCIATE_RSP_FRM_STRUC	pAssRspFrm;
	ULONGLONG					Timer1;
	ZD1211_IRQL				OldIrql;
	DEBUGFUNC(_T("Associate"));
	INITSTR((_T("\n")));

	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

    //UpdateSupportedRates(Adapter, &Adapter->EnetFrame[index-6]);

	GetEnetFrameLock(Adapter);
	// Destination address
	Adapter->EnetFrame[index++] = Adapter->BSSID[0];
	Adapter->EnetFrame[index++] = Adapter->BSSID[1];
	Adapter->EnetFrame[index++] = Adapter->BSSID[2];
	Adapter->EnetFrame[index++] = Adapter->BSSID[3];
	Adapter->EnetFrame[index++] = Adapter->BSSID[4];
	Adapter->EnetFrame[index++] = Adapter->BSSID[5];

	// Source address
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[5];

	// TypeLength
	TypeLength = 2+2+Adapter->CardSetting.Info_SSID[1]+2+
				Adapter->CardSetting.Info_SupportedRates[1]+2;
#ifdef OFDM
	if (Adapter->CardSetting.Info_ExtSupportedRates[1]){
		TypeLength += (Adapter->CardSetting.Info_ExtSupportedRates[1] + 2);
	}
#endif
	if ( Adapter->CardSetting.WPASupport && 
		 (
			(Adapter->CardSetting.AuthMode == Ndis802_11AuthModeWPA) 
				||
			(Adapter->CardSetting.AuthMode == Ndis802_11AuthModeWPAPSK)
		  )
		  ){
        TypeLength += Adapter->CardSetting.IEtotal;
	}
	Adapter->EnetFrame[index++] = (UCHAR)(TypeLength >> 8); // For open-system
	Adapter->EnetFrame[index++] = (UCHAR)(TypeLength);

	// Capability information
	Adapter->EnetFrame[index++] = (UCHAR)Adapter->CardSetting.Capability;
	Adapter->EnetFrame[index++] = (UCHAR)(Adapter->CardSetting.Capability >> 8);

	// Listen Interval
	Adapter->EnetFrame[index++] = (UCHAR)Adapter->ListenInterval;
	Adapter->EnetFrame[index++] = (UCHAR)(Adapter->ListenInterval >> 8);

	// SSID
	for (i=0;i<(Adapter->CardSetting.Info_SSID[1]+2);i++){
		Adapter->EnetFrame[index++] = Adapter->CardSetting.Info_SSID[i];
	}

	// Supported rates
	for (i=0;i<(Adapter->CardSetting.Info_SupportedRates[1]+2);i++){
		Adapter->EnetFrame[index++] = Adapter->CardSetting.Info_SupportedRates[i];
	}
#ifdef OFDM
	//Extend Supported rates
	if (Adapter->CardSetting.Info_ExtSupportedRates[1]){
		for (i=0;i<(Adapter->CardSetting.Info_ExtSupportedRates[1]+2);i++){
			Adapter->EnetFrame[index++] = Adapter->CardSetting.Info_ExtSupportedRates[i];
		}
	}
#endif
	
	INITSTRWPA((_T("WPA Support = %d\n"), Adapter->CardSetting.WPASupport));
	if ( Adapter->CardSetting.WPASupport && 
		 (
			(Adapter->CardSetting.AuthMode == Ndis802_11AuthModeWPA) 
				||
			(Adapter->CardSetting.AuthMode == Ndis802_11AuthModeWPAPSK)
		  )
		  ){
        for (i=0; i<Adapter->CardSetting.IEtotal; i++)
		    Adapter->EnetFrame[index++] = Adapter->CardSetting.wpaIEs[i];
	}

	// Form a packet for authentication frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	while ( (Status = SetupNextSend(Adapter, Packet, ASSOCIATE, NO_CIPHER, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return TRUE;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);


#ifdef CE_ALL
    Timer1 = GetTickCount();
	while(1){
		// Check if time expired
		if (abs((LONG)(GetTickCount()-Timer1)) > 3000){ // 3s
#else
    Timer1 = KeQueryInterruptTime();
	while(1){
		// Check if time expired
		if (abs((LONG)(KeQueryInterruptTime()-Timer1)) > 30000000){ // 3s
#endif
			INITSTR((_T("Associate timeout\n")));
			break;
		}

		if(! Mgt_RetrieveQueue(Adapter, &DefragEntityPtr)){
			// Sleep 1ms
			ZD1205_Sleep(1000);
			continue;
		}
		// We have retrieved a mgt frame
		if (QueueGetCount(&DefragEntityPtr->ElementList) > 1){
			// Make sure no fragmentation
			GetReturnedMgtFrame(Adapter, DefragEntityPtr);
			continue;
		}
		ASSERT(!QueueEmpty(&DefragEntityPtr->ElementList));
		SwRfd = (PZD1205SwRfd)QueueGetHead(
			&DefragEntityPtr->ElementList);
		pAssRspFrm = (PWLAN_ASSOCIATE_RSP_FRM_STRUC)&SwRfd->Rfd->RxBuffer;
		if ( (pAssRspFrm->FrameCtrl[0] & 0xfc) == 0x10 ){ // Association response frame
			if ((pAssRspFrm->Status[0] == 0) &&
				(pAssRspFrm->Status[1] == 0) ){
				INITSTR((_T("Associate successfully\n")));
				if (!Adapter->Infra_Process_Associate) {
				  Adapter->AID = (((USHORT)(pAssRspFrm->AID[1] & 0x3f)) << 8) +
								  pAssRspFrm->AID[0];
				  INITSTR((_T("Our AID is %x\n"), Adapter->AID));
				  Adapter->PkInstalled = 0; // 3808 by yarco
                }
				GetReturnedMgtFrame(Adapter, DefragEntityPtr);
				return TRUE;
			}
			INITSTR((_T("status code = 0x%x\n"), (pAssRspFrm->Status[1]<<8)+pAssRspFrm->Status[0]));
		}
		GetReturnedMgtFrame(Adapter, DefragEntityPtr);
	}

	INITSTR((_T("Associate fail\n")));
	return FALSE;
}


BOOLEAN
Deauthenticate(IN PZD1205_ADAPTER Adapter)
{
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	USHORT						TypeLength;
	NDIS_STATUS					Status;
	ZD1211_IRQL					OldIrql;

	DEBUGFUNC(_T("DeAuthenticate"));

	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

	// Destination address
	Adapter->EnetFrame[0] = Adapter->BSSID[0];
	Adapter->EnetFrame[1] = Adapter->BSSID[1];
	Adapter->EnetFrame[2] = Adapter->BSSID[2];
	Adapter->EnetFrame[3] = Adapter->BSSID[3];
	Adapter->EnetFrame[4] = Adapter->BSSID[4];
	Adapter->EnetFrame[5] = Adapter->BSSID[5];

	// Source address
	Adapter->EnetFrame[6] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[7] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[8] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[9] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[10] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[11] = Adapter->AiNodeAddress[5];

	// TypeLength
	TypeLength = 0x02;
	Adapter->EnetFrame[12] = (UCHAR)(TypeLength >> 8); // For open-system
	Adapter->EnetFrame[13] = (UCHAR)(TypeLength);

	// Reason code
	Adapter->EnetFrame[14] = Adapter->ReasonCode % 256;
	Adapter->EnetFrame[15] = Adapter->ReasonCode >> 8;

	// Form a packet for authentication frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	while ( (Status = SetupNextSend(Adapter, Packet, DEAUTHENTICATION, NO_CIPHER, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return TRUE;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

	return TRUE;
}

BOOLEAN
isSupportRate(UCHAR rate)
{
    UCHAR tmp;

	tmp = rate & 0x7f;

	switch(tmp)
	{
	    case SR_1M:
	    case SR_2M:
	    case SR_5_5M:
	    case SR_11M:
	    case SR_6M:
	    case SR_9M:
	    case SR_12M:
	    case SR_18M:
	    case SR_24M:
	    case SR_36M:
	    case SR_48M:
	    case SR_54M:
			return TRUE;
        default:
			return FALSE;
	}
}

#define		ELEMENT_ID_SSID		0
BOOLEAN
ProbeRequest(IN PZD1205_ADAPTER Adapter)
{
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	USHORT						TypeLength;
	NDIS_STATUS					Status;
	PZD1205_DEFRAG_ENTITY		DefragEntityPtr = NULL;
	PZD1205SwRfd				SwRfd;
	USHORT						i;
	USHORT						j;
	USHORT						k;
	USHORT						index = 0;
	ULONGLONG					Timer1;
	PWLAN_MANAGEMENT_FRM_HEADER_STRUC	pMgtFrm;
	BOOLEAN						bssidmatched;
	PNDIS_802_11_VARIABLE_IEs   pVarIe;
	USHORT                      IEnow=0, start;
	UCHAR   oui[4] = { 0, 0x50, 0xf2, 0x01 };
	UCHAR   superG[3] = { 0, 0x03, 0x7f };
	ZD1211_IRQL					OldIrql;
	UCHAR                       length;

	DEBUGFUNC(_T("ProbeRequest"));
	//INITSTR((_T("\n")));

	// Clear BSSInfo data base
	//Adapter->BSSCnt = 0;


	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

	GetEnetFrameLock(Adapter);
	// Destination address
	Adapter->EnetFrame[index++] = 0xff;
	Adapter->EnetFrame[index++] = 0xff;
	Adapter->EnetFrame[index++] = 0xff;
	Adapter->EnetFrame[index++] = 0xff;
	Adapter->EnetFrame[index++] = 0xff;
	Adapter->EnetFrame[index++] = 0xff;

	// Source address
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[5];

	// TypeLength
	if ( Adapter->ProbeReqWithSSID )
	    //TypeLength = 2+Adapter->CardSetting.Info_SSID[1] + (Adapter->CardSetting.Info_SupportedRates[1]+2); // Broadcast_SSID(2) + supportedRates(6)
	    TypeLength = 2+Adapter->CardSetting.Info_SSID[1] + 6; // 2 + SSID_len + supportedRates(6)
    else
		//TypeLength = 2+(Adapter->CardSetting.Info_SupportedRates[1]+2); // Broadcast_SSID(2) + supportedRates(6)
		TypeLength = 2+6; // Broadcast_SSID(2) + supportedRates(6)
#ifdef OFDM
	// We send probe request with fix format
	TypeLength += 10;	// add ext support rate
#endif
	Adapter->EnetFrame[index++] = (UCHAR)(TypeLength >> 8); 
	Adapter->EnetFrame[index++] = (UCHAR)(TypeLength);

	if ( Adapter->ProbeReqWithSSID ){
        length = Adapter->CardSetting.Info_SSID[1] + 2;
		memcpy(&Adapter->EnetFrame[index], Adapter->CardSetting.Info_SSID, length);
		index += length;
	}else{  // Broadcast SSID
	    Adapter->EnetFrame[index++] = ELEMENT_ID_SSID;
	    Adapter->EnetFrame[index++] = 0; // Length 0 -- means broadcast SSID
	}


	Adapter->EnetFrame[index++] = 0x01;
	Adapter->EnetFrame[index++] = 0x04;
	Adapter->EnetFrame[index++] = 0x82;
	Adapter->EnetFrame[index++] = 0x84;
	Adapter->EnetFrame[index++] = 0x0b;
	Adapter->EnetFrame[index++] = 0x16;
#ifdef OFDM
	Adapter->EnetFrame[index++] = 0x32;
	Adapter->EnetFrame[index++] = 0x08;
	Adapter->EnetFrame[index++] = 0x0C;
	Adapter->EnetFrame[index++] = 0x12;
	Adapter->EnetFrame[index++] = 0x18;
	Adapter->EnetFrame[index++] = 0x24;
	Adapter->EnetFrame[index++] = 0x30;
	Adapter->EnetFrame[index++] = 0x48;
	Adapter->EnetFrame[index++] = 0x60;
	Adapter->EnetFrame[index++] = 0x6c;
#endif

	// Form a packet for ProbeRequest frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

	// for halting the device
	if ( Adapter->DisableTx ){  
	    //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		NdisFreeBuffer(Buffer);
		NdisFreePacket(Packet);
	    return TRUE;
    }

	while ( (Status = SetupNextSend(Adapter, Packet, PROBEREQUEST, NO_CIPHER, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return TRUE;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

#ifdef CE_ALL
    Timer1 = GetTickCount();
	while(1){
		// Check if time expired 
		// Change wait-time to 40ms for ZCT compatibility.
		IEnow = 0;
		if (abs((LONG)(GetTickCount()-Timer1)) > 40){ // 40ms
#else
    Timer1 = KeQueryInterruptTime();
	while(1){
		// Check if time expired 
		// Change wait-time to 40ms for ZCT compatibility.
		IEnow = 0;
		if (abs((LONG)(KeQueryInterruptTime()-Timer1)) > 1000000){ // 40ms
#endif
			break;
		}

		if(! Mgt_RetrieveQueue(Adapter, &DefragEntityPtr)){
			// Sleep 1ms
			ZD1205_Sleep(1000);
			continue;
		}
		// We have retrieved a mgt frame
		// Make sure no fragmentation
		if (QueueGetCount(&DefragEntityPtr->ElementList) > 1){
			GetReturnedMgtFrame(Adapter, DefragEntityPtr);
			continue;
		}

		ASSERT(!QueueEmpty(&DefragEntityPtr->ElementList));
		SwRfd = (PZD1205SwRfd)QueueGetHead(
							&DefragEntityPtr->ElementList);
		pMgtFrm = (PWLAN_MANAGEMENT_FRM_HEADER_STRUC)&SwRfd->Rfd->RxBuffer;
		// Check if Beacon / Probe response
		if (!( ((pMgtFrm->FrameCtrl[0] & 0xfc) == 0x80) || 
			((pMgtFrm->FrameCtrl[0] & 0xfc) == 0x50) )){ // Beacon or Probe rsp{
			//INITSTR((_T("call GetReturnedMgtFrame\n")));
			GetReturnedMgtFrame(Adapter, DefragEntityPtr);
			continue;
		}

		// Check if WEP enabled
		if (pMgtFrm->FrameCtrl[1] & BIT_6){
			GetReturnedMgtFrame(Adapter, DefragEntityPtr);
			continue;
		}

		// Check if this BSS had been stored
		bssidmatched = FALSE;
		for (k=0;k<Adapter->BssIndex;k++){
			for (j=0;j<6;j++){
				if (Adapter->BSSInfo[k].BSSID[j] != pMgtFrm->BSSID[j]){
					break;
				}
			}
			if (j==6){
				bssidmatched = TRUE;
				break;
			}
		}
		
		if (bssidmatched == TRUE){
			if (Adapter->BSSInfo[k].Channel != Adapter->CardSetting.Channel){
				GetReturnedMgtFrame(Adapter, DefragEntityPtr);
				continue;
			}
			// Update Correct Signal Quality
		    Adapter->BSSInfo[k].SignalQuality = CalculateQuality(Adapter, SwRfd, 0);
			//++
			if ((pMgtFrm->FrameCtrl[0] & 0xfc) != 0x50){
				GetReturnedMgtFrame(Adapter, DefragEntityPtr);
				continue;
			}
		}

		NdisZeroMemory(&(Adapter->BSSInfo[k]), sizeof(BSS_INFO));

        // initialize WPASupport
        Adapter->BSSInfo[k].WPASupport = FALSE;

		// Mark OnUpdating
		Adapter->BSSInfo[k].OnUpdating = TRUE;

		// Get BSSID
		for (i=0;i<6;i++){
			Adapter->BSSInfo[k].BSSID[i] = pMgtFrm->BSSID[i];
		}

		// Get TimeStamp
		for (i=0;i<8;i++){
			Adapter->BSSInfo[k].TimeStamp[i] = 
				SwRfd->Rfd->RxBuffer[BCNINTERVAL_OFFSET+i-8];
		}

		// Get BeaconInterval
		for (i=0;i<2;i++){
			Adapter->BSSInfo[k].BeaconInterval[i] = 
				SwRfd->Rfd->RxBuffer[BCNINTERVAL_OFFSET+i];
		}
		// Get Capability
		for (i=0;i<2;i++){
			Adapter->BSSInfo[k].Capability[i] = 
				SwRfd->Rfd->RxBuffer[BCNINTERVAL_OFFSET+2+i];
		}

		//++ Reset Information Elements
		Adapter->BSSInfo[k].CountryInfo[1] = 0;
		//--
		index = BCNINTERVAL_OFFSET+4;
		while((index+1+SwRfd->Rfd->RxBuffer[index+1])
				< (USHORT)(SwRfd->Rfd->ActualCount-CRC32_SIZE-EXT_INFO_SIZE) ){ // Minus CRC32(4), EXTRINFO(4)            
			switch(SwRfd->Rfd->RxBuffer[index]){
				case 0:	// SSID
                   // ZDDbgPrint((_T("SSID = \n")));
					if ((SwRfd->Rfd->RxBuffer[index+1]+2) <= MAX_SSID_LEN){
						if (SwRfd->Rfd->RxBuffer[index+1] > 0){ // Workaround with Zero-
																// padding problem.
																// D-Link 7500 AP.
							for (i=0;i<(SwRfd->Rfd->RxBuffer[index+1]+2);i++){
								Adapter->BSSInfo[k].SSID[i] =
									SwRfd->Rfd->RxBuffer[index+i];
								//if ( i>1 )
								    //ZDDbgPrint((_T("%c "), SwRfd->Rfd->RxBuffer[index+i]));
							}
						}
					}
                    //ZDDbgPrint((_T("\n")));
					break;
				case 1: // Supported Rates
					j = 0;
					for (i=0;i<(SwRfd->Rfd->RxBuffer[index+1]+2);i++){
						if (i<NUM_SUPPORTED_RATE){
							if ( (i<2)||(isSupportRate(SwRfd->Rfd->RxBuffer[index+i])) )
							    Adapter->BSSInfo[k].SupportedRates[j++] 
								             = SwRfd->Rfd->RxBuffer[index+i];
							else
								Adapter->BSSInfo[k].SupportedRates[1]--;
						}
					}
					//ZDDbgPrint((_T("# of rates = %d\n"), Adapter->BSSInfo[k].SupportedRates[1]));
					break;
				case 2: // FH parameter
					break;
				case 3: // DS parameter
					Adapter->BSSInfo[k].Channel = 
						SwRfd->Rfd->RxBuffer[index+2];
					break;
				case 4: // CF parameter
					if ((SwRfd->Rfd->RxBuffer[index+1]+2) <= MAX_CFPar_SIZE){
						for (i=0;i<(SwRfd->Rfd->RxBuffer[index+1]+2);i++){
							Adapter->BSSInfo[k].CFPar[i] =
								SwRfd->Rfd->RxBuffer[index+i];
						}
					}
					break;
				case 5: // TIM
					break;
				case 6: // IBSS parameter	
					for (i=0;i<2;i++){
						Adapter->BSSInfo[k].ATIMWindow[i] = 
							SwRfd->Rfd->RxBuffer[index+2+i];
					}
					break;
				case 7:	// Country Information Element
					if ((SwRfd->Rfd->RxBuffer[index+1]+2) <= MAX_COUNTRY_INFO_SIZE){
						for (i=0;i<(SwRfd->Rfd->RxBuffer[index+1]+2);i++){
							Adapter->BSSInfo[k].CountryInfo[i] =
								SwRfd->Rfd->RxBuffer[index+i];
						}
					}
					break;
#if (defined(GCCK) && defined(OFDM))
				case 42:
					Adapter->BSSInfo[k].ERP_Info = 
						SwRfd->Rfd->RxBuffer[index+2];
					break;
				case 50:
					j = 0;
					for (i=0;i<(SwRfd->Rfd->RxBuffer[index+1]+2);i++){
						if (i<NUM_SUPPORTED_RATE){
							if ( (i<2)||(isSupportRate(SwRfd->Rfd->RxBuffer[index+i])) )
							    Adapter->BSSInfo[k].ExtSupportedRates[j++] 
								      = SwRfd->Rfd->RxBuffer[index+i];
							else
								Adapter->BSSInfo[k].ExtSupportedRates[1]--;
						}
					}
					break;
#endif
				case 0xDD:
					ZDDbgPrint((_T("WPA IE = \n")));
					if ((SwRfd->Rfd->RxBuffer[index+1]+2+IEnow) <= MAX_IEs_SIZE){
						start = IEnow;
						// bypass the vendor specific IE of Atheros SuperG 
						//if ( !memcmp(&SwRfd->Rfd->RxBuffer[index+2], superG, 3) )
						if ( memcmp(&SwRfd->Rfd->RxBuffer[index+2], oui, 4) )
				            break;

						for (i=0;i<(SwRfd->Rfd->RxBuffer[index+1]+2);i++){
							Adapter->BSSInfo[k].wpaIEs[start+i] =
								SwRfd->Rfd->RxBuffer[index+i];
							IEnow++;
							ZDDbgPrint((_T("%2x "), SwRfd->Rfd->RxBuffer[index+i]));
							if ( (i+1)%16 == 0 )
								ZDDbgPrint((_T("\n")));
						}
                        ZDDbgPrint((_T("\n")));
						pVarIe = (PNDIS_802_11_VARIABLE_IEs) (&Adapter->BSSInfo[k].wpaIEs[start]);
						if ( pVarIe->Length>=6 && (memcmp(pVarIe->data, oui, 4)==0) ){
							Adapter->BSSInfo[k].WPASupport = TRUE;
							Adapter->BSSInfo[k].IEstart = start;
                            ZDDbgPrint((_T("IEstart = %d\n"), start));
						}
					    Adapter->BSSInfo[k].IEtotal = IEnow;
					}
					break;
				default:
					break;
			}
			index += (SwRfd->Rfd->RxBuffer[index+1]+2);
		}

		// Get Signal Quality
		Adapter->BSSInfo[k].SignalQuality = CalculateQuality(Adapter, SwRfd, 0);

		// Get Signal Strength
		Adapter->BSSInfo[k].SignalStrength = CalculateStrength(Adapter, SwRfd);
				
		// UnMark OnUpdating
		Adapter->BSSInfo[k].OnUpdating = FALSE;

		if ((k==Adapter->BssIndex) && (k<(BSS_INFO_NUM-1))){

			if (!((1 << (Adapter->BSSInfo[k].Channel-1)) & Adapter->AllowedChannel)){
				// This channel not allowed
				NdisZeroMemory(&(Adapter->BSSInfo[k]), sizeof(BSS_INFO));
			}
			else{
				// this channel allowed
				Adapter->BssIndex ++;
			}
		}
		// end of retrieve information

		GetReturnedMgtFrame(Adapter, DefragEntityPtr);

	} // end of while

	//ZDDbgPrint((_T("end\n")));
	return TRUE;
}

BOOLEAN
ProbeResponse(IN PZD1205_ADAPTER Adapter,
			IN PZD1205_DEFRAG_ENTITY DefragEntityPtr
			  )
{
    PZD1205SwRfd		SwRfd;
	PWLAN_MANAGEMENT_FRM_HEADER_STRUC	pMgtFrm;
	USHORT				index = 0;
	ULONG				tmpvalue;
	USHORT						TypeLength;
	USHORT						i;
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	NDIS_STATUS					Status;
	ZD1211_IRQL     			OldIrql;

    DEBUGFUNC(_T("ProbeResponse"));

	SwRfd = (PZD1205SwRfd)QueueGetHead(
		&DefragEntityPtr->ElementList);
	pMgtFrm = (PWLAN_MANAGEMENT_FRM_HEADER_STRUC)&SwRfd->Rfd->RxBuffer;

	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

	GetEnetFrameLock(Adapter);
	// Destination address
	Adapter->EnetFrame[index++] = pMgtFrm->SA[0];
	Adapter->EnetFrame[index++] = pMgtFrm->SA[1];
	Adapter->EnetFrame[index++] = pMgtFrm->SA[2];
	Adapter->EnetFrame[index++] = pMgtFrm->SA[3];
	Adapter->EnetFrame[index++] = pMgtFrm->SA[4];
	Adapter->EnetFrame[index++] = pMgtFrm->SA[5];

	// Source address
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[index++] = Adapter->AiNodeAddress[5];

	// TypeLength
	TypeLength = (8+2+2+(Adapter->CardSetting.Info_SSID[1]+2)+
				(Adapter->CardSetting.Info_SupportedRates[1]+2)+
				3+4);
#ifdef OFDM
	//Supported rates
	if (Adapter->CardSetting.Info_ExtSupportedRates[1]){
		TypeLength += (Adapter->CardSetting.Info_ExtSupportedRates[1]+2);
	}
#endif
	Adapter->EnetFrame[index++] = (UCHAR)(TypeLength >> 8); 
	Adapter->EnetFrame[index++] = (UCHAR)(TypeLength);

	// Timestamp
	ZD1205_READ_REGISTER(Adapter,TSF_LowPart, &tmpvalue);
	Adapter->EnetFrame[index++] = (UCHAR)(tmpvalue);
	Adapter->EnetFrame[index++] = (UCHAR)(tmpvalue>>8);
	Adapter->EnetFrame[index++] = (UCHAR)(tmpvalue>>16);
	Adapter->EnetFrame[index++] = (UCHAR)(tmpvalue>>24);
	ZD1205_READ_REGISTER(Adapter,TSF_HighPart, &tmpvalue);
	Adapter->EnetFrame[index++] = (UCHAR)(tmpvalue);
	Adapter->EnetFrame[index++] = (UCHAR)(tmpvalue>>8);
	Adapter->EnetFrame[index++] = (UCHAR)(tmpvalue>>16);
	Adapter->EnetFrame[index++] = (UCHAR)(tmpvalue>>24);

	// Beacon Interval
	Adapter->EnetFrame[index++] = (UCHAR)(Adapter->CardSetting.BeaconInterval);
	Adapter->EnetFrame[index++] = (UCHAR)(Adapter->CardSetting.BeaconInterval>>8);

	// Capability
	Adapter->EnetFrame[index++] = (UCHAR)(Adapter->CardSetting.Capability);
	Adapter->EnetFrame[index++] = (UCHAR)(Adapter->CardSetting.Capability>>8);

	// SSID
	for (i=0;i<(Adapter->CardSetting.Info_SSID[1]+2);i++){
		Adapter->EnetFrame[index++] = Adapter->CardSetting.Info_SSID[i];
	}

	// Supported rates
	for (i=0;i<(Adapter->CardSetting.Info_SupportedRates[1]+2);i++){
		Adapter->EnetFrame[index++] = Adapter->CardSetting.Info_SupportedRates[i];
	}

	// DS Parameter Set
	Adapter->EnetFrame[index++] = 0x3;
	Adapter->EnetFrame[index++] = 0x1;
	Adapter->EnetFrame[index++] = (UCHAR)Adapter->CardSetting.Channel;

	if (Adapter->CardSetting.BSSType != AP_BSS){
		// IBSS Parameter Set
		Adapter->EnetFrame[index++] = 0x6;
		Adapter->EnetFrame[index++] = 0x2;
		Adapter->EnetFrame[index++] = (UCHAR)(Adapter->CardSetting.ATIMWindow);
		Adapter->EnetFrame[index++] = (UCHAR)(Adapter->CardSetting.ATIMWindow>>8);
	}

#ifdef OFDM
	//Extend Supported rates
	if (Adapter->CardSetting.Info_ExtSupportedRates[1]){
		for (i=0;i<(Adapter->CardSetting.Info_ExtSupportedRates[1]+2);i++){
			Adapter->EnetFrame[index++] = Adapter->CardSetting.Info_ExtSupportedRates[i];
		}
	}
#endif


	// Form a packet for ProbeRequest frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	while ( (Status = SetupNextSend(Adapter, Packet, PROBERESPONSE, NO_CIPHER, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return TRUE;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);


	return TRUE;
}

void
Mgt_QueueInit(IN PZD1205_ADAPTER Adapter)
{
    QueueInitList(&Adapter->MgtList);

    NdisAllocateSpinLock(&Adapter->MgtLock);
	//Adapter->MgtLock = CreateEvent(NULL, FALSE, TRUE,NULL);
	Adapter->MgtEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	return;
}

BOOLEAN 
Mgt_SendQueue(
			IN PZD1205_ADAPTER Adapter,
			IN PZD1205_DEFRAG_ENTITY DefragEntityPtr
			)
{
    DEBUGFUNC(_T("Mgt_SendQueue"));

	//KeAcquireSpinLock(&Adapter->MgtLock, &Adapter->SavedIrql);
	//ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->MgtLock, &Adapter->SavedIrql);

	QueuePutTail(&Adapter->MgtList, &DefragEntityPtr->Link);

#ifdef HOST_IF_USB
#ifndef CE_ALL
	KeSetEvent(&Adapter->MgtEvent, 2, FALSE);
#else  // CE_ALL
    SetEvent(Adapter->MgtEvent);
#endif  // CE_ALL
#else
	KeSetEvent(&Adapter->MgtEvent, 0, FALSE);
#endif


	//KeReleaseSpinLock(&Adapter->MgtLock, Adapter->SavedIrql);
	//ZDDbg((_T("---------->>> Release SpinLock\n")));
    ZD1211_ReleaseSpinLock(Adapter, &Adapter->MgtLock, Adapter->SavedIrql);

	return TRUE;
}

BOOLEAN
Mgt_SendNotification(
			IN PZD1205_ADAPTER Adapter,
			IN ULONG	NotificationType
			)
{
    ULONGLONG  time;

	time = GetTickCount();

    if ( (NotificationType & RQ_ZD_BSSID_LIST_SCAN)&&((time-Adapter->scantime)<10000) )
		return TRUE;

	if (NotificationType & RQ_DISCONNECT){
		// Let state leave CONNECT immediately. Prevent from the latency of 
		// MgtThread.
		Adapter->adapterReady = FALSE;
		ZDDbgPrint((_T("Adapter DisConnect! - 1\n")));
	}

	if ( (Adapter->Notification & RQ_DISCONNECT)&&(NotificationType == RQ_CONNECTNETWORK) ){
        Adapter->ConnAfterDisconn = 1;
	}


	Adapter->Notification |= NotificationType;
#ifdef HOST_IF_USB
#ifndef CE_ALL
	KeSetEvent(&Adapter->MgtEvent, 2, FALSE);
#else  // CE_ALL
    SetEvent(Adapter->MgtEvent);
#endif  // CE_ALL
#else
	KeSetEvent(&Adapter->MgtEvent, 0, FALSE);
#endif
	return TRUE;
}

BOOLEAN
APMgt_SendNotification(
			IN PZD1205_ADAPTER Adapter,
			IN ULONG	NotificationType
			)
{

	Adapter->FlagAccessRegister |= NotificationType;

#ifdef HOST_IF_USB
#ifndef CE_ALL
	KeSetEvent(&Adapter->AP_MgtEvent, 2, FALSE);
#else  // CE_ALL
    SetEvent(Adapter->MgtEvent);
#endif  // CE_ALL

#else
	KeSetEvent(&Adapter->AP_MgtEvent, 0, FALSE);
#endif

	return TRUE;
}


BOOLEAN
Mgt_ClearNotification(
			IN PZD1205_ADAPTER Adapter,
			IN ULONG	NotificationType
			)
{
	Adapter->Notification &= ~NotificationType;
	return TRUE;
}


BOOLEAN
Mgt_RetrieveQueue(
			IN PZD1205_ADAPTER Adapter,
			OUT PZD1205_DEFRAG_ENTITY *pDefragEntityPtr
			)
{

	DEBUGFUNC(_T("Mgt_RetrieveQueue"));

	if (!QueueEmpty(&Adapter->MgtList))
	{
		//KeAcquireSpinLock(&Adapter->MgtLock, &Adapter->SavedIrql);
		//ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->MgtLock, &Adapter->SavedIrql);
		*(pDefragEntityPtr) = (PZD1205_DEFRAG_ENTITY)QueuePopHead(&Adapter->MgtList);
		//KeReleaseSpinLock(&Adapter->MgtLock, Adapter->SavedIrql);
		//ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->MgtLock, Adapter->SavedIrql);
		return TRUE;
	}else{
		return FALSE;
	}
}

void
IBSS_ConnectNetwork(IN PZD1205_ADAPTER Adapter)
{
	USHORT		index=0;
	UCHAR		Beacon[128];
	USHORT		i = 100;
	USHORT		j;
	BOOLEAN		BSSFound = FALSE;
	ULONGLONG	Timer1;
	UCHAR		SearchCnt;
	ULONGLONG	TimeStamp = 0xFFFFFFFFFFFFFFFF;

#if defined(OFDM) 
	ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_NormalSlot);
	// not to use short slot time
	Adapter->CardSetting.Capability &= ~BIT_10;
	Adapter->ShortSlotEnable = FALSE;

	if (Adapter->bIBSS_PureG_MODE){
		// not use protection mechanism
		Adapter->SelfCTSOn = FALSE;
	}
	else{
		// force use protection mechanism
		Adapter->SelfCTSOn = TRUE;
	}
#endif

	Adapter->CardSetting.Capability &= ~BIT_0;
	Adapter->CardSetting.Capability |= BIT_1;
    ZDDbgPrint((_T("IBSS_CONNECTNETWORK_1\n")));
	SearchCnt = 0;
	while (SearchCnt < 2){
		if (Adapter->BSSCnt){	// IBSS found
			for (j=0;j<Adapter->BSSCnt;j++)
			{
				// Add end of str token
				Adapter->CardSetting.Info_SSID[Adapter->CardSetting.Info_SSID[1]+2] = 0;
				Adapter->BSSInfo[j].SSID[Adapter->BSSInfo[j].SSID[1]+2] = 0;
				if ( strcmp((char*)&Adapter->CardSetting.Info_SSID[2], 
					(char*)&Adapter->BSSInfo[j].SSID[2]) == 0 ){ //Found desired SSID
					
					ZDDbgPrint((_T("SSID found : %s\n"), &Adapter->BSSInfo[j].SSID[2]));

					if (Adapter->bNeedCmpBSSID){
					
						if (memcmp(Adapter->BSSID_to_be_Connect, Adapter->BSSInfo[j].BSSID, 6) != 0){
							ZDDbgPrint((_T("Not match BSSID\n")));
							continue;	
						}

						ZDDbgPrint((_T("BSSID found : %x-%x-%x-%x-%x-%x\n"), Adapter->BSSInfo[j].BSSID[0],
							Adapter->BSSInfo[j].BSSID[1],
							Adapter->BSSInfo[j].BSSID[2],
							Adapter->BSSInfo[j].BSSID[3],
							Adapter->BSSInfo[j].BSSID[4],
							Adapter->BSSInfo[j].BSSID[5]
							));
					}

					if (Adapter->BSSInfo[j].Capability[0] & BIT_1){ // This BSS is IBSS

						if ( *((PULONGLONG)Adapter->BSSInfo[j].TimeStamp) < TimeStamp ){
							TimeStamp = *((PULONGLONG)Adapter->BSSInfo[j].TimeStamp);
							i = j;
						}
					}
				}
			}
			if (i < Adapter->BSSCnt){
				Adapter->ChosenBSS = (UCHAR)i;
				Set_RF_Channel(Adapter, Adapter->BSSInfo[i].Channel, (UCHAR)Adapter->RF_Mode, 1);  
				/****************************************************************/
				/* Setup Beacon													*/
				/****************************************************************/
				
				BSSFound = TRUE;
				//Frame control
				Beacon[index++] = 0x80;	
				Beacon[index++] = 0x00;
				
				//Duration
				Beacon[index++] = 0x00;	
				Beacon[index++] = 0x00;
				
				//Address1
				for (j=0;j<6;j++){		
					Beacon[index++] = 0xff;	
				}
				//Address2
				for (j=0;j<6;j++){		
					Beacon[index++] = Adapter->AiNodeAddress[j];	
				}
				//Address3
				for (j=0;j<6;j++){		
					Beacon[index++] = Adapter->BSSInfo[i].BSSID[j];	
				}
				UpdateBSSID(Adapter, &Adapter->BSSInfo[i].BSSID[0]);

				Adapter->IBSS_BSSID_update = TRUE;
				
				//Sequence control
				Beacon[index++] = 0x00;	
				Beacon[index++] = 0x00;
				
				//Timestamp
				for (j=0;j<8;j++){		
					Beacon[index++] = 0x00;
				}
				
				//BeaconInterval
				for (j=0;j<2;j++){		
					Beacon[index++] = Adapter->BSSInfo[i].BeaconInterval[j];
				}
				UpdateBeaconInterval(Adapter, &Adapter->BSSInfo[i].BeaconInterval[0]);
				//---------------------------------------------------------------------------------------    
				//Capability  chi---2004/09/03
				if (( Adapter->BSSInfo[i].Capability[0] & 0x10) >>4 )   // set the privacy bit
					Adapter->CardSetting.Capability |= 0x0010;
				else
					Adapter->CardSetting.Capability &= 0xffef;
				//---------------------------------------------------------------------------------------
				//Capability
				Beacon[index++] = (Adapter->BSSInfo[i].Capability[0] & (0x32));
				Beacon[index++] = 0;
				if (Adapter->BSSInfo[i].Capability[0] & BIT_5){ // Short Preamble
					Adapter->CardSetting.PreambleType = 0x20;
					Adapter->CardSetting.Capability |= BIT_5;
				}
				else{
					Adapter->CardSetting.PreambleType = 0x00;
					Adapter->CardSetting.Capability &= ~BIT_5;
				}
				
				//SSID
				for (j=0;j<(Adapter->BSSInfo[i].SSID[1]+2);j++){
					Beacon[index++] = Adapter->BSSInfo[i].SSID[j];
				}
				
				UpdateSupportedRates(Adapter, &Adapter->BSSInfo[i].SupportedRates[0]);
#if	(defined(GCCK) && defined(OFDM))
				if (Adapter->bIBSS_G_MODE || Adapter->bIBSS_PureG_MODE){
					if (Adapter->BSSInfo[i].ExtSupportedRates[0] == 50){
						UpdateSupportedRates(Adapter, &Adapter->BSSInfo[i].ExtSupportedRates[0]);
					}
				}
#endif
				//Supported rates
				for (j=0;j<(Adapter->CardSetting.Info_SupportedRates[1]+2);j++){
					Beacon[index++] = Adapter->CardSetting.Info_SupportedRates[j];
				}
				
				//DS parameter
				Beacon[index++] = 0x3;
				Beacon[index++] = 0x1;
				Beacon[index++] = (UCHAR)Adapter->BSSInfo[i].Channel;
				
				//IBSS parameter
				Beacon[index++] = 0x6;
				Beacon[index++] = 0x2;
				Beacon[index++] = Adapter->BSSInfo[i].ATIMWindow[0];
				Beacon[index++] = Adapter->BSSInfo[i].ATIMWindow[1];
				UpdateATIMWindow(Adapter, &Adapter->BSSInfo[i].ATIMWindow[0]);
				
#ifdef OFDM
				if (Adapter->bIBSS_G_MODE || Adapter->bIBSS_PureG_MODE){
					//ERP info
					Beacon[index++] = 0x2A;	//42
					Beacon[index++] = 0x1;
					if (Adapter->bIBSS_PureG_MODE){
						Beacon[index++] = 0x0;
					}
					else{
						Beacon[index++] = 0x7;
					}

					//Extend Supported rates
					if (Adapter->CardSetting.Info_ExtSupportedRates[1]){
						for (j=0;j<(Adapter->CardSetting.Info_ExtSupportedRates[1]+2);j++){
							Beacon[index++] = Adapter->CardSetting.Info_ExtSupportedRates[j];
						}
					}
				}
#endif				
				//CRC32
				for (j=0;j<4;j++){
					Beacon[index++] = 0x00;
				}
				
				//Beacon send
				UpdateBeaconInterval_Send(Adapter, index, 0, Beacon);

				//Indicate connected
				if (!Adapter->bImmDiscAfterRFOff){
					Adapter->adapterReady = TRUE;
					ZDDbgPrint((_T("Adapter Connect! - 1\n")));
				}

				return;
			}		
		}
		SearchCnt++;
		ChannelScan(Adapter,1);
	}
	if ( ! BSSFound){
		/****************************************************/
		/* We generate an IBSS								*/
		/****************************************************/
		Adapter->ChosenBSS = 0xff;
		//Frame control
		Beacon[index++] = 0x80;	
		Beacon[index++] = 0x00;
		
		//Duration
		Beacon[index++] = 0x00;	
		Beacon[index++] = 0x00;
		
		//Address1
		for (j=0;j<6;j++){		
			Beacon[index++] = 0xff;	
		}
		//Address2
		for (j=0;j<6;j++){		
			Beacon[index++] = Adapter->AiNodeAddress[j];	
		}
		//Address3
#ifdef CE_ALL
        Timer1 = GetTickCount();
#else
		Timer1 = KeQueryInterruptTime();
#endif
		srand((UINT)Timer1);
		Beacon[index++] = (UCHAR)((rand() & ~0x3) | 0x2); // I/G = 0, U/L = 1
		Beacon[index++] = (UCHAR)rand();
		Beacon[index++] = (UCHAR)rand();
		Beacon[index++] = (UCHAR)rand();
		Beacon[index++] = (UCHAR)rand();
		Beacon[index++] = (UCHAR)rand();
		UpdateBSSID(Adapter, &Beacon[index-6]);

		Adapter->IBSS_BSSID_update = TRUE;
		
		//Sequence control
		Beacon[index++] = 0x00;	
		Beacon[index++] = 0x00;
		
		//Timestamp
		for (j=0;j<8;j++){		
			Beacon[index++] = 0x00;
		}
		
		//BeaconInterval
		Beacon[index++] = (UCHAR)(Adapter->CardSetting.BeaconInterval);
		Beacon[index++] = (UCHAR)(Adapter->CardSetting.BeaconInterval >> 8);
		UpdateBeaconInterval(Adapter, &Beacon[index-2]);
		
		//Capability
        if ( Adapter->CardSetting.EncryMode == NO_WEP )   // set the privacy bit
            Adapter->CardSetting.Capability &= 0xffef;
		else
			Adapter->CardSetting.Capability |= 0x0010;

		// default is long preamble
		if ( Adapter->preambleMode == 2 )                 // set the preamble bit
			Adapter->CardSetting.Capability |= 0x0020;
		else
            Adapter->CardSetting.Capability &= 0xffdf;    // long preamble;
		Beacon[index++] = (UCHAR)(Adapter->CardSetting.Capability);
		Beacon[index++] = (UCHAR)(Adapter->CardSetting.Capability>>8);

		if (Adapter->CardSetting.Capability & BIT_5){ // Short Preamble
			Adapter->CardSetting.PreambleType = 0x20;//jhlee
			Adapter->CardSetting.Capability |= BIT_5;
		}
		else{
			Adapter->CardSetting.PreambleType = 0x00;
			Adapter->CardSetting.Capability &= ~BIT_5;
		}
		
		//SSID
		for (j=0;j<(Adapter->CardSetting.Info_SSID[1]+2);j++){
			Beacon[index++] = Adapter->CardSetting.Info_SSID[j];
		}
		
		//Supported rates
		Beacon[index++] = 0x01;
		Beacon[index++] = 0x04;
		Beacon[index++] = 0x82;
		Beacon[index++] = 0x84;
		Beacon[index++] = 0x8b;
		Beacon[index++] = 0x96;
		UpdateSupportedRates(Adapter, &Beacon[index-6]);

	
		//DS parameter
		Beacon[index++] = 0x3;
		Beacon[index++] = 0x1;
		Beacon[index++] = (UCHAR)Adapter->CardSetting.Channel;
#if DBG // jxiaoDbg
		//DbgPrint("Create IBSS at Channel %d\n", Adapter->CardSetting.Channel);
#endif
		//IBSS parameter
		Beacon[index++] = 0x6;
		Beacon[index++] = 0x2;
		Beacon[index++] = (UCHAR)(Adapter->CardSetting.ATIMWindow);
		Beacon[index++] = (UCHAR)(Adapter->CardSetting.ATIMWindow>>8);
		UpdateATIMWindow(Adapter, &Beacon[index-2]);
		
#if	(defined(GCCK) && defined(OFDM))

		if (Adapter->bIBSS_G_MODE || Adapter->bIBSS_PureG_MODE){
			//ERP info
			Beacon[index++] = 0x2A;	//42
			Beacon[index++] = 0x1;
			if (Adapter->bIBSS_PureG_MODE){
				Beacon[index++] = 0x0;
			}
			else{
				Beacon[index++] = 0x7;
			}
			
			// WIFI say OFDM can't support in Ad-hoc
			//OFDM support rate
			Beacon[index++] = 0x32;
			
			if ( (Adapter->IsUSB2_0) || (Adapter->busbswitch == 1))
				Beacon[index++] = 0x08;
			else
				Beacon[index++] = 0x03;

			Beacon[index++] = 0x0C;
			Beacon[index++] = 0x12;
			Beacon[index++] = 0x18;

			if ( (Adapter->IsUSB2_0) || (Adapter->busbswitch == 1))
			{
				Beacon[index++] = 0x24;
				Beacon[index++] = 0x30;
				Beacon[index++] = 0x48;
				Beacon[index++] = 0x60;
				Beacon[index++] = 0x6C;
			}

			if ( (Adapter->IsUSB2_0) || (Adapter->busbswitch == 1))
				UpdateSupportedRates(Adapter, &Beacon[index-10]);
			else
				UpdateSupportedRates(Adapter, &Beacon[index-5]);

		}
#endif		
		
		//CRC32
		for (j=0;j<4;j++){
			Beacon[index++] = 0x00;
		}
		
		//Beacon Send
		UpdateBeaconInterval_Send(Adapter, index, 0, Beacon);
	
		//Indicate connected
		if (!Adapter->bImmDiscAfterRFOff){
			Adapter->adapterReady = TRUE;
			ZDDbgPrint((_T("Adapter Connect! - 2 , create its own IBSS\n")));
		}
	}
}

void
UpdateBSSID(IN PZD1205_ADAPTER Adapter,
			PUCHAR	pbssid)
{
	USHORT		j;
	ULONG		Address;

	for (j=0;j<6;j++){
		Adapter->BSSID[j] = *(pbssid+j);
	}
	Address = 0;
	Address = ( (Adapter->BSSID[0]) |
		((ULONG)Adapter->BSSID[1] << 8) |
		((ULONG)Adapter->BSSID[2] << 16) |
		((ULONG)Adapter->BSSID[3] << 24) 
		);
	ZD1205_WRITE_REGISTER(Adapter,BSSID_P1, Address);
	Address = 0;
	Address = ( (Adapter->BSSID[4]) |
		((ULONG)Adapter->BSSID[5] << 8) 
		);
	ZD1205_WRITE_REGISTER(Adapter,BSSID_P2, Address);
}


void 
UpdateBeaconInterval_Send(IN PZD1205_ADAPTER Adapter,
			USHORT	index, BOOLEAN	bIsSoftAP, PUCHAR Beacon)
{
	ULONG				DebugPrint = 0;
	ULONG				tmpvalue;
	int					j;
	ULONG				BCNPlcp;
	ULONG				TryCount;

    DEBUGFUNC(_T("UpdateBeaconInterval_Send"));

	//protect BeaconUpdate
#ifndef HOST_IF_USB
	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#else
#ifndef CE_ALL
	KeWaitForSingleObject(  &Adapter->BeaconUpdateLockEvent, 
							Executive, 
							KernelMode, 
							FALSE, 
							NULL
							);      
#else  // CE_ALL
    WaitForSingleObject(Adapter->BeaconUpdateLockEvent, INFINITE);
#endif // CE_ALL

#endif

	INITSTRTX(DebugPrint,(_T("- start\n")));

	ZD1205_WRITE_REGISTER(Adapter,BCN_FIFO_Semaphore, 0);
	ZD1205_READ_REGISTER(Adapter,BCN_FIFO_Semaphore, &tmpvalue);
	TryCount = 0;
	while (tmpvalue & BIT_1){
		//ZD1205StallExecution(1);
		if (Adapter->Notification & RQ_TERMINATION){
			break;
		}
		TryCount++;
		if (TryCount > 300){
			ZDDbgPrint((_T("BCN_FIFO_Semaphore error 1 !!!!!!!!!!!!!!\n")));
			break;
		}
		ZD1205_Sleep(1000);
		ZD1205_READ_REGISTER(Adapter,BCN_FIFO_Semaphore, &tmpvalue);
	}
	//Write (Beacon_Len -1) to Beacon-FIFO
	ZD1205_WRITE_REGISTER(Adapter,BCNFIFO, (index-1));
	
	for (j=0;j<index;j++){
		ZD1205_WRITE_REGISTER(Adapter,BCNFIFO, Beacon[j]);
	}
	ZD1205_WRITE_REGISTER(Adapter,BCN_FIFO_Semaphore, 1);
	
	//Configure BCNPLCP
	BCNPlcp = 0x00000400;
	index = (index << 3);
	BCNPlcp |= (((ULONG)index) << 16);
	ZD1205_WRITE_REGISTER(Adapter,BCNPLCPCfg, BCNPlcp);
	
	if (!bIsSoftAP){// not soft ap
		//Start send beacon
		ZD1205_READ_REGISTER(Adapter,BCNInterval, &tmpvalue);
		tmpvalue |= BIT_25;
		ZD1205_WRITE_REGISTER(Adapter,BCNInterval, tmpvalue);

		// Make sure that BCN_FIFO write-pointer has been reset. So that 
		// the next coming-soon beacon-content could be set correctly.
		ZD1205_READ_REGISTER(Adapter,BCN_FIFO_Semaphore, &tmpvalue);
		TryCount = 0;
		while (!(tmpvalue & BIT_1)){
			//ZD1205StallExecution(1);
			if (Adapter->Notification & RQ_TERMINATION){
				break;
			}
			TryCount++;
			if (TryCount > 300){
				ZDDbgPrint((_T("BCN_FIFO_Semaphore error 2 !!!!!!!!!!!!!!\n")));
				break;
			}
			ZD1205_Sleep(1000);
			ZD1205_READ_REGISTER(Adapter,BCN_FIFO_Semaphore, &tmpvalue);
		}
	}

	INITSTRTX(DebugPrint,(_T("- end\n")));

#ifndef HOST_IF_USB
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#else
#ifndef CE_ALL
	KeSetEvent(&Adapter->BeaconUpdateLockEvent, IO_NO_INCREMENT, FALSE);
#else  // CE_ALL
    SetEvent(Adapter->BeaconUpdateLockEvent);
#endif  // CE_ALL

#endif

}

void 
UpdateBeaconInterval_Stop(IN PZD1205_ADAPTER Adapter)
{
	ULONG	tmpvalue;
	ULONG	DebugPrint = 1;
	
	DEBUGFUNC(_T("UpdateBeaconInterval_Stop"));

#ifndef HOST_IF_USB
	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#else
#ifndef CE_ALL
	//protect BeaconUpdate
	KeWaitForSingleObject(  &Adapter->BeaconUpdateLockEvent, 
							Executive, 
							KernelMode, 
							FALSE, 
							NULL
							);      
#else  // CE_ALL
    INITSTR((_T("wait BeaconUpdateLockEvent\n")));
	WaitForSingleObject(Adapter->BeaconUpdateLockEvent, INFINITE);
    INITSTR((_T("got BeaconUpdateLockEvent\n")));
#endif  //CE_ALL
#endif

	INITSTRTX(DebugPrint,(_T("\n")));

	ZD1205_READ_REGISTER(Adapter,BCNInterval, &tmpvalue);
	tmpvalue &= ~BIT_25;
	ZD1205_WRITE_REGISTER(Adapter,BCNInterval, tmpvalue);

#ifndef HOST_IF_USB
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#else
#ifndef CE_ALL
	KeSetEvent(&Adapter->BeaconUpdateLockEvent, IO_NO_INCREMENT, FALSE);
#else  // CE_ALL
    SetEvent(Adapter->BeaconUpdateLockEvent);
#endif  // CE_ALL

#endif
}

void 
UpdateBeaconInterval_StartPS(IN PZD1205_ADAPTER Adapter)
{
	ULONG	tmpvalue;
	ULONG	DebugPrint = 1;

    DEBUGFUNC(_T("UpdateBeaconInterval_StartPS"));

#ifndef HOST_IF_USB
	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#else
#ifndef CE_ALL
	//protect BeaconUpdate
	KeWaitForSingleObject(  &Adapter->BeaconUpdateLockEvent, 
							Executive, 
							KernelMode, 
							FALSE, 
							NULL
							);      
#else
	WaitForSingleObject(Adapter->BeaconUpdateLockEvent, INFINITE);
#endif
#endif

	INITSTRTX(DebugPrint,(_T("\n")));

	ZD1205_READ_REGISTER(Adapter,BCNInterval, &tmpvalue);
	tmpvalue |= BIT_26;
	ZD1205_WRITE_REGISTER(Adapter,BCNInterval, tmpvalue);

#ifndef HOST_IF_USB
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#else
#ifndef CE_ALL
	KeSetEvent(&Adapter->BeaconUpdateLockEvent, IO_NO_INCREMENT, FALSE);
#else  // CE_ALL
    SetEvent(Adapter->BeaconUpdateLockEvent);
#endif  // CE_ALL

#endif
}

void 
UpdateBeaconInterval_StopPS(IN PZD1205_ADAPTER Adapter)
{
	ULONG	tmpvalue;
	ULONG	DebugPrint = 1;
	
	DEBUGFUNC(_T("UpdateBeaconInterval_StopPS"));

#ifndef HOST_IF_USB
	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#else
#ifndef CE_ALL
	//protect BeaconUpdate
	KeWaitForSingleObject(  &Adapter->BeaconUpdateLockEvent, 
							Executive, 
							KernelMode, 
							FALSE, 
							NULL
							);      
#else
	WaitForSingleObject(Adapter->BeaconUpdateLockEvent, INFINITE);
#endif
#endif

	INITSTRTX(DebugPrint,(_T("\n")));

	ZD1205_READ_REGISTER(Adapter,BCNInterval, &tmpvalue);
	tmpvalue &= ~BIT_26;
	ZD1205_WRITE_REGISTER(Adapter,BCNInterval, tmpvalue);

#ifndef HOST_IF_USB
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#else
#ifndef CE_ALL
	KeSetEvent(&Adapter->BeaconUpdateLockEvent, IO_NO_INCREMENT, FALSE);
#else  // CE_ALL
    SetEvent(Adapter->BeaconUpdateLockEvent);
#endif  // CE_ALL
#endif
}

void 
UpdateBeaconInterval(IN PZD1205_ADAPTER Adapter,
			PUCHAR	pbcnintvl)
{
	USHORT	BcnInterval;
	ULONG	tmpvalue;
	ULONG	ul_PreTBTT;
	ULONG	ul_ATIMWnd;

    DEBUGFUNC(_T("UpdateBeaconInterval"));

#ifndef HOST_IF_USB
	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#else
#ifndef CE_ALL
	//protect BeaconUpdate
	KeWaitForSingleObject(  &Adapter->BeaconUpdateLockEvent, 
							Executive, 
							KernelMode, 
							FALSE, 
							NULL
							);      
#else
	WaitForSingleObject(Adapter->BeaconUpdateLockEvent, INFINITE);
#endif
#endif

	BcnInterval = (((USHORT)*(pbcnintvl+1)) << 8) + *(pbcnintvl);

	//++ 
	// Make sure that BcnInterval > Pre_TBTT > ATIMWnd >= 0
	if (BcnInterval < 5){
		BcnInterval = 5;
	}
    
	ZD1205_READ_REGISTER(Adapter, Pre_TBTT, &ul_PreTBTT);
	if (ul_PreTBTT < 4){
		ul_PreTBTT = 4;
	}
	if (ul_PreTBTT >= BcnInterval){
		ul_PreTBTT = BcnInterval-1;
	}
	ZD1205_WRITE_REGISTER(Adapter, Pre_TBTT, ul_PreTBTT);

	ZD1205_READ_REGISTER(Adapter, ATIMWndPeriod, &ul_ATIMWnd);
	if (ul_ATIMWnd >= ul_PreTBTT){
		ul_ATIMWnd = ul_PreTBTT-1;
	}

	ZD1205_WRITE_REGISTER(Adapter, ATIMWndPeriod, ul_ATIMWnd);
	//--

	ZD1205_READ_REGISTER(Adapter,BCNInterval, &tmpvalue);
	tmpvalue &= ~0xffff;
	tmpvalue |= BcnInterval;
	ZD1205_WRITE_REGISTER(Adapter,BCNInterval, tmpvalue);//jhlee
	Adapter->CardSetting.BeaconInterval = BcnInterval;

#ifndef HOST_IF_USB
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#else
#ifndef CE_ALL
	KeSetEvent(&Adapter->BeaconUpdateLockEvent, IO_NO_INCREMENT, FALSE);
#else  // CE_ALL
    SetEvent(Adapter->BeaconUpdateLockEvent);
#endif  // CE_ALL
#endif
}

#define	SR_1M		0x2
#define SR_2M		0x4
#define SR_5_5M		0xb
#define SR_11M		0x16
#define	SR_16_5M	0x21
#define	SR_27_5M	0x37

#define	SR_6M		0xc
#define	SR_9M		0x12
#define	SR_12M		0x18
#define	SR_18M		0x24
#define	SR_24M		0x30
#define	SR_36M		0x48
#define	SR_48M		0x60
#define	SR_54M		0x6c

BOOLEAN
IsPureG(PUCHAR	prates, PUCHAR	pExtrates)
{
	int j;

	for (j=0;j<(*(prates+1));j++){

		switch((*(prates+2+j)) & 0x7f){
			case SR_1M:
			case SR_2M:
			case SR_5_5M:
			case SR_11M:

				break;

			default:
				// not 11b rate
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZDDbgPrint((_T("Is a pure G AP!\n")));
					return TRUE;
				}
				break;
		}
	}

	for (j=0;j<(*(pExtrates+1));j++){

		switch((*(pExtrates+2+j)) & 0x7f){
			case SR_1M:
			case SR_2M:
			case SR_5_5M:
			case SR_11M:

				break;

			default:
				// not 11b rate
				if ((*(pExtrates+2+j)) & 0x80){	//It's a basic rate
					ZDDbgPrint((_T("Is a pure G AP!\n")));
					return TRUE;
				}
				break;
		}
	}

	ZDDbgPrint((_T("Not a pure G AP!\n")));

	return FALSE;

}

void
UpdateSupportedRates(IN PZD1205_ADAPTER Adapter,
			PUCHAR	prates)
{
	UCHAR	HighestBasicRate = SR_1M;
	UCHAR	HighestRate = SR_1M;
	UCHAR	SRate;
	ULONG	tmpvalue;
	USHORT	i,j;
	BOOLEAN	bNowIsUSB_11 = 0;

	if (*(prates) == 50){
		// setting about Extend Supported rates
		Adapter->CardSetting.Info_ExtSupportedRates[0] = *(prates);		
		Adapter->CardSetting.Info_ExtSupportedRates[1] = 0;
	}
	else if (*(prates) == 1){
		// setting about Supported rates
		Adapter->CardSetting.Info_SupportedRates[0] = *(prates);
		Adapter->CardSetting.Info_SupportedRates[1] = 0;
		
		ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, 0);
		if ( Adapter->CardSetting.BSSType != AP_BSS )
		    NdisZeroMemory(Adapter->CardSetting.Info_ExtSupportedRates, sizeof(UCHAR) * NUM_SUPPORTED_RATE );
	}

	i = 0;

	for (j=0;j<(*(prates+1));j++){

		if ( (!(Adapter->IsUSB2_0) && (Adapter->busbswitch == 0)))
		{
			switch((*(prates+2+j)) & 0x7f)
			{
			
				case SR_6M:
				case SR_9M:
				case SR_12M:
				case SR_18M:
				case SR_24M:
				case SR_36M:
				case SR_48M:
				case SR_54M:
					bNowIsUSB_11 = 1;
					break;
					
				default:
					bNowIsUSB_11 = 0;
					break;
			}
		}

		if (bNowIsUSB_11){
			// this rate not allow
			continue;
		}

		if (*(prates) == 50){
			// setting about Extend Supported rates
			Adapter->CardSetting.Info_ExtSupportedRates[2+i] = *(prates+2+j);
			Adapter->CardSetting.Info_ExtSupportedRates[1] ++;
		}
		else{
			// setting about Supported rates
			Adapter->CardSetting.Info_SupportedRates[2+i] = *(prates+2+j);
			Adapter->CardSetting.Info_SupportedRates[1] ++;
		}
		i++;

		switch((*(prates+2+j)) & 0x7f){
			case SR_1M:
				SRate = SR_1M;
#ifdef AMAC
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_0;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
#endif
				break;
			case SR_2M:
				SRate = SR_2M;
#ifdef AMAC
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_1;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
#endif
				break;
			case SR_5_5M:
				SRate = SR_5_5M;
#ifdef AMAC
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_2;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
#endif
				break;
			case SR_11M:
				SRate = SR_11M;
#ifdef AMAC
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_3;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
#endif
				break;
#if	(defined(GCCK) && defined(OFDM))
			case SR_6M:
				SRate = SR_6M;
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_8;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
				break;
			case SR_9M:
				SRate = SR_9M;
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_9;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
				break;
			case SR_12M:
				SRate = SR_12M;
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_10;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
				break;
			case SR_18M:
				SRate = SR_18M;
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_11;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
				break;
			case SR_24M:
				SRate = SR_24M;
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_12;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
				break;
			case SR_36M:
				SRate = SR_36M;
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_13;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
				break;
			case SR_48M:
				SRate = SR_48M;
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_14;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
				break;
			case SR_54M:
				SRate = SR_54M;
				if ((*(prates+2+j)) & 0x80){	//It's a basic rate
					ZD1205_READ_REGISTER(Adapter,BasicRateTbl, &tmpvalue);
					tmpvalue |= BIT_15;
					ZD1205_WRITE_REGISTER(Adapter,BasicRateTbl, tmpvalue);
				}
				break;
#endif
			default:
				SRate = SR_1M;
				break;
		}

		if (HighestRate < SRate){
			HighestRate = SRate;
		}

		if ((*(prates+2+j)) & 0x80){	//It's a basic rate
			if (HighestBasicRate < SRate){
				HighestBasicRate = SRate;
			}
		}
	}

	switch(HighestRate){
		case SR_1M:
			Adapter->CardSetting.HighestTxRate = 0x0;
			break;
		case SR_2M:
			Adapter->CardSetting.HighestTxRate = 0x1;
			break;
		case SR_5_5M:
			Adapter->CardSetting.HighestTxRate = 0x2;
			break;
		case SR_11M:
			Adapter->CardSetting.HighestTxRate = 0x3;
			break;
#if	(defined(GCCK) && defined(OFDM))
		case SR_6M:
			Adapter->CardSetting.HighestTxRate = 0x4;
			break;
		case SR_9M:
			Adapter->CardSetting.HighestTxRate = 0x5;
			break;
		case SR_12M:
			Adapter->CardSetting.HighestTxRate = 0x6;
			break;
		case SR_18M:
			Adapter->CardSetting.HighestTxRate = 0x7;
			break;
		case SR_24M:
			Adapter->CardSetting.HighestTxRate = 0x8;
			break;
		case SR_36M:
			Adapter->CardSetting.HighestTxRate = 0x9;
			break;
		case SR_48M:
			Adapter->CardSetting.HighestTxRate = 0xa;
			break;
		case SR_54M:
			Adapter->CardSetting.HighestTxRate = 0xb;
			break;
#endif
		default:
			ASSERT(0);
			break;
	}

	if (!Adapter->RateLock){
		Adapter->CardSetting.TxRate = Adapter->CardSetting.HighestTxRate;
	}

#ifndef AMAC
	ZD1205_READ_REGISTER(Adapter,CtlReg1, &tmpvalue);
	
	if (Adapter->CardSetting.BSSType == INDEPENDENT_BSS){
		if (HighestBasicRate == SR_1M){
			// Workaround compatibility issue.
			// For resonable case, HighestBasicRate should larger than 2M if
			// short-preamble is supported.
			HighestBasicRate = SR_2M;
		}
	}

	switch(HighestBasicRate){
		case SR_1M:
			tmpvalue &= ~0x1c;
			tmpvalue |= 0x00;
			ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
			break;
		case SR_2M:
			tmpvalue &= ~0x1c;
			tmpvalue |= 0x04;
			ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
			break;
		case SR_5_5M:
			tmpvalue &= ~0x1c;
			tmpvalue |= 0x08;
			ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
			break;
		case SR_11M:
			tmpvalue &= ~0x1c;
			tmpvalue |= 0x0c;
			ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
			break;
		default:
			break;
	}
#else
#endif

}

void
UpdateATIMWindow(IN PZD1205_ADAPTER Adapter,
			PUCHAR	patimwnd)
{
	USHORT	AtimWnd;
	ULONG	ul_PreTBTT;

	AtimWnd = (((USHORT)*(patimwnd+1)) << 8) + *(patimwnd);

	//++ 
	// Make sure that Pre_TBTT > ATIMWnd >= 0
	ZD1205_READ_REGISTER(Adapter, Pre_TBTT, &ul_PreTBTT);
	if (AtimWnd >= ul_PreTBTT){
		AtimWnd = (USHORT)(ul_PreTBTT-1);
	}
	//--

	ZD1205_WRITE_REGISTER(Adapter,ATIMWndPeriod, AtimWnd);
	Adapter->CardSetting.ATIMWindow = AtimWnd;

}

void
UpdatePreTBTT(IN PZD1205_ADAPTER Adapter,
			ULONG	pretbtt)
{
	ULONG	ul_BcnItvl;
	ULONG	ul_AtimWnd;

	//++ 
	// Make sure that BcnInterval > Pre_TBTT > ATIMWnd 
	ZD1205_READ_REGISTER(Adapter, BCNInterval, &ul_BcnItvl);
	ul_BcnItvl &= 0xff;
	if (pretbtt >= ul_BcnItvl){
		pretbtt = ul_BcnItvl-1;
	}
	ZD1205_READ_REGISTER(Adapter, ATIMWndPeriod, &ul_AtimWnd);
	if (pretbtt <= ul_AtimWnd){
		pretbtt = ul_AtimWnd+1;
	}
	//--

	ZD1205_WRITE_REGISTER(Adapter,Pre_TBTT, pretbtt);
}


BOOLEAN
Infra_ConnectNetwork(IN PZD1205_ADAPTER Adapter)
{
	USHORT		i = 100, j, quality = 10000;
	UCHAR		CurrentCh = 1;

	//Disable IBSS mode
	//UpdateBeaconInterval_Stop(Adapter);

	Adapter->CardSetting.Capability |= BIT_0;
	Adapter->CardSetting.Capability &= ~BIT_1;

	ZDDbgPrint((_T("Adapter->BSSCnt %d\n"), Adapter->BSSCnt));

    if ( Adapter->AnyBSSFlag )
	{
	    Adapter->CardSetting.Info_SSID[0] = 0;
        Adapter->CardSetting.Info_SSID[1] = 0;
	}

	if (Adapter->BSSCnt){	// BSS found
        ZDDbg2(("Search SSID - %s\n", &Adapter->CardSetting.Info_SSID[2]));
		for (j=0;j<Adapter->BSSCnt;j++)
		{
			// Add end of str token
			Adapter->CardSetting.Info_SSID[Adapter->CardSetting.Info_SSID[1]+2] = 0;
			Adapter->BSSInfo[j].SSID[Adapter->BSSInfo[j].SSID[1]+2] = 0;
			// match the SSID

            ZDDbg2(("SSID - %s\n", &Adapter->BSSInfo[j].SSID[2]));
			//ZDDbgPrint((_T("Search SSID - %s\n"), &Adapter->BSSInfo[j].SSID[2]));

			if ( ( strcmp((char*)&Adapter->CardSetting.Info_SSID[2], 
				   (char*)&Adapter->BSSInfo[j].SSID[2]) == 0 ) ||
				 // connect to any BSS which is WEP and WPA off
				 ( ( Adapter->CardSetting.Info_SSID[1] == 0 ) && 
				   ((!(Adapter->BSSInfo[j].Capability[0] & BIT_4))||(Adapter->CardSetting.EncryMode == WEP64)||(Adapter->CardSetting.EncryMode == WEP128)||(Adapter->CardSetting.EncryMode == WEP256)) &&
				   (!Adapter->BSSInfo[j].WPASupport) ) ){ //Found desired SSID

				if ( !CheckAuthentication(Adapter, &Adapter->CardSetting.Info_SSID[1]) ) // for WPA negative tests
                    continue;

				//ZDDbgPrint((_T("SSID found : %s\n"), &Adapter->BSSInfo[j].SSID[2]));
				ZDDbg2(("SSID found : %s\n", &Adapter->BSSInfo[j].SSID[2]));
				i=j;
				break;
				/*
				if (Adapter->bNeedCmpBSSID){
				
					if (memcmp(Adapter->BSSID_to_be_Connect, Adapter->BSSInfo[j].BSSID, 6) != 0){
						ZDDbgPrint((_T("Not match BSSID\n")));
						continue;	
					}

					ZDDbgPrint((_T("BSSID found : %x-%x-%x-%x-%x-%x\n"), Adapter->BSSInfo[j].BSSID[0],
						Adapter->BSSInfo[j].BSSID[1],
						Adapter->BSSInfo[j].BSSID[2],
						Adapter->BSSInfo[j].BSSID[3],
						Adapter->BSSInfo[j].BSSID[4],
						Adapter->BSSInfo[j].BSSID[5]
						));
				}
				*/
				/*
				if (Adapter->BSSInfo[j].Capability[0] & BIT_0){ // This BSS is Infra-BSS
                    if ( Adapter->BSSInfo[j].SignalQuality < quality ){
                        quality = Adapter->BSSInfo[j].SignalQuality;
						i = j;
						ZDDbgPrint((_T("Got more power AP\n")));
					}
				}
				*/
			}
		}
		
		if (i < Adapter->BSSCnt){
			AdoptRegulatorySetting(Adapter, i);

			memcpy((PVOID)&Adapter->CardSetting.Info_SSID[0], 
					(PVOID)&Adapter->BSSInfo[i].SSID[0],
					(Adapter->BSSInfo[i].SSID[1]+3));
			Adapter->ChosenBSS = (UCHAR)i;
			Set_RF_Channel(Adapter, Adapter->BSSInfo[i].Channel, (UCHAR)Adapter->RF_Mode, 1);  

			/********************************/
			/* Join this BSS				*/
			/********************************/
			UpdateBSSID(Adapter, &Adapter->BSSInfo[i].BSSID[0]);
			Adapter->ListenInterval = 0x0000;

			if (IsPureG(&Adapter->BSSInfo[i].SupportedRates[0],
				&Adapter->BSSInfo[i].ExtSupportedRates[0])){

				// we assume this is pure G, remove USB1.1 protect
				ZDDbgPrint((_T("This is Pure G AP\n")));
				Adapter->IsUSB2_0 = 1;

			}

			UpdateSupportedRates(Adapter, &Adapter->BSSInfo[i].SupportedRates[0]);
#if	(defined(GCCK) && defined(OFDM))
			if (Adapter->BSSInfo[i].ExtSupportedRates[0] == 50){
				UpdateSupportedRates(Adapter, &Adapter->BSSInfo[i].ExtSupportedRates[0]);
			}
#endif

			UpdateBeaconInterval(Adapter, &Adapter->BSSInfo[i].BeaconInterval[0]);
			
            if ( Adapter->BSSInfo[i].WPASupport ){
			    memcpy(Adapter->CardSetting.wpaIEs, 
					   Adapter->BSSInfo[i].wpaIEs,
					   Adapter->BSSInfo[i].IEtotal);
				Adapter->CardSetting.WPASupport = TRUE;
				Adapter->CardSetting.IEstart = Adapter->BSSInfo[i].IEstart;
				Adapter->CardSetting.IEtotal = Adapter->BSSInfo[i].IEtotal;
			}else{
			    Adapter->CardSetting.WPASupport = FALSE;
			}

			if (Adapter->BSSInfo[i].Capability[0] & BIT_5){ // Short preamble
				Adapter->CardSetting.PreambleType = 0x20;
				Adapter->CardSetting.Capability |= BIT_5;
			}
			else{
				Adapter->CardSetting.PreambleType = 0x00;
				Adapter->CardSetting.Capability &= ~BIT_5;
			}
//#ifdef OFDM
			//if (Adapter->BSSInfo[i].Capability[1] & BIT_2){
			//	ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_ShortSlot);
			//	Adapter->CardSetting.Capability |= BIT_10;
			//	Adapter->ShortSlotEnable = TRUE;
			//}
			//else{
			//	ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_NormalSlot);
			//	Adapter->CardSetting.Capability &= ~BIT_10;
			//	Adapter->ShortSlotEnable = FALSE;
			//}
//#endif

			if ( (Adapter->BSSInfo[i].Capability[0] & BIT_4)&&
				 ((Adapter->CardSetting.AuthMode == Ndis802_11AuthModeOpen)||
				 (Adapter->CardSetting.AuthMode == Ndis802_11AuthModeShared)||
				 (Adapter->CardSetting.AuthMode == Ndis802_11AuthModeAutoSwitch)) )
			{ // Privacy enabled
			
				if ( Adapter->encryMode == Ndis802_11WEPDisabled )
				{
					ZDDbgPrint((_T("Adapter no WEP and Try to Connect WEP AP !!!!\n")));
					return FALSE;
				}
				
				if (Adapter->CardSetting.AuthMode != Ndis802_11AuthModeOpen)
				{
					// share key, or auto switch
					if (Authenticate_SharedKey(Adapter))
					{
						Adapter->Infra_Process_Associate=TRUE;
						if (Associate(Adapter))
						{
							//Adapter->adapterReady = TRUE;
							ZDDbgPrint((_T("Adapter Connect! - 3\n")));
							return TRUE;
						}
					}
				}

				//open, or auto switch
				if (Adapter->CardSetting.AuthMode != Ndis802_11AuthModeShared)
				{
				    if (Authenticate_OpenSystem(Adapter))
					{
						Adapter->Infra_Process_Associate=TRUE;
					    if (Associate(Adapter))
						{
						    //Adapter->adapterReady = TRUE;
							ZDDbgPrint((_T("Adapter Connect! - 4\n")));
						    return TRUE;
						}
					}
				} 
			}
			else
			{
				// No encryption needed
	            if ( Adapter->BSSInfo[i].Capability[0] & BIT_4 )  // set the privacy bit for WPA 
                    Adapter->CardSetting.Capability |= 0x0010;
				else
                    Adapter->CardSetting.Capability &= 0xffef;
			
				if (Authenticate_OpenSystem(Adapter))
				{
					Adapter->Infra_Process_Associate=TRUE;
					if (Associate(Adapter))
					{
						//Adapter->adapterReady = TRUE;
						ZDDbgPrint((_T("Adapter Connect! - 5\n")));
						return TRUE;
					}
				}
			}
		
			ZDDbgPrint((_T("Adapter Connect fail!\n")));
			// return TRUE to indicate the desired AP has been in our list.
			return TRUE;
		}
		else{
            ZDDbg2(("Desire SID not Found - %s\n", &Adapter->CardSetting.Info_SSID[2]));
			//ZDDbgPrint((_T("Desire SID not Found - %s\n"), &Adapter->CardSetting.Info_SSID[2]));
		}
	}
	return FALSE;
}

void 
IssuePsPoll(IN PZD1205_ADAPTER Adapter)
{
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	USHORT						TypeLength;
	NDIS_STATUS					Status;
	PZD1205_DEFRAG_ENTITY		DefragEntityPtr = NULL;
	ZD1211_IRQL				OldIrql;
	DEBUGFUNC(_T("IssuePsPoll"));

	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

	GetEnetFrameLock(Adapter);
	// Destination address
	Adapter->EnetFrame[0] = Adapter->BSSID[0];
	Adapter->EnetFrame[1] = Adapter->BSSID[1];
	Adapter->EnetFrame[2] = Adapter->BSSID[2];
	Adapter->EnetFrame[3] = Adapter->BSSID[3];
	Adapter->EnetFrame[4] = Adapter->BSSID[4];
	Adapter->EnetFrame[5] = Adapter->BSSID[5];

	// Source address
	Adapter->EnetFrame[6] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[7] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[8] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[9] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[10] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[11] = Adapter->AiNodeAddress[5];

	// TypeLength
	TypeLength = 0x00;
	Adapter->EnetFrame[12] = (UCHAR)(TypeLength >> 8); // For open-system
	Adapter->EnetFrame[13] = (UCHAR)(TypeLength);

	// Form a packet for authentication frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	while ( (Status = SetupNextSend(Adapter, Packet, PS_POLL, NO_CIPHER, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

	return;
}

void 
IssueNullData(IN PZD1205_ADAPTER Adapter)
{
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	USHORT						TypeLength;
	NDIS_STATUS					Status;
	PZD1205_DEFRAG_ENTITY		DefragEntityPtr = NULL;
	ZD1211_IRQL				OldIrql;
	DEBUGFUNC(_T("IssueNullData"));

	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

	GetEnetFrameLock(Adapter);
	// Destination address
	Adapter->EnetFrame[0] = Adapter->BSSID[0];
	Adapter->EnetFrame[1] = Adapter->BSSID[1];
	Adapter->EnetFrame[2] = Adapter->BSSID[2];
	Adapter->EnetFrame[3] = Adapter->BSSID[3];
	Adapter->EnetFrame[4] = Adapter->BSSID[4];
	Adapter->EnetFrame[5] = Adapter->BSSID[5];

	// Source address
	Adapter->EnetFrame[6] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[7] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[8] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[9] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[10] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[11] = Adapter->AiNodeAddress[5];

	// TypeLength
	TypeLength = 0x00;
	Adapter->EnetFrame[12] = (UCHAR)(TypeLength >> 8); // For open-system
	Adapter->EnetFrame[13] = (UCHAR)(TypeLength);

	// Form a packet for authentication frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	while ( (Status = SetupNextSend(Adapter, Packet, DATA, NO_CIPHER, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

	return;
}

BOOLEAN
AnyActivity(			
			IN PZD1205_ADAPTER Adapter,
			IN PZD1205_DEFRAG_ENTITY DefragEntityPtr
			)
{
    PZD1205SwRfd		SwRfd;
	PWLAN_MANAGEMENT_FRM_HEADER_STRUC	pMgtFrm;
	USHORT				index = 0;
	USHORT				N1;
	USHORT				N2;
	USHORT				quotient;
	USHORT				remainder;
	ZD1211_IRQL			OldIrql;

	DEBUGFUNC(_T("AnyActivity"));

	ASSERT(!QueueEmpty(&DefragEntityPtr->ElementList));
	SwRfd = (PZD1205SwRfd)QueueGetHead(
		&DefragEntityPtr->ElementList);
	pMgtFrm = (PWLAN_MANAGEMENT_FRM_HEADER_STRUC)&SwRfd->Rfd->RxBuffer;

	// More date in AP
	if (pMgtFrm->FrameCtrl[1] & BIT_5){
		Mgt_SendNotification(Adapter, RQ_PSPOLL);
		return TRUE;
	}

	// Any frame wait for transmission.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	if (!(QueueEmpty(&Adapter->ActiveChainList))){
		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		return TRUE;
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

	// Our AID in TIM??
	if ( (pMgtFrm->FrameCtrl[0] == BEACON) && 
		 (memcmp(&pMgtFrm->BSSID[0], &Adapter->BSSID[0], 6) == 0) ){

		index = BCNINTERVAL_OFFSET+4;
		while(index < (SwRfd->Rfd->ActualCount-CRC32_SIZE-EXT_INFO_SIZE) ){ // Minus CRC32(4), EXTRINFO(4)
			switch(SwRfd->Rfd->RxBuffer[index]){
				case 5: // TIM
					if (SwRfd->Rfd->RxBuffer[index+1] >= 3){	

						// Multicast frames queued in AP
						if (SwRfd->Rfd->RxBuffer[index+4] & BIT_0){
							ZDDbgPrint((_T("Got multicast framed queued information!\n")));

							return TRUE;
						}

						N1 = (SwRfd->Rfd->RxBuffer[index+4] & ~BIT_0);
						N2 = (SwRfd->Rfd->RxBuffer[index+1]+N1-4);
						quotient = (Adapter->AID >> 3);
						remainder = (Adapter->AID & 0x7);
						if ( (quotient < N1) || (quotient > N2) ){
							return FALSE;
						}
						if((SwRfd->Rfd->RxBuffer[index+5+quotient-N1] >> remainder)
							& BIT_0){
							Mgt_SendNotification(Adapter, RQ_PSPOLL);

							return TRUE;
						}
						else{
							return FALSE;
						}
					}
					break;
				// Feature: ECCK-Rate Adoption
				// - Announce ECCK-Rate in Extended Supported Rate information element.
				// - Create data structure "OppositeContext" for each opposite peer STA
				//   in IBSS mode
				//   OppositeContext{
				//	     MAC Address,
				//		 MAXRate,
				//		 CurrentRate,
				//		 ConsecutiveSuccessFrames
				//	 }
				// - Recognization rule:
				//   STA mode: Monitor the Extended Supported Rate information element
				//             in beacon. If yes, record it in "OppositeContext".
				//   AP mode : Monitor the Extended Supported Rate information element
				//             in Association Request. If yes, record it in 
				//             "OppositeContext".
				default:
					break;
			}
			index += (SwRfd->Rfd->RxBuffer[index+1]+2);
		}
	}

	// No any activity.

	return TRUE;
}


int RateReMap[] = {	1,
					2,
					5,
					11,
					6,
					9,
					12,
					18,
					24,
					36,
					48,
					54
					};

void
SortRateArray(UCHAR *RateArray, int ArrayLen)
{
	// we sort Adapter->OppositeRateInfo[i].RateArray[] to has right order
	int		x,y;
	UCHAR	tmpChar;

	for (x=0; x<(ArrayLen-1); x++){
		for (y=(x+1); y<ArrayLen; y++){

			if(RateReMap[RateArray[x]] > RateReMap[RateArray[y]]){
				tmpChar = RateArray[x];
				RateArray[x] = RateArray[y];
				RateArray[y] = tmpChar;
			}
		}
	}

	ZDDbgPrint((_T("Sorted rate ")));
	for (x=0; x<ArrayLen; x++)
		ZDDbgPrint((_T("0x%x "), RateArray[x]));
	ZDDbgPrint((_T("\n")));

}


ULONG
ParseRxMgtFrame(			
			IN PZD1205_ADAPTER Adapter,
			IN PZD1205_DEFRAG_ENTITY DefragEntityPtr
			)
{
    PZD1205SwRfd		SwRfd;
	PWLAN_MANAGEMENT_FRM_HEADER_STRUC	pMgtFrm;
	USHORT				index = 0;
	USHORT              length;
	USHORT				Capability;
    UCHAR               getERP;
	BOOLEAN				bRet;

	SwRfd = (PZD1205SwRfd)QueueGetHead(
		&DefragEntityPtr->ElementList);
	pMgtFrm = (PWLAN_MANAGEMENT_FRM_HEADER_STRUC)&SwRfd->Rfd->RxBuffer;

	if (pMgtFrm->FrameCtrl[0] == DEAUTHENTICATION){
		ZDDbgPrint((_T("ParseRxMgtFrame -> Rx DEAUTHENTICATION\n")));
	}

	// rate adaption
	if ( (pMgtFrm->FrameCtrl[0] == BEACON)&&(!memcmp(pMgtFrm->BSSID, Adapter->BSSID, 6)) ){

		Capability = SwRfd->Rfd->RxBuffer[BCNINTERVAL_OFFSET+2]+
			(SwRfd->Rfd->RxBuffer[BCNINTERVAL_OFFSET+3] << 8);

#ifdef OFDM
		//if (Capability & BIT_10){
		//	if (!Adapter->ShortSlotEnable){
		//		ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_ShortSlot);
		//		Adapter->ShortSlotEnable = TRUE;
		//	}
		//}
		//else{
		//	if (Adapter->ShortSlotEnable){
		//		ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_NormalSlot);
		//		Adapter->ShortSlotEnable = FALSE;
		//	}
		//}
#endif

		index = BCNINTERVAL_OFFSET+4;
		while(index < (SwRfd->Rfd->ActualCount-CRC32_SIZE-EXT_INFO_SIZE) ){ // Minus CRC32(4), EXTRINFO(4)
			switch(SwRfd->Rfd->RxBuffer[index]){
				
				case 0:	//SSID

					length = SwRfd->Rfd->RxBuffer[index+1];
					
					if ((Adapter->CardSetting.Info_SSID[1] == length) &&
						(Adapter->CardSetting.BSSType == INDEPENDENT_BSS)){
						//ssid length equal
						if( !memcmp(	Adapter->CardSetting.Info_SSID+2,
										SwRfd->Rfd->RxBuffer+index+2, 
										length) 
										){
							//ssid are same
							if ( memcmp(pMgtFrm->BSSID, Adapter->BSSID, 6) ){
								// not the BSSID we connect
								ULONGLONG			TSFTimer, DstTSFTimer;
								ULONG				tmpvalue;
			
								DstTSFTimer = *((PULONGLONG)(SwRfd->Rfd->RxBuffer + BCNINTERVAL_OFFSET - 8));
								ZD1205_READ_REGISTER(Adapter,TSF_LowPart, &tmpvalue);
								TSFTimer = tmpvalue;
								ZD1205_READ_REGISTER(Adapter,TSF_HighPart, &tmpvalue);
								TSFTimer += (((ULONGLONG)tmpvalue) << 32);
					
								ZDDbgPrint((_T("DstTSFTimer %d\n"), DstTSFTimer));
								ZDDbgPrint((_T("   TSFTimer %d\n"), TSFTimer));
								if (DstTSFTimer > TSFTimer){
									// change our BSSID to Dst
									ZDDbgPrint((_T("Roaming to new BSSID\n")));
									Mgt_SendNotification(Adapter, RQ_DISCONNECT);			
									Mgt_SendNotification(Adapter, RQ_BSSID_LIST_SCAN);
									Mgt_SendNotification(Adapter, RQ_CONNECTNETWORK);
								}
							}
						}
					}

					break;
				/*
				case 4: // CF Parameter
					if (SwRfd->Rfd->RxBuffer[index+1] >= 6){
						ZDDbgPrint((_T("############# Got CF Parameter\n")));
						ZDDbgPrint((_T(" CFPDurRemaining = %x\n"), 
							SwRfd->Rfd->RxBuffer[index+6]+
							(((USHORT)SwRfd->Rfd->RxBuffer[index+7]) << 8)));
						index = (USHORT)SwRfd->Rfd->ActualCount;
					}
					break;
				*/
				// Feature: ECCK-Rate Adoption
				// - Announce ECCK-Rate in Extended Supported Rate information element.
				// - Create data structure "OppositeContext" for each opposite peer STA
				//   in IBSS mode
				//   OppositeContext{
				//	     MAC Address,
				//		 MAXRate,
				//		 CurrentRate,
				//		 ConsecutiveSuccessFrames
				//	 }
				// - Recognization rule:
				//   STA mode: Monitor the Extended Supported Rate information element
				//             in beacon. If yes, record it in "OppositeContext".
				//   AP mode : Monitor the Extended Supported Rate information element
				//             in Association Request. If yes, record it in 
				//             "OppositeContext".

				// rate adaption by jhlee
				case 1:		//Supported Rate
					if ( !memcmp(pMgtFrm->BSSID, Adapter->BSSID, 6) &&
							(Adapter->CardSetting.BSSType == INDEPENDENT_BSS)){
						// the BSSID we connect

						bRet = BuildOppositeRateInfo(	Adapter,
														pMgtFrm->SA,
														&(SwRfd->Rfd->RxBuffer[index]),
														Capability,
														0
														);
					}
					break;
				case 50:	//Extended Supported Rate
					if ( !memcmp(pMgtFrm->BSSID, Adapter->BSSID, 6) &&
							(Adapter->CardSetting.BSSType == INDEPENDENT_BSS)){
						// the BSSID we connect

						if (bRet)
							BuildOppositeRateInfo(	Adapter,
													pMgtFrm->SA,
													&(SwRfd->Rfd->RxBuffer[index]),
													Capability,
													1
													);
					}
					break;
				case 42:	//ERP information
					//jxiaoDbg
					//DbgPrint("Rx ERP info\n");
					if (Adapter->CardSetting.Capability & BIT_0)
					{
						// infrastruture
						if ( !memcmp(pMgtFrm->BSSID, Adapter->BSSID, 6) ){
							// the BSSID we connect

							// check if need SelfCTS
							length = SwRfd->Rfd->RxBuffer[index+1];					    
							//DebugERP
							//DbgPrint(" ERP info=%u\n",SwRfd->Rfd->RxBuffer[index+2]);
							if (SwRfd->Rfd->RxBuffer[index+2] & BIT_1){
								//jxiaoDbg
								//if (Adapter->SelfCTSOn != TRUE)
								{
									//DbgPrint("SelfCTS ON\n");
									Adapter->SelfCTSOn = TRUE;
								}
							}
							else{
								//jxiaoDbg
								//if (Adapter->SelfCTSOn == TRUE)
								{
									//DbgPrint("------- off\n");
									Adapter->SelfCTSOn = FALSE;
								}
							}

							//if ( (SwRfd->Rfd->RxBuffer[index+2] & BIT_0)&&
							//	 (!Adapter->LongDIFS) )
							//{
							//	Adapter->LongDIFS = 1;
							//	ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x547c032);
							//}
							//else if ( (!(SwRfd->Rfd->RxBuffer[index+2] & BIT_0))&&
							//	      (Adapter->LongDIFS) )
							//{
							//	Adapter->LongDIFS = 0;
                            //    ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x547c00a);
							//}
							
							// check Barker_Preamble_Mode							
							if (SwRfd->Rfd->RxBuffer[index+2] & BIT_2){
								// use short preamble when Self-CTS
								if (!Adapter->BakerPreamble){
									ZD1205_WRITE_REGISTER(Adapter,RTS_CTS_Rate, 0x30000);
									//ZD1205_WRITE_REGISTER(Adapter,RTS_CTS_Rate, 0x2030000);
									Adapter->BakerPreamble = TRUE;
								}
							}
							else{
								if (Adapter->BakerPreamble){
									ZD1205_WRITE_REGISTER(Adapter,RTS_CTS_Rate, 0x30000);
									Adapter->BakerPreamble = FALSE;
								}
							}

							// check B-STA
							if ( SwRfd->Rfd->RxBuffer[index+2] & BIT_0 )
							{
							    if ( !Adapter->LongSlot )
                                {
									if (Adapter->RF_Mode == AL2230_RF ||
										Adapter->RF_Mode == AL2210_RF)
									{
 										ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0xA47c032);
									}
									else
									{
										ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x547c032);
									}
						            ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_LongSlot);
						            Adapter->LongSlot = 1;			    
								}
							}else{
							    if ( Adapter->LongSlot )
                                {
									if (Adapter->RF_Mode == AL2230_RF ||
										Adapter->RF_Mode == AL2210_RF)
									{
 										ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0xA47c00a);
									}
									else
									{
										ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x547c00a);
									}
						            ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_NormalSlot);
						            Adapter->LongSlot = 0;			    
								}
							}
						}
					}
					break;

				default:
					break;
			}
			index += (SwRfd->Rfd->RxBuffer[index+1]+2);
		}
		Adapter->Bcn_Acc_Num ++;
		Adapter->Bcn_Acc_SQ += SwRfd->Rfd->RxBuffer[SwRfd->Rfd->ActualCount-EXT_INFO_SIZE];
	}

	if ( (pMgtFrm->FrameCtrl[0] == BEACON)&&
		 (Adapter->CardSetting.BSSType == AP_BSS)&&
		 (SwRfd->Rfd->RxBuffer[SwRfd->Rfd->ActualCount-EXT_INFO_SIZE+1] > 0x0A)&&
		 (Adapter->BssIndex) ){  // Adapter->BssIndex == 0 => channel scanning
		getERP = 0;
		index = BCNINTERVAL_OFFSET+4;
		while(index < (SwRfd->Rfd->ActualCount-CRC32_SIZE-EXT_INFO_SIZE) ){ // Minus CRC32(4), EXTRINFO(4)
			switch(SwRfd->Rfd->RxBuffer[index]){				
				case 42:	//ERP information
					{
						getERP = 1;
						// need protection mode
						if ( SwRfd->Rfd->RxBuffer[index+2] & BIT_0 ){ 
					        Adapter->SelfCTSOn = TRUE;
						    Adapter->CardSetting.ERP[2] |= BIT_1;
							Adapter->ProtectionModeTimer = 0;
						}
					}
					break;

				default:
					break;
			}
			index += (SwRfd->Rfd->RxBuffer[index+1]+2);
		}
		if ( (!getERP)&&(Adapter->CheckBeacon) ){
		    Adapter->SelfCTSOn = TRUE;
            Adapter->CardSetting.ERP[2] |= BIT_1;
			Adapter->ProtectionModeTimer = 0;
		}
	}
	
	return (pMgtFrm->FrameCtrl[0]);

}


void
ChannelScan(
			IN PZD1205_ADAPTER Adapter,
			IN UINT ScanAll     // ScanAll = 0 : default
			)              // ScanAll = 1 : scan all channels
{
	UCHAR		CurrentCh = 1;

	if ((!Adapter->adapterReady)||(ScanAll)){
		Adapter->BssIndex = 0;
		Adapter->SaveChannel = Adapter->CardSetting.Channel;
		while(CurrentCh < 15){
			if ( Adapter->DisableTx )  // for halting the device
				return;
			//if ( (1<<(CurrentCh-1)) & Adapter->AllowedChannel)
			{
				//ZDDbgPrint((_T("scan channel = %x"), CurrentCh));
				DbgPrint("scan channel = %x\n", CurrentCh);
				Set_RF_Channel(Adapter, CurrentCh, (UCHAR)Adapter->RF_Mode, 1);  
				ProbeRequest(Adapter);
			}
			CurrentCh++;
		}
		// Restore the original channel
		Set_RF_Channel(Adapter, Adapter->SaveChannel, (UCHAR)Adapter->RF_Mode, 1);  
	}
	else{
		ProbeRequest(Adapter);
	}

	Adapter->BSSCnt = Adapter->BssIndex;
}

void 
AP_Monitoring(
			IN PZD1205_ADAPTER Adapter,
			IN PZD1205_DEFRAG_ENTITY DefragEntityPtr
			)
{
    PZD1205SwRfd		SwRfd;
	PWLAN_DATA_FRM_HEADER_STRC	pFrm;

	SwRfd = (PZD1205SwRfd)QueueGetHead(&DefragEntityPtr->ElementList);

	pFrm = (PWLAN_DATA_FRM_HEADER_STRC)&SwRfd->Rfd->RxBuffer;
	//if ( (pFrm->FrameCtrl[0] & 0xfc) == 0x80 ){ // Beacon
	    if ( memcmp(&pFrm->Address2[0], &Adapter->BSSID[0], 6) == 0 ){
		    Adapter->AP_Alive = TRUE;
		}
	//}
}

void 
IBSS_Monitoring(
			IN PZD1205_ADAPTER Adapter,
			IN PZD1205_DEFRAG_ENTITY DefragEntityPtr
			)
{
    PZD1205SwRfd		SwRfd;
	PWLAN_DATA_FRM_HEADER_STRC	pFrm;

	if (Adapter->CardSetting.BSSType == INDEPENDENT_BSS){
		if (Adapter->bImmDiscAfterRFOff){
			SwRfd = (PZD1205SwRfd)QueueGetHead(&DefragEntityPtr->ElementList);

			pFrm = (PWLAN_DATA_FRM_HEADER_STRC)&SwRfd->Rfd->RxBuffer;
			//if ( (pFrm->FrameCtrl[0] & 0xfc) == 0x80 ){ // Beacon
			if (( memcmp(&pFrm->Address3[0], &Adapter->BSSID[0], 6) == 0 )&&
				(Adapter->IBSS_BSSID_update == TRUE))
			{
				Adapter->IBSSPartner_Alive = TRUE;
				if (!Adapter->adapterReady){
					Adapter->adapterReady = TRUE;
					NdisMIndicateStatus(Adapter->ZD1205AdapterHandle,
						NDIS_STATUS_MEDIA_CONNECT,
						NULL,     // Not used.
						0);                 // Not used.
					NdisMIndicateStatusComplete(Adapter->ZD1205AdapterHandle);
				}
			}
			//}
		}
	}
}

BOOLEAN
Mgt_SendTestFrame(
					IN PZD1205_ADAPTER Adapter,
					UCHAR	pattern
				 )
{
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	USHORT						TypeLength;
	NDIS_STATUS					Status;
	USHORT						i;
	ZD1211_IRQL				OldIrql;

    DEBUGFUNC(_T("Mgt_SendTestFrame"));

	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

	GetEnetFrameLock(Adapter);
	// Destination address
	Adapter->EnetFrame[0] = 0xff;
	Adapter->EnetFrame[1] = 0xff;
	Adapter->EnetFrame[2] = 0xff;
	Adapter->EnetFrame[3] = 0xff;
	Adapter->EnetFrame[4] = 0xff;
	Adapter->EnetFrame[5] = 0xff;

	// Source address
	Adapter->EnetFrame[6] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[7] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[8] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[9] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[10] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[11] = Adapter->AiNodeAddress[5];

	// TypeLength
	if (Adapter->TestFlag & LENGTH_SCAN){
		if (Adapter->LBFrameLen >= (4095 - 28 - 8)){
			Adapter->LBFrameLen = 0;
		}
		Adapter->LBFrameLen ++;
	}
	TypeLength = Adapter->LBFrameLen;
	Adapter->EnetFrame[12] = (UCHAR)(TypeLength >> 8); // For open-system
	Adapter->EnetFrame[13] = (UCHAR)(TypeLength);

//JWEI 2004/01/29
    for (i = 0; i < TypeLength; i ++)
    #if !fZD1211_LOOPBACK
        Adapter->EnetFrame[14 + i] = pattern;
    #else
        Adapter->EnetFrame[14 + i] = (pattern + i) & 0xFF;
    #endif

	// Form a packet for authentication frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	if ( (Status = SetupNextSend(Adapter, Packet, MGT_LBTEST, 
						Adapter->CardSetting.EncryMode, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){
		NdisFreeBuffer(Buffer);
		NdisFreePacket(Packet);
		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		if (Adapter->TestFlag & LENGTH_SCAN){
			Adapter->LBFrameLen --;
		}
		return FALSE;
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

	return TRUE;
}

void
Mgt_ScanTest(
				IN PZD1205_ADAPTER Adapter,
				USHORT	ScanLen
			)
{
	PNDIS_PACKET				Packet;
	PNDIS_BUFFER				Buffer;
    NDIS_STATUS					AllocationStatus;
	USHORT						TypeLength;
	NDIS_STATUS					Status;
	USHORT						i;
	ZD1211_IRQL					OldIrql;

    DEBUGFUNC(_T("Mgt_ScanTest"));

	//******************************************//
	// Construct the framebody to a 802.3 format
	//******************************************//

	GetEnetFrameLock(Adapter);
	// Destination address
	Adapter->EnetFrame[0] = 0xff;
	Adapter->EnetFrame[1] = 0xff;
	Adapter->EnetFrame[2] = 0xff;
	Adapter->EnetFrame[3] = 0xff;
	Adapter->EnetFrame[4] = 0xff;
	Adapter->EnetFrame[5] = 0xff;

	// Source address
	Adapter->EnetFrame[6] = Adapter->AiNodeAddress[0];
	Adapter->EnetFrame[7] = Adapter->AiNodeAddress[1];
	Adapter->EnetFrame[8] = Adapter->AiNodeAddress[2];
	Adapter->EnetFrame[9] = Adapter->AiNodeAddress[3];
	Adapter->EnetFrame[10] = Adapter->AiNodeAddress[4];
	Adapter->EnetFrame[11] = Adapter->AiNodeAddress[5];

	// TypeLength
	TypeLength = ScanLen;
	Adapter->EnetFrame[12] = (UCHAR)(TypeLength >> 8); // For open-system
	Adapter->EnetFrame[13] = (UCHAR)(TypeLength);

	for (i=0;i<TypeLength;i++){
		Adapter->EnetFrame[14+i] = 0x55;
	}

	// Form a packet for authentication frame
	NdisAllocatePacket(&AllocationStatus,
			&Packet,
			Adapter->ReceivePacketPool);
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NDIS_SET_PACKET_HEADER_SIZE(Packet,	ETHERNET_HEADER_SIZE);

	NdisAllocateBuffer(&AllocationStatus,
				&Buffer,
				Adapter->ReceiveBufferPool,
				(PVOID)(&Adapter->EnetFrame), 
				(UINT)(TypeLength+14));

	// probably should do some error handling here
	ASSERT(AllocationStatus == NDIS_STATUS_SUCCESS);
	NdisChainBufferAtBack(Packet, Buffer);

	// Call SetupNextSend procedure.
    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	while ( (Status = SetupNextSend(Adapter, Packet, MGT_SCANTEST, 
						Adapter->CardSetting.EncryMode, STA_MGT, NULL)) ==
		NDIS_STATUS_RESOURCES ){
		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

		//ZD1205StallExecution(10);
		ZD1205_Sleep(1000);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

		if (Adapter->Notification & RQ_TERMINATION){
	        //KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    NdisFreeBuffer(Buffer);
		    NdisFreePacket(Packet);
			return;
		}
	}
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

	return;
}

void
AccLBRxCnt(			
			IN PZD1205_ADAPTER Adapter,
			IN PZD1205_DEFRAG_ENTITY DefragEntityPtr
			)
{
    PZD1205SwRfd		SwRfd;
	PWLAN_MANAGEMENT_FRM_HEADER_STRUC	pMgtFrm;

    SwRfd = (PZD1205SwRfd)QueueGetHead(
        &DefragEntityPtr->ElementList);
    pMgtFrm = (PWLAN_MANAGEMENT_FRM_HEADER_STRUC)&SwRfd->Rfd->RxBuffer;
    // More date in AP
    if (pMgtFrm->FrameCtrl[0] == MGT_LBTEST){
        if (pMgtFrm->FrameCtrl[1] & 0x40){  // WEP
            if ((SwRfd->Rfd->ActualCount-ENCAP_LEN_CIPHER_ON) == Adapter->LBFrameLen)
            {
        //JWEI 2004/03/22
            #if !fZD1211_LOOPBACK
                Adapter->LBRxCnt += (SwRfd->Rfd->ActualCount-ENCAP_LEN_CIPHER_ON); 
                // Minus CRC32(4), EXTRINFO(4), PLCP(4/5), IV(4), ICV(4)
            #else
                #if 0
            UINT    ii, jj = 0;

                for (ii = 0; ii < (UINT) Adapter->LBFrameLen + 2; ii ++)
                {
                    JWeiDbgPrint5(("0x%02X ", SwRfd->Rfd->RxBuffer[OFFSET1_SIZE_CIPHER_ON - 1 + ii]));
                    if (jj ++ == 10)
                    {
                        jj = 0;
                        JWeiDbgPrint5(("\n"));
                    }
                }
                JWeiDbgPrint5(("\n"));
                #endif
                Adapter->LBRxCnt ++;
            #endif
            }
            else{
                Adapter->LBErrFrame ++;
            }
        }
        else
        {                               // No WEP
            if ((SwRfd->Rfd->ActualCount-ENCAP_LEN_CIPHER_OFF) == Adapter->LBFrameLen)
            {
        //JWEI 2004/03/22
            #if !fZD1211_LOOPBACK
                Adapter->LBRxCnt += (SwRfd->Rfd->ActualCount-ENCAP_LEN_CIPHER_OFF);
                // Minus CRC32(4), EXTRINFO(4), PLCP(4/5)
            #else
            UINT    ii;
                #if 0
            UINT    jj = 0;

                for (ii = 0; ii < (UINT) Adapter->LBFrameLen + 2; ii ++)
                {
                    JWeiDbgPrint5(("0x%02X ", SwRfd->Rfd->RxBuffer[OFFSET1_SIZE_CIPHER_OFF - 1 + ii]));
                    if (jj ++ == 10)
                    {
                        jj = 0;
                        JWeiDbgPrint5(("\n"));
                    }
                }
                JWeiDbgPrint5(("\n"));
                #endif
                for (ii = 0; ii < (UINT) Adapter->LBFrameLen; ii ++)
                {
                    if (SwRfd->Rfd->RxBuffer[OFFSET1_SIZE_CIPHER_OFF + ii]
                        != ((cLBTEST_PATN + ii) & 0xFF))
                        break;
                }
                if (ii == (UINT) Adapter->LBFrameLen)
                    Adapter->LBRxCnt ++;
            #endif
            }
            else{
                Adapter->LBErrFrame ++;
            }
        }
    }
}
BOOLEAN
CheckSupportedRates(
			IN PZD1205_ADAPTER Adapter,
			UCHAR	rate
			)
{

#if defined(PHY_1202)
	switch (rate & 0x7f){
		case SR_1M:
		case SR_2M:
		case SR_5_5M:
		case SR_11M:
		case SR_16_5M:
		case SR_27_5M:
			status = TRUE;
			break;
		default:
			status = FALSE;
			break;		
	}
	return status;
#else
	return TRUE;
#endif
}

void
AdoptRegulatorySetting(
			IN PZD1205_ADAPTER Adapter,
			USHORT	BssIdx
			)
{
	UCHAR	index=5;
	UCHAR	i;

	if (Adapter->BSSInfo[BssIdx].CountryInfo[1] && Adapter->b802_11D){
		// Reset Adapter->AllowedChannel
		Adapter->AllowedChannel = 0;
		// Set first allowed channel as default channel.
		Adapter->AllowedChannel |= 
			(((ULONG)Adapter->BSSInfo[BssIdx].CountryInfo[5]) << 16);
		// Set Allowed-Channel and Maximum Transmit Power Level
		while (index < (Adapter->BSSInfo[BssIdx].CountryInfo[1]+2)){
			if (Adapter->BSSInfo[BssIdx].CountryInfo[index] == 0){ // Check if Pad?
				break;
			}
			for (i=0;i<Adapter->BSSInfo[BssIdx].CountryInfo[index+1];i++){
				Adapter->AllowedChannel |= 
					(1<<(Adapter->BSSInfo[BssIdx].CountryInfo[index]-1+i));
				Adapter->SetPoint[Adapter->BSSInfo[BssIdx].CountryInfo[index]-1+i] =
					ConvertToSetPoint(Adapter, 
						(signed char)Adapter->BSSInfo[BssIdx].CountryInfo[index+2]);
			}
			index+=3;
		}

	}
	return;
}

void
GetEnetFrameLock(
			IN PZD1205_ADAPTER Adapter
			)
{

	if (Adapter->Notification & RQ_SCANTEST){
		while (Adapter->EnetFrameLock){
			
			//ZD1205StallExecution(10);
			ZD1205_Sleep(1000);
			
			if (Adapter->Notification & RQ_TERMINATION){
				return;
			}
		}
		Adapter->EnetFrameLock = TRUE;
	}
}
USHORT
ConvertToSetPoint(
			IN PZD1205_ADAPTER Adapter,
			signed char	txpwr
			)
{
	USHORT	setpoint = 0;

	if (txpwr >= 19){
		setpoint = (USHORT)(20*ZDLog10multiply100(0xAB)/100);
	}
	else if (txpwr <= -3){
		setpoint = (USHORT)(20*ZDLog10multiply100(0x6d)/100);
	}
	else{
		switch(txpwr){
		case 18:
			setpoint = (USHORT)(20*ZDLog10multiply100(0xA4)/100);
			break;
		case 17:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x9A)/100);
			break;
		case 16:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x91)/100);
			break;
		case 15:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x8C)/100);
			break;
		case 14:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x88)/100);
			break;
		case 13:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x83)/100);
			break;
		case 12:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x80)/100);
			break;
		case 11:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x7C)/100);
			break;
		case 10:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x79)/100);
			break;
		case 9:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x79)/100);
			break;
		case 8:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x76)/100);
			break;
		case 7:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x76)/100);
			break;
		case 6:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x71)/100);
			break;
		case 5:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x71)/100);
			break;
		case 4:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x71)/100);
			break;
		case 3:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x71)/100);
			break;
		case 2:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x6E)/100);
			break;
		case 1:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x6d)/100);
			break;
		case 0:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x6d)/100);
			break;
		case -1:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x6d)/100);
			break;
		case -2:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x6d)/100);
			break;
		default:
			setpoint = (USHORT)(20*ZDLog10multiply100(0x6d)/100);
			break;
		}
	}
	return setpoint;
}

void
ClearOppositeRateInfo(PZD1205_ADAPTER Adapter)
{
	ZD1211_IRQL	OldIrql;

    DEBUGFUNC(_T("ClearOppositeRateInfo"));

	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
		Adapter->OppositeCnt = 0;
		NdisZeroMemory(Adapter->OppositeRateInfo, sizeof(OPPOSITE_RATE_INFO) * OPPOSITE_RATE_INFO_NUM);
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
}

BOOLEAN
BuildOppositeRateInfo(	PZD1205_ADAPTER Adapter,
						PUCHAR			OppositeSA,
						PUCHAR			SupportRate,
						USHORT			Capability,
						BOOLEAN			bUpdateExtSupportRate
						)
{
	UINT	i,j,k,L,m;
	UCHAR	length;
	ZD1211_IRQL	OldIrql;

	DEBUGFUNC(_T("BuildOppositeRateInfo"));

	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	for( i=0; i<Adapter->OppositeCnt; i++ ){
		if ( !memcmp(OppositeSA, Adapter->OppositeRateInfo[i].MacAddr, 6) )	
			break;
	}
	if ( (i == Adapter->OppositeCnt)&&(Adapter->OppositeCnt == OPPOSITE_RATE_INFO_NUM) ){
		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		return FALSE;
	}
 	if ((i==Adapter->OppositeCnt) || bUpdateExtSupportRate){
		// new dest address
		UCHAR tmpRate;

		memcpy(Adapter->OppositeRateInfo[i].MacAddr, OppositeSA, 6);
		ZDDbgPrint((_T("***New AP Rate for ")));
		ZDDbgPrint((_T("MAC: %x-%x-%x-%x-%x-%x\n"),
					OppositeSA[0],
					OppositeSA[1],
					OppositeSA[2],
					OppositeSA[3],
					OppositeSA[4],
					OppositeSA[5]
							));

		length = SupportRate[1];					    
		ZDDbgPrint((_T("SupportRate size = %d\n"), length));
		
		k = 0;
		L = 0;
		for( j=1; j<=length; j++ ){
			
			tmpRate = (SupportRate[1+j] & 0x7f);
		
			Adapter->OppositeRateInfo[i].SupportRateArray[L+Adapter->OppositeRateInfo[i].SupportRateArrayCount] = Rate_Convert(tmpRate, 0);
			L++;

			for (m = 0; m < Adapter->AllowRateArrayCount; m++){

				if ((Rate_Convert(tmpRate, 0) == Adapter->AllowRateArray[m])){

					if ((!(Adapter->bIBSS_G_MODE || Adapter->bIBSS_PureG_MODE)) && (Adapter->CardSetting.BSSType == INDEPENDENT_BSS)){
						// onlu allow 1M ~ 11M
						if (Rate_Convert(tmpRate, 0) > 0x3) break;
					}

					//this rate is allowed. add to RateArray
					//consider Supported Rate & Extended Supported Rate
					Adapter->OppositeRateInfo[i].RateArray[k+Adapter->OppositeRateInfo[i].RateArrayCount] = Rate_Convert(tmpRate, 0);
					k++;

					break;	
				}
			}

		}
		Adapter->OppositeRateInfo[i].RateArrayCount += (UCHAR)k;
		Adapter->OppositeRateInfo[i].SupportRateArrayCount += (UCHAR)L;
		Adapter->OppositeRateInfo[i].bJustRiseRate = FALSE;
		Adapter->OppositeRateInfo[i].RiseCondition = 0;
		Adapter->OppositeRateInfo[i].DownCondition = 0;
		Adapter->OppositeRateInfo[i].Capability = Capability;
		Adapter->OppositeRateInfo[i].MaxRate = Adapter->OppositeRateInfo[i].RateArrayCount - 1;
		//sort .RateArray[RATEARRAY_NUM]
		SortRateArray(Adapter->OppositeRateInfo[i].RateArray, Adapter->OppositeRateInfo[i].RateArrayCount);
	
		Adapter->OppositeRateInfo[i].CurrentRate = 
			Adapter->OppositeRateInfo[i].MaxRate;
		Adapter->OppositeRateInfo[i].ConsecutiveSuccessFrames = 0;
		Adapter->OppositeRateInfo[i].FailFrames = 0;
		
		if (!bUpdateExtSupportRate)
			Adapter->OppositeCnt++;
	}
	else{
		// OppositeRateInfo has been built before
		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		return FALSE;
	}

	Adapter->CardSetting.LastSentTxRate = // Notify Config tool
		Adapter->OppositeRateInfo[i].RateArray[Adapter->OppositeRateInfo[i].CurrentRate];

	if (Adapter->OppositeRateInfo[i].RateArrayCount == 0){
		Adapter->CardSetting.LastSentTxRate = 3;	//set to 11M
	}
	
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		
	ZDDbgPrint((_T("BuildOppositeRateInfo - end\n")));

	return TRUE;
}
