/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    mp_rid.h

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#ifndef _mP_rID_
#define _mP_rID_

/////////////////////////////////////////////////////////////////////
//
// Constant for RID identification
//
// N.B.: STA is station, AP is Access Point, and PRI is primary function
//


// Configuration Records
//
//  The following are configuration RIDs which are used to set certain
//  operational parameters. These are read/write RIDS. The values are stored in
//  volatile memory so it is not possible to preserve a configuration value change
//  across power-up/reset events.

#define RID_CFG_PORT_TYPE                       0xFC00          // STA
#define RID_MAC_ADDRESS                         0xFC01          // STA
#define RID_CFG_SSID                            0xFC02          // STA
#define RID_CFG_CHANNEL                         0xFC03
#define RID_CFG_OWN_SSID                        0xFC04          // IBSS creation (STA) or ESS (AP)
#define RID_CFG_OWN_ATIM                        0xFC05          // STA: ATIM window time for IBSS creation
#define RID_CFG_SYSTEM_SCALE            0xFC06          // STA: AP Density
#define RID_CFG_MAX_DATA_LENGTH         0xFC07
#define RID_CFG_WDS_ADDRESS                     0xFC08          // STA: MAC addr of WDS link node
#define RID_CFG_PM_ENABLE                       0xFC09          // STA: ESS PM on/off
#define RID_CFG_PM_EPS                          0xFC0A          // STA: ESS PM enhanced PM mode on/off
#define RID_CFG_MULTICAST_RCV           0xFC0B          // STA: ESS PM multicast rx on/off
#define RID_CFG_MAX_SLEEP_TIME          0xFC0C          // STA: max sleep time for ESS PM
#define RID_CFG_PM_HOLDOVER_TIME        0xFC0D          // STA: holdover time for ESS PM
#define RID_CFG_NAME                            0xFC0E          // identification text for diagnostics

#define RID_CFG_OWN_DTIM_PERIOD         0xFC10          // AP: Beacon interval between DTIMS
#define RID_CFG_WDS_ADDR_1                      0xFC11          // AP
#define RID_CFG_WDS_ADDR_2                      0xFC12          // AP
#define RID_CFG_WDS_ADDR_3                      0xFC13          // AP
#define RID_CFG_WDS_ADDR_4                      0xFC14          // AP
#define RID_CFG_WDS_ADDR_5                      0xFC15          // AP
#define RID_CFG_WDS_ADDR_6                      0xFC16          // AP
#define RID_CFG_MULTICAST_PM_BUFFERING  0xFC17  // AP: PM buffering of multicast messages

#define RID_CFG_DEFAULT_KEY_ID          0xFC23          // WEP: default key id used
#define RID_CFG_DEFAULT_KEY_1           0xFC24          // WEP: default key 1
#define RID_CFG_DEFAULT_KEY_2           0xFC25          // WEP: default key 2
#define RID_CFG_DEFAULT_KEY_3           0xFC26          // WEP: default key 3
#define RID_CFG_DEFAULT_KEY_4           0xFC27          // WEP: default key 4
#define RID_CFG_WEP_FLAGS                       0xFC28          // WEP: flags
#define RID_CFG_WEP_KEY_MAP                     0xFC29          // WEP: Key Mapping Table
#define RID_CFG_AUTHENTICATION          0xFC2A          // WEP: Use shared key authen

#define RID_CFG_ROAMING_MODE            0xFC2D          // Roaming control

#define RID_CFG_OWN_BEACON_INTERVAL 0xFC33              // HostAP/AP: Beacon interval between DTIMS

#define RID_CFG_TIM_CTRL                        0xFC40          // TIM Ctrl: set/clear TIM bit
#define RID_CFG_OWN_BSSID                       0xFC41          // HostIBSS: set BSSID

#define RID_GROUP_ADDR                          0xFC80          // STA: multicast MAC addrs for message RX.
#define RID_CREATE_IBSS                         0xFC81          // STA: IBSS creation on/off
#define RID_FRAG_THRESHOLD                      0xFC82          // STA: fragment length for unicast TX
#define RID_RTS_THRESHOLD                       0xFC83          // STA: frame length used for RTS/CTS control
#define RID_TX_RATE                                     0xFC84          // STA: TX rate control
#define RID_PROMISCUOUS_MODE                    0xFC85          // STA: Promiscuous mode on/off
#define RID_ANTENNA_SELECT						0xFC86  // Used to select antenna mode.
#define RID_TARCE_SET_POINT                     0xFC36
#define RID_TX_POWER_SET_POINT                  0xFC37

#define RID_FRAG_THRESHOLD_0            0xFC90          // AP Port 0
#define RID_FRAG_THRESHOLD_1            0xFC91          // AP Port 1
#define RID_FRAG_THRESHOLD_2            0xFC92          // AP Port 2
#define RID_FRAG_THRESHOLD_3            0xFC93          // AP Port 3
#define RID_FRAG_THRESHOLD_4            0xFC94          // AP Port 4
#define RID_FRAG_THRESHOLD_5            0xFC95          // AP Port 5
#define RID_FRAG_THRESHOLD_6            0xFC96          // AP Port 6

#define RID_RTS_THRESHOLD_0                     0xFC97          // AP port 0
#define RID_RTS_THRESHOLD_1                     0xFC98          // AP port 1
#define RID_RTS_THRESHOLD_2                     0xFC99          // AP port 2
#define RID_RTS_THRESHOLD_3                     0xFC9A          // AP port 3
#define RID_RTS_THRESHOLD_4                     0xFC9B          // AP port 4
#define RID_RTS_THRESHOLD_5                     0xFC9C          // AP port 5
#define RID_RTS_THRESHOLD_6                     0xFC9D          // AP port 6

#define RID_TX_RATE_CTL_0                       0xFC9E          // AP port 0
#define RID_TX_RATE_CTL_1                       0xFC9F          // AP port 1
#define RID_TX_RATE_CTL_2                       0xFCA0          // AP port 2
#define RID_TX_RATE_CTL_3                       0xFCA1          // AP port 3
#define RID_TX_RATE_CTL_4                       0xFCA2          // AP port 4
#define RID_TX_RATE_CTL_5                       0xFCA3          // AP port 5
#define RID_TX_RATE_CTL_6                       0xFCA4          // AP port 6

#define RID_CFG_PREAMBLE                        0xFCB0          // Preamble mode used by firmware
#define RID_CFG_BASIC_RATES                     0xFCB3          // Basic rates for AP and IBSS modes
#define RID_CFG_SUPP_RATES                      0xFCB4          // Supported rates reported in Mgmt frames

#define RID_TICK_TIME                           0xFCE0          // PRI: AUX timer tick interval
#define RID_SCAN_REQUEST                        0xFCE1          // Host desires a scan
#define RID_JOIN_REQUEST                        0xFCE2          // Host desires to join BSS

//
// Information Records
//
//  The information RIDS are read only.
//
#define RID_MAX_DOWNLOAD_TIME           0xFD00          // PRI: Maximum response time of download cmd
#define RID_DOWNLOAD_BUFFER                     0xFD01          // PRI: Download buffer location and size
#define RID_PRI_IDENTITY                        0xFD02          // PRI: Primary functions firmware ID
#define RID_PRI_SUP_RANGE                       0xFD03          // PRI: Primary functions interface; supplier
                                                                                                //              compatibility range.
#define RID_PRI_CFI_ACT_RANGE           0xFD04          // PRI: Controller interface; actor compat range
#define RID_NIC_SERIAL_NUM                      0xFD0A          // PRI: NIC serial number
#define RID_NIC_IDENTITY                        0xFD0B          // PRI: NIC ID
                                                                                                //              LUCENT: 1.1.1.0
                                                                                                //              ZD10: 1.1.0.0
                                                                                                //              FatNIC: 1.0.1.0
#define RID_MFI_SUP_RANGE                       0xFD0C          // PRI: Modem interface, supplier compat range
#define RID_CFI_SUP_RANGE                       0xFD0D          // PRI: Controller interface, supplier c range

#define RID_CHANNEL_LIST                        0xFD10          // Allowed communication channels.
#define RID_REGULATORY_DOMAINS          0xFD11          // List of intended regulatory domains.
#define RID_TEMP_TYPE                           0xFD12          // Hardware temp range code
#define RID_CIS                                         0xFD13          // PC Card standard card information structure

#define RID_STA_IDENTITY                        0xFD20          // STA: station functions firmware ID
#define RID_STA_SUP_RANGE                       0xFD21          // STA: station interface; supplier c range
#define RID_STA_MFI_ACTOR_RANGE         0xFD22          // STA: modem interface; Actor comp range
#define RID_STA_CFI_ACTOR_RANGE         0xFD23          // STA: controller interface; Actor comp range

#define RID_PORT_STATUS                         0xFD40          // STA: MAC port connection status
#define RID_CURRENT_SSID                        0xFD41          // STA: Current SSID
#define RID_CURRENT_BSSID                       0xFD42          // STA: Current BSSID
#define RID_COMMS_QUALITY                       0xFD43          // STA: Quality of the BSS connection
#define RID_CURR_TX_RATE                        0xFD44          // STA: current TX rate
#define RID_CURR_BEACON_INTERVAL        0xFD45          // Interval time for beacon TX and RX
#define RID_CURR_SCALE_THRESHOLD        0xFD46          // Actual system scale thresholds settings
#define RID_PROT_RSP_TIME                       0xFD47          // Max rsp time to req msgs
#define RID_SHORT_RETRY_LIMIT           0xFD48          // Max # TX attempts for short frames
#define RID_LONG_RETRY_LIMIT            0xFD49          // Max # TX attempts for long frame
#define RID_MAX_TX_LIFE                         0xFD4A          // Max TX frame handling duration
#define RID_MAX_RX_LIFE                         0xFD4B          // Max RX frame handling duration (fragments)
#define RID_CF_POLLABLE                         0xFD4C          // STA: CF pollable capability indication
#define RID_AUTH_ALGORITHMS                     0xFD4D          // available authentication algor indication
#define RID_WEP_AVAIL                           0xFD4F          // WEP option availability indication
#define RID_CURR_TX_RATE_1                      0xFD80          // AP: Port 1
#define RID_CURR_TX_RATE_2                      0xFD81          // AP: Port 2
#define RID_CURR_TX_RATE_3                      0xFD82          // AP: Port 3
#define RID_CURR_TX_RATE_4                      0xFD83          // AP: Port 4
#define RID_CURR_TX_RATE_5                      0xFD84          // AP: Port 5
#define RID_CURR_TX_RATE_6                      0xFD85          // AP: Port 6
#define RID_AP_OWN_MAC_ADDR                     0xFD86          // AP: unique local node MAC address
#define RID_PCF_INFO                            0xFD87          // AP: PCF capability information

#define RID_PHY_TYPE                            0xFDC0          // Physical layer type ID
#define RID_CURR_CHANNEL                        0xFDC1          // Current frequency channel used for TX
#define RID_CURR_PWR_STATE                      0xFDC2          // Current power consumption status
#define RID_CCA_MODE                            0xFDC3          // Clear channel assessment mode indication
#define RID_SUPPORTED_DATA_RATES        0xFDC6          // Data rates capability information

//AS: Dec 15 1999, added rid to read string file names
#define	RID_TxPowerGain							0xFE00

#define RID_EEPROM_WRITE						0xFFF0
#define	RID_EEPROM_SUBSYS_ID					0xFFF5
#define RID_EEPROM_REGION_CODE					0xFFF6
#define	RID_EEPROM_DEFAULT_CH					0xFFF7
#define	RID_EEPROM_ALLOW_CH						0xFFF8
#define RID_EEPROM_HW_VER						0xFFF9
#define RID_EEPROM_PWR_CAL_VAL					0xFFFA
#define	RID_EEPROM_MACADDR						0xFFFB			// Write EEPROM									
#define RID_EEPROM_REFRESH						0xFFFC			// Refresh EEPROM
#define RID_EEPROM_CONTENT						0xFFFD			// EEPROM Read/Write
#define RID_MONITOR								0xFFFE			// Monitor CRC32,...
#define RID_HEX_FILE_NAMES                      0xFFFF          // firmware hex file names

/*//////////////////////////////////////////////////////////////////////////////////////////////
//
// RID Support Structures
//
*/
#include <pshpack1.h>


// Note: Bits 6 and 5 are used to determine the Wep IV Reuse number with
//       the following coding:
//      bits:   6 5
//                      0 0  -  Increment IV for every frame
//                      0 1  -  Increment IV every 10 tx'd frames
//                      1 0  -  Increment IV every 50 tx'd frames
//                      1 1  -  Increment IV every 100 tx'd frames
//
//
// RID_CFG_ROAMING_MODE (0xFC2D)
//


typedef struct _DOWNLOAD_BUFFER_STRUCT {
        USHORT  page;
        USHORT  offset;
        USHORT  length;
} DOWNLOAD_BUFFER_STRUCT, *PDOWNLOAD_BUFFER_STRUCT;


// RID_PRI_IDENTY (0xFD02)
//
// RID_NIC_IDENTITY (0xFD0B)
//              LUCENT: 1.1.1.0
//              ZD10:   1.1.0.0
//              FatNIC: 1.0.1.0
//
// RID_STA_IDENTY (0xFD20) station functions firmware ID
//


typedef struct _ZD10_ID_STRUCT {
        USHORT  id;                                     // ID: 21..30
        USHORT  variant;                        // Variant number
        USHORT  major;                          // Major version
        USHORT  minor;                          // Minor version
} ZD10_ID_STRUCT, *PZD10_ID_STRUCT;




typedef struct _ZD10_RANGE_STRUCT {
        USHORT  role;                           // Role of implemented Primary functions firmware
        USHORT  id;                                     // ID of interface
        USHORT  variant;                        // Variant number of interface
        USHORT  bottom;                         // Low compatability number for interface
        USHORT  top;                            // High compatability number for interface
} ZD10_RANGE_STRUCT, *PZD10_RANGE_STRUCT;



/*
// RID_COMMS_QUALITY (0xFD43)
// STA: Quality of the BSS connection
//
// For the Lucent radio the signal and silence levels are raw numbers from the
// radio and need to be converted to dB with the following formula:
//              Lucent:
//                      Chopped the number between 47 & 139 (if lesser/greater then bounds make it bounds).
//                      subtract 149
//                      the result is your dB value.
//                      N.B.:  signal level and noise level are in a range of 27 to 154.
//                               To normalize to a dB value you need to Chopp at 47 and 139
//                               and then subtract 149 (they linearly map to dB... at least
//                               for lucent radios).
*/

typedef struct _COMMS_QUALITY_STRUCT {
        USHORT  snr;                            // Alreading in dB
        USHORT  avSignalLevel;
        USHORT  avSilenceLevel;
} COMMS_QUALITY_STRUCT, *PCOMMS_QUALITY_STRUCT;

//
// BSS INFO structure returned in scan result info frame
//

typedef struct _ZD10_BSS_INFO_STRUCT {
        USHORT  chid;
        USHORT  anl;
        USHORT  sl;
        UCHAR   bssid[6];
        USHORT  bcnInt;
        USHORT  cap;
        USHORT  ssidLength;
        UCHAR   ssid[32];
        UCHAR   supRates[8];
        USHORT  supRatesZeroStamp;
        USHORT  rxRate;
} ZD10_BSS_INFO_STRUCT, *PZD10_BSS_INFO_STRUCT;
#define SIZE_BSS_INFO_IN_WORDS          ((USHORT) (sizeof(ZD10_BSS_INFO_STRUCT)/2))

//
// RID_SCAN_RESULTS (0xF101)
//

typedef struct _SCAN_RESULTS_STRUCT {
        USHORT                                  reserved;
        USHORT                                  reason;
        ZD10_BSS_INFO_STRUCT    ZD10BssInfo[1];
} SCAN_RESULTS_STRUCT, *PSCAN_RESULTS_STRUCT;
#define SIZE_SCAN_RESULTS_IN_WORDS      ((USHORT) (sizeof(SCAN_RESULTS_STRUCT)/2))

/*
// RID_CURR_SCALE_THRESHOLD (0xFD46)
// Actual system scale thresholds settings
*/

typedef struct _CURR_SCALE_THRESHOLDS_STRUCT {
        USHORT energyDetect;            // Energy detect threshold used by the receiver hardware
        USHORT carrierDetect;           // Carrier detect threshold used by the receiver hardware
        USHORT defer;                           // Defer threshold value used by the receiver hardware
        USHORT cellSearch;                      // STA: Cell search threshold value used by the roaming
                                                                //              function of the receiver process to determine
                                                                //              if a better AP has to be searched for.
        USHORT deadSpot;                        // STA: Out of range threshold value used by the roaming
                                                                //              function of the receiver process to determine if
                                                                //              any other AP has to be searched for.
} CURR_SCALE_THRESHOLDS_STRUCT, *PCURR_SCALE_THRESHOLDS_STRUCT;

/*
// RID_AUTH_ALGORITHMS (0xFD4D)
// available authentication algorithms indication
*/

/*
// RID_PCF_INFO (0xFD87)
// AP: PCF capability information
*/

typedef struct _AP_PCF_INFO_STRUCT {
        USHORT mediumOccupancyLimit;    // max time in Kus a PC may control the medium without
                                                                        //  allowing at least one instance of DCF access
        USHORT cfpPeriod;                               // # of DTIMs between start of contention free periods
        USHORT cfpMaxDuration;                  // Maximum time in Kus of CFP duration.
} AP_PCF_INFO_STRUCT, *PAP_PCF_INFO_STRUCT;


// RID_INFO_COMM_TALLIES (0xF100)

typedef struct _TALLY_STRUCT {
        USHORT  txUnicast;
        USHORT  txMulticast;
        USHORT  txFrags;
        USHORT  txUniOctets;
        USHORT  txMultiOctets;
        USHORT  txDeferredTx;
        USHORT  txSingleRetries;
        USHORT  txMultipleRetries;
        USHORT  txRetryLimitExc;
        USHORT  txDiscards;

        USHORT  rxUnicast;
        USHORT  rxMulticast;
        USHORT  rxFrags;
        USHORT  rxUniOctets;
        USHORT  rxMultiOctets;
        USHORT  rxFCSErrors;
        USHORT  rxNoBuffer;
        USHORT  txWrongSA;
        USHORT  rxWEPUndecryp;
        USHORT  rxMsgInMsg;
        USHORT  rxMsgInBadMsg;
        USHORT  huntCmd;

        USHORT  rxOvr;
        USHORT  retryMany;
        USHORT  retrySingle;
        USHORT  retryExhaust;
        USHORT  crcError;
        USHORT  plcpCrc;
        USHORT  frameTooLong;
        USHORT  retryLong;
        USHORT  retryShort;
        USHORT  expReport;
} TALLY_STRUCT, *PTALLY_STRUCT;


typedef struct _LOCAL_TALLY_STRUCT {
        ULONGLONG  txUnicastFrames;
        ULONGLONG  txMulticastFrames;
        ULONGLONG  reserved1;
        ULONGLONG  txUniOctets;
        ULONGLONG  txMultiOctets;
        ULONGLONG  reserved2;
        ULONGLONG  reserved3;
        ULONGLONG  txMultipleRetriesFrames;
        ULONGLONG  txRetryLimitExceeded;
        ULONGLONG  reserved4;

        ULONGLONG  rxUnicastFrames;
        ULONGLONG  rxMulticastFrames;
        ULONGLONG  rxPLCPCRCErrCnt;
        ULONGLONG  rxUniOctets;
        ULONGLONG  rxMultiOctets;
        ULONGLONG  rxCRC32ErrCnt;
        ULONGLONG  rxDiscardedCnt;
        ULONGLONG  rxTotalCnt;
        ULONGLONG  rxDecrypFailCnt;
        ULONGLONG  reserved7;
        ULONGLONG  reserved8;

} LOCAL_TALLY_STRUCT, *PLOCAL_TALLY_STRUCT;

// From AP firmware to report association change of status of stations (0xF201)

typedef struct _ASSOC_STATUS {
        USHORT          assocStatus;
        UCHAR           staAddr[6];
        UCHAR           oldApAddr[6];                   // Optional: only if status is ASSOC_STATUS_STA_REASSOCIATED
} ASSOC_STATUS, *PASSOC_STATUS;

#define ASSOC_STATUS_STA_ASSOCIATED             1
#define ASSOC_STATUS_STA_REASSOCIATED   2
#define ASSOC_STATUS_STA_DISASSOCIATED  3

typedef struct _RID_STRUCT {
        USHORT          length;
        USHORT          rid;
        union {
                USHORT                                                  ushort;
                USHORT                                                  linkStatus;
                ASSOC_STATUS                                    assocStatus;
                TALLY_STRUCT                                    tally;
                DOWNLOAD_BUFFER_STRUCT                  download;
                ZD10_ID_STRUCT                                  id;
                ZD10_RANGE_STRUCT                               range;
                COMMS_QUALITY_STRUCT                    commsQ;
                CURR_SCALE_THRESHOLDS_STRUCT    scale;
                AP_PCF_INFO_STRUCT                              apPCF;
                SCAN_RESULTS_STRUCT                             scan;
                USHORT                                                  data[250];
        } u;
} RID_STRUCT, *PRID_STRUCT;
#include <poppack.h>


#endif  //_mP_rID_
