/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    mp_uif.h

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#ifndef _MPUIF_
#define _MPUIF_

// 1:Intel Flash;   0: MXIC, Winbond, AMD, Atmel...
#define cFLASH_MXIC             0
#define cFLASH_INTEL            1
#define cFLASH_MXIC_8M          2

// According to the docs. We define the user OIDs according to the following
//   guidelines:
//
// 0xAABBCCDD
//
//      where
//              AA - 0xFF for implementation specific information
//              BB - 0x01 operation characteristics
//                   0x02 statistics
//              CC - 0x01 Required  (Only have this type be defined)
//                   0x02 Optional
//              DD - differentiates each OID when other fields are the same
//

#define OID_ZD_GETRID                                 0xFF010181
#define OID_ZD_SETRID                                 0xFF010181

#define OID_ZD_CMD                                    0xFF010183

#define OID_ZD_READ_AUX                               0xFF010186
#define OID_ZD_WRITE_AUX                              0xFF010186

#define OID_ZD_BLOCK                                  0xFF010187
#define OID_ZD_UNBLOCK                                0xFF010188

#define OID_ZD_DOWNLOAD_RAM                           0xFF01018B
#define OID_ZD_DOWNLOAD_ROM                           0xFF01018C

#define OID_ZD_GET_TALLIES                            0xFF02018D
#define OID_ZD_SET_TALLIES                            0xFF02018D

#define OID_ZD_DEFAULT_KEY_ID                         0xFF020190
#define OID_ZD_FW_VERSION                              0xFF020196
#define OID_ZD_ADAPTER_SOFT_BOOT                        0xFF020198

// Some Object Identifiers for debugging.
#define OID_ZD_TEST                                     0xFF0201A0
#define OID_ZD_TEST1                                    0xFF0201A1
#define OID_ZD_IO_WRITE                                 0xFF0201A2
#define OID_ZD_IO_WRITE_BYTE                            0xFF0201A3
#define OID_ZD_IO_READ                                  0xFF0201A4
#define OID_ZD_IO_READ_BYTE                             0xFF0201A5
#if DBG
#define OID_ZD_SET_DEBUG_PRINT                          0xFF0201A6
#endif
#define OID_ZD_RESET_TALLIES                            0xFF0201A7
#define OID_ZD_IO32                                     0xFF0201AC
#define OID_ZD_TEST2                                    0xFF0201AD

#define OID_ZD_802_11_BSSID_LIST_SCAN                   0xFF0201B0
#define OID_ZD_802_11_BSSID_LIST                        0xFF0201B1
#define OID_ZD_802_11_SSID                              0xFF0201B2
#define OID_ZD_802_11_INFRASTRUCTURE_MODE               0xFF0201B3
#define OID_ZD_802_11_ADD_WEP                           0xFF0201B4
#define OID_ZD_802_11_WEP_STATUS                        0xFF0201B5
#define OID_ZD_802_11_REMOVE_WEP                        0xFF0201B6
#define OID_ZD_802_11_DISASSOCIATE                      0xFF0201B7
#define OID_ZD_802_11_AUTHENTICATION_MODE               0xFF0201B8

#define OID_LINK_QUALITY                                0xFF02019E
#define OID_LINK_SIGNAL_STRENGTH						0xFF02019F

#define	OID_ZD_GET_STA_LIST								0xFF0201C0
#define	OID_ZD_FILTER_LIST								0xFF0201C1
#define	OID_ZD_FILTER_MODE								0xFF0201C2
#define	OID_ZD_FIRMWARE_RELOAD							0xFF0201AE
#define	OID_ZD_AP_MODE									0xFF0201AF
#define	OID_ZD_GET_TALLIES_1202							0xFFFFFFFF

#define	CMD_ContinuousTx								0x40
#define CMD_AdjustPSTxPower								0x41
#define CMD_RateControl									0x42
#define CMD_QueryTxPwr									0x43
#define CMD_QueryIntValue                               0x45
#define	CMD_DOZE_FOREVER								0x46

//
// Structure used for OID_MP_CMD
//
typedef struct _ZD_CMD_STRUCT {
        union {
                struct {
                        USHORT  cmd;
                        USHORT  param0;
                        USHORT  param1;
                        USHORT  param2;
                } send;
                struct {
                        USHORT  status;
                        USHORT  resp0;
                        USHORT  resp1;
                        USHORT  resp2;
                } rsp;
        } u;
} ZD_CMD_STRUCT, *PZD_CMD_STRUCT;

typedef struct _ZD_IMAGE_BLOCK_STRUCT {
        ULONG   address;
        USHORT  size;
        UCHAR   buffer[1];
} ZD_IMAGE_BLOCK_STRUCT, *PZD_IMAGE_BLOCK_STRUCT;


#define	OID_ZD_CUSTOM	0xFF129901
#define	OID_ZD_RD		0xFF129902

typedef enum _ZD_CUSTOM_FUNC_ID
{
	ZDAdapterOperationMode		= 0x00,

	ZDLinkStatus				= 0x10,
	ZDCommQuality				= 0x11,

	ZDPreambleMode				= 0x30,
	ZDDesiredSSID				= 0x31,
	ZDAdapterRegion				= 0x32,
	ZDAdapterSupportChannel		= 0x33,
	ZDWepInfo					= 0x34,
	ZDMultiDomainCapability		= 0x35,

	ZDGetAccessPointStationList	= 0x50,
	ZDAccessPointFilterMode		= 0x51,
	ZDAccessPointFilterList		= 0x52,
	ZDAccessPointHideSSID       = 0x53,
	ZDAccessPointTxPower		= 0x54,
	ZDAccessPointBasicRate		= 0x55,
	ZDDelOneStationFromAPStationList = 0x60,

	ZDFirmwareVersion			= 0x70,
	ZDUseZDXOid                 = 0x71,
	ZDRadioState				= 0x72,
	ZDUSBType                   = 0x73,
	ZDTxPowerLevel              = 0x74,
	ZDAdhocMode                 = 0x75,
    ZDUseGinUsb1_1                 = 0x76,

	ZDIssueDeAuthentication		= 0x80

} ZD_CUSTOM_FUNC_ID, *PZD_CUSTOM_FUNC_ID;

typedef enum _ZD_OPERATION_MODE
{
	ZD_ModeStation,
	ZD_ModeAccessPoint,
	ZD_ModeMax
} ZD_OPERATION_MODE, *PZD_OPERATION_MODE;


typedef enum _ZD_USB_TYPE
{
    ZD_USB11,
    ZD_USB20
} ZD_RADIO_STATE, *PZD_RADIO_STATE;

// Function ID define for OID_ZD_RD
#define	ZDAccessPHYRegister4B		0
#define	ZDAccessPHYRegister2B		1
#define	ZDAccessPHYRegister1B		2
#define	ZDAccessMACRegister4B		3
#define	ZDAccessMACRegister2B		4
#define	ZDAccessMACRegister1B		5
#define	ZDAccessROMData				6
#define	ZDROMUpdate					7
#define	ZDContinuousTx				8
#define ZDGetNICAdapterTally		9
#define ZDSetMACAddress				10
#define ZDBootCodeUpdate			11
#define ZDFlashErase				12
#define ZDFlashProgram				13
#define ZDFlashRead					14
#define ZDEEPROMDataWrite			15
#define ZDTxPowerGainControl		16
#define ZDFlashIDQuery				17
#define ZDFlashGetChkSum			18
#define ZDFlashGetSubChkSum			19


typedef enum _ZD_LINK_STATUS
{
	ZD_LinkStatus_Disable		= 1,
	ZD_LinkStatus_Searching,
	ZD_LinkStatus_ConnectIBSS,
	ZD_LinkStatus_ConnectESS,
	ZD_LinkStatus_OutOfRange,
	ZD_LinkStatus_ConnectWDS,
	ZD_LinkStatus_Reserved7,
	ZD_LinkStatus_StartHostAP,
	ZD_LinkStatus_ConnectPIBSS
} ZD_LINK_STATUS, *PZD_LINK_STATUS;

typedef enum _ZD_802_11_PREAMBLE_MODE
{
    ZD_PreambleLong,
    ZD_PreambleShort,
    ZD_PreambleAuto
} ZD_802_11_PREAMBLE_MODE, *PZD_802_11_PREAMBLE_MODE;

typedef enum _ZD_ALLOWED_CHANNEL
{
	ZD_CH_3_9   = 0x01FC,
	ZD_CH_10_11 = 0x0600,
	ZD_CH_1_11	= 0x07FF,
	ZD_CH_10_13 = 0x1E00,
	ZD_CH_1_13  = 0x1FFF,
	ZD_CH_1_14	= 0x3FFF
} ZD_ALLOWED_CHANNEL, *PZD_ALLOWED_CHANNEL;
	
typedef enum _ZD_REGION
{
	ZD_Region_Default			= 0x00,//ch1-11
	ZD_Region_USA				= 0x10,//ch1-11
	ZD_Region_Canada			= 0x20,//ch1-11
	ZD_Region_EuropeAustralia	= 0x30,
	ZD_Region_Spain				= 0x31,// ETSI ch1-13
	ZD_Region_France			= 0x32,//and Singapore ch10-13
	ZD_Region_Japan				= 0x40,// ch1-14
	ZD_Region_Israel			= 0x50,// ch3-9
	ZD_Region_Mexico			= 0x51 // ch10,11
} ZD_REGION, *PZD_REGION;


#define	ContTx_Normal				0
#define	ContTx_CW					1
#define	ContTx_CarrierSuppression	2

#define	ContTx_Start				0
#define	ContTx_Stop					1

#define OID_ZDX_802_11_BSSID                    0xFF12FF01		// 1
#define OID_ZDX_802_11_SSID                     0xFF12FF02		// 2
#define OID_ZDX_802_11_INFRASTRUCTURE_MODE      0xFF12FF08		// 3 
#define OID_ZDX_802_11_CONFIGURATION            0xFF12FF11		// 4
#define OID_ZDX_802_11_ADD_WEP                  0xFF12FF13		// 5
#define OID_ZDX_802_11_REMOVE_WEP               0xFF12FF14		// 6
#define OID_ZDX_802_11_DISASSOCIATE             0xFF12FF15		// 7
#define OID_ZDX_802_11_AUTHENTICATION_MODE      0xFF12FF18		// 8
#define OID_ZDX_802_11_BSSID_LIST_SCAN          0xFF12FF1A		// 9
#define OID_ZDX_802_11_WEP_STATUS               0xFF12FF1B		// 10
#define OID_ZDX_802_11_ADD_KEY                  0xFF12FF1D		// 11
#define OID_ZDX_802_11_REMOVE_KEY               0xFF12FF1E		// 12
#define OID_ZDX_802_11_ASSOCIATION_INFORMATION  0xFF12FF1F		// 13
#define OID_ZDX_802_11_TEST                     0xFF12FF20		// 14
#define OID_ZDX_802_11_WPA_MIC_ERROR            0xFF12FF21		// 15


#endif // _MPUIF_
