/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    parse.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/
#include "precomp.h"
#define NUM_REG_PARAM ( sizeof (ZD1205GlobalRegTab) / sizeof (ZD1205RegTabType) )

//-----------------------------------------------------------------------------
// ZD1205RegTabType
//
//      One instance of this structure will be used for every configuration
//      parameter that this driver supports.  The table contains all of the
//      relavent information about each parameter:  Name, whether or not it is
//      required, where it is located in the "Adapter" structure, the size of
//      the parameter in bytes, the default value for the parameter, and what
//      the minimum and maximum values are for the parameter.  In the debug
//      version of the driver, this table also contains a field for the ascii
//      name of the parameter.
//-----------------------------------------------------------------------------
typedef struct _ZD1205RegTabType {

    NDIS_STRING RegVarName;             // variable name text
    char       *RegAscName;             // variable name text
    UINT        Mandantory;             // 1 -> manditory, 0 -> optional
    UINT        FieldOffset;            // offset to ZD1205_ADAPTER field loaded
    UINT        FieldSize;              // size (in bytes) of the field
    UINT        Default;                // default value to use
    UINT        Min;                    // minimum value allowed
    UINT        Max;                    // maximum value allowed

} ZD1205RegTabType;


//-----------------------------------------------------------------------------
// Global Registry Parameters Table
//
//      This table contains a list of all of the configuration parameters
//      that the driver supports.  The driver will attempt to find these
//      parameters in the registry and use the registry value for these
//      parameters.  If the parameter is not found in the registry, then the
//      default value is used.
//
//-----------------------------------------------------------------------------

ZD1205RegTabType ZD1205GlobalRegTab[ ] = {
    //                                                             Offset in to Adapter Struct                                 Default Value                   Min          Max
#if DBG
    //{NDIS_STRING_CONST("Debug"),            "Debug",            0, ZD1205_OFFSET(Debug),           ZD1205_SIZE(Debug),             DBG_NORMAL,                      0,          0xffffffff},
#endif
    //{NDIS_STRING_CONST("BusNumber"),        "BusNumber",        0, ZD1205_OFFSET(BusNumber),       ZD1205_SIZE(BusNumber),         0,                               0,          16},
    //{NDIS_STRING_CONST("SlotNumber"),       "SlotNumber",       0, ZD1205_OFFSET(AiSlot),          ZD1205_SIZE(AiSlot),            0,                               0,          32},
    {NDIS_STRING_CONST("NumRfd"),           "NumRfd",           0, ZD1205_OFFSET(NumRfd),          ZD1205_SIZE(NumRfd),            32,                              1,          MAX_RECEIVE_DESCRIPTORS},
    {NDIS_STRING_CONST("NumTcb"),           "NumTcb",           0, ZD1205_OFFSET(RegNumTcb),       ZD1205_SIZE(RegNumTcb),         16,                              1,          0x40},
    {NDIS_STRING_CONST("NumTbdPerTcb"),     "NumTbdPerArray",   0, ZD1205_OFFSET(NumTbdPerTcb),    ZD1205_SIZE(NumTbdPerTcb),      8,                               1,          MAX_PHYS_DESC},
    {NDIS_STRING_CONST("NumCoalesce"),      "NumCoalesce",      0, ZD1205_OFFSET(NumCoalesce),     ZD1205_SIZE(NumCoalesce),       16,                               1,         0x40},
#ifndef HOST_IF_USB
    //{NDIS_STRING_CONST("MapRegisters"),     "MapRegisters",     0, ZD1205_OFFSET(NumMapRegisters), ZD1205_SIZE(NumMapRegisters),   64,                              0,          0xffff},
#endif
	/*
    //{NDIS_STRING_CONST("PhyAddress"),       "PhyAddress",       0, ZD1205_OFFSET(PhyAddress),      ZD1205_SIZE(PhyAddress),        0xFF,                            0,          0xFF},
    //{NDIS_STRING_CONST("Connector"),        "Connector",        0, ZD1205_OFFSET(Connector),       ZD1205_SIZE(Connector),         0,                               0,          0x2},
    //{NDIS_STRING_CONST("BusTypeLocal"),     "BusTypeLocal",     0, ZD1205_OFFSET(AiBusType),       ZD1205_SIZE(AiBusType),         5,                               EISABUS,    PCIBUS},
    //{NDIS_STRING_CONST("TxFifo"),           "TxFifo",           0, ZD1205_OFFSET(AiTxFifo),        ZD1205_SIZE(AiTxFifo),          DEFAULT_TX_FIFO_LIMIT,           0,          15},
    //{NDIS_STRING_CONST("RxFifo"),           "RxFifo",           0, ZD1205_OFFSET(AiRxFifo),        ZD1205_SIZE(AiRxFifo),          DEFAULT_RX_FIFO_LIMIT,           0,          15},
    //{NDIS_STRING_CONST("TxDmaCount"),       "TxDmaCount",       0, ZD1205_OFFSET(AiTxDmaCount),    ZD1205_SIZE(AiTxDmaCount),      0,                               0,          63},
    //{NDIS_STRING_CONST("RxDmaCount"),       "RxDmaCount",       0, ZD1205_OFFSET(AiRxDmaCount),    ZD1205_SIZE(AiRxDmaCount),      0,                               0,          63},
    //{NDIS_STRING_CONST("UnderrunRetry"),    "UnderrunRetry",    0, ZD1205_OFFSET(AiUnderrunRetry), ZD1205_SIZE(AiUnderrunRetry),   DEFAULT_UNDERRUN_RETRY,          0,          3},
    {NDIS_STRING_CONST("ForceDpx"),         "ForceDpx",         0, ZD1205_OFFSET(AiForceDpx),      ZD1205_SIZE(AiForceDpx),        0,                               0,          2},
    {NDIS_STRING_CONST("Speed"),            "Speed",            0, ZD1205_OFFSET(AiTempSpeed),     ZD1205_SIZE(AiTempSpeed),       0,                               0,          100},
    {NDIS_STRING_CONST("Threshold"),        "Threshold",        0, ZD1205_OFFSET(AiThreshold),     ZD1205_SIZE(AiThreshold),       200,                             0,          200},
    {NDIS_STRING_CONST("MCWA"),             "MCWA",             0, ZD1205_OFFSET(McTimeoutFlag),   ZD1205_SIZE(McTimeoutFlag),     2,                               0,          2},
    {NDIS_STRING_CONST("MWIEnable"),        "MWIEnable",        0, ZD1205_OFFSET(MWIEnable),       ZD1205_SIZE(MWIEnable),         1,                               0,          1},
    {NDIS_STRING_CONST("Congest"),          "Congest",          0, ZD1205_OFFSET(Congest),         ZD1205_SIZE(Congest),           0,                               0,          0x1},
    {NDIS_STRING_CONST("DisableTx"),        "DisableTx",        0, ZD1205_OFFSET(DisableTx),       ZD1205_SIZE(DisableTx),         0,								0,          0x1},
    {NDIS_STRING_CONST("PreambleMode"),     "PreambleMode",     0, ZD1205_OFFSET(preambleMode),    ZD1205_SIZE(preambleMode),      3,								1,          0x3},
    {NDIS_STRING_CONST("IBSS_G_Mode"),      "IBSS_G_Mode",      0, ZD1205_OFFSET(bIBSS_G_MODE),    ZD1205_SIZE(bIBSS_G_MODE),      0,								0,          1},
    {NDIS_STRING_CONST("IBSS_PureG_Mode"),  "IBSS_PureG_Mode",  0, ZD1205_OFFSET(bIBSS_PureG_MODE),ZD1205_SIZE(bIBSS_PureG_MODE),  0,								0,          1},
    {NDIS_STRING_CONST("Use_G_Mode_in_Usb1.1"),  "Use_G_Mode_in_Usb1.1",  0, ZD1205_OFFSET(busbswitch),ZD1205_SIZE(busbswitch),    0,								0,          1},
    {NDIS_STRING_CONST("ImmDiscAfterRFOff"),  "ImmDiscAfterRFOff",  0, ZD1205_OFFSET(bImmDiscAfterRFOff),ZD1205_SIZE(bImmDiscAfterRFOff),    0,						0,          1},
	{NDIS_STRING_CONST("AutoRateMonitorInterval"),"AutoRateMonitorInterval", 0, ZD1205_OFFSET(autoRateMonitorInterval),     ZD1205_SIZE(autoRateMonitorInterval), 200, 100, 5000}, // The following 6 items are added for Rate Adaption on 4811B
	{NDIS_STRING_CONST("ERThresholdAt36M"),"ERThresholdAt36M", 0, ZD1205_OFFSET(RateLine1),     ZD1205_SIZE(RateLine1), 100, 20, 150},
	{NDIS_STRING_CONST("ERThresholdAt24M"),"ERThresholdAt24M", 0, ZD1205_OFFSET(RateLine2),     ZD1205_SIZE(RateLine2), 10, 5, 20},
	{NDIS_STRING_CONST("ERThresholdAt18M"),"ERThresholdAt18M", 0, ZD1205_OFFSET(RateLine3),     ZD1205_SIZE(RateLine3), 3, 2, 5},
	{NDIS_STRING_CONST("RiseConditionThreshold"),"RiseConditionThreshold", 0, ZD1205_OFFSET(RiseConditionThreshold),     ZD1205_SIZE(RiseConditionThreshold), 5, 2, 10},
	{NDIS_STRING_CONST("DownConditionThreshold"),"DownConditionThreshold", 0, ZD1205_OFFSET(DownConditionThreshold),     ZD1205_SIZE(DownConditionThreshold), 3, 1, 8},
	{NDIS_STRING_CONST("Radio"),"Radio", 0, ZD1205_OFFSET(ZDConfigTurnOffRF),     ZD1205_SIZE(ZDConfigTurnOffRF), 0, 0, 1},
	{NDIS_STRING_CONST("PSMode"),"PSMode", 0, ZD1205_OFFSET(PSMode),     ZD1205_SIZE(PSMode), 0, 0, 1},
	{NDIS_STRING_CONST("RTSCTSThreshold"),"RTSCTSThreshold", 0, ZD1205_OFFSET(rTSThreshold),     ZD1205_SIZE(rTSThreshold), 2347, 0, 2347},
	{NDIS_STRING_CONST("FragmentThreshold"),"FragmentThreshold", 0, ZD1205_OFFSET(fragThreshold),     ZD1205_SIZE(fragThreshold), 2346, 256, 2346},
	{NDIS_STRING_CONST("IBSSChannelNumber"),"IBSSChannelNumber", 0, ZD1205_OFFSET(iBSSChannel),     ZD1205_SIZE(iBSSChannel), 1, 1,14},
	{NDIS_STRING_CONST("Rate"),"Rate", 0, ZD1205_OFFSET(defaultTxRate),     ZD1205_SIZE(defaultTxRate), 0, 0,108},
	{NDIS_STRING_CONST("LedIndicatorLevel1"),"LedIndicatorLevel1", 0, ZD1205_OFFSET(LedIndicatorLevel1),     ZD1205_SIZE(LedIndicatorLevel1), 10, 0,100},
	{NDIS_STRING_CONST("LedIndicatorLevel2"),"LedIndicatorLevel2", 0, ZD1205_OFFSET(LedIndicatorLevel2),     ZD1205_SIZE(LedIndicatorLevel2), 30, 0,100},
	{NDIS_STRING_CONST("LedIndicatorLevel3"),"LedIndicatorLevel3", 0, ZD1205_OFFSET(LedIndicatorLevel3),     ZD1205_SIZE(LedIndicatorLevel3), 50, 0,100},
	{NDIS_STRING_CONST("LedIndicatorLevel4"),"LedIndicatorLevel4", 0, ZD1205_OFFSET(LedIndicatorLevel4),     ZD1205_SIZE(LedIndicatorLevel4), 70, 0,100},
	{NDIS_STRING_CONST("LedIndicatorLevel5"),"LedIndicatorLevel5", 0, ZD1205_OFFSET(LedIndicatorLevel5),     ZD1205_SIZE(LedIndicatorLevel5), 90, 0,100},
	{NDIS_STRING_CONST("ReducePowerLevel"),"ReducePowerLevel", 0, ZD1205_OFFSET(ReducePowerLevel),     ZD1205_SIZE(ReducePowerLevel), 0, 0,48},
	{NDIS_STRING_CONST("EnableAntennaDiversity"),"EnableAntennaDiversity", 0, ZD1205_OFFSET(EnableSwAntennaDiv),     ZD1205_SIZE(EnableSwAntennaDiv), 0, 0,1},
	{NDIS_STRING_CONST("Ant_MonitorDur1"),"Ant_MonitorDur1", 0, ZD1205_OFFSET(Ant_MonitorDur1),     ZD1205_SIZE(Ant_MonitorDur1), 100, 50,1000},
	{NDIS_STRING_CONST("Ant_MonitorDur2"),"Ant_MonitorDur2", 0, ZD1205_OFFSET(Ant_MonitorDur2),     ZD1205_SIZE(Ant_MonitorDur2), 1, 1,30},
	{NDIS_STRING_CONST("NiceSQThr"),"NiceSQThr", 0, ZD1205_OFFSET(NiceSQThr),     ZD1205_SIZE(NiceSQThr), 48, 32,72},
	//PSMode
#ifdef HOST_IF_USB
    {NDIS_STRING_CONST("TxRxEPRestartWaitTime"),	"TxRxEPRestartWaitTime",	0,	ZD1205_OFFSET(TxRxEPRestartWaitTime),	ZD1205_SIZE(TxRxEPRestartWaitTime),	50,	0,			1000},
	
#endif
*/
};



NDIS_STATUS
ParseRegistryParameters(
                        IN PZD1205_ADAPTER Adapter,
                        IN NDIS_HANDLE ConfigHandle
                        );


//-----------------------------------------------------------------------------
// Procedure:   ParseRegistryParameters
//
// Description: This routine will parse all of the parameters out of the
//              registry/PROTOCOL.INI, and store the values in the "Adapter"
//              Structure.  If the parameter is not present in the registry,
//              then the default value for the parameter will be placed into
//              the "Adapter" structure.  This routine also checks the validity
//              of the parameter value, and if the value is out of range, the
//              driver will the min/max value allowed.
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//      ConfigHandle - NDIS Configuration Registery handle
//
// Returns:
//      NDIS_STATUS_SUCCESS - All mandatory parameters were parsed
//      NDIS_STATUS_FAILED - A mandatory parameter was not present
//-----------------------------------------------------------------------------

NDIS_STATUS
ParseRegistryParameters(
                        IN PZD1205_ADAPTER Adapter,
                        IN NDIS_HANDLE ConfigHandle
                        )

{
    NDIS_STATUS         Status;
    ZD1205RegTabType      *RegTab;
    UINT                i;
    UINT                value;
    PUCHAR              fieldPtr;
	// For country code read
	NDIS_STRING			RegCountry = NDIS_STRING_CONST("Country");
	ANSI_STRING			AnsiString;
	UCHAR				AnsiBuf[64];
//#if DBG
    char                ansiRegName[32];
//#endif
    PNDIS_CONFIGURATION_PARAMETER ReturnedValue;
	DEBUGFUNC(_T("ParseRegistryParameters"));

	// Initialize ANSI string buffer.
	AnsiString.Length = 0;
	AnsiString.MaximumLength = sizeof(AnsiBuf);
	AnsiString.Buffer = AnsiBuf;


    INITSTR((_T("\n")));


    // Grovel through the registry parameters and aquire all of the values
    // stored therein.
    for (i = 0, RegTab = ZD1205GlobalRegTab; i < NUM_REG_PARAM; i++, RegTab++)
    {
        fieldPtr = ((PUCHAR) Adapter) + RegTab->FieldOffset;
        strcpy(ansiRegName, RegTab->RegAscName);

        // Get the configuration value for a specific parameter.  Under NT the
        // parameters are all read in as DWORDs.
        NdisReadConfiguration(&Status,
            &ReturnedValue,
            ConfigHandle,
            &RegTab->RegVarName,
            NdisParameterInteger);

        // If the parameter was present, then check its value for validity.
        if (Status == NDIS_STATUS_SUCCESS)
        {
            // Check that param value is not too small or too large
            if (ReturnedValue->ParameterData.IntegerData < RegTab->Min ||
                ReturnedValue->ParameterData.IntegerData > RegTab->Max)
            {
                value = RegTab->Default;
#if DBG
				DbgPrint("The entry exist, but exceed limit, use default:%-25s %d\n", ansiRegName, value); // jxiaoDbg
#endif
            }

            // Use the value if it is within range
            else
            {
                value = ReturnedValue->ParameterData.IntegerData;
#if DBG
				DbgPrint("Use Reg setting:%-25s %d\n", ansiRegName, value); // jxiaoDbg
#endif
            }

        }
        // If a mandatory parameter wasn't present then error out.
        else if (RegTab->Mandantory)
        {
            DEBUGSTR((_T("Could not find mandantory '%s' in registry\n\n"), ansiRegName));
                return (NDIS_STATUS_FAILURE);
        }
        // If a non-mandatory parameter wasn't present, then set it to its
        // default value.
        else
        {
            value = RegTab->Default;
            //INITSTR(("%-25s 0x%X\n", ansiRegName, value));
#if DBG // jxiaoDbg
			DbgPrint("Cound not find optional entry %s in registry, use default value:%d\n", ansiRegName,value); // jxiaoDbg
#endif
        }

        // Store the value in the adapter structure.
        switch (RegTab->FieldSize)
        {
        case 1:
            {
                *((PUCHAR) fieldPtr) = (UCHAR) value;
                break;
            }
        case 2:
            {
                *((PUSHORT) fieldPtr) = (USHORT) value;
                break;
            }
        case 4:
            {
                *((PULONG) fieldPtr) = (ULONG) value;
                break;
            }
        default:
            DEBUGSTR((_T("Bogus field size %d\n"), RegTab->FieldSize));
            break;
        }
		//DbgPrint("Set %s to %u\n",ansiRegName, *((PULONG) fieldPtr) );
    }
	INITSTR((_T("Adapter->NumTcb = %d,  Adapter->NumtbdPerTcb = %d\n"),Adapter->NumTcb,Adapter->NumTbdPerTcb));

	NdisReadConfiguration( &Status,
                           &ReturnedValue,
                           ConfigHandle,
                           &RegCountry,
                           NdisParameterString);

    if ( Status != NDIS_STATUS_SUCCESS ){
#if DBG // jxiaoDbg
		DbgPrint("Can't read country code in s/w registry\n");
#endif
		//NdisCloseConfiguration(ConfigHandle);
		//NdisOpenConfiguration(&Status,
		//	&ConfigHandle,
		//	Adapter->ZD1205WrapperConfigurationContext);
		Adapter->CountryCode[0]=0xFF; // The country code is not assigned in s/w registry.
	}else{
        NdisUnicodeStringToAnsiString(&AnsiString, &ReturnedValue->ParameterData.StringData);
        strcpy(Adapter->CountryCode, AnsiString.Buffer);
#if DBG // jxiaoDbg
        DbgPrint("Country setting in Registry: %s\n", Adapter->CountryCode); // jxiaoDbg
#endif
	}
    return NDIS_STATUS_SUCCESS;
}

