/****************************************************************************
** COPYRIGHT (C) 2002 ZYDAS CORPORATION                                    **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    pci.c

This driver runs on the following hardware:
	ZD1205

Environment:
    Kernel Mode - Or whatever is the equivalent on WinNT

Revision History

*****************************************************************************/

#include "precomp.h"

//-----------------------------------------------------------------------------
// Procedure:   FindPciDevice50Scan
//
// Description: This routine finds an adapter for the driver to load on
//              The critical piece to understanding this routine is that
//              the System will not let us read any information from PCI
//              space from any slot but the one that the System thinks
//              we should be using. The configuration manager rules this
//              land... The Slot number used by this routine is just a
//              placeholder, it could be zero even.
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//      VendorID - Vendor ID of the adapter.
//      DeviceID - Device ID of the adapter.
//      PciCardsFound - A structure that contains an array of the IO addresses,
//                   IRQ, and node addresses of each PCI card that we find.
//
//    NOTE: due to NT 5's Plug and Play configuration manager
//          this routine will never return more than one device.
//
// Returns:
//      USHORT - Number of ZD1205 based PCI adapters found in the scanned bus
//-----------------------------------------------------------------------------

#define NIC_RESOURCE_BUF_SIZE           (sizeof(NDIS_RESOURCE_LIST) + \
                                        (10*sizeof(CM_PARTIAL_RESOURCE_DESCRIPTOR)))
USHORT
FindPciDevice50Scan(
                    IN PZD1205_ADAPTER Adapter,
                    IN USHORT     VendorID,
                    IN USHORT     DeviceID,
                    OUT PPCI_CARDS_FOUND_STRUC pPciCardsFound
                    )

{
    UINT                i;
    USHORT              found = 0;
    USHORT              PciCommandWord;
    USHORT              EepromWordValue;
    ULONG               Device_Vendor_Id;
    UCHAR               BridgeRevId = 0;
    NDIS_STATUS         Status;
    USHORT              Slot;
    UCHAR               resBuf[NIC_RESOURCE_BUF_SIZE];
    PNDIS_RESOURCE_LIST  AssignedResources = (PNDIS_RESOURCE_LIST)resBuf;
    UINT                bufSize = NIC_RESOURCE_BUF_SIZE;

    // Zero out our Device and Vendor ID storage area.
    Device_Vendor_Id = 0;

    //Slot = Adapter->AiSlot;

    // Read in the Device and Vendor ID of a given slot.
    NdisReadPciSlotInformation(
        Adapter->ZD1205AdapterHandle,
        Slot,
        PCI_VENDOR_ID_REGISTER,
        (PVOID) &Device_Vendor_Id,
        0x4);

    // Check to see if we found a ZD1205 based adapter.
    //if ((((USHORT) Device_Vendor_Id) == VendorID) &&
    //    (((USHORT) (Device_Vendor_Id >> 16)) == DeviceID))
	if (1)
    {

        DEBUGSTR((_T("\n\n\n                          Found adapter %x at slot=%x\n"),
            Device_Vendor_Id, Slot));

        //  We found our adapter, so now we need to get the resources that were
        //  assigned to the adapter that we just found.

#ifdef NDIS51_MINIPORT
		NdisMQueryAdapterResources(&Status, Adapter->ZD1205WrapperConfigurationContext,
				AssignedResources, 
				&bufSize);
#else
        Status = NdisMPciAssignResources(
            Adapter->ZD1205AdapterHandle,
            Slot,
            &AssignedResources);
#endif

        if (Status != NDIS_STATUS_SUCCESS)
        {
            //ZD1205LogError(Adapter, EVENT_18, NDIS_ERROR_CODE_ADAPTER_DISABLED, 0);

            DEBUGSTR((_T("NdisMPciAssignResources on slot %x Failed - %x\n"), Slot, Status));

            found = 0;
            pPciCardsFound->NumFound = found;
            return found;
        }

        pPciCardsFound->PciSlotInfo[found].MemPhysAddress = 0;

        for (i=0;i < AssignedResources->Count;i++ )
        {
            switch (AssignedResources->PartialDescriptors[i].Type)
            {
            case CmResourceTypePort:
                if (AssignedResources->PartialDescriptors[i].Flags & CM_RESOURCE_PORT_IO)
                {
                    pPciCardsFound->PciSlotInfo[found].BaseIo =
                        (ULONG) ((AssignedResources)->PartialDescriptors[i].u.Port.Start.u.LowPart);
 					DEBUGSTR((_T("CmResourceTypePort BaseIo = %x\n"), (ULONG) ((AssignedResources)->PartialDescriptors[i].u.Port.Start.u.LowPart)));
               }
                break;
            case CmResourceTypeInterrupt:
                pPciCardsFound->PciSlotInfo[found].Irq =
                    (UCHAR) ((AssignedResources)->PartialDescriptors[i].u.Interrupt.Level);
				DEBUGSTR((_T("CmResourceTypeInterrupt Irq = %x\n"), (UCHAR) ((AssignedResources)->PartialDescriptors[i].u.Interrupt.Level)));
                break;
            case CmResourceTypeMemory:
                // this if gets only the last memory bar with a length of 0x1000, as others
                // might be our flash address, a boot ROM address, or otherwise.
 			   DEBUGSTR((_T("CmResourceTypeMemory Length = %x\n"), (AssignedResources)->PartialDescriptors[i].u.Memory.Length));
 			   DEBUGSTR((_T("CmResourceTypeMemory MemPhysAddress = %x\n"), (ULONG) ((AssignedResources)->PartialDescriptors[i].u.Memory.Start.u.LowPart)));
               if ( (((AssignedResources)->PartialDescriptors[i].u.Memory.Length) == 0x1000) ||
				    (((AssignedResources)->PartialDescriptors[i].u.Memory.Length) == 0x10000) )
                {
                    pPciCardsFound->PciSlotInfo[found].MemPhysAddress =
                        (ULONG) ((AssignedResources)->PartialDescriptors[i].u.Memory.Start.u.LowPart);
					DEBUGSTR((_T("CmResourceTypeMemory MemPhysAddress = %x\n"), (ULONG) ((AssignedResources)->PartialDescriptors[i].u.Memory.Start.u.LowPart)));
                }
                break;
            }
        }

        // read the revision id of the ZD1205 (or controller)
        NdisReadPciSlotInformation(
            Adapter->ZD1205AdapterHandle,
            Slot,
            PCI_REV_ID_REGISTER,
            &pPciCardsFound->PciSlotInfo[found].ChipRevision,
            0x1);

        DEBUGSTR((_T("NIC Controller revision id = %x\n"), pPciCardsFound->PciSlotInfo[found].ChipRevision));


        // Read in the SubDevice and SubVendor ID
        NdisReadPciSlotInformation(
            Adapter->ZD1205AdapterHandle,
            Slot,
            PCI_SUBVENDOR_ID_REGISTER,
            &pPciCardsFound->PciSlotInfo[found].SubVendor_DeviceID,
            0x4);


        pPciCardsFound->PciSlotInfo[found].SlotNumber = (USHORT) Slot;


        // We found a card and read it's assigned resources.
        // Now make sure that the resources are valid.
        if ((pPciCardsFound->PciSlotInfo[found].MemPhysAddress == 0) ||
            (pPciCardsFound->PciSlotInfo[found].MemPhysAddress == 0xfffffff0) ||
            //(pPciCardsFound->PciSlotInfo[found].BaseIo == 0) ||
            //(pPciCardsFound->PciSlotInfo[found].BaseIo == 0xfffffffc) ||
            (pPciCardsFound->PciSlotInfo[found].Irq == 0xff))
        {
            DEBUGSTR((_T("PCI resources were invalid\n")));

            // One of the resources wasn't valid, so log a message
            // into the event viewer.
            //ZD1205LogError(Adapter,
            //    EVENT_1,
            //    NDIS_ERROR_CODE_ADAPTER_DISABLED,
            //    0);
        }

        else
        {
			//++ Request by plee.
			PciCommandWord = 0x20;
			NdisWritePciSlotInformation(
				Adapter->ZD1205AdapterHandle,
				Slot,
				PCI_LATENCY_TIMER,
				&PciCommandWord,
				0x2);
			//--

            ++found;
        } // End if valid resources
    } // End if ZD1205 adapter
    pPciCardsFound->NumFound = found;
    return found;
}


