/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    queue.h

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include <ndis.h>
//#include <efilter.h>

#include "equates.h"
#include "pci.h"
#include "zd1205chip.h"
#include "zd1205dbg.h"
#include "zd1205sw.h"


//-------------------------------------------------------------------------
// QueueInitList -- Macro which will initialize a queue to NULL.
//-------------------------------------------------------------------------
#define QueueInitList(_L) (_L)->Link.Flink = (_L)->Link.Blink = (PLIST_ENTRY)0; \
		(_L)->Count = 0;


//-------------------------------------------------------------------------
// QueueEmpty -- Macro which checks to see if a queue is empty.
//-------------------------------------------------------------------------
#define QueueEmpty(_L) (QueueGetHead((_L)) == (PZD1205_LIST_ENTRY)0)


//-------------------------------------------------------------------------
// QueueGetHead -- Macro which returns the head of the queue, but does not
// remove the head from the queue.
//-------------------------------------------------------------------------
#define QueueGetHead(_L) ((PZD1205_LIST_ENTRY)((_L)->Link.Flink))


//-------------------------------------------------------------------------
// QueuePushHead -- Macro which puts an element at the head of the queue.
//-------------------------------------------------------------------------
#define QueuePushHead(_L,_E) \
    ASSERT(_L); \
    ASSERT(_E); \
    if (!((_E)->Link.Flink = (_L)->Link.Flink)) \
    { \
        (_L)->Link.Blink = (PLIST_ENTRY)(_E); \
    } \
	else \
	{ \
		((PZD1205_LIST_ENTRY)((_E)->Link.Flink))->Link.Blink = (PLIST_ENTRY)(_E); \
	} \
	(_E)->Link.Blink = (PLIST_ENTRY)0; \
	(_L)->Link.Flink = (PLIST_ENTRY)(_E); \
	(_L)->Count ++;


//-------------------------------------------------------------------------
// QueueRemoveHead -- Macro which removes the head of the head of queue.
//-------------------------------------------------------------------------
#define QueueRemoveHead(_L) \
{                                                     \
    PZD1205_LIST_ENTRY ListElem;                        \
    ASSERT((_L));                                     \
    if (ListElem = (PZD1205_LIST_ENTRY)(_L)->Link.Flink) /* then fix up our our list to point to next elem */ \
    {   \
        if(!((_L)->Link.Flink = ListElem->Link.Flink)) /* rechain list pointer to next link */ \
            /* if the list pointer is null, null out the reverse link */ \
            (_L)->Link.Blink = (PLIST_ENTRY) 0; \
		else \
			((PZD1205_LIST_ENTRY)((ListElem)->Link.Flink))->Link.Blink = \
								(PLIST_ENTRY) 0; \
		(_L)->Count--; \
    } \
}

//-------------------------------------------------------------------------
// QueueRemoveEntity -- Macro which removes the entity in the queue.
//-------------------------------------------------------------------------
#define QueueRemoveEntity(_L, _E) \
{ \
    PZD1205_LIST_ENTRY PreListElem;   \
    PZD1205_LIST_ENTRY PostListElem;  \
    ASSERT((_E)); \
    if (PreListElem = (PZD1205_LIST_ENTRY)((_E)->Link.Blink)) \
	{ \
		if (PostListElem = (PZD1205_LIST_ENTRY)((_E)->Link.Flink)) \
		{ \
			PreListElem->Link.Flink = (PLIST_ENTRY)PostListElem; \
			PostListElem->Link.Blink = (PLIST_ENTRY)PreListElem; \
		} \
		else \
		{ \
			PreListElem->Link.Flink = (PLIST_ENTRY)0; \
			(_L)->Link.Blink = (PLIST_ENTRY)PreListElem; \
		} \
	} \
	else \
	{ \
		if (PostListElem = (PZD1205_LIST_ENTRY)((_E)->Link.Flink)) \
		{ \
			PostListElem->Link.Blink = (PLIST_ENTRY)0; \
			(_L)->Link.Flink = (PLIST_ENTRY)PostListElem; \
		} \
		else \
		{ \
			(_L)->Link.Flink = (_L)->Link.Blink = (PLIST_ENTRY)0; \
		} \
	} \
	(_L)->Count--; \
}

//-------------------------------------------------------------------------
// QueuePutTail -- Macro which puts an element at the tail (end) of the queue.
//-------------------------------------------------------------------------
#define QueuePutTail(_L,_E) \
    ASSERT(_L); \
    ASSERT(_E); \
    if ((_L)->Link.Blink) \
    { \
		(_E)->Link.Blink = (_L)->Link.Blink; \
        ((PZD1205_LIST_ENTRY)(_L)->Link.Blink)->Link.Flink = (PLIST_ENTRY)(_E); \
        (_L)->Link.Blink = (PLIST_ENTRY)(_E); \
    } \
    else \
    { \
		(_E)->Link.Blink = (PLIST_ENTRY)0; \
        (_L)->Link.Flink = \
        (_L)->Link.Blink = (PLIST_ENTRY)(_E); \
    } \
	(_E)->Link.Flink = (PLIST_ENTRY)0; \
	(_L)->Count++;

//-------------------------------------------------------------------------
// QueueGetTail -- Macro which returns the tail of the queue, but does not
// remove the tail from the queue.
//-------------------------------------------------------------------------
#define QueueGetTail(_L) ((PZD1205_LIST_ENTRY)((_L)->Link.Blink))

//-------------------------------------------------------------------------
// QueueGetNext -- Macro which returns the next of entity of the queue, but does not
// remove the entity from the queue.
//-------------------------------------------------------------------------
#define QueueGetNext(_L, _E) ((PZD1205_LIST_ENTRY)((_E)->Link.Flink))

//-------------------------------------------------------------------------
// QueuePopHead -- Macro which  will pop the head off of a queue (list), and
//                 return it (this differs only from queueremovehead only in the 1st line)
//-------------------------------------------------------------------------
#define QueuePopHead(_L) \
(PZD1205_LIST_ENTRY) (_L)->Link.Flink; QueueRemoveHead(_L);


//-------------------------------------------------------------------------
// QueueGetCount -- Macro which  will return the Count in this List
//-------------------------------------------------------------------------
#define QueueGetCount(_L) ((_L)->Count)


	