/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    send.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

extern	ADAPTER_INFO	gAdapterInfo[MAX_SUPPORT_DEVICE_NUM];
const UCHAR     SnapBridgeTunnel[6] = { 0xAA, 0xAA, 0x03, 0x00, 0x00, 0xF8 };
const UCHAR     Snap802_1H[6] = { 0xAA, 0xAA, 0x03, 0x00, 0x00, 0x00 };
//-----------------------------------------------------------------------------
// Procedure: ZD1205MultipleSend
//
// Description:This function takes an array from NDIS and puts as many as it can
//             in our list to be immediately transferred. Each packet has its
//             status set (NDIS_STATUS_RESOURCES, NDIS_STATUS_PENDED,
//             or NDIS_STATUS_SUCCESS) in the PacketArray individually.
//
//
// Arguments: MiniportAdapterContext (Adapter Structure pointer)
//            PacketArray - an array of pointers to NDIS_PACKET structs
//            PacketCount - number of packets in PacketArray
//
// Returns: nothing
//
//-----------------------------------------------------------------------------
VOID
ZD1205MultipleSend(NDIS_HANDLE MiniportAdapterContext,
                 PPNDIS_PACKET PacketArray,
                 UINT NumberOfPackets)
{
    PZD1205_ADAPTER       Adapter;
    NDIS_STATUS         Status;
    UINT                PacketCount;
	ULONG				DebugPrint;
    PNDIS_BUFFER        CurrBuff;
	ETH_HEADER_STRUC	EnetHdr;
	PSTA_CONTEXT		STAContextPtr;
	USHORT				i;
	ZD1211_IRQL			OldIrql;
    PVOID               FirstBufferVA;
	UINT                FirstBufferLength, TotalBufferLength;

    DEBUGFUNC(_T("ZD1205MultipleSend"));

	//DbgPrint("ZD1205MultipleSend = %d", NumberOfPackets);

    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

	for (i=0;i<MAX_SUPPORT_DEVICE_NUM;i++){
		if (gAdapterInfo[i].AdapterContext == Adapter){
			if (!gAdapterInfo[i].AdapterActive){
				for(PacketCount=0;PacketCount < NumberOfPackets; PacketCount++)
				{
					NdisMSendComplete(
						gAdapterInfo[i].AdapterHandle,
						PacketArray[PacketCount],
						NDIS_STATUS_SUCCESS);
					//NDIS_STATUS_FAILURE);
				}
				return;
			}
			else{
				break;
			}
		}
	}

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_TX;
	
#ifndef PHY_LOOKBACK_TEST
#ifndef RF_TEST
	if ((!Adapter->adapterReady) || (Adapter->DisableTx)){
		for(PacketCount=0;PacketCount < NumberOfPackets; PacketCount++)
		{
			NdisMSendComplete(
                 Adapter->ZD1205AdapterHandle,
                 PacketArray[PacketCount],
				 NDIS_STATUS_SUCCESS);
                 //NDIS_STATUS_FAILURE);
		}
		return;
	}
#else
	if ((Adapter->DisableTx)){
		for(PacketCount=0;PacketCount < NumberOfPackets; PacketCount++)
		{
			NdisMSendComplete(
                 Adapter->ZD1205AdapterHandle,
                 PacketArray[PacketCount],
				 NDIS_STATUS_SUCCESS);
                 //NDIS_STATUS_FAILURE);
		}
		return;
	}
#endif
#endif

    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

    for(PacketCount=0;PacketCount < NumberOfPackets; PacketCount++)
    {
        // check for a zero pointer
        ASSERT(PacketArray[PacketCount]);

		if (Adapter->CardSetting.BSSType == AP_BSS){
			//NdisQueryPacket(PacketArray[PacketCount], NULL, NULL, &CurrBuff, NULL);
			NdisGetFirstBufferFromPacket( PacketArray[PacketCount], 
										  &CurrBuff, 
										  &FirstBufferVA, 
										  &FirstBufferLength, 
										  &TotalBufferLength
										  );
			GetEthernetHeaderFromPacket(Adapter, PacketArray[PacketCount], CurrBuff, &EnetHdr);
			if ((EnetHdr.Destination[0] & BIT_0)){ // Multicast frame
				if (Adapter->aSTAInPSMode){
				    //Adapter->QueueCount++;
					Adapter->NumMulPendingQueued ++;
					EnqueuePacket(Adapter->FirstMulPendingQueue, 
									Adapter->LastMulPendingQueue, 
										PacketArray[PacketCount]);
					NDIS_SET_PACKET_STATUS(PacketArray[PacketCount], 
						NDIS_STATUS_PENDING);
					continue;
				}
			}
			else{
				STAContextPtr = FindSTAContext(Adapter, EnetHdr.Destination);
				if (!STAContextPtr){
					NdisMSendComplete(
						Adapter->ZD1205AdapterHandle,
						PacketArray[PacketCount],
						NDIS_STATUS_FAILURE);
					continue;
				}
				if (STAContextPtr->LinkStatus != CONNECTED){
					NdisMSendComplete(
						Adapter->ZD1205AdapterHandle,
						PacketArray[PacketCount],
						NDIS_STATUS_FAILURE);
					continue;
				}
				if (STAContextPtr->PSMode){
					if (!STAContextPtr->FirstPendingQueue){
						// Set Aging Timer 5 sec.
						NdisMSetTimer(&STAContextPtr->AgeTimer, 5000);
					}			
				    //STAContextPtr->UniQCount++;
					STAContextPtr->NumPendingQueued++;
					EnqueuePacket(STAContextPtr->FirstPendingQueue, 
						STAContextPtr->LastPendingQueue, 
						PacketArray[PacketCount]);
					NDIS_SET_PACKET_STATUS(PacketArray[PacketCount], 
						NDIS_STATUS_PENDING);
					continue;
				}
			}
		}

		// If there is any packet queued, queue any packet after that to keep 
		// the order
		if ((Adapter->FirstTxQueue) || (Adapter->DeviceInSleep)||(Adapter->SleepProtect)){
			//ASSERT(0);
			// Queue This packet for the Deserialized Miniport
			Adapter->NumPacketsQueued++;
			EnqueuePacket(Adapter->FirstTxQueue, Adapter->LastTxQueue, PacketArray[PacketCount]);
			Status = NDIS_STATUS_PENDING;
		}
		else{
			//DbgPrint("call SetupNexSend");
			Status = SetupNextSend(Adapter, PacketArray[PacketCount], DATA, 
						Adapter->CardSetting.EncryMode, UPPER_NDIS, NULL);

			if (Status == NDIS_STATUS_RESOURCES)
			{
				//ASSERT(0);
				// Queue This packet for the Deserialized Miniport
				Adapter->NumPacketsQueued++;
				EnqueuePacket(Adapter->FirstTxQueue, Adapter->LastTxQueue, PacketArray[PacketCount]);
				Status = NDIS_STATUS_PENDING;
			}
		}
		// The status returned is either NDIS_STATUS_PENDING 
		// or NDIS_STATUS_INVALID_PACKET
        //NDIS_SET_PACKET_STATUS(PacketArray[PacketCount], Status);
		NdisMSendComplete(
			Adapter->ZD1205AdapterHandle,
			PacketArray[PacketCount],
			NDIS_STATUS_SUCCESS);

    }

	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);


    return;
}


//WPA support
VOID
ZD1205ScanFromPacket(PZD1205_ADAPTER Adapter,
                     PNDIS_PACKET Packet,
                     UINT RemoveLen,
                     UINT BytesToScan,
                     PNDIS_BUFFER FirstBuffer,
                     PUINT BytesScaned)

{
    PNDIS_BUFFER    CurrentBuffer;
    PVOID           VirtualAddress;
    UINT            CurrentLength;
    UINT            AmountToMove;
	UINT			i;
	PUCHAR			pByte;
	UCHAR			FirstBlock = 1;
	UCHAR			BufNum = 0;
	UINT			BufLen;
	DEBUGFUNC(_T("ZD1205ScanFromPacket"));
	
    *BytesScaned = 0;
    
    if (!BytesToScan)
        return;

    CurrentBuffer = FirstBuffer;

#ifdef NDIS51_MINIPORT
	NdisQueryBufferSafe(CurrentBuffer, &VirtualAddress, &CurrentLength, NormalPagePriority);
#else
    NdisQueryBuffer(CurrentBuffer, &VirtualAddress, &CurrentLength);
#endif            

    while (BytesToScan)
    {
        while (!CurrentLength)
        {
            NdisGetNextBuffer(
                CurrentBuffer,
                &CurrentBuffer);

            // If we've reached the end of the packet.  We return with what
            // we've done so far (which must be shorter than requested).
            if (!CurrentBuffer)
                return;

#ifdef NDIS51_MINIPORT
			NdisQueryBufferSafe(CurrentBuffer, &VirtualAddress, &CurrentLength, 
				NormalPagePriority);
#else
            NdisQueryBuffer(
                CurrentBuffer,
                &VirtualAddress,
                &CurrentLength);
#endif
        }


        // Compute how much data to move from this fragment
        if (CurrentLength > BytesToScan)
            AmountToMove = BytesToScan;
        else
            AmountToMove = CurrentLength;
		
		 // Update counters
        *BytesScaned +=AmountToMove;
        BytesToScan -=AmountToMove;
        CurrentLength = 0;
        
		//do MIC
		pByte = (PUCHAR)VirtualAddress;
		if (FirstBlock){
			FirstBlock = 0;
			pByte += RemoveLen; //wpa2
			BufLen = AmountToMove - RemoveLen;
		}
		else
			BufLen = AmountToMove;
		
		BufNum++;
		INITSTRWPA((_T("BufNum = %x\n"), BufNum));
		INITSTRWPA((_T("AmountToMove = %x\n"), AmountToMove));
		INITSTRWPA((_T("BufLen = %x\n"), BufLen));
		INITSTRWPA((_T("Dump Payload, include SNAP header\n")));
		DumpWPATX(pByte, BufLen, FALSE, 1);
		
		for (i=0; i<BufLen; i++){
			MICappendByte(*pByte, &Adapter->TxMicKey);
			pByte++; // Next byte
		}
    }
}

//-----------------------------------------------------------------------------
// Procedure:   SetupNextSend
//
// Description: This routine is called by ZD1205Send.  It will setup all of the
//              necessary structures (TCBs, TBDs, etc.) to send a packet.  If
//              the device has the necessary resources, the packet will be
//              sent out on the active chain.  If the device doesn't have the
//              resources to send the packet, then the routine will return
//              NDIS_STATUS_RESOURCES.
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//      Packet - A pointer to a descriptor for the packet that is to be
//               transmitted.
// Returns:
//      NDIS_STATUS_SUCCESS - We copied the entire packet into a host buffer,
//                            (either it was a short frame, or we coalesced
//                            the packet into a host buffer), so we can
//                            immediately return the buffer back to the upper
//                            layers.
//      NDIS_STATUS_PENDING - If we were able to acquire the necessary TBD's
//                            or Coalesce buffer for the packet.  This means
//                            that the device will send this packet soon.
//                            Eventually we'll return the packet back to the
//                            protocol stack by using the "SendComplete" call.
//      NDIS_STATUS_RESOURCES - We didn't have the resouces (TCBs or TBDs) to
//                              accept this packet. The NDIS packet should
//                              queue this packet, and give it back to us at a
//                              later time.
//-----------------------------------------------------------------------------

NDIS_STATUS
SetupNextSend(PZD1205_ADAPTER Adapter,
              PNDIS_PACKET Packet,
			  UCHAR	FrameType,
			  UCHAR EnCipher,
			  UCHAR	Provider,
			  PVOID ReturnPara
			  )
{
    PZD1205SwTcb			SwTcb;
	PHWTCB_STRUC			HwTcb;
    NDIS_STATUS				Status;
	ETH_HEADER_STRUC		EnetHdr;
	//UINT					NumPhysDesc;
	//UINT					BufferCount;
	UINT					PacketLength;
	UINT					NumOfFrag;
    PNDIS_BUFFER            CurrBuff;
	PNDIS_PHYSICAL_ADDRESS_UNIT pUnit;
	UINT					i;
	ULONG					CurrFragLen;
	ULONG					NextFragLen;
	PTBD_STRUC				Tbd;
	ULONG					Length;
	ULONG					RemoveLen;
	ULONG					TypeLength;
	ULONG					FrmBodyLen;
    PCOALESCE				Coalesce = NULL;
    UINT                    BytesCopied;
	ULONG					DebugPrint;
	BOOLEAN					b_EAPOL=FALSE;
#ifdef HOST_IF_USB
	ULONG					PrvFragLen = 0;
#endif
 	//WPA support
	PUCHAR					pByte;	
	UINT                    BytesScaned;
	UCHAR					CalMIC[8];
	UINT					AvailTbdPerTcb;
	//UINT					ReqPhysDescNum;
	MICvar					*pMicKey;
	PVOID                   FirstBufferVA;
	UINT                    FirstBufferLength;
	UCHAR                   HandleNo;

    DEBUGFUNC(_T("SetupNextSend"));
	//INITSTR((_T("\n")));

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_TX;
//#ifdef HOST_IF_USB
//	Adapter->TestCoalesce = TRUE;
//#endif	

    
	for (i=0; i<30; i++)
	{
         if ( Adapter->TxTransfer[i].status == 3 )
		 {
		     (Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(Adapter->TxTransfer[i].handle);
             Adapter->TxTransfer[i].status = 0;
			 Adapter->TxTransfer[i].handle = NULL;
			 //DbgPrint("free %d\r\n", i);
		 }
	}

	for (i=0; i<30; i++)
	{
		if ( !Adapter->TxTransfer[i].status ) 
		{
			Adapter->TxTransfer[i].status++;
			HandleNo = (UCHAR) i;
			//DbgPrint("get %d\r\n", i);
			break;
		}
	}

	if ( i== 30 )
	{
		DbgPrint("Tx queue is full\r\n");
		return NDIS_STATUS_SUCCESS;
	}

	if (Provider == UPPER_NDIS){
		Adapter->txFrmUpperNDIS++;
		Adapter->Traffic ++;
	}
	else
		Adapter->txFrmDrvMgt++;

	//NdisQueryPacket(Packet, &NumPhysDesc, &BufferCount, &CurrBuff, &PacketLength);
	NdisGetFirstBufferFromPacket( Packet, 
								  &CurrBuff, 
								  &FirstBufferVA, 
								  &FirstBufferLength, 
								  &PacketLength
								  );

	GetEthernetHeaderFromPacket(Adapter, Packet, CurrBuff, &EnetHdr);

	//WPA support
	if ((EnCipher == TKIP) && (Adapter->PkInstalled == 1)){
		int	i;
		
		pMicKey=&Adapter->TxMicKey;
		MICclear(pMicKey);
		pByte = (PUCHAR)&EnetHdr;
		
		INITSTRWPA((_T("Dump DA, SA\n")));
		DumpWPATX(pByte, 12, FALSE, 1);
		
		for(i=0; i<12; i++){ //for DA, SA
			MICappendByte(*pByte, pMicKey);
			pByte++;
		}

		MICappendByte(0,pMicKey);
		MICappendByte(0,pMicKey);
		MICappendByte(0,pMicKey);
		MICappendByte(0,pMicKey);
	}

	// Judge the type of encapsulation, and then mark some indications.
	TypeLength = ((((USHORT)EnetHdr.TypeLength[0]) << 8) + EnetHdr.TypeLength[1]);
	if ( (TypeLength <= 1500) || (FrameType == MGT_SCANTEST) ||
		 (FrameType == MGT_LBTEST) ){ // IEEE 802.2/802.3 Encapsulation
		//FrmBodyLen = TypeLength;

		//ASSERT(FrmBodyLen == (PacketLength-14));
		//if ( (FrmBodyLen < 60)&&(Provider == UPPER_NDIS) )
         FrmBodyLen = PacketLength - 14;

		RemoveLen = 14;
	}
	else{ // Ethernet Encapsulation
		FrmBodyLen = PacketLength - 6; // PacketLength - 14 + 8(802.2 LLC, 802.2 SNAP)
		RemoveLen = 6;
	}
	

	NumOfFrag = CalNumOfFrag(Adapter, FrmBodyLen);

	//WPA support
	//ReqPhysDescNum = NumPhysDesc + 2;
	AvailTbdPerTcb = Adapter->NumTbdPerTcb;
	
	if ((EnCipher == TKIP) && (Adapter->PkInstalled == 1)){
		//ReqPhysDescNum ++;
		AvailTbdPerTcb --;
	}

	// NumPhysDesc+2 is because the extra Ctrl-Setting and 802.11b header
	//if ((NumPhysDesc+2) > Adapter->NumTbdPerTcb || Adapter->TestCoalesce)
	if (1)
	{ // Need Coalesce buffer
		if (QueueEmpty(&Adapter->CoalesceBufferList)){
			INITSTRTX(DebugPrint,(_T("no enough free CoalesceBufferList to send packet\n")));
			ASSERT(0);
			return NDIS_STATUS_RESOURCES;
		}
	}

	// (QueueGetCount-1) is because we need an extra TCB to indicate last TCB (suspend)
	if ( (QueueGetCount(&Adapter->TxCBList)-1) < NumOfFrag ){ 
		// TCBs are no enough free Tcb for use
		//ASSERT(0);
		return NDIS_STATUS_RESOURCES;
	}


	// Now, we are sure that there are enough resources for use.

	//**********************************************************************//
	// If the encapsulation type is DIX ethernet type, we will write the
	// 802.2 LLC, 802.2 SNAP before the data.
	// The frame modification rule is:
	// --------------------------------------------------------------------- 
	// |Dst Addr|Src Addr|Type|              Data                          |
	// ---------------------------------------------------------------------
	//
	//						changed to
	//
	// ---------------------------------------------------------------------
	// |Dst Addr|LLC/SNAP|Type|              Data                          |
	// ---------------------------------------------------------------------
	//**********************************************************************//
	if (RemoveLen == 6){
		// It's Ethernet Encapsulation, overwrite source addr by 802.2 LLC
		// and OrgCode
        if ((TypeLength == 0x8137) || (TypeLength == 0x80F3)) {
			ZD1205CopyFromBufferToPacket(Adapter,
                        Packet,
                        6, // BytesToCopy
                        (PUCHAR)&SnapBridgeTunnel[0], // SrcBuffer
                        CurrBuff, // First buffer
						6
                        );
        } 
		else {
			if (TypeLength == 0x888E){ // EAPOL Type (802.1x)
				b_EAPOL = TRUE;
			}
			else{
				b_EAPOL = FALSE;
			}

			ZD1205CopyFromBufferToPacket(Adapter,
                        Packet,
                        6,
                        (PUCHAR)&Snap802_1H[0],
                        CurrBuff,
                        6
						);
        }
	}
	//--

	//WPA support
	if (Adapter->PkInstalled == 1)
		b_EAPOL = FALSE; //response for Group key update
		
	//WPA support
	if ((EnCipher == TKIP) && (Adapter->PkInstalled == 1)){
		//NdisQueryPacket(Packet, &NumPhysDesc, &BufferCount, &CurrBuff, &PacketLength);
		NdisGetFirstBufferFromPacket( Packet, 
									  &CurrBuff, 
									  &FirstBufferVA, 
									  &FirstBufferLength, 
									  &PacketLength
									  );
		ZD1205ScanFromPacket(Adapter, //for payload include SNAP header
            Packet,
            RemoveLen,
            PacketLength,
            CurrBuff,
            &BytesScaned);
        
        INITSTRWPA((_T("PacketLength = %d\n"), PacketLength)); 
        INITSTRWPA((_T("BytesScaned = %d\n"), BytesScaned));
           
		MICgetMIC(CalMIC, pMicKey);
		
		INITSTRWPA((_T("Dump Calculate MIC\n")));
		DumpWPATX(CalMIC, 8, FALSE, 1);
	}	

	//*********************************************************************//
	// Now, construct a physical mapping table for this packet.
	// If this packet need be coalesced to a coalesce buffer, the physical
	// mapping table will be constructed to meet the coalesce buffer.
	//*********************************************************************//
	// (NumPhysDesc+2 ) is because we need extra 2 Tcbs for control-setting
	// and MacHeader.
	//if ( ((NumPhysDesc+2) > Adapter->NumTbdPerTcb) ||
	//if ( ((NumPhysDesc+2) > AvailTbdPerTcb) || //WPA support
	//		Adapter->TestCoalesce ){ // Need Coalesce buffer
    if (1)
	{
		//ASSERT(0);
		if (!AcquireCoalesceBuffer(Adapter, &Coalesce)){
			INITSTR((_T("Fatal error!\n")));
			ASSERT(0);
		}
	    // Copy all of the packet data to our coalesce buffer
		ZD1205CopyFromPacketToBuffer(Adapter,
			Packet,
			PacketLength,
			(PCHAR)Coalesce->HwCoalesce,
			CurrBuff,
			&BytesCopied);

        ASSERT(BytesCopied == PacketLength);

		// restore the source address
		if ( (Adapter->CardSetting.BSSType == AP_BSS)&&(Provider == AP_FORWARD_MUL) )
			ZD1205CopyFromBufferToPacket(Adapter,
                        Packet,
                        6,
                        (PUCHAR) EnetHdr.Source,
                        CurrBuff,
                        6
						);

		//wpa2
		//WPA support
		//if ((EnCipher == TKIP) && (Adapter->PkInstalled == 1)){ //append MIC to payload
		//	INITSTR((_T("*****Need Coalesce buffer situation\\n")));
		//	memcpy(&Coalesce->HwCoalesce[PacketLength], CalMIC, MIC_SIZE);
		//	PacketLength += MIC_SIZE; 
		//}

#ifdef HOST_IF_USB
		Coalesce->CoalesceRemoveLen = RemoveLen;
#endif
		pUnit = Adapter->pUnits;
		
        NdisSetPhysicalAddressLow(pUnit->PhysicalAddress, 
							Coalesce->HwCoalescePhys);
        NdisSetPhysicalAddressHigh(pUnit->PhysicalAddress, 0);
		pUnit->Length = PacketLength;
							
	}


	//************************************************************************//
	// Physical mapping had been constructed.
	// Now, remove the unnecessary leading part of this frame.
	//************************************************************************//
	pUnit = Adapter->pUnits;
	// Remove Ethernet Header
	while(RemoveLen){
		if (RemoveLen >= pUnit->Length){
			RemoveLen -= pUnit->Length;
			pUnit++;
		}
		else{
			NdisSetPhysicalAddressLow(pUnit->PhysicalAddress,
							NdisGetPhysicalAddressLow(pUnit->PhysicalAddress) +
							RemoveLen);
			pUnit->Length -= RemoveLen;
			RemoveLen = 0;
		}
	}

	//***********************************************************************//
	// Setup HwTcb for each fragmentation.
	//***********************************************************************//
	for (i=0;i<NumOfFrag;i++){

		ULONG					TotalLength = 0;


		SwTcb = (PZD1205SwTcb) QueuePopHead(&Adapter->TxCBList);

#ifdef HOST_IF_USB

		SwTcb->bHasCompleteBeforeSend = FALSE;
		SwTcb->bHasBeenDelayedBefore = FALSE;

#endif
		// Decide Current fragmentation length
		if (i == (NumOfFrag-1)){
			CurrFragLen = FrmBodyLen - i*Adapter->CardSetting.FragThreshold;
			NextFragLen = 0;
			SwTcb->LastFrag = 1;
			SwTcb->Provider = Provider;
			SwTcb->ReturnPara = ReturnPara;

			//WPA support	
			if ((EnCipher == TKIP) && (Adapter->PkInstalled == 1))
				CurrFragLen += MIC_SIZE;
		}
		else{
			CurrFragLen = Adapter->CardSetting.FragThreshold;
			if (i == (NumOfFrag-2)){
				NextFragLen = FrmBodyLen - (i+1)*Adapter->CardSetting.FragThreshold;
			}
			else{
				NextFragLen = Adapter->CardSetting.FragThreshold;

				//WPA support
				if ((EnCipher == TKIP) && (Adapter->PkInstalled == 1)) 
					NextFragLen += MIC_SIZE;
			}
			SwTcb->LastFrag = 0;
		}
		SwTcb->Packet = Packet;
		SwTcb->Coalesce = Coalesce;
		HwTcb = SwTcb->Tcb;
		Tbd = SwTcb->FirstTbd;
		HwTcb->TxCbTbdNumber = 0;

		SwTcb->FrameType = FrameType;
		Length = Cfg_CtrlSetting(Adapter, SwTcb, &EnetHdr, FrameType, 
							i, CurrFragLen, NextFragLen, EnCipher, b_EAPOL);
		
		//INITSTRTX(DebugPrint,(_T("Cfg_CtrlSetting Len = %d\n"), Length));
		TotalLength = Length;

		Tbd->TbdBufferAddrHighPart = 0;
		Tbd->TbdBufferAddrLowPart = SwTcb->HwCtrlPhys;
		Tbd->TbdCount = Length;
		Tbd++;
		HwTcb->TxCbTbdNumber ++;

		Length = Cfg_MacHeader(Adapter, SwTcb, &EnetHdr, FrameType, 
			SwTcb->LastFrag, EnCipher, i, b_EAPOL);
		
		//INITSTRTX(DebugPrint,(_T("Cfg_MacHeader Len = %d\n"), Length));
		TotalLength += Length;
		
		Tbd->TbdBufferAddrHighPart = 0;
		Tbd->TbdBufferAddrLowPart = SwTcb->HwHeaderPhys;
		Tbd->TbdCount = Length;
		Tbd++;
		HwTcb->TxCbTbdNumber ++;

		//WPA support
		if ((EnCipher == TKIP) && (i == (NumOfFrag-1)) && (Adapter->PkInstalled == 1)){
			CurrFragLen -= MIC_SIZE; // restore the CurrFragLen
			//INITSTR((_T("CurrFragLen(Not include MIC) = %x\n"), CurrFragLen));
		}

#ifdef HOST_IF_USB
		if (CurrFragLen == 0){

			Tbd->TbdCount = 0;
		}
#endif

#ifdef OFDM
		if ((EnCipher == TKIP) && (i == (NumOfFrag-1)) && (Adapter->PkInstalled == 1))
			TotalLength += MIC_SIZE;

		TotalLength += CurrFragLen;
		SwTcb->HwCtrlPtr->CtrlSetting[18] = (UCHAR)TotalLength;
		SwTcb->HwCtrlPtr->CtrlSetting[19] = (UCHAR)(TotalLength >> 8);
#endif

		while(CurrFragLen){
			if (CurrFragLen >= pUnit->Length){
				ASSERT(pUnit->Length);
				Tbd->TbdBufferAddrHighPart =0;
				Tbd->TbdBufferAddrLowPart = 
						NdisGetPhysicalAddressLow(pUnit->PhysicalAddress);
				Tbd->TbdCount = pUnit->Length;

				//INITSTRTX(DebugPrint,(_T("Data Len1(CurrFragLen >= pUnit->Length) = %d\n"), Tbd->TbdCount));

#ifdef HOST_IF_USB
				Tbd->PrvFragLen = PrvFragLen;
				PrvFragLen += CurrFragLen;
#endif
				CurrFragLen -= pUnit->Length;
				pUnit++;
			}
			else{
				Tbd->TbdBufferAddrHighPart = 0;
				Tbd->TbdBufferAddrLowPart = 
						NdisGetPhysicalAddressLow(pUnit->PhysicalAddress);
				Tbd->TbdCount = CurrFragLen;
				
				//INITSTRTX(DebugPrint,(_T("Data Len2(CurrFragLen) = %d\n"), Tbd->TbdCount));

				NdisSetPhysicalAddressLow(pUnit->PhysicalAddress,
									Tbd->TbdBufferAddrLowPart+CurrFragLen);
				pUnit->Length -= CurrFragLen;

#ifdef HOST_IF_USB
				Tbd->PrvFragLen = PrvFragLen;
				PrvFragLen += CurrFragLen;
#endif
				CurrFragLen = 0;
			}
			Tbd++;
			HwTcb->TxCbTbdNumber ++;
		}
		//WPA support
		if ((EnCipher == TKIP) && (i == (NumOfFrag-1)) && (Adapter->PkInstalled == 1)){
			// force append MIC to the last fragment (MPDU) or MSDU
			INITSTRWPA((_T("Set MIC to an extra Tbd!\n")));
			//wpa2
			if (Adapter->FakeMic)
				memset(CalMIC, 0x0, MIC_SIZE);
			memcpy(SwTcb->HwMicPtr, CalMIC, MIC_SIZE);

			Tbd->TbdBufferAddrHighPart = 0;
			Tbd->TbdBufferAddrLowPart = SwTcb->HwMicPhys;
			Tbd->TbdCount = MIC_SIZE;
			Tbd++;
			HwTcb->TxCbTbdNumber ++;
			INITSTRWPA((_T("HwTcb->TxCbTbdNumber = %x\n"), HwTcb->TxCbTbdNumber));
			Adapter->FakeMic = 0;
		}
		
        QueuePutTail(&Adapter->ActiveChainList, &SwTcb->Link);
		//INITSTRTX(DebugPrint, (_T("Serial No: %x\n"), Adapter->TxTotalCnt++));
		//INITSTRTX(DebugPrint, (_T("StartTx  HwTcb = %x (Phys %x)\n"), HwTcb, SwTcb->TcbPhys));

#ifdef HOST_IF_USB
		{   
            NDIS_STATUS   status;
			ZD1211_IRQL   OldIrql;

		    Adapter->txrun = 1;
            ZDDbg((_T("---------->>> Release SpinLock\n")));
	        ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		    status = USB_MPSenToTxEP(Adapter, HandleNo);
			ZDDbg((_T("Get SpinLock\n")));

			if ( status != NDIS_STATUS_SUCCESS )
			{
				Adapter->TxTransfer[HandleNo].handle = NULL;
				Adapter->TxTransfer[HandleNo].status = 0;
			}

			ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

			//if ( status == NDIS_STATUS_FAILURE )
				return NDIS_STATUS_SUCCESS;
		}
		//USB_MPSenToTxEP_Complete(Adapter);
		//if ( Adapter->USBInfo.hTxEP == NULL )
		//{
			//DbgPrint(" SetEvent StartTxEvent from SetupNextSend\n");
		//	SetEvent(Adapter->StartTxEvent);
		//}
		//Status = USB_StartTxEP(Adapter);

		//if ((Status != STATUS_SUCCESS) && (Status != STATUS_PENDING)){
		//	USB_ResetTxQueue(Adapter);
		//}
#endif	
	}

	return(NDIS_STATUS_PENDING);

}


//-----------------------------------------------------------------------------
// Procedure:   ZD1205MgtSend
//
// Description: 
//				Adapter - Ptr to the Adapter structure
//
// Arguments:
//
// Returns:
//      NDIS_STATUS_SUCCESS - We copied the entire packet into a host buffer,
//                            (either it was a short frame, or we coalesced
//                            the packet into a host buffer), so we can
//                            immediately return the buffer back to the upper
//                            layers.
//      NDIS_STATUS_PENDING - If we were able to acquire the necessary TBD's
//                            or Coalesce buffer for the packet.  This means
//                            that the device will send this packet soon.
//                            Eventually we'll return the packet back to the
//                            protocol stack by using the "SendComplete" call.
//      NDIS_STATUS_RESOURCES - We didn't have the resouces (TCBs or TBDs) to
//                              accept this packet. The NDIS packet should
//                              queue this packet, and give it back to us at a
//                              later time.
//-----------------------------------------------------------------------------
NDIS_STATUS    
ZD1205MgtSend(IN PZD1205_ADAPTER Adapter)
{

    DEBUGFUNC(_T("ZD1205MgtSend"));
    TRACE2(Adapter, ("\n"));
/*

    // Attempt to acquire a Software TCB for the packet
    SwTcb = (PZD1205SwTcb) QueuePopHead(&Adapter->TxCBList);

    if (!(SwTcb))
    {
        TRACE2(Adapter, ("FailNoTcb\n"));

        // No TCBs available so return NO_RESOURCES
        Status = NDIS_STATUS_RESOURCES;
        return (Status);
    }

    // Is 'S' bit on our head
    if (QueueEmpty(&Adapter->TxCBList))
    {
        TRACE2(Adapter,("Don't WRAP queue!!! Fail With Only 1 TCB left\n"));
        QueuePushHead(&Adapter->TxCBList, &SwTcb->Link);
        Status = NDIS_STATUS_RESOURCES;
        return(Status);
    }

    ASSERT(SwTcb);
    // Init some variables in the SwTcb
    SwTcb->MapsUsed = 0;
    SwTcb->NumPhysDesc = 0;
    SwTcb->TbdsUsed = 0;
    SwTcb->Coalesce = (PCOALESCE) 0;
    SwTcb->CoalesceBufferLen = 0;

    if (!AcquireCoalesceBuffer(Adapter, Coalesce))
		return (NDIS_STATUS_RESOURCES);

    Coalesce = SwTcb->Coalesce;
	CoalescePhy = Coalesce->HwCoalescePhys;

	SwTcb->TbdsUsed = 3;
	Tbd = SwTcb->FirstTbd;
	// Set Control-setting field
    Tbd->TbdBufferAddrLowPart = CoalescePhy;
	Tbd->TbdBufferAddrHighPart = 0;
    Tbd->TbdCount = 64;
	//FillCtrlSetField();

	Tbd++;
	// Set 802.11 Mac-Header field
    Tbd->TbdBufferAddrLowPart = CoalescePhy+100;
	Tbd->TbdBufferAddrHighPart = 0;
    Tbd->TbdCount = 24;
	//FillMacHeaderField();

	// Set 802.11 Frame-Body field
    Tbd->TbdBufferAddrLowPart = CoalescePhy+200;
	Tbd->TbdBufferAddrHighPart = 0;
    Tbd->TbdCount = 100;
	//FillFrmBodyField();

    HwTcb = SwTcb->Tcb;
    HwTcb->CbCommand = 0;
    HwTcb->TxCbTbdNumber = (UCHAR) SwTcb->TbdsUsed;

    QueuePutTail(&Adapter->ActiveChainList, &SwTcb->Link);

    //ZD1205_WRITE_REGISTER(Adapter,START_TX, 1);

*/
	return NDIS_STATUS_SUCCESS;
}

//-----------------------------------------------------------------------------
// Procedure:   ZD1205CopyFromPacketToBuffer
//
// Description: This routine will copy a packet to a the passed buffer (which
//              in this case will be a coalesce buffer).
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//      Packet - The packet to copy from.
//      BytesToCopy - The number of bytes to copy from the packet.
//      DestBuffer - The destination of the copy.
//      FirstBuffer - The first buffer of the packet that we are copying from.
//
// Result:
//      BytesCopied - The number of bytes actually copied
//
// Returns:
//-----------------------------------------------------------------------------

VOID
ZD1205CopyFromPacketToBuffer(  PZD1205_ADAPTER Adapter,
                           PNDIS_PACKET Packet,
                           UINT BytesToCopy,
                           PCHAR DestBuffer,
                           PNDIS_BUFFER FirstBuffer,
                           PUINT BytesCopied)

{
    PNDIS_BUFFER    CurrentBuffer;
    PVOID           VirtualAddress;
    UINT            CurrentLength;
    UINT            AmountToMove;

    DEBUGFUNC(_T("ZD1205CopyFromPacketToBuffer"));
    *BytesCopied = 0;
    if (!BytesToCopy)
        return;

    CurrentBuffer = FirstBuffer;

#ifdef NDIS51_MINIPORT
	NdisQueryBufferSafe(CurrentBuffer, &VirtualAddress, &CurrentLength, 
		NormalPagePriority);
#else
    NdisQueryBuffer(
        CurrentBuffer,
        &VirtualAddress,
        &CurrentLength);
#endif            

    while (BytesToCopy)
    {
        while (!CurrentLength)
        {
            NdisGetNextBuffer(
                CurrentBuffer,
                &CurrentBuffer);

            // If we've reached the end of the packet.  We return with what
            // we've done so far (which must be shorter than requested).
            if (!CurrentBuffer)
                return;

#ifdef NDIS51_MINIPORT
			NdisQueryBufferSafe(CurrentBuffer, &VirtualAddress, &CurrentLength, 
				NormalPagePriority);
#else
            NdisQueryBuffer(
                CurrentBuffer,
                &VirtualAddress,
                &CurrentLength);
#endif
        }


        // Compute how much data to move from this fragment
        if (CurrentLength > BytesToCopy)
            AmountToMove = BytesToCopy;
        else
            AmountToMove = CurrentLength;

        // Copy the data.
        NdisMoveMemory(DestBuffer, VirtualAddress, AmountToMove);

        // Update destination pointer
        DestBuffer = (PCHAR) DestBuffer + AmountToMove;

        // Update counters
        *BytesCopied +=AmountToMove;
        BytesToCopy -=AmountToMove;
        CurrentLength = 0;
    }
}


//-----------------------------------------------------------------------------
// Procedure:   ZD1205CopyFromBufferToPacket
//
// Description: This routine will copy the passed buffer to the packet
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//      Packet - The packet to copy from.
//      BytesToCopy - The number of bytes to copy from the packet.
//      SrcBuffer - The source of the copy.
//      FirstBuffer - The first buffer of the packet that we are copying from.
//      Offset - The number of bytes offset from the first byte in the first buffer
//
// Returns:
//-----------------------------------------------------------------------------

VOID
ZD1205CopyFromBufferToPacket(  PZD1205_ADAPTER Adapter,
                           PNDIS_PACKET Packet,
                           UINT BytesToCopy,
                           PUCHAR SrcBuffer,
                           PNDIS_BUFFER FirstBuffer,
                           UINT Offset)

{
    PNDIS_BUFFER    CurrentBuffer;
    PVOID           VirtualAddress;
    UINT            CurrentLength;
    UINT            RestSpace;
	UINT			i;

    if (!BytesToCopy)
        return;

    CurrentBuffer = FirstBuffer;

#ifdef NDIS51_MINIPORT
	NdisQueryBufferSafe(CurrentBuffer, &VirtualAddress, &CurrentLength, 
		NormalPagePriority);
#else
    NdisQueryBuffer(
        CurrentBuffer,
        &VirtualAddress,
        &CurrentLength);
#endif

	while (Offset){
		if (Offset <= CurrentLength){
			RestSpace = CurrentLength-Offset;
			VirtualAddress = (PCHAR)VirtualAddress + Offset;
			break;
		}
		else{
			Offset -= CurrentLength;
            NdisGetNextBuffer(
                CurrentBuffer,
                &CurrentBuffer);

#ifdef NDIS51_MINIPORT
			NdisQueryBufferSafe(CurrentBuffer, &VirtualAddress, &CurrentLength, 
				NormalPagePriority);
#else
            NdisQueryBuffer(
                CurrentBuffer,
                &VirtualAddress,
                &CurrentLength);
#endif
		}
	}

	while (BytesToCopy){
		if (BytesToCopy <= RestSpace){
			for (i=0;i<BytesToCopy;i++){
				*((PCHAR)VirtualAddress+i) = *(SrcBuffer+i);
			}
			break;
		}
		else{
			for (i=0;i<RestSpace;i++){
				*((PCHAR)VirtualAddress+i) = *(SrcBuffer+i);
			}
			BytesToCopy -= RestSpace;

            NdisGetNextBuffer(
                CurrentBuffer,
                &CurrentBuffer);

#ifdef NDIS51_MINIPORT
			NdisQueryBufferSafe(CurrentBuffer, &VirtualAddress, &CurrentLength, 
				NormalPagePriority);
#else
            NdisQueryBuffer(
                CurrentBuffer,
                &VirtualAddress,
                &CurrentLength);
#endif

			RestSpace = CurrentLength;
		}
	}

}


//-----------------------------------------------------------------------------
// Procedure:   AcquireCoalesceBuffer
//
// Description: This routine will attempt to acquire a coalesce buffer, if
//              there were too many physical vectors for a single packet.
//              The coalesce buffer will allow us to double buffer using a
//              only a single destination buffer.
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//      SwTcb - Pointer to a software structure that represents a hardware TCB.
//
// Returns:
//      TRUE If we were able to acquire the necessary Coalesce buffer.
//      FALSE If we were not able to get a coalesce buffer.
//-----------------------------------------------------------------------------

BOOLEAN
AcquireCoalesceBuffer(PZD1205_ADAPTER Adapter,
                      PCOALESCE *pCoalesce
                      )
{
    DEBUGFUNC(_T("AcquireCoalesceBuffer"));

    // Check preconditions for use. Return FALSE if there are none
    // available.
    if (QueueEmpty(&Adapter->CoalesceBufferList))
    {
        INITSTR((_T("No free coalesce buffers!!!\n")));
        return (FALSE);
    }

    // Allocate the coalesce buffer from the list. The SwTcb will now
    // own it until it's returned to the list.

    *(pCoalesce) = (PCOALESCE) QueuePopHead(&Adapter->CoalesceBufferList);


    return (TRUE);
}

//-----------------------------------------------------------------------------
// Procedure:   TransmitCleanup
//
// Description: This routine will clean up after a transmitted frame.  It will
//              update the transmit statistic counters, free up TCBs and map
//              regs, and issue a send complete which will unlock the pages.
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//
// Returns:
//      TRUE - If we indicated any loopback packets during this function call
//      FALSE - If we didn't indicate any loopaback packets
//-----------------------------------------------------------------------------

BOOLEAN
TransmitCleanup(PZD1205_ADAPTER Adapter,
				PZD1205SwTcb SwTcb
				)
{
	PHWTCB_STRUC		HwTcb;
    BOOLEAN             Status = FALSE;
	PZD1205SwTcb		SwTcbTmp;
	ULONG				DebugPrint;
    DEBUGFUNC(_T("TransmitCleanup"));

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_TX;

	HwTcb = SwTcb->Tcb;

    ASSERT(SwTcb);

	//INITSTRTX(DebugPrint,(_T("Freed serial No %x\n"), (USHORT)Adapter->FreeTotalCnt++));
	//INITSTRTX(DebugPrint,(_T("Free HwTcb = %x (Phys = %x)\n"), SwTcb->Tcb, SwTcb->TcbPhys));

	SwTcbTmp = (PZD1205SwTcb)QueueGetTail(&Adapter->TxCBList);
	if (SwTcbTmp->TcbCount != (SwTcb->TcbCount -1)){
#ifdef HOST_IF_USB
//		ASSERT( (SwTcbTmp->TcbCount == 31) && (SwTcb->TcbCount == 0) );
#else
		ASSERT( (SwTcbTmp->TcbCount == 15) && (SwTcb->TcbCount == 0) );
#endif
	}
	QueuePutTail(&Adapter->TxCBList, &SwTcb->Link);

	SwTcb->HangDur = 0;
	HwTcb->CbCommand = CB_S_BIT;
	HwTcb->CbStatus = 0xffffffff;
	//HwTcb->TxCbTbdNumber = 0;

    return TRUE;
}

void 
GetEthernetHeaderFromPacket(
		PZD1205_ADAPTER Adapter, 
		PNDIS_PACKET Packet,
		PNDIS_BUFFER FirstBuf,
		PETH_HEADER_STRUC pEnetHdr
		)
{
	ULONG	BytesCopied;

	ZD1205CopyFromPacketToBuffer(Adapter,
            Packet,
            14, // Length of Ethnet header
            (PCHAR) pEnetHdr,
            FirstBuf,
            &BytesCopied);
	ASSERT(BytesCopied == 14);	
	return;
}

ULONG
CalNumOfFrag(PZD1205_ADAPTER Adapter, 
			 ULONG length
			 )
{
	ULONG	FragNo = 0;

	if (Adapter->CardSetting.FragThreshold == 0){
		// mean disable fragmetation
		return(FragNo+1);
	}

	if (Adapter->CardSetting.FragThreshold < 256){
		Adapter->CardSetting.FragThreshold = 256;
	}
	while(1){
		if (length > Adapter->CardSetting.FragThreshold){
			FragNo ++;
			length -= Adapter->CardSetting.FragThreshold;
		}
		else
			break;
	}
	return(FragNo+1);
}

void
StartDownload(PZD1205_ADAPTER Adapter, 
			  ULONG		PhyAddr
			  )
{
	ULONG		DebugPrint;
    DEBUGFUNC(_T("StartDownload"));	

	DebugPrint = Adapter->DbgMsgFlag & DBGMSG_TX;

#ifdef HOST_IF_USB
	return;
#endif

	if (!Adapter->DeviceInSleep){
		ZD1205_WRITE_REGISTER(Adapter,PCI_TxAddr_p1,PhyAddr);
		INITSTRTX(DebugPrint,(_T("********Write PCI_TxAddr_p1 %x\n"), PhyAddr));
	}
}