/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    u_RegSet.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

#define bmZD_IF_LE              1
#define bmZD_RF_CLK             2
#define bmZD_RF_DATA            3

BOOLEAN
USB_READ_REGISTER(  IN PZD1205_ADAPTER  Adapter,
                    ULONG               Address,
                    PULONG              pValue,
                    BOOLEAN             AddUSBCSRAddress
                    )
{
	BOOLEAN	bIsPhyReg = 0;
	USHORT  ReadAddr[2];
	USHORT  ReadData[2];
	BOOLEAN bRet;


	if (AddUSBCSRAddress && (Address > CR0) && (Address < PHY_END))
		bIsPhyReg = 1;

	if (bIsPhyReg)
	{
		ULONG		tmpvalue;

		ZD1205_READ_REGISTER(Adapter,CtlReg1, &tmpvalue);
		tmpvalue &= ~0x80;
		ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
	}

    if (AddUSBCSRAddress)
    {
        Address += (USHORT) (Adapter->USBCSRAddress);
        if ((Address & BASE_ADDR_MASK_HOST) == USB_BASE_ADDR_HOST)
            ReadAddr[1] = (USHORT) Address + 1;
        else
            ReadAddr[1] = (USHORT) Address + 2;
    }
    else
	{
        ReadAddr[1] = (USHORT) Address + 1;
	}

    ReadAddr[0] = (USHORT) Address;         // Read Low Word first

    bRet = USB_PACKAGE_READ_REGISTER(Adapter, ReadAddr, ReadData, 2, FALSE);
    *pValue = (((ULONG) ReadData[1]) << 16) + ReadData[0];

	if (bIsPhyReg)
	{
		ULONG		tmpvalue;

		ZD1205_READ_REGISTER(Adapter,CtlReg1, &tmpvalue);
		tmpvalue |= 0x80;
		ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
	}

    return bRet;
}


BOOLEAN
USB_WRITE_REGISTER( IN PZD1205_ADAPTER  Adapter,
                    ULONG               Address,
                    ULONG               Value,
                    BOOLEAN             AddUSBCSRAddress
                    )
{
#ifdef fQuickPhySet
	BOOLEAN	bIsPhyReg = 0;
#endif
	USHORT  WriteAddr[6];
	USHORT  WriteData[6];
	//static  int ShowCount = 0;

	// Setup Debug flag	
	ULONG				DebugPrint = Adapter->USBDbgFlag & DBGMSG_EP0;
	DEBUGFUNC(_T("USB_WRITE_REGISTER"));


#ifdef fQuickPhySet
	if (AddUSBCSRAddress && (Address > CR0) && (Address < PHY_END))
		bIsPhyReg = 1;

	if (bIsPhyReg)
	{
		ULONG		tmpvalue;

		ZD1205_READ_REGISTER(Adapter,CtlReg1, &tmpvalue);
		tmpvalue &= ~0x80;
		{
			if (((Adapter->USBCSRAddress+CtlReg1) & BASE_ADDR_MASK_HOST) == USB_BASE_ADDR_HOST)
				WriteAddr[0] = (USHORT) (Adapter->USBCSRAddress+CtlReg1) + 1;
			else
				// must write High word first
				WriteAddr[0] = (USHORT) (Adapter->USBCSRAddress+CtlReg1) + 2;
		}
		WriteData[0] = (USHORT) (tmpvalue >> 16);
		WriteAddr[1] = (USHORT) (Adapter->USBCSRAddress+CtlReg1);
		WriteData[1] = (USHORT) (tmpvalue & 0xFFFF);
		if (AddUSBCSRAddress)
		{
			Address += (USHORT) (Adapter->USBCSRAddress);
			if ((Address & BASE_ADDR_MASK_HOST) == USB_BASE_ADDR_HOST)
				WriteAddr[2] = (USHORT) Address + 1;
			else
				// must write High word first
				WriteAddr[2] = (USHORT) Address + 2;
		}
		else
		{
			WriteAddr[2] = (USHORT) Address + 1;
		}

		WriteData[2] = (USHORT) (Value >> 16);
		WriteAddr[3] = (USHORT) Address;
		WriteData[3] = (USHORT) (Value & 0xFFFF);


		tmpvalue |= 0x80;
		{
			if (((Adapter->USBCSRAddress+CtlReg1) & BASE_ADDR_MASK_HOST) == USB_BASE_ADDR_HOST)
				WriteAddr[4] = (USHORT) (Adapter->USBCSRAddress+CtlReg1) + 1;
			else
				// must write High word first
				WriteAddr[4] = (USHORT) (Adapter->USBCSRAddress+CtlReg1) + 2;
		}
		WriteData[4] = (USHORT) (tmpvalue >> 16);
		WriteAddr[5] = (USHORT) (Adapter->USBCSRAddress+CtlReg1);
		WriteData[5] = (USHORT) (tmpvalue & 0xFFFF);

		return USB_PACKAGE_WRITE_REGISTER(Adapter, WriteAddr, WriteData, 6, FALSE);
	}
	else
	{
#endif
		if (AddUSBCSRAddress)
		{
			Address += (USHORT) (Adapter->USBCSRAddress);
			if ((Address & BASE_ADDR_MASK_HOST) == USB_BASE_ADDR_HOST)
				WriteAddr[0] = (USHORT) Address + 1;
			else
		        // must write High word first
				WriteAddr[0] = (USHORT) Address + 2;
		}
		else
		{
			WriteAddr[0] = (USHORT) Address + 1;
		}

		WriteData[0] = (USHORT) (Value >> 16);
		WriteAddr[1] = (USHORT) Address;
		WriteData[1] = (USHORT) (Value & 0xFFFF);
		return USB_PACKAGE_WRITE_REGISTER(Adapter, WriteAddr, WriteData, 2, FALSE);

#ifdef fQuickPhySet
	}
#endif
    
}

VOID
Set_IF_Synthesizer(
			IN PZD1205_ADAPTER Adapter, 
			IN ULONG InputValue
		)
{

	ULONG	S_bit_cnt;
	ULONG	tmpvalue;

	//USHORT  WriteAddr[cMAX_MULTI_WRITE_REG_NUM];
	USHORT  WriteData[cMAX_MULTI_RF_REG_NUM];
	USHORT  WriteIndex = 0;
	static  USHORT ShowCount = 0;

	// Setup Debug flag	
	ULONG				DebugPrint = Adapter->USBDbgFlag & DBGMSG_EP0;
	DEBUGFUNC(_T("Set_IF_Synthesizer"));


    S_bit_cnt = Adapter->S_bit_cnt;
    InputValue = InputValue << (31 - S_bit_cnt);


    AcquireCtrOfPhyReg(Adapter);
    ZD1205_READ_REGISTER(Adapter, CR203, &tmpvalue);

    tmpvalue = mCLR_BIT(tmpvalue, bmZD_IF_LE);

    // Configure RF by Software
    tmpvalue = mCLR_BIT(tmpvalue, bmZD_RF_CLK);

    while (S_bit_cnt)
    {
        InputValue = InputValue << 1;
        if (InputValue & 0x80000000)
        {
            tmpvalue = mSET_BIT(tmpvalue, bmZD_RF_DATA);
            mFILL_RF_REGISTER((USHORT) tmpvalue);
        }
        else
        {
            tmpvalue = mCLR_BIT(tmpvalue, bmZD_RF_DATA);
            mFILL_RF_REGISTER((USHORT) tmpvalue);
        }

        if (WriteIndex >= cMAX_MULTI_RF_REG_NUM)
        {
            JWeiDbgPrint5(("S_bit_cnt over range! %d, ", Adapter->S_bit_cnt));
            break;
        }

        S_bit_cnt --;
    }

    USB_SET_RF_REG(Adapter, WriteData, 0);

    ReleaseCtrOfPhyReg(Adapter);
    

}

void
USB_SET_RF_REG( IN PZD1205_ADAPTER  Adapter,
                USHORT              *InputValue,
                BOOLEAN             bIs3683A
              )
{
	PVOID					pRegBuffer = 0;
	ULONG					S_bit_cnt;
	ZD1211_EVENT			event;
    USHORT                  ii;
    USB_TRANSFER            hRegOut;
	DWORD                   timeToWait, WaitStatus;
	// Setup Debug flag	
	ULONG				DebugPrint = Adapter->USBDbgFlag & DBGMSG_EP0;
	DEBUGFUNC(_T("USB_SET_RF_REG"));

	if (!Adapter->bUSBDeveiceAttached)
	{
		//USBWARNSTR(("bUSBDeveiceAttached = %d\n", Adapter->bUSBDeveiceAttached));
	    //INITSTR((_T("Device not attached!!\n")));
		return;
	}

#ifdef fQuickPhySet
	{
		ULONG		tmpvalue;

		ZD1205_READ_REGISTER(Adapter,CtlReg1, &tmpvalue);
		tmpvalue &= ~0x80;
		ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
	}
#endif
	S_bit_cnt = Adapter->S_bit_cnt;

	//INITSTR((_T("waiting for RegSpinlockEvent\n")));
	WaitForSingleObject(Adapter->RegSpinlockEvent, INFINITE);
	//INITSTR((_T("got RegSpinlockEvent\n")));

	//USBDBGSTR(DebugPrint,(" - InputValue=0x%x S_bit_cnt=0X%x ^^^^^^^^^^^^^^^^\n", InputValue, S_bit_cnt));
    //INITSTR((_T(" - InputValue=0x%x S_bit_cnt=0X%x ^^^^^^^^^^^^^^^^\n"), InputValue, S_bit_cnt));

	NdisAllocateMemory(	&pRegBuffer,
						sizeof(USB_SET_RF),
						0,
						HighestAcceptableMax
						);

	NdisZeroMemory(pRegBuffer, sizeof(USB_SET_RF));

	((PUSB_SET_RF)pRegBuffer)->RequestID	= REGID_RFOFDMSET;
	
	if (bIs3683A)
		((PUSB_SET_RF)pRegBuffer)->Value	= 1;
	else
		((PUSB_SET_RF)pRegBuffer)->Value	= 2;
	
	((PUSB_SET_RF)pRegBuffer)->Index		= (USHORT)S_bit_cnt;

    for (ii = 0; ii < S_bit_cnt; ii ++)
        ((PUSB_SET_RF)pRegBuffer)->Data[ii] = InputValue[ii];

	event = CreateEvent(NULL, FALSE, FALSE, NULL);
	Adapter->RegEvent = event;
    //Adapter->epstate |= BIT_0;

	//DbgPrint("write reg TransferLockEvent");
	//WaitForSingleObject(Adapter->TransferLockEvent, INFINITE);
    hRegOut = (Adapter->USBInfo.lpUsbFuncs->lpIssueBulkTransfer)(
		                  Adapter->USBInfo.hPipe[EP_REG_OUT],
						  USB_MPWriteReg_Complete,
						  Adapter,
                          USB_OUT_TRANSFER,
                          sizeof(USHORT) * (3 + S_bit_cnt),
                          pRegBuffer,
						  0
		                  );
    //SetEvent(Adapter->TransferLockEvent);
	//Adapter->epstate &= ~BIT_0;
    if ( hRegOut != NULL )
	{
        timeToWait = 1000 * IOWAITTIME; 

        WaitStatus = WaitForSingleObject( event, timeToWait);

		(Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(hRegOut);

		if ( WaitStatus == WAIT_TIMEOUT )
		{			
			INITSTR((_T("write register error\n")));        
	        WaitForSingleObject(event, INFINITE);
		}

    }
	else
	{
	    (Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(hRegOut);
	}

	if (pRegBuffer)
		NdisFreeMemory(pRegBuffer, sizeof(USB_SET_RF), 0);

	//USBDBGSTR(DebugPrint,(" - InputValue=0x%x S_bit_cnt=0X%x end ------------\n", InputValue, S_bit_cnt));
    SetEvent(Adapter->RegSpinlockEvent);

	CloseHandle(event);

#ifdef fQuickPhySet
	{
		ULONG		tmpvalue;

		ZD1205_READ_REGISTER(Adapter,CtlReg1, &tmpvalue);
		tmpvalue |= 0x80;
		ZD1205_WRITE_REGISTER(Adapter,CtlReg1, tmpvalue);
	}
#endif

}
