/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    u_rx.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

#ifndef DUMPTX
    #if fDUMP_LOOPBACK_DATA
#define DUMPRX
    #endif
#endif

//---------------------------------------------------------------------------
//	Function prototype
//---------------------------------------------------------------------------

VOID
ZD1205DisableInterrupt(
                     IN PZD1205_ADAPTER Adapter
)

{
	ULONG				DebugPrint = Adapter->USBDbgFlag & DBGMSG_RX;
    DEBUGFUNC(_T("ZD1205DisableInterrupt"));
    USBDBGSTR(DebugPrint,(_T("\n")));

	Adapter->bRxInterruptEnable = FALSE;

	ZD1205_WRITE_REGISTER(Adapter,InterruptCtrl, (0)<<16);

	// For WHQL RESET test, We can't calcel Rx Irp too often, so don't stop Rx here
	//USB_StopRxEP(Adapter);
	
}


//	Start Rx IRP if present
//	and setup IRP, URB, and Context for reuse

VOID
ZD1205EnableInterrupt(
                    IN PZD1205_ADAPTER Adapter
)
{
	ULONG				DebugPrint = Adapter->USBDbgFlag & DBGMSG_RX;
	ZD1211_IRQL			OldIrql;
    
	DEBUGFUNC(_T("ZD1205EnableInterrupt"));
    USBDBGSTR(DebugPrint,(_T("\n")));

	ZD1205_WRITE_REGISTER(Adapter,SendToHost, 0xDfffffff);
	
	ZD1205_WRITE_REGISTER(Adapter,InterruptCtrl, (Adapter->IntMask & ~TX_COMPLETE & ~RX_COMPLETE & ~RETRY_FAIL)<<16);

	//ZDDbg((_T("Get SpinLock\n")));
	//ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	//USB_StartRxEP(Adapter);
	//ZDDbg((_T("---------->>> Release SpinLock\n")));
	//ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
	SetEvent(Adapter->StartRxEvent);

	Adapter->bRxInterruptEnable = TRUE;

}



NDIS_STATUS
USB_StartRxEP(PZD1205_ADAPTER Adapter)
{
    NDIS_STATUS   status;
	ZD1211_IRQL   OldIrql;

	DEBUGFUNC(_T("USB_StartRxEP"));
	//INITSTR((_T("\n")));

	if ( Adapter->USBInfo.hRxEP != NULL )
	{
		INITSTR((_T("Rx EP already started!!!!\n")));
	
		return NDIS_STATUS_SUCCESS;
	}
    
    //ResetEvent(Adapter->USBRxEPStopEvent);

	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	status = USB_MPRcvFromRxEP(Adapter);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);


	return status;
}

NDIS_STATUS
USB_StopRxEP(PZD1205_ADAPTER Adapter)
{
	return NDIS_STATUS_SUCCESS;
}

NDIS_STATUS
USB_MPRcvFromRxEP(PZD1205_ADAPTER Adapter)
{

    PZD1205SwRfd			SwRfd;
	UINT					MaxBufLen = MAX_WLAN_SIZE;
    PVOID                   tmpPtr;
	ZD1211_IRQL		    	OldIrql;
	HANDLE                  handle;
	DWORD                   oldTime;
	int                     oldp;
	BOOL                    bRet;
	DWORD                   dwLen, dwError, ret;

	DEBUGFUNC(_T("USB_MPRcvFromRxEP"));

    ZDUSBDbg((_T("----- start -----\n")));

	if (!Adapter->bUSBDeveiceAttached)
	{
	    //INITSTR((_T("Device not attached!!\n")));
		USB_FreeRxEPResource(Adapter);
		return NDIS_STATUS_FAILURE;
	}
	
    if ( Adapter->USBInfo.hRxEP != NULL )
	{
	    return NDIS_STATUS_SUCCESS;
	}

    ResetEvent(Adapter->WaitRxEvent);


    SwRfd = (PZD1205SwRfd) QueueGetHead(&Adapter->RfdList);
	
	//if ( (MAX_WLAN_SIZE - SwRfd->UnFinishFrmLen) > USBD_DEFAULT_MAXIMUM_TRANSFER_SIZE)
	//	MaxBufLen = USBD_DEFAULT_MAXIMUM_TRANSFER_SIZE;
	//else
	MaxBufLen = MAX_WLAN_SIZE - SwRfd->UnFinishFrmLen;

    if (SwRfd->UnFinishFrmLen)
        tmpPtr = (PVOID)(SwRfd->Rfd->RxBuffer + SwRfd->UnFinishFrmLen);
    else
        tmpPtr = (PVOID)SwRfd->Rfd->RxBuffer;

    //(Adapter->USBInfo.lpUsbFuncs->lpResetPipe)(Adapter->USBInfo.hPipe[EP_DATA_IN]);
    handle = GetCurrentThread();
	//oldTime = CeGetThreadQuantum(handle);
	//CeSetThreadQuantum(handle, 0);  // no context switching
    oldp = CeGetThreadPriority(handle);
	//ZDDbg((_T("---------->>> Release SpinLock\n")));
	//ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
    //WaitForSingleObject(Adapter->TransferLockEvent, INFINITE);
	//if ( Adapter->txrun )
        //DbgPrint("rx get event");
	//ZDDbg((_T("Get SpinLock\n")));
	//ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	Adapter->USBInfo.hRxEP = (Adapter->USBInfo.lpUsbFuncs->lpIssueBulkTransfer)(
									   Adapter->USBInfo.hPipe[EP_DATA_IN],
									   USB_MPRcvFromRxEP_Complete,
									   (LPVOID) Adapter,
									   USB_IN_TRANSFER | USB_SHORT_TRANSFER_OK,
									   MaxBufLen,
									   tmpPtr,
									   0
									   );
	//SetEvent(Adapter->TransferLockEvent);
	//CeSetThreadQuantum(handle, oldTime);

	if ( Adapter->USBInfo.hRxEP == NULL )
	{
		USB_FreeRxEPResource(Adapter);
	    INITSTR((_T("Failed to start RX EP!!\n")));
        //SetEvent(Adapter->TransferLockEvent);
		//ZDDbg((_T("Get SpinLock\n")));
		//ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
		return NDIS_STATUS_FAILURE;
	}

	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

    ret = WaitForSingleObject(Adapter->WaitRxEvent, INFINITE);
	if ( Adapter->bEndThreads )
	{
		DbgPrint("h_ZD1211StartRxThread end");
		Adapter->h_ZD1211StartRxThread = NULL;
		ExitThread(0);
	}
  
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

    bRet = (Adapter->USBInfo.lpUsbFuncs->lpGetTransferStatus)(
		               Adapter->USBInfo.hRxEP,
					   &dwLen,
					   &dwError
					   );
     
    if ( (dwError != USB_NO_ERROR)||(!bRet) ) 
	{
		//INITSTR((_T("Rx EP error occurred!!\n")));
		INITSTR((_T("####### damm it = %x ####### "), dwError));
		DbgPrint("####### damm it = %x ####### ", dwError);
        //(Adapter->USBInfo.lpUsbFuncs->lpResetPipe)(Adapter->USBInfo.hPipe[EP_DATA_IN]);
		//Adapter->USBInfo.hRxEP = NULL;
        //USBWARNSTR(("IoStatus.Status != STATUS_SUCCESS = 0x%x\n", pIrp->IoStatus.Status));
		//USB_FreeRxEPResource(Adapter);
		//SetEvent(Adapter->StartRxEvent);

		//if (Adapter->bRxInterruptEnable){
		//	Mgt_SendNotification(Adapter, RQ_RESTART_TXRXEP);	
		//}		
    }
	else
	{
		PZD1205SwRfd			SwRfd;
        PUCHAR                  pRxBuffer;
        UINT                    TotalLength = (UINT) dwLen;
        PZD1205SwRfd            SwRfdArray[3];
        int                     RxSwRfdTotal = 1, SwRfdArrayCnt = 0;
        int                     i;
        UINT                    tmpLen = 0;
        USHORT                  last_pkt_len;
		PVOID                   tmpPtr;

		SwRfd = (PZD1205SwRfd) QueueGetHead(&Adapter->RfdList);
		if (SwRfd->UnFinishFrmLen)
			tmpPtr = (PVOID)(SwRfd->Rfd->RxBuffer + SwRfd->UnFinishFrmLen);
		else
			tmpPtr = (PVOID)SwRfd->Rfd->RxBuffer;
		//--

        pRxBuffer = (PUCHAR) (SwRfd->Rfd->RxBuffer);

        //USBDBGSTR(DebugPrint,("Get Length %d\n", TotalLength));
		//INITSTR((_T("Get Length %d\n"), dwLen));
        

        if (SwRfd->UnFinishFrmLen)
        {
            ASSERT(TotalLength == 8);
            TotalLength += SwRfd->UnFinishFrmLen; 
            SwRfd->UnFinishFrmLen = 0;
        }

        last_pkt_len = TotalLength & (Adapter->USBInfo.MaxPacketSize - 1);

        if (last_pkt_len <= (Adapter->USBInfo.MaxPacketSize - 4))
        {
            if (((PUSHORT) pRxBuffer)[TotalLength / sizeof(USHORT) - 1]
                    == 0x697E)
            {
                RxSwRfdTotal = 3;
                Adapter->Continue2Rx++;
            }

            for (i = 0; i < RxSwRfdTotal; i++)
            {
				int CurFrmLen;

                if (RxSwRfdTotal > 1)
                    CurFrmLen = ((PUSHORT) pRxBuffer)[TotalLength / sizeof(USHORT) + i - 4];
                else
                    CurFrmLen = TotalLength;

                if (CurFrmLen == 0)
                    break;

                if (QueueGetCount(&Adapter->RfdList) <= 1)
                    break;

                SwRfdArray[i] = (PZD1205SwRfd) QueueGetHead(&Adapter->RfdList);
                QueueRemoveHead(&Adapter->RfdList);
                SwRfdArrayCnt++;

                //USBDBGSTR(DebugPrint,("Get Length %d, SwRfdArray = 0x%x\n", CurFrmLen, SwRfdArray[i]));
                //INITSTR((_T("Get Length %d, SwRfdArray = 0x%x\n"), CurFrmLen, SwRfdArray[i]));

				SwRfdArray[i]->Rfd->CbStatus = RFD_STATUS_COMPLETE;
                SwRfdArray[i]->Rfd->ActualCount = CurFrmLen;

                if (i > 0)
                    NdisMoveMemory( SwRfdArray[i]->Rfd->RxBuffer, 
                                    pRxBuffer + tmpLen,
                                    SwRfdArray[i]->Rfd->ActualCount
                                    );

                tmpLen += (SwRfdArray[i]->Rfd->ActualCount & ~0x03);
                if (SwRfdArray[i]->Rfd->ActualCount & 0x03)
                    tmpLen += 4;
            }
        }
        else
        {
    // last_pkt_len = 509, 510, 511
    // wait next Rx
            Adapter->WaitLenInfoCnt++;
            SwRfd->UnFinishFrmLen = ((TotalLength / Adapter->USBInfo.MaxPacketSize) + 1) * Adapter->USBInfo.MaxPacketSize;
        }

        (Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(Adapter->USBInfo.hRxEP);
	    Adapter->USBInfo.hRxEP = NULL;

        if (!SwRfd->UnFinishFrmLen)
        {
			// indicate one frame to NDIS one time
            for (i = 0; i < SwRfdArrayCnt; i++){
                ProcessRXInterrupt(Adapter, &(SwRfdArray[i]), 1, &OldIrql);
			}
        }
	}

    ZDUSBDbg((_T("-----  end  -----\n")));

    return NDIS_STATUS_SUCCESS;
}

DWORD WINAPI
USB_MPRcvFromRxEP_Complete(LPVOID pContext)
{
    PZD1205_ADAPTER Adapter = (PZD1205_ADAPTER) pContext;

	DEBUGFUNC(_T("USB_MPRcvFromRxEP_Complete"));
    //INITSTR((_T("\n")));
 

    Adapter->RxCount++;

    ZDUSBDbg((_T("----- start -----\n")));

	SetEvent(Adapter->WaitRxEvent);

	ZDUSBDbg((_T("-----  end  -----\n")));

	return 0;
}

void
USB_FreeRxEPResource(PZD1205_ADAPTER Adapter)
{
    if ( Adapter->USBInfo.hRxEP )
	{
	    (Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(Adapter->USBInfo.hRxEP);
        Adapter->USBInfo.hRxEP = NULL;
	}

    //SetEvent(Adapter->USBRxEPStopEvent);     
}