/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    u_inter.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

typedef struct _USB_REG_PACKAGE
{
    USHORT  Address;
    USHORT  Data;
} USB_REG_PACKAGE, *PUSB_REG_PACKAGE;

typedef struct _USB_READ_REG
{
    USHORT          RequestID;
    USB_REG_PACKAGE Package[cMAX_MULTI_READ_REG_NUM];
} USB_READ_REG, *PUSB_READ_REG;

typedef struct _USB_READ_REG_REQ
{
    USHORT  RequestID;
    USHORT  Address[cMAX_MULTI_READ_REG_NUM];
} USB_READ_REG_REQ, *PUSB_READ_REG_REQ;

typedef struct _USB_WRITE_REG_PACKAGE {

	USHORT	Address;
	USHORT	WriteData_low;

} USB_WRITE_REG_PACKAGE, * PUSB_WRITE_REG_PACKAGE;


typedef struct _USB_WRITE_REG
{
    USHORT  RequestID;
    USB_WRITE_REG_PACKAGE WritePackage[cMAX_MULTI_WRITE_REG_NUM];

} USB_WRITE_REG, * PUSB_WRITE_REG;


NDIS_STATUS
USB_StartIntEP(IN PZD1205_ADAPTER Adapter)
{
    NDIS_STATUS   status;
	// Setup Debug flag	
	ULONG				DebugPrint = Adapter->USBDbgFlag & DBGMSG_INT;
	DEBUGFUNC(_T("USB_StartIntEP"));


	if ( Adapter->USBInfo.hIntEP != NULL )
	{
		INITSTR((_T("Int Loop already start!!!!\n")));
	
		return STATUS_SUCCESS;
	}

	// prevent driver to e unload
	//ResetEvent(Adapter->USBIntEPStopEvent);

    status = USB_MPRcvFromIntEP(Adapter);

	return status;
}


NDIS_STATUS
USB_StopIntEP(IN PZD1205_ADAPTER Adapter)
{
	return NDIS_STATUS_SUCCESS;
}

NDIS_STATUS
USB_MPRcvFromIntEP(IN PZD1205_ADAPTER Adapter)
{

	DEBUGFUNC(_T("USB_MPRcvFromIntEP"));

	ZDUSBDbg((_T("----- start -----\n")));

    //if ( Adapter->bTerminateThreads )
	//	return NDIS_STATUS_FAILURE;

	if (!Adapter->bUSBDeveiceAttached)
	{
	    //INITSTR((_T("Device not attached!!\n")));
		USB_FreeINTEPResource(Adapter);
		return NDIS_STATUS_FAILURE;
	}

    if ( Adapter->USBInfo.hIntEP )
	{
		DbgPrint("INT error");
		return NDIS_STATUS_SUCCESS;
	}
    
    ResetEvent(Adapter->WaitIntEvent);
	//WaitForSingleObject(Adapter->TransferLockEvent, INFINITE);

    Adapter->USBInfo.hIntEP = (Adapter->USBInfo.lpUsbFuncs->lpIssueInterruptTransfer)(
		                                  Adapter->USBInfo.hPipe[EP_INT_IN],
										  USB_MPRcvFromIntEP_Complete,
										  (LPVOID) Adapter,
										  USB_IN_TRANSFER | USB_SHORT_TRANSFER_OK,
										  MAX_EPINT_BUFFER,
										  Adapter->IntEPBuffer,
										  0
										  );

	//SetEvent(Adapter->TransferLockEvent);
    
	if ( Adapter->USBInfo.hIntEP == NULL )
	{
	    //INITSTR((_T("Failed to start INT EP!!\n")));
        USB_FreeINTEPResource(Adapter);
		return NDIS_STATUS_FAILURE;
	}

    //INITSTR((_T("INT EP started!!\n")));
	//DbgPrint("INT EP started!!");

	ZDUSBDbg((_T("-----  end  -----\n")));

	return NDIS_STATUS_SUCCESS; 

}

DWORD WINAPI
USB_MPRcvFromIntEP_Complete(LPVOID pContext)
{
    PZD1205_ADAPTER Adapter = (PZD1205_ADAPTER) pContext;

	DEBUGFUNC(_T("USB_MPRcvFromIntEP_Complete"));   

	//INITSTR((_T("\n")));
    SetEvent(Adapter->WaitIntEvent);

    return 0;
}

void
USB_FreeINTEPResource(PZD1205_ADAPTER Adapter)
{
    if ( Adapter->USBInfo.hIntEP )
	{
        (Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(Adapter->USBInfo.hIntEP);
	    Adapter->USBInfo.hIntEP = NULL;
	}

    SetEvent(Adapter->EPINT_IORDRsp_Event);
    //SetEvent(Adapter->USBIntEPStopEvent);

}

BOOLEAN
USB_PACKAGE_READ_REGISTER(  IN PZD1205_ADAPTER  Adapter,
                            USHORT              *Address,
                            USHORT              *pValue,
                            USHORT              RegCount,
                            BOOLEAN             AddUSBCSRAddress
                          )
{
	PVOID					pRegBuffer = 0;
	ZD1211_EVENT			event;
    DWORD                   timeToWait, WaitStatus;

	// Setup Debug flag	
	ULONG				DebugPrint = Adapter->USBDbgFlag & DBGMSG_EP0;

	DEBUGFUNC(_T("USB_PACKAGE_READ_REGISTER"));

    ZDUSBDbg((_T("----- start -----\n")));

    ASSERT(RegCount <= cMAX_MULTI_READ_REG_NUM);
    if (RegCount == 0)
        return TRUE;

	if (!Adapter->bUSBDeveiceAttached)
	{
	    //INITSTR((_T("Device not attached!!\n")));	
		return TRUE;
	}

	WaitForSingleObject(Adapter->RegSpinlockEvent, INFINITE);
    
    //USBDBGSTR(DebugPrint,(" - Address=0x%x ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n", Address[0]));
    //INITSTR((_T(" - Address=0x%x ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"), Address[0]));
	
	if (!Adapter->bUSBDeveiceAttached)
	{
		//USBWARNSTR(("bUSBDeveiceAttached = %d\n", Adapter->bUSBDeveiceAttached));
	    //INITSTR((_T("USB device not attached!!\n")));
        SetEvent(Adapter->RegSpinlockEvent);

		return TRUE;
	}
	
	NdisAllocateMemory(	&pRegBuffer,
                        sizeof(USB_READ_REG_REQ),   //JWEI 2004/03/14
						0,
						HighestAcceptableMax
						);
    NdisZeroMemory(pRegBuffer, sizeof(USB_READ_REG_REQ));

    ((PUSB_READ_REG_REQ)pRegBuffer)->RequestID  = REGID_READ;

    {
        int ii;

        for (ii = 0; ii < RegCount; ii ++)
        {
            if ((Address[ii] & BASE_ADDR_MASK_HOST) == USB_BASE_ADDR_HOST)
                Address[ii] = Address[ii] - USB_BASE_ADDR_HOST + Adapter->AddrEntryTable;
            else if ((Address[ii] & BASE_ADDR_MASK_HOST) == USB_BASE_ADDR_EEPROM)
                Address[ii] = ((Address[ii] - USB_BASE_ADDR_EEPROM) / 2) + cFIRMWARE_EEPROM_OFFSET;

            ((PUSB_READ_REG_REQ) pRegBuffer)->Address[ii] = Address[ii];
        }
    }

	event = CreateEvent(NULL, FALSE, FALSE, NULL);
	Adapter->RegEvent = event;
    ResetEvent(Adapter->EPINT_IORDRsp_Event);

    SetThreadPriority(Adapter->h_ZD1211InterruptThread, THREAD_PRIORITY_TIME_CRITICAL);
	//DbgPrint("read reg TransferLockEvent");
	WaitForSingleObject(Adapter->TransferLockEvent, INFINITE);
    Adapter->USBInfo.hRegEP = (Adapter->USBInfo.lpUsbFuncs->lpIssueBulkTransfer)(
		                  Adapter->USBInfo.hPipe[EP_REG_OUT],
						  USB_MPWriteReg_Complete,
						  Adapter,
                          USB_OUT_TRANSFER,
                          sizeof(USHORT) * (1 + RegCount),
                          pRegBuffer,
						  0
		                  );
   //SetEvent(Adapter->TransferLockEvent);
    if ( Adapter->USBInfo.hRegEP != NULL )
	{

        timeToWait = 1000 * IOWAITTIME; 

        WaitStatus = WaitForSingleObject( event, timeToWait);

		(Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(Adapter->USBInfo.hRegEP);

		if ( WaitStatus == WAIT_TIMEOUT )
		{			
			//USBWARNSTR(("IoCallDriver timout! Cancel IRP\n"));
			INITSTR((_T("read register timeout\n"))); 
	        WaitForSingleObject( event, INFINITE);		
		}
    }
	else
	{
		INITSTR((_T("read register error")));
	    (Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(Adapter->USBInfo.hRegEP);
		WaitStatus = STATUS_TIMEOUT;
	}

	if ( pRegBuffer )
        NdisFreeMemory(pRegBuffer, sizeof(USB_READ_REG_REQ), 0);

	if ( WaitStatus == WAIT_OBJECT_0 )
	{
        timeToWait = 1000 * IOWAITTIME; 

        WaitStatus = WaitForSingleObject( Adapter->EPINT_IORDRsp_Event, timeToWait);

		SetThreadPriority(Adapter->h_ZD1211InterruptThread, THREAD_PRIORITY_NORMAL);

		if ( WaitStatus == WAIT_TIMEOUT )
		{
			
			//USBWARNSTR(("Wait EPINT_IORDRsp_Event timeout! 0x%x\n", *Address));
            INITSTR((_T("Wait EPINT_IORDRsp_Event timeout! 0x%x\n"), *Address));
			
			goto USB_READ_REGISTER_EXIT;
		}

		// Get data

    //JWEI 2004/03/14
        {
            UCHAR ii;

            if ((Adapter->ReadRegCount == 0) || (Adapter->ReadRegCount > MAX_EPINT_BUFFER))
			{
                //JWeiDbgPrint5(("Len %d Error!\n", Adapter->ReadRegCount));
				INITSTR((_T("Len %d Error!\n"), Adapter->ReadRegCount));
			}
            else
            {
                for (ii = 0; ii < (Adapter->ReadRegCount - 2) / 4; ii ++)
                    pValue[ii] = *((PUSHORT) Adapter->IntEPBuffer2 + 1 + ii * 2 + 1);
            }
        }
        
		//USBDBGSTR(DebugPrint,("Get Value - Address=0x%x Value=0x%x ----------------\n", Address, *pValue));
		//{
        //    int i;

		//	for(i=0; i<2; i++)
		//	{
		//		INITSTR((_T("Get Value - Address=0x%x ----------------\n"), Address[i]));
		//		INITSTR((_T("Get Value - Value=0x%x ----------------\n"), pValue[i]));	
		//	}
		//}
	}
	else{
		//USBWARNSTR((" - Address=0x%x first step fail ----------------\n", Address));
		INITSTR((_T(" - Address=0x%x first step fail ----------------\n"), Address));
	}

USB_READ_REGISTER_EXIT:	
	
    SetEvent(Adapter->RegSpinlockEvent);

    CloseHandle(event);

    ZDUSBDbg((_T("-----  end  -----\n")));

	if (WaitStatus == STATUS_TIMEOUT)
		return FALSE;
	else
		return TRUE;
}

BOOLEAN
USB_PACKAGE_WRITE_REGISTER( IN PZD1205_ADAPTER  Adapter,
                            USHORT              *Address,
                            USHORT              *Value,
                            USHORT              RegCount,
                            BOOLEAN             AddUSBCSRAddress
                          )
{
	PVOID					pRegBuffer = 0;
	ZD1211_EVENT			event;
	USB_TRANSFER            hRegOut;
    DWORD                   timeToWait, WaitStatus;
	//UCHAR                   i;
	
	// Setup Debug flag	
	ULONG				DebugPrint = Adapter->USBDbgFlag & DBGMSG_EP0;
	DEBUGFUNC(_T("USB_PACKAGE_WRITE_REGISTER"));

    ZDUSBDbg((_T("----- start -----\n")));

    ASSERT(RegCount <= cMAX_MULTI_WRITE_REG_NUM);

	if (RegCount == 0)	return TRUE;

	if (!Adapter->bUSBDeveiceAttached)
	{
	    //INITSTR((_T("Device not attached!!\n")));	
		return TRUE;
	}

	WaitForSingleObject(Adapter->RegSpinlockEvent, INFINITE);
    
	//USBDBGSTR(DebugPrint,(" - Address=0x%x Value=0X%x ^^^^^^^^^^^^^^^^^^^^^^\n", Address[0], Value[0]));
	//for(i=0; i<RegCount; i++)
	//{
	//	INITSTR((_T(" - Address=0x%x ^^^^^^^^^^^^^^^^^^^^^^\n"), Address[i]));
	//	INITSTR((_T(" - Value=0X%x ^^^^^^^^^^^^^^^^^^^^^^\n"), Value[i]));
	//}

	if (!Adapter->bUSBDeveiceAttached)
	{
	    //INITSTR((_T("USB device not attached!!\n")));
        SetEvent(Adapter->RegSpinlockEvent);	
		return TRUE;
	}

	NdisAllocateMemory(	&pRegBuffer,
						sizeof(USB_WRITE_REG),
						0,
						HighestAcceptableMax
						);
	
	NdisZeroMemory(pRegBuffer, sizeof(USB_WRITE_REG));

	((PUSB_WRITE_REG)pRegBuffer)->RequestID		= REGID_WRITE;

    if (RegCount > cMIN_MULTI_WRITE_REG_NUM)
    {
        int i;

        for (i = cMIN_MULTI_WRITE_REG_NUM; i < RegCount; i ++)
        {
            if (AddUSBCSRAddress)
                Address[i] += (USHORT) Adapter->USBCSRAddress;
            if ((Address[i] & BASE_ADDR_MASK_HOST) == USB_BASE_ADDR_HOST)
                Address[i] = Address[i] - USB_BASE_ADDR_HOST + Adapter->AddrEntryTable;
            else if ((Address[i] & BASE_ADDR_MASK_HOST) == USB_BASE_ADDR_EEPROM)
                Address[i] = ((Address[i] - USB_BASE_ADDR_EEPROM) / 2) + cFIRMWARE_EEPROM_OFFSET;

            ((PUSB_WRITE_REG)pRegBuffer)->WritePackage[i].Address = Address[i];
			((PUSB_WRITE_REG)pRegBuffer)->WritePackage[i].WriteData_low = Value[i];
       }
    }

	event = CreateEvent(NULL, FALSE, FALSE, NULL);
	Adapter->RegEvent = event;

    //(Adapter->USBInfo.lpUsbFuncs->lpResetPipe)(Adapter->USBInfo.hPipe[EP_REG_OUT]);
    //INITSTR((_T("debug check\n")));
	//Adapter->epstate |= BIT_1;
	//DbgPrint("write reg TransferLockEvent");
	//WaitForSingleObject(Adapter->TransferLockEvent, INFINITE);
	hRegOut = (Adapter->USBInfo.lpUsbFuncs->lpIssueBulkTransfer)(
		                  Adapter->USBInfo.hPipe[EP_REG_OUT],
						  USB_MPWriteReg_Complete,
						  Adapter,
                          USB_OUT_TRANSFER,
                          sizeof(USHORT) * (1 + RegCount * 2),
                          pRegBuffer,
						  0
		                  );
	//SetEvent(Adapter->TransferLockEvent);
    //Adapter->epstate &= ~BIT_1;
    if ( hRegOut != NULL )
	{

        timeToWait = 1000 * IOWAITTIME; 
        WaitStatus = WaitForSingleObject( event, timeToWait);

        (Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(hRegOut);

		if ( WaitStatus == WAIT_TIMEOUT )
		{	
			//USBWARNSTR(("IoCallDriver timout! Cancel IRP\n"));
			INITSTR((_T("write register timeout\n"))); 
	        WaitForSingleObject( event, INFINITE);		
		}
    }
	else
	{
	    (Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(hRegOut);
		WaitStatus = STATUS_TIMEOUT;
	}


	//USBDBGSTR(DebugPrint,(" - Address=0x%x Value=0X%x end ------------------\n", Address[0], Value[0]));
    //INITSTR((_T(" - Address=0x%x Value=0X%x end ------------------\n"), Address[0], Value[0]));

    if (pRegBuffer)
        NdisFreeMemory(pRegBuffer, sizeof(USB_WRITE_REG), 0);
    
    SetEvent(Adapter->RegSpinlockEvent);

    CloseHandle(event);

    ZDUSBDbg((_T("-----  end  -----\n")));

    if ( WaitStatus == WAIT_TIMEOUT )
        return FALSE;
    else
        return TRUE;
}

DWORD WINAPI
USB_MPWriteReg_Complete(LPVOID pContext)
{
    PZD1205_ADAPTER Adapter = (PZD1205_ADAPTER) pContext;
    
	DEBUGFUNC(_T("USB_MPWriteReg_Complete"));
	//INITSTR((_T("\n")));


    ZDUSBDbg((_T("----- start -----\n")));
    SetEvent(Adapter->RegEvent);

    ZDUSBDbg((_T("-----  end  -----\n")));

	return 0;
}

NDIS_STATUS
USB_WaitAllEPStop(IN PZD1205_ADAPTER Adapter)
{
    UCHAR   i;

	for( i=0; i<4; i++ )
        Adapter->USBInfo.lpUsbFuncs->lpClosePipe(Adapter->USBInfo.hPipe[i]);

	return STATUS_SUCCESS;
}

void WriteEEPROM(IN PZD1205_ADAPTER  Adapter,
                 USHORT rom_adr, USHORT ram_adr, USHORT len0)
{
}

