/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    wince.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

ZD1211_USB_INFO g_ZD1211USBInfo;

typedef struct _REG_VALUE_DESCR {
    LPWSTR val_name;
    DWORD  val_type;
    PBYTE  val_data;
} REG_VALUE_DESCR, * PREG_VALUE_DESCR;


// Values for [HKEY_LOCAL_MACHINE\Drivers\PCMCIA\<PnpID>]
REG_VALUE_DESCR PcmKeyValues[] = {
   (TEXT("Dll")), REG_SZ, (PBYTE)(TEXT("ndis.dll")),
   (TEXT("Prefix")), REG_SZ, (PBYTE)(TEXT("NDS")),
   (TEXT("Miniport")), REG_SZ, (PBYTE)(TEXT("zd1211u")),
   (TEXT("Index")), REG_DWORD, (PBYTE)1,
    NULL, 0, NULL
};

// Values for [HKEY_LOCAL_MACHINE\Comm\zd1211u]
// and [HKEY_LOCAL_MACHINE\Comm\zd1211u1]
REG_VALUE_DESCR CommKeyValues[] = {
   (TEXT("DisplayName")), REG_SZ, (PBYTE)(TEXT("zd1211U Ethernet Driver")),
   (TEXT("Group")), REG_SZ, (PBYTE)(TEXT("NDIS")),
   (TEXT("ImagePath")), REG_SZ, (PBYTE)(TEXT("zd1211u.dll")),
    NULL, 0, NULL
};

// Values for [HKEY_LOCAL_MACHINE\Comm\zd1211u1\Parms]
REG_VALUE_DESCR ParmKeyValues[] = {
   //(TEXT("IOBaseAddress")),		REG_DWORD,	(PBYTE)0x180,
   //(TEXT("InterruptNumber")),	REG_DWORD,	(PBYTE)0x05,
   //(TEXT("OSDeterResource")),	REG_DWORD,	(PBYTE)0,
   // Although BusType = 15 is USB, but only BusType = 0 can work fine for USB-NDIS driver.
   // It is NDIS's bug.
   (TEXT("BusType")),			REG_DWORD,	(PBYTE)0,				//Bus type , 15 for USB
   (TEXT("BusNumber")),	        REG_DWORD,	(PBYTE)0,
   (TEXT("ATIMWindow")),		REG_DWORD,	(PBYTE)0,				
   (TEXT("AuthentMode")),		REG_DWORD,	(PBYTE)3,	
   (TEXT("beaconInterval")),	REG_DWORD,	(PBYTE)100,	
   (TEXT("RFChannel")),			REG_DWORD,	(PBYTE)0x6,				//channel number, 1 to 12
   (TEXT("WEPDecryptInDriver")),REG_DWORD,	(PBYTE)0,
   (TEXT("WEPEncryptInDriver")),REG_DWORD,	(PBYTE)1,
   (TEXT("FragmentThreshold")),	REG_DWORD,	(PBYTE)2305,
   //(TEXT("AntennaSelect")),		REG_DWORD,	(PBYTE)2,
   (TEXT("TxPowerControl")),	REG_DWORD,	(PBYTE)1,
   (TEXT("NetworkType")),		REG_DWORD,	(PBYTE)1,				//0 for Ad-hoc, 1 for infrastructure, 3 for psIBSS
   (TEXT("RTSCTSThreshold")),	REG_DWORD,	(PBYTE)2305,
   (TEXT("TxDataRate")),		REG_DWORD,	(PBYTE)15,
   (TEXT("PSMode")),			REG_DWORD,	(PBYTE)0,				//power saving, 0 for none. 
   //(TEXT("PSLevel")),			REG_DWORD,	(PBYTE)0,				//power save level
   (TEXT("VendorDesc")),		REG_SZ, (PBYTE)(TEXT("ZyDAS ZD1211c")),
   (TEXT("SSID")),				REG_SZ,		(PBYTE)(TEXT("any")),	//SSID, AP name
   (TEXT("WEPStatus")),			REG_DWORD,	(PBYTE)0,
   (TEXT("WEPDefaultKeyId")),	REG_DWORD,	(PBYTE)1,
   (TEXT("WEPDefaultKeys")),	REG_SZ,		(PBYTE)(TEXT("0102030405101112131410111213141011121314")),
   NULL, 0, NULL
};

// Values for [HKEY_LOCAL_MACHINE\Comm\zd1211u\Linkage]
REG_VALUE_DESCR LinkageKeyValues[] = {
   (TEXT("Route")), REG_MULTI_SZ, (PBYTE)(TEXT("zd1211u1")), //yarco
    NULL, 0, NULL
};

//[HKEY_LOCAL_MACHINE\Comm\zd1211u1\Parms\TcpIp]
//"EnableDHCP"=dword:1
REG_VALUE_DESCR TcpIpKeyValues[] = {
   (TEXT("EnableDHCP")), REG_DWORD, (PBYTE)1,
    NULL, 0, NULL
};

//[HKEY_LOCAL_MACHINE\Comm\Tcpip\Linkage]
//"Bind"=multi_sz:"ppp","ne20001","zd1211u1"
REG_VALUE_DESCR BindKeyValues[] = {
   (TEXT("Bind")), REG_MULTI_SZ, (PBYTE)(TEXT("zd1211u1")),
    NULL, 0, NULL
};

PREG_VALUE_DESCR Values[] = {
    CommKeyValues,
    CommKeyValues,
    ParmKeyValues,
    LinkageKeyValues,
	BindKeyValues
};

LPWSTR KeyNames[] = {
    (TEXT("Comm\\zd1211u")),
    (TEXT("Comm\\zd1211u1")),
    (TEXT("Comm\\zd1211u1\\Parms")),
	(TEXT("Comm\\zd1211u1\\Linkage")),
	(TEXT("Comm\\Tcpip\\Linkage"))
};

const WCHAR gcszRegisterClientDriverId[] = L"RegisterClientDriverID";
const WCHAR gcszRegisterClientSettings[] = L"RegisterClientSettings";

const WCHAR gcszUnRegisterClientDriverId[] = L"UnRegisterClientDriverID";
const WCHAR gcszUnRegisterClientSettings[] = L"UnRegisterClientSettings";

const TCHAR gcszUCMDriverId[] = L"ZD1211U_DRIVER";


//
// Add the specified key and its values to the registry under HKEY_LOCAL_MACHINE
//
// NOTE: This function only supports REG_MULTI_SZ strings with one item.
//
BOOL
AddKeyValues(
    LPWSTR KeyName,
    PREG_VALUE_DESCR Vals
    )
{
    DWORD Status;
    DWORD dwDisp;
    HKEY hKey;
    PREG_VALUE_DESCR pValue;
    DWORD ValLen;
    PBYTE pVal;
    DWORD dwVal;
    LPWSTR pStr;

    Status = RegCreateKeyEx(
                 HKEY_LOCAL_MACHINE,
                 KeyName,
                 0,
                 NULL,
                 REG_OPTION_NON_VOLATILE,
                 0,
                 NULL,
                 &hKey,
                 &dwDisp);

    if (Status != ERROR_SUCCESS) {
        return FALSE;
    }

    pValue = Vals;
    while (pValue->val_name) {
        switch (pValue->val_type) {
        case REG_DWORD:
            pVal = (PBYTE)&dwVal;
            dwVal = (DWORD)pValue->val_data;
            ValLen = sizeof(DWORD);
            break;

        case REG_SZ:
            pVal = (PBYTE)pValue->val_data;
            ValLen = (wcslen((LPWSTR)pVal) + 1)*sizeof(WCHAR);
            break;

        case REG_MULTI_SZ:
            dwVal = wcslen((LPWSTR)pValue->val_data);
            ValLen = (dwVal+2)*sizeof(WCHAR);
            pVal = LocalAlloc(LPTR, ValLen);
            if (pVal == NULL) {
                goto akv_fail;
            }
            wcscpy((LPWSTR)pVal, (LPWSTR)pValue->val_data);
            pStr = (LPWSTR)pVal + dwVal;
            pStr[1] = 0;
            break;
        }
        Status = RegSetValueEx(
                     hKey,
                     pValue->val_name,
                     0,
                     pValue->val_type,
                     pVal,
                     ValLen
                     );
        if (pValue->val_type == REG_MULTI_SZ) {
            LocalFree(pVal);
        }
akv_fail:
        if (Status != ERROR_SUCCESS) {
            RegCloseKey(hKey);
            return FALSE;
        }
        pValue++;
    }
    RegCloseKey(hKey);
    return TRUE;
}   // AddKeyValues


BOOL
DllEntry( HANDLE hDllHandle, DWORD dwReason, LPVOID lpReserved) 
{
    switch (dwReason) {
        case DLL_PROCESS_ATTACH:
            //DEBUGREGISTER((HINSTANCE)hDllHandle);
            DEBUGMSG(1, (TEXT("zd1211: DLL_PROCESS_ATTACH\n")));
            break;
        case DLL_PROCESS_DETACH:
			DEBUGMSG(1, (TEXT("zd1211: DLL_PROCESS_DETACH\n")));
        default:
            break;
    }
    return TRUE ;
}

void SetupUCMDriverSettings(USB_DRIVER_SETTINGS *pDriverSettings)
{
    pDriverSettings->dwCount = sizeof(USB_DRIVER_SETTINGS);
    //pDriverSettings->dwVendorId          = 0x07b8;
    //pDriverSettings->dwProductId         = 0x6001;
    pDriverSettings->dwVendorId          = 0x0ACE;
    pDriverSettings->dwProductId         = 0x1211;
    //pDriverSettings->dwVendorId          = 0x07B8;
    //pDriverSettings->dwProductId         = 0x6001;
    pDriverSettings->dwReleaseNumber     = USB_NO_INFO;

    pDriverSettings->dwDeviceClass       = USB_DEVICE_CLASS_VENDOR_SPECIFIC;
    pDriverSettings->dwDeviceSubClass    = USB_NO_INFO;
    pDriverSettings->dwDeviceProtocol    = USB_NO_INFO;

    pDriverSettings->dwInterfaceClass    = USB_NO_INFO;
    pDriverSettings->dwInterfaceSubClass = USB_NO_INFO;
    pDriverSettings->dwInterfaceProtocol = USB_NO_INFO;
}

/*
 * USBInstallDriver
 *
 *   USB driver install routine - set up client registry settings.
 *
 * Return value:
 *   Return TRUE if install succeeds, or FALSE if there is some error.
 */
BOOL USBInstallDriver(
    LPCWSTR szDriverLibFile)  // Contains client driver DLL name
{
    	BOOL fRet = FALSE;
    	TCHAR tkey[100];
		UINT    i;
    	HINSTANCE hInst = LoadLibrary(L"USBD.DLL");  
    
    	RETAILMSG(1,(TEXT("zd1211u: Install function called (driver: %s)\r\n"),szDriverLibFile));
    	// This is the install routine for the HID sample driver.
    	if(hInst)
    	{
	        LPREGISTER_CLIENT_DRIVER_ID pRegisterId = 
        	(LPREGISTER_CLIENT_DRIVER_ID) GetProcAddress(hInst, gcszRegisterClientDriverId);

	        LPREGISTER_CLIENT_SETTINGS pRegisterSettings =
	        (LPREGISTER_CLIENT_SETTINGS) GetProcAddress(hInst,gcszRegisterClientSettings);
	
        	if (pRegisterId && pRegisterSettings) 
        	{
	            	USB_DRIVER_SETTINGS DriverSettings;

            		SetupUCMDriverSettings(&DriverSettings);

            		fRet = (*pRegisterId)(gcszUCMDriverId);

	            	if(fRet)
            		{
                		fRet = (*pRegisterSettings)(szDriverLibFile,gcszUCMDriverId, NULL, &DriverSettings);

                		if(!fRet)
                		{
                    			LPUN_REGISTER_CLIENT_DRIVER_ID pUnregisterId =
                        			(LPUN_REGISTER_CLIENT_DRIVER_ID) GetProcAddress(hInst,gcszUnRegisterClientDriverId);
                    			if (pUnregisterId) 
                        			(*pUnregisterId)(gcszUCMDriverId);
                		}                
            		}
        	}
        	else 
        	{
            		RETAILMSG(1,(TEXT("!ZD1211u: Error getting USBD function pointers\r\n")));
        	}
        	FreeLibrary(hInst);
    	}
    

    wsprintf(tkey,TEXT("Drivers\\USB\\LoadClients\\Default\\FF\\Default\\%s"),gcszUCMDriverId);
 
	if (!AddKeyValues(tkey, PcmKeyValues)) {
        goto wid_fail;
    }

    for (i = 0; i < (sizeof(KeyNames)/sizeof(LPWSTR)); i++) {
        if (!AddKeyValues(KeyNames[i], Values[i])) {
            goto wid_fail;
        }
    }



    return TRUE;

wid_fail:
    //
    // Clean up after ourself on failure.
    //
    RegDeleteKey(HKEY_LOCAL_MACHINE,tkey);
    for (i = 0; i < (sizeof(KeyNames)/sizeof(LPWSTR)); i++) {
        RegDeleteKey(HKEY_LOCAL_MACHINE,KeyNames[i]);
    }
		

    	return FALSE;    
}

/*
 * USBUnInstallDriver
 *
 *   This function can be called by a client driver to deregister itself
 *   with USBD.
 */
BOOL 
USBUnInstallDriver()
{
    return TRUE;
}

extern PZD1205_ADAPTER		ZD1205Adapter;

BOOL 
USB_DeviceNotify(LPVOID lpvNotifyParameter, DWORD dwCode,
        LPDWORD * dwInfo1, LPDWORD * dwInfo2, LPDWORD * dwInfo3,
        LPDWORD * dwInfo4)
{
    NDIS_STATUS   status;
	UINT count;

    DEBUGFUNC(_T("USB_DeviceNotify"));

    INITSTR((_T("\n")));

    DbgPrint("USB_DeviceNotify\r\n");
	TerminateThreads(ZD1205Adapter);


    //ZD1205Adapter->USBInfo.lpUsbFuncs->lpClosePipe(ZD1205Adapter->USBInfo.hPipe[0]);
    //ZD1205Adapter->USBInfo.lpUsbFuncs->lpClosePipe(ZD1205Adapter->USBInfo.hPipe[1]);
    //ZD1205Adapter->USBInfo.lpUsbFuncs->lpClosePipe(ZD1205Adapter->USBInfo.hPipe[2]);
    //ZD1205Adapter->USBInfo.lpUsbFuncs->lpClosePipe(ZD1205Adapter->USBInfo.hPipe[3]);    

    NdisDeregisterAdapter(&status, _T("zd1211u1"));
	
	DbgPrint("status = %x\r\n", status);

	return TRUE;
}

/*
 *  USBDeviceAttach 
 * 
 *    USB device attach routine.  This function is called by USBD when a device is attached
 *    to the USB, and a matching registry key is found off the  LoadClients registry key. 
 *    We must determine whether the device may be controlled by this driver, and load 
 *    drivers for any uncontrolled interfaces.
 *
 *  Return Value:
 *    Return TRUE upon success, or FALSE if an error occurs.
 */
BOOL 
USBDeviceAttach(
    USB_HANDLE hDevice,           // USB device handle
    LPCUSB_FUNCS lpUsbFuncs,      // Pointer to USBDI function table.
    LPCUSB_INTERFACE lpInterface, // If client is being loaded as an interface driver, contains a pointer to the USB_INTERFACE
                                  // structure that contains interface information. If client is not loaded for a specific interface,
                                  // this parameter will be NULL.
    LPCWSTR szUniqueDriverId,     // Contains client driver id string. USB_MODEM_DRIVER
    LPBOOL fAcceptControl,        // Filled in with TRUE if we accept control of the device, 
                                  // or FALSE if USBD should continue to try to load client drivers.
    LPCUSB_DRIVER_SETTINGS lpDriverSettings,// Contains pointer to USB_DRIVER_SETTINGS struct that indicates how we were loaded.

    DWORD dwUnused)               // Reserved for use with future versions of USBD
{	
    	LPCUSB_DEVICE lpDeviceInfo = NULL;
    	//LPCUSB_INTERFACE lpIF;
    	//DWORD  dwIndex;    
    	//DWORD  dwIndex1;    
    	USB_PIPE pInPipe=NULL;
    	USB_PIPE pOutPipe=NULL;
    	USB_PIPE pCtrlPipe=NULL;
    	USB_PIPE pIntPipe=NULL;    
		NDIS_STATUS status;
		TCHAR	KeyPath[]=TEXT("Drivers\\USB\\ClientDrivers\\ZD1211U_DRIVER\\ZD1211U");
		    		
		*fAcceptControl = FALSE;
	
        AddKeyValues(KeyPath, PcmKeyValues);

    	//Show the device message
    	if (!lpDeviceInfo)
    	{
	    	lpDeviceInfo = lpUsbFuncs->lpGetDeviceInfo(hDevice);
    	}

        RETAILMSG(1, (TEXT("zd1211u: Vender id = %x\r\n"),lpDeviceInfo->Descriptor.idVendor));
        RETAILMSG(1, (TEXT("zd1211u: Product id = %x\r\n"),lpDeviceInfo->Descriptor.idProduct));

        //RETAILMSG(1, (TEXT("bcdDevice = %x\r\n"),lpDeviceInfo->Descriptor.bcdDevice));
/*		    
    	if (lpDeviceInfo->lpActiveConfig->dwNumInterfaces == 0 )
    		return *fAcceptControl;
    
      
    	for (dwIndex =0; dwIndex <lpDeviceInfo->lpActiveConfig->dwNumInterfaces; dwIndex++)
    	{
	    	RETAILMSG(1,(TEXT("ZD1211U: Interface[%d] has %d endpoints!!\r\n"),dwIndex, lpDeviceInfo->lpActiveConfig->lpInterfaces[dwIndex].Descriptor.bNumEndpoints));    	
    		lpIF = &lpDeviceInfo->lpActiveConfig->lpInterfaces[dwIndex];
    		for (dwIndex1 =0; dwIndex1 <lpDeviceInfo->lpActiveConfig->lpInterfaces[dwIndex].Descriptor.bNumEndpoints; dwIndex1++)
    		{
	    		
	        	if ((lpIF->lpEndpoints[dwIndex1].Descriptor.bmAttributes & USB_ENDPOINT_TYPE_MASK)
        	            ==USB_ENDPOINT_TYPE_INTERRUPT) 
        		{
    	  			if (pIntPipe==NULL)         
            			{
	            			g_ZD1211USBInfo.lpCtrlIF = lpIF;            			
        	        		pIntPipe=(*lpUsbFuncs->lpOpenPipe)(hDevice,&lpIF->lpEndpoints[dwIndex1].Descriptor);
                			RETAILMSG(1,(TEXT("EP(%d), Interrupt Pipe initial ok!!\r\n"),dwIndex1));
                			continue;
            			}
	        	}
        	
    			if (USB_ENDPOINT_DIRECTION_IN(lpIF->lpEndpoints[dwIndex1].Descriptor.bEndpointAddress))
    			{
    				if (pInPipe==NULL && 
	                    		(lpIF->lpEndpoints[dwIndex1].Descriptor.bmAttributes & USB_ENDPOINT_TYPE_MASK)
        	            		==USB_ENDPOINT_TYPE_BULK) 
                		{                		
                			g_ZD1211USBInfo.lpDataIF = lpIF;		//PUCM->lpIF define the data interface
                			//PUCM->In_EndptIdx = dwIndex1;
           	        		pInPipe=(*lpUsbFuncs->lpOpenPipe)(hDevice,&lpIF->lpEndpoints[dwIndex1].Descriptor);
	                		RETAILMSG(1,(TEXT("EP(%d),Input Pipe initial ok!!\r\n"),dwIndex1));
	                		continue;
                		}
    			}		
    	
	        	if (USB_ENDPOINT_DIRECTION_OUT(lpIF->lpEndpoints[dwIndex1].Descriptor.bEndpointAddress))
	        	{
        			if (pOutPipe==NULL &&
	                    		(lpIF->lpEndpoints[dwIndex1].Descriptor.bmAttributes & USB_ENDPOINT_TYPE_MASK)
        	            		==USB_ENDPOINT_TYPE_BULK) 
            			{
	            			g_ZD1211USBInfo.lpDataIF = lpIF;
	            			//PUCM->Out_EndptIdx = dwIndex1;
        	        		pOutPipe=(*lpUsbFuncs->lpOpenPipe)(hDevice,&lpIF->lpEndpoints[dwIndex1].Descriptor);
                			RETAILMSG(1,(TEXT("EP(%d), Output Pipe initial ok!!\r\n"),dwIndex1));                		
                			continue;
            			}
        		}
    		}
    	}
    	
		if ( pOutPipe==NULL || pIntPipe==NULL)
	 		return *fAcceptControl;
*/        
        g_ZD1211USBInfo.hDevice = hDevice;
		g_ZD1211USBInfo.lpUsbFuncs = lpUsbFuncs;
        g_ZD1211USBInfo.lpDriverSettings = lpDriverSettings;
		g_ZD1211USBInfo.lpActiveConfig = lpDeviceInfo->lpActiveConfig;
        g_ZD1211USBInfo.lpDeviceInfo = lpDeviceInfo;

		
		NdisRegisterAdapter(&status, _T("zd1211u"), _T("zd1211u1"));
		
    	if ( status != NDIS_STATUS_SUCCESS ) 
    	{
       		RETAILMSG(1,(TEXT("zd1211u: calling NdisRegisterAdapter failed\r\n")));
       		return (FALSE);
    	}

    	lpUsbFuncs->lpSetInterface(hDevice, 
					NULL, 
					NULL, 
					USB_NO_WAIT , 
					0, 
					0 );

    	// Register notification routine for device status changes.
    	(*lpUsbFuncs->lpRegisterNotificationRoutine)(hDevice,USB_DeviceNotify, 0);

    	// Tell USBD to stop looking for drivers
    	*fAcceptControl = TRUE;      

    	return *fAcceptControl;    
}
