/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    zd1205sw.h

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/



#ifndef _ZD1205SW_
#define _ZD1205SW_

#ifndef CE_ALL

typedef KIRQL ZD1211_IRQL;
typedef PKIRQL PZD1211_IRQL;
typedef KSPIN_LOCK ZD1211_SPIN_LOCK;
typedef PKSPIN_LOCK PZD1211_SPIN_LOCK;

typedef KEVENT ZD1211_EVENT;

#else // CE_ALL

typedef ULONG ZD1211_IRQL;
typedef PULONG PZD1211_IRQL;
typedef NDIS_SPIN_LOCK ZD1211_SPIN_LOCK;
typedef PNDIS_SPIN_LOCK PZD1211_SPIN_LOCK;
//typedef HANDLE ZD1211_SPIN_LOCK;
//typedef PHANDLE PZD1211_SPIN_LOCK;

typedef HANDLE ZD1211_EVENT;

#endif // CE_ALL

#define fDRV_UPDATE_EEP         (0 && fWRITE_WORD_REG)
#define fDRV_WRITE_RF_REG       (1 && fWRITE_WORD_REG)

#include "tkipseed.h" //WPA support
#include "michael.h"

#define ZD1205_OFFSET(field) ( (UINT) FIELD_OFFSET(ZD1205_ADAPTER,field) )
#define ZD1205_SIZE(field)    sizeof( ((ZD1205_ADAPTER *)0)->field )


#ifdef NDIS50_MINIPORT
#define ZD1205_NDIS_MAJOR_VERSION        0x5
#define ZD1205_NDIS_MINOR_VERSION        0x0
#endif

#ifdef NDIS51_MINIPORT
#define ZD1205_NDIS_MAJOR_VERSION        0x5
#define ZD1205_NDIS_MINOR_VERSION        0x1
#endif


#define ZD1205_DRIVER_VERSION ((ZD1205_NDIS_MAJOR_VERSION*0x100) + ZD1205_NDIS_MINOR_VERSION)

// NDIS BusType values
#define     ISABUS           1
#define     EISABUS          2
#define     PCIBUS           5


//- Driver defaults
#define     LOOKAHEAD_SIZE      222

//================================================
// WinCE USB
//================================================
typedef struct _ZD1211_USB_INFO
{
	USB_HANDLE 		hDevice;          	//Store the USB device handle	
	LPCUSB_FUNCS 		lpUsbFuncs;		//Store the pointer to USBDI function table
	LPCUSB_INTERFACE 	lpCtrlIF;		//Store the control interface which has the control and interrupt Endpoint
	LPCUSB_DRIVER_SETTINGS 	lpDriverSettings;	//Store the pointer to USB_DRIVER_SETTINGS struct that indicates how we were loaded
	LPCUSB_CONFIGURATION 	lpActiveConfig;		//Store the Active configuration
	LPCUSB_DEVICE 		lpDeviceInfo;		//Store the Device Information
	LPCUSB_INTERFACE 	lpDataIF;		//Store the data interface which has the Input and Output Endpoint
	USB_PIPE 		hPipe[4];
	USB_TRANSFER    hIntEP;
    USB_TRANSFER    hRxEP;
	USB_TRANSFER    hTxEP;
	USB_TRANSFER    hRegEP;
	USHORT          MaxPacketSize;

} ZD1211_USB_INFO, *PZD1211_USB_INFO;

typedef struct _ZD1211_TX_TRANSFER
{
    USB_TRANSFER   handle;
	UCHAR          status;

} ZD1211_TX_TRANSFER, *PZD1211_TX_TRANSFER;


extern ZD1211_USB_INFO g_ZD1211USBInfo;

//================================================

//-------------------------------------------------------------------------
//- Macros to read and write from IO ports
//-------------------------------------------------------------------------
#define ZD1205_READ_UCHAR(_port, _pValue) \
    NdisRawReadPortUchar( \
    (ULONG)(_port), \
    (PUCHAR)(_pValue) \
)

#define ZD1205_WRITE_UCHAR(_port, _Value) \
    NdisRawWritePortUchar( \
    (ULONG)(_port), \
    (UCHAR) (_Value) \
)

#define ZD1205_READ_USHORT(_port, _pValue) \
    NdisRawReadPortUshort( \
    (ULONG)(_port), \
    (PUSHORT)(_pValue) \
)

#define ZD1205_WRITE_USHORT(_port, _Value) \
    NdisRawWritePortUshort( \
    (ULONG)(_port), \
    (USHORT) (_Value) \
)


//- Macros peculiar to NT
//- The highest physical address that can be allocated to buffers.
#define ZD1205_ALLOC_MEM(_pbuffer, _length) NdisAllocateMemoryWithTag( \
    (PVOID*)(_pbuffer), \
    (_length), \
    '001E')

#define NdisAllocateMemory(_pbuffer, _length, _X, _Y) NdisAllocateMemoryWithTag( \
    (PVOID*)(_pbuffer), \
    (_length), \
    '001E')

#define ZD1205_FREE_MEM(_buffer,_length) NdisFreeMemory((_buffer), (_length), 0)

//- Size of the Ethernet Header
#define ENET_HEADER_SIZE       sizeof(ETH_HEADER_STRUC)

#define MAX_PCI_CARDS 12

// WPA Support

#define NEXT_802_11_VAR_IE(pVarIE) \
	{	\
		PUCHAR	puChar=(PUCHAR)pVarIE;	\
		puChar += ((sizeof(NDIS_802_11_VARIABLE_IEs)-1) + pVarIE->Length); \
		pVarIE = (PNDIS_802_11_VARIABLE_IEs) puChar; \
	}
#define NEXT_802_11_BSSID(pCh, pCurrentPtr) \
		pCh += pCurrentPtr->Length; \
		pCurrentPtr = (PNDIS_WLAN_BSSID_EX) pCh;
#define FILL_802_11_VAR_IE(IEPtr, EID, SrcData, DataLen) \
		IEPtr->ElementID=EID; \
		IEPtr->Length=DataLen; \
		NdisMoveMemory(	\
			IEPtr->data,	\
			SrcData,	\
			IEPtr->Length);
#define VAR_IE_SIZE(p80211VarIe) \
		((sizeof(NDIS_802_11_VARIABLE_IEs)-sizeof(UCHAR))+ \
		p80211VarIe->Length)

//for counter measure
#define ntohs(s) (((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))
#define htons(s) (((s & 0x00FF) << 8) | ((s & 0xFF00) >> 8))

/*==================================================================*/
//JWEI 2004/04/23
#define mNtoHS(s)   (((((s) & 0xFF) << 8) & 0xFF00) | (((s) >> 8) & 0xFF))

//typedef unsigned char   BYTE8;
//typedef unsigned short  WORD16;
//typedef unsigned long   DWORD32;
#define BYTE8   UCHAR
#define WORD16  USHORT
#define DWORD32 ULONG

//#define cPROTOCOL_IP            0x0008
#define cPROTOCOL_IP            mNtoHS(0x0800)
#define cPROTOCOL_ARP           mNtoHS(0x0806)
#define cMAC_ADDR_LEN           6
#define cNET_ADDR_LEN           4

typedef struct _ETHR_HEADER
{
    BYTE8   destAddr[cMAC_ADDR_LEN];
    BYTE8   srcAddr[cMAC_ADDR_LEN];
    WORD16  protocols;
    BYTE8   payload;
} ETHR_HEADER, *pETHR_HEADER;

#define cPROTOCOL_TCP           6
#define cPROTOCOL_ICMP          1

typedef struct _IP_HEADER
{
    BYTE8   ver_headerLen;
    BYTE8   serviceType;
    WORD16  totalLen;
    WORD16  identifier;
    BYTE8   fragFlag;
    BYTE8   fragOfst;
    BYTE8   timeToLive;
    BYTE8   protocolIp;
    WORD16  headerChecksum;
    BYTE8   srcIpAddr[cNET_ADDR_LEN];
    BYTE8   destIpAddr[cNET_ADDR_LEN];
    BYTE8   payload;
} IP_HEADER, *pIP_HEADER;

#define cTYPE_REQUEST           8
#define cTYPE_REPLY             0

typedef struct _ICMP_HEADER
{
    BYTE8   icmp_type;
    BYTE8   code;
    WORD16  checksum;
    WORD16  identifier;
    WORD16  seqNum;
    BYTE8   payload;
} ICMP_HEADER, *pICMP_HEADER;

#define cPORT_TELNET            mNtoHS(23)
#define cPORT_FTP_DATA          mNtoHS(20)
#define cPORT_FTP_CTL           mNtoHS(21)

typedef struct _TCP_HEADER
{
    WORD16  srcPort;
    WORD16  dstPort;
    DWORD32 seqNum;
    DWORD32 ackNum;
    BYTE8   offset_reserved;
    BYTE8   flags;
    WORD16  window;
    WORD16  checksum;
    WORD16  urgentPointer;
    BYTE8   payload;
} TCP_HEADER, *pTCP_HEADER;

/*==================================================================*/
//JWEI 2004/03/19
#define mFILL_WRITE_REGISTER(addr0, value0) \
{                                           \
    WriteAddr[WriteIndex] = (USHORT) addr0;          \
    WriteData[WriteIndex ++] = (USHORT) value0;      \
}

// write 32_bit register ==> write high word first
#define mFILL_WRITE_REGISTER32(addr0, value32)                  \
{                                                               \
    WriteAddr[WriteIndex] = (addr0) + 2;                        \
    WriteData[WriteIndex ++] = (USHORT) ((value32) >> 16);      \
    WriteAddr[WriteIndex] = addr0;                              \
    WriteData[WriteIndex ++] = (USHORT) ((value32) & 0xFFFF);   \
}

#define mFILL_READ_REGISTER(addr0)  (ReadAddr[ReadIndex ++] = addr0)

#define mFILL_RF_REGISTER(value0)   (WriteData[WriteIndex ++] = value0)

/* Flash interface */
#define bmFLASH_A0                  1

#define mFLASH_SET_EVEN_ADDR(orgCR203)                              \
    mFILL_WRITE_REGISTER(CR203 + (USHORT) (Adapter->USBCSRAddress), \
                         mCLR_BIT((USHORT) (orgCR203), bmFLASH_A0))

#define mFLASH_SET_ODD_ADDR(orgCR203)                               \
    mFILL_WRITE_REGISTER(CR203 + (USHORT) (Adapter->USBCSRAddress), \
                         mSET_BIT((USHORT) (orgCR203), bmFLASH_A0))

#define mFLASH_WRITE_EVEN_ADDR(addr0, value0, orgCR203) \
{                                                       \
    mFLASH_SET_EVEN_ADDR(orgCR203);                     \
    mFILL_WRITE_REGISTER(((addr0) >> 1), value0);       \
}

#define mFLASH_WRITE_ODD_ADDR(addr0, value0, orgCR203)  \
{                                                       \
    mFLASH_SET_ODD_ADDR(orgCR203);                      \
    mFILL_WRITE_REGISTER(((addr0) >> 1), value0);       \
}

//-------------------------------------------------------------------------
    #if !fDRV_UPDATE_EEP
    //    #error 3
#define WRITE_WORD_TO_EEPROM_PER_TIME   8
    #else
    //    #error 4
//#define WRITE_WORD_TO_EEPROM_PER_TIME   cMAX_MULTI_WRITE_REG_NUM
//#define WRITE_WORD_TO_EEPROM_PER_TIME   24
#define WRITE_WORD_TO_EEPROM_PER_TIME   16
    #endif

typedef struct _USB_EEPROM_DATA {

    USHORT  RequestID;
    USHORT  Data[WRITE_WORD_TO_EEPROM_PER_TIME];

} USB_EEPROM_DATA, * PUSB_EEPROM_DATA;
#if (defined(USE_EP1_SET_REG) || defined(USE_EP4_SET_REG))

typedef struct _USB_SET_RF
{
    USHORT  RequestID;
    USHORT  Value;
    USHORT  Index;
    USHORT  Data[cMAX_MULTI_RF_REG_NUM];
} USB_SET_RF, * PUSB_SET_RF;

#endif

//-------------------------------------------------------------------------
// PCI Cards found - returns hardware info after scanning for devices
//-------------------------------------------------------------------------
typedef struct _PCI_CARDS_FOUND_STRUC
{
    USHORT NumFound;
    struct
    {
        ULONG           BaseIo;
        UCHAR           ChipRevision;
        ULONG           SubVendor_DeviceID;
        USHORT          SlotNumber;     // Ndis Slot number
        ULONG           MemPhysAddress; // CSR Physical address
        UCHAR           Irq;
        UCHAR           NodeAddress[ETH_LENGTH_OF_ADDRESS];
    } PciSlotInfo[MAX_PCI_CARDS];

} PCI_CARDS_FOUND_STRUC, *PPCI_CARDS_FOUND_STRUC;


//-------------------------------------------------------------------------
// ZD1205_LIST_ENTRY
//-------------------------------------------------------------------------
typedef struct _ZD1205_LIST_ENTRY {

    LIST_ENTRY  Link;
	ULONG		Count;

} ZD1205_LIST_ENTRY, *PZD1205_LIST_ENTRY;



typedef struct _ZD1205_DEFRAG_ENTITY {
	ZD1205_LIST_ENTRY	Link;
	ZD1205_LIST_ENTRY	ElementList;
	USHORT				BodyLen;

} ZD1205_DEFRAG_ENTITY, *PZD1205_DEFRAG_ENTITY;

//-------------------------------------------------------------------------
// COALESCE -- This structure describes a coalesce buffer resource
//-------------------------------------------------------------------------
typedef struct _COALESCE {

    ZD1205_LIST_ENTRY     Link;
    PVOID               OwningTcb;
	PHWCOALESCE			HwCoalesce;
	ULONG				HwCoalescePhys;
#ifdef HOST_IF_USB
	ULONG				CoalesceRemoveLen;
#endif

} COALESCE, *PCOALESCE;

typedef struct _TX_MGT_ENTITY{
    ZD1205_LIST_ENTRY     Link;
	UCHAR				Content[MGT_CONTENT_SIZE];
} TX_MGT_ENTITY, *PTX_MGT_ENTITY;

//-------------------------------------------------------------------------
// ZD1205SwTcb -- Software Transmit Control Block.  This structure contains
// all of the variables that are related to a specific Transmit Control
// block (TCB)
//-------------------------------------------------------------------------

typedef struct _ZD1205SwTcb {

    // Link to the next SwTcb in the list
    ZD1205_LIST_ENTRY     Link;

    // The NDIS packet that this TCB is sending
    PNDIS_PACKET        Packet;

    // physical and virtual pointers to the hardware TCB
    PHWTCB_STRUC         Tcb;
    ULONG               TcbPhys;

    // virtual pointer to the previous hardware TCB in the chain
    PHWTCB_STRUC         PreviousTcb;

    // Physical and virtual pointers to the TBD array for this TCB
    PTBD_STRUC          FirstTbd;
    ULONG               FirstTbdPhys;

    // number of map registers used by this TCB
    UINT                MapsUsed;

    // number of TBDs used by this TCB
    UINT                TbdsUsed;

    // When a virtual buffer is decomposed into it's constituent
    // physical vectors, each vector is stored here.
    UINT                NumPhysDesc;
    NDIS_PHYSICAL_ADDRESS_UNIT PhysDesc[MAX_PHYS_DESC];

    // If there are more physical segments then map registers,
    // then use a coalesce buffer.
    PCOALESCE           Coalesce;
    UINT                CoalesceBufferLen;

    // Describes the length of the packet as sent by the protocol.
    UINT                PacketLength;

    // Pointer to the first buffer in the packet
    PNDIS_BUFFER        FirstBuffer;

    // The number of buffers that the packet has
    UINT                BufferCount;
	UCHAR				FrameType;
	PCTRL_STRUC			HwCtrlPtr;
	ULONG				HwCtrlPhys;

	PHEADER_STRUC		HwHeaderPtr;
	ULONG				HwHeaderPhys;

	UCHAR				LastFrag;
	UCHAR				Provider;

	ULONG				TcbCount;

	USHORT				HangDur;

	//WPA support
	PMIC_STRUC			HwMicPtr;
	ULONG				HwMicPhys;

	PVOID				ReturnPara;
	PVOID				STAContextPtr;
#ifdef HOST_IF_USB

	BOOLEAN				bHasCompleteBeforeSend;
	BOOLEAN				bHasBeenDelayedBefore;

#endif


} ZD1205SwTcb, *PZD1205SwTcb;


//-------------------------------------------------------------------------
// ZD1205SwRfd
//
//  In addition to the Receive Entry fields which the ZD1205 defines, we need
//  some additional fields for our own purposes.  To ensure that these
//  fields are properly aligned (and to ensure that the actual Receive
//  Entry is properly aligned) we'll defined a Super Receive Entry.
//  This structure will contain a "normal" Hardware Receive Entry (RFD) plus
//  some additional fields.
//-------------------------------------------------------------------------
typedef struct _ZD1205SwRfd {

    ZD1205_LIST_ENTRY     Link;
#if DBG
    UINT RfdNum;
#endif

    //
    // References an active chain of buffers when
    // on the receive indication list.
    //
    volatile PRFD_STRUC Rfd;             // ptr to hardware RFD
    ULONG               RfdPhys;         // physical address of RFD
    PNDIS_BUFFER        ReceiveBuffer;   // Pointer to Buffer
    PNDIS_PACKET        ReceivePacket;   // Pointer to Packet
    USHORT              Status;          // receive status (quick retrieval)
    UINT                FrameLength;     // total size of receive frame

#ifdef HOST_IF_USB
    UINT                UnFinishFrmLen;
#endif
} ZD1205SwRfd, *PZD1205SwRfd;

// supporting piece of ReceiveMemoryDescriptor
typedef struct _CMD {
    ULONG   VirtualAddress;
    ULONG   Size;
} CACHED_MEM_DESCRIPTOR, *PCACHED_MEM_DESCRIPTOR;

// supporting piece of ReceiveMemoryDescriptor
typedef struct _UCMD {
    ULONG   VirtualAddress;
    NDIS_PHYSICAL_ADDRESS PhysicalAddress;
    ULONG   Size;
} UNCACHED_MEM_DESCRIPTOR, *PUNCACHED_MEM_DESCRIPTOR;

//-------------------------------------------------------------------------
// ReceiveMemoryDescriptor
//
// In order to manage receives in an environment where our receive
// buffer space might be growing, this structure was created to have
// a set of pointers that for each growth instance would allow us
// to monitor and free that growth area if need be.
// this structure is all just memory pointers and counters.
//-------------------------------------------------------------------------
typedef struct _RMD {
    ZD1205_LIST_ENTRY         Link;       // forward and backward links
    CACHED_MEM_DESCRIPTOR   CachedMem;
    UNCACHED_MEM_DESCRIPTOR UnCachedMem;
    UINT                    ActivePoolCount;
} ReceiveMemoryDescriptor, *PReceiveMemoryDescriptor;



//-------------------------------------------------------------------------
// This macro might be used to debug spinlock acquires and releases
// with the NdisAcquireSpinlock() and NdisReleaseSpinlock() call
// it aligns itself over 8 dwords, making reading in the debugger
// easy. Beware the NdisGetTime call sometimes returns the same
// value... when called in close proximity to each other...
// you might have a macro that replaced spinlock calls with some
// fill in of this structure and then acquire/release the spinlock
//-------------------------------------------------------------------------
#if DBG
typedef struct _ZD1205SpinDebug
{
    // 8 dwords total (hopefully) trying to debug a spin lock problem
    USHORT      action;
    USHORT      line;
    ULONG       file;
    LARGE_INTEGER    time;
    //    LONGLONG    blank;
} ZD1205SpinDebug, *PZD1205SpinDebug;
#endif

#define		NUM_SUPPORTED_RATE	32
#define		MAX_COUNTRY_INFO_SIZE		50
#define		MAX_SSID_LEN				36
#define		MAX_REPORT_IEs_SIZE			120
#define		MAX_IEs_SIZE				100
#define		MAX_CFPar_SIZE				8
typedef struct _CARD_SETTING
{
	UCHAR				PreambleType;
	UCHAR				BSSType;
	UCHAR				TxRate;			// use for old RID 
	UCHAR				LastSentTxRate;
	UCHAR               LastSentRxRate;
#ifdef OFDM
	UCHAR				HighestTxRate;
#endif
	UCHAR				Channel;
	UCHAR				EncryMode;
	UCHAR				WepIvType;
	UCHAR				EncryKeyId;
	UCHAR				AuthMode;
	// ElementID(1), Len(1), SSID
	UCHAR				Info_SSID[MAX_SSID_LEN];
	// ElementID(1), Len(1), SupportedRates(1-8)
	UCHAR				Info_SupportedRates[NUM_SUPPORTED_RATE];
	UCHAR				Info_ExtSupportedRates[NUM_SUPPORTED_RATE];
	ULONG				FragThreshold;
	ULONG				RTSThreshold;
	USHORT				BeaconInterval;
	USHORT				ATIMWindow;
	USHORT				Capability;
    UCHAR               wpaIEs[MAX_IEs_SIZE];
    BOOLEAN             WPASupport;
	USHORT              IEstart;
	USHORT              IEtotal;
	ULONG				DTIMPeriod;
	UCHAR               ERP[3];
} CARD_SETTING, *PCARD_SETTING;

typedef struct _BSS_INFO
{
	UCHAR		BSSID[6];
	UCHAR		BeaconInterval[2];
	UCHAR		Capability[2];
	UCHAR       TimeStamp[8];

	// SSID[34]
	// ElementID(1), Len(1), SSID
	UCHAR		SSID[MAX_SSID_LEN];

	// SupportedRates[10]
	// ElementID(1), Len(1), SupportedRates(1-8)
	UCHAR		SupportedRates[NUM_SUPPORTED_RATE];

	UCHAR		Channel;

	// CFPar[8]
	// ElementID(1), Len(1), CFP_Cnt(1), CFP_Period(1), CFP_MaxDur(2), CFP_DurRem(2)
	UCHAR		CFPar[MAX_CFPar_SIZE];
	UCHAR		CountryInfo[MAX_COUNTRY_INFO_SIZE];
	UCHAR		ATIMWindow[2];

#if (defined(GCCK) && defined(OFDM))
	UCHAR		ERP_Info;
	// ElementID(1), Len(1), SupportedRates(1-8)
	UCHAR		ExtSupportedRates[NUM_SUPPORTED_RATE];
#endif

	UCHAR		SignalStrength;
	UCHAR		SignalQuality;
    ULONG       IELength;
    UCHAR       wpaIEs[MAX_IEs_SIZE];
    BOOLEAN     WPASupport;
	USHORT      IEstart;
	USHORT      IEtotal;
	BOOLEAN		OnUpdating;
} BSS_INFO, *PBSS_INFO;

//--------------------------------------
// Structure for pended OID query request
//--------------------------------------
typedef struct _MP_QUERY_REQUEST
{
    IN NDIS_OID Oid;
    IN PVOID InformationBuffer;
    IN ULONG InformationBufferLength;
    OUT PULONG BytesWritten;
    OUT PULONG BytesNeeded;
} MP_QUERY_REQUEST, *PMP_QUERY_REQUEST;

//-----------------------------------------------
// Structure for Opposite-Peer Rate information
//-----------------------------------------------
#define RATEARRAY_NUM	16
typedef struct _OPPOSITE_RATE_INFO
{
	UCHAR	MacAddr[6];
	UCHAR	MaxRate;				//mean max item in RateArray
	UCHAR	CurrentRate;			//mean current item in RateArray
	USHORT	ConsecutiveSuccessFrames;
	USHORT	Capability;
	USHORT	FailFrames;
	UCHAR	RateArray[RATEARRAY_NUM];// this array is the rate adaption table
	UCHAR	RateArrayCount;
	UCHAR	SupportRateArray[RATEARRAY_NUM];
	UCHAR	SupportRateArrayCount;
	BOOLEAN bJustRiseRate;
	USHORT	RiseCondition;
	USHORT	DownCondition;
} OPPOSITE_RATE_INFO, *POPPOSITE_RATE_INFO;


#ifdef HOST_IF_USB

#define IOWAITTIME	2

#define EP_DATA_IN	1
#define EP_DATA_OUT	0
#define EP_INT_IN	2
#define EP_REG_OUT	3

// EP0 [request, ID] setting. 
#define REGID_WRITE		0x21
#define REGID_READ		0x22
#define REGID_RFOFDMSET	0x23
#define REGID_PROG_FLSH 0x24
#define REGID_CHKSUM            0x25
#define EEPROM_START	0x128
#define EEPROM_MID		0x28
#define EEPROM_END		0x228

#define FIRMWARE_DOWNLOAD	0x30
#define FIRMWARE_CONFIRM	0x31
#define FIRMWARE_READ_DATA  0x32


#define MAX_EPINT_BUFFER	64

typedef struct _USB_INT_CONTEXT {

	PVOID	Adapter;
	
} USB_INT_CONTEXT, * PUSB_INT_CONTEXT;

typedef struct _USB_RX_CONTEXT {

	PVOID	Adapter;
	
} USB_RX_CONTEXT, * PUSB_RX_CONTEXT;

typedef struct _USB_TX_CONTEXT {

	PVOID	Adapter;
	BOOLEAN	IsRegSet;
	
} USB_TX_CONTEXT, * PUSB_TX_CONTEXT;


typedef struct _OIDSET_WORKITEM_CONTEXT {

	NDIS_HANDLE	MiniportAdapterContext;
	NDIS_OID	Oid;
	PVOID		InformationBuffer;
	ULONG		InformationBufferLength;
	PULONG		BytesRead;
	PULONG		BytesNeeded;

} OIDSET_WORKITEM_CONTEXT, * POIDSET_WORKITEM_CONTEXT;

typedef struct _OIDQUERY_WORKITEM_CONTEXT {

	NDIS_HANDLE	MiniportAdapterContext;
	NDIS_OID	Oid;
	PVOID		InformationBuffer;
	ULONG		InformationBufferLength;
	PULONG		BytesWritten;
	PULONG		BytesNeeded;

} OIDQUERY_WORKITEM_CONTEXT, * POIDQUERY_WORKITEM_CONTEXT;

#endif

//////////////////////////// For software AP ////////////////////////////////
typedef struct _Element
{
	UCHAR	buf[34];
} Element;

typedef struct TrafficMap_s
{
	UCHAR	t[(MAX_AID/8)+1];
} TrafficMap_t;

typedef enum _AUTH_STS
{
	UN_AUTHENTICATED,
	WAIT_CHAL_RSP,
	AUTHENTICATED
} AUTH_STS;

typedef enum _LINK_STS
{
	DIS_CONNECTED,
	CONNECTED
} LINK_STS;

typedef	enum _PS_MODE
{
	STA_CAM,
	STA_PSM
} PS_MODE;

typedef struct _STA_CONTEXT
{
    ZD1205_LIST_ENTRY     Link;
	PVOID	        Adapter;
	UCHAR			MacAddr[6];
	UCHAR			AID;
	AUTH_STS		AuthStatus;
	LINK_STS		LinkStatus;
	PS_MODE			PSMode;
	//++ Pending Queue
    PNDIS_PACKET    FirstPendingQueue;
    PNDIS_PACKET    LastPendingQueue;
    UINT            NumPendingQueued;
	//--
    NDIS_MINIPORT_TIMER   AgeTimer;
	BOOLEAN			PolledBySTA;
	USHORT			IdleTime;
	// Rate Adaption Related
	USHORT	Capability;
	UCHAR   MixedMode;
	ULONG   UniQCount;
	//UCHAR	MaxRate;
	//UCHAR	CurrentRate;
	//UCHAR	ConsecutiveSuccessFrames;
} STA_CONTEXT, *PSTA_CONTEXT;

typedef	struct _ETHER_AUTH_FRM
{
	UCHAR		DestAddr[6];
	UCHAR		SrcAddr[6];
	UCHAR		TypeLen[2];
	UCHAR		Al_Num[2];
	UCHAR		Trans_SeqNo[2];
	UCHAR		Status[2];
	UCHAR		Challenge[130]; // the first 2 bytes are information ID, length
} ETHER_AUTH_FRM, *PETHER_AUTH_FRM;

typedef	struct _ETHER_ASSORSP_FRM
{
	UCHAR		DestAddr[6];
	UCHAR		SrcAddr[6];
	UCHAR		TypeLen[2];
	UCHAR		Capability[2];
	UCHAR		Status[2];
	UCHAR		AID[2]; 
	UCHAR		SupportedRates[1];
} ETHER_ASSORSP_FRM, *PETHER_ASSORSP_FRM;

typedef	struct _ETHER_DISASSO_FRM
{
	UCHAR		DestAddr[6];
	UCHAR		SrcAddr[6];
	UCHAR		TypeLen[2];
	UCHAR		Reason[2];
} ETHER_DISASSO_FRM, *PETHER_DISASSO_FRM;

typedef struct MACListData
{
    unsigned char Valid;
    unsigned char MAC_Addr[6];
}MAC_LIST_DATA, *PMAC_LIST_DATA;

#define trafficMap(trafficmap, aid)  (((trafficmap)->t[(aid/8)] & (1<<(7-(aid%8))) ) == 0 ? 0 : 1) 
//////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------
// ZD1205_ADAPTER
//
//  The main Adapter structure definition. This structure has all fields
//  relevant to the hardware.
//-------------------------------------------------------------------------
#ifndef CE_ALL
#define	BSS_INFO_NUM				64
#define	OPPOSITE_RATE_INFO_NUM		64
#else
#define	BSS_INFO_NUM				16
#define	OPPOSITE_RATE_INFO_NUM		 8
#endif
typedef struct _ZD1205_ADAPTER
{

    //UINT                Debug;

    // Holds the interrupt object for this adapter.
    //NDIS_MINIPORT_INTERRUPT     Interrupt;

    // setup the timer structure used for Async Resets
    NDIS_MINIPORT_TIMER        ZD1205AsyncResetTimer;
    //NDIS_MINIPORT_TIMER        LinkDetectionTimer;
    
    //for counter measure
    NDIS_MINIPORT_TIMER        OneSecTimer;
    LARGE_INTEGER               StartTime;  //JWEI 2004/04/23
    //ULONG                       timeDriverStart;

    // the pointer to the first packet we have queued in send
    // deserialized miniport support variables
    PNDIS_PACKET        FirstTxQueue;
    PNDIS_PACKET        LastTxQueue;
    UINT                NumPacketsQueued;

    // Handle given by NDIS when the Adapter registered itself.
    NDIS_HANDLE         ZD1205AdapterHandle;
	NDIS_HANDLE			ZD1205WrapperConfigurationContext;

    // Adapter Information Variable (set via Registry entries)
    //UINT                BusNumber;      //' BusNumber'
    //USHORT              BusDevice;      // PCI Bus/Device #

    //USHORT              AiBaseIo;       // Base I/O address
    //ULONG               AiBoardId;      // last 4 bytes of node address (EID), overloaded with PCI device id after init
    //UINT                AiBusType;      // 'BusType' (EISA or PCI)
    //USHORT              AiTxFifo;       // TX FIFO Threshold
    //USHORT              AiRxFifo;       // RX FIFO Threshold
    //UINT                AiInterrupt;    // 'InterruptNumber'
    //USHORT              AiTempSpeed;    // 'Speed', user over-ride of line speed
    UCHAR               AiNodeAddress[ETH_LENGTH_OF_ADDRESS];
    UCHAR               AiPermanentNodeAddress[ETH_LENGTH_OF_ADDRESS];
    //USHORT              AiSlot;         // 'Slot', PCI Slot Number
    //USHORT              AiThreshold;    // 'Threshold', Transmit Threshold
    //UCHAR               AiUnderrunRetry; // The underrun retry mechanism
    //UCHAR               AiForceDpx;     // duplex setting
    //UCHAR               AiTxDmaCount;   // Tx dma count
    //UCHAR               AiRxDmaCount;   // Rx dma count
    //UCHAR               Congest;        // Enables congestion control
    //UINT                McTimeoutFlag;  // MC workaround flag

    // Address of the phy component
    //UINT                PhyAddress;
    //UCHAR               Connector;      // 0=Auto, 1=TPE, 2=MII

    // Fields for various ZD1205 specific parameters
    UINT                NumCoalesce;    // 'NumCoalese'
    UINT                NumRfd;         // 'NumRfd'
    UINT                NumTbdPerTcb;   // 'NumTbdPerTcb'
    UINT                NumTbd;         // Total number of TBDs
    UINT                RegNumTcb;      // number of transmit control blocks the registry says
    UINT                NumTcb;         // 'NumTcb' (number of tcb we are actually using (usually RegNumTcb+1))

    // Mapped Base I/O values, used for accessing Adapter I/O
    //ULONG               MappedIoBase;
    //UINT                MappedIoRange;

    // 82557/82558 Control Structures. (note: VOLATILE memory area)
    //PUCHAR              CbUnCached;
    //UINT                CbUnCachedSize;
    //NDIS_PHYSICAL_ADDRESS   CbUnCachedPhys;

    // Receive memory parameters
    PUCHAR              RecvCached;
    UINT                RecvCachedSize;

    PUCHAR              RecvUnCached;
    UINT                RecvUnCachedSize;
    NDIS_PHYSICAL_ADDRESS      RecvUnCachedPhys;

    // support for async shared memory allocation

    ZD1205_LIST_ENTRY     RfdList;
	// Add by JSWAN -- 2002/06/12
	ZD1205_LIST_ENTRY		FreeDefragEntityList;

	ZD1205_LIST_ENTRY		DefragList;


    // Transmit memory structures.
    PUCHAR              XmitCached;
    UINT                XmitCachedSize;

    PUCHAR              XmitUnCached;
    UINT                XmitUnCachedSize;
    NDIS_PHYSICAL_ADDRESS   XmitUnCachedPhys;


    // Map into a local array to avoid overflow in the Tcb if the virtual
    // buffer is made of too many physical buffers. This is a temporary
    // scratch area.
    NDIS_PHYSICAL_ADDRESS_UNIT pUnits[MAX_PHYS_DESC];


    // Keep a list of free transmit descriptors.
    ZD1205_LIST_ENTRY     TxCBList;

    ZD1205_LIST_ENTRY     ActiveChainList;

    //ZD1205_LIST_ENTRY     CompletedChainList;

    // Keep a coalesce buffer list around for those times when there are too many
    // physical mappings.
    ZD1205_LIST_ENTRY     CoalesceBufferList;


    // The Current Global Packet Filter and look ahead size.
    ULONG               PacketFilter;
    //UINT                LookAhead;


    // Packet counts
    UINT                GoodTransmits;
    UINT                GoodReceives;

    // Count of transmit errors
    //UINT                TxAbortExcessCollisions;
    //UINT                TxLateCollisions;
    //UINT                TxDmaUnderrun;
    //UINT                TxLostCRS;
    //UINT                TxOKButDeferred;
    //UINT                OneRetry;
    //UINT                MoreThanOneRetry;

    // Count of receive errors
    //UINT                RcvCrcErrors;
    //UINT                RcvAlignmentErrors;
    //UINT                RcvResourceErrors;
    //UINT                RcvDmaOverrunErrors;
    //UINT                RcvRuntErrors;

    ZD1211_SPIN_LOCK	Lock;
	ULONG               LockNum;
	ULONG               epstate;
	ULONG               rxi;

    // Receive Routine Data Area
    NDIS_HANDLE          ReceivePacketPool;
    NDIS_HANDLE          ReceiveBufferPool;

    // some counters for our Free/Used pool
    //UINT                 UsedRfdCount;

    // place to hold the revision id of the ZD1205 chip
    //UCHAR               AiRevID;
    //USHORT              AiSubVendor;
    //USHORT              AiSubDevice;

    // save the status of the Memory Write Invalidate bit in the PCI command word
    //BOOLEAN             MWIEnable;

    HANDLE				h_ZD1205MgtThread;	//Handle of ZD1205MgtThread
	HANDLE				h_ZD1205HouseThread; //Handle of ZD1205HouseThread
	HANDLE              h_ZD1211SetQueryOIDThread; //Handle of ProcessRxThread 
	HANDLE              h_ZD1211StartRxThread;
	HANDLE              h_ZD1211InterruptThread;
	BOOLEAN				bTerminateThreads;

	// For ZD1205
	ULONG				S_bit_cnt;
	UCHAR				adapterReady;
	UCHAR			    Infra_Process_Associate;
	//UCHAR				PreState;
	//UCHAR				EnetFrame[4800];
	UCHAR				EnetFrame[256];
	BOOLEAN				EnetFrameLock;
	//ULONG				AESIV;
    ZD1211_SPIN_LOCK	MgtLock;
	ZD1211_IRQL			SavedIrql;
	ZD1205_LIST_ENTRY	MgtList;
    ZD1211_EVENT    	MgtEvent;    // Queue event object
	ZD1211_EVENT        AP_MgtEvent; 
	UCHAR				BSSID[6];
	UCHAR				BSSID_to_be_Connect[6];
	UCHAR				BSSID_to_be_Connect_tmp[6];
	UCHAR				bNeedCmpBSSID;
	USHORT				ListenInterval;
	BSS_INFO			BSSInfo[BSS_INFO_NUM];
	USHORT				BSSCnt;
	USHORT				BssIndex;
	//BOOLEAN				TestCoalesce;
	ULONG				RF_Mode;
	ULONG				PA_Type;
	ULONG				WepIv;
	//ULONG				SA2400DelayTime;
	USHORT				AID;
	ULONG				TxTotalCnt;
	//ULONG				FreeTotalCnt;
	//USHORT				WepOption;
	//BOOLEAN				ShowSeq;
	ULONG				RssiTrigLevel;
	CARD_SETTING		CardSetting;
	USHORT              ATIMWindow;
    USHORT              BeaconInterval;
	UCHAR				ChosenBSS;
	ULONG				Notification;
	//ULONG				CWmin;
	ULONG				TotalTxDataFrmBytes;
	ULONG				TotalRxDataFrmBytes;
	USHORT				TestFlag;
	UCHAR				SuggestionMode;
	UCHAR				PwrState;
	BOOLEAN				PSMSupported;
	ULONG				PSThreshhold;
	BOOLEAN				DeviceInSleep;
	BOOLEAN				AP_Alive;
	BOOLEAN				IBSSPartner_Alive;
	ULONG				LossAP;
	ULONG				LBTxCnt;
	ULONG				LBRxCnt;
	USHORT				LBFrameLen;
    USHORT              LBRxFrameLen;       //JWEI 2004/06/03
    ULONG               LBTxTestCnt;
	ULONGLONG			Timer1;
	ULONGLONG			Timer2;
	ULONG				LBErrFrame;

	ULONG               ZDConfigRunning;
	ULONG               ZDConfigTurnOffRF;
    USHORT              NotCheckConnect;
	USHORT              ConnAfterDisconn;
    //UCHAR               ZDDefaultSSID[36];
	//USHORT              ZDDefaultKeyId;
    //USHORT              ZDDefaultWEPStatus;
	//USHORT              ZDDefaultNetType;
	//USHORT              ZDDefaultRFChannel;
    //UCHAR               ZDDefaultKey[233];	//wep256 need at least 29*4*2=232
	//UCHAR				AllKeyBuf[52];
	UCHAR				AllKeyBuf[117];	// wep256 need at least 29*4=116

	OPPOSITE_RATE_INFO	OppositeRateInfo[OPPOSITE_RATE_INFO_NUM];
	
	UCHAR				AllowRateArray[RATEARRAY_NUM]; // 0~b
	UINT				AllowRateArrayCount;

	UCHAR				PernmentSupportedRates[NUM_SUPPORTED_RATE];
	UCHAR				PernmentExtSupportedRates[NUM_SUPPORTED_RATE];

	UCHAR				OppositeCnt;
	  	
	ULONG				DbgMsgFlag;

	ULONG				IntMask;


	UCHAR				RxSignalStrength;
	UCHAR				RxSignalQuality;
	UCHAR				RxSignalQualityIndB;
	UCHAR				RxSignalQuality1;
	UCHAR				RxSignalQuality2;

	BOOLEAN				DisableTx;

	UINT				CountrySelect;

    BOOLEAN             bQueryPending;
    MP_QUERY_REQUEST    QueryRequest;

	//USHORT				ATIMValue;

	UCHAR				LatestSeqCtrl[2];

	USHORT				NotifyNDISRxFrmCnt;

	//ULONG               stop;
	ULONG               MoreConnect;
	UCHAR               MulticastAddr[194]; // the first byte is the number of multicast addresses
    //USHORT              OldCTSRate;
	USHORT              CheckCTS;

	UCHAR				TxGainSetting;
#if fTX_GAIN_OFDM
    UCHAR               TxGainSetting2;     //JWEI 2003/12/23
#endif

	BOOLEAN				FrmRxed;
	BOOLEAN				FrmRxed1;
	BOOLEAN             TerminateAPMgtThread;
	//ULONG				ExceptionCnt;
	//BOOLEAN				TriggerReset;

	USHORT				SequenceNum;
	USHORT				SelfCTSMode;	//not 1 or 2:Auto 1:Force on 2:Force off
	BOOLEAN				SelfCTSOn;
	BOOLEAN             LongSlot;
	BOOLEAN				BakerPreamble;

	USHORT				func8021x;

	USHORT              RateLock;

	//ULONG				BusAbortCnt;

	//BOOLEAN				EnableInt;

	USHORT              SetPoint[14];
	USHORT              IntValue[14];
    USHORT              IntValueLevel;
	USHORT				ReducePowerLevel;
	USHORT				nPwrIntValueGuard;
    USHORT              SetPointOFDM[3][14];//JWEI 2003/12/31,For 36M,48M,54M
    USHORT              SetPointLevel;
	UCHAR               TraceSetPoint;
    BOOLEAN             EnableTxPwrCtrl;
    UCHAR               TxOFDMCnt;          //``JWEI 2003/12/22
    UCHAR               TxOFDMType;
    UCHAR               TxPwrCCK;
    UCHAR               TxPwrOFDM;          // 6M - 36M
    UCHAR               JWeiDbgCnt3;        //JWEI 2004/02/24

	BOOLEAN				DataTrafficLight;
    UCHAR               LinkTimer;          //JWEI 2004/02/09
    ULONG               LinkLEDn;
    ULONG               LinkLED_OnDur;
    ULONG               LinkLED_OffDur;
    ULONG               DataLED;
	ULONG               LED_Ctrl_Type;      //chi 2004/08/06

    UCHAR               PhyTest;            //JWEI 2004/02/13
#if fWRITE_WORD_REG || fREAD_MUL_REG
    USHORT              AddrEntryTable;     //JWEI 2004/03/18
#endif
#if fWRITE_WORD_REG
    BOOLEAN             AllowAccessRegister;
    //UCHAR               FlashType;
#endif
#ifdef fREAD_MUL_REG
    USHORT              ReadRegCount;       //JWEI 2004/03/19
#endif
    ULONG               ulGroupHash_P1;
    ULONG               ulGroupHash_P2;
    ULONG               AbNormalPOR;
    BOOLEAN             TestPS;
    ULONG               DoNotSleep;
    BOOLEAN             IBSS_Wakeup_Dest;
    BOOLEAN             SurpriseRemoved;
    USHORT              PERPort;

	BOOLEAN				bOverWritePhyRegFromE2P;
	BOOLEAN				bIsNormalSize;

	//++ Tally parameters
	ULONG               txUnicastFrm;
	ULONG               txMulticastFrm;
	ULONGLONG               txUnicastOctets;
	ULONG               txMulticastOctets;
	ULONG				RetryFailCnt;
	
	ULONG               txFrmUpperNDIS;
	ULONG               txFrmDrvMgt;

	ULONG               rxUnicastFrm;
	ULONG               rxMulticastFrm;
	ULONG               rxBroadcastFrm;
	ULONG               rxNeedFrgFrm;
	ULONGLONG           rxUnicastOctets;
	ULONG               rxMulticastOctets;
	ULONG               rxBroadcastOctets;
	ULONG				DriverDiscardedFrm;
	ULONG				DriverDiscardedFrmCauseByMulticastList;
	ULONG				DriverDiscardedFrmCauseByFrmCtrl;
	ULONG				DriverReceivedFrm;
	ULONG				LessThanDataMinLen;
	ULONG				GreaterThanMaxLen;
	ULONG				PLCP_MacLen_Mismatch;
	ULONG				DriverRxFrmCnt;
	ULONG				DriverRxMgtFrmCnt;

#if	(defined(GCCK) && defined(OFDM))
	ULONG               rx11bDataFrame;
	ULONG               rxOFDMDataFrame;
#endif

	ULONG				Hw_TotalRxFrm;
	ULONG				Hw_CRC32Cnt;
	ULONG				Hw_CRC16Cnt;
	ULONG				Hw_DecrypErr_UNI;
	ULONG				Hw_DecrypErr_Mul;
	ULONG				Hw_RxFIFOOverrun;
	ULONG				Hw_TotalTxFrm;
	ULONG				Hw_UnderrunCnt;
	ULONG				Hw_RetryCnt;
	//--

	USHORT				OperationRegionCode;
	USHORT				MultiDomainCapability;

	USHORT				ScanTime;
	UCHAR               encryMode;
	//ULONG				CR32Value;
	//ULONG				PhyRegErr;
	UCHAR               WPAOn;
	//USHORT				MaxScanLen;
	//USHORT				MinScanLen;
	//SHORT				LastScanFrmLen;
	//USHORT				ScanErrFrm;
	//USHORT				ScanGoodFrm;
	//USHORT				ScanLostFrm;
	//ULONG				ScanDelay;


	//WPA support
	ULONG				keyMgrState;
	Seedvar				TxSeed;
	Seedvar				RxSeed[4];
	MICvar				TxMicKey;
	MICvar				RxMicKey[4];
	UCHAR				groupKeyIndex;
	ULONG				wepStatus;
	USHORT				iv16;
	ULONG				iv32;
	ULONG               WepStatus;//If KeyID 0 set, set Bit0, If KeyID 1 set, set Bit1, etc
	UCHAR               wepKeyLen;
	UCHAR				PkInstalled;
	UCHAR				GkInstalled;
	//wpa2
	UCHAR				FakeMic;
	UCHAR               notCheckSN;

	//for counter measure 
	ULONG				MICFailureCount;
	ULONG				MICFailureTimer;
	USHORT              DisconnectTimer;
	ULONG				DisallowAssociateTimer;
	USHORT				AttackedSsidLen;
	UCHAR				AttackedSsid[33];

	BOOLEAN				bUseSignalQuality4AntDiv;
	BOOLEAN				bTxFoloRx;
	//UCHAR				AntDivState;
	//ULONG				RssiListDuringMT2[256];
	//UCHAR				RssiListIndex;
	ULONG				MinSamplesInMT2;
	ULONG				TimeElapsedInMT2;
	ULONG				CurrentAntenna;	
	BOOLEAN				EnableSwAntennaDiv;
	UCHAR				Ant_MonitorDur1;
	UCHAR				Ant_MonitorDur2;
	ULONG				Acc_Num;
	ULONG				Acc_SQ;
	UCHAR				NiceSQThr;
    ULONG               Acc_Num_OFDM;
    ULONG               Acc_SQ_OFDM;
    UCHAR               NiceSQThr_OFDM;
	ULONG				Bcn_Acc_Num;
	ULONG				Bcn_Acc_SQ;
	UCHAR				WorseSQThr;

	ULONG				AllowedChannel;
	BOOLEAN				b802_11D;
	//BOOLEAN				bSupport_27_5_Mbps;
	//ULONG				FeatureBitMap;

	//BOOLEAN				FilterSigSQ;
	//UCHAR				SigSQ_MAC_Addr[6];
	//ULONG				HMAC_TxRx_DeadLock;
	//ULONG				HMAC_NoPhyClk;
	//ULONG				HMAC_SuspiciousTKIPHang;
	UCHAR               vendorDesc[100];
	USHORT              vendorDesclen;
	UCHAR               preambleMode;
    USHORT              ReasonCode;
	UCHAR				NoBcnDetectedCnt;
	UCHAR				NoIBSSPartnerDetectedCnt;
	ULONG				CheckForHangLoop;

	//BOOLEAN				ProtectRegs;
	//UCHAR				sw_CR10;
	//UCHAR				sw_CR17;
	//UCHAR				sw_CR19;
	//UCHAR				sw_CR23;
	//UCHAR				sw_CR24;
	//UCHAR				sw_CR27;
	//UCHAR				sw_CR33;
	//UCHAR				sw_CR34;
	//UCHAR				sw_CR35;
	//UCHAR				sw_CR113;
	//UCHAR				sw_CR119;
	//UCHAR				sw_CR125;
	//UCHAR				sw_CR127;

	BOOLEAN             ProbeReqWithSSID;
	ULONG				RoamingCnt;
	ULONG				RxDeAuthCnt;
	ULONG				RxDisAssoCnt;
	
	// For Software AP
	//BOOLEAN				mcBuffered;
	UCHAR				mDtimCount;
	UCHAR				atimBitMap[(MAX_AID/8)+2];
	TX_MGT_ENTITY		TxMgtEntity[TX_MGT_NUM];
    ZD1205_LIST_ENTRY   MgtFreeList;

	ULONG				AID_RollTable;		// BitMap
	//STA_CONTEXT			STA_Context[MAX_AID];
    STA_CONTEXT			STA_Context[1];
	ZD1205_LIST_ENTRY	STAFreeList;
	ZD1205_LIST_ENTRY	STAContextList;
	// for STAContext debug
	ULONG               STAContextCount;
	//ULONG               framedebug;

	//++ MulPending Queue
    PNDIS_PACKET        FirstMulPendingQueue;
    PNDIS_PACKET        LastMulPendingQueue;
    UINT                NumMulPendingQueued;
	//--

	ULONG				ForwardingLoss_Mul;
	ULONG				ForwardingLoss_Uni;
	UCHAR				AP_TxPowerLevel;
	UCHAR				BasicRateSet;
	UCHAR				EnableHiddenSSID;
	UCHAR				AP_MacAddrFilterMode;
	//MAC_LIST_DATA		MACFilterListData[16];
    MAC_LIST_DATA		MACFilterListData[1];
	ULONG               MACFilterListDataLength;
	BOOLEAN				aSTAInPSMode;
	BOOLEAN				bcstAnnounce;

	
	// Auto Rate

	NDIS_MINIPORT_TIMER AutoRateMonitorTimer;

	USHORT				RateLine1;
	USHORT				RateLine2;
	USHORT				RateLine3;

	USHORT				RiseConditionThreshold;
	USHORT				DownConditionThreshold;

	ULONG				ZDContinuousTxMode;
	ULONG				ZDContinuousTxStart;
	UCHAR				LastZDContinuousTxRate;
	UCHAR               ContinuousTxEnable; 

	//PDEVICE_OBJECT		pLowerDeviceObject;
	
	ZD1211_EVENT		BeaconUpdateLockEvent;

#ifdef OFDM
	ULONG               ProtectionModeTimer;
	ULONG               SetProtectionModeTimer;
	ULONG				CWmin_CWmax_ShortSlot;
	ULONG				CWmin_CWmax_NormalSlot;
	ULONG				CWmin_CWmax_LongSlot;
	BOOLEAN				ShortSlotEnable;

	//BOOLEAN				bTxBurstEnable;
	//int					TxBurstCount;

	BOOLEAN				bIBSS_G_MODE;
	BOOLEAN				bIBSS_PureG_MODE;
#endif

#ifdef HOST_IF_USB

	//WCHAR				firmwareName[80];

	ULONG				USBDbgFlag;


	// The USB Device Descriptor pointer
	PUSB_DEVICE_DESCRIPTOR			pDeviceDesc;

	// The USB Device Configuration pointer
    //PUSB_CONFIGURATION_DESCRIPTOR	pConfigDesc;
        
	// The selected interface pointer
	//PUSBD_INTERFACE_INFORMATION 	pUsbInterface;
	//USHORT							UsbInterfaceLen;

	ULONG				USBCSRAddress;

	// count the EP0 IO
	LONG				EPIOCount;
	ZD1211_SPIN_LOCK	EPIOCountLock;

	ZD1211_EVENT		USBEPStopEvent;

	BOOLEAN				bUSBDeveiceAttached;
	BOOLEAN				bUSBDeveiceResetting;
	
	// Resource for Int EP
	//PURB                pIntUrb;
	//PIRP                pIntIrp;
	//PUSB_INT_CONTEXT    pIntContext;
	UCHAR				IntEPBuffer[MAX_EPINT_BUFFER];
	UCHAR				IntEPBuffer2[MAX_EPINT_BUFFER];
	UCHAR				IntEPBuffer3[MAX_EPINT_BUFFER];
	ZD1211_EVENT		RegSpinlockEvent;
	//ZD1211_EVENT		USBIntEPStopEvent;

	ZD1211_EVENT		EPINT_IORDRsp_Event;

	// Resource for Rx EP
	//PURB                pRxUrb;
	//PIRP                pRxIrp;
	//PUSB_RX_CONTEXT		pRxContext;
	//ZD1211_EVENT		USBRxEPStopEvent;
	BOOLEAN				bRxInterruptEnable;
	
	UINT				TxEPRestart;
	UINT				TxRxEPRestart;
	ULONG				TxRxEPRestartWaitTime;	// milisecond

	// Resource for Tx EP
	//PURB                pTxUrb;
	//PIRP                pTxIrp;
	//PUSB_TX_CONTEXT		pTxContext;
	//ZD1211_EVENT		USBTxEPStopEvent;
	PUCHAR				pTxEPBuffer;
	ULONGLONG			TxSendStartTime;

    NDIS_WORK_ITEM      WorkItemReset;
    NDIS_WORK_ITEM      WorkItemOIDQuery;
    NDIS_WORK_ITEM      WorkItemOIDSet;

	ZD1211_EVENT		OIDNotifyEvent;

	OIDQUERY_WORKITEM_CONTEXT	OID_Query_Context;
	OIDSET_WORKITEM_CONTEXT		OID_Set_Context;

	BOOLEAN				bHandleNonRxTxRunning;
    NDIS_WORK_ITEM      WorkItemHandleNonRxTx;

	ZD1211_EVENT		PSSpinlockEvent;
	ZD1211_EVENT        StartRxEvent;
	ZD1211_EVENT        WaitRxEvent;
	ZD1211_EVENT        StartTxEvent;
	ZD1211_EVENT        WaitIntEvent;
	ZD1211_EVENT        TransferLockEvent;
	ZD1211_EVENT        WaitTxEvent;

	ULONG				REG_6e4_Add;
    ULONG               Continue2Rx;
    ULONG               WaitLenInfoCnt;

	BOOLEAN				bFlashable;
#endif

	UCHAR               CR31Flag;
	UCHAR               CR122Flag;
	UCHAR               CR203Flag;
	UCHAR               PHYTestIndex;
	UCHAR               PHYTestTimer;
	UCHAR               PHYTestTimerCount;
	UCHAR               PHYTestRssiBound;
    UCHAR               PHYTestRssi;
    ULONG               PHYTestTotal;
	UCHAR               IsUSB2_0;
	UCHAR               InitUSB2_0;
	ULONG               TrafficBound;
	ULONG               Traffic;
	UCHAR               TrafficTimer;
	UCHAR               TrafficTimerCount;
	UCHAR               ScanAllChannel;
    UCHAR               SaveChannel;
	//UCHAR               MaxBasicRate;
	ULONG               QueueCount;
	UCHAR               GOnlyMode;
	UCHAR               BasicRates[16];

	ULONG				CurrentCAMMode;
	UCHAR               PHYFre[100];
	ULONG               PHYVarThr;
	ULONG               PHYFreCount;
	ULONG               PHYFreCountThr;
	ULONG               PHYFreTimer;
	ULONG               PHYFreTimerCount;
	UCHAR               PHYFreOFDMframe;
	ULONG               PHYFreTimerTotal;
	UCHAR               PHYLowPower;
	UCHAR               UpdateCountryInfo;
	UCHAR               CheckBeacon;
	UCHAR               AnyBSSFlag;
	UCHAR               DelMac[6];
	UCHAR               IPCFlag;
	UCHAR               CTFlag;
	UCHAR               CR138Flag;
	UCHAR               CR139Flag;
	UCHAR               AdapterMaxRate;
	ULONGLONG           rxDataPerSec;
	ULONGLONG           txDataPerSec;
	ULONG               FlagAccessRegister;

	USHORT				FailCnt;
	BOOLEAN				bImmDiscAfterRFOff;
	BOOLEAN				busbswitch;
	ULONGLONG           TxTimer;
	BOOLEAN             SleepProtect;
	UCHAR				APTxPowerLevel; // Added by JHLee on 2004/08/02
	ULONG				autoRateMonitorInterval; // Store the value from Registry:AutoRateMonitorInterval
	ULONG				PSMode; // Store default power save mode from registry.
	ULONG				fragThreshold; // Store default Frag threshold from registry.
	ULONG				rTSThreshold; // Store default RTS threshold from registry.
	ULONG				iBSSChannel; // Store default IBSS channel number from registry
	ULONG				defaultTxRate; // store default TxRate from registry
	UCHAR				CountryCode[4]; //Store country code from registry
	ULONG				MinFrameCountDuring3Sec; // For quick change on signal strength.
	ULONG				FrameCountDuring3Sec;	// For quick change on signal strength.
	ULONG				FrameCountDuring10ms;
	// 0->Group Key, 1->Pairwise key  
	// 2~7->Group Key Destionation , 8~13->Pairwise Key Destination 
	UCHAR				WPAMICErrorLog[14];		
	// LED indicator threshold (%)(for LED_Ctrl_Type==2)
	ULONG				LedIndicatorLevel1; // default: 10
	ULONG				LedIndicatorLevel2;
	ULONG				LedIndicatorLevel3;
	ULONG				LedIndicatorLevel4;
	ULONG				LedIndicatorLevel5; // default: 90
	//chi_2004/08/26
	BOOLEAN             IBSS_BSSID_update;
	ULONG				tRFChange;
	// for Software AP
	UCHAR				APWirelessMode;
	UCHAR				APProtectionMode;
	ZD1211_USB_INFO     USBInfo;
	ULONG               TotalLength;
	UCHAR               TxPending;
	UCHAR               MaxPending;
	UCHAR               txrun;
	ULONG               RxCount;
	ULONG               TxCount;
	ZD1211_EVENT        RegEvent;
	ULONGLONG           scantime;
	UCHAR               bEndThreads;
	ZD1211_EVENT        hReqInfoEvent;
	ZD1211_EVENT        hReqCompleteEvent;
    HANDLE              hReqInfoFile;
	PUCHAR              ReqInfoMap;
	HANDLE              hReqInfoFileMutex;
	ZD1211_TX_TRANSFER  TxTransfer[30];


} ZD1205_ADAPTER, *PZD1205_ADAPTER;

typedef struct _ADAPTER_INFO
{
	BOOLEAN				AdapterActive;
	PZD1205_ADAPTER		AdapterContext;
	NDIS_HANDLE			AdapterHandle;
} ADAPTER_INFO, *PADAPTER_INFO;

//-------------------------------------------------------------------------
// Miniport and MAC specific Defines
//-------------------------------------------------------------------------
typedef struct _ZD1205_RESERVED {

    // next packet in the chain of queued packets being allocated,
    // or waiting for the finish of transmission.
    //
    // We always keep the packet on a list so that in case the
    // the adapter is closing down or resetting, all the packets
    // can easily be located and "canceled".
    //
    PNDIS_PACKET Next;
} ZD1205_RESERVED,*PZD1205_RESERVED;

#define PZD1205_RESERVED_FROM_PACKET(_Packet) \
    ((PZD1205_RESERVED)((_Packet)->MiniportReserved))

#define PZD1205_RESERVED2_FROM_PACKET(_Packet) \
    ((PZD1205_RESERVED)((_Packet)->MiniportReserved) + 1)

#define EnqueuePacket(_Head, _Tail, _Packet)           \
{                                                      \
    if (!_Head) {                                      \
    _Head = _Packet;                                   \
    } else {                                           \
    PZD1205_RESERVED2_FROM_PACKET(_Tail)->Next = _Packet; \
}                                                      \
    PZD1205_RESERVED2_FROM_PACKET(_Packet)->Next = NULL;  \
    _Tail = _Packet;                                   \
}

#define DequeuePacket(Head, Tail)                      \
{                                                      \
    PZD1205_RESERVED Reserved =                          \
    PZD1205_RESERVED2_FROM_PACKET(Head);                  \
    if (!Reserved->Next) {                             \
    Tail = NULL;                                       \
    }                                                  \
    Head = Reserved->Next;                             \
}

//Given a MiniportContextHandle return the PZD1205_ADAPTER it represents.
#define PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(Handle) ((PZD1205_ADAPTER)(Handle))


// Uniquely defines the location of the error
#define ZD1205LogError(_Adapt, _ProcId, _ErrCode, _Spec1) \
    NdisWriteErrorLogEntry((NDIS_HANDLE)(_Adapt)->ZD1205AdapterHandle, \
        (NDIS_ERROR_CODE)(_ErrCode), (ULONG) 4, (ULONG_PTR)(_ProcId), \
        (ULONG_PTR)(_Spec1), (ULONG_PTR)(_Adapt), (ULONG_PTR)(_Adapt->CSRAddress))


// Each entry in the error log will be tagged with a unique event code so that
// we'll be able to grep the driver source code for that specific event, and
// get an idea of why that particular event was logged.  Each time a new
// "ZD1205LogError" statement is added to the code, a new Event tag should be
// added below.
typedef enum _ZD1205_EVENT_VIEWER_CODES
{
        EVENT_0,                    // couldn't register the specified interrupt
        EVENT_1,                    // One of our PCI cards didn't get required resources
        EVENT_2,                    // bad node address (it was a multicast address)
        EVENT_3,                    // failed self-test
        EVENT_4,                    // Wait for SCB failed
        EVENT_5,                    // NdisRegisterAdapter failed for the MAC driver
        EVENT_6,                    // WaitSCB failed
        EVENT_7,                    // Command complete status was never posted to the SCB
        EVENT_8,                    // Couldn't find a phy at over-ride address 0
        EVENT_9,                    // Invalid duplex or speed setting with the detected phy
        EVENT_10,                   // Couldn't memory map the CSR.
        EVENT_11,                   // couldn't allocate enough map registers
        EVENT_12,                   // couldn't allocate enough recv cached memory
        EVENT_13,                   // couldn't allocate enough recv uncached shared memory
        EVENT_14,                   // couldn't allocate enough xmit cached memory
        EVENT_15,                   // couldn't allocate enough cb uncached shared memory
        EVENT_16,                   // Didn't find any PCI boards
        EVENT_17,          // 11    // Multiple PCI were found, but none matched our id.
        EVENT_18,          // 12    // NdisMPciAssignResources Error
        EVENT_19,          // 13    // Didn't Find Any PCI Boards that matched our subven/subdev
        EVENT_20,          // 14    // ran out of cached memory to allocate in async allocation

#ifndef HOST_IF_USB
        EVENT_30           // 1e    // WAIT_TRUE timed out
#else
        EVENT_30,          // 1e    // WAIT_TRUE timed out
		USB_EVENT_22				// Firmware can't download
#endif

} ZD1205_EVENT_VIEWER_CODES;



//================================================
// Global Variables shared by all driver instances
//================================================

// arrange this any way you want, but it can't add up to more
// than regparam NUMRFD + MAX_RECEIVE_DESCRIPTORS,
// total max is 1024 OR 0X400 right now
static const UINT packet_count[NUM_RMD] =
    {0x10,0x20,0x30,0x40,0x50,0x20,0x20,0x20,0x20,0x20};

// This constant is used for places where NdisAllocateMemory needs to be
// called and the HighestAcceptableAddress does not matter.
static const NDIS_PHYSICAL_ADDRESS HighestAcceptableMax =
    NDIS_PHYSICAL_ADDRESS_CONST(-1,-1);



#endif /* _ZD1205SW_ */

