/****************************************************************************
** COPYRIGHT (C) 2003-2004 ZYDAS CORPORATION                               **
** HTTP://WWW.ZYDAS.COM.TW/                                                **
****************************************************************************/

/****************************************************************************
Module Name:
    zd1205.c

This driver runs on the following hardware:
	ZD1211

Environment:
    Windows CE

Revision History

*****************************************************************************/

#include "precomp.h"

ADAPTER_INFO	gAdapterInfo[MAX_SUPPORT_DEVICE_NUM];

UCHAR	BeaconFrm[ ] = {
	0x3B,	// Beacon_Len - 1
	0x80, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22,
	0x02, 0x90, 0x47, 0xb9, 0x7c, 0x4b, 0x00, 0x00,
	0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x64, 0x00, 0x02, 0x00, 0x00, 0x05, 0x5a, 0x79, 
	0x44, 0x41, 0x53, 0x01, 0x04, 0x82, 0x84, 0x0b,
	0x16, 0x03, 0x01, 0x01, 0x06, 0x02, 0x00, 0x00,
	// send arbitrary CRC32
	0x00, 0x00, 0x00, 0x00
};

UCHAR	BeaconFrm1[ ] = {
	0x3B,	// Beacon_Len - 1
	0x80, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
	0xff, 0xff, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22,
	0x06, 0x01, 0xb2, 0x02, 0x3d, 0x03, 0x00, 0x00,
	0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x64, 0x00, 0x02, 0x00, 0x00, 0x05, 0x5a, 0x79, 
	0x44, 0x41, 0x53, 0x01, 0x04, 0x82, 0x04, 0x0b,
	0x16, 0x03, 0x01, 0x01, 0x06, 0x02, 0x00, 0x00,
	// send arbitrary CRC32
	0x00, 0x00, 0x00, 0x00
};

//-----------------------------------------------------------------------------
// Procedure:   DeleteSharedAdapterMemory
//
// Description: This routine is responsible for the deallocation of the
//              shared memory data structures for the Adapter structure.  This
//              includes the both the cached and uncached memory allocations.
//              We also free any allocated map registers in this routine.
//
// Arguments:
//      Adapter - Ptr to the Adapter structure
//
// Returns:     (none)
//-----------------------------------------------------------------------------

VOID
DeleteSharedAdapterMemory(PZD1205_ADAPTER Adapter)

{
    DEBUGFUNC(_T("DeleteSharedAdapterMemory"));

    INITSTR((_T("\n")));

	// Free MapIoSpace
	//if (Adapter->CSRAddress){
	//	NdisMUnmapIoSpace(Adapter->ZD1205AdapterHandle, (PVOID)(Adapter->CSRAddress),
	//				sizeof (CSR_STRUC));
	//}
    
	// Free memory for receive packetpool and bufferpool
	if (Adapter->ReceiveBufferPool){
		NdisFreeBufferPool(Adapter->ReceiveBufferPool);
	}

	if (Adapter->ReceivePacketPool){
		NdisFreePacketPool(Adapter->ReceivePacketPool);
	}

    // Free any memory allocated for the Original Software
    // receive structures (SwRfds)
    if (Adapter->RecvCached)
    {
        NdisFreeMemory((PVOID) Adapter->RecvCached, Adapter->RecvCachedSize, 0);
        Adapter->RecvCached = (PUCHAR) 0;
    }

    // Free any memory allocated for the shared receive structures (RFDs)
    // the original ones...
    if (Adapter->RecvUnCached)
    {
        // Now free the shared memory that was used for the receive buffers.

		NdisFreeMemory(
            (PVOID) Adapter->RecvUnCached,
            Adapter->RecvUnCachedSize,
			0
			);
        Adapter->RecvUnCached = (PUCHAR) 0;
    }

    // Free any memory allocated for the Software transmit structures (SwTcbs)
    if (Adapter->XmitCached)
    {
        NdisFreeMemory((PVOID) Adapter->XmitCached, Adapter->XmitCachedSize, 0);
        Adapter->XmitCached = (PUCHAR) 0;
    }

    // Free any memory allocated for the shared transmit structures (TCBs,
    // TBDs, non transmit command blocks, etc.)
    if (Adapter->XmitUnCached)
    {
        // Now free the shared memory that was used for the command blocks and
        // transmit buffers.

        NdisFreeMemory(
            (PVOID) Adapter->XmitUnCached,
            Adapter->XmitUnCachedSize,
            0
			);
        Adapter->XmitUnCached = (PUCHAR) 0;
    }

}


//-----------------------------------------------------------------------------
// Procedure:   FreeAdapterObject
//
// Description: This routine releases all resources defined in the ADAPTER
//              object and returns the ADAPTER object memory to the free pool.
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//
// Returns:     (none)
//-----------------------------------------------------------------------------

VOID
FreeAdapterObject(PZD1205_ADAPTER Adapter)

{

    DEBUGFUNC(_T("FreeAdapterObject"));
    INITSTR((_T("\n")));

#ifdef HOST_IF_USB
	USB_StopRxEP(Adapter);
	USB_StopTxEP(Adapter);
	USB_StopIntEP(Adapter);

	USB_WaitAllEPStop(Adapter);
#endif

    // Free the transmit, receive, and SCB structures
    DeleteSharedAdapterMemory(Adapter);


    // free the adapter object itself
	//for (i=0;i<MAX_SUPPORT_DEVICE_NUM;i++){
	//	if (gAdapterInfo[i].AdapterContext == Adapter){
	//		gAdapterInfo[i].AdapterActive = FALSE;
	//		break;
	//	}
	//}

    ZD1205_FREE_MEM(Adapter, sizeof(ZD1205_ADAPTER));

}


//-----------------------------------------------------------------------------
// Procedure:   SoftwareReset
//
// Description: This routine is called by SelfTestHardware and InitializeZD1205.
//              It resets the ZD1205 by issuing a PORT SOFTWARE RESET.
//
// Arguments:
//      Adapter - ptr to Adapter object instance
//
// Returns:
//      (none)
//-----------------------------------------------------------------------------

VOID
SoftwareReset(PZD1205_ADAPTER Adapter)

{
	return;
}


//-----------------------------------------------------------------------------
// Procedure:   DriverEntry
//
// Description: This is the primary initialization routine for the ZD1205
//              driver. It is simply responsible for the intializing the
//              wrapper and registering the adapter driver.  The routine gets
//              called once per driver, but ZD1205Initialize(miniport) or
//              AddAdapter (mac) will get called multiple times if there are
//              multiple adapters.
//
// Arguments:
//      DriverObject - Pointer to driver object created by the system.
//      RegistryPath - The registry path of this driver
//
// Returns:
//  The status of the operation, normally this will be NDIS_STATUS_SUCCESS
//-----------------------------------------------------------------------------

NTSTATUS
DriverEntry(PDRIVER_OBJECT DriverObject,
            PUNICODE_STRING RegistryPath)

{

    NDIS_STATUS         Status;
    NDIS_HANDLE         NdisWrapperHandle;

    NDIS_MINIPORT_CHARACTERISTICS ZD1205Char;

    DEBUGFUNC(_T("ZD1205DriverEntry"));
    INITSTR((_T("\n")));
	INITSTR((_T("Pointer of DriverObject = %x\n"), (ULONG)DriverObject));
	NdisZeroMemory(gAdapterInfo, sizeof(gAdapterInfo));
	ZDDbgPrint((_T("Size of gAdapterInfo = %d\n"), sizeof(gAdapterInfo)));

    // Now we must initialize the wrapper, and then register the Miniport
    NdisMInitializeWrapper( &NdisWrapperHandle,
        DriverObject,
        RegistryPath,
        NULL
        );

    NdisZeroMemory(&ZD1205Char, sizeof(ZD1205Char));

    // Initialize the Miniport characteristics for the call to
    // NdisMRegisterMiniport.
    ZD1205Char.MajorNdisVersion       = ZD1205_NDIS_MAJOR_VERSION;
    ZD1205Char.MinorNdisVersion       = ZD1205_NDIS_MINOR_VERSION;
    ZD1205Char.CheckForHangHandler    = ZD1205CheckForHang;

    ZD1205Char.HaltHandler            = ZD1205Halt;
    ZD1205Char.InitializeHandler      = ZD1205Initialize;

//#ifdef USE_WORKITEM_HANDLE_OID
//    ZD1205Char.QueryInformationHandler= USB_QueryInformation;
//#else
    ZD1205Char.QueryInformationHandler= ZD1205QueryInformation;
//#endif
	ZD1205Char.ReconfigureHandler     = NULL;
    ZD1205Char.ResetHandler           = ZD1205Reset;
    
//#ifdef USE_WORKITEM_HANDLE_OID
	//ZD1205Char.SetInformationHandler  = USB_SetInformation;
//#else
    ZD1205Char.SetInformationHandler  = ZD1205SetInformation;
//#endif

    ZD1205Char.SendHandler            = NULL;
    ZD1205Char.SendPacketsHandler     = ZD1205MultipleSend;
    ZD1205Char.ReturnPacketHandler    = ZD1205GetReturnedPackets;
    ZD1205Char.TransferDataHandler    = NULL;
    ZD1205Char.AllocateCompleteHandler = ZD1205AllocateComplete;

#ifdef NDIS51_MINIPORT
	INITSTR((_T("######## For WindowsXP\n")));
    ZD1205Char.CancelSendPacketsHandler = ZD1205CancelSendPackets;
    ZD1205Char.PnPEventNotifyHandler    = ZD1205PnPEventNotify;
    ZD1205Char.AdapterShutdownHandler   = ZD1205Shutdown;
#endif


    DEBUGSTR((_T("DriverEntry: About to call NdisMRegisterMiniport\n")));

    // Register this driver with the NDIS wrapper
    //   This will cause ZD1205Initialize to be called before returning
    Status = NdisMRegisterMiniport(
        NdisWrapperHandle,
        &ZD1205Char,
        sizeof(NDIS_MINIPORT_CHARACTERISTICS));

    DEBUGSTR((_T("DriverEntry: NdisMRegisterMiniport returns %X\n"), Status));

    if (Status == NDIS_STATUS_SUCCESS)
        return STATUS_SUCCESS;

    DEBUGSTR((_T("NdisMRegisterMiniport failed (Status = 0x%x)\n"), Status));
    return Status;
}


//-----------------------------------------------------------------------------
// ZD1205GetReturnedPackets
//
// PARAMETERS: IN NDIS_HANDLE MiniportAdapterContext
//                 - a context version of our Adapter pointer
//             IN NDIS_PACKET Packet
//                 - the packet that is being freed
//
// DESCRIPTION: This function attempts to return to the receive free list the
//              packet passed to us by NDIS
//
// RETURNS: nothing
//
//-----------------------------------------------------------------------------

VOID
ZD1205GetReturnedPackets(NDIS_HANDLE  MiniportAdapterContext,
                       PNDIS_PACKET Packet)
{
    PZD1205_ADAPTER   Adapter;
    PZD1205SwRfd      SwRfd;
	PZD1205_DEFRAG_ENTITY	DefragEntityPtr;
	USHORT				i;
	ULONG				Count;
	ZD1211_IRQL			OldIrql;

    DEBUGFUNC(_T("ZD1205GetReturnedPackets"));

    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

    ASSERT(Packet);

    DefragEntityPtr = *(ZD1205_DEFRAG_ENTITY **)(Packet->MiniportReserved);
    ASSERT(DefragEntityPtr);
	Count = QueueGetCount(&DefragEntityPtr->ElementList);
	ASSERT(Count == 1);
	for (i=0;i < Count;i++){
		SwRfd = (PZD1205SwRfd)QueuePopHead(&DefragEntityPtr->ElementList);
		if (i==0){
			NdisFreeBuffer(SwRfd->ReceiveBuffer);
		}
	    InitAndChainRfd(Adapter, SwRfd);
	}

	QueuePutTail(&Adapter->FreeDefragEntityList, &DefragEntityPtr->Link);
	NdisFreePacket(Packet);


	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
}

//-----------------------------------------------------------------------------
// GetReturnedMgtFrame
//
// PARAMETERS: PZD1205_ADAPTER Adapter
//                 - a context version of our Adapter pointer
//             IN NDIS_PACKET Packet
//                 - the packet that is being freed
//
// DESCRIPTION: 
//
// RETURNS: nothing
//
//-----------------------------------------------------------------------------

VOID
GetReturnedMgtFrame(
						IN PZD1205_ADAPTER Adapter,
						IN PZD1205_DEFRAG_ENTITY DefragEntityPtr)
{
	USHORT				i;
    PZD1205SwRfd      SwRfd;
	ULONG				Count;
	ZD1211_IRQL			OldIrql;

    DEBUGFUNC(_T("GetReturnedMgtFrame"));

    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	Count = QueueGetCount(&DefragEntityPtr->ElementList);
	for (i=0;i < Count;i++){
		SwRfd = (PZD1205SwRfd)QueuePopHead(&DefragEntityPtr->ElementList);
	    InitAndChainRfd(Adapter, SwRfd);
	}

	QueuePutTail(&Adapter->FreeDefragEntityList, &DefragEntityPtr->Link);
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

	return;
}

VOID
AP_GetReturnedMgtFrame(
						IN PZD1205_ADAPTER Adapter,
						IN PZD1205_DEFRAG_ENTITY DefragEntityPtr)
{
	USHORT				i;
    PZD1205SwRfd      SwRfd;
	ULONG				Count;

    DEBUGFUNC(_T("GetReturnedMgtFrame"));

    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	Count = QueueGetCount(&DefragEntityPtr->ElementList);
	for (i=0;i < Count;i++){
		SwRfd = (PZD1205SwRfd)QueuePopHead(&DefragEntityPtr->ElementList);
	    InitAndChainRfd(Adapter, SwRfd);
	}

	QueuePutTail(&Adapter->FreeDefragEntityList, &DefragEntityPtr->Link);
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	return;
}


//-----------------------------------------------------------------------------
// PROCEDURE: InitAndChainRfd
//
// PARAMETERS: Adapter structure Pointer
//              Pointer to SwRfd
//
//  DESCRIPTION: Re-initializes the given SwRfd super-structure
//               and chains it onto the current receive chain
//               We don't need to lock around this routine because
//
//  RETURNS: nothing
//
//-----------------------------------------------------------------------------

VOID
InitAndChainRfd(PZD1205_ADAPTER Adapter,
                   PZD1205SwRfd SwRfdPtr)
{
    PRFD_STRUC      Rfd;
    PZD1205SwRfd      LastRfd;

	DEBUGFUNC(_T("InitAndChainRfd"));


    Rfd = SwRfdPtr->Rfd;

    // We are done with this RFD so re-initialize it.
    Rfd->CbStatus = 0xffffffff;
    //Rfd->ActualCount = 0;
    Rfd->CbCommand = (RFD_EL_BIT);
    Rfd->NextCbPhyAddrLowPart = DRIVER_NULL;
    Rfd->NextCbPhyAddrHighPart = 0;

	//INITSTR((_T("RfdPhy = %x\n"), SwRfdPtr->RfdPhys));

    // Append the RFD to the end of the RFD chain.  
    if (!QueueEmpty(&Adapter->RfdList))
    {
        LastRfd = (PZD1205SwRfd)QueueGetTail(&Adapter->RfdList);

        ASSERT(LastRfd);

        // Link it onto the end of the chain dynamically
        Rfd = LastRfd->Rfd;
        Rfd->NextCbPhyAddrLowPart = SwRfdPtr->RfdPhys;
		Rfd->NextCbPhyAddrHighPart = 0;
		// Start Rx bus master again
		//INITSTR((_T("****** Start Rx RfdPhy = %x\n"), LastRfd->RfdPhys));
		//++ Must be atomic
		//_asm pushfd
		//_asm cli
        Rfd->CbCommand = 0;
#ifndef HOST_IF_USB
		while(1){	// Read DeviceState
			ZD1205_READ_REGISTER(Adapter,DeviceState, &tmpvalue);
			tmpvalue &= 0xf0;
			if ( (tmpvalue == RX_READ_RCB) || (tmpvalue == RX_CHK_RCB) ){
				// Device is now checking suspend or not.
				// Keep watching until it finished check.
				NdisStallExecution(1);
				continue;
			}
			else{
				break;
			}
		}
		if (tmpvalue == RX_IDLE){ // Rx bus master is in idle state.
			if (Rfd->CbStatus != RFD_STATUS_COMPLETE){
				StartUpload(Adapter, LastRfd->RfdPhys);
			}
		}
#endif
		//_asm popfd
		//--

    }

    // The processing on this RFD is done, so put it back on the tail of
    // our list
	QueuePutTail(&Adapter->RfdList, &SwRfdPtr->Link);

	//ASSERT(0);
}

//-----------------------------------------------------------------------------
// Procedure:   ZD1205CheckForHang
//
// Description: This routine should check to see if the adapter is "hung", and
//              fix it if it is.  Right now this routine does not check for a
//              hang, because the adapter should never "timeout".  In the
//              future, this is where the code to check for a timeout would go,
//              if there were bugs in the chipset that could cause transmit
//              "timeouts".
//
// Arguments:
//      MiniportAdapterContext (both) - pointer to the adapter object data area
//
// Returns:
//      False (miniport) - Always
//      Nothing (mac) - Cause the function is a VOID
//-----------------------------------------------------------------------------

BOOLEAN
ZD1205CheckForHang(NDIS_HANDLE MiniportAdapterContext)
{
	PZD1205_ADAPTER Adapter;
    ULONG               temp=0;
	ZD1211_IRQL			OldIrql;
	ULONG				Bcn_Avg_SQ;
	ULONG               tmpvalue;
    
	DEBUGFUNC(_T("ZD1205CheckForHang"));
   
    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

	/*if ( !Adapter->adapterReady )
	{
		Adapter->ZDConfigRunning = TRUE;
	    Adapter->CardSetting.Info_SSID[0] = 0;
        Adapter->CardSetting.Info_SSID[1] = 5;
		Adapter->CardSetting.Info_SSID[2] = 'd';
		Adapter->CardSetting.Info_SSID[3] = 'l';
		Adapter->CardSetting.Info_SSID[4] = 'i';
		Adapter->CardSetting.Info_SSID[5] = 'n';
		Adapter->CardSetting.Info_SSID[6] = 'k';
	    Adapter->CardSetting.Info_SSID[7] = 0;
        SetBSSType(Adapter, INFRASTRUCTURE_BSS);
		Adapter->encryMode = Ndis802_11WEPDisabled;
		Change_EncryMode(Adapter, NO_WEP, NO_WEP);
		Adapter->CardSetting.AuthMode = Ndis802_11AuthModeOpen;
        Mgt_SendNotification(Adapter, RQ_CONNECTNETWORK);
	}*/

    //ZD1205_READ_REGISTER(Adapter,TotalRxFrm, &tmpvalue);
	//INITSTR((_T("TotalRxFrm = %d\n"), tmpvalue));

    //DbgPrint("Rx Count = %d\r\n", Adapter->RxCount);
	//DbgPrint("Tx Count = %d\r\n\r\n", Adapter->TxCount);
    DbgPrint("Tx Max = %d\r\n\r\n", Adapter->MaxPending);

	if ( Adapter->CardSetting.BSSType == AP_BSS ){
        Adapter->ProtectionModeTimer++;
        if ( (!(Adapter->CardSetting.ERP[2] & BIT_0))&&
		     (Adapter->ProtectionModeTimer == Adapter->SetProtectionModeTimer) ){
	        Adapter->CardSetting.ERP[2] &= ~BIT_1;
		    Adapter->SelfCTSOn = 0;
		}
	}

	// We must acquire spin-lock to leverage the IRQL to DISPATCH Level
	// to solve power-save problem in Win98. Otherwise, the management/housekeeping
	// thread might disappear in Win98.
	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
 
	Adapter->CheckForHangLoop++;
	if (Adapter->CardSetting.BSSType == AP_BSS){
		if ((Adapter->CheckForHangLoop % 15) == 0){	// 30 sec
			AP_CheckSTAsAlive(Adapter);
		}
	}

	
	if ( (!Adapter->NotCheckConnect) && 
		 (!Adapter->adapterReady) && 
		 (Adapter->CardSetting.BSSType != AP_BSS)&&
		 (!(Adapter->Notification & RQ_CONNECTNETWORK))&&
		 (Adapter->ZDConfigRunning) )
	{
		if (! (Adapter->bImmDiscAfterRFOff && Adapter->CardSetting.BSSType == INDEPENDENT_BSS) ){
			//if ( Adapter->ScanTime == 5 ){
			//	Mgt_SendNotification(Adapter, RQ_ZD_BSSID_LIST_SCAN);		    
			//	Adapter->ScanTime = 0;
			//}
			ZDDbgPrint((_T("ZD1205CheckForHang call Connect!!\n")));
			Mgt_SendNotification(Adapter, RQ_CONNECTNETWORK);		    
			Adapter->ScanTime++;
		}
	}
	else{
		Adapter->ScanTime = 0;
	}
	

#ifndef HOST_IF_USB
	// CWmin, CWmax modification mechanism
#if 0
	if (Adapter->TotalRxDataFrmBytes > (Adapter->TotalTxDataFrmBytes * 30)){
#if defined(PHY_1202) 
		ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, 0x7f0407);
#endif
	}
	else{
#if defined(PHY_1202) 
		ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, 0x7f047f);
#endif
	}
#endif
#endif
#ifdef ADJUST_BACK_OFF_TIME_IN_BASS_RX //jxiao
	if (Adapter->TotalRxDataFrmBytes > (Adapter->TotalTxDataFrmBytes * 30))
	//DbgPrint("TotalRxByte=%u,  TotalTxByte=%u\n",Adapter->TotalRxDataFrmBytes,Adapter->TotalTxDataFrmBytes);
	//if ((Adapter->TotalRxDataFrmBytes + Adapter->TotalTxDataFrmBytes) > 500000)
	{// Add by jxiao, Use short back off to Improve throughput for RaLink AP
		//ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, 0x7f040f);
		if (Adapter->CWmin_CWmax_ShortSlot==0x7f043f)
		{
			Adapter->CWmin_CWmax_ShortSlot=0x7f040f;
			//DbgPrint("Use Very Short back off\n");
			Mgt_SendNotification(Adapter, RQ_UPDATE_CW);
		}
	}
	else
	{ // Low traffic, restore to normal back off time.
		//ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, 0x7f043f);
		if (Adapter->CWmin_CWmax_ShortSlot==0x7f040f)
		{
			Adapter->CWmin_CWmax_ShortSlot=0x7f043f;
			Mgt_SendNotification(Adapter, RQ_UPDATE_CW);
		}
	}
#endif
    // check if roam
	if ((Adapter->CardSetting.BSSType==INFRASTRUCTURE_BSS) &&
		(Adapter->CardSetting.BSSType != AP_BSS)&&
		(Adapter->adapterReady == TRUE)){
		if ((Adapter->CheckForHangLoop % 10) == 0){
			if (Adapter->Bcn_Acc_Num){
				Bcn_Avg_SQ = Adapter->Bcn_Acc_SQ / Adapter->Bcn_Acc_Num;
				ZDDbgPrint((_T("BCN_Avg_SQ = %x, Adapter->WorseSQThr = %x\n"),
					Bcn_Avg_SQ, Adapter->WorseSQThr));
				ZDDbgPrint((_T("Adapter->TotalTxDataFrmBytes+Adapter->TotalRxDataFrmBytes = %x\n"),
					Adapter->TotalTxDataFrmBytes+Adapter->TotalRxDataFrmBytes));
				if ((Bcn_Avg_SQ > Adapter->WorseSQThr) &&
					((Adapter->TotalTxDataFrmBytes+Adapter->TotalRxDataFrmBytes)<8000)
					// Throughput < 32k bit/s
				   ){
					ZDDbgPrint((_T("Issue Roaming request\n")));
					Mgt_SendNotification(Adapter, RQ_ROAMING);
				}
			}
			Adapter->Bcn_Acc_SQ = 0;
			Adapter->Bcn_Acc_Num = 0;
		}
	}

	// Check if AP(Access Point) still alive in the current channel
	if ((Adapter->CardSetting.BSSType==INFRASTRUCTURE_BSS) &&
		(Adapter->adapterReady == TRUE)){
		if (Adapter->AP_Alive == FALSE){
			Adapter->NoBcnDetectedCnt ++;
			if (Adapter->NoBcnDetectedCnt == 3){
				Adapter->NoBcnDetectedCnt = 0;
				Adapter->LossAP ++;
				//Adapter->Notification |= RQ_DISCONNECT;
				//Adapter->Notification |= RQ_CONNECTNETWORK;
				//Mgt_SendNotification(Adapter, RQ_DISCONNECT);			
				//ZDDbgPrint((_T("ZD1205CheckForHang call Connect!! 2\n")));
				//Mgt_SendNotification(Adapter, RQ_CONNECTNETWORK);
			}
		}
		else{
			Adapter->NoBcnDetectedCnt = 0;
		}
		Adapter->AP_Alive = FALSE;
	}
	
	// Check if IBSS partners still alive in the current channel
	if (Adapter->bImmDiscAfterRFOff){
		if ((Adapter->CardSetting.BSSType==INDEPENDENT_BSS) &&
			(Adapter->adapterReady == TRUE)){
				if (Adapter->IBSSPartner_Alive == FALSE){
					Adapter->NoIBSSPartnerDetectedCnt ++;
					if (Adapter->NoIBSSPartnerDetectedCnt == 3){
						Adapter->NoIBSSPartnerDetectedCnt = 0;
						NdisMIndicateStatus(Adapter->ZD1205AdapterHandle,
							NDIS_STATUS_MEDIA_DISCONNECT,
							NULL,     // Not used.
							0);                 // Not used.
						NdisMIndicateStatusComplete(Adapter->ZD1205AdapterHandle);
						Adapter->adapterReady = FALSE;
					}
				}
				else{
					Adapter->NoIBSSPartnerDetectedCnt = 0;
				}
				Adapter->IBSSPartner_Alive = FALSE;
		}
	}
	if ((Adapter->PSMSupported) &&
		(Adapter->adapterReady)){	
		if (Adapter->TestPS){
			Adapter->SuggestionMode = PSM;
		}
		else{
			// Check if we need to enter the PSM (power-save mode), CAM mode or no-change
			if ((Adapter->TotalTxDataFrmBytes < Adapter->PSThreshhold) && (Adapter->TotalRxDataFrmBytes < Adapter->PSThreshhold)){
				Adapter->SuggestionMode = PSM;
			}
			//else if((Adapter->TotalTxDataFrmBytes + Adapter->TotalRxDataFrmBytes) > 1000){
			else{
				Adapter->SuggestionMode = CAM;
			}
		}
	}
	Adapter->TotalTxDataFrmBytes = 0;
	Adapter->TotalRxDataFrmBytes = 0;

#ifndef HOST_IF_USB
	// Check if Device Remove
	ZD1205_READ_REGISTER(Adapter,MAC_PS_STATE, &tmpvalue);
	tmpvalue &= 0x7;
	if (tmpvalue != MAC_OPERATION){
		INITSTR((_T("Device Hang !!!!!!!!!!!!\n")));
		INITSTR((_T("MAC_PS_STATE = %x\n"), tmpvalue));
		//ZD1205DisableInterrupt(Adapter);
		
		// Check agian to judge False-Alarm-POR (power on reset) or Device-Removed
		if (tmpvalue != MAC_INI){
			Adapter->SurpriseRemoved = TRUE;
			while (!(QueueEmpty(&Adapter->ActiveChainList)))
			{
				// here we have to fail any sends that haven't completed yet
				PZD1205SwTcb pSwTcb = (PZD1205SwTcb) QueuePopHead(&Adapter->ActiveChainList);
				
				if (pSwTcb->LastFrag){
					ProviderMSendComplete(Adapter, pSwTcb, &OldIrql);
				}
			}
			// check to make sure there are no outstanding transmits
			while(Adapter->FirstTxQueue)
			{
				PNDIS_PACKET QueuePacket = Adapter->FirstTxQueue;
				
				Adapter->NumPacketsQueued--;
				DequeuePacket(Adapter->FirstTxQueue, Adapter->LastTxQueue);
				
				//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
				ZDDbg((_T("---------->>> Release SpinLock\n")));
				ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
				NdisMSendComplete(
					Adapter->ZD1205AdapterHandle,
					QueuePacket,
					NDIS_STATUS_SUCCESS);
				//NDIS_STATUS_FAILURE);
				//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
				ZDDbg((_T("Get SpinLock\n")));
				ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
				
			}
			APMgt_SendNotification(Adapter, BIT_2);
			//Adapter->adapterReady = FALSE;
			//Adapter->DeviceInSleep = FALSE;
			// Let Device go to sleep, so that device bus master will not action any more.
		}
		else{
			ZDDbgPrint((_T("Abnormal POR detected\n")));
			Adapter->AbNormalPOR ++;
			ASSERT(0);
			InitializeAdapter(Adapter, TRUE);

			for( i=0; i<4; i++ ){
				if ( Adapter->CardSetting.EncryMode == WEP64 ){
					ptmp = Adapter->AllKeyBuf+5*i;
					SetKeyContext( Adapter, DEFAULT_KEY_SET, WEP64, (UCHAR)i, 
						ptmp);
					loop = 20;
				}
				else if (Adapter->CardSetting.EncryMode == WEP128){
					ptmp = Adapter->AllKeyBuf+13*i;
					SetKeyContext( Adapter, DEFAULT_KEY_SET, WEP128, (UCHAR)i, 
						ptmp);
					loop = 52;
				}
				else if (Adapter->CardSetting.EncryMode == WEP256){
					ptmp = Adapter->AllKeyBuf+29*i;
					SetKeyContext( Adapter, DEFAULT_KEY_SET, WEP256, (UCHAR)i, 
						ptmp);
					loop = 52;
				}
				else{
					loop = 0;
				}
				
#ifndef AMAC
				for( j=0; j<loop; j+=4 ){
					ZD1205_READ_REGISTER(Adapter,WEPKey0+j, &tmpkey);
					switch( i ){
					case 0:
						tmpkey &= ~0x000000ff;
						tmpkey |= ((ULONG)*(ptmp) << 0);
						break;
					case 1:
						tmpkey &= ~0x0000ff00;
						tmpkey |= ((ULONG)*(ptmp) << 8);
						break;
					case 2:
						tmpkey &= ~0x00ff0000;
						tmpkey |= ((ULONG)*(ptmp) << 16);
						break;
					case 3:
						tmpkey &= ~0xff000000;
						tmpkey |= ((ULONG)*(ptmp) << 24);
						break;
					}
					ptmp++;
					ZD1205_WRITE_REGISTER(Adapter,WEPKey0+j, tmpkey);
				}
#endif
			}

			ZD1205EnableInterrupt(Adapter);
			Adapter->SurpriseRemoved = TRUE;
			SleepResetDevice(Adapter);
			Adapter->SurpriseRemoved = FALSE;
			if (Adapter->CardSetting.BSSType == AP_BSS){

				//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
				ZDDbg((_T("---------->>> Release SpinLock\n")));
				ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
				SetBSSType(Adapter, AP_BSS);
				//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
				ZDDbg((_T("Get SpinLock\n")));
				ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

			}
			Mgt_SendNotification(Adapter, RQ_DISCONNECT);			
			ZDDbgPrint((_T("ZD1205CheckForHang call Connect!! 3\n")));
			Mgt_SendNotification(Adapter, RQ_CONNECTNETWORK);
		}
	}
#endif

#ifdef HOST_IF_USB
	{
		
		// Check if Tx hang

		if (Adapter->TxSendStartTime){
#ifdef CE_ALL
			if (abs((LONG)(GetTickCount()-Adapter->TxSendStartTime)) > (1000 * IOWAITTIME)){ // 
#else
			if (abs((LONG)(KeQueryInterruptTime()-Adapter->TxSendStartTime)) > (10000000 * IOWAITTIME)){ // 
#endif
                {
				    DWORD  bytes, err;

					//(Adapter->USBInfo.lpUsbFuncs->lpGetTransferStatus)(Adapter->USBInfo.hTxEP, &bytes, &err);
					//DbgPrint("sent bytes = %d", bytes);
					//DbgPrint("error code = %d", err);
				}
				{
                    ULONG  tmpvalue=0;

                    //ZD1205_READ_REGISTER(Adapter, E2P_ALLOWED_CHANNEL, &tmpvalue);
					//DbgPrint("Tx timeout ==>> allowed channel = %x\r\n", tmpvalue);
                    ZD1211_READ_REGISTER(Adapter, 0x8003, &tmpvalue, FALSE);
					DbgPrint("Tx timeout ==>> 0x8003 = %x\r\n", tmpvalue);
					tmpvalue = 0;
                    ZD1205_READ_REGISTER(Adapter, 0x684, &tmpvalue);
					DbgPrint("Tx timeout ==>> 0x864 = %x\r\n", tmpvalue);
				}
                //DbgPrint("Tx timeout, try to cancel Tx EP !!!!!!!!!!!!\r\n");
				//INITSTR((_T("Tx timeout, try to cancel Tx EP !!!!!!!!!!!!\n")));

				//ASSERT(0);

				//(Adapter->USBInfo.lpUsbFuncs->lpAbortPipeTransfers)(Adapter->USBInfo.hPipe[EP_DATA_OUT], USB_NO_WAIT);
                Adapter->USBInfo.hTxEP = NULL;
				Adapter->TxSendStartTime = 0;

				//if ( Adapter->USBInfo.hTxEP )
				//{
                //    (Adapter->USBInfo.lpUsbFuncs->lpResetPipe)(Adapter->USBInfo.hPipe[EP_DATA_OUT]);
                    //(Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(Adapter->USBInfo.hTxEP);
				//    Adapter->USBInfo.hTxEP = NULL;
				//	Adapter->bTxReset = TRUE;
				//}
				//DbgPrint(" SetEvent StartTxEvent from CheckForHang\n");
				//SetEvent(Adapter->StartTxEvent);

				//if (Adapter->pTxIrp)
				//	IoCancelIrp(Adapter->pTxIrp);
			}
		}
	}

	if (!Adapter->bUSBDeveiceAttached){
		while(Adapter->FirstTxQueue)
		{
			PNDIS_PACKET QueuePacket = Adapter->FirstTxQueue;

			Adapter->NumPacketsQueued--;
			DequeuePacket(Adapter->FirstTxQueue, Adapter->LastTxQueue);

			// we must release the lock here before returning control to ndis
			// (even temporarily like this)
			//KeReleaseSpinLock(&Adapter->Lock, OldIrql);

			NdisMSendComplete(
				Adapter->ZD1205AdapterHandle,
				QueuePacket,
				NDIS_STATUS_SUCCESS);
				//NDIS_STATUS_FAILURE);
		
			//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		}
	}
#endif

	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

	return(FALSE);
}


//-----------------------------------------------------------------------------
// Procedure:   ZD1205Halt
//
// Description: Removes an adapter instance that was previously initialized.
//              To "halt" or "remove" an adapter, we disable its interrupt,
//              abort its receive unit (otherwise it would continue to DMA in
//              data), and release all of the resources (memory, i/o space,
//              etc.) that the adapter instance was using.
//              This routine is only called when the adapter is "stopped"
//              or unloaded with a "net stop e100b". To see what is called
//              at machine shutdown see ZD1205ShutdownHandler.
//
// Arguments:
//      MiniportAdapterContext - pointer to the adapter object data area.
//
// Returns:     (none)
//-----------------------------------------------------------------------------


VOID
ZD1205Halt(NDIS_HANDLE MiniportAdapterContext)

{
    PZD1205_ADAPTER Adapter;
    BOOLEAN     Cancelled;
    UINT        i;
	ZD1211_IRQL	OldIrql;

    DEBUGFUNC(_T("ZD1205Halt"));
    //ZD1205_Sleep(100000000);

	INITSTR((_T("\n")));
    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

	DbgPrint("enter ZD1205Halt\r\n");
	//ZDDbgPrint((_T("Issue DeAuthentication frame\n")));
	//Deauthenticate(Adapter);
	ZD1205_Sleep(10000);

	Adapter->PSMSupported = FALSE;

    //Adapter->stop = 1;

	//SaveDefaultConnection(Adapter);

	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);

//    NdisWriteErrorLogEntry( Adapter->ZD1205AdapterHandle,
//		                    NDIS_STATUS_FAILURE,
//							0
//							);

    
	
	// turn off RF power, fully.
	Adapter->PhyTest = 0;
	//TurnOff_RF(Adapter);
	
	// When suspend, call this
	//if ( Adapter->RF_Mode == AL2230_RF )
    //    Set_IF_Synthesizer(Adapter, 0x71687);
    //AcquireCtrOfPhyReg(Adapter);
	//ZD1205_WRITE_REGISTER(Adapter, CR252, 0xFF);
	//ZD1205_WRITE_REGISTER(Adapter, CR253, 0xc);
	//ZD1205_WRITE_REGISTER(Adapter, CR255, 0x3);
    //ReleaseCtrOfPhyReg(Adapter);

	//ZD1205DisableInterrupt(Adapter);
    Adapter->DisableTx = TRUE;


//JWEI 2004/02/09
    Adapter->LinkTimer = 0;
    // Turn off LEDs
    //ZD1205_WRITE_REGISTER(Adapter, rLED_CTRL, 0x0);
	// Turn off 5-LED
	//if (Adapter->LED_Ctrl_Type == 2)
	//{
	//	ZD1211_WRITE_REGISTER(Adapter, SOCGPIO, 0 , FALSE);
	//}
	//iLED_ON(Adapter, LED1);
	//iLED_OFF(Adapter, LED2);
	
	// turn of MAC
	//ZD1205_WRITE_REGISTER(Adapter, PS_Ctrl, 0x3);

	//USB_StopIntEP(Adapter);
	//USB_StopRxEP(Adapter);
	//USB_StopTxEP(Adapter);

    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	//ZDDbg((_T("Get SpinLock\n")));
	//ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
#ifndef HOST_IF_USB
    // check to make sure there are no outstanding transmits
    while(Adapter->FirstTxQueue)
    {
        PNDIS_PACKET QueuePacket = Adapter->FirstTxQueue;

        Adapter->NumPacketsQueued--;
        DequeuePacket(Adapter->FirstTxQueue, Adapter->LastTxQueue);

		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
        NdisMSendComplete(
            Adapter->ZD1205AdapterHandle,
            QueuePacket,
			NDIS_STATUS_SUCCESS);
            //NDIS_STATUS_FAILURE);
		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

    }

    while (!(QueueEmpty(&Adapter->ActiveChainList)))
    {
        // here we have to fail any sends that haven't completed yet
        PZD1205SwTcb pSwTcb = (PZD1205SwTcb) QueuePopHead(&Adapter->ActiveChainList);

		if (pSwTcb->LastFrag){
			ProviderMSendComplete(Adapter, pSwTcb, &OldIrql);
		}
    }
#endif

    //AP_CompleteAllPendingPktsAndTimers(Adapter, &OldIrql);
	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	//ZDDbg((_T("---------->>> Release SpinLock\n")));
	//ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);


#ifdef NDIS50_MINIPORT
    // deregister shutdown handler because we are halting now, and don't
    // want to have a bugcheck handler registered any more
    NdisMDeregisterAdapterShutdownHandler(Adapter->ZD1205AdapterHandle);
#endif 
    // added code to cancel our timer if for some reason it was active
    NdisMCancelTimer(&Adapter->ZD1205AsyncResetTimer,
        &Cancelled);
	
	//for counter measure
	NdisMCancelTimer(&Adapter->OneSecTimer,
        &Cancelled);

	NdisMCancelTimer(&Adapter->AutoRateMonitorTimer, &Cancelled );

      
#ifndef HOST_IF_USB  
	// Let Device go to sleep, so that device bus master will not action any more.
	ResetDevice(Adapter);
#endif

	// So that, we could terminate the following threads successfully.
	Adapter->DeviceInSleep = FALSE;
	
    //TerminateThreads(Adapter);

    //USB_WaitAllEPStop(Adapter);

    //APMgt_SendNotification(Adapter, BIT_31);	
    //while(Adapter->h_ZD1211APMgtThread){
	//	ZD1205_Sleep(10000);
	//}


    // Free the entire adapter object, including the shared memory structures.
	for (i=0;i<MAX_SUPPORT_DEVICE_NUM;i++){
		if (gAdapterInfo[i].AdapterContext == Adapter){
			gAdapterInfo[i].AdapterActive = FALSE;
			break;
		}
	}

    FreeAdapterObject(Adapter);

	DbgPrint("Exit ZD1205Halt\r\n");
	INITSTR((_T("Exit ZD1205Halt \n")));
}

//-----------------------------------------------------------------------------
// Procedure:   ZD1205ShutdownHandler
//
// Description: Removes an adapter instance that was previously initialized.
//              To Shutdown simply Disable interrupts and Stop the receive unit.
//              Since the system is shutting down there is no need to release
//              resources (memory, i/o space, etc.) that the adapter instance
//              was using.
//
// Arguments:
//      MiniportAdapterContext - pointer to the adapter object data area.
//
// Returns:     (none)
//-----------------------------------------------------------------------------

VOID
ZD1205ShutdownHandler(NDIS_HANDLE MiniportAdapterContext)

{
    PZD1205_ADAPTER Adapter;
    DEBUGFUNC(_T("ZD1205ShutdownHandler"));

    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

    DEBUGSTR((_T("ZD1205ShutdownHandler %8x\n"), Adapter));

    // Disable the device's interrupt line.
    ZD1205DisableInterrupt(Adapter);
}


#define		TOLERANCE		2
//-----------------------------------------------------------------------------
// Procedure:   ZD1205HouseThread
//
// Description: 
//				Adapter - Ptr to the Adapter structure
//
// Arguments:
//
// Returns:     (none)
//-----------------------------------------------------------------------------
ULONG    
ZD1205HouseThread(IN PZD1205_ADAPTER Adapter)
{
	ULONG						tmpvalue;
	UCHAR                       channel;
    USHORT                      setpoint;
	ULONG						accumulate =0;
    USHORT                      TrackingCnt = 0;
//``JWEI 2003/12/25
    ULONG                       accumulate_OFDM = 0;
    USHORT                      TrackingCnt_OFDM = 0;
    UCHAR                       PreTxOFDMType = cTX_CCK;
	ULONG                       average = 0;
	ULONG						HouseLoop=0;
	USHORT						TrackingLoop = 0;
	ULONG						ul_BcnItvl, ul_atimwnd;
	ULONGLONG					TSFTimer;
	UCHAR						AccState = ACC_1;
	ULONG						Avg1_SQ;
	ULONG						Avg2_SQ;
    ULONG                       Avg1_SQ_OFDM;
    ULONG                       Avg2_SQ_OFDM;
	UCHAR						SuspiciousNoPhyClk=0;
	ULONG						LastUnderrunCnt = 0;
	ULONG						ul_DifTotalTxFrm = 0;
	ULONG						ul_DifUnderrunCnt = 0;
	ULONG						HouseKeepingInterval=64;

	DEBUGFUNC(_T("ZD1205HouseThread"));
    INITSTR((_T("\n")));

    // if WinME or Win98, sleep 5 seconds 
    //if ( IoIsWdmVersionAvailable(1, 0x05) || IoIsWdmVersionAvailable(1, 0x00) )
    //    ZD1205_Sleep(7000000);

	while(1){
		// Sleep 10ms
		HouseLoop ++;
		ZD1205_Sleep(10000);

		USB_MPRcvFromIntEP(Adapter);
		if ( !Adapter->adapterReady )
		{
	        SetThreadPriority(Adapter->h_ZD1211StartRxThread, THREAD_PRIORITY_ABOVE_NORMAL);
		    //SetThreadPriority(Adapter->h_ZD1211StartRxThread, THREAD_PRIORITY_ABOVE_NORMAL);
		    SetThreadPriority(Adapter->h_ZD1205MgtThread, THREAD_PRIORITY_ABOVE_NORMAL);
		}


		while (Adapter->DeviceInSleep && (!Adapter->bTerminateThreads)){
			// If device is in sleep, do not access device register often to
			// prevent host from slowing down.
			ZD1205_Sleep(10000);
		}

    //JWEI 2004/02/09

		if (Adapter->bTerminateThreads){
			INITSTR((_T("Terminate ZD1205 House thread now..\n")));
			Adapter->h_ZD1205HouseThread = NULL;
#ifdef HOST_IF_USB
			USB_CompleteCountDecrement(Adapter);
#endif
			//PsTerminateSystemThread(STATUS_SUCCESS);
			ExitThread(0);
		}

        continue;

		//================================================================
		// HouseKeeping Every 10ms
		//================================================================
		// jxiaoDbg 4901
//		if (1){
//			NdisGetSystemUpTime(&systemUpTime);
			//DbgPrint("-----------House Keeping Timer at %u\n",systemUpTime);
			//DbgPrint("Rcv %u Packets at %u\n", Adapter->FrameCountDuring10ms,systemUpTime);
			//Adapter->FrameCountDuring10ms=0;
//		}
//``JWEI 2003/12/22 
    #if fANT_DIVERSITY
        // Software Antenna Diversity Mechanism
#if DBG_ANTDIV
		if (1)
#else
        if (Adapter->EnableSwAntennaDiv)
#endif
        {
            switch(AccState)
            {
            case ACC_1:
                if ((HouseLoop % Adapter->Ant_MonitorDur1) == 0)
                {
                    if (Adapter->Acc_Num_OFDM)
					{
                        Avg1_SQ_OFDM = Adapter->Acc_SQ_OFDM / Adapter->Acc_Num_OFDM;
#if DBG_ANTDIV
						DbgPrint("ANTDIV:Avg1_OFDM = %d, Count:%d\n", Avg1_SQ_OFDM, Adapter->Acc_Num_OFDM);
#endif
					}
                    else
                    {
                        Avg1_SQ_OFDM = 0;
                        if (Adapter->Acc_Num)
                            Avg1_SQ = Adapter->Acc_SQ / Adapter->Acc_Num;
                        else
                            Avg1_SQ = 0;
                        ZDDbgPrint((_T("Avg1_SQ = %d NiceSQThr = %d\n"), Avg1_SQ, Adapter->NiceSQThr));
#if DBG_ANTDIV
						DbgPrint("AntDiv:Avg1 = %d, Count:%d\n", Avg1_SQ, Adapter->Acc_Num);
#endif
                        JWeiDbgPrint2(("Avg1_SQ = %d; ", Avg1_SQ));
                    }
                    ZDDbgPrint((_T("Avg1_SQ_OFDM = %d NiceSQThr_OFDM = %d\n"), Avg1_SQ_OFDM, Adapter->NiceSQThr_OFDM));
                    JWeiDbgPrint2(("Avg1_SQ_OFDM = %d\n", Avg1_SQ_OFDM));
                // Higher SQ is better
                    if (((Avg1_SQ_OFDM < Adapter->NiceSQThr_OFDM) && (Avg1_SQ_OFDM > 0))
                        || ((Avg1_SQ_OFDM == 0)
                            && ((Avg1_SQ < Adapter->NiceSQThr) && (Avg1_SQ > 0)))
                        || (Adapter->adapterReady != TRUE)) // disconnected
                    {
#if DBG_ANTDIV
						if (Adapter->EnableSwAntennaDiv)
						{
							DbgPrint(" Monitor Antenna %d\n", Adapter->CurrentAntenna^1);
#else
						{
#endif
							SwitchAntenna(Adapter);
							AccState = ACC_2;
							//Adapter->RssiListIndex=0;
							//Adapter->AntDivState=ACC_2;
							Adapter->TimeElapsedInMT2=0;
						}
						
                    }
                    Adapter->Acc_SQ = 0;
                    Adapter->Acc_Num = 0;
                    Adapter->Acc_SQ_OFDM = 0;
                    Adapter->Acc_Num_OFDM = 0;
                }
                break;
            case ACC_2:
                //if ((HouseLoop % Adapter->Ant_MonitorDur2) == 0)
				Adapter->TimeElapsedInMT2++;
				if (Adapter->Acc_Num_OFDM > Adapter->MinSamplesInMT2 ||
					Adapter->Acc_Num > Adapter->MinSamplesInMT2 ||
					Adapter->TimeElapsedInMT2 >= Adapter->Ant_MonitorDur2)

                {
					//Adapter->AntDivState=ACC_1;
					//SampleCount=Adapter->RssiListIndex;
                    if (Adapter->Acc_Num_OFDM)
					{
                        Avg2_SQ_OFDM = Adapter->Acc_SQ_OFDM / Adapter->Acc_Num_OFDM;
#if DBG_ANTDIV
						DbgPrint("Avg2_OFDM = %d, Avg1_OFDM=%d, Avg2Count:%d, SmplCnt=%d\n", Avg2_SQ_OFDM, Avg1_SQ_OFDM, Adapter->Acc_Num_OFDM,SampleCount);
#endif
					}
                    else
                    {
                        Avg2_SQ_OFDM = 0;
                        if (Adapter->Acc_Num)
                            Avg2_SQ = Adapter->Acc_SQ / Adapter->Acc_Num;
                        else
                            Avg2_SQ = 0;
                        ZDDbgPrint((_T("Avg2_SQ = %d Avg1_SQ = %d\n"), Avg2_SQ, Avg1_SQ));
#if DBG_ANTDIV
						DbgPrint("AntDiv: Avg2 = %d, Avg1= %d, Avg2Count:%d, SmplCnt=%d\n", Avg2_SQ, Avg1_SQ, Adapter->Acc_Num, SampleCount); // jxiaoDbg
#endif
                        JWeiDbgPrint2(("Avg2_SQ = %d; ", Avg2_SQ));
                    }
                    ZDDbgPrint((_T("Avg2_SQ_OFDM = %d\n"), Avg2_SQ_OFDM));
                    JWeiDbgPrint2(("Avg2_SQ_OFDM = %d Avg1_SQ_OFDM = %d\n", Avg2_SQ_OFDM, Avg1_SQ_OFDM));
                // Higher SQ is better
                    if ((Avg2_SQ_OFDM < Avg1_SQ_OFDM)
                        || (((Avg2_SQ_OFDM == 0) && (Avg1_SQ_OFDM == 0))
                            && (Avg2_SQ < Avg1_SQ))
                        || (Adapter->adapterReady != TRUE)) // disconnected
                    {
                        JWeiDbgPrint2(("Re"));
#if DBG_ANTDIV
						if (Adapter->EnableSwAntennaDiv)
						{
							DbgPrint(" Switch Back to original antenna %u\n",Adapter->CurrentAntenna ^ 1);
#else
						{
#endif
							SwitchAntenna(Adapter);
						}
                    }
#if DBG_ANTDIV
					else
					{
						DbgPrint(" Stay at this better antenna %d\n",Adapter->CurrentAntenna);
					}
#endif
					
					/*{
						ULONG	i;
						for(i=0; i<SampleCount; i++)
						{
							DbgPrint("%u ms: %u dB\n",
								Adapter->RssiListDuringMT2[i] >> 16, // Rx Time
								Adapter->RssiListDuringMT2[i] & 255); // Rssi
							if (i > 16)
								break;
						}
					}*/
					//Adapter->RssiListIndex=0;
                    AccState = ACC_1;
					//DbgPrint("Switch to ACC_1 at %u\n",systemUpTime);
                    Adapter->Acc_SQ = 0;
                    Adapter->Acc_Num = 0;
                    Adapter->Acc_SQ_OFDM = 0;
                    Adapter->Acc_Num_OFDM = 0;
                }
				//else
				//	DbgPrint("Don't check before reach min samples\n");
                break;
            default:
                ASSERT(0);
                break;
            }
        }
    #endif
		// IBSS power-save monitor
		if ((Adapter->CardSetting.BSSType == INDEPENDENT_BSS) && (Adapter->adapterReady)){
			if (Adapter->PSMSupported){
				// Make sure that we have passed (ATIM-Window+TOLERANCE)
				ZD1205_READ_REGISTER(Adapter, BCNInterval, &ul_BcnItvl);
				ul_BcnItvl &= 0xffff;
				ZD1205_READ_REGISTER(Adapter, ATIMWndPeriod, &ul_atimwnd);
				ZD1205_READ_REGISTER(Adapter,TSF_LowPart, &tmpvalue);
				TSFTimer = tmpvalue;
				ZD1205_READ_REGISTER(Adapter,TSF_HighPart, &tmpvalue);
				TSFTimer += (((ULONGLONG)tmpvalue) << 32);
				TSFTimer = TSFTimer >> 10; // in unit of TU
				INITSTR((_T("TSF(TU) %d \n"), TSFTimer));
				INITSTR((_T("BeaconInterval = %d\n"), ul_BcnItvl));
				INITSTR((_T("TSF mod BeaconInterval = %d\n"), (TSFTimer % ul_BcnItvl)));
				if ((TSFTimer % ul_BcnItvl) > (ul_atimwnd+TOLERANCE)){
					// Make sure no traffic before (ATIMWnd+TOLERANCE)
					if ((!Adapter->FrmRxed1) && (Adapter->SuggestionMode==PSM)){
						// Any frame wait for transmission.
						if (QueueEmpty(&Adapter->ActiveChainList)){
							SleepResetDevice(Adapter);
						}
					}
				}
			}
		}
					
		//================================================================
		// Housekeeping Every 100ms
		//================================================================
		if ((HouseLoop % 10) == 0){
/*			
			if ( (Adapter->PHYTestTimerCount >= Adapter->PHYTestTimer)&&
				 (Adapter->PHYTestTimer)&&
				 (Adapter->PHYLowPower & BIT_0)&&
				 (Adapter->RF_Mode == RFMD_RF)&&
				 (!Adapter->ContinuousTxEnable)&&
				 (Adapter->adapterReady) )
			{
			    Adapter->PHYTestTimerCount = 0;
				if ( (Adapter->PHYTestRssi >= Adapter->PHYTestRssiBound)&&
					 ((!Adapter->CR122Flag)||(Adapter->CR122Flag==2)) )
				{
		            AcquireCtrOfPhyReg(Adapter);
				    ZD1205_WRITE_REGISTER(Adapter,CR122, 0xff);
					if ( (Adapter->PHYLowPower & BIT_1)&&
						 ((!Adapter->CR31Flag)||(Adapter->CR31Flag==2)) )
					{
						Adapter->TraceSetPoint = 0;
                    //``JWEI 2004/05/19
						if (Adapter->PA_Type == 1)
							tmpvalue = Adapter->IntValue[Adapter->CardSetting.Channel-1] - 0x28;
						else
							tmpvalue = Adapter->IntValue[Adapter->CardSetting.Channel-1] - cPWR_STRONG_SIG_DROP; // minus 0x10
                        ZD1205_WRITE_REGISTER(Adapter,CR31, tmpvalue);
						Adapter->CR31Flag = 1;
					}
                       
					ReleaseCtrOfPhyReg(Adapter);
					Adapter->CR122Flag = 1;
				}
				else if ( ((Adapter->PHYTestRssi < Adapter->PHYTestRssiBound)||
							(Adapter->FrameCountDuring3Sec < Adapter->MinFrameCountDuring3Sec)) && // Added at 4811A
					      ((Adapter->CR122Flag)||(Adapter->CR122Flag==2)) )
				//else if ( (Adapter->PHYTestRssi < Adapter->PHYTestRssiBound)&&
				//	      ((Adapter->CR122Flag)||(Adapter->CR122Flag==2)) )
				{
		            AcquireCtrOfPhyReg(Adapter);
				    ZD1205_WRITE_REGISTER(Adapter,CR122, 0x00);
                    ReleaseCtrOfPhyReg(Adapter);
					if ( (Adapter->PHYLowPower & BIT_1)&&
						 ((Adapter->CR31Flag)||(Adapter->CR31Flag==2)) )
					{
						Adapter->TraceSetPoint = 1;
                        UpdateIntegrationValue(Adapter, Adapter->CardSetting.Channel);
						Adapter->CR31Flag = 0;
					}
					Adapter->CR122Flag = 0;
					if (Adapter->FrameCountDuring3Sec < Adapter->MinFrameCountDuring3Sec)
						Adapter->PHYTestTotal=0;	// Reset the accumulator of signal strenth sample.
				}
				Adapter->FrameCountDuring3Sec=0; // Reset the received frame dounter.
			} 
			else if ( (Adapter->PHYTestTimerCount >= Adapter->PHYTestTimer)&&
				      (Adapter->PHYTestTimer)&&
				      (Adapter->PHYLowPower & BIT_0)&&
				      (Adapter->RF_Mode == AL2230_RF)&&
					  (!Adapter->ContinuousTxEnable)&&
				      (Adapter->adapterReady) )
			{
			    Adapter->PHYTestTimerCount = 0;
				if ( (Adapter->PHYTestRssi >= Adapter->PHYTestRssiBound)&&
					 ((!Adapter->CR203Flag)||(Adapter->CR203Flag==2)) )
				{
		            AcquireCtrOfPhyReg(Adapter);
				    ZD1205_WRITE_REGISTER(Adapter,CR203, 0x0a);
					if ( (Adapter->PHYLowPower & BIT_1)&&
						 ((!Adapter->CR31Flag)||(Adapter->CR31Flag==2)) )
					{
						Adapter->TraceSetPoint = 0;
                    //``JWEI 2004/05/19
						if (Adapter->PA_Type == 1)
							tmpvalue = Adapter->IntValue[Adapter->CardSetting.Channel-1] - 0x28;
						else
							tmpvalue = Adapter->IntValue[Adapter->CardSetting.Channel-1] - cPWR_STRONG_SIG_DROP; // minus 0x10
                        ZD1205_WRITE_REGISTER(Adapter,CR31, tmpvalue);
						Adapter->CR31Flag = 1;
					}
                       
					ReleaseCtrOfPhyReg(Adapter);
					Adapter->CR203Flag = 1;
				}
				else if ( ((Adapter->PHYTestRssi < Adapter->PHYTestRssiBound) ||
					(Adapter->FrameCountDuring3Sec < Adapter->MinFrameCountDuring3Sec)) && // Added at 4811A
					      ((Adapter->CR203Flag)||(Adapter->CR203Flag==2)) )
				//else if ( (Adapter->PHYTestRssi < Adapter->PHYTestRssiBound)&&
				//	      ((Adapter->CR203Flag)||(Adapter->CR203Flag==2)) )
				{
		            AcquireCtrOfPhyReg(Adapter);
				    ZD1205_WRITE_REGISTER(Adapter,CR203, 0x06);
                    ReleaseCtrOfPhyReg(Adapter);
					if ( (Adapter->PHYLowPower & BIT_1)&&
						 ((Adapter->CR31Flag)||(Adapter->CR31Flag==2)) )
					{
						Adapter->TraceSetPoint = 1;
                        UpdateIntegrationValue(Adapter, Adapter->CardSetting.Channel);
						Adapter->CR31Flag = 0;
					}
					Adapter->CR203Flag = 0;
					if (Adapter->FrameCountDuring3Sec < Adapter->MinFrameCountDuring3Sec)
						Adapter->PHYTestTotal=0; // added at 4811A
				}
				Adapter->FrameCountDuring3Sec=0; // added at 4811A
			}
			else
			{
			    Adapter->PHYTestTimerCount++;
			}

			//++ Collect HW Tally
			ZD1205_READ_REGISTER(Adapter,CRC16Cnt, &tmpvalue);
			Adapter->Hw_CRC16Cnt += tmpvalue;
			ZD1205_READ_REGISTER(Adapter,CRC32Cnt, &tmpvalue);
			Adapter->Hw_CRC32Cnt += tmpvalue;
			ZD1205_READ_REGISTER(Adapter,DecrypErr_UNI, &tmpvalue);
			Adapter->Hw_DecrypErr_UNI += tmpvalue;
			ZD1205_READ_REGISTER(Adapter,DecrypErr_Mul, &tmpvalue);
			Adapter->Hw_DecrypErr_Mul += tmpvalue;
			ZD1205_READ_REGISTER(Adapter,RxFIFOOverrun, &tmpvalue);
			Adapter->Hw_RxFIFOOverrun += tmpvalue;
			ZD1205_READ_REGISTER(Adapter,TotalRxFrm, &tmpvalue);
			Adapter->Hw_TotalRxFrm += tmpvalue;
			
			ZD1205_READ_REGISTER(Adapter,UnderrunCnt, &tmpvalue);
			Adapter->Hw_UnderrunCnt += (0xFFFF & tmpvalue);
			ZD1205_READ_REGISTER(Adapter,RetryCnt, &tmpvalue);
			Adapter->Hw_RetryCnt += tmpvalue;
			ZD1205_READ_REGISTER(Adapter,TotalTxFrm, &tmpvalue);
			Adapter->Hw_TotalTxFrm += tmpvalue;
*/
			if ( Adapter->TrafficTimerCount >= Adapter->TrafficTimer )
			{
			    Adapter->TrafficTimerCount = 0;
				if ( Adapter->Traffic >= Adapter->TrafficBound )
					Adapter->ScanAllChannel = 0;
				else
					Adapter->ScanAllChannel = 1;
				Adapter->Traffic = 0;
			}else{
			    Adapter->TrafficTimerCount++;
			}

/*
#ifdef HOST_IF_USB
			ZD1205_READ_REGISTER(Adapter,0x6e4, &tmpvalue);
			Adapter->REG_6e4_Add += tmpvalue;
#endif
                        
            // Infrastructure Power-State momitor
            if ((Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS) &&
                (Adapter->adapterReady) &&
                (Adapter->PSMSupported)){
                // Now, we assure that no any power-save related operation performing.
                // That's because all power-save related operations are either 
                // Mutexed by Adapter->Lock or Notified by Adapter->Notification.
                if ( (Adapter->SuggestionMode == PSM) && 
                    (Adapter->PwrState == CAM)
                    ){

                    UpdateBeaconInterval_StartPS(Adapter);

                    Adapter->PwrState = PSM;
                    Mgt_SendNotification(Adapter, RQ_INFORM_PS_CHANGE);
                    ZDDbgPrint((_T("=====CAM --> PSM \n")));
                }
                else if ( (Adapter->SuggestionMode == CAM) && (Adapter->PwrState == PSM) && 
                    (!Adapter->DeviceInSleep)){

                    UpdateBeaconInterval_StopPS(Adapter);

                    Adapter->PwrState = CAM;
                    Mgt_SendNotification(Adapter, RQ_INFORM_PS_CHANGE);
                    ZDDbgPrint((_T("=====PSM --> CAM \n")));
                }
            }
*/
        } // End of 100ms house-loop

        //================================================================
        // Housekeeping Every 0.5 s
        //================================================================
 
        //``JWEI 2003/12/22
        #if fTX_PWR_CTRL
		HouseKeepingInterval=(Adapter->CardSetting.BSSType==PSEUDO_IBSS) ? 10:64;
		
        if ((HouseLoop % HouseKeepingInterval) == 0) // changed from 64 to 10 in Pseudo IBSS mode.
        {
            if (Adapter->TraceSetPoint)
            {
                AcquireCtrOfPhyReg(Adapter);
                if (TrackingLoop == TRACKING_NUM)
                {
                    TrackingLoop = 0;

                    if (TrackingCnt)
                    {
                    //    average = (ULONG) (accumulate / TRACKING_NUM);
                        average = (ULONG) (accumulate / TrackingCnt);
                        channel = Adapter->CardSetting.Channel;
                        setpoint = Adapter->SetPoint[channel-1] - Adapter->SetPointLevel;
                        JWeiDbgPrint3(1, (" <avg = %x> ", average));
                    //JWEI 2004/04/21
                        if ( Adapter->EnableTxPwrCtrl )
                        {
                            if (average < (ULONG) (setpoint - cPWR_CTRL_GUARD))
                            IncreaseTxPower(Adapter, cTX_CCK);
                        else if (average > setpoint)
                            DecreaseTxPower(Adapter, cTX_CCK);
                        }

                        accumulate = 0;
                        TrackingCnt = 0;
                    }

                    if (TrackingCnt_OFDM)
                    {
                        average = (ULONG) (accumulate_OFDM / TrackingCnt_OFDM);
                        channel = Adapter->CardSetting.Channel;
                        setpoint = Adapter->SetPointOFDM[Adapter->TxOFDMType - cTX_OFDM][channel - 1] - Adapter->SetPointLevel;
                        JWeiDbgPrint3(1, (" <avg%d = %x> ", Adapter->TxOFDMType, average));
                        if ( Adapter->EnableTxPwrCtrl )
                        {
                            if (average < (ULONG) (setpoint - cPWR_CTRL_GUARD))
                            IncreaseTxPower(Adapter, cTX_OFDM);
                        else if (average > setpoint)
                            DecreaseTxPower(Adapter, cTX_OFDM);
                        }

                        accumulate_OFDM = 0;
                        TrackingCnt_OFDM = 0;
                    }
                }
                else
                {
                    TrackingLoop ++;
                    ZD1205_READ_REGISTER(Adapter, rLED_CTRL, &tmpvalue);
                    if (tmpvalue & BIT_0)   // Continuous Tx
                    {
                    //    JWeiDbgPrint3(1, (" [ContTx %d] ", tmpvalue & BIT_2));
                        if (tmpvalue & BIT_2)   // Tx OFDM
                        {
                            Adapter->TxPwrOFDM ++;
                            Adapter->TxOFDMCnt = cTX_SENT_LEN + 1;
                            ZD1205_READ_REGISTER(Adapter, CR132, &tmpvalue);
                            tmpvalue &= 0xFF;
                            Adapter->TxOFDMType = cTX_OFDM;
                            if (tmpvalue == 0xC)
                                Adapter->TxOFDMType = cTX_54M;
                            else if (tmpvalue == 0x8)
                                Adapter->TxOFDMType = cTX_48M;
                        }
                        else
                        {
                            Adapter->TxPwrCCK ++;   // Tx CCK
                        }
                    }

                    if (Adapter->TxPwrCCK)  // New sent after last read
                    {
                        ZD1205_READ_REGISTER(Adapter,CR58, &tmpvalue);
                        tmpvalue &= 0xFF;
                    //    JWeiDbgPrint3(1, (" [PwrCCK = %x] ", tmpvalue));
                        accumulate += tmpvalue;
                        TrackingCnt ++;
                        Adapter->TxPwrCCK = 0;
                    }
                    if (Adapter->TxPwrOFDM)
                    {
                        if (Adapter->TxOFDMCnt > cTX_SENT_LEN)
                        {                   // make sure Tx by HMAC (for UMAC)
                            ZD1205_READ_REGISTER(Adapter,CR57, &tmpvalue);
                            tmpvalue &= 0xFF;
                        //    JWeiDbgPrint3(1, (" [PwrOFDM%d = %x] ", Adapter->TxOFDMType, tmpvalue));
                            accumulate_OFDM += tmpvalue;
                            TrackingCnt_OFDM ++;
                            PreTxOFDMType = Adapter->TxOFDMType;
                        }
                        else
                        {
                            if (PreTxOFDMType != Adapter->TxOFDMType)
                        {
                            accumulate_OFDM = 0;
                            TrackingCnt_OFDM = 0;
                        }
                        }
                        Adapter->TxPwrOFDM = 0;
                    }
                }
                ReleaseCtrOfPhyReg(Adapter);    
            }
        }
        #endif

    //JWEI 2004/02/09
    #ifdef HOST_IF_USB
        //================================================================
        // Housekeeping Every 1s
        //================================================================
            if ((HouseLoop % 100) == 0)
        {
/*
            // bypass the weak signal in BSS and AP mode
			if ( (Adapter->adapterReady)&&
				 (Adapter->PHYTestRssi <= 0x14)&&
				 ((Adapter->CardSetting.BSSType == INDEPENDENT_BSS)||
				  (Adapter->CardSetting.BSSType == PSEUDO_IBSS)) )
			{
				if (!Adapter->CR138Flag)
				{
                    AcquireCtrOfPhyReg(Adapter);
                    ZD1205_WRITE_REGISTER(Adapter,CR138, 0xa8);
					if(Adapter->RF_Mode == AL2230_RF ||
						Adapter->RF_Mode == AL2210_RF)
					{
						ZD1205_WRITE_REGISTER(Adapter,CR122, 0xFE);
					}

				    ReleaseCtrOfPhyReg(Adapter);
				    Adapter->CR138Flag = 1;
				}
			}
			else if ( Adapter->CR138Flag )
			{
                AcquireCtrOfPhyReg(Adapter);
                ZD1205_WRITE_REGISTER(Adapter,CR138, 0x28);
				if(Adapter->RF_Mode == AL2230_RF ||
					Adapter->RF_Mode == AL2210_RF)
				{
					ZD1205_WRITE_REGISTER(Adapter,CR122, 0xf0);
				}
				ReleaseCtrOfPhyReg(Adapter);
				Adapter->CR138Flag = 0;
			}

			// solve the throughput problem when communicate with the IPC card
			if ( ((Adapter->rxDataPerSec + Adapter->txDataPerSec) > 50000) &&
				 (Adapter->RF_Mode == RFMD_RF)&&
				 (Adapter->CardSetting.BSSType != PSEUDO_IBSS)&&
				 (Adapter->IPCFlag != 4) )
			{
				if ( (Adapter->rxDataPerSec > 3*Adapter->txDataPerSec)&&
					 (Adapter->PHYTestRssi <= 0x24) )
				{
					if ( (!Adapter->IPCFlag)||(Adapter->IPCFlag!=1) )
					{
					    AcquireCtrOfPhyReg(Adapter);
					    ZD1205_WRITE_REGISTER(Adapter,CR87, 0x0a);
					    ZD1205_WRITE_REGISTER(Adapter,CR89, 0x04);
					    ReleaseCtrOfPhyReg(Adapter);
					    Adapter->AdapterMaxRate = 8;  // MAX = 24M
					    Adapter->IPCFlag = 1;
					}
				}
				else if ( 3*Adapter->rxDataPerSec < Adapter->txDataPerSec )
				{
					if ( (!Adapter->IPCFlag)||(Adapter->IPCFlag!=3) )
					{
					    AcquireCtrOfPhyReg(Adapter);
					    ZD1205_WRITE_REGISTER(Adapter,CR87, 0x2A);
					    ZD1205_WRITE_REGISTER(Adapter,CR89, 0x24);
					    ReleaseCtrOfPhyReg(Adapter);
					    Adapter->AdapterMaxRate = 0x0B;  // MAX = 54M
					    Adapter->IPCFlag = 3;
					}
				}
				else 					      
				{
					if ( (!Adapter->IPCFlag)||(Adapter->IPCFlag!=2) )
					{
					    AcquireCtrOfPhyReg(Adapter);
					    ZD1205_WRITE_REGISTER(Adapter,CR87, 0x10);
					    ZD1205_WRITE_REGISTER(Adapter,CR89, 0x0C);
					    ReleaseCtrOfPhyReg(Adapter);
					    Adapter->AdapterMaxRate = 9;  // MAX = 36M
					    Adapter->IPCFlag = 2;
					}
				}
			}
			else if ( (Adapter->RF_Mode == RFMD_RF)&&
				      (Adapter->CardSetting.BSSType == PSEUDO_IBSS)&&
					  (Adapter->IPCFlag != 4) )
			{
				if ( (!Adapter->IPCFlag)||(Adapter->IPCFlag!=3) )
				{
				    AcquireCtrOfPhyReg(Adapter);
				    ZD1205_WRITE_REGISTER(Adapter,CR87, 0x2A);
				    ZD1205_WRITE_REGISTER(Adapter,CR89, 0x24);
				    ReleaseCtrOfPhyReg(Adapter);
				    Adapter->AdapterMaxRate = 0x0B;  // MAX = 54M
				    Adapter->IPCFlag = 3;
				}
			}
			else if ( (Adapter->RF_Mode == RFMD_RF)&&
				      (Adapter->IPCFlag != 4) )
			{
				if ( (!Adapter->IPCFlag)||(Adapter->IPCFlag!=2) )
				{
				    AcquireCtrOfPhyReg(Adapter);
				    ZD1205_WRITE_REGISTER(Adapter,CR87, 0x10);
				    ZD1205_WRITE_REGISTER(Adapter,CR89, 0x0C);
				    ReleaseCtrOfPhyReg(Adapter);
				    Adapter->AdapterMaxRate = 9;  // MAX = 36M
				    Adapter->IPCFlag = 2;
				}			
			}
			Adapter->rxDataPerSec = 0;
			Adapter->txDataPerSec = 0;
            if (Adapter->LinkLEDn == LED2)//JWEI 2004/02/20, Two LED case.
                iLED_OFF(Adapter, LED1);//Always turn off Data LED if no data sent.
			else if (Adapter->adapterReady && Adapter->LinkLEDn == LED1)//In single LED case,(Data and Link use the same LED)
				iLED_ON(Adapter, LED1); //Always Turn on Data LED if no data sent.

            //      || (Adapter->CardSetting.BSSType == INDEPENDENT_BSS))
            if (!(// (Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS) &&
                    (Adapter->adapterReady == TRUE)))
            {
                Adapter->LinkTimer ++;
                JWeiDbgPrint7(("%d:", Adapter->LinkTimer));
                if ((Adapter->LinkTimer == 1) && (Adapter->LinkLED_OnDur != 0))
                {
                    iLED_ON(Adapter, Adapter->LinkLEDn);
                    JWeiDbgPrint7(("On"));
                }
                if (Adapter->LinkTimer == (Adapter->LinkLED_OnDur + 1))
                {
                    iLED_OFF(Adapter, Adapter->LinkLEDn);
                    JWeiDbgPrint7(("Off"));
                }
                if (Adapter->LinkTimer >= (Adapter->LinkLED_OnDur + Adapter->LinkLED_OffDur))
                    Adapter->LinkTimer = 0;
                JWeiDbgPrint7(("\n"));
            }

        //JWEI 2004/02/14
            if (Adapter->PhyTest & BIT_8)
            {
            ULONG   tmpvalue;

                AcquireCtrOfPhyReg(Adapter);
                ZD1205_READ_REGISTER(Adapter, CR122, &tmpvalue);
                if ((tmpvalue & 0xFF) != 0xFF)
                    ZD1205_WRITE_REGISTER(Adapter, CR122, 0xFF);
                else
                    ZD1205_WRITE_REGISTER(Adapter, CR122, 0x00);
                ReleaseCtrOfPhyReg(Adapter);
            }

			if ( Adapter->CTFlag )
			{
				if ( (Adapter->adapterReady)&&
					 (Adapter->RF_Mode == RFMD_RF)&&
					 (Adapter->BssIndex)&&     // not scanning channel
					 ((Adapter->CardSetting.Channel == 6)||
					  (Adapter->CardSetting.Channel == 7)) )
				{
					if ( (Adapter->PHYTestRssi >= 0x16)&&
						 ((Adapter->CR139Flag == 0)||(Adapter->CR139Flag == 2)) )
					{
						AcquireCtrOfPhyReg(Adapter);
						ZD1205_WRITE_REGISTER(Adapter, CR139, 0xBA);
						ReleaseCtrOfPhyReg(Adapter);
						Adapter->CR139Flag = 1;
					}
					else if ( (Adapter->PHYTestRssi < 0x16)&&
							  ((Adapter->CR139Flag == 0)||(Adapter->CR139Flag == 1)) )
					{
						AcquireCtrOfPhyReg(Adapter);
						ZD1205_WRITE_REGISTER(Adapter, CR139, 0xB4);
						ReleaseCtrOfPhyReg(Adapter);
						Adapter->CR139Flag = 2;				
					}
				}
				else if ( (Adapter->CardSetting.Channel != 6)&&
						  (Adapter->CardSetting.Channel != 7)&&
						  (Adapter->CR139Flag != 0) )
				{
					AcquireCtrOfPhyReg(Adapter);
					ZD1205_WRITE_REGISTER(Adapter, CR139, 0xB4);
					ReleaseCtrOfPhyReg(Adapter);
					Adapter->CR139Flag = 0;			
				}
			}
        
			if (Adapter->LED_Ctrl_Type == 2 && 
				Adapter->CardSetting.BSSType!=AP_BSS)
			{
				ULONG	ulGPIO_WriteValue=BIT_8_12; // Set SGPIO0-4's direction to Output type.
				ULONG	percentageofRSSI;
			
				if (Adapter->adapterReady)
				{
					percentageofRSSI=(100*(Adapter->RxSignalStrength+10))/70;
					if(percentageofRSSI>=Adapter->LedIndicatorLevel5)
					{ // Set SGPIO4 - SGPIO0 to HIGH
						ulGPIO_WriteValue |= (BIT_4+BIT_3+BIT_2+BIT_1+BIT_0);			
					}
					else if(percentageofRSSI>=Adapter->LedIndicatorLevel4)
					{// Set SGPIO4 - SGPIO1 to HIGH
						ulGPIO_WriteValue |= (BIT_4+BIT_3+BIT_2+BIT_1);
					}
					else if(percentageofRSSI>=Adapter->LedIndicatorLevel3)
					{// Set SGPIO4 - SGPIO2 to HIGH
						ulGPIO_WriteValue |= (BIT_4+BIT_3+BIT_2);
					}
					else if(percentageofRSSI>=Adapter->LedIndicatorLevel2)
					{// Set SGPIO4 - SGPIO3 to HIGH
						ulGPIO_WriteValue |= (BIT_4+BIT_3);
					}
					else if(percentageofRSSI>=Adapter->LedIndicatorLevel1)
					{// Set SGPIO4 to HIGH
						ulGPIO_WriteValue |= (BIT_4);
					}
				}
				ZD1211_WRITE_REGISTER(Adapter, SOCGPIO, ulGPIO_WriteValue , FALSE);
			}
*/
		} // End of 1s house-loop
        //================================================================
        // Housekeeping Every 10s
        //================================================================
		if ( (HouseLoop % 1000) == 0 )
		{
/*
		    if ( Adapter->adapterReady )
			{
				AcquireCtrOfPhyReg(Adapter);

				 if ((Adapter->CardSetting.BSSType==INFRASTRUCTURE_BSS) && 
					 (Adapter->SelfCTSOn) && 
					 (Adapter->PHYTestRssi >= 0x20) )
				 {
					 ZD1205_WRITE_REGISTER(Adapter, CR99, 0x28);
				 }
				 else
				 {// Adhoc always use 
                     
					 ZD1205_WRITE_REGISTER(Adapter, CR99, 0x18);
				 }

				 ReleaseCtrOfPhyReg(Adapter);
			}
*/
		}
    #endif


	} // End of while

	return 0;
}

/*USHORT my_log(ULONG log)
{
    USHORT exp;
    log_table_1[]
    log_table_2[]

}*/

//-----------------------------------------------------------------------------
// Procedure:   ZD1205MgtThread
//
// Description: 
//				Adapter - Ptr to the Adapter structure
//
// Arguments:
//
// Returns:     (none)
//-----------------------------------------------------------------------------
ULONG    
ZD1205MgtThread(IN PZD1205_ADAPTER Adapter)
{
	PZD1205_DEFRAG_ENTITY		DefragEntityPtr = NULL;
	USHORT						i;
	USHORT						j;
	ULONG						tmpvalue;
	UCHAR						IBSS_BSSID[6];
	ULONG                       average = 0;
	USHORT						quality = 10000;
	USHORT						IdleLoop_Under_Seq1;
    ZD1211_IRQL					OldIrql;
	ULONGLONG                   timer;

	DEBUGFUNC(_T("ZD1205MgtThread"));
    INITSTR((_T("\n")));

	for (i=0;i<6;i++){
		IBSS_BSSID[i] =0;
	}

	while(1)
	{
#ifndef CE_ALL
		KeWaitForSingleObject(&Adapter->MgtEvent, Executive, KernelMode,
								FALSE, NULL);
		KeClearEvent(&Adapter->MgtEvent);
#else  // CE_ALL
	    WaitForSingleObject(Adapter->MgtEvent, INFINITE);
		ResetEvent(Adapter->MgtEvent);
#endif  // CE_ALL    

        if ( Adapter->bEndThreads )
		{
			DbgPrint("h_ZD1205MgtThread end");
            Adapter->h_ZD1205MgtThread = NULL;
		    ExitThread(0);
		}

		while (Adapter->DeviceInSleep && (!(Adapter->Notification & RQ_TERMINATION)))
		{
			// If device is in sleep, do not access device register often to
			// prevent host from slowing down.
			ZD1205_Sleep(10000);
		}

DoNotLikeGoTo:

		if (Adapter->CardSetting.BSSType == AP_BSS)
		{
			AP_ProcessEvents(Adapter);
			continue;
		}

		while(1){	

			if (Adapter->Notification & RQ_TERMINATION)
			{
				INITSTR((_T("Terminate ZD1205 thread now..\n")));
				Adapter->h_ZD1205MgtThread = NULL;
				Adapter->Notification &= ~RQ_TERMINATION;
#ifdef HOST_IF_USB
				USB_CompleteCountDecrement(Adapter);
#endif
				//PsTerminateSystemThread(STATUS_SUCCESS);
				ExitThread(0);
			}
			// set OID_802_11_CONFIGURATION
			else if ( Adapter->Notification & RQ_SET_OID ){
				Adapter->Notification &= ~RQ_SET_OID;
				{	// update beacon
					UCHAR	BcnInterval[2];	// convert to as network 

					BcnInterval[0] = (UCHAR)(Adapter->BeaconInterval);
					BcnInterval[1] = (UCHAR)(Adapter->BeaconInterval >> 8);

					UpdateBeaconInterval(Adapter, BcnInterval);
				}
				{	// update ATIMWindow
					UCHAR	ATIMWindow[2];	// convert to as network 

					ATIMWindow[0] = (UCHAR)(Adapter->ATIMWindow);
					ATIMWindow[1] = (UCHAR)(Adapter->ATIMWindow >> 8);
			
					UpdateATIMWindow(Adapter, ATIMWindow);
				}
				// update channel
			    Set_RF_Channel(Adapter, Adapter->SaveChannel, (UCHAR)Adapter->RF_Mode, 1);  			    
			}
			// Process RQ_ROAMING
			else if (Adapter->Notification & RQ_ROAMING){
				Adapter->Notification &= ~RQ_ROAMING;
				Adapter->ProbeReqWithSSID = TRUE;
				ChannelScan(Adapter, 1);
				Adapter->ProbeReqWithSSID = FALSE;
				i = 100;
				quality = 10000;
				for (j=0;j<Adapter->BSSCnt;j++)
				{
					// Add end of str token
					Adapter->CardSetting.Info_SSID[Adapter->CardSetting.Info_SSID[1]+2] = 0;
					Adapter->BSSInfo[j].SSID[Adapter->BSSInfo[j].SSID[1]+2] = 0;
					// match the SSID
					if ( ( strcmp((char*)&Adapter->CardSetting.Info_SSID[2], 
						   (char*)&Adapter->BSSInfo[j].SSID[2]) == 0 )
					   ){ //Found desired SSID						
						if (Adapter->BSSInfo[j].Capability[0] & BIT_0){ // This BSS is Infra-BSS
							if ( Adapter->BSSInfo[j].SignalQuality < quality ){
								quality = Adapter->BSSInfo[j].SignalQuality;
								i = j;
							}
						}
					}
				}
				ZDDbgPrint((_T("i = %x\n"), i));
				if (i<Adapter->BSSCnt){
					ZDDbgPrint((_T("Adapter->BSSInfo[i].SignalQuality = %x\n"),
						Adapter->BSSInfo[i].SignalQuality));
					if ((Adapter->BSSInfo[i].SignalQuality < Adapter->WorseSQThr)
					   ){
						if ((!memcmp(Adapter->BSSInfo[i].BSSID, Adapter->BSSID, 6))
							&&
							(Adapter->BSSInfo[i].Channel == Adapter->CardSetting.Channel)
							){
							// Do not Roamming.
						}
						else{
							// There exists another AP with SignalQaulity better than
							// Adapter->WorseSQThr. We execute roaming
							ZDDbgPrint((_T("########### ZD1202 Roamming........\n")));
							Adapter->RoamingCnt++;
							Mgt_SendNotification(Adapter, RQ_DISCONNECT);			
							Mgt_SendNotification(Adapter, RQ_CONNECTNETWORK);
						}
					}
				}
			}
			// Process RQ_DEAUTHENTICATION
			else if (Adapter->Notification & RQ_DEAUTHENTICATION){
				Deauthenticate(Adapter);
                Adapter->Notification &= ~RQ_DEAUTHENTICATION;
			}
			// Process RQ_DISCONNECT
			else if (Adapter->Notification & RQ_DISCONNECT){
				//ASSERT(0);
				// Note: The following sequence is important, do not change it arbitrarily.
				Adapter->adapterReady = FALSE;
				ZDDbgPrint((_T("Adapter DisConnect! - 4\n")));
				Adapter->PwrState = CAM;

				// PwrMgt = 0
				UpdateBeaconInterval_StopPS(Adapter);

				while (Adapter->DeviceInSleep)
				{
					ZD1205_Sleep(1000);
					ZDDbgPrint((_T("DeviceInSleep - 1\n")));
					if (Adapter->Notification & RQ_TERMINATION){
						break;
					}
				}

				{
					int WaitCnt = 0;
					// After all pending packets are served, then notify NDIS with
					// DISCONNECT.
					// Otherwise, it might cause IP Address = 0.0.0.0
					while (!(QueueEmpty(&Adapter->ActiveChainList))){
						// After this step, we can make sure that no ATIM pended.
						ZD1205_Sleep(1000);
						
						WaitCnt ++;

						if (WaitCnt > 100){
#ifdef HOST_IF_USB
							//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
							ZDDbg((_T("Get SpinLock\n")));
							ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
							USB_ResetTxQueue(Adapter);
							//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
							ZDDbg((_T("---------->>> Release SpinLock\n")));
							ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#endif
						}

						ZDDbgPrint((_T("ActiveChainList not empty - 1\n")));
						if (Adapter->Notification & RQ_TERMINATION){
							break;
						}
					}
				}

				// Notice PS_Change
				IssueNullData(Adapter);

				// IBSS = 0, i.e, stop sending beacon.
				UpdateBeaconInterval_Stop(Adapter);

				Adapter->Notification &= ~RQ_DISCONNECT;
				if ( ( Adapter->Notification & RQ_CONNECTNETWORK ) &&
					 ( !Adapter->ConnAfterDisconn ) )
                    Adapter->Notification &= ~RQ_CONNECTNETWORK;
				else
                    Adapter->ConnAfterDisconn = 0;
				// We may need issue disassociate frame.

				// Issue notification
				//if ((Adapter->adapterReady == FALSE) && (Adapter->PreState == TRUE)){
				//	Adapter->PreState = FALSE;
					NdisMIndicateStatus(Adapter->ZD1205AdapterHandle,
						NDIS_STATUS_MEDIA_DISCONNECT,
						NULL,     // Not used.
						0);                 // Not used.
					NdisMIndicateStatusComplete(Adapter->ZD1205AdapterHandle);

					Adapter->IBSS_BSSID_update=FALSE;
                //JWEI 2004/02/09
                #ifndef HOST_IF_USB
                //    ZD1205_WRITE_REGISTER(Adapter, LED1, 0x0);
                    iLED_OFF(Adapter, LED1);
                #else
                    ZD1205_WRITE_REGISTER(Adapter, FW_LINK_STATUS, 0x0);
                #endif
				//for counter measure	
				if (Adapter->DisallowAssociateTimer==0)		
					Mgt_SendNotification(Adapter, RQ_ZD_BSSID_LIST_SCAN);
				//}
			}
			// Process RQ_CONNECTNETWORK
			else if (Adapter->Notification & RQ_CONNECTNETWORK){
				DbgPrint("=====>>>>> MGT_CONNECTNETWORK <<<<<=====\r\n");
				//Adapter->Notification &= ~RQ_CONNECTNETWORK;
				//Adapter->adapterReady = TRUE;
				//		  NdisMIndicateStatus(Adapter->ZD1205AdapterHandle,
				//		                    NDIS_STATUS_MEDIA_CONNECT,
				//		                    NULL,     // Not used.
				//		                    0);                 // Not used.
				//	      NdisMIndicateStatusComplete(Adapter->ZD1205AdapterHandle);


				ZDDbgPrint((_T("MGT_CONNECTNETWORK\n")));

				//Adapter->Notification &= ~RQ_CONNECTNETWORK;
				NdisZeroMemory(Adapter->WPAMICErrorLog,14);
				if ( (Adapter->DisallowAssociateTimer) &&
					 (!memcmp(Adapter->AttackedSsid, &Adapter->CardSetting.Info_SSID[2], Adapter->CardSetting.Info_SSID[1])) )
				{
					ZDDbgPrint((_T("DisallowAssociateTimer : %d seconds remaining\n"), Adapter->DisallowAssociateTimer));	
				} else if (!Adapter->DisableTx){
					//ZD1205_WRITE_REGISTER(Adapter, CAM_MODE, CAM_STA);  // STA mode
					//Adapter->CurrentCAMMode = CAM_STA;
		
					ClearOppositeRateInfo(Adapter);

					// We may remove IsUSB2_0 setting when connect PURE G AP, or set PSEUDO IBSS
					// So restore it now
					#ifdef HOST_IF_USB
						//ZD1205_READ_REGISTER(Adapter, FW_USB_SPEED, &tmpvalue);
						Adapter->IsUSB2_0 = Adapter->InitUSB2_0;
					#else
						Adapter->IsUSB2_0 = 1;
					#endif

	                
					switch(Adapter->CardSetting.BSSType)
					{
					case INDEPENDENT_BSS:
						//++
						// Recover the EIFS to 0x47c.
						// We need not use SleepResetDevice. In IBSS mode, we can easily
						// get others traffic. (not less of Rx-Frame)

						Adapter->CardSetting.Capability = 0x422;
						if ( Adapter->bIBSS_PureG_MODE )
						{
							Adapter->CardSetting.Capability |= 0x400;
							if (Adapter->RF_Mode == AL2230_RF ||
								Adapter->RF_Mode == AL2210_RF)
							{
								ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0xA47c00a);
							}
							else
							{
 								ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x547c00a);
							}
						}
						else
						{
							Adapter->CardSetting.Capability &= ~(0x400);
							if (Adapter->RF_Mode == AL2230_RF ||
								Adapter->RF_Mode == AL2210_RF)
							{
								ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0xA47c032);
							}
							else
							{
								ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x547c032);
							}
						}
						ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_NormalSlot);
						//--

						ZDDbgPrint((_T("MGT_CONNECTNETWORK_2\n")));
						IBSS_ConnectNetwork(Adapter);
                        
						{	// use BSSID to set OppositeRateInfo is useless,
							// Its object is only to let ZDWLan show the correct
							// rate at the first time.
							BOOLEAN bRet;
							
							bRet = BuildOppositeRateInfo(	Adapter, 
															Adapter->BSSID, 
															Adapter->CardSetting.Info_SupportedRates,
															Adapter->CardSetting.Capability,
															0);
				#if	(defined(GCCK) && defined(OFDM))
							if (Adapter->CardSetting.Info_ExtSupportedRates[1]){
								if (bRet)
									BuildOppositeRateInfo(	Adapter, 
															Adapter->BSSID, 
															Adapter->CardSetting.Info_ExtSupportedRates, 
															Adapter->CardSetting.Capability,
															1);
							}
				#endif
						}

						if (Adapter->CardSetting.ATIMWindow != 0){
							// Do not set Adapter->PwrState = PSM, otherwise
							// throughput with Cirrus card (in PS mode) will down.
							//Adapter->PwrState = PSM;
							// PwrMgt = 1
							UpdateBeaconInterval_StartPS(Adapter);
						}
						else{
							//Adapter->PwrState = CAM;
							// PwrMgt = 0;
							UpdateBeaconInterval_StopPS(Adapter);
						}
						break;
					case INFRASTRUCTURE_BSS:
						//++
						// Recover the EIFS to 0x47c.
						// We need not use SleepResetDevice. In Infra mode, we can easily
						// get others traffic. (not less of Rx-Frame)
						//if (Adapter->RF_Mode == AL2230_RF ||
						//		Adapter->RF_Mode == AL2210_RF)
						//{
						//	ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0xA47c00a);
						//}
						//else
						//{
 						//	ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x547c00a);
						//}
						//ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_NormalSlot);
						//Adapter->LongSlot = 0;
						//--

						
						if ( Infra_ConnectNetwork(Adapter) != TRUE ){
							Adapter->ProbeReqWithSSID = TRUE;
							ZDDbgPrint((_T("scan channel after connecting failed\n")));
							//ChannelScan(Adapter,1);
							Adapter->ProbeReqWithSSID = FALSE;
						}
						else{
							//connect
							if ( !Adapter->CheckCTS )
								Adapter->CheckCTS = 1;
						
							// when connect, no need BSSID, or roamming will have problem
							Adapter->bNeedCmpBSSID = FALSE;	
							
							{
								BOOLEAN bRet;
								
								bRet = BuildOppositeRateInfo(	Adapter, 
																Adapter->BSSID, 
																Adapter->CardSetting.Info_SupportedRates,
																Adapter->CardSetting.Capability,
																0);
					#if	(defined(GCCK) && defined(OFDM))
								if (Adapter->CardSetting.Info_ExtSupportedRates[1]){
									if (bRet)
										BuildOppositeRateInfo(	Adapter, 
																Adapter->BSSID, 
																Adapter->CardSetting.Info_ExtSupportedRates, 
																Adapter->CardSetting.Capability,
																1);
								}
					#endif
							}
						}
						break;
					case PSEUDO_IBSS:
						//++
						// Set EIFS=0x32 to prevent chamber low Tx-throughput (sometimes)
						// problem. In chamber environment, almost no Rx-frame, once 
						// we detect a CRC16/CRC32 error frame, we adopt EIFS, because of
						// less of RX-frame, it's less posibility to change EIFS to DIFS
						// by FA (Frame Analyzer), and therefore damage the Tx-Throughput.
						// We must use SleepResetDevice to trigger FA to adpot 0x32.
						if (Adapter->RF_Mode == AL2230_RF ||
								Adapter->RF_Mode == AL2210_RF)
						{
							ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0xA032032);
						}
						else
						{
							ZD1205_WRITE_REGISTER(Adapter,IFS_Value, 0x5032032);
						}
						ZD1205_WRITE_REGISTER(Adapter,CWmin_CWmax, Adapter->CWmin_CWmax_NormalSlot);
						Adapter->SelfCTSOn = 0;
						//SleepResetDevice(Adapter);
						//--

						//Discard the IsUSB2_0 limit
						Adapter->CardSetting.Info_SupportedRates[0] = 0x01;
						Adapter->CardSetting.Info_SupportedRates[1] = 0x04;
						Adapter->CardSetting.Info_SupportedRates[2] = 0x82;
						Adapter->CardSetting.Info_SupportedRates[3] = 0x84;
						Adapter->CardSetting.Info_SupportedRates[4] = 0x0b;
						Adapter->CardSetting.Info_SupportedRates[5] = 0x16;
						Adapter->CardSetting.Info_ExtSupportedRates[0] = 0x32;
						Adapter->CardSetting.Info_ExtSupportedRates[1] = 0x08;
						Adapter->CardSetting.Info_ExtSupportedRates[2] = 0x0C;
						Adapter->CardSetting.Info_ExtSupportedRates[3] = 0x12;
						Adapter->CardSetting.Info_ExtSupportedRates[4] = 0x18;
						Adapter->CardSetting.Info_ExtSupportedRates[5] = 0x24;
						Adapter->CardSetting.Info_ExtSupportedRates[6] = 0x30;
						Adapter->CardSetting.Info_ExtSupportedRates[7] = 0x48;
						Adapter->CardSetting.Info_ExtSupportedRates[8] = 0x60;
						Adapter->CardSetting.Info_ExtSupportedRates[9] = 0x6c;

						Adapter->IsUSB2_0 = 1;

						UpdateBSSID(Adapter, &IBSS_BSSID[0]);
						Adapter->adapterReady = TRUE;
						ZDDbgPrint((_T("Adapter Connect! - 6\n")));
						break;
					case AP_BSS: // may run here when switch from STA to AP
                        Adapter->Notification = 0;
						Adapter->Notification |= RQ_CONNECTNETWORK;
						goto DoNotLikeGoTo;
					default:
						ASSERT(0);
						break;
					}				
                }

 				if ( Adapter->MoreConnect )
				{
					Adapter->MoreConnect--;
					if (Adapter->adapterReady) Adapter->adapterReady=FALSE;
				}
				else
				{
				    Adapter->Notification &= ~RQ_CONNECTNETWORK;
				    if ( Adapter->adapterReady ){
						DbgPrint("connected\r\n");
		              //SetThreadPriority(Adapter->h_ZD1211StartTxThread, THREAD_PRIORITY_NORMAL);
		              //SetThreadPriority(Adapter->h_ZD1211StartRxThread, THREAD_PRIORITY_NORMAL);
		              //SetThreadPriority(Adapter->h_ZD1205MgtThread, THREAD_PRIORITY_NORMAL);
	                  //Adapter->Infra_Process_Associate=FALSE;
					  if (!Adapter->Infra_Process_Associate) {
						  NdisMIndicateStatus(Adapter->ZD1205AdapterHandle,
						                    NDIS_STATUS_MEDIA_CONNECT,
						                    NULL,     // Not used.
						                    0);                 // Not used.
					      NdisMIndicateStatusComplete(Adapter->ZD1205AdapterHandle);
					  } else {
						  Adapter->Infra_Process_Associate=FALSE;
					  }
                    //JWEI 2004/02/09
                    //    ZD1205_WRITE_REGISTER(Adapter, LED1, 0x1);
                        iLED_ON(Adapter, Adapter->LinkLEDn);
                    #ifdef HOST_IF_USB
                        Adapter->LinkTimer = 0;
						if ( Adapter->LED_Ctrl_Type==1 )//chi_2004/08/10
							ZD1205_WRITE_REGISTER(Adapter, FW_LINK_STATUS, 0x0);
                        else if  ( Adapter->DataLED == 0 )     
#ifdef ROBIN_KAO
                            ZD1205_WRITE_REGISTER(Adapter, FW_LINK_STATUS, 0x3);
#else
                            ZD1205_WRITE_REGISTER(Adapter, FW_LINK_STATUS, 0x1);
#endif
                        else 
                            ZD1205_WRITE_REGISTER(Adapter, FW_LINK_STATUS, 0x0);
                    #endif
					}
				}	

				// Issue Notification
				//if ((Adapter->adapterReady == TRUE) && (Adapter->PreState == FALSE)){
				//	Adapter->PreState = TRUE;
				//}
			}
			// Process RQ_BSSID_LIST_SCAN
			else if (Adapter->Notification & RQ_BSSID_LIST_SCAN){
				INITSTR((_T("RQ_BSSID_LIST_SCAN..\n")));

                if ( !Adapter->DisableTx )
				{
				    if ( Adapter->adapterReady )
					    ChannelScan(Adapter,Adapter->ScanAllChannel);
						//ChannelScan(Adapter,0);
				    else
				        ChannelScan(Adapter,1);

					Adapter->scantime = GetTickCount();

				}

				Adapter->Notification &= ~RQ_BSSID_LIST_SCAN;
			}
			// Process RQ_ZD_BSSID_LIST_SCAN
			// It is for ZDConfig
			else if (Adapter->Notification & RQ_ZD_BSSID_LIST_SCAN){
				//if ((!Adapter->adapterReady) && (!Adapter->DisableTx)){
				INITSTR((_T("RQ_ZD_BSSID_LIST_SCAN..\n")));
				if ((!Adapter->DisableTx)){
				    if ( Adapter->adapterReady )
					    //ChannelScan(Adapter,Adapter->ScanAllChannel);
						ChannelScan(Adapter, 1);
				    else
				        ChannelScan(Adapter, 1);

					Adapter->scantime = GetTickCount();
				}
				Adapter->Notification &= ~RQ_ZD_BSSID_LIST_SCAN;
			}			
			// Process RQ_INFORM_PS_CHANGE
			else if (Adapter->Notification & RQ_INFORM_PS_CHANGE){
				Adapter->Notification &= ~RQ_INFORM_PS_CHANGE;
				// We issue a NULL Data frame (with PwrMgt set) to inform AP we are now 
				// in power-save mode.
				IssueNullData(Adapter);
			}
			// Process RQ_PSPOLL
			else if (Adapter->Notification & RQ_PSPOLL){
				Adapter->Notification &= ~RQ_PSPOLL;
				if (!Adapter->DisableTx)
					IssuePsPoll(Adapter);
			}
			// Process received management frame
			else if (Mgt_RetrieveQueue(Adapter, &DefragEntityPtr))
			{
				if (QueueGetCount(&DefragEntityPtr->ElementList) == 1)
				{
					// jxiaoDbg
					//DbgPrint(" ElementList == 1\n");
					if (Adapter->PSMSupported && Adapter->PwrState && 
						(!Adapter->DeviceInSleep) && (Adapter->adapterReady) &&
						(Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS))
					{
						// Solve Sequence number duplication problem after wakeup.
						if ((!AnyActivity(Adapter, DefragEntityPtr)) &&
							(QueueEmpty(&Adapter->MgtList)))
						{
							// solve ping loss problem
							//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
							ZDDbg((_T("Get SpinLock\n")));
							ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
							Adapter->SleepProtect = TRUE;
							//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
							ZDDbg((_T("---------->>> Release SpinLock\n")));
							ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
							if ((Adapter->SequenceNum != 1) || (IdleLoop_Under_Seq1 > 20))
							{
#ifdef CE_ALL
								timer = GetTickCount();
						        if ( (timer - Adapter->TxTimer) > 20 ) // delay 20 ms
#else
								timer = KeQueryInterruptTime();
						        if ( (timer - Adapter->TxTimer) > 200000 ) // delay 20 ms
#endif
								{
									INITSTR((_T("Prepare to enter sleep mode\n")));
									GetReturnedMgtFrame(Adapter, DefragEntityPtr);
									SleepResetDevice(Adapter);
									IdleLoop_Under_Seq1 = 0;
									// Avoid accessing Registers to save computation power.
									break; // Exit while checking loop.
								}
							}
							else
							{
								IdleLoop_Under_Seq1 ++;
							}
							Adapter->SleepProtect = FALSE;
						}
					}

					switch(ParseRxMgtFrame(Adapter, DefragEntityPtr)){
					case DEAUTHENTICATION:
						if (Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS){
							INITSTR((_T("Rx DeAuthentication\n")));
							DbgPrint("Rx DeAuthentication\r\n");
							//wpa2
							Adapter->PkInstalled = 0;
							Adapter->RxDeAuthCnt++;
							Mgt_SendNotification(Adapter, RQ_DISCONNECT);
						}
						break;
					case DISASSOCIATION:
						if (Adapter->CardSetting.BSSType == INFRASTRUCTURE_BSS){
							INITSTR((_T("Rx DisAssociation\n")));
							Adapter->PkInstalled = 0;
							Adapter->RxDisAssoCnt++;
							Mgt_SendNotification(Adapter, RQ_DISCONNECT);
						}
						break;
					case PROBEREQUEST:
						if ((Adapter->CardSetting.BSSType == INDEPENDENT_BSS) &&
							(!Adapter->DisableTx)){
							ProbeResponse(Adapter, DefragEntityPtr);
						}
						break;
					default:
						break;
					}

					if (Adapter->TestFlag & LOOPBACKTEST){
						AccLBRxCnt(Adapter, DefragEntityPtr);
					}
				}
				GetReturnedMgtFrame(Adapter, DefragEntityPtr);
			}
			// Process Loopback-Throughput-Test
            else if (Adapter->Notification & RQ_LBTEST)
            {
            //JWEI 2004/03/22
                if (Mgt_SendTestFrame(Adapter, cLBTEST_PATN) == FALSE)
                    Adapter->Notification &= ~RQ_LBTEST;
			}
			else if (Adapter->Notification & RQ_TKIPDBG){
				//MIC BUG
							
#ifdef HOST_IF_USB
				USB_StopRxEP(Adapter);
#endif
/*						
				ZDDbgPrint((_T(" Dump debug fifo\n")));
				ZD1205_WRITE_REGISTER(Adapter,0x804, 0x200);
				ZD1205_READ_REGISTER(Adapter,0x804, &tmpvalue);
				tmpvalue >>= 16;
				ZDDbgPrint((_T("length = %x\n"),tmpvalue));
				for (i=0;i<tmpvalue;i++){
					ZD1205_READ_REGISTER(Adapter,0x800, &tmpvalue1);
					ZDDbgPrint((_T("%8x  "), tmpvalue1));
				}
*/
				ASSERT(0);

				Adapter->Notification &= ~RQ_TKIPDBG;
			}
#ifdef HOST_IF_USB
			else if (Adapter->Notification & RQ_RESTART_TXRXEP){
    
				NdisMSleep(1000 * Adapter->TxRxEPRestartWaitTime);

				if (!Adapter->SurpriseRemoved){
					//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
					ZDDbg((_T("Get SpinLock\n")));
					ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
						Adapter->TxRxEPRestart++;
						USBWARNSTR((_T("TxRxEPRestart = %d\n"), Adapter->TxRxEPRestart));
						USB_StartRxEP(Adapter);
						USB_StartTxEP(Adapter);
					//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
					ZDDbg((_T("---------->>> Release SpinLock\n")));
					ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
				}
				
				Adapter->Notification &= ~RQ_RESTART_TXRXEP;
			}
#endif
			/*
			else if (Adapter->Notification & RQ_ATIM_WND_END){
				Adapter->Notification &= ~RQ_ATIM_WND_END;
				if (Adapter->PwrState && 
					(!Adapter->DeviceInSleep) && (Adapter->adapterReady) &&
					(Adapter->CardSetting.BSSType == INDEPENDENT_BSS)){
					if (!Adapter->FrmRxed1){
						// Any frame wait for transmission.
						if (QueueEmpty(&Adapter->ActiveChainList)){
							SleepResetDevice(Adapter);
							break;	// Avoid accessing Registers to save computation power.
						}
					}
					Adapter->FrmRxed1 = FALSE;
				}
			}
			*/
			// We have processed all possible events, go to wait event now.
			else{
				Adapter->Notification = 0;
				break;
			}
		}
	}

	return 0;
}

//-----------------------------------------------------------------------------
// Procedure:   ZD1205Initialize/ZD1205AddAdapter
//
// Description: This routine is called once per each supported adapter card in
//              the system.  This routine is responsible for initializing each
//              adapter.  This includes parsing all of the necessary parameters
//              from the registry, allocating and initializing shared memory
//              structures, configuring the ZD1205 chip, registering the
//              interrupt, and starting the receive unit.
//
// Arguments:
//      OpenErrorStatus (mini) - Returns more info about any failure
//      SelectedMediumIndex (mini) - Returns the index in MediumArray of the
//                                   medium that the miniport is using
//      MediumArraySize (mini) - An array of medium types that the driver
//                               supports
//      MiniportAdapterHandle (mini) - pointer to the adapter object data area.
//
//      WrapperConfigurationContext (both) - A value that we will pass to
//                                           NdisOpenConfiguration.
//
//
// Returns:
//      NDIS_STATUS_SUCCESS - If the adapter was initialized successfully.
//      <not NDIS_STATUS_SUCCESS> - If for some reason the adapter didn't
//                                  initialize
//-----------------------------------------------------------------------------
PZD1205_ADAPTER		ZD1205Adapter;
NDIS_STATUS
ZD1205Initialize(PNDIS_STATUS OpenErrorStatus,
               PUINT SelectedMediumIndex,
               PNDIS_MEDIUM MediumArray,
               UINT MediumArraySize,
               NDIS_HANDLE MiniportAdapterHandle,
               NDIS_HANDLE WrapperConfigurationContext)
{
    ULONG               i, jj;
    NDIS_STATUS         Status;
    PZD1205_ADAPTER       Adapter;
    //NDIS_HANDLE         ConfigHandle;
    NDIS_INTERFACE_TYPE IfType;
	ULONG               tmpvalue;
	UINT                size=0;

    DEBUGFUNC(_T("ZD1205Initialize"));
    INITSTR((_T("\n")));
	
    // If this is a miniport, then fill in the media information.

    // if medium type 802.3 not found in list, exit with error)
    for (i = 0; i < MediumArraySize; i++)
    {
        if (MediumArray[i] == NdisMedium802_3) break;
    }

    if (i == MediumArraySize)
    {
        DEBUGSTR((_T("802.3 Media type not found.\n")));
        return NDIS_STATUS_UNSUPPORTED_MEDIA;
    }

    // Select ethernet
    *SelectedMediumIndex = i;

    // Allocate the Adapter Object, exit if error occurs
	// Non-Paged memory
	size = sizeof(ZD1205_ADAPTER);
    Status = ZD1205_ALLOC_MEM(&Adapter, size);

    INITSTR((_T("sizeof(ZD1205_ADAPTER) = %d\n"), size));

    if (Status != NDIS_STATUS_SUCCESS)
    {
        DEBUGSTR((_T("ADAPTER Allocate Memory failed (Status = 0x%x)\n"), Status));
        return Status;
    }
	
	ZD1205Adapter = Adapter;
    //Zero out the adapter object space
    NdisZeroMemory(Adapter, size);
   
    Adapter->ZD1205AdapterHandle = MiniportAdapterHandle;
	Adapter->ZD1205WrapperConfigurationContext = WrapperConfigurationContext;
    
    Adapter->USBInfo.hDevice = g_ZD1211USBInfo.hDevice;
	Adapter->USBInfo.lpUsbFuncs = g_ZD1211USBInfo.lpUsbFuncs;
    Adapter->USBInfo.lpDriverSettings = g_ZD1211USBInfo.lpDriverSettings;
	Adapter->USBInfo.lpActiveConfig = g_ZD1211USBInfo.lpActiveConfig;
    Adapter->USBInfo.lpDeviceInfo = g_ZD1211USBInfo.lpDeviceInfo;
    

    INITSTR((_T("Adapter structure pointer is %08x\n"), Adapter));
	// Save to Global AdpterInfo Structure.
	for (i=0;i<MAX_SUPPORT_DEVICE_NUM;i++){
		if (gAdapterInfo[i].AdapterActive == FALSE){
			gAdapterInfo[i].AdapterActive = TRUE;
			gAdapterInfo[i].AdapterContext = Adapter;
			gAdapterInfo[i].AdapterHandle = MiniportAdapterHandle;
			break;
		}
	}
	if (i==MAX_SUPPORT_DEVICE_NUM){
		// No more space
		ZDDbgPrint((_T("No more global AdapterInfo space\n")));
        FreeAdapterObject(Adapter);
        return NDIS_STATUS_FAILURE;
	}

    Adapter->bUSBDeveiceAttached = TRUE;

    // Open Registry, exit if error occurs.

    //NdisOpenConfiguration(&Status,
    //    &ConfigHandle,
    //    WrapperConfigurationContext);
    //if (Status != NDIS_STATUS_SUCCESS)
    //{
    //    DEBUGSTR((_T("NdisOpenConfiguration failed (Status = 0x%x)\n"), Status));
    //    FreeAdapterObject(Adapter);
    //    return NDIS_STATUS_FAILURE;
    //}
    
    // Parse all of our configuration parameters.
    //Status = ParseRegistryParameters(Adapter, ConfigHandle);

    // If a required configuration parameter was not present, then error out
    //if (Status != NDIS_STATUS_SUCCESS)
    //{
    //    DEBUGSTR((_T("ParseRegistryParameters failed (Status = 0x%x)\n"), Status));
    //    NdisCloseConfiguration(ConfigHandle);
    //    FreeAdapterObject(Adapter);
    //    return Status;
    //}
	// get the default connection

    //DEBUGSTR((_T("ParseRegistryParameters Completed successfully\n")));

    // Look for a Node Address (IA) override
    // this is where the registry parameter NodeAddress in
    // HKLM\System\CCS\Services\E100BX\Parameters

    //NdisReadNetworkAddress(&Status,
    //    &OverrideNetAddress,
    //    (UINT *) &i,
    //    ConfigHandle);

    // If there is an IA override save it to the adapter object
    //if ((i == ETH_LENGTH_OF_ADDRESS) && (Status == NDIS_STATUS_SUCCESS))
    //    NdisMoveMemory(Adapter->AiNodeAddress,
    //    OverrideNetAddress,
    //    ETH_LENGTH_OF_ADDRESS);

    // We read out all of our config info, so close the gateway to the registry
    //NdisCloseConfiguration(ConfigHandle);
    
    Adapter->NumTcb = 16;
	Adapter->NumRfd = 16;
	Adapter->NumCoalesce = 16;
	Adapter->NumTbdPerTcb = 4;

    IfType = NdisInterfaceInternal;
    
	NdisMSetAttributesEx(
        Adapter->ZD1205AdapterHandle,
        (NDIS_HANDLE) Adapter,
        10,
        (ULONG)
        NDIS_ATTRIBUTE_DESERIALIZE |
		//NDIS_ATTRIBUTE_SURPRISE_REMOVE_OK |
		0,
        IfType
        );

    Adapter->hReqInfoEvent = CreateEvent(NULL, FALSE, FALSE, _T("ZD1211ReqInfoEvent"));

    if ( Adapter->hReqInfoEvent == NULL )
	{
		FreeAdapterObject(Adapter);
	    return NDIS_STATUS_FAILURE;
	}

    Adapter->hReqCompleteEvent = CreateEvent(NULL, FALSE, FALSE, _T("ZD1211ReqComplete"));

    if ( Adapter->hReqCompleteEvent == NULL )
	{
	    CloseHandle(Adapter->hReqInfoEvent);
		FreeAdapterObject(Adapter);
	    return NDIS_STATUS_FAILURE;
	}

    Adapter->hReqInfoFile = CreateFileMapping( INVALID_HANDLE_VALUE,
		                                       NULL,
											   PAGE_READWRITE,
											   0,
											   4096,
                                               _T("ZD1211ReqInfoFile")
		                                       );

    if ( Adapter->hReqInfoFile == NULL )
	{
	    CloseHandle(Adapter->hReqInfoEvent);
		CloseHandle(Adapter->hReqCompleteEvent);
        FreeAdapterObject(Adapter);
	    return NDIS_STATUS_FAILURE;
	}

    Adapter->ReqInfoMap = (PUCHAR) MapViewOfFile( Adapter->hReqInfoFile,
		                                          FILE_MAP_READ | FILE_MAP_WRITE,
												  0,
												  0,
												  0
		                                          );

	if ( Adapter->ReqInfoMap == NULL )
	{
	    CloseHandle(Adapter->hReqInfoEvent);
		CloseHandle(Adapter->hReqCompleteEvent);
		CloseHandle(Adapter->hReqInfoFile);
        FreeAdapterObject(Adapter);
	    return NDIS_STATUS_FAILURE;
	}

	Adapter->hReqInfoFileMutex = CreateMutex(NULL, FALSE, NULL);

	if ( Adapter->hReqInfoFileMutex == NULL )
	{
	    CloseHandle(Adapter->hReqInfoEvent);
		CloseHandle(Adapter->hReqCompleteEvent);
		UnmapViewOfFile(Adapter->ReqInfoMap);
		CloseHandle(Adapter->hReqInfoFile);
        FreeAdapterObject(Adapter);
	    return NDIS_STATUS_FAILURE;	
	}

    //
    // Now to get Lower Device Object, it is USB host
    //              
    //NdisMGetDeviceProperty( MiniportAdapterHandle,  //MiniportAdapterHandle 
    //                        NULL,//PhysicalDeviceObject 
    //                        NULL,//FunctionalDeviceObject 
    //                        &Adapter->pLowerDeviceObject,//NextDeviceObject 
    //                        NULL,//AllocatedResources 
    //                        NULL//AllocatedResourcesTranslated 
    //                        );

	NdisAllocateSpinLock(&Adapter->Lock);
	//Adapter->Lock = CreateEvent(NULL, FALSE, TRUE,NULL);
 

	// Assign (Claim) a physical Adapter for this Adapter object
    // In this function is where we find our adapter on the PCI bus and
    // call NdisMPciAssignResources
    if (ClaimAdapter(Adapter) != NDIS_STATUS_SUCCESS)
    {
        DEBUGSTR((_T("No adapter detected\n")));
        FreeAdapterObject(Adapter);
        return NDIS_STATUS_FAILURE;
    }


    // allocate & Initialize the required (ZD1205) Shared Memory areas
    Status = SetupSharedAdapterMemory(Adapter);

    // Check the status returned from SetupSharedAdapterMemory
    if (Status != NDIS_STATUS_SUCCESS)
    {        // Since we couldn't allocate enough shared memory, free any resources
        // that we previously allocated and error out.
        //ZD1205LogError(Adapter, EVENT_10, NDIS_ERROR_CODE_OUT_OF_RESOURCES, 0);
        DEBUGSTR((_T("Shared Memory Allocation failed (Status = 0x%x)\n"), Status));

        // Free our adapter object
        FreeAdapterObject(Adapter);

        return NDIS_STATUS_FAILURE;
    }

    // Disable interrupts while we finish with the initialization
    // Must SetupSharedAdapterMemory() before you can do this
    // (fixed bug) must check for success of alloc sharedmem before
    // calling ZD1205DisableInterrupt.
    ZD1205DisableInterrupt(Adapter);

#ifdef HOST_IF_USB
	USB_CompleteCountIncrement(Adapter);
#endif
    Adapter->h_ZD1211InterruptThread = CreateThread( NULL,
		                                         0,
									             ZD1211InterruptThread,
									             Adapter,
									             0,
									             NULL
									             );
    if( Adapter->h_ZD1211InterruptThread != NULL )
	{
		DEBUGSTR((_T("****** h_ZD1211InterruptThread created successfully !\n")));
		//SetThreadPriority(Adapter->h_ZD1211InterruptThread, THREAD_PRIORITY_TIME_CRITICAL);
    }
    else
	{
#ifdef HOST_IF_USB
		USB_CompleteCountDecrement(Adapter);
#endif
		DEBUGSTR((_T("xxxxxx h_ZD1211InterruptThread failed to create !\n")));
        
		TerminateThreads(Adapter);
        FreeAdapterObject(Adapter);
        return NDIS_STATUS_FAILURE;
	}

	Mgt_QueueInit(Adapter);
    // Setup and initialize the transmit structures.
    SetupTransmitQueues(Adapter, TRUE);

    // configure the ZD1205 chip.
    if (!InitializeAdapter(Adapter, FALSE))
    {
        // Since the adapter failed to initialize, free any resources that
        // we previously allocated and error out.
        DEBUGSTR((_T("InitializeAdapter Failed.\n")));

        FreeAdapterObject(Adapter);
        return NDIS_STATUS_FAILURE;
    }

    // Setup and initialize the receive structures
    SetupReceiveQueues(Adapter);

    // Start the receive unit -- we can now receive packets off the wire
    StartReceiveUnit(Adapter);

	//AP_Initialize(Adapter, TRUE);
#ifdef NDIS50_MINIPORT
    // register a shutdown handler...
    NdisMRegisterAdapterShutdownHandler(Adapter->ZD1205AdapterHandle,
        (PVOID) Adapter,
        (ADAPTER_SHUTDOWN_HANDLER) ZD1205ShutdownHandler);
#endif 

    // Init a timer for use with our reset routine...
    NdisMInitializeTimer(&Adapter->ZD1205AsyncResetTimer,
        Adapter->ZD1205AdapterHandle,
        (PNDIS_TIMER_FUNCTION) ZD1205ResetComplete,
        (PVOID) Adapter);
	
	//for counter measure	
	NdisMInitializeTimer(
           &Adapter->OneSecTimer,
           Adapter->ZD1205AdapterHandle,
           CMPerSecTimer,
           (PVOID)Adapter);	
	NdisMSetPeriodicTimer(&Adapter->OneSecTimer, TIMER_PERIOD_FOR_ONE_SEC);

	NdisMInitializeTimer(
           &Adapter->AutoRateMonitorTimer,
           Adapter->ZD1205AdapterHandle,
           AutoRateTimer,
           (PVOID)Adapter);	
	//DbgPrint("AutoRateMonitorInterval=%u\n",Adapter->autoRateMonitorInterval); // Used by Adapter->AutoRateMonitorTimer
	NdisMSetPeriodicTimer(&Adapter->AutoRateMonitorTimer, Adapter->autoRateMonitorInterval);


    Adapter->bTerminateThreads = FALSE;

#ifdef HOST_IF_USB
	USB_CompleteCountIncrement(Adapter);
#endif

    Adapter->h_ZD1205MgtThread = CreateThread( NULL,
		                                       0,
									           ZD1205MgtThread,
									           Adapter,
									           0,
									           NULL
									           );

    if( Adapter->h_ZD1205MgtThread != NULL )
	{
		INITSTR((_T("Handle of ZD1205MgtThread = %x\n"), Adapter->h_ZD1205MgtThread));
		DEBUGSTR((_T("****** ZD1205MgtThread created successfully !\n")));
        SetThreadPriority(Adapter->h_ZD1205MgtThread, THREAD_PRIORITY_ABOVE_NORMAL);
    }
    else
	{
#ifdef HOST_IF_USB
		USB_CompleteCountDecrement(Adapter);
#endif
		DEBUGSTR((_T("xxxxxx ZD1205MgtThread failed to create !\n")));

        FreeAdapterObject(Adapter);
        return NDIS_STATUS_FAILURE;
	}

/*
//#if (defined(HOUSETHREADRUN) || !defined(HOST_IF_USB))
	// Create a house-keeping thread
#ifdef HOST_IF_USB
	USB_CompleteCountIncrement(Adapter);
#endif
    Adapter->h_ZD1205HouseThread = CreateThread( NULL,
		                                         0,
									             ZD1205HouseThread,
									             Adapter,
									             0,
									             NULL
									             );
    if( Adapter->h_ZD1205HouseThread != NULL )
	{
		DEBUGSTR((_T("****** ZD1205HouseThread created successfully !\n")));
    }
    else
	{
#ifdef HOST_IF_USB
		USB_CompleteCountDecrement(Adapter);
#endif
		DEBUGSTR((_T("xxxxxx ZD1205HouseThread failed to create !\n")));
        
		TerminateThreads(Adapter);
        FreeAdapterObject(Adapter);
        return NDIS_STATUS_FAILURE;
	}
*/

#ifdef HOST_IF_USB
	USB_CompleteCountIncrement(Adapter);
#endif
    Adapter->h_ZD1211SetQueryOIDThread = CreateThread( NULL,
													 0,
													 ZD1211SetQueryOIDThread,
													 Adapter,
													 0,
													 NULL
													 );
    if( Adapter->h_ZD1211SetQueryOIDThread != NULL )
	{
		DEBUGSTR((_T("****** ZD1211SetQueryOIDThread created successfully !\n")));
    }
    else
	{
#ifdef HOST_IF_USB
		USB_CompleteCountDecrement(Adapter);
#endif
		DEBUGSTR((_T("xxxxxx ZD1211SetQueryOIDThread failed to create !\n")));
        
		TerminateThreads(Adapter);
        FreeAdapterObject(Adapter);
        return NDIS_STATUS_FAILURE;
	}



#ifdef HOST_IF_USB
	USB_CompleteCountIncrement(Adapter);
#endif
    Adapter->h_ZD1211StartRxThread = CreateThread( NULL,
		                                         0,
									             ZD1211StartRxThread,
									             Adapter,
									             0,
									             NULL
									             );
    if( Adapter->h_ZD1211StartRxThread != NULL )
	{
		DEBUGSTR((_T("****** ZD1211StartTxThread created successfully !\n")));
		//SetThreadPriority(Adapter->h_ZD1211StartTxThread, THREAD_PRIORITY_TIME_CRITICAL);
		CeSetThreadPriority(Adapter->h_ZD1211StartRxThread, 251);
    }
    else
	{
#ifdef HOST_IF_USB
		USB_CompleteCountDecrement(Adapter);
#endif
		DEBUGSTR((_T("xxxxxx ZD1211StartRxThread failed to create !\n")));
        
		TerminateThreads(Adapter);
        FreeAdapterObject(Adapter);
        return NDIS_STATUS_FAILURE;
	}

/*
    // Create AP_MGT_Thread
#ifdef HOST_IF_USB
	USB_CompleteCountIncrement(Adapter);
#endif
   Adapter->h_ZD1211APMgtThread = CreateThread( NULL,
		                                         0,
									             ZD1211APMgtThread,
									             Adapter,
									             0,
									             NULL
									             );
    if( Adapter->h_ZD1211APMgtThread != NULL ){
		DEBUGSTR((_T("****** ZD1211APMgtThread create successfully !\n")));
    }
    else{
#ifdef HOST_IF_USB
		USB_CompleteCountDecrement(Adapter);
#endif
		DEBUGSTR((_T("xxxxxx ZD1211APMgtThread create fail !\n")));

		// Terminate Mgt thread		
		Mgt_SendNotification(Adapter, RQ_TERMINATION);
		while(Adapter->h_ZD1205MgtThread){
			ZD1205_Sleep(10000);
		}

	    Adapter->TerminateHouseThread = TRUE;
	    while(Adapter->h_ZD1205HouseThread){
		    ZD1205_Sleep(10000);
		}

        FreeAdapterObject(Adapter);
        return NDIS_STATUS_FAILURE;
	}
#endif

*/
    // Enable board interrupts
    ZD1205EnableInterrupt(Adapter);

	NdisMIndicateStatus(Adapter->ZD1205AdapterHandle,
		NDIS_STATUS_MEDIA_DISCONNECT,
		NULL,     // Not used.
		0);                 // Not used.
	NdisMIndicateStatusComplete(Adapter->ZD1205AdapterHandle);
	Mgt_SendNotification(Adapter, RQ_BSSID_LIST_SCAN);
    DEBUGSTR((_T("ZD1205Initialize: Completed Init Successfully\n")));

	//GetDefaultConnection(Adapter, ConfigHandle);

	Adapter->WPAOn = 0;
	Adapter->ZDConfigRunning = 0;
    Adapter->NotCheckConnect = 0;
	Adapter->MoreConnect = 0;
	Adapter->MulticastAddr[0] = 0;
	Adapter->CheckCTS = 1;
	//Adapter->CardSetting.FragThreshold = 300;
	//set preamble default mode in inf
	//Adapter->preambleMode = 3; // auto
	
	ClearOppositeRateInfo(Adapter);
	Adapter->TraceSetPoint = 1;
    Adapter->IntValueLevel = 0;
    Adapter->SetPointLevel = 0;
	// Patrick 6.28 2004, determine if need to use TxPwrCtrl
	{
        ZD1205_READ_REGISTER(Adapter, 0x904, &tmpvalue);
		
		if (((tmpvalue >> 16) & 0xF) == 1){
			Adapter->EnableTxPwrCtrl = FALSE;
			ZDDbgPrint((_T("Tx Power Control ----- Disable!!\n")));
		}
		else{
			Adapter->EnableTxPwrCtrl = TRUE;
			ZDDbgPrint((_T("Tx Power Control ----- Enable!!\n")));
		}
	}
	if (Adapter->ReducePowerLevel != 0)
	{// Advanced Page Override the e2prom's setting
		Adapter->EnableTxPwrCtrl = FALSE;
		Adapter->IntValueLevel = Adapter->ReducePowerLevel;
	}
	Adapter->PSThreshhold = 10000;
    Adapter->WorseSQThr = 0x48;
	// read Set Point from EEPROM
	//ZD1205_WRITE_REGISTER(Adapter,ROMDIR, 0x424);
    JWeiDbgPrint3(0, ("\n**********************************\n"));
    for (i = 0; i < 4; i++)
    {
        ZD1205_READ_REGISTER(Adapter, E2P_PWR_CAL_VALUE + i * 4, &tmpvalue);
        JWeiDbgPrint3(1, (" <SetPoint%d = 0x%02X> ", i, tmpvalue));
        Adapter->SetPoint[i * 4] = (UCHAR) tmpvalue;
        Adapter->SetPoint[i * 4 + 1] = (UCHAR) (tmpvalue >> 8);
        if (i != 3)
        {
            Adapter->SetPoint[i * 4 + 2] = (UCHAR) (tmpvalue >> 16);
            Adapter->SetPoint[i * 4 + 3] = (UCHAR) (tmpvalue >> 24);
        }
        ZD1205_READ_REGISTER(Adapter, E2P_PWR_INT_VALUE + i * 4, &tmpvalue);
        JWeiDbgPrint3(1, (" <IntValue%d = 0x%02X,", i, tmpvalue));
    //``JWEI 2004/05/19
		if (Adapter->EnableTxPwrCtrl)
		{
			Adapter->IntValue[i * 4 + 0] = (UCHAR) tmpvalue-Adapter->nPwrIntValueGuard;
			Adapter->IntValue[i * 4 + 1] = (UCHAR) (tmpvalue >> 8)-Adapter->nPwrIntValueGuard;
			if (i != 3)
			{
				Adapter->IntValue[i * 4 + 2] = (UCHAR) (tmpvalue >> 16)-Adapter->nPwrIntValueGuard;
				Adapter->IntValue[i * 4 + 3] = (UCHAR) (tmpvalue >> 24)-Adapter->nPwrIntValueGuard;
			}
		}
		else{
			Adapter->IntValue[i * 4] = (UCHAR) tmpvalue;
			Adapter->IntValue[i * 4 + 1] = (UCHAR) (tmpvalue >> 8);
			if (i != 3)
			{
				Adapter->IntValue[i * 4 + 2] = (UCHAR) (tmpvalue >> 16);
				Adapter->IntValue[i * 4 + 3] = (UCHAR) (tmpvalue >> 24);
			}
        }
	}

//JWEI 2003/12/31
	#if fTX_PWR_CTRL
	for (jj = 0; jj < 3; jj ++)
	{//jj=0:36MSetPoint, jj=1:48MSetPoint, jj=2:54MSetPoint
		for (i = 0; i < 4; i++)
		{
			ZD1205_READ_REGISTER(Adapter, E2P_36M_CAL_VALUE + jj * 0x20 + i * 4, &tmpvalue);
			JWeiDbgPrint3(1, (" <OFDM%d = 0x%02X> ", jj, tmpvalue));
			Adapter->SetPointOFDM[jj][i * 4] = (UCHAR) tmpvalue;
			Adapter->SetPointOFDM[jj][i * 4 + 1] = (UCHAR) (tmpvalue >> 8);
			if (i != 3)
			{
				Adapter->SetPointOFDM[jj][i * 4 + 2] = (UCHAR) (tmpvalue >> 16);
				Adapter->SetPointOFDM[jj][i * 4 + 3] = (UCHAR) (tmpvalue >> 24);
			}
		}
	}
	#endif



    return NDIS_STATUS_SUCCESS;
}


//-----------------------------------------------------------------------------
// Procedure:   ZD1205Reset
//
// Description: Instructs the Miniport to issue a hardware reset to the
//              network adapter.  The driver also resets its software state.
//              this function also resets the transmit queues.
//
// Arguments:
//      AddressingReset - TRUE if the wrapper needs to call
//                        MiniportSetInformation to restore the addressing
//                        information to the current values
//      MiniportAdapterContext - pointer to the adapter object data area.
//
// Returns:
//      NDIS_STATUS_PENDING - This function sets a timer to complete, so
//                            pending is always returned
//-----------------------------------------------------------------------------

NDIS_STATUS
ZD1205Reset(PBOOLEAN AddressingReset,
          NDIS_HANDLE MiniportAdapterContext)
{
    PZD1205_ADAPTER Adapter;
	ZD1211_IRQL		OldIrql;
    DEBUGFUNC(_T("ZD1205Reset"));
    INITSTR((_T("\n")));

    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);

#ifdef HOST_IF_USB
    *AddressingReset = TRUE;

	USB_CompleteCountIncrement(Adapter);
	
	Adapter->bUSBDeveiceResetting = TRUE;

    NdisScheduleWorkItem(&Adapter->WorkItemReset);
   
	return(NDIS_STATUS_PENDING);
#endif

	Adapter->PSMSupported = FALSE;

    //KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

    *AddressingReset = TRUE;

    // *** possible temporary code
    // *** NDIS may actually handle this
    //Adapter->ResetInProgress = TRUE;

    // Disable interrupts while we re-init the transmit structures
    ZD1205DisableInterrupt(Adapter);

    // The NDIS 5 support for deserialized miniports requires that
    // when reset is called, the driver de-queue and fail all uncompleted
    // sends, and complete any uncompleted sends. Essentially we must have
    // no pending send requests left when we leave this routine.


	// Let Device go to sleep, so that device bus master will not action any more.
	//ZD1205_WRITE_REGISTER(Adapter,Sleep_Ctrl, 0x1);

    // we will fail all sends that we have left right now.
    DEBUGSTR((_T("DeQing: ")));
    while(Adapter->FirstTxQueue)
    {
        PNDIS_PACKET QueuePacket = Adapter->FirstTxQueue;

        Adapter->NumPacketsQueued--;
        DequeuePacket(Adapter->FirstTxQueue, Adapter->LastTxQueue);

        // we must release the lock here before returning control to ndis
        // (even temporarily like this)
		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
        NdisMSendComplete(
            Adapter->ZD1205AdapterHandle,
            QueuePacket,
			NDIS_STATUS_SUCCESS);
            //NDIS_STATUS_FAILURE);

		//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
    }
    DEBUGSTR((_T("\nDone!\n")));

    // clean up all the packets we have successfully TX'd
    ProcessTXInterrupt(Adapter, &OldIrql);

    while (!(QueueEmpty(&Adapter->ActiveChainList)))
    {
        // here we have to fail any sends that haven't completed yet
        PZD1205SwTcb pSwTcb = (PZD1205SwTcb) QueuePopHead(&Adapter->ActiveChainList);

		if (pSwTcb->LastFrag){
			ProviderMSendComplete(Adapter, pSwTcb, &OldIrql);
		}
    }

    APMgt_SendNotification(Adapter, BIT_2);


    // Clear out our software transmit structures
    NdisZeroMemory((PVOID) Adapter->XmitCached, Adapter->XmitCachedSize);

    // re-init the map register related variables
#ifndef HOST_IF_USB
    Adapter->NextFreeMapReg = 0;
    Adapter->OldestUsedMapReg = 0;
#endif

    // re-initialize the transmit structures
    DEBUGSTR((_T("ZD1205Reset: Calling SetupTransmitQueues\n")));
    SetupTransmitQueues(Adapter, FALSE);

	// Sleep Reset Device
	ResetDevice(Adapter);

    // set a timer to call us back so
    // that we don't try to hold a spinlock too long
    // delay 500 ms
    NdisMSetTimer(&Adapter->ZD1205AsyncResetTimer,500);

	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

    // return status_pending so that we can finish this later
    return(NDIS_STATUS_PENDING);
}

//-----------------------------------------------------------------------------
// ZD1205ResetComplete
//
// PARAMETERS: NDIS_HANDLE MiniportAdapterContext
//
// DESCRIPTION: This function is called by a timer indicating our
//              reset is done (by way of .5 seconds expiring)
//
// RETURNS: nothing, but sets NdisMResetComplete, enables ints
//          and starts the receive unit
//
//-----------------------------------------------------------------------------

VOID
ZD1205ResetComplete(PVOID sysspiff1,
                  NDIS_HANDLE MiniportAdapterContext,
                  PVOID sysspiff2, PVOID sysspiff3)
{
    PZD1205_ADAPTER Adapter;
    DEBUGFUNC(_T("ZD1205ResetComplete"));

    INITSTR((_T("\n")));
    Adapter = PZD1205_ADAPTER_FROM_CONTEXT_HANDLE(MiniportAdapterContext);


    NdisMResetComplete(Adapter->ZD1205AdapterHandle,
        (NDIS_STATUS) NDIS_STATUS_SUCCESS,
        FALSE);

#ifndef HOST_IF_USB
    ZD1205EnableInterrupt(Adapter);
#endif

    return;
}
//-----------------------------------------------------------------------------
// Procedure: ZD1205AllocateComplete
//
// Description: This function handles initialization of new receive memory
//              when the os returns some shared memory to us because we
//              called NdisMAllocateSharedMemoryAsync when we ran low on
//              receive buffers.
//
// Arguments: MiniportAdapterContext - a pointer to our adapter structure
//            VirtualAddress - The virtual address of the new memory
//            PhysicalAddress - _pointer to_ The physical address of the
//                              new memory
//            Length - The length of the new memory
//            Context - The offset into our MemoryDescriptor array to
//                      initialize (zero-based)
//
// Returns: Nothing
//
//-----------------------------------------------------------------------------

VOID
ZD1205AllocateComplete(NDIS_HANDLE MiniportAdapterContext,
                     IN PVOID VirtualAddress,
                     IN PNDIS_PHYSICAL_ADDRESS PhysicalAddress,
                     IN ULONG Length,
                     IN PVOID Context)
{
    DEBUGFUNC(_T("ZD1205AllocateComplete"));
    INITSTR((_T("\n")));
    return;
}

/*++

Routine Description:
    
    Timer function for postponed link negotiation
    
Arguments:

    SystemSpecific1     Not used
    FunctionContext     Pointer to our adapter
    SystemSpecific2     Not used
    SystemSpecific3     Not used

Return Value:

    None
    
--*/
VOID 
LinkDetectionDpc(
    IN  PVOID	SystemSpecific1,
    IN  PVOID	FunctionContext,
    IN  PVOID	SystemSpecific2, 
    IN  PVOID	    SystemSpecific3
    )
{
    PZD1205_ADAPTER Adapter = (PZD1205_ADAPTER)FunctionContext;


    if (Adapter->bQueryPending)
    {
        
        switch(Adapter->QueryRequest.Oid)
        {
            case OID_GEN_LINK_SPEED:
            #if !(defined(GCCK) && defined(OFDM))
                if (Adapter->adapterReady){
                    *((PULONG) Adapter->QueryRequest.InformationBuffer) = 110000;
                }
                else{
                    *((PULONG) Adapter->QueryRequest.InformationBuffer) = 110000;
                }
            #else                           //``JWEI 2003/12/02
           		*((PULONG) Adapter->QueryRequest.InformationBuffer) = 
					Rate_Convert_to_MS(Adapter->CardSetting.LastSentTxRate) * 10000 / 2;
				//*((PULONG) Adapter->QueryRequest.InformationBuffer) = 540000;
            #endif
                *((PULONG) Adapter->QueryRequest.BytesWritten) = sizeof(ULONG);

                break;

            case OID_GEN_MEDIA_CONNECT_STATUS:
            default:
                ASSERT(Adapter->QueryRequest.Oid == OID_GEN_MEDIA_CONNECT_STATUS);
				if (Adapter->adapterReady){
					*((PULONG) Adapter->QueryRequest.InformationBuffer) = NdisMediaStateConnected; 
				}
				else{
					*((PULONG) Adapter->QueryRequest.InformationBuffer) = NdisMediaStateDisconnected; 
				}
                *((PULONG) Adapter->QueryRequest.BytesWritten) = sizeof(ULONG);
        }

        Adapter->bQueryPending = FALSE;
        NdisMQueryInformationComplete(Adapter->ZD1205AdapterHandle, NDIS_STATUS_SUCCESS);
    }
	return;
}


VOID 
CMPerSecTimer(
    IN  PVOID	SystemSpecific1,
    IN  PVOID	FunctionContext,
    IN  PVOID	SystemSpecific2, 
    IN  PVOID	    SystemSpecific3
    )
{
    PZD1205_ADAPTER Adapter = (PZD1205_ADAPTER)FunctionContext;
    	
	if (Adapter->MICFailureTimer)
	{
		Adapter->MICFailureTimer--;
		if (Adapter->MICFailureTimer==0)
		{
			Adapter->MICFailureCount=0;
			ZDDbgPrint((_T("MICFailureTimer Expires!\n")));
		}
	}
		
	if (Adapter->DisallowAssociateTimer)
	{
		Adapter->DisallowAssociateTimer--;
		//Adapter->DisableTx = TRUE;
		if (Adapter->DisallowAssociateTimer==0)
		{
			Adapter->DisableTx = FALSE;
			ZDDbgPrint((_T("DisallowAssociateTimer Expires!\n")));
		}
	}

	if (Adapter->DisconnectTimer){
	    Adapter->DisconnectTimer--;
		ZDDbgPrint((_T("DisconnectTimer--!\n")));
		if (!Adapter->DisconnectTimer){
            ZDDbgPrint((_T("DisconnectTimer Expires!\n")));
			Adapter->AttackedSsidLen=Adapter->CardSetting.Info_SSID[1];
			NdisMoveMemory( Adapter->AttackedSsid, 
							&Adapter->CardSetting.Info_SSID[2],
							Adapter->CardSetting.Info_SSID[1]);
			Adapter->DisallowAssociateTimer=60;
            Adapter->ReasonCode = 14;	// MIC failure	 
			Adapter->WepStatus=0;
			Adapter->PkInstalled = 0;
			Adapter->GkInstalled = 0;
			Mgt_SendNotification(Adapter, RQ_DEAUTHENTICATION); 
			Mgt_SendNotification(Adapter, RQ_DISCONNECT);		    
		}
	}


}


//for counter measure
void ReportAuthEvent(PZD1205_ADAPTER	Adapter,
					ULONG	Flags)
{
	//NDIS_802_11_AUTHENTICATION_REQUEST	AuthRequest;
	UCHAR	statusBuffer[sizeof(NDIS_802_11_AUTHENTICATION_REQUEST)+sizeof(NDIS_802_11_STATUS_TYPE)];
	NDIS_STATUS	GeneralStatus=NDIS_STATUS_MEDIA_SPECIFIC_INDICATION;
	PNDIS_802_11_STATUS_INDICATION	pNdis80211StatusIndication=(PNDIS_802_11_STATUS_INDICATION)statusBuffer;
	PNDIS_802_11_AUTHENTICATION_REQUEST pNdis80211AuthRequest;
	if (Flags == NDIS_802_11_AUTH_REQUEST_GROUP_ERROR ||
		Flags == NDIS_802_11_AUTH_REQUEST_PAIRWISE_ERROR)
	{
				Adapter->MICFailureCount++;
				if (Adapter->MICFailureCount==1) // The first MIC failure.
				{
					ZDDbgPrint((_T("*****First MIC Failure occur\n")));
					Adapter->MICFailureTimer=60; // Init countermeasures timer.
				}
				else if (Adapter->MICFailureCount==2)
				{
					//USHORT	CmdParam[3];
					ZDDbgPrint((_T("*****2nd MIC Failure occur within 60 seconds\n")));
				//	if (Adapter->MICFailureTimer != 0) // Less than 60 seconds have passed since a previous Michael MIC failure. 
				//	{
						Adapter->DisconnectTimer = 2;

				//	}
				}
	}
	pNdis80211StatusIndication->StatusType=Ndis802_11StatusType_Authentication;
	pNdis80211AuthRequest = (PNDIS_802_11_AUTHENTICATION_REQUEST)&statusBuffer[4];
	pNdis80211AuthRequest->Length=sizeof(NDIS_802_11_AUTHENTICATION_REQUEST);
	NdisMoveMemory(pNdis80211AuthRequest->Bssid,
										Adapter->BSSID,
										6);
	pNdis80211AuthRequest->Flags = Flags;
	NdisMIndicateStatus(Adapter->ZD1205AdapterHandle,
											GeneralStatus,
											statusBuffer,
											sizeof(statusBuffer));
	ZDDbgPrint((_T("Report Auth Event: 0x%X\n"), pNdis80211AuthRequest->Flags));
	if (pNdis80211AuthRequest->Flags == NDIS_802_11_AUTH_REQUEST_KEYUPDATE)
	{
		Adapter->keyMgrState |= BIT_1; // Bit1 is reset when the subsequent EAPoL key message is received.
		// If the received EapOL message is NOT encrypted, then Set Bit 2 to force send the
		// next EAPoL message without encryption.
	}
	//Dump(statusBuffer, sizeof(statusBuffer), FALSE, 1);
}


typedef struct _KEY_INFORMATION {
	USHORT	Type:3;
	USHORT	KeyType:1;
	USHORT	KeyIndex:2;
	USHORT	Install:1;
	USHORT	KeyAck:1;
	USHORT	KeyMIC:1;
	USHORT	Secure:1;
	USHORT	Error:1;
	USHORT	Request:1;
	USHORT	Reserved:4;
} KEY_INFORMATION, *PKEY_INFORMATION;

#pragma pack(1)
typedef struct _EAPOL_KEY {
	UCHAR	DescriptorType;
	USHORT	KeyInformation; 		// Big Endian
	USHORT	KeyLength;			// Big Endian
	UCHAR	ReplayCounter[8]; 	// Big Endian
	UCHAR	KeyNonce[32];
	UCHAR	EAPOL_KeyIV[16];
	UCHAR	KeyRSC[8];			// LIttle Endian
	UCHAR	KeyID[8];
	UCHAR	KeyMIC[16];
	USHORT	KeyDataLength; 		// Big endian
	UCHAR	KeyData[1];
} EAPOL_KEY, *PEAPOL_KEY;
#pragma pack()


typedef struct _DOT1X_HEADER {
	UCHAR	PV;	// Protocol Version. It should be 01 for WPA
	UCHAR	PT;	// Protocol Type. 00:EAP packet; 01:EAPOL-Start; 03:EAPOL-Key
	USHORT PBL; // Packet Body Length, Big endian.
} DOT1X_HEADER, *PDOT1X_HEADER;

typedef struct _EAP_PACKET {
	UCHAR	Co;	// Code, 1:Request(Data available), 2:Response(Data available), 3:Success, 4:Failure
	UCHAR	ID; // Contains a value that is used to match Response to Requests.
	USHORT	Len; // Big-Endian format, This value is the same as PBL field of 802.1X header
	UCHAR	EapData[1]; // For EAP-Request and EAP-Response packet, First byte is a Type field that indicates which authentication algorithm is in use.
						// If EapData[0]==1 means Identity,
						// If EapData[0]==13 means EAP-TLS.
} EAP_PACKET, *PEAP_PACKET;

typedef struct _DOT1X_FORMAT{
	DOT1X_HEADER	 Header;
	union {
			EAP_PACKET		eapPacket;
			EAPOL_KEY			eapolKey;
		} u;
}	DOT1X_FORMAT, *PDOT1X_FORMAT;

UCHAR	*MessagePacketType[5]={
			"EAP-Packet",
			"EAPoL-Start",
			"EAPoL-Logoff",
			"EAPoL-Key",
			"EAPoL-Encapsulated-ASF-Alert"
};

UCHAR	*MessageEapPacketCode[]={
		"",
		"Request",
		"Response",
		"Success",
		"Failure"};


UCHAR	*MessageKeyType[2]={
			"Group Key",
			"Pairwise Key"
};


// This routine is used to parse 802.1X packet message.
// Parameters:
//		Adapter:	Per-Card maintained context.
//		data: Beginning of 802.1X data(Start from 802.1x header)
//		length: the length of input data.
void EapolParser(PZD1205_ADAPTER Adapter,
					PVOID		data,
					ULONG		length)
{
	PDOT1X_FORMAT	pDot1x=(PDOT1X_FORMAT) data;
	PEAPOL_KEY	pEapolKey;
	PKEY_INFORMATION		pkeyInformation;

	ZDDbgPrint((_T("PacketType= %s\n"), MessagePacketType[pDot1x->Header.PT]));
	if(pDot1x->Header.PT==3)
	{// Start of Dumping EAPoL-KEY message.
		USHORT	keyInformation;
		pEapolKey=(PEAPOL_KEY)&pDot1x->u.eapolKey;
		
		keyInformation=ntohs(pEapolKey->KeyInformation);
		
		pkeyInformation = (PKEY_INFORMATION)&keyInformation;
//      IF_LOG_WPA(
//          MP_DBG_PRINT1("    Descriptor Type= %d\n", pEapolKey->DescriptorType);
            ZDDbgPrint((_T("   Key Information: 0x%X\n"), keyInformation));
//          MP_DBG_PRINT1("    Descriptor Version= %d\n", pkeyInformation->Type);
            ZDDbgPrint((_T("   %s"), MessageKeyType[pkeyInformation->KeyType]));
            ZDDbgPrint((_T("   KeyID: %d "), pkeyInformation->KeyIndex));
            if (pkeyInformation->Install)
                ZDDbgPrint((_T("   ,Install ")));
            if (pkeyInformation->KeyAck)
                ZDDbgPrint((_T("   ,ACK ")));
            if (pkeyInformation->KeyMIC)
                ZDDbgPrint((_T("   ,MIC ")));
            if (pkeyInformation->Secure)
                ZDDbgPrint((_T("   ,Secure ")));
            if (pkeyInformation->Error)
                ZDDbgPrint((_T("   ,Error ")));
            if (pkeyInformation->Request)
                ZDDbgPrint((_T("   ,Request ")));
            ZDDbgPrint((_T("\n")));
            ZDDbgPrint((_T("   KeyDataLength=%d\n"), ntohs(pEapolKey->KeyDataLength)));
			//MP_DBG_PRINT1("KeyDataLength=0x%X\n", pEapolKey->KeyDataLength);
			if (pEapolKey->KeyDataLength)
			{
				Dump(pEapolKey->KeyData, ntohs(pEapolKey->KeyDataLength),FALSE,1);
			}
			//Dump(data, length, FALSE, 1);
//		)

	}// End of Dumping EAPoL-KEY message.
	if(pDot1x->Header.PT==0) 
	{// Start to process EAP Packet
		PEAP_PACKET	pEapPacket;
		pEapPacket=&pDot1x->u.eapPacket;
		//IF_LOG_WPA(
            ZDDbgPrint((_T("   Code:%s\n"), MessageEapPacketCode[pEapPacket->Co]));
            ZDDbgPrint((_T("   Identifier:%d\n"), pEapPacket->ID));
            ZDDbgPrint((_T("   Length:%d\n"), ntohs(pEapPacket->Len)));
            ZDDbgPrint((_T("   Type:%d\n"), pEapPacket->EapData[0]));
            if (pEapPacket->EapData[0]==1) // Type:Identity
            {
                ZDDbgPrint((_T("   Type: Identity\n")));
                Dump(&pEapPacket->EapData[1], ntohs(pEapPacket->Len)-5, FALSE, 1);
            }
            else if (pEapPacket->EapData[0]==13)
            {
                ZDDbgPrint((_T("   Type: EAP-TLS\n")));
			}
		//)
	}
}


// Patrick 4112
// Save the default Connection to registry

void
BinToHexStr( UCHAR* pOutStr,
			 UCHAR* pInBinData,		// Need (1 * ConvLen) bytes space to use(NULL Terminate is not needed).
			 UINT	ConvLen			// len of pInBinData
			 )			// How many couple of hexdecimal string to be convert
{
	UINT i;
	int highPart, lowPart;

	for (i = 0; i < ConvLen; i++){

		highPart = (*pInBinData >> 4);
		lowPart = (*pInBinData & 0xF);

		if ( highPart < 0xA)
			*pOutStr = highPart+'0';
		else 
			*pOutStr = highPart+'A'-10;

		pOutStr++;

		if ( lowPart < 0xA)
			*pOutStr = lowPart+'0';
		else 
			*pOutStr = lowPart+'A'-10;

		pOutStr++;
		pInBinData++;
	}
	// Convert OK!!
}

VOID
SaveDefaultConnection(
                    IN PZD1205_ADAPTER Adapter
                    )
{
#if 0
    ANSI_STRING    AnsiString;
    NDIS_STRING    Key;
	NDIS_STRING    RegRFChannel = NDIS_STRING_CONST("RFChannel");     
	NDIS_STRING    RegNetType = NDIS_STRING_CONST("NetworkType"); 
	NDIS_STRING    RegSSID = NDIS_STRING_CONST("SSID");
	NDIS_STRING    RegWEPStatus = NDIS_STRING_CONST("WEPStatus");
	NDIS_STRING    RegWEPKeyId = NDIS_STRING_CONST("WEPDefaultKeyId");
	NDIS_STRING    RegWEPKeys = NDIS_STRING_CONST("WEPDefaultKeys");
    NDIS_STATUS    Status;
	UCHAR          buf[233];
	WCHAR		   wBuf[466];
    //PNDIS_CONFIGURATION_PARAMETER    ReturnValue, ReturnValue1, ReturnValue2, ReturnValue3, ReturnValue4, ReturnValue5;
	//NDIS_CONFIGURATION_PARAMETER	SaveValue;
	int				i;
	//NDIS_HANDLE		ConfigHandle;
	HANDLE			keyHandle = 0; 
	UNICODE_STRING	uStr;
	ULONG			tmpLong;

    DEBUGFUNC(_T("SaveDefaultConnection"));
    INITSTR((_T("\n")));

	// only can use IoOpenDeviceRegistryKey
	Status = IoOpenDeviceRegistryKey(	Adapter->pLowerDeviceObject,
										PLUGPLAY_REGKEY_DRIVER,
										STANDARD_RIGHTS_WRITE,
										&keyHandle);


	if (keyHandle == 0) return;

	AnsiString.MaximumLength = sizeof(buf);
	AnsiString.Buffer = buf;
	
	//NdisInitUnicodeString( &uStr, wBuf);
	uStr.MaximumLength = sizeof(wBuf);
	uStr.Buffer = wBuf;

	{	//save SSID
		for (i=0;i<Adapter->CardSetting.Info_SSID[1];i++){
			buf[i] = Adapter->CardSetting.Info_SSID[2+i];
		}
		buf[Adapter->CardSetting.Info_SSID[1]] = 0;

	    AnsiString.Length = Adapter->CardSetting.Info_SSID[1];
    
		Status = NdisAnsiStringToUnicodeString(	&uStr,
												&AnsiString
												);

		if (NDIS_STATUS_SUCCESS != Status){
		
			INITSTR((_T("error\n")));
		}

		INITSTR((_T("Save %ws %d\n"), uStr.Buffer, uStr.Length));

		Status = ZwSetValueKey(	keyHandle, &RegSSID, 0, REG_SZ, 
								uStr.Buffer, 
								uStr.Length+2
								);
	}


	{	//save WEP status
		
		if ( Adapter->CardSetting.EncryMode == WEP64 ){
			tmpLong = 2;
		}
		else if ( Adapter->CardSetting.EncryMode == WEP128 ){
			tmpLong = 6;
		}
		else if ( Adapter->CardSetting.EncryMode == WEP256 ){
			tmpLong = 7;
		}
		else{
			tmpLong = 0;
		}
		
		Status = ZwSetValueKey(	keyHandle, &RegWEPStatus, 0, REG_DWORD, 
								&tmpLong, 
								4
								);
	}

	{	//save Net Type
		tmpLong = Adapter->CardSetting.BSSType;
		
		Status = ZwSetValueKey(	keyHandle, &RegNetType, 0, REG_DWORD, 
								&tmpLong, 
								4
								);
	}

	{	//save Channel
		tmpLong = Adapter->CardSetting.Channel;
		
		Status = ZwSetValueKey(	keyHandle, &RegRFChannel, 0, REG_DWORD, 
								&tmpLong, 
								4
								);
	}

	{	//save Wep deault key ID
		tmpLong = Adapter->CardSetting.EncryKeyId + 1;
		
		Status = ZwSetValueKey(	keyHandle, &RegWEPKeyId, 0, REG_DWORD, 
								&tmpLong, 
								4
								);
	}

	if ( Adapter->CardSetting.EncryMode != NO_WEP )
	{	//save Wep Key
		if ( Adapter->CardSetting.EncryMode == WEP64 ){
			BinToHexStr(buf, Adapter->AllKeyBuf, 5*4);
			AnsiString.Length = 5*4*2;
		}
		if ( Adapter->CardSetting.EncryMode == WEP128 ){
			BinToHexStr(buf, Adapter->AllKeyBuf, 13*4);
			AnsiString.Length = 13*4*2;
		}
		if ( Adapter->CardSetting.EncryMode == WEP256 ){
			BinToHexStr(buf, Adapter->AllKeyBuf, 29*4);
			AnsiString.Length = 29*4*2;
		}

	    AnsiString.Buffer = buf;
		buf[AnsiString.Length] = 0;
		
		NdisAnsiStringToUnicodeString(	&uStr,
										&AnsiString
										);
		INITSTR((_T("Save %ws %d\n"), uStr.Buffer, uStr.Length));

		Status = ZwSetValueKey(	keyHandle, &RegWEPKeys, 0, REG_SZ, 
								uStr.Buffer, 
								uStr.Length+2
								);

	}

	ZwClose(keyHandle);
#endif
}


// jhlee 3317
// Get the default Connection from registry
/*
VOID
GetDefaultConnection(
                    IN PZD1205_ADAPTER Adapter,
                    IN NDIS_HANDLE ConfigHandle
                    )
{
    ANSI_STRING    AnsiString;
	NDIS_STRING    RegRFChannel = NDIS_STRING_CONST("RFChannel");     
	NDIS_STRING    RegNetType = NDIS_STRING_CONST("NetworkType"); 
	NDIS_STRING    RegSSID = NDIS_STRING_CONST("SSID");
	NDIS_STRING    RegWEPStatus = NDIS_STRING_CONST("WEPStatus");
	NDIS_STRING    RegWEPKeyId = NDIS_STRING_CONST("WEPDefaultKeyId");
	NDIS_STRING    RegWEPKeys = NDIS_STRING_CONST("WEPDefaultKeys");
	NDIS_STRING    RegWPAOn = NDIS_STRING_CONST("WPAOn");  // for WHQL test
	NDIS_STRING    RegVendorDesc = NDIS_STRING_CONST("VendorDesc");
    NDIS_STATUS    Status;
	UCHAR          buf[232];
    PNDIS_CONFIGURATION_PARAMETER    ReturnValue, ReturnValue1, ReturnValue2, ReturnValue3, ReturnValue4, ReturnValue5;
  
    DEBUGFUNC(_T("GetDefaultConnection"));
    INITSTR((_T("\n")));
 
    //NdisInitializeString(&Key, "SSID");
    
	AnsiString.Length = 0;
	AnsiString.MaximumLength = sizeof(buf);
	AnsiString.Buffer = buf;

    strcpy(Adapter->ZDDefaultSSID, "");
    Adapter->ZDDefaultWEPStatus = 0;
    Adapter->ZDDefaultNetType = INFRASTRUCTURE_BSS;
    Adapter->ZDDefaultRFChannel = 1;

    NdisOpenConfiguration(&Status,
        &ConfigHandle,
        Adapter->ZD1205WrapperConfigurationContext);

    NdisReadConfiguration( &Status,
                           &ReturnValue,
                           ConfigHandle,
                           &RegSSID,
                           NdisParameterString);

    if ( Status != NDIS_STATUS_SUCCESS ){
		NdisCloseConfiguration(ConfigHandle);
		NdisOpenConfiguration(&Status,
			&ConfigHandle,
			Adapter->ZD1205WrapperConfigurationContext);
	}else{
        NdisUnicodeStringToAnsiString(&AnsiString, &ReturnValue->ParameterData.StringData);
        strcpy(Adapter->ZDDefaultSSID, AnsiString.Buffer);
        Adapter->ZDDefaultSSID[AnsiString.Length] = 0;	
	}

    NdisReadConfiguration( &Status,
                           &ReturnValue4,
                           ConfigHandle,
                           &RegWPAOn,
                           NdisParameterInteger);

    if ( Status != NDIS_STATUS_SUCCESS ){
		NdisCloseConfiguration(ConfigHandle);
		NdisOpenConfiguration(&Status,
			&ConfigHandle,
			Adapter->ZD1205WrapperConfigurationContext);
	}else{
        Adapter->WPAOn = (UCHAR) ReturnValue4->ParameterData.IntegerData;
	}
    // vendor description
    NdisReadConfiguration( &Status,
                           &ReturnValue5,
                           ConfigHandle,
                           &RegVendorDesc,
                           NdisParameterString);

    if ( Status != NDIS_STATUS_SUCCESS ){
		NdisCloseConfiguration(ConfigHandle);
		NdisOpenConfiguration(&Status,
			&ConfigHandle,
			Adapter->ZD1205WrapperConfigurationContext);
	}else{
        NdisUnicodeStringToAnsiString(&AnsiString, &ReturnValue5->ParameterData.StringData);
        strcpy(Adapter->vendorDesc, AnsiString.Buffer);
		Adapter->vendorDesclen = AnsiString.Length;
	}

    //NdisInitializeString(&Key, "WEPStatus");
    NdisReadConfiguration( &Status,
                           &ReturnValue1,
                           ConfigHandle,
                           &RegWEPStatus,
                           NdisParameterInteger);

    if ( Status != NDIS_STATUS_SUCCESS ){
		NdisCloseConfiguration(ConfigHandle);
		NdisOpenConfiguration(&Status,
			&ConfigHandle,
			Adapter->ZD1205WrapperConfigurationContext);
	}else{
        Adapter->ZDDefaultWEPStatus = (USHORT) ReturnValue1->ParameterData.IntegerData;	
	}
   
    NdisReadConfiguration( &Status,
                           &ReturnValue2,
                           ConfigHandle,
                           &RegNetType,
                           NdisParameterInteger);

    if ( Status != NDIS_STATUS_SUCCESS ){
		NdisCloseConfiguration(ConfigHandle);
		NdisOpenConfiguration(&Status,
			&ConfigHandle,
			Adapter->ZD1205WrapperConfigurationContext);
	}else{
        Adapter->ZDDefaultNetType = (USHORT) ReturnValue2->ParameterData.IntegerData;
	}

    NdisReadConfiguration( &Status,
                           &ReturnValue3,
                           ConfigHandle,
                           &RegRFChannel,
                           NdisParameterInteger);

    if ( Status != NDIS_STATUS_SUCCESS ){
		NdisCloseConfiguration(ConfigHandle);
		NdisOpenConfiguration(&Status,
			&ConfigHandle,
			Adapter->ZD1205WrapperConfigurationContext);
	}else{
        Adapter->ZDDefaultRFChannel = (USHORT) ReturnValue3->ParameterData.IntegerData;
	}

    INITSTR((_T("ZD_GetDefaultConnection:")));
    INITSTR((_T("ssid = %s\n"), Adapter->ZDDefaultSSID));
    INITSTR((_T("length = %d\n"), AnsiString.Length));


    INITSTR((_T("status = %d\n"), Adapter->ZDDefaultWEPStatus));

    //NdisInitializeString(&Key, "WEPDefaultKeyId");
    NdisReadConfiguration( &Status,
                           &ReturnValue,
                           ConfigHandle,
                           &RegWEPKeyId,
                           NdisParameterInteger);
      
    if ( Status != NDIS_STATUS_SUCCESS ){
		NdisCloseConfiguration(ConfigHandle);
		NdisOpenConfiguration(&Status,
			&ConfigHandle,
			Adapter->ZD1205WrapperConfigurationContext);
	}else{
	    Adapter->ZDDefaultKeyId = (USHORT) ReturnValue->ParameterData.IntegerData - 1;
 	}
    INITSTR((_T("KeyId = %d\n"), Adapter->ZDDefaultKeyId));

        //NdisInitializeString(&Key, "WEPDefaultKeys");
    NdisReadConfiguration( &Status,
                           &ReturnValue,
                           ConfigHandle,
                           &RegWEPKeys,
                           NdisParameterString);

    if ( Status == NDIS_STATUS_SUCCESS ){
        NdisUnicodeStringToAnsiString(&AnsiString, &ReturnValue->ParameterData.StringData);
        strcpy(Adapter->ZDDefaultKey, AnsiString.Buffer);
    }

	NdisCloseConfiguration(ConfigHandle);

	DefaultConnSetup(Adapter);
    
}

// jhlee 3317
// set up the default connection

VOID
DefaultConnSetup(
                 IN PZD1205_ADAPTER Adapter
                 )
{
    //UCHAR pKey[14];
	//UCHAR pHEXKey[27];
    UCHAR pKey[30];
	UCHAR pHEXKey[59];
	ULONG i, loop;

	if ( strlen(Adapter->ZDDefaultSSID) <= 0 )
        return;

	SetBSSType(Adapter, (UCHAR)Adapter->ZDDefaultNetType);

    Adapter->CardSetting.Info_SSID[1] = (UCHAR) strlen(Adapter->ZDDefaultSSID);
    strcpy(&Adapter->CardSetting.Info_SSID[2], Adapter->ZDDefaultSSID);
	Set_RF_Channel(Adapter, Adapter->ZDDefaultRFChannel, (UCHAR)Adapter->RF_Mode, 1);	    

	if ( Adapter->ZDDefaultWEPStatus )
	    Adapter->CardSetting.EncryKeyId = (UCHAR) Adapter->ZDDefaultKeyId;
	else
		return;

	for( i=0; i<4; i++ ){
	    if ( Adapter->ZDDefaultWEPStatus == 2 ){
		    Change_EncryMode(Adapter, WEP64, WEP64);
			strncpy(pHEXKey, Adapter->ZDDefaultKey+i*10, 10);  
		    HexStrToBin(pHEXKey, pKey, 5);
			memcpy(Adapter->AllKeyBuf+5*i, pKey, 5);
		    SetKeyContext( Adapter, DEFAULT_KEY_SET, WEP64, (UCHAR)i, pKey);
		    loop = 20;
		}else if ( Adapter->ZDDefaultWEPStatus == 6 ){
	    	Change_EncryMode(Adapter, WEP128, WEP128);
			strncpy(pHEXKey, Adapter->ZDDefaultKey+i*26, 26);  
		    HexStrToBin(pHEXKey, pKey, 13);
			memcpy(Adapter->AllKeyBuf+13*i, pKey, 13);
		    SetKeyContext( Adapter, DEFAULT_KEY_SET, WEP128, (UCHAR)i, pKey);
            loop = 52;
		}else if ( Adapter->ZDDefaultWEPStatus == 7 ){
			Change_EncryMode(Adapter, WEP256, WEP256);
			strncpy(pHEXKey, Adapter->ZDDefaultKey+i*58, 58);  
		    HexStrToBin(pHEXKey, pKey, 29);
			memcpy(Adapter->AllKeyBuf+29*i, pKey, 29);
		    SetKeyContext( Adapter, DEFAULT_KEY_SET, WEP256, (UCHAR)i, pKey);
            //loop = 52;
		}
		ZDDbgPrint((_T("Key = %s\n"), pHEXKey));

#ifndef AMAC
	    ptmp = pKey;
        for( j=0; j<loop; j+=4 ){
	  	    ZD1205_READ_REGISTER(Adapter,WEPKey0+j, &tmpkey);
		    switch( i ){
			    case 0:
				    tmpkey &= ~0x000000ff;
				    tmpkey |= ((ULONG)*(ptmp) << 0);
				    break;
			    case 1:
	    	    	tmpkey &= ~0x0000ff00;
		    		tmpkey |= ((ULONG)*(ptmp) << 8);
			    	break;
			    case 2:
				    tmpkey &= ~0x00ff0000;
				    tmpkey |= ((ULONG)*(ptmp) << 16);
				    break;
			    case 3:
				    tmpkey &= ~0xff000000;
				    tmpkey |= ((ULONG)*(ptmp) << 24);
				    break;
			}
		    ptmp++;
		    ZD1205_WRITE_REGISTER(Adapter,WEPKey0+j, tmpkey);
		}
#endif
	}
}

*/
BOOLEAN IsHexDigit( UCHAR cChar )
{
	if(	(cChar >= '0' && cChar <= '9') ||
		(cChar >= 'A' && cChar <= 'F') ||
		(cChar >= 'a' && cChar <= 'f') ) {
		return TRUE;
	} else {
		return FALSE;
	}
}

UINT HexToUINT(UCHAR* pstr)
{
    UINT uResult = 0;
	
	while(*pstr)
	{
		if(!IsHexDigit(*pstr)) {
			return 0xFFFFFFFF;
		}

		uResult = uResult * 16;
		if(*pstr > '9')
			uResult = uResult + (char)(*pstr & 0xF) + 9;
		else
			uResult = uResult + (char)(*pstr & 0xF);
		pstr++;
	}
	return uResult;
}

//JWEI 2004/02/18
#ifdef HOST_IF_USB
void ZD1211_WRITE_REGISTER(
                    IN PZD1205_ADAPTER Adapter,
                    ULONG   _offset, 
                    ULONG   _Value,
					BOOLEAN	AddUSBCSRAddress
                    )
{
int     ii = 0;
BOOLEAN bRet = 0;

    while (bRet == 0)
    {
        if (ii > 0)
            ZDDbgPrint((_T("Try Again !!!!!!-\n")));

        if (ii > 5)
        {
            ZDDbgPrint((_T("+++++ We have try five times, not work !!!!!!-\n")));
            break;
        }
        ii ++;
        bRet = USB_WRITE_REGISTER(Adapter, _offset, _Value, AddUSBCSRAddress);
    }

}

    #ifndef fWRITE_WORD_REG
void ZD1211_WriteMultiRegister( IN PZD1205_ADAPTER Adapter,
                                USHORT  *Address,
                                ULONG   *Value,
                                USHORT  RegCount,
                                BOOLEAN AddUSBCSRAddress
                              )
    #else
void ZD1211_WriteMultiRegister( IN PZD1205_ADAPTER Adapter,
                                USHORT  *Address,
                                USHORT  *Value,
                                USHORT  RegCount,
                                BOOLEAN AddUSBCSRAddress
                              )
    #endif
{
int     ii = 0;
BOOLEAN bRet = FALSE;

    while (!bRet)
    {
        if (ii > 0)
            ZDDbgPrint((_T("Try Again !!!!!!-\n")));

        if (ii > 5)
        {
            ZDDbgPrint((_T("+++++ We have try five times, not work !!!!!!-\n")));
            break;
        }
        ii ++;
        bRet = USB_PACKAGE_WRITE_REGISTER(Adapter, Address, Value, RegCount, AddUSBCSRAddress);
    }
}
#endif

void 
ZD1205_WRITE_REGISTER(
                    IN PZD1205_ADAPTER Adapter,
                    ULONG   _offset, 
                    ULONG   _Value
                    )
{

    //return;

#ifdef HOST_IF_USB
    #if fPROG_FLASH
    if (Adapter->AllowAccessRegister != TRUE)
        return;
    #endif
    ZD1211_WRITE_REGISTER(Adapter, _offset, _Value, TRUE);
    return;
#endif

}

//JWEI 2004/02/18
#ifdef HOST_IF_USB
void ZD1211_READ_REGISTER(
                    IN PZD1205_ADAPTER Adapter,
                    ULONG _offset, 
                    PULONG _pValue,
                    BOOLEAN AddUSBCSRAddress
                    ) 
{
int     ii = 0;
BOOLEAN bRet = 0;
//ULONGLONG   time;


    //time = GetTickCount();

    while (bRet == 0)
    {
        if (ii > 0)
		{
			(Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(Adapter->USBInfo.hRegEP);
            ZDDbgPrint((_T("Try Again !!!!!!-\n")));
		}

        if (ii > 5)
        //if (ii > 0)
        {
            ZDDbgPrint((_T("+++++ We have try five times, not work !!!!!!-\n")));
            break;
        }
        ii ++;
        bRet = USB_READ_REGISTER(Adapter, _offset, _pValue, AddUSBCSRAddress);
    }

	//DbgPrint("interval = %d", GetTickCount() - time);
}


    #ifndef fREAD_MUL_REG
void ZD1211_ReadMultiRegister(  IN PZD1205_ADAPTER  Adapter,
                                USHORT              *Address,
                                ULONG               *pValue,
                                USHORT              RegCount,
                                BOOLEAN             AddUSBCSRAddress
                             )
    #else
void ZD1211_ReadMultiRegister(  IN PZD1205_ADAPTER  Adapter,
                                USHORT              *Address,
                                USHORT              *pValue,
                                USHORT              RegCount,
                                BOOLEAN             AddUSBCSRAddress
                             )
    #endif
{
int     ii = 0;
BOOLEAN bRet = FALSE;

    while (!bRet)
    {
        if (ii > 0)
            ZDDbgPrint((_T("Try Again !!!!!!-\n")));

        if (ii > 5)
        {
            ZDDbgPrint((_T("+++++ We have try five times, not work !!!!!!-\n")));
            break;
        }
        ii ++;
        bRet = USB_PACKAGE_READ_REGISTER(Adapter, Address, pValue, RegCount, AddUSBCSRAddress);
    }
}
#endif

void
ZD1205_READ_REGISTER(
                    IN PZD1205_ADAPTER Adapter,
                    ULONG _offset, 
                    PULONG _pValue
                    ) 
{

#ifdef HOST_IF_USB
    #if fPROG_FLASH
    if (Adapter->AllowAccessRegister != TRUE)
        return;
    #endif
    ZD1211_READ_REGISTER(Adapter, _offset, _pValue, TRUE);
    return;

#endif



}
/*
BOOLEAN
NeedWriteProtection(
					IN PZD1205_ADAPTER Adapter,
					ULONG   _offset, 
					ULONG	_Value
				) 
{
	BOOLEAN	status=TRUE;

	//if (!Adapter->ProtectRegs){
		return FALSE;
	//}

	switch(_offset){
	case CR17:
		Adapter->sw_CR17 = (UCHAR)_Value;
		break;
	case CR19:
		Adapter->sw_CR19 = (UCHAR)_Value;
		break;
	case CR23:
		Adapter->sw_CR23 = (UCHAR)_Value;
		break;
	case CR24:
		Adapter->sw_CR24 = (UCHAR)_Value;
		break;
	case CR27:
		Adapter->sw_CR27 = (UCHAR)_Value;
		break;
	case CR33:
		Adapter->sw_CR33 = (UCHAR)_Value;
		break;
	case CR34:
		Adapter->sw_CR34 = (UCHAR)_Value;
		break;
	case CR35:
		Adapter->sw_CR35 = (UCHAR)_Value;
		break;
	case CR113:
		Adapter->sw_CR113 = (UCHAR)_Value;
		break;
	case CR119:
		Adapter->sw_CR119 = (UCHAR)_Value;
		break;
	case CR125:
		Adapter->sw_CR125 = (UCHAR)_Value;
		break;
	case CR127:
		Adapter->sw_CR127 = (UCHAR)_Value;
		break;
	default:
		status = FALSE;
		break;
	}
	return(status);
}

BOOLEAN
NeedReadProtection(
					IN PZD1205_ADAPTER Adapter,
					ULONG _offset, 
					PULONG _pValue
				) 
{
	BOOLEAN	status=TRUE;

	//if (!Adapter->ProtectRegs){
		return FALSE;
	//}

	switch(_offset){
	case CR17:
		*(_pValue) = Adapter->sw_CR17;
		break;
	case CR19:
		*(_pValue) = Adapter->sw_CR19;
		break;
	case CR23:
		*(_pValue) = Adapter->sw_CR23;
		break;
	case CR24:
		*(_pValue) = Adapter->sw_CR24;
		break;
	case CR27:
		*(_pValue) = Adapter->sw_CR27;
		break;
	case CR33:
		*(_pValue) = Adapter->sw_CR33;
		break;
	case CR34:
		*(_pValue) = Adapter->sw_CR34;
		break;
	case CR35:
		*(_pValue) = Adapter->sw_CR35;
		break;
	case CR113:
		*(_pValue) = Adapter->sw_CR113;
		break;
	case CR119:
		*(_pValue) = Adapter->sw_CR119;
		break;
	case CR125:
		*(_pValue) = Adapter->sw_CR125;
		break;
	case CR127:
		*(_pValue) = Adapter->sw_CR127;
		break;
	default:
		status = FALSE;
		break;
	}
	return(status);
}
*/
void
InitializeProtectedRegs(
					IN PZD1205_ADAPTER Adapter
					)
{
	//Adapter->sw_CR17 = 0x1e;
	//Adapter->sw_CR19 = 0x44;
	//Adapter->sw_CR23 = 0x90;
	//Adapter->sw_CR24 = 0x14;
	//Adapter->sw_CR27 = 0x00;
	//Adapter->sw_CR33 = 0x28;
	//Adapter->sw_CR34 = 0x6;
	//Adapter->sw_CR35 = 0x0a;
	//Adapter->sw_CR113 = 0xa0;
	//Adapter->sw_CR119 = 0x16;
	//Adapter->sw_CR125 = 0x00;
	//Adapter->sw_CR127 = 0x00;
}

//---------------------------------------------------------------------------
// Input:
//   pInStr		: Point to the Input string to convert
//   pBinData	: Point to the Output string (binary data)
//   ConvLen	: How many Hex need to convert to binary.
//
// Output:
//   0: Failed
//   1: Success
//---------------------------------------------------------------------------
UINT
HexStrToBin( UCHAR* pInStr,
			 UCHAR* pOutBinData,		// Need (1 * ConvLen) bytes space to use(NULL Terminate is not needed).
			 UINT	ConvLen
			 )			// How many couple of hexdecimal string to be convert
{
	BOOLEAN	  exitFlag = 0;					// Exit parsing
	BOOLEAN	  stateTrigger = 0;				// Find data to convert
	BOOLEAN	  lastHex = 0;					// Last input character is Hex?
	UINT	  bytesCount = 0;					// How many Hex character have been find
	UINT	  HexCoupleCount = 0;				// How many hex already convert
	UCHAR	  TempStr[3];						// TempStr for convert Hex to byte
	UCHAR	  *pch1;							// pointer
	UCHAR	  *pch2;							// pointer
	UINT	  i;

	// Set all output buffer to 0.
	pch1 = pOutBinData;
	for(i=0; i<ConvLen; i++) {
		*pch1++ = 0;						// Clear pOutBinData
	}

	pch1 = pInStr;
	pch2 = pOutBinData;

	// Generate the output binary string.
	while(1)								// Parsing input string
	{
		switch(*pch1) {
		  case NULL:						// Termainal string
			exitFlag = 1;
			if(bytesCount > 0)
				stateTrigger = 1;			// *** Need parsing the number.
			break;

		  case 0x20:						// SPACE
		  case 0x09:						// TAB
 			if(lastHex)
				stateTrigger = 1;			// *** Need parsing the number.
			break;

		  default:
			if(!IsHexDigit(*pch1)) {		// Not hexdecimal character
				exitFlag = 1;
				break;
			}
			if(HexCoupleCount >= ConvLen){	// Too much input.
				exitFlag = 1;
				break;
			}
			lastHex = 1;
			bytesCount++;
			if(bytesCount == 2)
				stateTrigger = 1;			// *** Need parsing the number.
			break;
		} // End of switch(*pch1)

		pch1++;
		// Generate binary output
		if(stateTrigger) {
			pch1 = pch1 - 2;
			strncpy(TempStr,pch1,bytesCount);
			TempStr[bytesCount] = 0;		// Set the NULL Terminated.

			*pch2++ = (char)HexToUINT(TempStr);
			HexCoupleCount++;
			pch1 = pch1 + 2;
			stateTrigger = 0;
			bytesCount = 0;
			lastHex = 0;
		} // End of if(stateTrigger)

		if(exitFlag)
			break;

	} // End of while(*pch != NULL)

	return 1;								// Convert OK!!
}

void
ZD1205_Sleep(
			 LONG	TimeInUs
			 )
{
#ifndef CE_ALL
	LONG            ltemp;
	LARGE_INTEGER   liInterval;
	if (KeGetCurrentIrql() == PASSIVE_LEVEL){
		liInterval.QuadPart = 0 - (TimeInUs * 10);
		// Use KeDelayExecutionThread instead of NdisMSleep to 
		// workaround Win98 in-accuracy problem.
		KeDelayExecutionThread(KernelMode,FALSE,&liInterval);
	}
	else{
		NdisStallExecution(TimeInUs);
	}
#else
    NdisStallExecution(TimeInUs);
#endif
}
			 
#ifdef NDIS51_MINIPORT
/*++

Routine Description:
    
    MiniportPnPEventNotify handler - NDIS51 and later
    
Arguments:

    MiniportAdapterContext      Pointer to our adapter
    PnPEvent                    Self-explanatory 
    InformationBuffer           Self-explanatory 
    InformationBufferLength     Self-explanatory 

Return Value:

    None
    
--*/
VOID ZD1205PnPEventNotify(
    IN  NDIS_HANDLE             MiniportAdapterContext,
    IN  NDIS_DEVICE_PNP_EVENT   PnPEvent,
    IN  PVOID                   InformationBuffer,
    IN  ULONG                   InformationBufferLength
    )
{
    PZD1205_ADAPTER       Adapter;
	ZD1211_IRQL 		  OldIrql;
	PVOID                 FirstBufferVA;
    UINT                  FirstBufferLength, TotalBufferLength;

    DEBUGFUNC(_T("ZD1205PnPEventNotify"));
    INITSTR((_T("\n")));

	Adapter = (PZD1205_ADAPTER)MiniportAdapterContext;

    switch (PnPEvent)
    {
        case NdisDevicePnPEventQueryRemoved:
            INITSTR((_T("MPPnPEventNotify: NdisDevicePnPEventQueryRemoved\n")));
            break;

        case NdisDevicePnPEventRemoved:
            INITSTR((_T("MPPnPEventNotify: NdisDevicePnPEventRemoved\n")));
            break;       

        case NdisDevicePnPEventSurpriseRemoved:
            INITSTR((_T("MPPnPEventNotify: NdisDevicePnPEventSurpriseRemoved\n")));

			Adapter->SurpriseRemoved = TRUE;

#ifdef HOST_IF_USB
			USB_StopRxEP(Adapter);
			USB_StopTxEP(Adapter);
			USB_StopIntEP(Adapter);
#endif

			Adapter->PSMSupported = FALSE;
			if (Adapter->DeviceInSleep){
				Adapter->DeviceInSleep = FALSE;
			}

			// Disable the device's interrupt line.
			ZD1205DisableInterrupt(Adapter);
			
			// check to make sure there are no outstanding transmits
			//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
			ZDDbg((_T("Get SpinLock\n")));
			ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
			while(Adapter->FirstTxQueue)
			{
				PNDIS_PACKET QueuePacket = Adapter->FirstTxQueue;
				
				Adapter->NumPacketsQueued--;
				DequeuePacket(Adapter->FirstTxQueue, Adapter->LastTxQueue);
				
				//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
				ZDDbg((_T("---------->>> Release SpinLock\n")));
				ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
				NdisMSendComplete(
					Adapter->ZD1205AdapterHandle,
					QueuePacket,
					NDIS_STATUS_SUCCESS);
					//NDIS_STATUS_FAILURE);
				//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
				ZDDbg((_T("Get SpinLock\n")));
				ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
				
			}
			
			while (!(QueueEmpty(&Adapter->ActiveChainList)))
			{
				// here we have to fail any sends that haven't completed yet
				PZD1205SwTcb pSwTcb = (PZD1205SwTcb) QueuePopHead(&Adapter->ActiveChainList);
				
				if (pSwTcb->LastFrag){
					if (pSwTcb->Provider==UPPER_NDIS){
						//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
						ZDDbg((_T("---------->>> Release SpinLock\n")));
						ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
#ifdef HOST_IF_USB
						if (!pSwTcb->bHasCompleteBeforeSend){
							NdisMSendComplete(
								Adapter->ZD1205AdapterHandle,
								pSwTcb->Packet,
								NDIS_STATUS_SUCCESS);
						}
#else
						NdisMSendComplete(
							Adapter->ZD1205AdapterHandle,
							pSwTcb->Packet,
							NDIS_STATUS_SUCCESS);
							//NDIS_STATUS_FAILURE);
#endif
						//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
						ZDDbg((_T("Get SpinLock\n")));
						ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
					}
					else{
						Adapter->EnetFrameLock = FALSE;
						//ASSERT(0);
						//NdisQueryPacket(pSwTcb->Packet, NULL, NULL, &pSwTcb->FirstBuffer, NULL);
						NdisGetFirstBufferFromPacket( pSwTcb->Packet, 
													  &pSwTcb->FirstBuffer, 
													  &FirstBufferVA, 
													  &FirstBufferLength, 
													  &TotalBufferLength
													  );
						NdisFreeBuffer(pSwTcb->FirstBuffer);
						NdisFreePacket(pSwTcb->Packet);
					}
				}
			}
			Adapter->adapterReady = FALSE;
			ZDDbgPrint((_T("Adapter DisConnect! - 5\n")));
			//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
			ZDDbg((_T("---------->>> Release SpinLock\n")));
			ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
            break;

        case NdisDevicePnPEventQueryStopped:
            INITSTR((_T("MPPnPEventNotify: NdisDevicePnPEventQueryStopped\n")));
            break;

        case NdisDevicePnPEventStopped:
            INITSTR((_T("MPPnPEventNotify: NdisDevicePnPEventStopped\n")));
            break;      
            
        case NdisDevicePnPEventPowerProfileChanged:
            INITSTR((_T("MPPnPEventNotify: NdisDevicePnPEventPowerProfileChanged\n")));
            break;      
            
        default:
            INITSTR((_T("MPPnPEventNotify: unknown PnP event %x \n"), PnPEvent));
            break;         
    }

}

/*++

Routine Description:
    
    MiniportCancelSendpackets handler - NDIS51 and later
    
Arguments:

    MiniportAdapterContext      Pointer to our adapter
    CancelId                    All the packets with this Id should be cancelled

Return Value:

    None
    
--*/
VOID ZD1205CancelSendPackets(
    IN  NDIS_HANDLE     MiniportAdapterContext,
    IN  PVOID           CancelId)
{
    DEBUGFUNC(_T("ZD1205CancelSendPackets"));
    INITSTR((_T("\n")));
	return;
}

/*++

Routine Description:
    
    MiniportShutdown handler
    
Arguments:

    MiniportAdapterContext  Pointer to our adapter

Return Value:

    None
    
--*/
VOID ZD1205Shutdown(
    IN  NDIS_HANDLE     MiniportAdapterContext)
{

    DEBUGFUNC(_T("ZD1205Shutdown"));
    INITSTR((_T("\n")));
	ZD1205Halt(MiniportAdapterContext);
}

#endif

// Caller of this function should Acquire Spin-Lock (Adapter->Lock)
void
SetBSSType(
		IN PZD1205_ADAPTER Adapter,
		UCHAR	bsstype
		)
{

    DEBUGFUNC(_T("SetBSSType"));
    //INITSTR((_T("\n bsstype = %d\n"), bsstype));

	if (bsstype == AP_BSS){
		// Set Backoff Parameters
		//Adapter->NormalBackoff = 0x3ff501f;
		//Adapter->UrgentBackoff = 0x3ff501f;
		//Adapter->LooseBackoff = 0x3ff501f;
		// Initialize AP mode
		InitializeAccessPoint(Adapter);
	}
	else{
		// Set Backoff Parameters
		//Adapter->NormalBackoff = 0x7f047f;
		//Adapter->UrgentBackoff = 0x7f0407;
		//Adapter->LooseBackoff = 0x7f107f;

		// Initialize STA mode
		if ( Adapter->CardSetting.BSSType == AP_BSS )
		{
            Adapter->FlagAccessRegister |= BIT_4;
			Adapter->FlagAccessRegister |= BIT_5;
		}
		APMgt_SendNotification(Adapter, BIT_2);
	}
	Adapter->CardSetting.BSSType = bsstype;
}

// Caller of ProviderMSendComplete must Acquire a SPIN-LOCK (Adapter->Lock)
void
ProviderMSendComplete(
		IN PZD1205_ADAPTER Adapter,
		PZD1205SwTcb       pSwTcb,
	    ZD1211_IRQL		   *pOldIrql
		)
{

	PVOID     FirstBufferVA;
	UINT      FirstBufferLength, TotalBufferLength;

    DEBUGFUNC(_T("ProviderMSendComplete"));    

	switch(pSwTcb->Provider){
	case UPPER_NDIS:
		//KeReleaseSpinLock(&Adapter->Lock, *pOldIrql);
		//ZDDbg((_T("---------->>> Release SpinLock\n")));
		//ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, *pOldIrql);
		//NdisMSendComplete(
		//	Adapter->ZD1205AdapterHandle,
		//	pSwTcb->Packet,
		//	NDIS_STATUS_SUCCESS);
		//NDIS_STATUS_FAILURE);
		//KeAcquireSpinLock(&Adapter->Lock, pOldIrql);
		//ZDDbg((_T("Get SpinLock\n")));
		//ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, pOldIrql);
		break;
	case STA_MGT:
		//NdisQueryPacket(pSwTcb->Packet, NULL, NULL, &pSwTcb->FirstBuffer, NULL);
		NdisGetFirstBufferFromPacket( pSwTcb->Packet, 
									  &pSwTcb->FirstBuffer, 
									  &FirstBufferVA, 
									  &FirstBufferLength, 
									  &TotalBufferLength
									  );
		NdisFreeBuffer(pSwTcb->FirstBuffer);
		NdisFreePacket(pSwTcb->Packet);
		break;
	case AP_MGT:
		//NdisQueryPacket(pSwTcb->Packet, NULL, NULL, &pSwTcb->FirstBuffer, NULL);
		NdisGetFirstBufferFromPacket( pSwTcb->Packet, 
									  &pSwTcb->FirstBuffer, 
									  &FirstBufferVA, 
									  &FirstBufferLength, 
									  &TotalBufferLength
									  );
		NdisFreeBuffer(pSwTcb->FirstBuffer);
		NdisFreePacket(pSwTcb->Packet);
		{
			PTX_MGT_ENTITY		TxMgtEntityPtr;
			TxMgtEntityPtr = (PTX_MGT_ENTITY)pSwTcb->ReturnPara;
			QueuePutTail(&Adapter->MgtFreeList, &TxMgtEntityPtr->Link);
		}
		break;
	case AP_FORWARD_MUL:
		// Releasing resource is accomplished by ProcessRxInterrupt.
		// We need not free resources here.
		break;
	case AP_FORWARD_UNI:
		//KeReleaseSpinLock(&Adapter->Lock, *pOldIrql);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, *pOldIrql);
		ZD1205GetReturnedPackets(Adapter, pSwTcb->Packet);
		//KeAcquireSpinLock(&Adapter->Lock, pOldIrql);
		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, pOldIrql);
		break;
	default:
		ASSERT(0);
		break;
	}
}

VOID
JudgeRate(PZD1205_ADAPTER	Adapter, int i)
{

	UCHAR			CurrentTxRate;
	USHORT	        SuccessFrames, FailFrames;


	SuccessFrames = Adapter->OppositeRateInfo[i].ConsecutiveSuccessFrames;
	FailFrames = Adapter->OppositeRateInfo[i].FailFrames;
		
	if ((SuccessFrames + FailFrames) < 20){
		//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
		return;
	}

	ZDDbgPrint((_T("SuccessFrames=%d FailFrames=%d\n"), SuccessFrames, FailFrames));

	Adapter->OppositeRateInfo[i].ConsecutiveSuccessFrames = 0;
	Adapter->OppositeRateInfo[i].FailFrames = 0;

	CurrentTxRate = Adapter->OppositeRateInfo[i].RateArray[Adapter->OppositeRateInfo[i].CurrentRate];
		
	if ((SuccessFrames > 0) && (FailFrames == 0)){
		// perfect
		Adapter->OppositeRateInfo[i].RiseCondition ++;
		Adapter->OppositeRateInfo[i].DownCondition = 0;

	}
	else if ((SuccessFrames > 300) && (FailFrames <= 2)){
		// perfect
		Adapter->OppositeRateInfo[i].RiseCondition ++;
		Adapter->OppositeRateInfo[i].DownCondition = 0;
	}
	else if( SuccessFrames >= (FailFrames * Adapter->RateLine1)){
		//36M
		if (CurrentTxRate > 9){
			Adapter->OppositeRateInfo[i].RiseCondition = 0;
			Adapter->OppositeRateInfo[i].DownCondition ++;
		}
	    else if (CurrentTxRate == 9){
			Adapter->OppositeRateInfo[i].RiseCondition = 0;
			Adapter->OppositeRateInfo[i].DownCondition = 0;
		}
		else{
			Adapter->OppositeRateInfo[i].RiseCondition ++;
			Adapter->OppositeRateInfo[i].DownCondition = 0;
		}

	}
	else if( SuccessFrames >= (FailFrames * Adapter->RateLine2)){
		//24M
		if (CurrentTxRate > 8){
			Adapter->OppositeRateInfo[i].RiseCondition = 0;
			Adapter->OppositeRateInfo[i].DownCondition ++;
		}
		else if (CurrentTxRate == 8){
			Adapter->OppositeRateInfo[i].RiseCondition = 0;
			Adapter->OppositeRateInfo[i].DownCondition = 0;
		}
		else{
			Adapter->OppositeRateInfo[i].RiseCondition ++;
			Adapter->OppositeRateInfo[i].DownCondition = 0;
		}

	}
	else if( SuccessFrames >= (FailFrames * Adapter->RateLine3)){
		//18M
		if (CurrentTxRate > 7){
			Adapter->OppositeRateInfo[i].RiseCondition = 0;
			Adapter->OppositeRateInfo[i].DownCondition ++;
		}
		else if (CurrentTxRate == 7){
			Adapter->OppositeRateInfo[i].RiseCondition = 0;
			Adapter->OppositeRateInfo[i].DownCondition = 0;
		}
		else{
			Adapter->OppositeRateInfo[i].RiseCondition ++;
			Adapter->OppositeRateInfo[i].DownCondition = 0;
		}
	}
	else{
		Adapter->OppositeRateInfo[i].RiseCondition = 0;
		Adapter->OppositeRateInfo[i].DownCondition ++;
	}


	if (Adapter->OppositeRateInfo[i].bJustRiseRate){

		Adapter->OppositeRateInfo[i].bJustRiseRate = 0;

		if (Adapter->OppositeRateInfo[i].DownCondition){
			// rise rate is not a wise decesion, down rate immediately
			if (Adapter->OppositeRateInfo[i].CurrentRate > 0){
				Adapter->OppositeRateInfo[i].CurrentRate --;
			}
			Adapter->OppositeRateInfo[i].DownCondition = 0;
			ZDDbgPrint((_T("Rate2 << %x\n"), Adapter->OppositeRateInfo[i].RateArray[Adapter->OppositeRateInfo[i].CurrentRate]));

			return;
		}
	}

	if (Adapter->OppositeRateInfo[i].RiseCondition >= Adapter->RiseConditionThreshold){

		if ( Adapter->OppositeRateInfo[i].MaxRate > Adapter->OppositeRateInfo[i].CurrentRate ){
			Adapter->OppositeRateInfo[i].CurrentRate ++;
			ZDDbgPrint((_T("Rate1 >> %x\n"), Adapter->OppositeRateInfo[i].RateArray[Adapter->OppositeRateInfo[i].CurrentRate]));
		}		
		Adapter->OppositeRateInfo[i].RiseCondition = 0;
		Adapter->OppositeRateInfo[i].DownCondition = 0;

		Adapter->OppositeRateInfo[i].bJustRiseRate = TRUE;

	}

	if (Adapter->OppositeRateInfo[i].DownCondition >= Adapter->DownConditionThreshold){

		if (Adapter->OppositeRateInfo[i].CurrentRate > 0){
			Adapter->OppositeRateInfo[i].CurrentRate--;
			ZDDbgPrint((_T("Rate1 << %x\n"), Adapter->OppositeRateInfo[i].RateArray[Adapter->OppositeRateInfo[i].CurrentRate]));
		}
		Adapter->OppositeRateInfo[i].RiseCondition = 0;
		Adapter->OppositeRateInfo[i].DownCondition = 0;
	}

}



/*

|----------------|------------------|-------------------|------------------|
                Line1              Line2              Line3                1
All rise
   54 48 Down
   36 not change
   24~1 Up
                     54~36 Down
					 24 not change
					 18~1 Up
					                   54~24 Down
									   18 not change
									   11~1 Up
									                         All down
*/
VOID
AutoRateTimer(
        IN PVOID SystemSpecific1,
        IN NDIS_HANDLE MiniportAdapterContext,
        IN PVOID SystemSpecific2,
        IN PVOID SystemSpecific3
        )
{
	PZD1205_ADAPTER	Adapter = (PZD1205_ADAPTER)MiniportAdapterContext;
	ZD1211_IRQL		OldIrql;
	int				i;
	PSTA_CONTEXT	STAContextPtr;

	DEBUGFUNC(_T("AutoRateTimer"));

	//KeAcquireSpinLock(&Adapter->Lock, &OldIrql);
	ZDDbg((_T("Get SpinLock\n")));
	ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	
    if ( Adapter->CardSetting.BSSType == AP_BSS ){
	    STAContextPtr = (PSTA_CONTEXT)QueueGetHead(&Adapter->STAContextList);
		while( STAContextPtr != NULL ){
		    JudgeRate(Adapter, STAContextPtr->AID);
		    STAContextPtr = (PSTA_CONTEXT) QueueGetNext( &Adapter->STAContextList,
													     &STAContextPtr->Link);		    
		}
	}else{
	    for (i=0;i<Adapter->OppositeCnt;i++)
		    JudgeRate(Adapter, i);
	}

	//KeReleaseSpinLock(&Adapter->Lock, OldIrql);
	ZDDbg((_T("---------->>> Release SpinLock\n")));
	ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);

}

void
QueryOppositeRatePreamble(
		IN PZD1205_ADAPTER Adapter,
		PUCHAR			pAddr,
		PUCHAR			pRate,
		PUCHAR			pPreamble,
		PZD1205SwTcb	SwTcb
		)
{
	UCHAR       aid;
	USHORT		i;
	int			j;
	PSTA_CONTEXT	STAContextPtr;

	DEBUGFUNC(_T("QueryOppositeRatePreamble"));

	// Set default value for Rate, Preamble
	*pRate = Adapter->CardSetting.TxRate;
	*pPreamble = 0x00;	// Long preamble

	if (Adapter->CardSetting.BSSType != AP_BSS){
		for( i=0; i<Adapter->OppositeCnt; i++ ){
			if ( !memcmp(pAddr, Adapter->OppositeRateInfo[i].MacAddr, 6) ){
				
				*pRate = Adapter->OppositeRateInfo[i].RateArray[Adapter->OppositeRateInfo[i].CurrentRate];

				if (Adapter->OppositeRateInfo[i].RateArrayCount == 0){
					// mean user may set only one rate, but none is allow
					// use 11M instead. because this may be USB1.1
					*pRate = 3;
					ZDDbgPrint((_T("Force use 11M\n")));
				}

				if (Adapter->OppositeRateInfo[i].Capability & BIT_5){
					*pPreamble = 0x20;	// Short preamble
				}
				// force ZD1202 series use short preamble if utility don't specify
				if (Adapter->OppositeRateInfo[i].MaxRate > 3){
					*pPreamble = 0x20;	// Short preamble
				}
				break;
			}
		}

		if(i == Adapter->OppositeCnt){

            // IBSS and Broadcast
			//if ( (!memcmp(pAddr, broadcast, 6))&&   // broadcast
			if ( (pAddr[0] & BIT_0)&&   // multicast
				 (Adapter->CardSetting.BSSType == INDEPENDENT_BSS) )
			{
			    *pRate = 0x00; // 1M
				return;
			}
			// This packet may be Broadcast
			// No OppositeRateInfo, use highest rate
			*pRate = Adapter->AllowRateArray[Adapter->AllowRateArrayCount - 1];

			if ((!(Adapter->bIBSS_G_MODE || Adapter->bIBSS_PureG_MODE)) && (Adapter->CardSetting.BSSType == INDEPENDENT_BSS)){
				// in Ad-hoc only allow 1M ~ 11M
				for(j=(Adapter->AllowRateArrayCount-1); j >= 0; j--){
			
					if (Adapter->AllowRateArray[j] <= 0x3){
						// search the biggest one in 1M ~ 11M
						*pRate = Adapter->AllowRateArray[j];
						break;
					}
				}

				// if no rate match, use 11M
				if (*pRate > 0x3) *pRate = 3;
			}
		}
	}
	else{

		if ( pAddr[0] & 0x01 ){
		    //*pRate = Rate_Convert(Adapter->BasicRates[Adapter->BasicRates[0]], 0);
            *pRate = 0;
			return;
		}

		STAContextPtr = (PSTA_CONTEXT)FindSTAContext(Adapter, pAddr);
		SwTcb->STAContextPtr = STAContextPtr;
		if (STAContextPtr){
		    aid = STAContextPtr->AID;
			*pRate = Adapter->OppositeRateInfo[aid].RateArray[Adapter->OppositeRateInfo[aid].CurrentRate];
			if (STAContextPtr->Capability & BIT_5){
				*pPreamble = 0x20;	// Short preamble
			}
			// force ZD1202 series use short preamble if utility don't specify
			//if (STAContextPtr->MaxRate > 3){
			//	*pPreamble = 0x20;	// Short preamble
			//}
		}
	}
}


USHORT ZDLOGTEN[] = {0, 
	0   ,  30 ,  47 ,  60 ,  69 ,  77 ,  84 ,  90 ,  95 , 100 ,
	104 , 107 , 111 , 114 , 117 , 120 , 123 , 125 , 127 , 130 ,
	132 , 134 , 136 , 138 , 139 , 141 , 143 , 144 , 146 , 147 ,
	149 , 150 , 151 , 153 , 154 , 155 , 156 , 157 , 159 , 160 ,
	161 , 162 , 163 , 164 , 165 , 166 , 167 , 168 , 169 , 169 ,
	170 , 171 , 172 , 173 , 174 , 174 , 175 , 176 , 177 , 177 ,
	178 , 179 , 179 , 180 , 181 , 181 , 182 , 183 , 183 , 184 ,
	185 , 185 , 186 , 186 , 187 , 188 , 188 , 189 , 189 , 190 ,
	190 , 191 , 191 , 192 , 192 , 193 , 193 , 194 , 194 , 195 ,
	195 , 196 , 196 , 197 , 197 , 198 , 198 , 199 , 199 , 200 ,
	200 , 200 , 210 , 210 , 220 , 220 , 220 , 230 , 230 , 240 ,
	240 , 240 , 250 , 250 , 260 , 260 , 260 , 270 , 270 , 270 ,
	280 , 280 , 280 , 290 , 290 , 210 , 210 , 210 , 211 , 211 ,
	211 , 212 , 212 , 212 , 213 , 213 , 213 , 213 , 214 , 214 ,
	214 , 215 , 215 , 215 , 216 , 216 , 216 , 217 , 217 , 217 ,
	217 , 218 , 218 , 218 , 219 , 219 , 219 , 219 , 220 , 220 ,
	220 , 220 , 221 , 221 , 221 , 222 , 222 , 222 , 222 , 223 ,
	223 , 223 , 223 , 224 , 224 , 224 , 224 , 225 , 225 , 225 ,
	225
};


USHORT
ZDLog10multiply100(int data)
{
	if ((data >= 0) && (data <= 0xb5))
		return ZDLOGTEN[data];
	else
		return 225;
}

VOID 
ZD1211_AcquireSpinLock( IN PZD1205_ADAPTER Adapter,
                        IN PZD1211_SPIN_LOCK  SpinLock,
                        OUT PZD1211_IRQL  pOldIrql
)
{
    //LPCRITICAL_SECTION  p = (LPCRITICAL_SECTION) SpinLock;

    //ZDDbgPrint((_T("SpinLock = %x\n"), (ULONG) SpinLock));
	//ZDDbgPrint((_T("LockCount = %x\n"), p->LockCount));
	//ZDDbgPrint((_T("OwnerThread = %x\n"), p->OwnerThread));
	//ZDDbgPrint((_T("hCrit = %x\n"), p->hCrit));
	//ZDDbgPrint((_T("needtrap = %x\n"), p->needtrap));
	//ZDDbgPrint((_T("dwContentions = %x\n"), p->dwContentions));

	NdisAcquireSpinLock(SpinLock);
	//WaitForSingleObject(*SpinLock, INFINITE);
    //ZDDbgPrint((_T("########################\n")));
    

}

VOID 
ZD1211_ReleaseSpinLock( IN PZD1205_ADAPTER Adapter,
                        IN PZD1211_SPIN_LOCK  SpinLock,
                        IN ZD1211_IRQL  NewIrql
)
{
	//ZDDbgPrint((_T("ReleaseSpinLock\n")));
	//ZDDbgPrint((_T("*SpinLock = %x\n"), (*SpinLock)));

    NdisReleaseSpinLock(SpinLock);
	//SetEvent(*SpinLock);
}
/*
ULONG    
ZD1211StartRxThread(IN PZD1205_ADAPTER Adapter)
{
    ZD1211_IRQL			OldIrql;

    DEBUGFUNC(_T("ZD1211StartRxThread")); 

    while(1)
	{
        WaitForSingleObject(Adapter->StartRxEvent, 2000); // 2 sec

        //SetThreadPriority(Adapter->h_ZD1211StartRxThread, THREAD_PRIORITY_NORMAL);

	    if ( Adapter->bTerminateThreads )
		{
			INITSTR((_T("Terminate ZD1211StartRxThread now..\n")));
			Adapter->h_ZD1211StartRxThread = NULL;
		    ExitThread(0);
		}

        ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
		USB_MPRcvFromRxEP(Adapter);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);		
	}

    return 0;
}
*/

ULONG    
ZD1211SetQueryOIDThread(IN PZD1205_ADAPTER Adapter)
{
    UCHAR   flag;
	ULONG   length, BytesRead, BytesNeeded;
	NDIS_OID    oid;
	NDIS_STATUS status;
	char    tmpbuffer[4096];

    DEBUGFUNC(_T("ZD1211SetQueryOIDThread"));

    while(1)
	{
        WaitForSingleObject(Adapter->hReqInfoEvent, INFINITE);
        ResetEvent(Adapter->hReqInfoEvent);

        if ( Adapter->bEndThreads )
		{
			DbgPrint("h_ZD1211SetQueryOIDThread end");
			Adapter->h_ZD1211SetQueryOIDThread = NULL;
		    ExitThread(0);
		}

        flag = Adapter->ReqInfoMap[0];
        //DbgPrint("flag = %d\n", flag);
		if ( flag == 1 )
		{
			//DbgPrint("oid = %x\n", oid);
		    memcpy(&oid, Adapter->ReqInfoMap+1, 4);
			memcpy(&length, Adapter->ReqInfoMap+5, 4);
			status = ZD1205QueryInformation( Adapter,
				                             oid,
											 tmpbuffer,
											 length,
											 &BytesRead,
											 &BytesNeeded
				                             );
			memcpy(Adapter->ReqInfoMap+9, tmpbuffer, BytesRead);
		}
		else if ( flag == 2 )
		{
			//DbgPrint("oid = %x\n", oid);
		    memcpy(&oid, Adapter->ReqInfoMap+1, 4);
			memcpy(&length, Adapter->ReqInfoMap+5, 4);
            memcpy(tmpbuffer, Adapter->ReqInfoMap+9, length);
			status = ZD1205SetInformation( Adapter,
				                           oid,
										   tmpbuffer,
										   length,
										   &BytesRead,
										   &BytesNeeded
				                           );
		}
		else if ( flag == 3 )
		{
			//DbgPrint("init\n");
		    Adapter->ReqInfoMap[1] = 1;
			SetEvent(Adapter->hReqCompleteEvent);
			continue;
		}
        memcpy(Adapter->ReqInfoMap+5, &status, 4);
		SetEvent(Adapter->hReqCompleteEvent);
		//DbgPrint("end\n");
	}

    return 0;
}


ULONG
ZD1211StartRxThread(IN PZD1205_ADAPTER Adapter)
{
    ZD1211_IRQL			OldIrql;
	HANDLE              hArray[2];
	DWORD               ret;
	NDIS_STATUS         status;
	int                 count;

    DEBUGFUNC(_T("ZD1211StartRxThread"));

    INITSTR((_T("started!!\n")));

	USB_StartTxEP(Adapter);

    while(1)
	{
		if ( Adapter->bEndThreads )
		{
			DbgPrint("h_ZD1211StartRxThread end");
			Adapter->h_ZD1211StartRxThread = NULL;
			ExitThread(0);
		}

		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
		status = USB_MPRcvFromRxEP(Adapter);
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
	}

    return 0;
}


ULONG
ZD1211InterruptThread(IN PZD1205_ADAPTER Adapter)
{
    NDIS_STATUS         status;
	ZD1211_IRQL			OldIrql;
	PSTA_CONTEXT        STAContextPtr;
    UCHAR               aid;
	BOOL                bRet;
	DWORD               dwLen, dwError;
	ULONG				DebugPrint = Adapter->USBDbgFlag & DBGMSG_INT;
 
    DEBUGFUNC(_T("ZD1211InterruptThread"));
	INITSTR((_T("started!!\n")));

    while(1)
	{

        if ( Adapter->bEndThreads )
		{
			DbgPrint("h_ZD1211InterruptThread end");
			Adapter->h_ZD1211InterruptThread = NULL;
		    ExitThread(0);
		}


		ZDDbg((_T("Get SpinLock\n")));
		ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);
	    //DbgPrint("send int\r\n");
		status = USB_MPRcvFromIntEP(Adapter);
	    
		ZDDbg((_T("---------->>> Release SpinLock\n")));
		ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);
		//}
        //else
		//	status = NDIS_STATUS_SUCCESS;

        if ( status == NDIS_STATUS_SUCCESS )
		{

		    WaitForSingleObject(Adapter->WaitIntEvent, INFINITE);

			if ( Adapter->bEndThreads )
			{
				DbgPrint("h_ZD1211InterruptThread end");
                Adapter->h_ZD1211InterruptThread = NULL;
				ExitThread(0);
			}
			//ZDDbg((_T("Get SpinLock\n")));
			//ZD1211_AcquireSpinLock(Adapter, &Adapter->Lock, &OldIrql);

			//DbgPrint("get WaitIntEvent");
			bRet = (Adapter->USBInfo.lpUsbFuncs->lpGetTransferStatus)(
							   Adapter->USBInfo.hIntEP,
							   &dwLen,
							   &dwError
							   );

			if ( (dwError != USB_NO_ERROR)||(!bRet) ) 
			{
				//INITSTR((_T("Int EP error occurred!!\n"))); 
				DbgPrint("Int EP error occurred!!");
				//Adapter->bUSBDeveiceAttached = FALSE;
				//Adapter->SurpriseRemoved = TRUE;
				USB_FreeINTEPResource(Adapter);
			}
			else
			{
				//INITSTR((_T("Get Length %d\n"), dwLen));
                //DbgPrint("int return = %x\r\n", Adapter->IntEPBuffer[1]);
				if (Adapter->IntEPBuffer[0] == 1)
				{
					// valid interrupt packet
					switch (Adapter->IntEPBuffer[1]){
						
						case EPINT_IORDRsp:
							// this is USB_READ_REGISTER response
							
							Adapter->ReadRegCount = (USHORT) dwLen;

							if (*(PUSHORT)(Adapter->IntEPBuffer+2) == (InterruptCtrl | Adapter->USBCSRAddress)){
								// Handle non-RxTx interrupt
								if (Adapter->bHandleNonRxTxRunning){
									
									//USBWARNSTR(("Impossible, interrupt happen!!!!! 0x%x\n", *(PUSHORT)(Adapter->IntEPBuffer+4)));
									INITSTR((_T("Impossible, interrupt happen!!!!! 0x%x\n"), *(PUSHORT)(Adapter->IntEPBuffer+4)));
									ASSERT(0);
									break;
								}

								// disable non-RxTx interrupt
								// No needed to diable interrupt, firmware will do it.
								//ZD1205_WRITE_REGISTER(Adapter,InterruptCtrl, (0)<<16);
							
								USB_CompleteCountIncrement(Adapter);

								Adapter->bHandleNonRxTxRunning = TRUE;
								
								NdisMoveMemory(Adapter->IntEPBuffer3, Adapter->IntEPBuffer, MAX_EPINT_BUFFER);
								
								//USBDBGSTR(DebugPrint,("get non-RxTx interrupt 0x%x\n", *(PUSHORT)(Adapter->IntEPBuffer3+4)));
								INITSTR((_T("get non-RxTx interrupt 0x%x\n"), *(PUSHORT)(Adapter->IntEPBuffer3+4)));
								//NdisScheduleWorkItem(&Adapter->WorkItemHandleNonRxTx);
							}
							else{
								//INITSTR((_T("get Register data\n")));
								//DbgPrint("get register data");
								// handle read register
								NdisMoveMemory(Adapter->IntEPBuffer2, Adapter->IntEPBuffer, MAX_EPINT_BUFFER);
								SetEvent(Adapter->EPINT_IORDRsp_Event);
							}

							break;

						case EPINT_RetryFial_Event:
							{

								UCHAR		i;
								PUCHAR	pMacAddr = Adapter->IntEPBuffer + 4;
								UCHAR	NewRate = (UCHAR)(*(PUSHORT)(Adapter->IntEPBuffer + 2));

								if ( Adapter->IntEPBuffer[10] & BIT_0 )
									Adapter->IBSS_Wakeup_Dest = TRUE;

							//JWEI 2004/03/24
							#if fTX_PWR_CTRL
								if (Adapter->TxOFDMType >= cTX_48M)
									Adapter->TxOFDMCnt = 0;
							#endif
								
								if (Adapter->CardSetting.BSSType == AP_BSS){
									STAContextPtr = FindSTAContext(Adapter, pMacAddr); 
									if (STAContextPtr){
										//++ JSWAN 7/27/2004
										STAContextPtr->IdleTime ++;
										//--
										aid = STAContextPtr->AID;
										Adapter->OppositeRateInfo[aid].FailFrames++;
									}
								}      
 								else{
									for (i=0;i<Adapter->OppositeCnt;i++){
										if (!memcmp(pMacAddr, Adapter->OppositeRateInfo[i].MacAddr, 6) ){

											Adapter->OppositeRateInfo[i].FailFrames++;
											//Adapter->OppositeRateInfo[i].FailFrames += *(PUSHORT)(Adapter->IntEPBuffer + 10);
										//JWEI 2004/02/25
											Adapter->RetryFailCnt +=  *(PUSHORT)(Adapter->IntEPBuffer + 10);
										
  											break;
										}
									}
								}

							}

							break;

						default:
							break;
					}
				}

				bRet = (Adapter->USBInfo.lpUsbFuncs->lpCloseTransfer)(Adapter->USBInfo.hIntEP);
				Adapter->USBInfo.hIntEP = NULL;
				if ( !bRet )
				{
					//INITSTR((_T("Int EP close error!!\n")));
                    DbgPrint("Int EP close error!!");
					//Adapter->bUSBDeveiceAttached = FALSE;
					//Adapter->SurpriseRemoved = TRUE;
					//SetEvent(Adapter->USBIntEPStopEvent);				    
				}
			}


			//ZDDbg((_T("---------->>> Release SpinLock\n")));
			//ZD1211_ReleaseSpinLock(Adapter, &Adapter->Lock, OldIrql);            
		}		
	}

    return 0;
}


VOID
TerminateThreads(PZD1205_ADAPTER Adapter)
{

	DEBUGFUNC(_T("TerminateThreads"));

    Adapter->bTerminateThreads = TRUE;

    Adapter->bEndThreads = TRUE;

	SetEvent(Adapter->MgtEvent);
    while ( Adapter->h_ZD1205MgtThread != NULL ) 
           Sleep(10);

	SetEvent(Adapter->WaitIntEvent);
    while ( Adapter->h_ZD1211InterruptThread != NULL ) 
           Sleep(10);

    Adapter->USBInfo.lpUsbFuncs->lpClosePipe(ZD1205Adapter->USBInfo.hPipe[2]);
    while ( Adapter->h_ZD1211StartRxThread != NULL ) 
           Sleep(10);

	SetEvent(Adapter->hReqInfoEvent);
	while ( Adapter->h_ZD1211SetQueryOIDThread != NULL )
		   Sleep(10);

}

